/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/
/*
 * Agent-side handlers for incoming messages.
 */

#define NEED_EVENTS

/* The length field in requests is specified as 4-byte words */
#define REQ_UNITS_IN_BYTES 4

#include <stdio.h>
#ifndef ZAP
#include <stdlib.h>
#endif
#include "agent.h"
#include <X11/StringDefs.h>
#include <X11/ICE/ICEproto.h>
#include <X11/ICE/ICEmsg.h>
#include <X11/Xproto.h>

#include "RapLibint.h"


void
FreeHelloReply(HelloReply *hr)
{
    if (!hr) return;
    
    free(hr);
}

void
FreeQueryTreeReply(QueryTreeReply *qtr)
{
    register int i;

    if (!qtr) return;
    
    for (i=0 ; i<qtr->numShells ; i++) {
	register int j;

	for (j=0 ; j<qtr->trees[i].numWidgets ; j++) {
	    free(qtr->trees[i].widgets[j].name);
	    free(qtr->trees[i].widgets[j].className);
	    free(qtr->trees[i].widgets[j].toolkit);
	}

	free(qtr->trees[i].widgets);
    }

    free(qtr->trees);
    free(qtr);
}


void
FreeFullQueryTreeReply(FullQueryTreeReply *qtr)
{
    register int i;

    if (!qtr) return;
 
    for (i=0 ; i<qtr->numShells ; i++) {
        register int j;
 
        for (j=0 ; j<qtr->trees[i].numWidgets ; j++) {
	    register int k;

        	for (k=0 ; k<qtr->trees[i].widgets[j].numResources ; k++) {

            	   free(qtr->trees[i].widgets[j].resourceData[k].name);
            	   free(qtr->trees[i].widgets[j].resourceData[k].className);
            	   free(qtr->trees[i].widgets[j].resourceData[k].nativeType);
            	   free(qtr->trees[i].widgets[j].resourceData[k].returnType);
            	   free(qtr->trees[i].widgets[j].resourceData[k].value);
		}
		
            free(qtr->trees[i].widgets[j].resourceData);
            free(qtr->trees[i].widgets[j].name);
            free(qtr->trees[i].widgets[j].className);
            free(qtr->trees[i].widgets[j].toolkit);
        }
 
        free(qtr->trees[i].widgets);
    }
 
    free(qtr->trees);

    free(qtr);
}


void
FreeGetValuesReply(GetValuesReply *gvr)
{
    register int i;

    if (!gvr) return;
    
    for (i=0 ; i<gvr->numValues ; i++) {
	free(gvr->values[i].name);

	if (gvr->values[i].errorCode != RapNoError)
	    free(gvr->values[i].data.errorString);
	else {
	    free(gvr->values[i].data.v.nativeType);
	    free(gvr->values[i].data.v.value);
	    free(gvr->values[i].data.v.returnType);
	}
    }

    free(gvr->values);
    free(gvr);
}

void
FreeGetResourcesReply(GetResourcesReply *grr)
{
    register int i;

    if (!grr) return;

    for (i=0 ; i<grr->numWidgets; i++) {
	if (grr->widgetResources[i].errorCode != RapNoError)
	    free(grr->widgetResources[i].data.errorString);
	else {
	    register int j;

	    for (j=0 ; j<grr->widgetResources[i].data.r.numResources ; j++) {
		free(grr->widgetResources[i].data.r.resources[j].name);
		free(grr->widgetResources[i].data.r.resources[j].className);
		free(grr->widgetResources[i].data.r.resources[j].type);
	    }

	    free(grr->widgetResources[i].data.r.resources);
	}
    }

    free(grr->widgetResources);
    free(grr);
}

void 
FreeGetGCValuesReply(GetGCValuesReply *ggcv)
{
  register int i;

  if (!ggcv) return;

  for (i=0; i<ggcv->numGCs; i++) {
      if (ggcv->GCValues[i].errorCode != RapNoError)
	  free(ggcv->GCValues[i].data.errorString);
  }

  free(ggcv->GCValues);
  free(ggcv);

}

void
FreeLocateObjectReply(LocateObjectReply *reply)
{
    register int i;

    if (!reply) return;

    for (i=0 ; i<reply->numLocations ; i++) {
	if (reply->locations[i].errorCode != RapNoError)
	    free(reply->locations[i].data.errorString);
    }

    free(reply->locations);
    free(reply);
}

void
FreeCreateNotify(CreateNotif *cn)
{
    register int i;

    if (!cn) return;
    
    free(cn->type);
    free(cn->name);
    free(cn->className);
    free(cn->toolkit);

    for (i=0 ; i<cn->numResources ; i++) {
	free(cn->resources[i].name);
	free(cn->resources[i].className);
	free(cn->resources[i].type);
	free(cn->resources[i].value);
	free(cn->resources[i].returnType);
    }
    
    free(cn->resources);
    free(cn);
}

void
FreeDestroyNotify(DestroyNotif *dn)
{
    if (!dn) return;
    
    free(dn->type);
    free(dn);
}

void
FreeChangeNotify(ChangeNotif *cn)
{
    if (!cn) return;
    
    /* FOR NOW, WE LEAK ... */
    free(cn); 
}

void
FreeConfigureNotify(ConfigureNotif *cn)
{
    if (!cn) return;
    
    free(cn->type);
    free(cn);
}

void
FreeGeometryNotify(GeometryNotif *gn)
{
    if (!gn) return;

    free(gn->type); 
    free(gn); 
}

void
FreeRequestNotify(RequestNotif *rn)
{
    if (!rn) return;

    free(rn->request);
    free(rn);
}

void
FreeEventNotify(EventNotif *en)
{
    if (!en) return;

    free(en->event);
    free(en);
}

static void
GetResourceValue(char **bufp, CARD32 *nativeSize, char **valueReturn)
{
    EXTRACT_BYTES(*bufp, FALSE, *nativeSize, *valueReturn);
}


HelloReply *
HandleRapHelloReply(IceConn iceConn)
{
    RapHelloMessage *hello;
    HelloReply *ret;

    IceReadMessageHeader(iceConn,
			 SIZEOF(RapHelloMessage),
			 RapHelloMessage,
			 hello);

    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return NULL;
    }
#ifdef RAP_DEBUG
    fprintf(stderr, "RECEIVED HELLO REPLY with window id %ld\n",
   			hello->window); fflush(stderr);
#endif
    ret = (HelloReply *) malloc(sizeof(HelloReply));
    memset(ret, '\0', sizeof(HelloReply));
    ret->window = hello->window;
    
    return ret;
}

QueryTreeReply *
HandleRapQueryTreeReply(IceConn iceConn)
{
    RapMessageHeader *header;
    char *pdata, *bufPointer;
    register int i;
    QueryTreeReply *ret;
#ifdef RAP_DEBUG
    fprintf(stderr, "RECEIVED QUERY TREE REPLY.\n"); fflush(stderr);
#endif
    IceReadCompleteMessage(iceConn,
			   SIZEOF(RapMessageHeader),
			   RapMessageHeader,
			   header,
			   pdata);

    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return NULL;
    }

    bufPointer = pdata;
    ret = (QueryTreeReply *) malloc(sizeof(QueryTreeReply));
    memset(ret, '\0', sizeof(QueryTreeReply));
		
    EXTRACT_CARD32(bufPointer, FALSE, ret->numShells);

#ifdef RAP_DEBUG
    printf("NUM SHELLS: %d\n", ret->numShells);
#endif

    ret->trees = (TreeList) malloc(sizeof(Tree) * ret->numShells);

    for (i=0 ; i<ret->numShells ; i++) {
	register int j;

	EXTRACT_CARD32(bufPointer, FALSE, ret->trees[i].numWidgets);
#ifdef RAP_DEBUG
	printf("NUM WIDGETS IN TREE %d: %d\n", i,
	       ret->trees[i].numWidgets);
#endif

	ret->trees[i].widgets = (TreeWidgetList)
	    malloc(sizeof(TreeWidget) * ret->trees[i].numWidgets);

	for (j=0 ; j<ret->trees[i].numWidgets ; j++) {
	    EXTRACT_CARD32(bufPointer, FALSE, ret->trees[i].widgets[j].widget);
	    EXTRACT_CARD32(bufPointer, FALSE, ret->trees[i].widgets[j].parent);
	    EXTRACT_STRING(bufPointer, FALSE, ret->trees[i].widgets[j].name);
	    EXTRACT_STRING(bufPointer, FALSE, ret->trees[i].widgets[j].className);
	    EXTRACT_CARD32(bufPointer, FALSE, ret->trees[i].widgets[j].window);
	    EXTRACT_CARD32(bufPointer, FALSE, ret->trees[i].widgets[j].managed);
	    EXTRACT_STRING(bufPointer, FALSE, ret->trees[i].widgets[j].toolkit);
	    
#ifdef RAP_DEBUG	    
	    printf("---------------------------\n");
	    printf("\twidget = %lx\n", ret->trees[i].widgets[j].widget);
	    printf("\tparent = %lx\n", ret->trees[i].widgets[j].parent);
	    printf("\tname = '%s'\n", ret->trees[i].widgets[j].name);
	    printf("\tclassName = '%s'\n", ret->trees[i].widgets[j].className);
	    printf("\twindow = %lx\n", ret->trees[i].widgets[j].window);
	    printf("\tmanaged = %lx\n", ret->trees[i].widgets[j].managed);
	    printf("\ttoolkit = '%s'\n", ret->trees[i].widgets[j].toolkit);
#endif
	}
    }
		
    IceDisposeCompleteMessage(iceConn, pdata);

    return ret;
}

FullQueryTreeReply *
HandleRapFullQueryTreeReply(IceConn iceConn)
{
    
    RapMessageHeader *header;
    char *pdata, *bufPointer;
    CARD32 numShells;
    register int i;
    FullQueryTreeReply *ret;
 
    ret = (FullQueryTreeReply *) malloc(sizeof(FullQueryTreeReply));
    memset(ret, '\0', sizeof(FullQueryTreeReply));
#ifdef RAP_DEBUG
    fprintf(stderr, "RECEIVED FULL QUERY TREE REPLY.\n"); fflush(stderr);
#endif
    IceReadCompleteMessage(iceConn,
                           SIZEOF(RapMessageHeader),
                           RapMessageHeader,
                           header,
                           pdata);
    
    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return NULL;
    }

#ifdef RAP_DEBUG
   fprintf(stderr," just finished reading complete message\n");
#endif

    bufPointer = pdata;

    EXTRACT_CARD32(bufPointer, FALSE, ret->numShells);
 
#ifdef RAP_DEBUG
    printf("NUM SHELLS: %d\n", ret->numShells);
#endif

    ret->trees = (FullTreeList) malloc(sizeof(FullTree) * ret->numShells);

    for (i=0 ; i<ret->numShells ; i++) {
        register int j;
 
        EXTRACT_CARD32(bufPointer, FALSE, ret->trees[i].numWidgets);
#ifdef RAP_DEBUG
        printf("+++++ NUM WIDGETS IN TREE %d: %d\n", i,
               ret->trees[i].numWidgets);
#endif
 
        ret->trees[i].widgets = (FullTreeWidgetList)
            malloc(sizeof(FullTreeWidget) * ret->trees[i].numWidgets);
 
        for (j=0 ; j<ret->trees[i].numWidgets ; j++) {
            register int k;

            EXTRACT_CARD32(bufPointer, FALSE, ret->trees[i].widgets[j].widget);
            EXTRACT_CARD32(bufPointer, FALSE, ret->trees[i].widgets[j].parent);
            EXTRACT_STRING(bufPointer, FALSE, ret->trees[i].widgets[j].name);
            EXTRACT_STRING(bufPointer, FALSE, ret->trees[i].widgets[j].className);
            EXTRACT_CARD32(bufPointer, FALSE, ret->trees[i].widgets[j].window);
            EXTRACT_CARD32(bufPointer, FALSE, ret->trees[i].widgets[j].managed);
     	    EXTRACT_STRING(bufPointer, FALSE, ret->trees[i].widgets[j].toolkit);

#ifdef RAP_DEBUG
            printf("---------------------------\n");
            printf("\twidget = %lx\n", ret->trees[i].widgets[j].widget);
            printf("\tparent = %lx\n", ret->trees[i].widgets[j].parent);
            printf("\tname = '%s'\n", ret->trees[i].widgets[j].name);
            printf("\tclassName = '%s'\n", ret->trees[i].widgets[j].className);
            printf("\twindow = %lx\n", ret->trees[i].widgets[j].window);
            printf("\tmanaged = %lx\n", ret->trees[i].widgets[j].managed);
            printf("\ttoolkit = '%s'\n", ret->trees[i].widgets[j].toolkit);
#endif

	    /* now the tough stuff, getting the resources ...  */
           EXTRACT_CARD32(bufPointer, FALSE,
                         ret->trees[i].widgets[j].numResources);
#ifdef RAP_DEBUG
           printf("\tnum resources  = '%ld'\n",
                        ret->trees[i].widgets[j].numResources);
#endif
          ret->trees[i].widgets[j].resourceData = (ResourceValueList)
                malloc(sizeof(ResourceValue) *
                       ret->trees[i].widgets[j].numResources);

           for (k=0 ; k<ret->trees[i].widgets[j].numResources; k++) {
	 
	        CARD32 resType;
 
                EXTRACT_STRING(bufPointer, FALSE,
			       ret->trees[i].widgets[j].resourceData[k].name);
                EXTRACT_STRING(bufPointer, FALSE,
                           ret->trees[i].widgets[j].resourceData[k].className);
                EXTRACT_STRING(bufPointer, FALSE,
                           ret->trees[i].widgets[j].resourceData[k].nativeType);
                EXTRACT_STRING(bufPointer, FALSE,
                           ret->trees[i].widgets[j].resourceData[k].returnType);
                EXTRACT_CARD32(bufPointer, FALSE, resType);
 
                ret->trees[i].widgets[j].resourceData[k].kind =
                    (RapResourceKind) resType;

#ifdef RAP_DEBUG
                printf("\t%d name = '%s'\n", k,
                           ret->trees[i].widgets[j].resourceData[k].name);
                printf("\t%d className = '%s'\n", k,
                           ret->trees[i].widgets[j].resourceData[k].className);
                printf("\t%d type = '%s'\n", k,
                           ret->trees[i].widgets[j].resourceData[k].nativeType);
                printf("\t%d type = '%s'\n", k,
                           ret->trees[i].widgets[j].resourceData[k].returnType);
                printf("\t%d kind = %ld\n", k, resType);
#endif
		
		/* now the value size and actual value */
               GetResourceValue(&bufPointer, 
		&(ret->trees[i].widgets[j].resourceData[k].valueSize),
                &(ret->trees[i].widgets[j].resourceData[k].value));
		
	  }

	}
    }

   IceDisposeCompleteMessage(iceConn, pdata);
 
    return ret;
}



GetResourcesReply *
HandleRapGetResourcesReply(IceConn iceConn)
{
    RapMessageHeader *header;
    char *pdata, *bufPointer;
    register int i;
    GetResourcesReply *ret;
#ifdef RAP_DEBUG
    printf("RECEIVED GET RESOURCES REPLY\n");
#endif
    IceReadCompleteMessage(iceConn,
			   SIZEOF(RapMessageHeader),
			   RapMessageHeader,
			   header,
			   pdata);

    bufPointer = pdata;

#ifdef RAP_DEBUG
    printf("the message was %ld word 64's in length (%ld bytes)\n",
	   header->length, header->length << 3);
#endif
    
    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return NULL;
    }

    ret = (GetResourcesReply *) malloc(sizeof(GetResourcesReply));
    memset(ret, '\0', sizeof(GetResourcesReply));
    
    EXTRACT_CARD32(bufPointer, FALSE, ret->numWidgets);
#ifdef RAP_DEBUG
    printf("GET RESOURCES REPLY: NUM WIDGETS: %d\n", ret->numWidgets);
#endif

    ret->widgetResources = (WidgetResourcesList)
	malloc(sizeof(WidgetResources) * ret->numWidgets);

    for (i=0 ; i<ret->numWidgets ; i++) {
	EXTRACT_CARD32(bufPointer, FALSE, ret->widgetResources[i].widget);
	EXTRACT_CARD32(bufPointer, FALSE, ret->widgetResources[i].errorCode);

#ifdef RAP_DEBUG
	printf("\t%d widgeth = %lx, errorCode = %ld\n", i,
	       ret->widgetResources[i].widget,
	       ret->widgetResources[i].errorCode);
#endif

	if (ret->widgetResources[i].errorCode == RapNoError) {
	    register int j;
	    
	    /*NOTE that we're sending numResources always!? */
	    EXTRACT_CARD32(bufPointer, FALSE,
			   ret->widgetResources[i].data.r.numResources);

#ifdef RAP_DEBUG
	    printf("Widget %d has %ld resources\n", i,
		   ret->widgetResources[i].data.r.numResources);
#endif
	    
	    ret->widgetResources[i].data.r.resources = (ResourceList)
		malloc(sizeof(ResourceInfo) *
		       ret->widgetResources[i].data.r.numResources);

	    for (j=0 ; j<ret->widgetResources[i].data.r.numResources ; j++) {
		CARD32 resType;
		
		EXTRACT_STRING(bufPointer, FALSE,
			   ret->widgetResources[i].data.r.resources[j].name);
		EXTRACT_STRING(bufPointer, FALSE,
			   ret->widgetResources[i].data.r.resources[j].className);
		EXTRACT_STRING(bufPointer, FALSE,
			   ret->widgetResources[i].data.r.resources[j].type);
		EXTRACT_CARD32(bufPointer, FALSE, resType);

		ret->widgetResources[i].data.r.resources[j].kind =
		    (RapResourceKind) resType;

#ifdef RAP_DEBUG
		printf("\t%d name = '%s'\n", i,
			   ret->widgetResources[i].data.r.resources[j].name);
		printf("\t%d className = '%s'\n", i,
			   ret->widgetResources[i].data.r.resources[j].className);
		printf("\t%d type = '%s'\n", i,
			   ret->widgetResources[i].data.r.resources[j].type);
		printf("\t%d kind = %ld\n", i, resType);
#endif
	    }
	} else {
	    EXTRACT_STRING(bufPointer, FALSE,
			   ret->widgetResources[i].data.errorString);
#ifdef RAP_DEBUG
	    printf("\t%d ERROR CODE %d: '%s'\n", i,
		   ret->widgetResources[i].errorCode,
		   ret->widgetResources[i].data.errorString);
#endif
	}
    }
    
    IceDisposeCompleteMessage(iceConn, pdata);

    return ret;
}

GetGCValuesReply *
HandleRapGetGCValuesReply(IceConn iceConn)
{
    RapMessageHeader *header;
    char *pdata, *bufp;
    register int i;
    GetGCValuesReply *ret;
#ifdef RAP_DEBUG
    printf("RECEIVED GET GC VALUES REPLY\n");
#endif
    IceReadCompleteMessage(iceConn,
                           SIZEOF(RapMessageHeader),
                           RapMessageHeader,
                           header,
                           pdata);
 
    
    if (!IceValidIO(iceConn)) {
        fprintf(stderr, "BOGUS I/O on connection.\n");
        fflush(stderr);
        return 0;
    }
#ifdef RAP_DEBUG
   fprintf(stderr,"finished read complete message w/out a problem\n");
#endif
   bufp = pdata;

   /* now the fun part!! */
   ret = (GetGCValuesReply *) malloc (sizeof(GetGCValuesReply) );
   memset(ret, '\0', sizeof(GetResourcesReply));

   EXTRACT_CARD32(bufp, FALSE, ret->numGCs);

   ret->GCValues = (GCValuesList)
       malloc(sizeof(GCValues) * ret->numGCs);
#ifdef RAP_DEBUG
   printf("got %ld GCs\n", ret->numGCs);
#endif
   for (i=0 ; i<ret->numGCs ; i++) {
       EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].gc);
       EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].errorCode);


#ifdef RAP_DEBUG 
       printf("\t%d gc = %lx, errorCode = %ld\n", i,
	      ret->GCValues[i].gc,
	      ret->GCValues[i].errorCode);
#endif 
 
       if (ret->GCValues[i].errorCode == RapNoError) {
	   long dashes;
 
	/* get the structure!!  */
	
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.function);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.plane_mask);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.foreground);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.background);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.line_width);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.line_style);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.cap_style);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.join_style);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.fill_style);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.fill_rule);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.arc_mode);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.tile);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.stipple);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.ts_x_origin);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.ts_y_origin);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.font);
	   EXTRACT_CARD32(bufp, FALSE,
			  ret->GCValues[i].data.gcv.subwindow_mode);
	   EXTRACT_CARD32(bufp, FALSE,
			  ret->GCValues[i].data.gcv.graphics_exposures);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.clip_x_origin);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.clip_y_origin);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.clip_mask);
	   EXTRACT_CARD32(bufp, FALSE, ret->GCValues[i].data.gcv.dash_offset);
	   EXTRACT_CARD32(bufp, FALSE, dashes);

	   ret->GCValues[i].data.gcv.dashes = (char) dashes;	
       } else {
	   EXTRACT_STRING(bufp, FALSE,
			  ret->GCValues[i].data.errorString);


#ifdef RAP_DEBUG
	   printf("\t%d ERROR CODE %d: '%s'\n", i,
		  ret->GCValues[i].errorCode,
		  ret->GCValues[i].data.errorString);
#endif
       }
   }

   IceDisposeCompleteMessage(iceConn, pdata);
 
   return ret;
}

LocateObjectReply *
HandleRapLocateObjectReply(IceConn iceConn)
{
    RapMessageHeader *header;
    char *pdata, *bufp;
    register int i;
    LocateObjectReply *ret;
    
    IceReadCompleteMessage(iceConn,
			   SIZEOF(RapMessageHeader),
			   RapMessageHeader,
			   header,
			   pdata);
    
    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return 0;
    }

    bufp = pdata;

    ret = (LocateObjectReply *) malloc(sizeof(LocateObjectReply));
    memset(ret, '\0', sizeof(LocateObjectReply));

    EXTRACT_CARD32(bufp, FALSE, ret->numLocations);

    ret->locations = (LocateObjectList) malloc(sizeof(LocateObjectStruct) *
					       ret->numLocations);

    for (i=0 ; i<ret->numLocations ; i++) {
	EXTRACT_CARD32(bufp, FALSE, ret->locations[i].widget);
	EXTRACT_CARD32(bufp, FALSE, ret->locations[i].errorCode);

	if (ret->locations[i].errorCode == RapNoError) {
	    char *nativeType, *returnType;

	    EXTRACT_CARD32(bufp, FALSE, ret->locations[i].data.location.window);
	    EXTRACT_CARD32(bufp, FALSE,
			   ret->locations[i].data.location.managed);
	    EXTRACT_CARD16(bufp, FALSE, ret->locations[i].data.location.x);
	    EXTRACT_CARD16(bufp, FALSE, ret->locations[i].data.location.y);
	    EXTRACT_CARD16(bufp, FALSE, ret->locations[i].data.location.width);
	    EXTRACT_CARD16(bufp, FALSE, ret->locations[i].data.location.height);
	    EXTRACT_CARD16(bufp, FALSE,
			   ret->locations[i].data.location.borderWidth);
	} else {
	    char *errorString;
	    EXTRACT_STRING(bufp, FALSE, ret->locations[i].data.errorString);
	}
    }

    IceDisposeCompleteMessage(iceConn, pdata);

    return ret;
}


GetValuesReply *
HandleRapGetValuesReply(IceConn iceConn)
{
    RapMessageHeader *header;
    char *pdata, *bufPointer;
    register int i;
    GetValuesReply *ret;
#ifdef RAP_DEBUG
    printf("RECEIVED GET VALUES REPLY\n");
#endif
    IceReadCompleteMessage(iceConn,
			   SIZEOF(RapMessageHeader),
			   RapMessageHeader,
			   header,
			   pdata);
    
    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return 0;
    }

    bufPointer = pdata;

    ret = (GetValuesReply *) malloc(sizeof(GetValuesReply));
    memset(ret, '\0', sizeof(GetValuesReply));

    EXTRACT_CARD32(bufPointer, FALSE, ret->widget);
    EXTRACT_CARD32(bufPointer, FALSE, ret->numValues);
#ifdef RAP_DEBUG
    printf("WIDGET: %lx\n", ret->widget);
    printf("NUM ENTRIES: %ld\n", ret->numValues);
#endif

    ret->values = (ValueList) malloc(sizeof(Value) * ret->numValues);

    for (i=0 ; i<ret->numValues ; i++) {
	EXTRACT_STRING(bufPointer, FALSE, ret->values[i].name);
#ifdef RAP_DEBUG
	printf("\tNAME = '%s'\n", ret->values[i].name);
#endif
	EXTRACT_CARD32(bufPointer, FALSE, ret->values[i].errorCode);

	if (ret->values[i].errorCode == RapNoError) {
	    char *nativeType, *returnType;
	    
	    EXTRACT_STRING(bufPointer, FALSE,
			   ret->values[i].data.v.nativeType);
#ifdef RAP_DEBUG
	    printf("\tNATIVE TYPE = '%s'\n",
		   ret->values[i].data.v.nativeType);
#endif
	    EXTRACT_STRING(bufPointer, FALSE,
			   ret->values[i].data.v.returnType);
#ifdef RAP_DEBUG
	    printf("\tRETURN TYPE = '%s'\n",
		   ret->values[i].data.v.returnType);
#endif

	    GetResourceValue(&bufPointer, &(ret->values[i].data.v.valueSize),
			     &(ret->values[i].data.v.value));
	} else {
	    char *errorString;
	    EXTRACT_STRING(bufPointer, FALSE, ret->values[i].data.errorString);
#ifdef RAP_DEBUG
	    printf("\tERROR CODE %d: '%s'\n", ret->values[i].errorCode,
		   ret->values[i].data.errorString);
#endif
	}
    }

    IceDisposeCompleteMessage(iceConn, pdata);

    return ret;
}


CreateNotif *
HandleRapCreateNotify(IceConn iceConn)
{
    RapMessageHeader *header;
    char *pdata, *bufPointer;
    register int i;
    CreateNotif *ret;
#ifdef RAP_DEBUG
    printf("HANDLE RAP CREATE NOTIFY RECEIVED!!\n");
#endif
    IceReadCompleteMessage(iceConn,
			   SIZEOF(RapMessageHeader),
			   RapMessageHeader,
			   header,
			   pdata);

    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return 0;
    }

    ret = (CreateNotif *) malloc(sizeof(CreateNotif));
    memset(ret, '\0', sizeof(CreateNotif));

    bufPointer = pdata;
    
    EXTRACT_STRING(bufPointer, FALSE, ret->type);
    EXTRACT_CARD32(bufPointer, FALSE, ret->widget);
    EXTRACT_CARD32(bufPointer, FALSE, ret->parent);
    EXTRACT_STRING(bufPointer, FALSE, ret->name);
    EXTRACT_STRING(bufPointer, FALSE, ret->className);
    EXTRACT_CARD32(bufPointer, FALSE, ret->window);
    EXTRACT_STRING(bufPointer, FALSE, ret->toolkit);
    EXTRACT_CARD32(bufPointer, FALSE, ret->numResources);

#ifdef RAP_DEBUG
    printf("Got message TYPE = '%s'\n", ret->type);
    printf("widget = %lx\n", ret->widget);
    printf("parent = %lx\n", ret->parent);
    printf("name = '%s'\n", ret->name);
    printf("className = '%s'\n", ret->className);
    printf("window = %lx\n", ret->window);
    printf("toolkit = '%s'\n", ret->toolkit);
    printf("num resources = %ld\n", ret->numResources);
#endif

    ret->resources = (CreateResourcesList)
	malloc(sizeof(CreateResources) * ret->numResources);

    for (i=0 ; i<ret->numResources ; i++) {
	CARD32 kind;
	
	EXTRACT_STRING(bufPointer, FALSE, ret->resources[i].name);
	EXTRACT_STRING(bufPointer, FALSE, ret->resources[i].className); 
	EXTRACT_STRING(bufPointer, FALSE, ret->resources[i].type);
	EXTRACT_CARD32(bufPointer, FALSE, kind);
	ret->resources[i].kind = (RapResourceKind) kind;
	EXTRACT_STRING(bufPointer, FALSE, ret->resources[i].returnType);

#ifdef RAP_DEBUG
	printf("RESOURCE %d\n", i);
	printf("\tname = '%s'\n", ret->resources[i].name);
	printf("\tclassName = '%s'\n", ret->resources[i].className);
	printf("\ttype = '%s'\n", ret->resources[i].type);
	printf("\tkind = %ld\n", kind);
	printf("\treturn type = '%s'\n", ret->resources[i].returnType);
#endif

	GetResourceValue(&bufPointer, &(ret->resources[i].valueSize),
			 &(ret->resources[i].value));
    }

    IceDisposeCompleteMessage(iceConn, pdata);
    return ret;
}

DestroyNotif *
HandleRapDestroyNotify(IceConn iceConn)
{
    RapMessageHeader *header;
    char *pdata, *bufPointer;
    DestroyNotif *ret;
#ifdef RAP_DEBUG
    printf("HANDLE RAP DESTROY NOTIFY\n");
#endif
    IceReadCompleteMessage(iceConn,
			   SIZEOF(RapMessageHeader),
			   RapMessageHeader,
			   header,
			   pdata);

    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return 0;
    }

    ret = (DestroyNotif *) malloc(sizeof(DestroyNotif));
    memset(ret, '\0', sizeof(DestroyNotif));

    bufPointer = pdata;
    
    EXTRACT_STRING(bufPointer, FALSE, ret->type);
    EXTRACT_CARD32(bufPointer, FALSE, ret->widget);
    
    IceDisposeCompleteMessage(iceConn, pdata);

#ifdef RAP_DEBUG
    printf("WIDGET %lx destroyed\n", ret->widget);
#endif
    return ret;
}

ConfigureNotif *
HandleRapConfigureNotify(IceConn iceConn)
{
    RapMessageHeader *header;
    char *pdata, *bufPointer;
    ConfigureNotif *ret;
#ifdef RAP_DEBUG
    printf("HANDLE RAP CONFIG NOTIFY\n");
#endif
    IceReadCompleteMessage(iceConn,
			   SIZEOF(RapMessageHeader),
			   RapMessageHeader,
			   header,
			   pdata);

    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return 0;
    }

    ret = (ConfigureNotif *) malloc(sizeof(ConfigureNotif));
    memset(ret, '\0', sizeof(ConfigureNotif));
    
    bufPointer = pdata;
    
    EXTRACT_STRING(bufPointer, FALSE, ret->type);
    EXTRACT_CARD32(bufPointer, FALSE, ret->widget);

    EXTRACT_CARD32(bufPointer, FALSE, ret->geometryMask);
    EXTRACT_CARD32(bufPointer, FALSE, ret->x);
    EXTRACT_CARD32(bufPointer, FALSE, ret->y);
    EXTRACT_CARD32(bufPointer, FALSE, ret->width);
    EXTRACT_CARD32(bufPointer, FALSE, ret->height);
    EXTRACT_CARD32(bufPointer, FALSE, ret->borderWidth);
    EXTRACT_CARD32(bufPointer, FALSE, ret->sibling);
    EXTRACT_CARD32(bufPointer, FALSE, ret->stackMode);

#ifdef RAP_DEBUG
    if (ret->geometryMask & CWX)
	printf("\tNEW X = %ld\n", ret->x);

    if (ret->geometryMask & CWY)
	printf("\tNEW Y = %ld\n", ret->y);

    if (ret->geometryMask & CWWidth)
	printf("\tNEW WIDTH = %ld\n", ret->width);

    if (ret->geometryMask & CWHeight)
	printf("\tNEW HEIGHT = %lx\n", ret->height);

    if (ret->geometryMask & CWBorderWidth)
	printf("\tNEW BORDERWIDTH = %lx\n", ret->borderWidth);

    if (ret->geometryMask & CWSibling)
	printf("\tNEW SIBLING = %lx\n", ret->sibling);

    if (ret->geometryMask & CWStackMode)
	printf("\tNEW STACKMODE = %lx\n", ret->stackMode);
#endif
    
    IceDisposeCompleteMessage(iceConn, pdata);

    return ret;
}

ChangeNotif *
HandleRapChangeNotify(IceConn iceConn)
{
    RapMessageHeader *header = NULL;
    char *pdata = NULL, *bufPointer = NULL;
    register int i = 0;
    ChangeNotif *ret = NULL;
#ifdef RAP_DEBUG
    printf("HANDLE RAP CHANGE NOTIFY\n");
#endif
    IceReadCompleteMessage(iceConn,
			   SIZEOF(RapMessageHeader),
			   RapMessageHeader,
			   header,
			   pdata);

    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return NULL;
    }

    ret = (ChangeNotif *) malloc(sizeof(ChangeNotif));
    memset(ret, '\0', sizeof(ChangeNotif));
    
    bufPointer = pdata;
    
    EXTRACT_STRING(bufPointer, FALSE, ret->type);
    EXTRACT_CARD32(bufPointer, FALSE, ret->widget);

#ifdef RAP_DEBUG
    printf("ChangeNotify: widget = %lx, type = %s\n", ret->widget, ret->type);
#endif
    if (strcmp(ret->type, XtHsetValues) == 0) {
	EXTRACT_CARD32(bufPointer, FALSE, ret->c.setValues.numResources);
/* #ifdef RAP_DEBUG */
	printf("\tSV: NUM RESOURCES = %ld\n", ret->c.setValues.numResources);
/* #endif */

	ret->c.setValues.resources = (CNSVResourceList)
	    malloc(sizeof(CNSVResource) * ret->c.setValues.numResources);
	
	for (i=0 ; i<ret->c.setValues.numResources ; i++) {
	    EXTRACT_STRING(bufPointer, FALSE,
			   ret->c.setValues.resources[i].name);
	    EXTRACT_STRING(bufPointer, FALSE,
			   ret->c.setValues.resources[i].type);
/* #ifdef RAP_DEBUG */
	    printf("\tSV: RESOURCE %d: %s (%s)\n", i,
		   ret->c.setValues.resources[i].name,
		   ret->c.setValues.resources[i].type);
/* #endif */
	    
	    GetResourceValue(&bufPointer,
			     &(ret->c.setValues.resources[i].valueSize),
			     &(ret->c.setValues.resources[i].value));
	}
    } else if (strcmp(ret->type, XtHmanageChildren) == 0) {
	EXTRACT_CARD32(bufPointer, FALSE, ret->c.manageChildren.numWidgets);
#ifdef RAP_DEBUG
	printf("\tNUM WIDGETS = %ld\n", ret->c.manageChildren.numWidgets);
#endif
	ret->c.manageChildren.widgets = (CARD32List)
	    malloc(sizeof(CARD32) * ret->c.manageChildren.numWidgets);
	for (i=0 ; i<ret->c.manageChildren.numWidgets ; i++) {
	    EXTRACT_CARD32(bufPointer, FALSE, ret->c.manageChildren.widgets[i]);
#ifdef RAP_DEBUG
	    printf("\tWIDGET %d: %lx\n", i, ret->c.manageChildren.widgets[i]);
#endif
	}
    } else if (strcmp(ret->type, XtHunmanageChildren) == 0) {
	EXTRACT_CARD32(bufPointer, FALSE, ret->c.unmanageChildren.numWidgets);
#ifdef RAP_DEBUG
	printf("\tNUM WIDGETS = %ld\n", ret->c.unmanageChildren.numWidgets);
#endif
	ret->c.unmanageChildren.widgets = (CARD32List)
	    malloc(sizeof(CARD32) * ret->c.unmanageChildren.numWidgets);
	for (i=0 ; i<ret->c.unmanageChildren.numWidgets ; i++) {
	    EXTRACT_CARD32(bufPointer, FALSE,
			   ret->c.unmanageChildren.widgets[i]);
#ifdef RAP_DEBUG
	    printf("\tWIDGET %d: %lx\n", i, ret->c.unmanageChildren.widgets[i]);
#endif
	}
    } else if (strcmp(ret->type, XtHunmanageSet) == 0) {
	EXTRACT_CARD32(bufPointer, FALSE, ret->c.unmanageSet.numWidgets);
#ifdef RAP_DEBUG
	printf("\tNUM WIDGETS = %ld\n", ret->c.unmanageSet.numWidgets);
#endif
	ret->c.unmanageSet.widgets = (CARD32List)
	    malloc(sizeof(CARD32) * ret->c.unmanageSet.numWidgets);
	for (i=0 ; i<ret->c.unmanageSet.numWidgets ; i++) {
	    EXTRACT_CARD32(bufPointer, FALSE, ret->c.unmanageSet.widgets[i]);
#ifdef RAP_DEBUG
	    printf("\tWIDGET %d: %lx\n", i, ret->c.unmanageSet.widgets[i]);
#endif
	}
    } else if (strcmp(ret->type, XtHmanageSet) == 0) {
	EXTRACT_CARD32(bufPointer, FALSE, ret->c.manageSet.numWidgets);
#ifdef RAP_DEBUG
	printf("\tNUM WIDGETS = %ld\n", ret->c.manageSet.numWidgets);
#endif
	ret->c.manageSet.widgets = (CARD32List)
	    malloc(sizeof(CARD32) * ret->c.manageSet.numWidgets);
	for (i=0 ; i<ret->c.manageSet.numWidgets ; i++) {
	    EXTRACT_CARD32(bufPointer, FALSE, ret->c.manageSet.widgets[i]);
#ifdef RAP_DEBUG
	    printf("\tWIDGET %d: %lx\n", i, ret->c.manageSet.widgets[i]);
#endif
	}
    } else if (strcmp(ret->type, XtHrealizeWidget) == 0) {
	EXTRACT_CARD32(bufPointer, FALSE, ret->c.realizeWidget.window);
#ifdef RAP_DEBUG
	printf("\tWINDOW = %lx\n", ret->c.realizeWidget.window);
#endif
    } else if (strcmp(ret->type, XtHunrealizeWidget) == 0) {
	/* no event data */
    } else if (strcmp(ret->type, XtHaddCallback) == 0) {
	EXTRACT_STRING(bufPointer, FALSE, ret->c.addCallback.callbackName);
#ifdef RAP_DEBUG
	printf("\tCALLBACK NAME = %s\n", ret->c.addCallback.callbackName);
#endif
    } else if (strcmp(ret->type, XtHaddCallbacks) == 0) {
	EXTRACT_STRING(bufPointer, FALSE, ret->c.addCallbacks.callbackName);
#ifdef RAP_DEBUG
	printf("\tCALLBACK NAME = %s\n", ret->c.addCallbacks.callbackName);
#endif
    } else if (strcmp(ret->type, XtHremoveCallback) == 0) {
	EXTRACT_STRING(bufPointer, FALSE, ret->c.removeCallback.callbackName);
#ifdef RAP_DEBUG
	printf("\tCALLBACK NAME = %s\n", ret->c.removeCallback.callbackName);
#endif
    } else if (strcmp(ret->type, XtHremoveCallbacks) == 0) {
	EXTRACT_STRING(bufPointer, FALSE, ret->c.removeCallbacks.callbackName);
#ifdef RAP_DEBUG
	printf("\tCALLBACK NAME = %s\n", ret->c.removeCallbacks.callbackName);
#endif
    } else if (strcmp(ret->type, XtHremoveAllCallbacks) == 0) {
	/* no event data */
    } else if (strcmp(ret->type, XtHaugmentTranslations) == 0) {
	/* no event data */
    } else if (strcmp(ret->type, XtHoverrideTranslations) == 0) {
	/* no event data */
    } else if (strcmp(ret->type, XtHuninstallTranslations) == 0) {
	/* no event data */
    } else if (strcmp(ret->type, XtHsetKeyboardFocus) == 0) {
	EXTRACT_CARD32(bufPointer, FALSE, ret->c.setKeyboardFocus.descendant);
#ifdef RAP_DEBUG
	printf("\tDESCENDANT = %lx\n", ret->c.setKeyboardFocus.descendant);
#endif
    } else if (strcmp(ret->type, XtHsetWMColormapWindows) == 0) {
	CARD32 numWidgets;
	EXTRACT_CARD32(bufPointer, FALSE,
		       ret->c.setWMColormapWindows.numWidgets);
	ret->c.setWMColormapWindows.widgets = (CARD32List)
	    malloc(sizeof(CARD32) * ret->c.setWMColormapWindows.numWidgets);
#ifdef RAP_DEBUG
	printf("\tNUM WIDGETS = %ld\n",
	       ret->c.setWMColormapWindows.numWidgets);
#endif
	for (i=0 ; i<ret->c.setWMColormapWindows.numWidgets ; i++) {
	    EXTRACT_CARD32(bufPointer, FALSE,
			   ret->c.setWMColormapWindows.widgets[i]);
#ifdef RAP_DEBUG
	    printf("\tWIDGET %d: %lx\n", i,
		   ret->c.setWMColormapWindows.widgets[i]);
#endif
	}
    } else if (strcmp(ret->type, XtHsetMappedWhenManaged) == 0) {
	CARD32 bbool;
	EXTRACT_CARD32(bufPointer, FALSE, bbool);
	ret->c.setMappedWhenManaged.mappedWhenManaged = (Boolean) bbool;
#ifdef RAP_DEBUG
	printf("\tBOOL = %ld\n", ret->c.setMappedWhenManaged.mappedWhenManaged);
#endif
    } else if (strcmp(ret->type, XtHmapWidget) == 0) {
	/* no event data */
    } else if (strcmp(ret->type, XtHunmapWidget) == 0) {
	/* no event data */
    } else if (strcmp(ret->type, XtHpopup) == 0) {
	CARD32 grabKind;
	EXTRACT_CARD32(bufPointer, FALSE, grabKind);
	ret->c.popup.grabKind = (XtGrabKind) grabKind;
#ifdef RAP_DEBUG
	printf("\tGRAB KIND = %lx\n", ret->c.popup.grabKind);
#endif
    } else if (strcmp(ret->type, XtHpopupSpringLoaded) == 0) {
	/* no event data */
    } else if (strcmp(ret->type, XtHpopdown) == 0) {
	/* no event data */
    } else {
	printf("UNKNOWN CHANGE HOOK TYPE: %s\n", ret->type);
    }
    
    IceDisposeCompleteMessage(iceConn, pdata);

    return ret;
}

GeometryNotif *
HandleRapGeometryNotify(IceConn iceConn)
{
    RapMessageHeader *message = NULL;
    GeometryNotif *ret;
    char *pdata, *bufPointer;
    char *type;
    CARD32 result;
    CARD32 width, height, border_width, sibling;
    CARD32 garbage;
#ifdef RAP_DEBUG
    printf("RECEIVED GEOMETRY NOTIFY\n");
#endif
    IceReadCompleteMessage(iceConn,
			   SIZEOF(RapMessageHeader),
			   RapMessageHeader,
			   message,
			   pdata);
    
    if (!IceValidIO(iceConn)) {
	fprintf(stderr, "BOGUS I/O on connection.\n");
	fflush(stderr);
	return NULL;
    }
    
    bufPointer = pdata;

    EXTRACT_STRING(bufPointer, FALSE, type);
#ifdef RAP_DEBUG
    printf("\tTYPE = %s\n", type);
#endif

    /*
     * For now, we only pass XtHpostGeometry messages up to the agent.
     */
    if (strcmp(type, XtHpostGeometry) != 0) {
	free(type);
	IceDisposeCompleteMessage(iceConn, pdata);
	return NULL;
    }
    
    ret = (GeometryNotif *) malloc(sizeof(GeometryNotif));
    memset(ret, '\0', sizeof(GeometryNotif));
    
    EXTRACT_CARD32(bufPointer, FALSE, ret->widget);
#ifdef RAP_DEBUG
    printf("\tWIDGET = %lx\n", ret->widget);
#endif
    /*
     * Ignore it if result != XtGeometryYes
     */
    EXTRACT_CARD32(bufPointer, FALSE, result);
#ifdef RAP_DEBUG
    printf("\tRESULT = %ld\n", result);

    printf("XtGeometryYes = %d, No = %d, Almost = %d, Done = %d\n",
	   XtGeometryYes, XtGeometryNo, XtGeometryAlmost, XtGeometryDone);
#endif
    if (result != XtGeometryYes) {
	#ifdef RAP_DEBUG
	printf("HandleRapGeometryNotify: result != XtGeometryYes, result = %ld\n",
	       result);
#endif
	free(type);
	free(ret);
	IceDisposeCompleteMessage(iceConn, pdata);
	return NULL;
    }
    
    ret->type = type;

    /*
     * NOTE - this is my understanding of how this stuff works.  The
     * application calls XtMakeGeometryRequest to request that its parent
     * change its size.  The parent will return either No, Yes, or Almost
     * to this call.
     *
     * Yes indicates that the request succeeded.
     * No indicates that it did not.
     * Almost indicates a compromise geometry.
     *
     * The caller can pass NULL in where the reply would be stored to
     * indicate that it is not interested in Almost values.
     *
     * This code does the following:
     *    - Ignores notifications that are not of type Yes.
     *	  - Uses the *request* fields (not the reply) to set the
     *	    geometry of the widget.
     *
     * Note that this is different than how I used to do it - I used to
     * use the reply fields, assuming that they contained the "correct"
     * values.  But they may be NULL.  We'll see how using the request
     * fields work instead...
     *
     *			-keith edwards
     */
     
#ifdef STILL_USING_REPLY
    /*****
     ***** THIS  IS THE OLD CODE
     *****/
    /*
     * We read in CARD32's to skip over the crap we don't want.
     * (Inefficient)
     */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* request_mode */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* x */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* y */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* width */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* height */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* border_width */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* sibling */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* stack_mode */

    EXTRACT_CARD32(bufPointer, FALSE, ret->final.request_mode);
    EXTRACT_CARD32(bufPointer, FALSE, ret->final.x); 
    EXTRACT_CARD32(bufPointer, FALSE, ret->final.y);  
    EXTRACT_CARD32(bufPointer, FALSE, width);
    EXTRACT_CARD32(bufPointer, FALSE, height);
    EXTRACT_CARD32(bufPointer, FALSE, border_width);
    EXTRACT_CARD32(bufPointer, FALSE, sibling);
    EXTRACT_CARD32(bufPointer, FALSE, ret->final.stack_mode);

    ret->final.width = (Dimension) width;
    ret->final.height = (Dimension) height;
    ret->final.border_width = (Dimension) border_width;
    ret->final.sibling = (Widget) sibling;

#else 
    /*****
     ***** THIS IS THE NEW CODE.
     *****/

    /* NOTE
     * We stuff the result in "final" because that's what the caller
     * code is expecting (even though we're not really using the reply
     * value here, we're using request).  We should change this, and
     * change the users of this code to be in sync.
     */
    EXTRACT_CARD32(bufPointer, FALSE, ret->final.request_mode);
    EXTRACT_CARD32(bufPointer, FALSE, ret->final.x); 
    EXTRACT_CARD32(bufPointer, FALSE, ret->final.y);  
    EXTRACT_CARD32(bufPointer, FALSE, width);
    EXTRACT_CARD32(bufPointer, FALSE, height);
    EXTRACT_CARD32(bufPointer, FALSE, border_width);
    EXTRACT_CARD32(bufPointer, FALSE, sibling);
    EXTRACT_CARD32(bufPointer, FALSE, ret->final.stack_mode);

    ret->final.width = (Dimension) width;
    ret->final.height = (Dimension) height;
    ret->final.border_width = (Dimension) border_width;
    ret->final.sibling = (Widget) sibling;

    /*
     * We read in CARD32's to skip over the crap we don't want.
     * (Inefficient)
     */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* request_mode */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* x */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* y */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* width */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* height */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* border_width */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* sibling */
    EXTRACT_CARD32(bufPointer, FALSE, garbage);	/* stack_mode */
#endif 

    IceDisposeCompleteMessage(iceConn, pdata);
#ifdef RAP_DEBUG
    printf("\t...returning from geometry notifyhandler\n");
#endif
    return ret;
}

RequestNotif *
HandleRapRequestNotify(IceConn iceConn)
{
   RapMessageHeader  *message = NULL;
   xReq             *incoming_req;
   char               *pdata;
   RequestNotif		*ret;
   int			done = 0;
 
   /* read the complete message */
#ifdef RAP_DEBUG
   printf("RECEIVED REQUEST NOTIFY\n");
#endif
 
   IceReadCompleteMessage(iceConn,
                           SIZEOF(RapMessageHeader),
                           RapMessageHeader,
                           message,
                           pdata);
 
   incoming_req = (xReq *) pdata;

#ifdef RAP_DEBUG
    fprintf(stderr, "agent just got requst of type %d\n",
                incoming_req->reqType);

   printf("Request type = %d, length = %d, msg length = %d\n",
	  incoming_req->reqType,
	  incoming_req->length,
	  message->length << 3);
#endif
   ret = (RequestNotif *) malloc(sizeof(RequestNotif));
   ret->type = incoming_req->reqType;
   ret->request = (xReq *) malloc(incoming_req->length * REQ_UNITS_IN_BYTES);
   memcpy(ret->request, incoming_req,
	  incoming_req->length * REQ_UNITS_IN_BYTES);

   return ret;
}

EventNotif *
HandleRapEventNotify(IceConn iceConn)
{
   RapMessageHeader   *message = NULL;
   xEvent             *incoming_event;
   char               *pdata;
   EventNotif	      *ret;
 
   /* read the complete message */
#ifdef RAP_DEBUG
   printf("RECEIVED EVENT NOTIFY\n");
#endif
 
   IceReadCompleteMessage(iceConn,
                           SIZEOF(RapMessageHeader),
                           RapMessageHeader,
                           message,
                           pdata);
 
   incoming_event = (xEvent *) pdata;
 
#ifdef RAP_DEBUG
    fprintf(stderr,"got a rap event notify\n");
   /* just print it out for now  */
    fprintf(stderr, "agent just got event notify for event %d\n",
                incoming_event->u.u.type);
#endif

   ret = (EventNotif *) malloc(sizeof(EventNotif));
   ret->type = incoming_event->u.u.type;
   ret->event = (xEvent *) malloc(sizeof(xEvent));
   memcpy(ret->event, pdata, sizeof(xEvent));
    
    IceDisposeCompleteMessage(iceConn, pdata);

    return ret;
}


int
HandleRapAcknowledge(IceConn iceConn)
{
    RapAckMessage *ack_msg = NULL;

#ifdef RAP_DEBUG
    printf("RECEIVED AN ACK\n");
    printf("got rap ack ... big deal just read it in\n");
#endif
   /* is it ok to just use sizeof?  I padded it inside defn of msg header .. */
   /* or do I need a SIZEOF defn here too? */
   IceReadMessageHeader(iceConn,
                        sizeof(RapAckMessage),
                        RapAckMessage,
                        ack_msg);

#ifdef RAP_DEBUG
    printf("just after rap ack\n");
#endif
}
 
int
HandleRapError(IceConn iceConn)
{
    RapErrMessage *msg_header = NULL;
    char          *error_buf;
#ifdef RAP_DEBUG
    printf("RECEIVED AN ERROR\n");
#endif
    IceReadCompleteMessage(iceConn,
                           SIZEOF(RapMessageHeader),
                           RapErrMessage,
                           msg_header,
                           error_buf);
 
#ifdef RAP_DEBUG
   fprintf(stderr, "well, we got an error from the client ...");
   fprintf(stderr, "should we do anything?\n");
#endif
    
    IceDisposeCompleteMessage(iceConn, error_buf);
}

