/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/

/* We have to have this for the xEvent struct to be defined in Xproto.h */
#define NEED_EVENTS

#include <X11/Xproto.h>
#include <X11/Intrinsic.h>
#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICEconn.h>
#include "RapProto.h"

#ifndef _AGENT_H_
#define _AGENT_H_


typedef struct _RapClientStruct {
    Window		window;
    IceConn		iceConn;
    int			majorOpcode;
    Status		connectStatus;
    int			fd;
    struct _RapClientStruct *next;
} RapClientStruct, *RapClientTable, *RapClient;

/*
* These struct types are used to return data from the incoming messages.
*/

typedef struct _HelloReply {
    CARD32 window;
} HelloReply;

typedef CARD32 *CARD32List;

typedef struct _TreeWidget {
    CARD32		widget;
    CARD32		parent;
    char		*name;
    char		*className;
    CARD32		window;
    CARD32		managed;
    char		*toolkit;
} TreeWidget, *TreeWidgetList;

typedef struct _ResourceValue {
    char                *name;
    char                *className;
    char                *nativeType;
    char                *returnType;
    RapResourceKind     kind;
    CARD32		valueSize;
    char		*value;
} ResourceValue, *ResourceValueList;

typedef struct _FullTreeWidget {
    CARD32		  widget;
    CARD32		  numResources;
    ResourceValueList     resourceData;
    CARD32		  parent;
    char		  *name;
    char		  *className;
    CARD32		  window;
    CARD32		  managed;
    char		  *toolkit;
} FullTreeWidget, *FullTreeWidgetList;

typedef struct _FullTree {
    CARD32		numWidgets;
    FullTreeWidgetList	widgets;
} FullTree, *FullTreeList;


typedef struct _Tree {
    CARD32		numWidgets;
    TreeWidgetList	widgets;
} Tree, *TreeList;

typedef struct _QueryTreeReply {
    CARD32		numShells;
    TreeList		trees;
} QueryTreeReply;

typedef struct _FullQueryTreeReply {
    CARD32		numShells;
    FullTreeList	trees;
} FullQueryTreeReply;

/*****/

typedef struct _ResourcesInfo {
    char		*name;
    char		*className;
    char		*type;
    RapResourceKind	kind;
} ResourceInfo, *ResourceList;

typedef struct _WidgetResources {
    CARD32		widget;
    CARD32		errorCode;
    union {
	char		*errorString;
	struct {
	    CARD32	numResources;
	    ResourceList resources;
	} r;
    } data;
} WidgetResources, *WidgetResourcesList;

typedef struct _GetResourcesReply {
    CARD32		numWidgets;
    WidgetResourcesList	widgetResources;
} GetResourcesReply;

typedef struct _GCValuesList{
    CARD32              gc;
    CARD32              errorCode;
    union {
        char            *errorString;
        XGCValues       gcv;
    } data;
} GCValues, *GCValuesList;
 
 
typedef struct _GetGCValuesReply {
    CARD32              numGCs;
    GCValuesList        GCValues;
} GetGCValuesReply;
 


/*****/

typedef struct _LocateObjectRequest {
    CARD32		numWidgets;
    CARD32List		widgets;
} LocateObjectRequest;

typedef struct _LocationData {
    CARD32		window;
    CARD32		managed;
    INT16		x, y;
    CARD16		width, height;
    CARD16		borderWidth;
} LocationData;

typedef struct _LocateObjectStruct { 
    CARD32		widget;
    CARD32		errorCode;
    union {
	char		*errorString;
	LocationData	location;
    } data;
} LocateObjectStruct, *LocateObjectList;

typedef struct _LocateObjectReply {
    CARD32		numLocations;
    LocateObjectList	locations;
} LocateObjectReply;

/*****/

typedef struct _Value {
    char		*name;
    CARD32		errorCode;
    union {
	char		*errorString;
	struct {
	    char	*nativeType;
	    char	*returnType;
	    CARD32	valueSize;
	    char	*value;
	} v;
    } data;
} Value, *ValueList;

typedef struct _GetValuesReply {
    CARD32		widget;
    CARD32		numValues;
    ValueList		values;
} GetValuesReply;

/*****/

typedef struct _CreateResourcesList {
    char		*name;
    char		*className;
    char		*type;
    RapResourceKind	kind;
    char		*returnType;
    CARD32		valueSize;
    char		*value;
} CreateResources, *CreateResourcesList;

typedef struct _CreateNotif {
    char		*type;
    CARD32		widget;
    CARD32		parent;
    char		*name;
    char		*className;
    CARD32		window;
    char		*toolkit;
    CARD32		numResources;
    CreateResourcesList	resources;
} CreateNotif;

/*****/

typedef struct _DestroyNotif {
    char		*type;
    CARD32		widget;
} DestroyNotif;

/*****/

typedef struct _ConfigureNotif {
    char		*type;
    CARD32		widget;
    XtGeometryMask	geometryMask;
    CARD32		x;
    CARD32		y;
    CARD32		width;
    CARD32		height;
    CARD32		borderWidth;
    CARD32		sibling;
    CARD32		stackMode;
} ConfigureNotif;

/*****/

typedef struct _GeometryNotif {
    char		*type;
    CARD32		widget;
    XtWidgetGeometry	final;
} GeometryNotif;

/*****/

typedef struct _CNSVResource {
    char		*name;
    char		*type;
    CARD32		valueSize;
    char		*value;
} CNSVResource, *CNSVResourceList;

typedef struct _CNSetValues {
    CARD32		numResources;
    CNSVResourceList	resources;
}CNSetValues;

typedef struct _CNManageChildren {
    CARD32		numWidgets;
    CARD32List		widgets;
} CNManageChildren;

typedef struct _CNUnmanageChildren {
    CARD32		numWidgets;
    CARD32List		widgets;
} CNUnmanageChildren;

typedef struct _CNManageSet {
    CARD32		numWidgets;
    CARD32List		widgets;
} CNManageSet;

typedef struct _CNUnmanageSet {
    CARD32		numWidgets;
    CARD32List		widgets;
} CNUnmanageSet;

typedef struct _CNRealizeWidget {
    CARD32		window;
} CNRealizeWidget;

typedef struct _CNAddCallback {
    char		*callbackName;
} CNAddCallback;

typedef struct _CNAddCallbacks {
    char		*callbackName;
} CNAddCallbacks;

typedef struct _CNRemoveCallback {
    char		*callbackName;
} CNRemoveCallback;

typedef struct _CNRemoveCallbacks {
    char		*callbackName;
} CNRemoveCallbacks;

typedef struct _CNSetKeyboardFocus {
    CARD32		descendant;
} CNSetKeyboardFocus;

typedef struct _CNSetWMColormapWindows {
    CARD32		numWidgets;
    CARD32List		widgets;
} CNSetWMColormapWindows;

typedef struct _CNSetMappedWhenManaged {
    Boolean		mappedWhenManaged;
} CNSetMappedWhenManaged;

typedef struct _CNPopup {
    XtGrabKind		grabKind;
} CNPopup;

typedef struct _ChangeNotif {
    char		*type;
    CARD32		widget;
    union {
	CNSetValues		setValues;
	CNManageChildren 	manageChildren;
	CNUnmanageChildren 	unmanageChildren;
	CNManageSet		manageSet;
	CNUnmanageSet		unmanageSet;
	CNRealizeWidget		realizeWidget;
	CNAddCallback		addCallback;
	CNAddCallbacks		addCallbacks;
	CNRemoveCallback 	removeCallback;
	CNRemoveCallbacks 	removeCallbacks;
	CNSetKeyboardFocus 	setKeyboardFocus;
	CNSetWMColormapWindows	setWMColormapWindows;
	CNSetMappedWhenManaged	setMappedWhenManaged;
	CNPopup			popup;
    } c;
} ChangeNotif;

typedef struct _RequestNotif {
    unsigned char		type;
    xReq			*request;
} RequestNotif;

typedef struct _EventNotif {
    unsigned char		type;
    xEvent			*event; 
} EventNotif;

/*****/

int 			RAPHelloRequest(RapClient);
int 			RAPQueryTreeRequest(RapClient);
int 			RAPFullQueryTreeRequest(RapClient);
int			RAPGetResourcesRequest(RapClient,
					       WidgetList widgets,
					       long numWidgets);
int			RAPGetGCValuesRequest(RapClient,
					      GContext *GCs,
					      long numGCs);
int 			RAPGetValuesRequest(RapClient,
					    Widget widget,
					    const char **resourceNames,
					    long numResources);
int 			RAPSetValuesRequest(RapClient,
					    const char *name,
					    const char *type,
					    const char *value,
					    WidgetList widgets,
					    long numWidgets);
int 			RAPAddNotifyRequest(RapClient, short *, short);
int 			RAPRemoveNotifyRequest(RapClient, short *, short);
int 			RAPObjectToWindowRequest(RapClient);
int 			RAPWindowToObjectRequest(RapClient);
int 			RAPLocateObjectRequest(RapClient, WidgetList widgets,
					       long numWidgets);
int 			RAPGetActionsRequest(RapClient);
int 			RAPDoActionsRequest(RapClient);
int 			RAPSelectEventRequest(RapClient, short *, short);
int 			RAPSelectRequestRequest(RapClient, short *, short);
int 			RAPCloseConnectionRequest(RapClient);

int 			HandleRapAcknowledge(IceConn);
int 			HandleRapError(IceConn);

HelloReply		*HandleRapHelloReply(IceConn);
QueryTreeReply 		*HandleRapQueryTreeReply(IceConn);
FullQueryTreeReply	*HandleRapFullQueryTreeReply(IceConn);
GetValuesReply		*HandleRapGetValuesReply(IceConn);
GetResourcesReply	*HandleRapGetResourcesReply(IceConn);
GetGCValuesReply        *HandleRapGetGCValuesReply(IceConn);
LocateObjectReply	*HandleRapLocateObjectReply(IceConn);
CreateNotif 		*HandleRapCreateNotify(IceConn);
ChangeNotif		*HandleRapChangeNotify(IceConn);
DestroyNotif		*HandleRapDestroyNotify(IceConn);
ConfigureNotif		*HandleRapConfigureNotify(IceConn);
GeometryNotif		*HandleRapGeometryNotify(IceConn);
RequestNotif 		*HandleRapRequestNotify(IceConn);
EventNotif 		*HandleRapEventNotify(IceConn);

void 			FreeHelloReply(HelloReply *);
void 			FreeQueryTreeReply(QueryTreeReply *);
void 			FreeFullQueryTreeReply(FullQueryTreeReply *);
void			FreeGetValuesReply(GetValuesReply *);
void			FreeGetResourcesReply(GetResourcesReply *);
void                    FreeGetGCValuesReply(GetGCValuesReply *);
void                    FreeLocateObjectReply(LocateObjectReply *);
void			FreeCreateNotify(CreateNotif *);
void			FreeDestroyNotify(DestroyNotif *);
void			FreeChangeNotify(ChangeNotif *);
void			FreeConfigureNotify(ConfigureNotif *);
void			FreeGeometryNotify(GeometryNotif *);
void			FreeRequestNotify(RequestNotif *);
void			FreeEventNotify(EventNotif *);

#endif /* _AGENT_H_ */

