/*
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
 */

#include <sys/poll.h>
#include <stropts.h>
#include <stdio.h>
#include <errno.h>
#ifndef ZAP
#include <stdlib.h>
#endif

#include "agent.h"

#include <X11/Xmd.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>		/* for XA_STRING */
#include <X11/Xmu/WinUtil.h>
#include <X11/Xproto.h>
#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICEconn.h>
#include <X11/ICE/ICEproto.h>
#include <X11/ICE/ICEmsg.h>

#include "RapLibint.h"
#include "Xtea.h"

Display *dpy;
int majorOpcode;
Atom protoNameAtom;
Window agentWindow;
struct pollfd	fds[256];
static RapClientTable knownClients = NULL;


/**********************************************************************/

/*
 * Stole this stuff from Xtea.c so that we wouldn't have
 * to drag in libXtea.a and libRap.a
 */

void
XteaSetupAgent(Display     *theDisplay,
	       Window      agentWindow,
	       char        *networkIds)
{
    XTextProperty       textProp;
 
    textProp.value = (unsigned char *)networkIds;
    textProp.encoding = XA_STRING;
    textProp.format = 8;
    textProp.nitems = strlen(networkIds);
    XSetTextProperty(theDisplay,
                     agentWindow,
                     &textProp,
                     XInternAtom(theDisplay,ICE_NETWORK_IDS,False));
 
}

void
XteaInitiateConnection(Display     *theDisplay,
		       Window      clientWindow,
		       Window      agentWindow,
		       Atom        protoAtom)
{
    XClientMessageEvent cme;
 
    cme.window = clientWindow;
    cme.type = ClientMessage;
    cme.message_type = XInternAtom(theDisplay, EXTAGENT_NAME, False);
    cme.format = EXTAGENT_FORMAT;
    cme.data.l[0] = protoAtom;
    cme.data.l[1] = XInternAtom(theDisplay, ICE_NETWORK_IDS, False);
    cme.data.l[2] = agentWindow;
    cme.data.l[3] = 0L;                         /* not used */
    cme.data.l[4] = 0L;                         /* not used */
    XSendEvent(theDisplay,clientWindow,True,0,(XEvent *)&cme);
 
}

/**********************************************************************/

/*
 * Maintain a list of RapClient structures.
 */

Boolean 
AddClient(RapClient client)
{
    int fd;

    if (!client)
	return 0;

    /* put it in the table */
    client->next = knownClients;
    knownClients = client;

    /* put it in the fdset too */
    fd = IceConnectionNumber(client->iceConn);
    fds[fd].fd = fd;
    fds[fd].events = POLLIN;
} 

RapClient
FindClientByFd(short fd)
{
    RapClient current = knownClients;

    while (current) {
	if (IceConnectionNumber(current->iceConn) == fd) {
	    break;
	}
	current = current->next;
    }

    return current;

}

RapClient
FindClientByWindow(Window w)
{
    RapClient current = knownClients;

    while (current) {
	if (current->window = w) {
	    break;
	}
	current = current->next;
    }

    return current;

}

void
RemoveClient(short fd)
{
    RapClient current = knownClients;
    RapClient prev = NULL;

    while (current) {
	if (IceConnectionNumber(current->iceConn) == fd) {
	    if (prev) 
		prev->next = current->next;
	    else 
	        knownClients = current->next;

	    free(current);
	    break;
	}
	prev = current;
	current = current->next;
    }
    return;

}

/**************************************************************************/

/*
 * Maintain a list of Listeners.
 */

typedef struct _ListenerTableEntry {
    IceListenObj		listener;
    struct _ListenerTableEntry	*next;
} ListenerTableEntry, *ListenerTable, *RapListener;

static ListenerTable knownListeners = NULL;

Boolean 
AddListener(RapListener listener)
{
    ListenerTableEntry *newEntry,*current,*prev;

    if (!listener)
	return 0;

    /* Install it in the table. */
    listener->next = knownListeners;
    knownListeners = listener;
}



RapListener
FindListener(short fd)
{
    ListenerTableEntry *current = knownListeners;

    while (current) {
	if (IceGetListenConnectionNumber(current->listener) == fd) {
	    break;
	}
	current = current->next;
    }

    return current;

}

void
RemoveListener(short fd)
{
    ListenerTableEntry *current = knownListeners;
    ListenerTableEntry *prev = NULL;

    while (current) {
	if (IceGetListenConnectionNumber(current->listener) == fd) {
	    if (prev) 
		prev->next = current->next;
	    else 
	        knownListeners = current->next;

	    free(current);
	    break;
	}
	prev = current;
	current = current->next;
    }
    return;

}

/*************************************************************************/

void
InitiateConnection(Window clientWindow)
{
    char		**cliargv=NULL;
    int			cliargc;
    XTextProperty	nametp;
    XClientMessageEvent	cme;

    if (clientWindow == None)
	return;
    
    /* Make sure we haven't already established a 
     * connection with this client.
     */
    if (FindClientByWindow(clientWindow) != NULL)
	return;
	    
    /* Make sure this is the client's main window
     */
    if (!XGetCommand(dpy, clientWindow, &cliargv, &cliargc))
	return;
    else
	XFreeStringList(cliargv);
	    
    if(XGetWMName(dpy, clientWindow, &nametp)) {
	if(nametp.value) {
	    printf("agent: Contacting client %lx %s\n", clientWindow, nametp.value);
	    XFree(nametp.value);
	    XteaInitiateConnection(dpy,
				   clientWindow,
				   agentWindow,
				   protoNameAtom);
	    XFlush(dpy);
	}
    }

} /* InitiateConnection */

void
ContactExistingClients()
{
    int                 screen,child,numchildren;
    Window              dummy, *children = NULL, client;
 
    for (screen = 0; screen < ScreenCount(dpy); screen++) {
        if (XQueryTree(dpy,
                       RootWindow(dpy,screen),
                       &dummy, &dummy,
                       &children, (unsigned int *)&numchildren)) {
            for (child = 0; child < numchildren; child++) {
                client = XmuClientWindow(dpy, children[child]);
                InitiateConnection(client);
            }
            XFree(children);
        }
    }
 
    return;
 
} /* ContactExistingClients */


Status
protoSetupProc(IceConn		iceConn,
	       int		majorVersion,
	       int		minorVersion,
	       char		*vendorStr,
	       char 		*releaseStr,
	       IcePointer	*clientDataRet,
	       char		**failureReasonRet)
{
    printf("agent: PROTOCOL SETUP PROC\n");
    printf("       Version: %1d.%1d\n",majorVersion,minorVersion);
    printf("       Vendor:  %s\n",vendorStr);
    printf("       Release: %s\n",releaseStr);

    return 1; /* 0 == failure */
    
}

void protoActivateProc(IceConn		iceConn,
		       IcePointer	clientData)
{

    printf("agent: PROTOCOL ACTIVATE PROC\n");

    /* send a rap hello request message */
   IceSimpleMessage(iceConn,
                     majorOpcode,
                     RapHelloRequest);
 
    IceFlush(iceConn);

}

void ioErrorProc(IceConn iceConn)
{
    int fd = IceConnectionNumber(iceConn);
    
    printf("agent: ioErrorProc\n");
    RemoveClient(IceConnectionNumber(iceConn));

    fds[fd].fd = -1;
    fds[fd].events = 0;
}

void ioErrorHandler(IceConn iceConn)
{
    printf("agent: ioErrorHandler\n");
}


/************************************************************************/
/*									*/
/* authorizeProc							*/
/*									*/
/*	Disables authentication.					*/
/*									*/
/************************************************************************/
Bool authorizeProc(char *hostName)
{
    printf("agent: authorizeProc %s\n",hostName);
    return True;

}

void
Dispatch(short fd)
{
    RapListener l;
    RapClient rapClient;
    IceListenObj listener;
    Status status;

    /* If it's a listener descriptor, we have to build a connection on it. */
    if ((l = FindListener(fd)) != NULL) {
	IceConn newIceConn;
	RapClient newClient;
	IceAcceptStatus statusRet;
	
	listener = l->listener;
	printf("connection request on addr '%s'\n",
	       IceGetListenConnectionString(listener));
	newIceConn = IceAcceptConnection(listener, &statusRet);

	if (statusRet != IceAcceptSuccess) {
	    fprintf(stderr, "Could not accept ice connection.\n");
	    return;
	}

	newClient = (RapClient) malloc(sizeof(RapClientStruct));
	newClient->majorOpcode = majorOpcode;
	newClient->iceConn = newIceConn;
	newClient->connectStatus = statusRet;
	newClient->fd = IceConnectionNumber(newIceConn);

	AddClient(newClient);

	printf("ACCEPTED connection to client on fd %d\n",
	       IceConnectionNumber(newIceConn));
	return;
    }

    if ((rapClient = FindClientByFd(fd)) != NULL) {

	/*
	 * We've got the client associated with this descriptor.  It may be
	 * pending a connection, or it may have already accepted a connection.
	 */
	status = IceProcessMessages(rapClient->iceConn, NULL, NULL);
	if (status != IceProcessMessagesSuccess) {
	    switch (status) {
	    case IceProcessMessagesIOError:
		fprintf(stderr, "IO Error on connection.\n");
		break;
	    case IceProcessMessagesConnectionClosed:
		fprintf(stderr, "Connection closed.\n");
		break;
	    default:
		fprintf(stderr, "Unknown status\n");
		break;
	    }
	}

	/* update the connection status if we've been pending. */
	if (rapClient->connectStatus == IceConnectPending)
	    rapClient->connectStatus = IceConnectionStatus(rapClient->iceConn);

    } else {
	fprintf(stderr, "I don't know what this descriptor is; why am I here?\n");
    }
}

void
processProtoMsgProc(IceConn iceConn, IcePointer clientData, int opcode,
		    unsigned long length, Bool swap)
{
    RapMessageHeader *header = NULL;
    
    switch(opcode)
	{
	    /* The RapHello message is the client's response to our 
	     * RapAcknowledge message.  Once we have this, save the
	     * client in the knownClients list.  This allows us to
	     * make sure we don't keep trying to initiate a connection
	     * with a client should someone do something like restart
	     * the window manager.
	 */
	case RapHelloReply:
	    (void) HandleRapHelloReply(iceConn);
	    break;
	
	case RapQueryTreeReply:
	    FreeQueryTreeReply(HandleRapQueryTreeReply(iceConn));
	    break;

	case RapFullQueryTreeReply:
	    FreeFullQueryTreeReply(HandleRapFullQueryTreeReply(iceConn));
	    break;

	case RapGetResourcesReply:
	    FreeGetResourcesReply(HandleRapGetResourcesReply(iceConn));
	    break;
	
	case RapGetGCValuesReply:
	    FreeGetGCValuesReply(HandleRapGetGCValuesReply(iceConn));
	    break;

	case RapGetValuesReply:
	    FreeGetValuesReply(HandleRapGetValuesReply(iceConn));
	    break;

	case RapCreateNotify:
	    FreeCreateNotify(HandleRapCreateNotify(iceConn));
	    break;

	case RapDestroyNotify:
	    FreeDestroyNotify(HandleRapDestroyNotify(iceConn));
	    break;

	case RapChangeNotify:
	    FreeChangeNotify(HandleRapChangeNotify(iceConn));
	    break;

	case RapConfigureNotify:
	    FreeConfigureNotify(HandleRapConfigureNotify(iceConn));
	    break;

	case RapGeometryNotify:
	    (void) HandleRapGeometryNotify(iceConn);
	    break;

	case RapRequestNotify:
	    (void) HandleRapRequestNotify(iceConn);
	    break;

	case RapEventNotify:
	{
          printf("got a rap event notify\n");
	    (void) HandleRapEventNotify(iceConn);
	    break;
 	}	
	case RapAcknowledge:
        {
          printf("got a rapack message from client \n");
          (void) HandleRapAcknowledge(iceConn);
                  break;
        }

        case RapError:
        {
          (void) HandleRapError(iceConn);
                break;
        }
 


	default:
	    printf("Yikes!  Unkown opcode: %d\n",opcode);
	    break;
    } 

} /* processProtoMsgProc */

void
PingReplyProc(IceConn conn, IcePointer data)
{
    fprintf(stderr, "RECEIVED A PING.\n");
}

void
DoCommand()
{
    char buf[256];
    char command[512];
    RapClient current = knownClients;
    
    fgets(buf, 256, stdin);
    sscanf(buf, "%s", &command);

    if (strcmp(command, "querytree") == 0) {
	while (current) {
	    RAPQueryTreeRequest(current);
	    current = current->next;
	}
    } else if (strcmp(command, "fullquerytree") == 0) {
        current = knownClients;
	while (current) {
		RAPFullQueryTreeRequest(current);
		current = current->next;
	}
    } else if (strcmp(command, "cec") == 0) {
	ContactExistingClients();
    } else if (strcmp(command, "ping") == 0) {
	IcePing(current->iceConn, PingReplyProc, NULL);
    } else if (strcmp(command, "hello") == 0) {
	current = knownClients;
	while (current) {
	    RAPHelloRequest(current);
	    current = current->next;
	}
        } else if (strcmp(command, "notify") == 0) {
        short notifications[5];
        notifications[0] = (short) RapCreateHook;
        notifications[1] = (short) RapDestroyHook;
        notifications[2] = (short) RapChangeHook;
        notifications[3] = (short) RapConfigureHook;
        notifications[4] = (short) RapGeometryHook;
	current = knownClients; 
        while (current) {
            RAPAddNotifyRequest(current, notifications, 5);
            current = current->next;
        }
 
    } else if (strcmp(command, "events") == 0) {
        short events[5];
        events[0] = (short) KeyPress;
        events[1] = (short) KeyRelease;
        events[2] = (short) ButtonPress;
        events[3] = (short) ButtonRelease;
        events[4] = (short) MapNotify;
	current = knownClients; 
        while (current) {
            RAPSelectEventRequest(current, events, 5);
            current = current->next;
        }
 
        } else if (strcmp(command, "requests") == 0) {
        short requests[5];
        requests[0] = (short) X_GetWindowAttributes;
        requests[1] = (short) X_ChangeWindowAttributes;
        requests[2] = (short) X_GetProperty;
        requests[3] = (short) X_ConfigureWindow;
        requests[4] = (short) X_InternAtom;
	current = knownClients; 
        while (current) {
            RAPSelectRequestRequest(current, requests, 5);
            current = current->next;
        }
 
    } else if (strcmp(command, "resources") == 0) {
	Widget widget;
	if (sscanf(buf, "%*s %lx", &widget) != 1) {
	    printf("expected: resources <widget>\n");
	    return;
	} else {
	    current = knownClients;	
	    while (current) {
		RAPGetResourcesRequest(current, &widget, 1);
		current = current->next;
	    }
	}
    } else if (strcmp(command, "values") == 0) {
	Widget widget;
	const char *resource = (char *) malloc(256);
	if (sscanf(buf, "%*s %lx %s", &widget, resource) != 2) {
	    printf("expected: values <widget> <resource>\n");
	    free(resource);
	    return;
	} else {
	    printf("looking for resource '%s' on widget %lx\n", resource, widget);
	    current = knownClients;	
	    while (current) {
		printf("sending...\n");
		RAPGetValuesRequest(current, widget, &resource, 1);
		current = current->next;
	    }
	} free(resource);
    } else if (strcmp(command, "gcvalues") == 0) {
         GC gc;

         if (sscanf(buf, "%*s %ld", &(gc)) != 1) {
		printf("expected: gvalues <gcid>\n");
		return;
	} else {

	    while (current) {
#ifdef ZAP
		RAPGetGCValuesRequest(current, &gc, 1);
#else
		RAPGetGCValuesRequest(current, (ulong*)&gc, 1);
#endif
		current = current->next;
	    }
	}

    } else if (strcmp(command, "help") == 0) {
	printf("Options are:  hello\t- send a hello request to all clients\n"
	       "              querytree\t- send a querytree request to all clients\n"
	       "              fullquerytree\t- send fullquerytree req to all clients\n"
	       "              cec\t- contact all existing clients on the display\n"
	       "              resources\t- send a GetResources to all clients\n"
	       "              values\t- send a GetValues to all clients\n"
	       "              gcvalues\t- send values for specified gcs\n"
               "              notify\t\t- installs all 5 notifies\n"
               "              events\t\t- installs 5 events\n"
               "              requests\t\t- installs 5 requests\n"
	       "              ping\t- send an ICE ping to all clients\n"
	       "              help\t- print this help message\n");
    } else {
	printf("Unknown command, type \'help\' for help\n");
    }

    fprintf(stdout, "AGENT> "); fflush(stdout);
}
    
main(int argc, char *argv[])
{
    IcePaVersionRec	protocolVersions[1];
    int			count;
    IceListenObj	*listenObjs;
    char		*networkIds;
    char		errorStr[256];
    register int	i;
    Status		status;
    int			xServerFd;

    IceSetIOErrorHandler(ioErrorHandler);

    protocolVersions[0].major_version = RapMajorVersion;
    protocolVersions[0].minor_version = RapMinorVersion;
    protocolVersions[0].process_msg_proc = processProtoMsgProc;

    majorOpcode = IceRegisterForProtocolReply(RapName,
					      "X Consortium",
					      "1.0",
					      1,
					      protocolVersions,
					      0,
					      NULL,
					      NULL,
					      authorizeProc,
					      protoSetupProc,
					      protoActivateProc,
					      ioErrorProc);

    if (majorOpcode == -1) {
	fprintf(stderr, "%s: IceRegisterForProtocolReply failed.\n", argv[0]);
	exit(1);
    }

    status = IceListenForConnections(&count, &listenObjs, 255, errorStr);

    if (!status) {
	fprintf(stderr, "%s: IceListenForConnections failed.\n", argv[0]);
	exit(1);
    }

    networkIds = IceComposeNetworkIdList(count, listenObjs);
    printf("network id string is %s\n", networkIds);

    if ((dpy = XOpenDisplay(NULL)) == NULL) {
	fprintf(stderr, "%s: could not open display.\n", argv[0]);
	exit(1);
    }

    /* Create a window to publish our network id list on */
    agentWindow = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy),
				      0, 0, 1, 1, 1,
				      BlackPixel(dpy, DefaultScreen(dpy)),
				      BlackPixel(dpy, DefaultScreen(dpy)));

    if (agentWindow == None) {
	fprintf(stderr, "%s: could not create window.\n", argv[0]);
	exit(1);
    }

    /* publish our property on agentWindow */
    XteaSetupAgent(dpy, agentWindow, networkIds);

    /* Get the atom that names our protocol */
    protoNameAtom = XInternAtom(dpy, "RAP", False);

    /* look for substructure notifies on the root window */
    XSelectInput(dpy, DefaultRootWindow(dpy), SubstructureNotifyMask);
    XFlush(dpy);

    /* initialize our poll fd array */
    for (i=0 ; i<256 ; i++) {
	fds[i].fd = -1;
	fds[i].events = 0;
    }

    /* listen on stdin for window IDs */
    fds[0].fd = 0;
    fds[0].events = POLLIN;

    /* listen on the X server connection for SubstructureNotifies */
    fds[ConnectionNumber(dpy)].fd = ConnectionNumber(dpy);
    fds[ConnectionNumber(dpy)].events = POLLIN;
    xServerFd = ConnectionNumber(dpy);

    /* listen on the descriptors of all our listener objects */
    for (i=0 ; i<count ; i++) {
	short fd = IceGetListenConnectionNumber(listenObjs[i]);
	RapListener rl = (RapListener) malloc(sizeof(ListenerTableEntry));

	fds[fd].fd = fd;
	fds[fd].events = POLLIN;

	IceSetHostBasedAuthProc(listenObjs[i], authorizeProc);

	rl->listener = listenObjs[i];
	AddListener(rl);
    }

    fprintf(stdout, "AGENT> "); fflush(stdout);
    
    /* Now start doing some work! */
    while (1) {
	int result;
	
	if ((result = poll(fds, 256, INFTIM)) < 0) {
	    if (errno != EINTR) {
		perror("poll failed");
	    }
	    /* else the syscall was interrupted by a signal */
	    continue;
	}

	for (i=0 ; i<256 ; i++) {
	    if (result == 0)
		break;

	    if (fds[i].revents == 0)
		continue;

	    result--;

	    /* If the peer has gone away, shut down the descriptor */
	    if (fds[i].revents & POLLIN) {
		int bytes;

		if (ioctl(fds[i].fd, I_NREAD, &bytes) == -1){
		    perror("ioctl NREAD");
		} else if (bytes == 0) {
		    fprintf(stderr,
			    "The dude who was connected to %d went away\n",
			    i);
		    if (i == xServerFd) {
			fprintf(stderr, "It was the server!  shutting down\n");
			exit(0);
		    } else if (i == 0) {
			fprintf(stderr, "STDIN closed, shutting down.\n");
			exit(0);
		    } else if (FindClientByFd(i) != NULL) {
			fprintf(stderr, "it was a client.\n");
			RemoveClient(i);
			fds[i].fd = -1;
			fds[i].events = 0;
		    }
		    /* for listeners, zero-length reads indicate
		     * connection setup protocol */

		}
	    }

	    switch (fds[i].revents) {
	    case POLLIN:
		if (fds[i].fd == 0) {
		    DoCommand();
		} else if (fds[i].fd == xServerFd) {
		    XEvent ev;

		    while (XCheckMaskEvent(dpy, SubstructureNotifyMask, &ev)) {
			/* an event has come in... possibly a new client? */
			if (ev.type == ReparentNotify) {
			    Window c;
			    printf("REPARENTED WINDOW = %lx\n", ev.xreparent.window);
			    printf("Looking up client window...\n");
			    c = XmuClientWindow(dpy, ev.xreparent.window);
			    printf("New client mapped, contacting.  "
				   "map window = %lx, client window = %lx.\n",
				   ev.xreparent.window, c);
			    InitiateConnection(c);
			} 
		    }
		} else {
		    /* it's an ICE client or listener descriptor */
		    Dispatch(i);
		}
		break;
	    default:
		/* for now, all we handle is reads on our descriptors */
		fprintf(stderr, "%s: unhandled event on descriptor %d\n", i);
		break;
	    }
       
	}
    }
}

