/*
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
 * Routines to generate outgoing RAP messages from the client.  All
 * replies, notifies, and error/ack messages should be wrapped by
 * functions here.
 */
/* tmp */

#define NEED_EVENTS
#define REQ_UNITS_IN_BYTES 4
/* this is 4*23 - cause there are 23 attributes send over
   as CARD32s each */
#define GCVALUES_BUF_SIZE 92
#define GC_ID_SIZE 4
#define ERR_CODE_SIZE 4


#include <stdio.h>
#include <X11/Xmd.h>
#include <X11/StringDefs.h>	/* for XtNnumShells, etc. */
#include <X11/IntrinsicP.h>	/* to get inside widget structure opaque types */
#include <X11/ShellP.h>		/* for ApplicationShellWidgetClass */
#include <X11/Xproto.h>		/* for select events and requests */

#ifdef NeedFunctionPrototypes
#undef NeedFunctionPrototypes
#endif


#include <X11/Xlibint.h>

#include <X11/ICE/ICElib.h>
#include <X11/ICE/ICEmsg.h>
#include <X11/ICE/ICEproto.h>
#include <X11/ICE/ICEconn.h>

#include "RapProto.h"
#include "RapLibint.h"
#include "Rap.h"
#include "XGCList.h"


/***
**** We have a special case for XmRXmStrings, so I #include this here
**** to make it work out.  Bad news!
***/
#include <Xm/Xm.h>

/****
**** I Hacked a version of R6 libXt to fix the realize bug.  Hence this define
****/
#define WORKING_LIBXT

#define TOOLKIT	"Xt"

#define ValidGCValuesBits (GCFunction | GCPlaneMask | GCForeground | \
                           GCBackground | GCLineWidth | GCLineStyle | \
                           GCCapStyle | GCJoinStyle | GCFillStyle | \
                           GCFillRule | GCTile | GCStipple | \
                           GCTileStipXOrigin | GCTileStipYOrigin | \
                           GCFont | GCSubwindowMode | GCGraphicsExposures | \
                           GCClipXOrigin | GCClipYOrigin | GCDashOffset | \
                           GCArcMode)


/* just a filler for now */
static CurrentSequenceNum = 0;

static void ComputeTreeSizes(Widget, CARD32 *, CARD32 *);
static void ComputeFullTreeSizes(Widget, CARD32 *, CARD32 *);
static void ComputeFullResourceSizes(Widget, CARD32 *, CARD32 *);
static void StoreTree(Widget w, char **bufPointer);
static void StoreFullResources(Widget w, char **bufPointer);

/* used (EVENTUALLY) by GetResources */
typedef struct _ResourceNames {
    char *name, *class, *type;
    CARD32 kind;
} ResourceNames;

/* used (EVENTUALLY) by GetValues */
typedef struct _ResourceValues {
    char *name;
    char *returnType;
    CARD32 size;
    char *data;
} ResourceValues;

/* used by CreateNotify */
typedef struct _ResourceEverything {
    char *name, *class, *type;
    CARD32 kind;
    char *returnType;
    CARD32 size;
    char *data;
    CARD32 dataSize;
} ResourceEverything;


/**
*** Utility functions used by other routines in this file.
**/

/*
 * Note the distinction between the native size (the actual size of the
 * representation within the widgets, and the return value, which is the
 * number of bytes it will take to write this value on the protocol stream
 * (given length fields and padding).
 *
 * Native size is simply the size field from the resource EXCEPT in the case
 * of strings, where we return strlen().
 *
 * We return the type and a pointer to the data.  Note that the return value is
 * the length of the return buffer.  nativeSize is the value which should be passed
 * as the size through the protocol.  The return buffer is a fully padded, length-
 * included representation which may be transmitted.
 *
 */
static int
ResourceValueBytesRequired(Widget w, const char *resource, CARD32 *nativeSize,
			   char **type, char **returnBuffer)
{
    XtResourceList normals, constraints;
    int numNormals, numConstraints;
    register int i;
    int bytesRequired = 0;
    char *_type = NULL;
    CARD32 _nativeSize = 0;
    int done = 0;
    char c;
    short s;
    long l;


    XtGetResourceList(XtClass(w), &normals, &numNormals);

    for (i=0 ; i<numNormals && !done ; i++) {
	if (strcmp(resource, normals[i].resource_name) == 0) {
	    _type = normals[i].resource_type;
	    _nativeSize = normals[i].resource_size;
	    done = 1;
	}
    }

    if (XtParent(w) != NULL)
	XtGetConstraintResourceList(XtClass(XtParent(w)), &constraints,
				    &numConstraints);
    else
	numConstraints = 0;

    for (i=0 ; i<numConstraints && !done ; i++) {
	if (strcmp(resource, constraints[i].resource_name) == 0) {
	    _type = constraints[i].resource_type;
	    _nativeSize = constraints[i].resource_size;
	    done = 1;
	}
    }

    if (!done) {
#ifdef RAP_DEBUG
	printf("ERROR, couldn't find resource\n");
#endif
	if (nativeSize)
	    *nativeSize = 0;
	if (type)
	    *type = NULL;
	if (returnBuffer)
	    *returnBuffer = NULL;
	
	return 0;
    } else {
	char *data;
	int freeData = 0;

	/*
	 * NOTE hack/special case for strings -- both Xt strings and
	 * Xm strings.
	 */
	if (strcmp(_type, XtRString) == 0) {
	    String p;
	    XtVaGetValues(w, resource, &p, NULL);
	    if (p) {
		_nativeSize = strlen(p);
#ifdef RAP_DEBUG
		printf("\tSTRING (%d) = '%s'\n", _nativeSize, p);
#endif
	    } else {
		_nativeSize = 0;
#ifdef RAP_DEBUG
		printf("\tSTRING (0) = NULL\n");
#endif
	    }
	    data = (char *) p;
	    /* DONT FREE THIS STRING!  It points to Xt-internal data */
	} else if (strcmp(_type, XmRXmString) == 0) {
	    XmString xmString;
	    char *p = NULL;
	    XtVaGetValues(w, resource, &xmString, NULL);
	    XmStringGetLtoR(xmString, XmSTRING_DEFAULT_CHARSET, &p);
	    if (p) {
		_nativeSize = strlen(p);
#ifdef RAP_DEBUG
		printf("\tXM STRING (%d) = '%s'\n", _nativeSize, p);
#endif
	    } else {
		_nativeSize = 0;
#ifdef RAP_DEBUG
		printf("\tXM STRING (0) = NULL\n");
#endif
	    }
	    data = (char *) p;
	} else {
	    switch (_nativeSize) {
	    case 1:
		XtVaGetValues(w, resource, &c, NULL);
#ifdef RAP_DEBUG
		printf("\tCHAR = %d\n", c);
#endif
		data = (char *) &c;
		break;
	    case 2:
		XtVaGetValues(w, resource, &s, NULL);
#ifdef RAP_DEBUG
		printf("\tSHORT = %d\n", s);
#endif
		data = (char *) &s;
		break;
	    case 4:
		XtVaGetValues(w, resource, &l, NULL);
#ifdef RAP_DEBUG
		printf("\tLONG = %ld\n", l);
#endif
		data = (char *) &l;
		break;
	    default: {
		char *buf = (char *) malloc(_nativeSize);
		memset(buf, '\0', _nativeSize);
		XtVaGetValues(w, resource, buf, NULL);
#ifdef RAP_DEBUG
		printf("\tBUF size = %ld\n", _nativeSize);
#endif
		data = buf;
		freeData = 1;
		break;
	    }
	    }
	}
	
	bytesRequired = STRING_BYTES(_nativeSize);

	if (returnBuffer) {
	    char *bufp;
	    *returnBuffer = (char *) malloc(bytesRequired);
	    memset(*returnBuffer, '\0', bytesRequired);
	    bufp = *returnBuffer;
	    STORE_BYTES(bufp, _nativeSize, data);
#ifdef RAP_DEBUG
	    printf("ResourceValueBytesRequired: alloced a chunk of %d bytes at %lx\n",
		   bytesRequired, *returnBuffer);
#endif
	}

	if (freeData)
	    free(data);
    }

    XtFree((char *) normals);
    
    if (numConstraints > 0)
	XtFree((char *) constraints);

    if (nativeSize)
	*nativeSize = _nativeSize;

    if (type)
	*type = _type;

    return bytesRequired;
}


static int
FindChildren(Widget parent, Widget **children, Boolean normal, Boolean popup)
{
    CompositeWidget cw = (CompositeWidget) parent;
    int i, numChildren = 0, current = 0;

    if (XtIsWidget(parent) && popup)
	numChildren += parent->core.num_popups;

    if (XtIsComposite(parent) && normal)
	numChildren += cw->composite.num_children;

    if (numChildren == 0) {
	*children = NULL;
	return 0;
    }

    *children = (Widget *) XtMalloc((Cardinal) sizeof(Widget) * numChildren);

    if (XtIsComposite(parent) && normal)
	for (i=0 ; i<cw->composite.num_children; i++, current++)
	    (*children)[current] = cw->composite.children[i];

    if (XtIsWidget(parent) && popup)
	for (i=0 ; i<parent->core.num_popups ; i++, current++)
	    (*children)[current] = parent->core.popup_list[i];

    return numChildren;
}

/* This is a trick/kludge.  To make shared libraries happier (linking
 * against Xmu but not linking against Xt, and apparently even work
 * as we desire on SVR4, we need to avoid an explicit data reference
 * to applicationShellWidgetClass.  XtIsTopLevelShell is known
 * (implementation dependent assumption!) to use a bit flag.  So we
 * go that far.  Then, we test whether it is an applicationShellWidget
 * class by looking for an explicit class name.  Seems pretty safe.
 */
static Bool
IsApplicationShell(w)
    Widget w;
{
    register WidgetClass c;
 
    if (!XtIsTopLevelShell(w))
        return False;
    for (c = XtClass(w); c; c = c->core_class.superclass) {
        if (!strcmp(c->core_class.class_name, "ApplicationShell"))
            return True;
    }
    return False;
}

static void
ComputeTreeSizes(Widget w, CARD32 *numBytes, CARD32 *numWidgets)
{
    Widget *children;
    int numChildren;
    register int i;
    char *class;

    (*numWidgets)++;

    if (IsApplicationShell(w))
	class = ((ApplicationShellWidget) w)->application.class;
    else
	class = XtClass(w)->core_class.class_name;

    (*numBytes) += sizeof(Widget);			/* our widget id */
    (*numBytes) += sizeof(Widget);			/* our parent's id */
    (*numBytes) += STRING_BYTES(strlen(XtName(w)));	/* our name */
    (*numBytes) += STRING_BYTES(strlen(class));		/* our class */
    (*numBytes) += sizeof(Window);			/* our window id */
    (*numBytes) += sizeof(CARD32);			/* manage state */
    (*numBytes) += STRING_BYTES(strlen(TOOLKIT));	/* our toolkit */
    
    numChildren = FindChildren(w, &children, TRUE, TRUE);
    for (i=0 ; i<numChildren ; i++) {
	ComputeTreeSizes(children[i], numBytes, numWidgets);
    }

    XtFree((char *) children);
}

static void
ComputeFullTreeSizes(Widget w, CARD32 *numBytes, CARD32 *numWidgets)
{
    Widget *children;
    int numChildren;
    register int i;
    char *class;
    CARD32 numResources = 0;
 
    (*numWidgets)++;
 
    if (IsApplicationShell(w))
        class = ((ApplicationShellWidget) w)->application.class;
    else
        class = XtClass(w)->core_class.class_name;
 
    (*numBytes) +=  sizeof(Widget);                      /* our widget id */
    (*numBytes) += sizeof(Widget);                      /* our parent's id */
    (*numBytes) += STRING_BYTES(strlen(XtName(w)));     /* our name */
    (*numBytes) += STRING_BYTES(strlen(class));         /* our class */
    (*numBytes) += sizeof(Window);                      /* our window id */
    (*numBytes) += sizeof(CARD32);             		/* manage state */
    (*numBytes) += STRING_BYTES(strlen(TOOLKIT));       /* our toolkit */
    ComputeFullResourceSizes(w, numBytes, (CARD32 *) &(numResources));	
    numChildren = FindChildren(w, &children, TRUE, TRUE);
    for (i=0 ; i<numChildren ; i++) {
        ComputeFullTreeSizes(children[i], numBytes, numWidgets);
    }
 
    XtFree((char *) children);
}
 




static void
StoreTree(Widget w, char **bufPointer)
{
    Widget *children;
    int numChildren;
    register int i;
    char *class;
    Window window;
    CARD32 parent;
    CARD32 managed;

    if (IsApplicationShell(w))
	class = ((ApplicationShellWidget) w)->application.class;
    else
	class = XtClass(w)->core_class.class_name;

    if (XtIsWidget(w)) {
	if (XtIsRealized(w))
	    window = XtWindow(w);
	else
	    window = RapUnrealized;
    } else {
	window = RapGadget;
    }

#ifdef RAP_DEBUG
    printf("StoreTree: storing data for widget %x, bufpointer is %lx, "
	   "name is %s, class is %s, window = %lx, parent = %lx\n",
	   w, *bufPointer, XtName(w), class, window, XtParent(w));
#endif

    parent = (CARD32) XtParent(w);
    managed = (CARD32) XtIsManaged(w);
#ifdef RAP_DEBUG
    printf("StoreTree: widget %s (%s) manage state = %ld\n",
	   XtName(w), class, managed);
#endif
    STORE_CARD32(*bufPointer, w);
    STORE_CARD32(*bufPointer, parent);
    STORE_STRING(*bufPointer, strlen(XtName(w)), XtName(w));
    STORE_STRING(*bufPointer, strlen(class), class);
    STORE_CARD32(*bufPointer, window);
    STORE_CARD32(*bufPointer, managed);
    STORE_STRING(*bufPointer, strlen(TOOLKIT), TOOLKIT);
    
    numChildren = FindChildren(w, &children, TRUE, TRUE);
    for (i=0 ; i<numChildren ; i++) {
	StoreTree(children[i], bufPointer);
    }

    XtFree((char *) children);
}

static void
StoreFullTree(Widget w, char **bufPointer)
{
    Widget *children;
    int numChildren;
    register int i;
    char *class;
    Window window;
    CARD32 parent;
    CARD32 managed;

    if (IsApplicationShell(w))
        class = ((ApplicationShellWidget) w)->application.class;
    else
        class = XtClass(w)->core_class.class_name;
 
    if (XtIsWidget(w)) {
        if (XtIsRealized(w))
            window = XtWindow(w);
        else
            window = RapUnrealized;
    } else {
        window = RapGadget;
    }
 
#ifdef RAP_DEBUG
    printf("----------\nStoreTree: bufpointer is %lx\n"
	   "\twidget = %lx\n"
	   "\tparent = %lx\n"
	   "\tname = %s\n"
	   "\tclassName = %s\n"
	   "\twindow = %lx\n"
	   "\tmanaged = %lx\n"
	   "\ttoolkit = %s\n",
	   *bufPointer, w, XtParent(w), XtName(w), class, window,
	   XtIsManaged(w), TOOLKIT);
#endif
 
    parent = (CARD32) XtParent(w);
    managed = (CARD32) XtIsManaged(w);

    STORE_CARD32(*bufPointer, w);
    STORE_CARD32(*bufPointer, parent);
    STORE_STRING(*bufPointer, strlen(XtName(w)), XtName(w));
    STORE_STRING(*bufPointer, strlen(class), class);
    STORE_CARD32(*bufPointer, window);
    STORE_CARD32(*bufPointer, managed);
    STORE_STRING(*bufPointer, strlen(TOOLKIT), TOOLKIT);
    StoreFullResources(w, bufPointer);	

    numChildren = FindChildren(w, &children, TRUE, TRUE);
#ifdef RAP_DEBUG
    printf("\t(widget has %d children)\n", numChildren);
#endif
    for (i=0 ; i<numChildren ; i++) {
        StoreFullTree(children[i], bufPointer);
    }
 
    XtFree((char *) children);
}
 

static void
ComputeResourceSizes(Widget w, long *numBytes, CARD32 *numResources)
{
    register int i;
    XtResourceList normals, constraints;
    Cardinal numNormals, numConstraints;

    (*numBytes) += 4;               /* num of resources */

    /* do our resources */
    XtGetResourceList(XtClass(w), &normals, &numNormals);

    for (i=0 ; i<numNormals ; i++) {

	
	(*numBytes) += STRING_BYTES(strlen(normals[i].resource_name));
	(*numBytes) += STRING_BYTES(strlen(normals[i].resource_class));
	(*numBytes) += STRING_BYTES(strlen(normals[i].resource_type));
	(*numBytes) += 4;		/* kind boolean */

	(*numResources)++;
    }
    
    XtFree((char *) normals);

    if (XtParent(w) != NULL)
	XtGetConstraintResourceList(XtClass(XtParent(w)), &constraints,
				    &numConstraints);
    else
	numConstraints = 0;

    if (numConstraints > 0) {
	for (i=0 ; i<numConstraints ; i++) {
	    (*numBytes) += STRING_BYTES(strlen(constraints[i].resource_name));
	    (*numBytes) += STRING_BYTES(strlen(constraints[i].resource_class));
	    (*numBytes) += STRING_BYTES(strlen(constraints[i].resource_type));
	    (*numBytes) += 4;		/* kind boolean */

	    (*numResources)++;
	}
	XtFree((char *) constraints);
    }
}

static void
ComputeFullResourceSizes(Widget w, CARD32 *numBytes, CARD32 *numResources)
{
    register int i;
    XtResourceList normals, constraints;
    Cardinal numNormals, numConstraints, size;
    char *native, *name;
 
    /* do our resources */
    XtGetResourceList(XtClass(w), &normals, &numNormals);
   
    (*numBytes) += 4;               /* num of resources */
 
    for (i=0 ; i<numNormals ; i++) {
        (*numBytes) += STRING_BYTES(strlen(normals[i].resource_name));
        (*numBytes) += STRING_BYTES(strlen(normals[i].resource_class));
	/* one for native and one for return type */
        (*numBytes) += STRING_BYTES(strlen(normals[i].resource_type));
        (*numBytes) += STRING_BYTES(strlen(normals[i].resource_type));
        (*numBytes) += 4;               /* kind boolean */
 
        (*numResources)++;

	/* now size for values .... */
            /* NOTE THAT because of the special hack for strings, I'm
             * doing some weirdage for the size field.  I use STORE_STRING to
             * save the string onto the protocol stream, and it includes its own
             * size.  So I only add on a sizeof(CARD32) to hold the size member
             * if it's not a string
             */
 
        /* We may want to get all values at once, but for now... */
	native = normals[i].resource_type;
        name = normals[i].resource_name;
	size = normals[i].resource_size;
	/* figure out how many bytes to add for the representation */
	(*numBytes) += ResourceValueBytesRequired(w, normals[i].resource_name,
						 NULL, NULL, NULL);
      }
 
    XtFree((char *) normals);

    if (XtParent(w) != NULL)
        XtGetConstraintResourceList(XtClass(XtParent(w)), &constraints,
                                    &numConstraints);
    else
        numConstraints = 0;
 
    if (numConstraints > 0) {
        for (i=0 ; i<numConstraints ; i++) {
            (*numBytes) += STRING_BYTES(strlen(constraints[i].resource_name));
            (*numBytes) += STRING_BYTES(strlen(constraints[i].resource_class));
	  /* one for native and one for return type */
            (*numBytes) += STRING_BYTES(strlen(constraints[i].resource_type));
            (*numBytes) += STRING_BYTES(strlen(constraints[i].resource_type));
            (*numBytes) += 4;           /* kind boolean */
 
            (*numResources)++;

	    native = constraints[i].resource_type;
	    name = constraints[i].resource_name;
	    size = constraints[i].resource_size;
	    (*numBytes) += ResourceValueBytesRequired(w, constraints[i].resource_name,
						      NULL, NULL, NULL);
        }

        XtFree((char *) constraints);
    }
}
 


static void
StoreResources(Widget w, char **bufPointer)
{
    register int i;
    XtResourceList normals, constraints;
    Cardinal numNormals, numConstraints, numResources;

    /* do our resources */
    XtGetResourceList(XtClass(w), &normals, &numNormals);

    if (XtParent(w) != NULL)
	XtGetConstraintResourceList(XtClass(XtParent(w)), &constraints,
				    &numConstraints);
    else
	numConstraints = 0;

    numResources = numNormals + numConstraints;

    /* start by storing the num resources */
    STORE_CARD32(*bufPointer, numResources);
#ifdef RAP_DEBUG
    printf("Scratch: NUM RESOURCES = %ld\n", numResources);
#endif
    
    for (i=0 ; i<numNormals ; i++) {
	STORE_STRING(*bufPointer, strlen(normals[i].resource_name),
		     normals[i].resource_name);
	STORE_STRING(*bufPointer, strlen(normals[i].resource_class),
		     normals[i].resource_class);
	STORE_STRING(*bufPointer, strlen(normals[i].resource_type),
		     normals[i].resource_type);
	STORE_CARD32(*bufPointer, RapNormalResource);
	
#ifdef RAP_DEBUG
	printf("\t%d scratch: NAME = %s, CLASS = %s, TYPE = %s, KIND = %d\n",
	       i, normals[i].resource_name, normals[i].resource_class,
	       normals[i].resource_type, RapNormalResource);
#endif
    }
    
    XtFree((char *) normals);


    if (numConstraints > 0) {
	for (i=0 ; i<numConstraints ; i++) {
	    STORE_STRING(*bufPointer, strlen(constraints[i].resource_name),
			 constraints[i].resource_name);
	    STORE_STRING(*bufPointer, strlen(constraints[i].resource_class),
			 constraints[i].resource_class);
	    STORE_STRING(*bufPointer, strlen(constraints[i].resource_type),
			 constraints[i].resource_type);
	    STORE_CARD32(*bufPointer, RapConstraintResource);
	    
#ifdef RAP_DEBUG
	    printf("\t%d scratch: NAME = %s, CLASS = %s, TYPE = %s, KIND = %d\n",
		   i, constraints[i].resource_name,
		   constraints[i].resource_class,
		   constraints[i].resource_type, RapConstraintResource);

#endif
	}
	XtFree((char *) constraints);
    }
}

static void
StoreFullResources(Widget w, char **bufPointer)
{
    register int i;
    XtResourceList normals, constraints;
    Cardinal numNormals, numConstraints, numResources;
    char *native, *name;
    Cardinal  size;	
    long bufSize;
    char *valueBuffer;
 
    /* do our resources */
    XtGetResourceList(XtClass(w), &normals, &numNormals);

    if (XtParent(w) != NULL)
        XtGetConstraintResourceList(XtClass(XtParent(w)), &constraints,
                                    &numConstraints);
     else
        numConstraints = 0;

    numResources = numNormals + numConstraints;

    /* start by storing the num resources */
    STORE_CARD32(*bufPointer, numResources);
#ifdef RAP_DEBUG
    printf("Scratch: NUM RESOURCES = %ld\n", numResources);
#endif

    for (i=0 ; i<numNormals ; i++) {
        STORE_STRING(*bufPointer, strlen(normals[i].resource_name),
                     normals[i].resource_name);
        STORE_STRING(*bufPointer, strlen(normals[i].resource_class),
                     normals[i].resource_class);
	/* store resource type twice, once for native and once for 
	   return type */
        STORE_STRING(*bufPointer, strlen(normals[i].resource_type),
                     normals[i].resource_type);
        STORE_STRING(*bufPointer, strlen(normals[i].resource_type),
                     normals[i].resource_type);
        /* How to store bools? */
        STORE_CARD32(*bufPointer, RapNormalResource);
	
 	native = normals[i].resource_type;	
 	size = normals[i].resource_size;
	name = normals[i].resource_name;
	bufSize = ResourceValueBytesRequired(w, name, NULL, NULL, &valueBuffer);
	memcpy(*bufPointer, valueBuffer, bufSize);
	free(valueBuffer);
	*bufPointer += bufSize;
    }		  /* end of for loop */ 

    XtFree((char *) normals);
  
    if (XtParent(w) != NULL)
        XtGetConstraintResourceList(XtClass(XtParent(w)), &constraints,
                                    &numConstraints);
    else
        numConstraints = 0;
 
    if (numConstraints > 0) {
        for (i=0 ; i<numConstraints ; i++) {
            STORE_STRING(*bufPointer, strlen(constraints[i].resource_name),
                         constraints[i].resource_name);
            STORE_STRING(*bufPointer, strlen(constraints[i].resource_class),
                         constraints[i].resource_class);
	    /* store type twice, once for native and once for return type */
            STORE_STRING(*bufPointer, strlen(constraints[i].resource_type),
                         constraints[i].resource_type);
            STORE_STRING(*bufPointer, strlen(constraints[i].resource_type),
                         constraints[i].resource_type);
            /* How to store bools */
            STORE_CARD32(*bufPointer, RapConstraintResource);
	
	    native = constraints[i].resource_type;
	    size = constraints[i].resource_size;
	    name = constraints[i].resource_name;
	    bufSize = ResourceValueBytesRequired(w, name, NULL, NULL, &valueBuffer);
	    memcpy(*bufPointer, valueBuffer, bufSize);
	    free(valueBuffer);
	    *bufPointer += bufSize;
	}
        XtFree((char *) constraints);
    }
}

static void
StoreGCValues(GC gc, XGCValues gcv, char **bufPointer)
{

   long dashes = 0L ;
	dashes = (long) gcv.dashes;


   /* store the fields, one at a time */
    STORE_CARD32(*bufPointer, gcv.function);
    STORE_CARD32(*bufPointer, gcv.plane_mask);
    STORE_CARD32(*bufPointer, gcv.foreground);
    STORE_CARD32(*bufPointer, gcv.background);
    STORE_CARD32(*bufPointer, gcv.line_width);
    STORE_CARD32(*bufPointer, gcv.line_style);
    STORE_CARD32(*bufPointer, gcv.cap_style);
    STORE_CARD32(*bufPointer, gcv.join_style);
    STORE_CARD32(*bufPointer, gcv.fill_style);
    STORE_CARD32(*bufPointer, gcv.fill_rule);
    STORE_CARD32(*bufPointer, gcv.arc_mode);
    STORE_CARD32(*bufPointer, gcv.tile);
    STORE_CARD32(*bufPointer, gcv.stipple);
    STORE_CARD32(*bufPointer, gcv.ts_x_origin);
    STORE_CARD32(*bufPointer, gcv.ts_y_origin);
    STORE_CARD32(*bufPointer, gcv.font);
    STORE_CARD32(*bufPointer, gcv.subwindow_mode);
    STORE_CARD32(*bufPointer, gcv.graphics_exposures);
    STORE_CARD32(*bufPointer, gcv.clip_x_origin);
    STORE_CARD32(*bufPointer, gcv.clip_y_origin);
    STORE_CARD32(*bufPointer, gcv.clip_mask);
    STORE_CARD32(*bufPointer, gcv.dash_offset);
    STORE_CARD32(*bufPointer, dashes);

}

static void
ComputeGCValuesSize(GC gc, long *numBytes)
{

    (*numBytes) += 4;  /* gcv.function */
    (*numBytes) += 4;  /* gcv.plane_mask */
    (*numBytes) += 4;  /* gcv.foreground */
    (*numBytes) += 4;  /* gcv.background */
    (*numBytes) += 4;  /* gcv.line_width */
    (*numBytes) += 4;  /* gcv.line_style */
    (*numBytes) += 4;  /* gcv.cap_style */
    (*numBytes) += 4;  /* gcv.join_style */
    (*numBytes) += 4;  /* gcv.fill_style */
    (*numBytes) += 4;  /* gcv.fill_rule */
    (*numBytes) += 4;  /* gcv.arc_mode */
    (*numBytes) += 4;  /* gcv.tile */
    (*numBytes) += 4;  /* gcv.stipple */
    (*numBytes) += 4;  /* gcv.ts_x_origin */
    (*numBytes) += 4;  /* gcv.ts_y_origin */
    (*numBytes) += 4;  /* gcv.font */
    (*numBytes) += 4;  /* gcv.subwindow_mode */
    (*numBytes) += 4;  /* gcv.graphics_exposures */
    (*numBytes) += 4;  /* gcv.clip_x_origin */
    (*numBytes) += 4;  /* gcv.clip_y_origin */
    (*numBytes) += 4;  /* gcv.clip_mask */
    (*numBytes) += 4;  /* gcv.dash_offset */
    (*numBytes) += 4;  /*  gcv.dashes */

}
 


/***
*** Open issue:
*** I can't figure out how to get the address type of the actual representation.
*** If I could figure out whether to treat it as a pointer or immediate value,
*** I could do some smarter things when transmitting.  For now I just have a
*** special hack to look for XtRstring though.
***/
static void
ComputeValueSizes(Widget w, int numResources, char **resources,
		  CARD32 *numBytes)
{
    register int i;
    XtResourceList normals, constraints;
    Cardinal numNormals, numConstraints;

    /* We'll need our resource lists to check types, etc. */
    XtGetResourceList(XtClass(w), &normals, &numNormals);
    
    if (XtParent(w) != NULL)
	XtGetConstraintResourceList(XtClass(XtParent(w)), &constraints,
				    &numConstraints);
    else
	numConstraints = 0;

    for (i=0 ; i<numResources ; i++) {
	Arg arg;
	XtPointer valueResult;
	char *name;
	char *native;
	Cardinal size;
	int found = 0;
	register int j = 0;

#ifdef RAP_DEBUG
	printf("ComputeValueSizes: looking for resource %s\n", resources[i]);
#endif

	/* See if we can find it in the normals list */
	while (!found && j < numNormals) {
	    if (strcmp(resources[i], normals[j].resource_name) == 0) {
		found = TRUE;
		name = normals[j].resource_name;
		native = normals[j].resource_type;
		size = normals[j].resource_size;
#ifdef RAP_DEBUG
		printf("ComputeValueSizes: Found resource %s as a normal "
		       "resource: type = %s, size = %d\n",
		       resources[i], native, size);
#endif
	    }
	    j++;
	}

	/* See if we can find it in the constraints list */
	j = 0;
	while (!found && j < numConstraints) {
	    if (strcmp(resources[i], constraints[j].resource_name) == 0) {
		found = TRUE;
		name = constraints[j].resource_name;
		native = constraints[j].resource_type;
		size = normals[j].resource_size;
#ifdef RAP_DEBUG
		printf("ComputeValueSizes: Found resource %s as a constraint "
		       "resource: type = %s, size = %d\n",
		       resources[i], native, size);
#endif
	    }
	    j++;
	}

	/* Add on size for name */
	*numBytes += STRING_BYTES(strlen(resources[i]));

	/* Add on size for errorcode */
	*numBytes += 4;

	if (!found) {
	    /* ERROR - this resource doesn't exist*/
#ifdef RAP_DEBUG
	    printf("Couldn't find resource %s\n", resources[i]);
#endif
	    *numBytes += STRING_BYTES(strlen("Resource doesn't exist"));
	} else {
	    /* At this point, name, native, and size contain correct values */

	    /* Compute the number of bytes to send the native type. */
	    *numBytes += STRING_BYTES(strlen(native));
	    /* note that for now we use native type as return type */
	    *numBytes += STRING_BYTES(strlen(native));
#ifdef RAP_DEBUG
	    printf("Added native size twice, numbytes now %ld\n", *numBytes);

	    printf("COMPUTE VALUE SIZES: looking for resource '%s' on widget %lx\n",
		   resources[i], w);
#endif
	    *numBytes += ResourceValueBytesRequired(w, resources[i],
						    NULL, NULL, NULL);
	}
    }

#ifdef RAP_DEBUG
    printf("After computing size, numbytes = %d\n", *numBytes);
#endif

    XtFree((char *) normals);

    if (numConstraints > 0)
	XtFree((char *) constraints);
}

static void
StoreValues(Widget w, int numResources, char **resources,
	    char **bufPointer)
{
    register int i;
    XtResourceList normals, constraints;
    Cardinal numNormals, numConstraints;

    /* We'll need our resource lists to check types, etc. */
    XtGetResourceList(XtClass(w), &normals, &numNormals);
    
    if (XtParent(w) != NULL)
	XtGetConstraintResourceList(XtClass(XtParent(w)), &constraints,
				    &numConstraints);
    else
	numConstraints = 0;

    for (i=0 ; i<numResources ; i++) {
	Arg arg;
	XtPointer valueResult;
	char *name;
	char *native;
	Cardinal size;
	register int j;
	int found = 0;

	/* See if we can find it in the normals list */

	j = 0;
	
	while (!found && j < numNormals) {
	    if (strcmp(resources[i], normals[j].resource_name) == 0) {
		found = TRUE;
		name = normals[j].resource_name;
		native = normals[j].resource_type;
		size = normals[j].resource_size;
	    }
	    j++;
	}

	/* See if we can find it in the constraints list */

	j = 0;
	
	while (!found && j < numConstraints) {
	    if (strcmp(resources[i], constraints[j].resource_name) == 0) {
		found = TRUE;
		name = constraints[j].resource_name;
		native = constraints[j].resource_type;
		size = normals[j].resource_size;
	    }
	    j++;
	}

	STORE_STRING(*bufPointer, strlen(resources[i]), resources[i]);

	if (!found) {
	    /* ERROR - this resource doesn't exist*/
	    STORE_CARD32(*bufPointer, RapNoSuchResource);
	    STORE_STRING(*bufPointer, strlen("Resource doesn't exist"),
			 "Resource doesn't exist");
	} else {
	    char *valueReturn = NULL;
	    long bufSize = 0;
	    
	    /* At this point, name, native, and size contain correct values */
	    STORE_CARD32(*bufPointer, RapNoError);

	    STORE_STRING(*bufPointer, strlen(native), native);
	    STORE_STRING(*bufPointer, strlen(native), native);
	    /* We may want to get all values at once, but for now... */

#ifdef RAP_DEBUG
	    printf("STOREVALUES: looking for resource '%s' on widget %lx\n",
		   resources[i], w);
#endif
	    bufSize = ResourceValueBytesRequired(w, resources[i], NULL,
						 NULL, &valueReturn);
	    memcpy(*bufPointer, valueReturn, bufSize);
	    free(valueReturn);
	    *bufPointer += bufSize;
	}
    }

    XtFree((char *) normals);

    if (numConstraints > 0)
	XtFree((char *) constraints);
}

/*
 * Need to store everything but the widget
 */
void
GetResourceInformation(Widget w, ResourceEverything **resources,
		       int *numResources, int *numBytes)
{
    XtResourceList normals, constraints;
    Cardinal numNormals, numConstraints;
    register int i;
    register int nextResource = 0;

    XtGetResourceList(XtClass(w), &normals, &numNormals);

    if (XtParent(w) != NULL)
	XtGetConstraintResourceList(XtClass(XtParent(w)), &constraints,
				    &numConstraints);
    else
	numConstraints = 0;

    *numResources = numNormals + numConstraints;

    (*resources) = (ResourceEverything *)
	malloc(sizeof(ResourceEverything) * *numResources);

    memset(*resources, '\0', sizeof(ResourceEverything) * *numResources);

    for (i=0 ; i<numNormals ; i++) {
	char *buf;
	long bufSize;
	CARD32 nativeSize;

	(*numBytes) += STRING_BYTES(strlen(normals[i].resource_name));
	(*resources)[nextResource].name = normals[i].resource_name;
	
	(*numBytes) += STRING_BYTES(strlen(normals[i].resource_class));
	(*resources)[nextResource].class = normals[i].resource_class;
	
	(*numBytes) += STRING_BYTES(strlen(normals[i].resource_type));
	(*resources)[nextResource].type = normals[i].resource_type;

#ifdef RAP_DEBUG
	printf("RESOURCE %d (normal): name = %s, class = %s, type = %s\n",
	       i, normals[i].resource_name, normals[i].resource_class,
	       normals[i].resource_type);
#endif
	
	(*numBytes) += 4;
	(*resources)[nextResource].kind = RapNormalResource;

	(*numBytes) += STRING_BYTES(strlen(normals[i].resource_type));
	(*resources)[nextResource].returnType = normals[i].resource_type;

	(*resources)[nextResource].dataSize = ResourceValueBytesRequired(w,
					          normals[i].resource_name,
						  &nativeSize, NULL, &buf);

	(*numBytes) += (*resources)[nextResource].dataSize;
	
	(*resources)[nextResource].size = nativeSize;
	(*resources)[nextResource].data = buf;
	
	nextResource++;
    }

    for (i=0 ; i<numConstraints ; i++) {
	char *buf;
	long bufSize = 0;
	CARD32 nativeSize = 0;

	(*numBytes) += STRING_BYTES(strlen(constraints[i].resource_name));
	(*resources)[nextResource].name = constraints[i].resource_name;
	
	(*numBytes) += STRING_BYTES(strlen(constraints[i].resource_class));
	(*resources)[nextResource].class = constraints[i].resource_class;
	
	(*numBytes) += STRING_BYTES(strlen(constraints[i].resource_type));
	(*resources)[nextResource].type = constraints[i].resource_type;
	
#ifdef RAP_DEBUG
	printf("RESOURCE %d (constraint): name = %s, class = %s, type = %s\n",
	       i, constraints[i].resource_name, constraints[i].resource_class,
	       constraints[i].resource_type);
#endif
	
	(*numBytes) += 4;
	(*resources)[nextResource].kind = RapConstraintResource;

	(*numBytes) += STRING_BYTES(strlen(constraints[i].resource_type));
	(*resources)[nextResource].returnType = constraints[i].resource_type;

	(*resources)[nextResource].dataSize = ResourceValueBytesRequired(w,
					         constraints[i].resource_name,
						 &nativeSize, NULL, &buf);
	
	(*numBytes) += (*resources)[nextResource].dataSize;
	
	(*resources)[nextResource].size = nativeSize;
	(*resources)[nextResource].data = buf;

	nextResource++;
    }
}

static void
StoreCreate(Widget w, char **bufPointer)
{
}

/**
***
**/

int
RAPHelloReply(IceConn iceConn, int majorOpcode, Widget w)
{
    RapHelloMessage *reply;
    
    IceGetHeader(iceConn,
		 majorOpcode,
		 RapHelloReply,
		 SIZEOF(RapHelloMessage),
		 RapHelloMessage,
		 reply);

#ifdef RAP_DEBUG
    printf("FLUSHING HELLO REPLY\n");
#endif
    
    if (reply)
     {
	   reply->window = (CARD32) XtWindow(w);	
#ifdef RAP_DEBUG
 	   fprintf(stderr," about to send out %ld wid\n", reply->window);	
#endif
           IceFlush(iceConn);
     }
    else {
#ifdef RAP_DEBUG
	fprintf(stderr, "Unable to do IceGetHeader\n");
#endif
	exit(1);
    }	
}

int 
RAPFullQueryTreeReply(IceConn iceConn, int majorOpcode, Widget w)
{
   RapMessageHeader *reply;
   Widget      hooksWidget = XtHooksOfDisplay(XtDisplay(w));
   Cardinal    numShells;
   WidgetList  shells, widgets;
   Arg         args[2];
   CARD32      xmitNumShells;
   CARD32      totalSizeInBytes = 0;
   CARD32      *numBytes;
   CARD32      *numWidgets;
   char        **scratch;
   int         padding;
   int 	       i;
	
   /* start by writing out the header */
   IceGetHeader(iceConn, majorOpcode, RapFullQueryTreeReply,
                 sizeof(RapMessageHeader), RapMessageHeader, reply);
  
    
   /* get some values from the hooks widget? */
    XtSetArg(args[0], XtNnumShells, &numShells);
    XtSetArg(args[1], XtNshells, &shells);
    XtGetValues(hooksWidget, args, 2);

  /* FIRST write the number of trees we're sending */
    xmitNumShells = numShells;
 
#ifdef RAP_DEBUG
    fprintf(stderr, "Got %d shells:\n", numShells); fflush(stderr);
#endif

   /* The total message len must include the number of shells */
    totalSizeInBytes += 4;

    numBytes = (CARD32 *) malloc(sizeof(CARD32) * numShells);
    numWidgets = (CARD32 *) malloc(sizeof(CARD32) * numShells);
    scratch = (char **) malloc(sizeof(char *) * numShells);

   /* for each shell */
   for (i=0 ; i<numShells ; i++) {
        char *bufPointer;
        numBytes[i] = 0;
        numWidgets[i] = 0;
#ifdef RAP_DEBUG
	printf("+++++ Shell %d = %lx\n", i, shells[i]);
#endif
	/* get size of entire tree */
        ComputeFullTreeSizes(shells[i], &(numBytes[i]),
                         &(numWidgets[i]));
 
#ifdef RAP_DEBUG 
        fprintf(stderr, "\ttotal tree size = %d widgets, %ld bytes\n",
                numWidgets[i], numBytes[i]);
#endif 

#if 0
        /* Pad along 64-bit boundaries */
        numBytes[i] += PAD64(numBytes[i]);
#endif
        /* allocate a scratch buffer for this tree */
	scratch[i] = (char *) malloc(numBytes[i]);
	memset(scratch[i], '\0', numBytes[i]);
 
        /* And write the tree into it */
        bufPointer = scratch[i];

	StoreFullTree(shells[i], &bufPointer);	

        /* Add in the size for the widget count (which we'll add later) */
        totalSizeInBytes += 4;
        /* Add in the size of the scratch buffer */
        totalSizeInBytes += numBytes[i];
    }
 
    padding = PAD64(totalSizeInBytes);
 
            /* Pad the total to 64-bit boundaries */
    totalSizeInBytes = totalSizeInBytes + padding;
 
    /* Set the size of the message */
    reply->length += WORD64COUNT(totalSizeInBytes);
#ifdef RAP_DEBUG
    printf("OUTGOING MSG = %d WORD64s, %d bytes\n",
	   reply->length, totalSizeInBytes);
#endif
    /* First write out the number of widget lists we'll be sending */
    IceWriteData32(iceConn, sizeof(CARD32), &xmitNumShells);
 
    /* Now, for each tree, send it as a list */
    for (i=0 ; i<numShells ; i++) {
        /* Write the number of elems in the list */
        IceWriteData32(iceConn, sizeof(CARD32), &(numWidgets[i]));
#ifdef RAP_DEBUG
	printf(">>>> shell %d WRITING numwidgets = %ld, %ld bytes scratch\n",
	       i, numWidgets[i], numBytes[i]);
#endif
        /* Write the scratch buffer for this tree */
        IceWriteData(iceConn, numBytes[i], scratch[i]);
	free(scratch[i]);
    }
 
    /* write out zeros to pad... is this necessary?? */
    for (i=0 ; i<padding ; i++) {
        char zero = '\0';
        IceWriteData(iceConn, 1, &zero);
    }

#ifdef RAP_DEBUG
    printf("FLUSHING FULL QUERY TREE REPLY!\n");
#endif
    IceFlush(iceConn);
}
 


int
RAPQueryTreeReply(IceConn iceConn, int majorOpcode, Widget w)
{
    RapMessageHeader *reply;
    Widget	hooksWidget = XtHooksOfDisplay(XtDisplay(w));
    Cardinal	numShells;
    register int i;
    WidgetList	shells;
    Arg 	args[2];
    CARD32	xmitNumShells;
    CARD32	totalSizeInBytes = 0;
    CARD32	*numBytes;
    CARD32	*numWidgets;
    char	**scratch;
    int 	padding;

    /* Write our header */
    IceGetHeader(iceConn, majorOpcode, RapQueryTreeReply,
		 sizeof(RapMessageHeader), RapMessageHeader, reply);
	
    XtSetArg(args[0], XtNnumShells, &numShells);
    XtSetArg(args[1], XtNshells, &shells);
    XtGetValues(hooksWidget, args, 2);

    /* FIRST write the number of trees we're sending */
    xmitNumShells = numShells;

#ifdef RAP_DEBUG
    fprintf(stderr, "Got %d shells:\n", numShells); fflush(stderr);
#endif

    /* The total message len must include the number of shells */
    totalSizeInBytes += 4;

    numBytes = (CARD32 *) malloc(sizeof(CARD32) * numShells);
    numWidgets = (CARD32 *) malloc(sizeof(CARD32) * numShells);
    scratch = (char **) malloc(sizeof(char *) * numShells);

    /* Then for each tree, send it as a list:  size, then elems */
    for (i=0 ; i<numShells ; i++) {
	char *bufPointer;
	numBytes[i] = 0;
	numWidgets[i] = 0;
#ifdef RAP_DEBUG
	fprintf(stderr, "\t%x\n", shells[i]); fflush(stderr);
#endif
		
	/* 
	 * This sucks.  We don't know how many widgets will be in this
	 * tree, so we have to recurse to count them all, and then
	 * recurse again to add them in.
	 */
	ComputeTreeSizes(shells[i], &(numBytes[i]),
			 &(numWidgets[i]));

#ifdef RAP_DEBUG
	fprintf(stderr, "\ttotal tree size = %d widgets, %ld bytes\n",
		numWidgets[i], numBytes[i]);
#endif
#if 0
	/* Pad along 64-bit boundaries */
	numBytes[i] += PAD64(numBytes[i]);
#endif	
	/* allocate a scratch buffer for this tree */
	scratch[i] = (char *) malloc(numBytes[i]);
	memset(scratch[i], '\0', numBytes[i]);

	/* And write the tree into it */
	bufPointer = scratch[i];

	StoreTree(shells[i], &bufPointer);

 


	/* Add in the size for the widget count (which we'll add later) */
	totalSizeInBytes += 4;
	/* Add in the size of the scratch buffer */
	totalSizeInBytes += numBytes[i];
    }

    padding = PAD64(totalSizeInBytes);

	    /* Pad the total to 64-bit boundaries */
    totalSizeInBytes = totalSizeInBytes + padding;

    /* Set the size of the message */
    reply->length += WORD64COUNT(totalSizeInBytes);

    /* First write out the number of widget lists we'll be sending */
    IceWriteData32(iceConn, sizeof(CARD32), &xmitNumShells);

    /* Now, for each tree, send it as a list */
    for (i=0 ; i<numShells ; i++) {
	/* Write the number of elems in the list */
	IceWriteData32(iceConn, sizeof(CARD32), &(numWidgets[i]));

	/* Write the scratch buffer for this tree */
	IceWriteData(iceConn, numBytes[i], scratch[i]);
	free(scratch[i]);
    }

    /* write out zeros to pad... is this necessary?? */
    for (i=0 ; i<padding ; i++) {
	char zero = '\0';
	IceWriteData(iceConn, 1, &zero);
    }
#ifdef RAP_DEBUG
    printf("FLUSHING QUERY TREE REPLY!\n");
#endif
    IceFlush(iceConn);
}

int
RAPAddNotifyError(IceConn iceConn, int majorOpcode, int rseq) 
{

    RapErrMessage *err_msg = NULL;
    char 	  *error_str;
    int 	   padding, msg_size = sizeof(RapErrMessage);
 
    error_str = (char *)
           malloc( strlen("Error installing hooks callbacks")+1);

#ifdef RAP_DEBUG
    printf("sening over an error\n");
#endif

    strcpy(error_str, "Error installing hooks callbacks");
    IceGetHeader(iceConn, majorOpcode, RapError,
          sizeof(RapErrMessage), RapErrMessage, err_msg);

    if (err_msg)
    {
        /* add error code and request seq num*/
         err_msg->errCode = (CARD16) RapErrorAddNotify;
 	 err_msg->reqSequenceNumber = rseq;
 
	 /* update the msg header */
	 msg_size += STRING_BYTES(strlen(error_str));
	 padding = PAD64(msg_size);
	 msg_size += padding; 
	 
	 err_msg->length += WORD64COUNT(msg_size);  	
  	 
        /* now write out the string */
        WRITE_STRING(iceConn, error_str);

 	/* and write out the padding */
	 while (padding--) {
        char zero = '\0';
        IceWriteData(iceConn, 1, &zero);
    	}
#ifdef RAP_DEBUG
	 printf("FLUSHING ADD NOTIFY ERROR\n");
#endif
        /* don't forget to flush!!  */
        IceFlush(iceConn);
    }
    else
    {
#ifdef RAP_DEBUG
        fprintf(stderr, "Unable to do IceGetHeader\n");
#endif
         exit(1);
    }

}

int
RAPSendAck(IceConn iceConn, int majorOpcode, int rseq)
{

    RapAckMessage *ack_msg = NULL;


    /* send over the  ack message,  ok */
    IceGetHeader(iceConn, majorOpcode, RapAcknowledge,
           sizeof(RapAckMessage), RapAckMessage, ack_msg);


#ifdef RAP_DEBUG
    printf("sending over an ack\n");
#endif

    if (ack_msg)
     {
        /* just a filler number for now */
        ack_msg->sequenceNumber = CurrentSequenceNum;
 
        /* and the requestors seq number */
        ack_msg->reqSequenceNumber = rseq;
#ifdef RAP_DEBUG
	printf("FLUSHING SEND ACK\n");
#endif
        /* don't forget to flush!!  */
        IceFlush(iceConn);
     }

     else
     {
#ifdef RAP_DEBUG
       fprintf(stderr, "Unable to do IceGetHeader\n");
#endif
        exit(1);
     }

}

int
RAPGetResourcesReply(IceConn iceConn, int majorOpcode, CARD32 numWidgets,
		     WidgetList widgets)
{
    RapMessageHeader *reply;
    register int i;
    int totalSizeInBytes = 0;
    long *numBytes;
    CARD32 *numResources;
    CARD32 *errorCodes;
    char ** scratch;
    int padding;
    
    IceGetHeader(iceConn, majorOpcode, RapGetResourcesReply,
		 sizeof(RapMessageHeader), RapMessageHeader, reply);

    /* increment our total size to account for the number of entries field */
    totalSizeInBytes += 4;

    numBytes = (long *) malloc(sizeof(long) * numWidgets);
    numResources = (CARD32 *) malloc(sizeof(CARD32) * numWidgets);
    errorCodes = (CARD32 *) malloc(sizeof(CARD32) * numWidgets);
    scratch = (char **) malloc(sizeof(char *) * numWidgets);

    for (i=0 ; i<numWidgets ; i++) {
	char *bufPointer;
	
	numBytes[i] = 0;
	numResources[i] = 0;

	/* Here we should do some check to see if the widget actually exists */

	errorCodes[i] = RapNoError;

	ComputeResourceSizes(widgets[i], &(numBytes[i]), &(numResources[i]));
#if 0
	numBytes[i] += PAD64(numBytes[i]);
#endif
	scratch[i] = (char *) malloc(numBytes[i]);
	memset(scratch[i], '\0', numBytes[i]);
	
 
	bufPointer = scratch[i];

 
#ifdef RAP_DEBUG
	printf("Widget %lx, num resources = %d, total size to store = %ld\n",
	       widgets[i], numResources[i], numBytes[i]);
#endif
	
	StoreResources(widgets[i], &bufPointer);


	totalSizeInBytes += 4;			/* widget id */
	totalSizeInBytes += 4;			/* error code */
	totalSizeInBytes += numBytes[i];	/* resource data */
    }

    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;

    reply->length += WORD64COUNT(totalSizeInBytes);

    /* Write out the number of entries we're returning */
    IceWriteData32(iceConn, sizeof(CARD32), &numWidgets);
#ifdef RAP_DEBUG
    printf("WROTE NUM WIDGETS = %ld\n", numWidgets);
#endif
    
    for (i=0 ; i<numWidgets ; i++) {
	IceWriteData32(iceConn, sizeof(CARD32), &(widgets[i]));
#ifdef RAP_DEBUG
	printf("\t%d WIDGET = %lx\n", i, widgets[i]);
#endif
	IceWriteData32(iceConn, sizeof(CARD32), &(errorCodes[i]));
#ifdef RAP_DEBUG
	printf("\t%d ERROR CODE = %ld\n", i, errorCodes[i]);
#endif

	IceWriteData(iceConn, numBytes[i], scratch[i]);
#ifdef RAP_DEBUG
	printf("\t%d WROTE %ld bytes scratch data\n", i, numBytes[i]);
#endif
	
	free(scratch[i]);
    }

    for (i=0 ; i<padding ; i++) {
	char zero = '\0';
	IceWriteData(iceConn, 1, &zero);
    }
#ifdef RAP_DEBUG
    printf("FLUSHING GET RESOURCES REPLY, %ld bytes, %d WORD64s.\n",
	   totalSizeInBytes, reply->length);
#endif
    IceFlush(iceConn);
}

int
RAPLocateObjectReply(IceConn iceConn, int majorOpcode, CARD32 numWidgets,
		     WidgetList widgets)
{
    RapMessageHeader *reply;
    register int i;
    int totalSizeInBytes = 0;
    long *numBytes;
    long *errorCodes;
    char **scratch;
    int padding;
#define SIZEOF_LOCATION (sizeof(Widget) + sizeof(CARD32) /* error code */ + sizeof(Window) + sizeof(CARD32) /* managed */ + sizeof(INT16) + sizeof(INT16) + sizeof(CARD16) + sizeof(CARD16) + sizeof(CARD16)) /* x, y, width, height, border */
    
    IceGetHeader(iceConn, majorOpcode, RapLocateObjectReply,
		 sizeof(RapMessageHeader), RapMessageHeader, reply);

    /* increment our total size to account for the number of entries field */
    totalSizeInBytes += 4;

    /*
     * This code currently never sends back an error condition.  Since we
     * never generate an errorString, all replies are fixed size so we can
     * statically allocate the message data.
     */

    totalSizeInBytes += (SIZEOF_LOCATION * numWidgets);
    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;

    reply->length += WORD64COUNT(totalSizeInBytes);

    IceWriteData32(iceConn, 4, &numWidgets);

    for (i=0 ; i<numWidgets ; i++) {
	Widget w = widgets[i];
	CARD32 window;
	CARD32 managed;
	CARD32 err = RapNoError;

	IceWriteData32(iceConn, 4, &w);
	IceWriteData32(iceConn, 4, &err);
	
	if (XtIsWidget(w)) {
	    if (XtIsRealized(w))
		window = XtWindow(w);
	    else
		window = RapUnrealized;
	} else {
	    window = RapGadget;
	}
	
	IceWriteData32(iceConn, 4, &window);

	managed = (CARD32) XtIsManaged(w);
	IceWriteData32(iceConn, 4, &managed);

	IceWriteData16(iceConn, 2, &w->core.x);
	IceWriteData16(iceConn, 2, &w->core.y);
	IceWriteData16(iceConn, 2, &w->core.width);
	IceWriteData16(iceConn, 2, &w->core.height);
	IceWriteData16(iceConn, 2, &w->core.border_width);
    }
    
    for (i=0 ; i<padding ; i++) {
	char zero = '\0';
	IceWriteData(iceConn, 1, &zero);
    }

    IceFlush(iceConn);
    return 1;
}

int
RAPWriteAllGCValues(IceConn iceConn,RapMessageHeader* reply, Widget w)
{
    XGCListPtr head = XGCGetListHead();
    XGCListPtr temp = head;
    int  i, padding, numGCs = 0;
    int  gcBufsize = GCVALUES_BUF_SIZE + GC_ID_SIZE + ERR_CODE_SIZE;
    CARD32 errCode = RapNoError;
    Display *dpy = XtDisplay(w);
 
     while (temp)
     {
        if (dpy == temp->dpy)
                numGCs++;
 
        temp = temp->next;
     }
 
 
    /* a gcbuf per gc plus the total number of entries, which is
       a 4 byte entry */
    gcBufsize = (gcBufsize*numGCs) + 4;
 
    padding = PAD64(gcBufsize);
    gcBufsize += padding;
 
    reply->length += WORD64COUNT(gcBufsize);
 
 
#ifdef RAP_DEBUG
    printf("WRITE: %d GCs\n", numGCs);
#endif
 
        IceWriteData32( iceConn, sizeof(CARD32),&(numGCs) );
 
 
    temp = head;
    while (temp)
    {
        XGCValues gcv;
        GContext gcId;
 
        if (dpy == temp->dpy)
        {
 
           gcId = XGContextFromGC(temp->gc);
 
           XGetGCValues(XtDisplay(w), temp->gc,
                                ValidGCValuesBits, &gcv);
 
           IceWriteData32( iceConn, sizeof(CARD32),&(gcId) );
 
           IceWriteData32( iceConn, sizeof(CARD32),&(errCode) );
 
           /* write the fields, one at a time longs, ints and even dashes,
            which is a char, are sent as card32 ... is this the right
            way to do things?  I know I can't use WRITE_CARD32 in its
            current state */
          IceWriteData32( iceConn, sizeof(CARD32),&(gcv.function) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.plane_mask) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.foreground) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.background) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.line_width) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.line_style) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.cap_style) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.join_style) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.fill_style) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.fill_rule) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.arc_mode) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.tile) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.stipple) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.ts_x_origin) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.ts_y_origin) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.font) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.subwindow_mode) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.graphics_exposures) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.clip_x_origin) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.clip_y_origin) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.clip_mask) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.dash_offset) );
          IceWriteData32( iceConn, sizeof(CARD32), &(gcv.dashes) );
 
        }
 
      temp = temp->next;
 
    }
 
        /* write out padding */
    for (i=0 ; i<padding ; i++) {
        char zero = '\0';
        IceWriteData(iceConn, 1, &zero);
    }

#ifdef RAP_DEBUG
	fprintf(stderr, "FINISHED SENDING FULL GCValues REQUEST\n");
 
        printf("about to return to flush full GCs\n");
#endif
}


int
RAPGetGCValuesReply(IceConn iceConn, int majorOpcode, CARD32 numGCs,
		     GContext* GCs, Widget w)
{
    RapMessageHeader *reply;
    register int i;
    int totalSizeInBytes = 0;
    long *numBytes;
    CARD32 *errorCodes;
    char ** scratch;
    int padding;
 
    IceGetHeader(iceConn, majorOpcode, RapGetGCValuesReply,
                 sizeof(RapMessageHeader), RapMessageHeader, reply);
 

    if (numGCs)
    {  /* you actually have some GCs to work with */
    /* increment our total size to account for the number of entries field */
     totalSizeInBytes += 4;
 
    numBytes = (long *) malloc(sizeof(long) * numGCs);
    errorCodes = (CARD32 *) malloc(sizeof(CARD32) * numGCs);
    scratch = (char **) malloc(sizeof(char *) * numGCs);
 
    for (i=0 ; i<numGCs ; i++) {
        char *bufPointer;
	Status success;
	XGCValues gcv;
	GC gcptr = (GC) NULL;
 
        numBytes[i] = 0;
 
        /* Here we should do some check to see if the GC is valid */
        /* XGCFromGContext will return null if the gcid isn't valid */
        gcptr = XGCFromGContext(XtDisplay(w), GCs[i]);
         if (gcptr)
        {
        	errorCodes[i] = RapNoError;

		XGetGCValues(XtDisplay(w), gcptr,
                                ValidGCValuesBits, &gcv);
 
		ComputeGCValuesSize(gcptr, &(numBytes[i]));
        	scratch[i] = (char *) malloc(numBytes[i]);
        	memset(scratch[i], '\0', numBytes[i]);
 
        	bufPointer = scratch[i];

#ifdef RAP_DEBUG
        printf("GC %lx, total size to store = %ld\n",
               gcptr, numBytes[i]);
#endif
 
        	StoreGCValues(gcptr, gcv, &bufPointer); 

 
        	totalSizeInBytes += 4;                  /* GC id */
        	totalSizeInBytes += 4;                  /* error code */
        	totalSizeInBytes += numBytes[i];        /* gc values */
	} else {
		/* invalid GC */
        	errorCodes[i] = RapNoSuchGraphicsContext;

		/* just put the string in the scratch area!! */
		 numBytes[i] += STRING_BYTES(strlen("GC doesn't exist"));
		 scratch[i] = (char *) malloc(numBytes[i]);
        	 bufPointer = scratch[i];
		 STORE_STRING(bufPointer, strlen("GC doesn't exist"),
                         "GC doesn't exist");
        	totalSizeInBytes += 4;                  /* GC id */
        	totalSizeInBytes += 4;                  /* error code */
        	totalSizeInBytes += numBytes[i];        
		}
		
    }
 
    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;
 
    reply->length += WORD64COUNT(totalSizeInBytes);
 
    /* Write out the number of entries we're returning */
    IceWriteData32(iceConn, sizeof(CARD32), &numGCs);
#ifdef RAP_DEBUG
    printf("WROTE NUM GCS = %ld\n", numGCs);
#endif
 
    for (i=0 ; i<numGCs ; i++) {
        IceWriteData32(iceConn, sizeof(CARD32), &(GCs[i]));
#ifdef RAP_DEBUG
        printf("\t%d GC = %lx\n", i, GCs[i]);
#endif
        IceWriteData32(iceConn, sizeof(CARD32), &(errorCodes[i]));
#ifdef RAP_DEBUG
        printf("\t%d ERROR CODE = %ld\n", i, errorCodes[i]);
#endif
 
        IceWriteData(iceConn, numBytes[i], scratch[i]);
#ifdef RAP_DEBUG
        printf("\t%d WROTE %ld bytes scratch data\n", i, numBytes[i]);
#endif
 
        free(scratch[i]);
    }
 
    for (i=0 ; i<padding ; i++) {
        char zero = '\0';
        IceWriteData(iceConn, 1, &zero);
    }
#ifdef RAP_DEBUG
    printf("FLUSHING GET GCS, %ld bytes, %d WORD64s.\n",
           totalSizeInBytes, reply->length);
#endif
    } else {
 
        /* go through and get every GC and send to it */
        /* make sure this else is in the right place ... */
        /* need to change length of msg and then send stuff out */
        RAPWriteAllGCValues(iceConn, reply, w);
 
    }
    
    IceFlush(iceConn);
#ifdef RAP_DEBUG
    fprintf(stderr,"DONE FLUSHING FULL GCs\n");
#endif
}




int
RAPGetValuesReply(IceConn iceConn, int majorOpcode, Widget widget,
		  CARD32 numResources, char **resources)
{
    RapMessageHeader *reply;
    register int i;
    int totalSizeInBytes = 0;
    CARD32 numBytes = 0;
    char *scratch;
    char *bufPointer;
    int padding;
    
    IceGetHeader(iceConn, majorOpcode, RapGetValuesReply,
		 sizeof(RapMessageHeader), RapMessageHeader, reply);

    /* increment our total size to account for the widget */
    totalSizeInBytes += 4;
    
    /* increment our total size to account for the number of entries field */
    totalSizeInBytes += 4;

	/* Here we should do some check to see if the widget actually exists */


    /* I'm doing separate compute/store, but this can probably be done
     * in one pass easily.
     */
    ComputeValueSizes(widget, numResources, resources, &numBytes);

#ifdef RAP_DEBUG
    printf("GETVALUESREPLY: Widget %lx, num resources = %d, total size to store = %ld\n",
	   widget, numResources, numBytes);
#endif
#if 0
    numBytes += PAD64(numBytes);
#endif
    scratch = (char *) malloc(numBytes);
    memset(scratch, '\0', numBytes);
    totalSizeInBytes += numBytes;	/* resource data */

    bufPointer = scratch;
	
    StoreValues(widget, numResources, resources, &bufPointer);



    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;

    reply->length += WORD64COUNT(totalSizeInBytes);

    /* Write out the widget id */
    IceWriteData32(iceConn, sizeof(CARD32), &widget);
    
    /* Write out the number of entries we're returning */
    IceWriteData32(iceConn, sizeof(CARD32), &numResources);

/*    for (i=0 ; i<numResources ; i++) { */
    IceWriteData(iceConn, numBytes, scratch);
    free(scratch);
/*    } */

    for (i=0 ; i<padding ; i++) {
	char zero = '\0';
	IceWriteData(iceConn, 1, &zero);
    }
#ifdef RAP_DEBUG
    printf("FLUSHING GET VALUES REPLY, %ld bytes, %d WORD64s.\n",
	   totalSizeInBytes, reply->length);
#endif
    IceFlush(iceConn);
}


/************************************************************************/
/*                                                                      */
/* RapCreateHookProc                                                    */
/*                                                                      */
/*      The Xt-type callback procedure that gets called by the          */
/*      Hooks object when a widget is created.  The create callback     */
/*      is called when a widget is created and just before returning    */
/*      from the create function.                                       */
/*                                                                      */
/*      The clientData parameter, iceConn, is the IceConn associated    */
/*      with connection with the agent that initiated a call to RapInit.*/
/*                                                                      */
/*      Send a msg that says a create cb was called                     */
/*                                                                      */
/************************************************************************/
void RapCreateHookProc(Widget           hookObject,
                       XtPointer        clientData,
                       XtPointer        callData)
{
    XtCreateHookData    callbackData = (XtCreateHookData) callData;
    IceConn             iceConn = (IceConn) clientData;
    RapMessageHeader    *notify = NULL;
    int			numBytes = 0;
    int			totalSizeInBytes = 0;
    CARD32		parent, window;
    char		*name, *class;
    ResourceEverything	*resources;
    int			numResources = 0;
    char		*scratch, *bufPointer;
    register int	i;
    int			padding;
    Widget		widget;
    int			rapMajorOpcode = RapGetMajorOpcode();
	
 
    /* Prepare a reply that contains the hook info.
     */

#ifdef RAP_DEBUG
    printf("IN CREATE NOTIFY HOOK\n");
#endif
    
    IceGetHeader(iceConn,
                 rapMajorOpcode,
                 RapCreateNotify,
                 sizeof(RapMessageHeader),
                 RapMessageHeader,
                 notify);

    widget = callbackData->widget;
#ifdef RAP_DEBUG
    printf("WIDGET IS %lx: %s\n", widget, XtName(widget));
#endif

    /* We'll have to store the type string */
    totalSizeInBytes += STRING_BYTES(strlen(callbackData->type));

    /* ...and the widget */
    totalSizeInBytes += 4;

    /* widget parent */
    parent = (CARD32) XtParent(widget);
    totalSizeInBytes += 4;

    /* ...widget name */
    name = XtName(widget);
    totalSizeInBytes += STRING_BYTES(strlen(name));

    /* widget class */
    if (IsApplicationShell(widget))
	class = ((ApplicationShellWidget) widget)->application.class;
    else
	class = XtClass(widget)->core_class.class_name;

    totalSizeInBytes += STRING_BYTES(strlen(class));

    /* widget window */
    if (XtIsWidget(widget)) {
	if (XtIsRealized(widget))
	    window = (CARD32) XtWindow(widget);
	else
	    window = RapUnrealized;
    } else {
	window = RapGadget;
    }
    totalSizeInBytes += 4;

    /* toolkit */
    totalSizeInBytes += STRING_BYTES(strlen(TOOLKIT));

    /* number of resources */
    totalSizeInBytes += 4;

    /* and resource junk */
    GetResourceInformation(widget, &resources, &numResources, &numBytes);
    
    numBytes += PAD64(numBytes);
    scratch = (char *) malloc(numBytes);
    memset(scratch, '\0', numBytes);
    bufPointer = scratch;

    for (i=0 ; i<numResources ; i++) {
	STORE_STRING(bufPointer, strlen(resources[i].name),
		     resources[i].name);
	STORE_STRING(bufPointer, strlen(resources[i].class),
		     resources[i].class);
	STORE_STRING(bufPointer, strlen(resources[i].type),
		     resources[i].type);
	STORE_CARD32(bufPointer, resources[i].kind);
	STORE_STRING(bufPointer, strlen(resources[i].returnType),
		     resources[i].returnType);
	memcpy(bufPointer, resources[i].data, resources[i].dataSize);
	bufPointer += resources[i].dataSize;
    }

    totalSizeInBytes += numBytes;
    
    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;

    notify->length = WORD64COUNT(totalSizeInBytes);

    WRITE_STRING(iceConn, callbackData->type);
    WRITE_CARD32(iceConn, widget);
    WRITE_CARD32(iceConn, parent);
    WRITE_STRING(iceConn, name);
    WRITE_STRING(iceConn, class);
    WRITE_CARD32(iceConn, window);	/* should probably be writeCARD32*/
    WRITE_STRING(iceConn, TOOLKIT);
    WRITE_CARD32(iceConn, numResources);	/* write card32*/
    IceWriteData(iceConn, numBytes, scratch);

    free(scratch);
#ifdef RAP_DEBUG
    printf("FLUSHING CREATE NOTIFY, %ld bytes, %d WORD64s, %ld RESOURCES\n",
	   numBytes, notify->length, numResources);
#endif
    IceFlush(iceConn);
 
} /* RapCreateHookProc */


#ifndef WORKING_LIBXT
/*
 * THIS CODE IS ONLY USED WHEN THE R6 LIBXT HOOK BUG IS UNFIXED.
 *
 * It recursively sends realize notifications for a widget and its children.
 *
 *		-keith
 */

void
DoSendRealizeWidget(IceConn iceConn, Widget w)
{
    RapMessageHeader    *hookInfo = NULL;
    int			totalSizeInBytes = 0, padding = 0, scratchSize = 0;
    register int	i;
    char		*scratch, *pbuf;
    int			rapMajorOpcode = RapGetMajorOpcode();
    
#ifdef RAP_DEBUG
    printf("DOSENDREALIZEWIDGET: %s (class %s) window = %lx (none = %lx)\n",
	   XtName(w), XtClass(w), XtWindow(w), None);
#endif
    if (XtWindow(w) == None)
	return;
    
    IceGetHeader(iceConn,
                 rapMajorOpcode,
                 RapChangeNotify,
                 sizeof(RapMessageHeader),
                 RapMessageHeader,
                 hookInfo);

    
    /* SIZE COMPUTATION */
    totalSizeInBytes += STRING_BYTES(strlen(XtHrealizeWidget));
    totalSizeInBytes += 4;	/* widget */
    totalSizeInBytes += 4;	/* window */

    totalSizeInBytes += scratchSize;
    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;

    hookInfo->length += WORD64COUNT(totalSizeInBytes);

    WRITE_STRING(iceConn, XtHrealizeWidget);
    WRITE_CARD32(iceConn, w);
    WRITE_CARD32(iceConn, XtWindow(w));
#ifdef RAP_DEBUG
    printf("REALIZING WIDGET %lx window = %lx\n", w, XtWindow(w));
#endif
    for (i=0 ; i<padding ; i++) {
	char zero = '\0';
	IceWriteData(iceConn, 1, &zero);
    }

    /* AND NOW RECURSIVELY DO THE CHILDREN */
    if (XtIsComposite(w)) {
	Cardinal i;
	CompositePart *cwp = &(((CompositeWidget) w)->composite);
	WidgetList children = cwp->children;
	for (i=cwp->num_children ; i!=0 ; --i)
	    DoSendRealizeWidget(iceConn, children[i-1]);
    }
}

#endif /* WORKING_LIBXT */


/************************************************************************/
/*                                                                      */
/* RapChangeHookProc                                                    */
/*                                                                      */
/*      The Xt-type callback procedure that gets called by the          */
/*      Hooks object when a widget is changed.  The change callback     */
/*      is called from XtSetValues, XtVaSetValues, XtManageChild,       */
/*      XtUnmanageChild, XtUnmanageChildren, XtRealizeWidget,           */
/*      XtUnrealizeWidget, XtAddCallback, XtRemoveCallback,             */
/*      XtAddCallbacks, XtRemoveCallbacks, XtRemoveAllCallbacks,        */
/*      XtAugmentTranslations, XtOverrideTranslations,                  */
/*      XtUninstallTranslations, XtSetKeyboardFocus,                    */
/*      XtSetWMColormapWindows, XtSetMappedWhenManaged, XtMapWidget,    */
/*      XtUnmapWidget, XtPopup, XtPopupSpringLoaded, and XtPopdown.     */
/*                                                                      */
/*      The clientData parameter, iceConn, is the IceConn associated    */
/*      with connection with the agent that initiated a call to RapInit.*/
/*                                                                      */
/*      [[[WDW - This is still to be designed.  For now, just send a    */
/*               message that says a hook was called.                   */
/*                                                                      */
/************************************************************************/
void RapChangeHookProc(Widget           hookObject,
                       XtPointer        clientData,
                       XtPointer        callData)
{
    XtChangeHookData    callbackData = (XtChangeHookData) callData;
    IceConn             iceConn = (IceConn) clientData;
    RapMessageHeader    *hookInfo = NULL;
    int			totalSizeInBytes = 0, padding = 0, scratchSize = 0;
    register int	i;
    char		*scratch, *pbuf;
    int			rapMajorOpcode = RapGetMajorOpcode();

    /* WORK AROUND BUG IN XT - special case for realize. */

#ifndef WORKING_LIBXT

    /* If the Xt bug isn't fixed, only write the header if we're NOT doing realize */
    if (strcmp(callbackData->type, XtHrealizeWidget) != 0) {

#endif /* WORKING_LIBXT */

    /* Prepare a reply that contains the hook info.
     */
    IceGetHeader(iceConn,
                 rapMajorOpcode,
                 RapChangeNotify,
                 sizeof(RapMessageHeader),
                 RapMessageHeader,
                 hookInfo);

#ifndef WORKING_LIBXT

    }
    
#endif /* WORKING_LIBXT */
    
    /* SIZE COMPUTATION */
    totalSizeInBytes += STRING_BYTES(strlen(callbackData->type));
    totalSizeInBytes += 4;	/* widget */

#ifdef RAP_DEBUG
    printf("CHANGE HOOK: widget = %lx, type = %s\n",
	   callbackData->widget, callbackData->type);
#endif

    if (strcmp(callbackData->type, XtHsetValues) == 0) {
	char **types;
	CARD32 *sizes;
	char **valueBuffer;
	CARD32 *valueBufferSize;
	CARD32 numArgs;
	XtChangeHookSetValuesDataRec * svd =
	    (XtChangeHookSetValuesDataRec *) callbackData->event_data;
	/*
	 * THIS IS BROKEN.  For now, we're just sending the arglist, rather
	 * than seeing which resources were actually changed.
	 */
#ifdef RAP_DEBUG
	printf("\tNum args changing: %d\n", svd->num_args);
#endif

	/*
	 * Note we start numArgs out at svd->num_args, but it may decrease if
	 * we run into errors
	 */
	numArgs = svd->num_args;

	types = (char **) malloc(sizeof(char *) * svd->num_args);
	sizes = (CARD32 *) malloc(sizeof(CARD32) * svd->num_args);
	valueBuffer = (char **) malloc(sizeof(char *) * svd->num_args);
	valueBufferSize = (CARD32 *) malloc(sizeof(CARD32) * svd->num_args);

	/* We'll need to store the number of resources changed */
	scratchSize += 4;

	/* And then for *each* widget... */
	for (i=0 ; i<svd->num_args ; i++) {

#ifdef RAP_DEBUG
	    printf("\t%d resourcename = %s\n", i, svd->args[i].name);
#endif
	    valueBufferSize[i] = ResourceValueBytesRequired(callbackData->widget,
							    svd->args[i].name,
							    &(sizes[i]),
							    &(types[i]),
							    &(valueBuffer[i]));

	    if (valueBufferSize[i]) {
#ifdef RAP_DEBUG
		printf("\tResourceValueBytesRequired returned bufsize = %ld, "
		       "native size = %ld, type = '%s', ADDR = %lx\n",
		       valueBufferSize[i], sizes[i], types[i], valueBuffer[i]);
#endif
		/* we'll need to store the name */
		scratchSize += STRING_BYTES(strlen(svd->args[i].name));
		/* the type */
		scratchSize += STRING_BYTES(strlen(types[i]));
		/* and the data */
		scratchSize += valueBufferSize[i];
	    } else {
#ifdef RAP_DEBUG
		printf("COULD NOT FIND RESOURCE: %s in widget %s\n",
		       svd->args[i].name, XtName(callbackData->widget));
#endif
		numArgs--;
	    }

	}
	
	scratchSize += PAD64(scratchSize);
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	
	/* store the number of changing resources.  NOTE that this is our
	 * running total of resources we DID NOT have an error on */
	STORE_CARD32(pbuf, numArgs);
	
	for (i=0 ; i<svd->num_args ; i++) {
	    /* Only store it if we got a value out */
	    if (valueBufferSize[i]) {
		/* store the name */
		STORE_STRING(pbuf, strlen(svd->args[i].name),
			     svd->args[i].name);
		/* the type */
		STORE_STRING(pbuf, strlen(types[i]), types[i]);
		/* and the data */
		memcpy(pbuf, valueBuffer[i], valueBufferSize[i]);
		free(valueBuffer[i]);
		pbuf += valueBufferSize[i];
	    }
	}

	free(valueBuffer);
	free(sizes);
    } else if (strcmp(callbackData->type, XtHmanageChildren) == 0) {
	WidgetList wl = (WidgetList) callbackData->event_data;
	scratchSize += 4;
	scratchSize += (4 * callbackData->num_event_data);
	scratchSize += PAD64(scratchSize);
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
#ifdef RAP_DEBUG
	printf("ALLOC'ed scratch, addr = %lx\n", scratch);
#endif
	pbuf = scratch;
	STORE_CARD32(pbuf, callbackData->num_event_data);
	for (i=0 ; i<callbackData->num_event_data ; i++)
	    STORE_CARD32(pbuf, wl[i]);
    } else if (strcmp(callbackData->type, XtHunmanageChildren) == 0) {
	WidgetList wl = (WidgetList) callbackData->event_data;
	scratchSize += 4;
	scratchSize += (4 * callbackData->num_event_data);
	scratchSize += PAD64(scratchSize);
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_CARD32(pbuf, callbackData->num_event_data);
	for (i=0 ; i<callbackData->num_event_data ; i++)
	    STORE_CARD32(pbuf, wl[i]);
    } else if (strcmp(callbackData->type, XtHunmanageSet) == 0) {
	WidgetList wl = (WidgetList) callbackData->event_data;
	scratchSize += 4;
	scratchSize += (4 * callbackData->num_event_data);
	scratchSize += PAD64(scratchSize);
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_CARD32(pbuf, callbackData->num_event_data);
	for (i=0 ; i<callbackData->num_event_data ; i++)
	    STORE_CARD32(pbuf, wl[i]);
    } else if (strcmp(callbackData->type, XtHmanageSet) == 0) {
	WidgetList wl = (WidgetList) callbackData->event_data;
	scratchSize += 4;
	scratchSize += (4 * callbackData->num_event_data);
	scratchSize += PAD64(scratchSize);
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_CARD32(pbuf, callbackData->num_event_data);
	for (i=0 ; i<callbackData->num_event_data ; i++)
	    STORE_CARD32(pbuf, wl[i]);
    } else if (strcmp(callbackData->type, XtHrealizeWidget) == 0) {
	/*
	 * NOTE - there is a {bug,feature} in the R6 libXt that causes us to
	 * have to do this... hopefully it will be fixed in a public patch.  The
	 * problem is that the callout to the ChangeHook for realization
	 * occurs in XtRealizeWidget, rather than RealizeWidget.  So in
	 * certain cases, the callback isn't invoked each time a widget is
	 * realized.
	 *
	 * FOR NOW I'm working around this by getting the set of managed
	 * children of the object, retrieving their windows, and generating
	 * a separate message for each.  Even this isn't completely correct,
	 * since the recursive call to RealizeWidget in libXt may not realize
	 * each widget (there's lots of strange conditionals and things going
	 * on).  We only send the message for children whose windows are not
	 * None.
	 *
	 * NOTE that because of the recursive descent of the widget
	 * hierarchy, we do not follow the normal control flow here.  We
	 * call out to DoSendRealizeWidget which recursively sends messages
	 * for each child.  We do not send a message at the end of this
	 * hook callback, since DoSendRealizeWidget does the transmission
	 * for us.
	 *
	 * If the bug is fixed, then we can get rid of the retrieval of children
	 * and the loop.
	 */

#ifndef WORKING_LIBXT	
	/* recursively send us and all our realized children */
	DoSendRealizeWidget(iceConn, callbackData->widget);
	IceFlush(iceConn);
#else
	CARD32 window;
	scratchSize += 4;
	scratchSize += PAD64(scratchSize);
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	window = (Window) XtWindow(callbackData->widget);
	STORE_CARD32(pbuf, window);
#ifdef RAP_DEBUG
	printf(" *** WORKING LIBXT: realized widget %s, window = %lx\n",
	       XtName(callbackData->widget), window);
#endif
#endif /* WORKING_LIBXT */
    } else if (strcmp(callbackData->type, XtHunrealizeWidget) == 0) {
	/* no event data */
    } else if (strcmp(callbackData->type, XtHaddCallback) == 0) {
	String callbackName = (String) callbackData->event_data;
	scratchSize += STRING_BYTES(strlen(callbackName));
	scratchSize += PAD64(scratchSize);
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_STRING(pbuf, strlen(callbackName), callbackName);
    } else if (strcmp(callbackData->type, XtHaddCallbacks) == 0) {
	String callbackName = (String) callbackData->event_data;
	scratchSize += STRING_BYTES(strlen(callbackName));
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_STRING(pbuf, strlen(callbackName), callbackName);
    } else if (strcmp(callbackData->type, XtHremoveCallback) == 0) {
	String callbackName = (String) callbackData->event_data;
	scratchSize += STRING_BYTES(strlen(callbackName));
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_STRING(pbuf, strlen(callbackName), callbackName);
    } else if (strcmp(callbackData->type, XtHremoveCallbacks) == 0) {
	String callbackName = (String) callbackData->event_data;
	scratchSize += STRING_BYTES(strlen(callbackName));
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_STRING(pbuf, strlen(callbackName), callbackName);
    } else if (strcmp(callbackData->type, XtHremoveAllCallbacks) == 0) {
	/* no event data */
    } else if (strcmp(callbackData->type, XtHaugmentTranslations) == 0) {
	/* no event data */
    } else if (strcmp(callbackData->type, XtHoverrideTranslations) == 0) {
	/* no event data */
    } else if (strcmp(callbackData->type, XtHuninstallTranslations) == 0) {
	/* no event data */
    } else if (strcmp(callbackData->type, XtHsetKeyboardFocus) == 0) {
	Widget descendant = (Widget) callbackData->event_data;
	scratchSize += 4;
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_CARD32(pbuf, descendant);
    } else if (strcmp(callbackData->type, XtHsetWMColormapWindows) == 0) {
	WidgetList wl = (WidgetList) callbackData->event_data;
	scratchSize += 4;
	scratchSize += (4 * callbackData->num_event_data);
	scratchSize += PAD64(scratchSize);
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_CARD32(pbuf, callbackData->num_event_data);
	for (i=0 ; i<callbackData->num_event_data ; i++)
	    STORE_CARD32(pbuf, wl[i]);
    } else if (strcmp(callbackData->type, XtHsetMappedWhenManaged) == 0) {
	Boolean b = (Boolean) callbackData->event_data;
	CARD32 sendable = b;
	scratchSize += 4;
	scratchSize += PAD64(scratchSize);
	scratch = (char *) malloc(scratchSize); 
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_CARD32(pbuf, sendable);
    } else if (strcmp(callbackData->type, XtHmapWidget) == 0) {
	/* no event data */
    } else if (strcmp(callbackData->type, XtHunmapWidget) == 0) {
	/* no event data */
    } else if (strcmp(callbackData->type, XtHpopup) == 0) {
	XtGrabKind kind = (XtGrabKind) callbackData->event_data;
	CARD32 sendable = kind;
	scratchSize += PAD64(scratchSize);
	scratch = (char *) malloc(scratchSize);
	memset(scratch, '\0', scratchSize);
	pbuf = scratch;
	STORE_CARD32(pbuf, sendable);
	scratchSize += 4;
    } else if (strcmp(callbackData->type, XtHpopupSpringLoaded) == 0) {
	/* no event data */
    } else if (strcmp(callbackData->type, XtHpopdown) == 0) {
	/* no event data */
    } else {
	printf("UNKNOWN CHANGE HOOK TYPE!  %s\n", callbackData->type);
    }

#ifndef WORKING_LIBXT

    /* If libXt isn't fixed, don't send messages if we're in the realize case */
    if (strcmp(callbackData->type, XtHrealizeWidget) != 0) {

#endif /* WORKING_LIBXT */
	
    totalSizeInBytes += scratchSize;
    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;

    hookInfo->length += WORD64COUNT(totalSizeInBytes);

    WRITE_STRING(iceConn, callbackData->type);
    WRITE_CARD32(iceConn, callbackData->widget);
    if (scratchSize) {
	IceWriteData(iceConn, scratchSize, scratch);
	free(scratch);
}

    for (i=0 ; i<padding ; i++) {
	char zero = '\0';
	IceWriteData(iceConn, 1, &zero);
    }

#ifndef WORKING_LIBXT

    }

#endif /* WORKING_LIBXT */
#ifdef RAP_DEBUG
    printf("FLUSHING CHANGE NOTIFY\n");
#endif
    IceFlush(iceConn);
} /* RapChangeHookProc */

/************************************************************************/
/*                                                                      */
/* RapConfigureHookProc                                                 */
/*                                                                      */
/*      The Xt-type callback procedure that gets called any time the    */
/*      Intrinsics move, resize, or configure a widget and when         */
/*      XtResizeWindow is called.  The callback is called after the     */
/*      changes have been made to the widget.                           */
/*                                                                      */
/*      The clientData parameter, iceConn, is the IceConn associated    */
/*      with connection with the agent that initiated a call to RapInit.*/
/*                                                                      */
/*      [[[WDW - This is still to be designed.  For now, just send a    */
/*               message that says a hook was called.                   */
/*                                                                      */
/************************************************************************/
void RapConfigureHookProc(Widget                hookObject,
                          XtPointer             clientData,
                          XtPointer             callData)
{
    XtConfigureHookData callbackdata = (XtConfigureHookData) callData;
    IceConn             iceConn = (IceConn) clientData;
    RapMessageHeader    *hookInfo = NULL;
    int			totalSizeInBytes = 0, padding = 0;
    register int 	i;
    int		 	rapMajorOpcode = RapGetMajorOpcode();
 
    /* Prepare a reply that contains the hook info.
     */
#ifdef RAP_DEBUG
	printf("got a configure notify ... sending msg to agent");
#endif
    
    IceGetHeader(iceConn,
                 rapMajorOpcode,
                 RapConfigureNotify,
                 sizeof(RapMessageHeader),
                 RapMessageHeader,
                 hookInfo);
    
    totalSizeInBytes += STRING_BYTES(strlen(callbackdata->type));
    totalSizeInBytes += 4;	/* widget */
    totalSizeInBytes += 4;	/* geometry mask */
    totalSizeInBytes += 28;	/* XWindowChanges - 7 longwords */

    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;

    hookInfo->length += WORD64COUNT(totalSizeInBytes);

    WRITE_STRING(iceConn, callbackdata->type);
    WRITE_CARD32(iceConn, callbackdata->widget);
    WRITE_CARD32(iceConn, callbackdata->changeMask);

    WRITE_CARD32(iceConn, callbackdata->changes.x);
    WRITE_CARD32(iceConn, callbackdata->changes.y);
    WRITE_CARD32(iceConn, callbackdata->changes.width);
    WRITE_CARD32(iceConn, callbackdata->changes.height);
    WRITE_CARD32(iceConn, callbackdata->changes.border_width);
    WRITE_CARD32(iceConn, callbackdata->changes.sibling);
    WRITE_CARD32(iceConn, callbackdata->changes.stack_mode);

    for (i=0 ; i<padding ; i++) {
	char zero = '\0';
	IceWriteData(iceConn, 1, &zero);
    }
#ifdef RAP_DEBUG
    printf("FLUSHING CONFIG NOTIFY: widget = %lx\n", callbackdata->widget);
#endif
    IceFlush(iceConn);
} /* RapConfigureHookProc */


/************************************************************************/
/*                                                                      */
/* RapGeometryHookProc                                                  */
/*                                                                      */
/*      The Xt-type callback procedure that gets called from            */
/*      XtMakeGeometryRequest and XtMakeResizeRequest once before and   */
/*      once after geometry negotiation occurs.                         */
/*                                                                      */
/*      The clientData parameter, iceConn, is the IceConn associated    */
/*      with connection with the agent that initiated a call to RapInit.*/
/*                                                                      */
/*      [[[WDW - This is still to be designed.  For now, just send a    */
/*               message that says a hook was called.                   */
/*                                                                      */
/************************************************************************/
void RapGeometryHookProc(Widget                 hookObject,
                         XtPointer              clientData,
                         XtPointer              callData)
{
    XtGeometryHookData  callbackData = (XtGeometryHookData) callData;
    IceConn             iceConn = (IceConn) clientData;
    RapMessageHeader    *hookInfo = NULL;
    int			rapMajorOpcode = RapGetMajorOpcode();
    int			totalSizeInBytes = 0, padding = 0;
    register int 	i;
 
    totalSizeInBytes += STRING_BYTES(strlen(callbackData->type));
    totalSizeInBytes += 4;	/* widget */

    if (strcmp(callbackData->type, XtHpreGeometry) == 0) {
	totalSizeInBytes += 32;	/* First XtWidgetGeometry - 8 longwords */
    } else if (strcmp(callbackData->type, XtHpostGeometry) == 0) {
	totalSizeInBytes += 32;	/* First XtWidgetGeometry  - 8 longwords*/
	totalSizeInBytes += 32;	/* Second XtWidgetGeometry - 8 longwords*/
	totalSizeInBytes += 4;	/* XtGeometryResult - enumeration */
    } else {
#ifdef RAP_DEBUG
	fprintf(stderr, "RapGeometryHookProc: Unknown callback type: %s\n",
		callbackData->type);
#endif
	return;
    }
    
    /* Prepare a reply that contains the hook info.
     */

    IceGetHeader(iceConn,
                 rapMajorOpcode,
                 RapGeometryNotify,
                 sizeof(RapMessageHeader),
                 RapMessageHeader,
                 hookInfo);
    
    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;

    hookInfo->length += WORD64COUNT(totalSizeInBytes);

    WRITE_STRING(iceConn, callbackData->type);
    WRITE_CARD32(iceConn, callbackData->widget);

    if (strcmp(callbackData->type, XtHpreGeometry) == 0) {
	WRITE_CARD32(iceConn, callbackData->request->request_mode);
	WRITE_CARD32(iceConn, callbackData->request->x);
	WRITE_CARD32(iceConn, callbackData->request->y);
	WRITE_CARD32(iceConn, callbackData->request->width);
	WRITE_CARD32(iceConn, callbackData->request->height);
	WRITE_CARD32(iceConn, callbackData->request->border_width);
	WRITE_CARD32(iceConn, callbackData->request->sibling);
	WRITE_CARD32(iceConn, callbackData->request->stack_mode);
    } else {
#ifdef RAP_DEBUG
	printf("Result of Geometry op is %d\n", callbackData->result);
#endif
	WRITE_CARD32(iceConn, callbackData->result);
	
	WRITE_CARD32(iceConn, callbackData->request->request_mode);
	WRITE_CARD32(iceConn, callbackData->request->x);
	WRITE_CARD32(iceConn, callbackData->request->y);
	WRITE_CARD32(iceConn, callbackData->request->width);
	WRITE_CARD32(iceConn, callbackData->request->height);
	WRITE_CARD32(iceConn, callbackData->request->border_width);
	WRITE_CARD32(iceConn, callbackData->request->sibling);
	WRITE_CARD32(iceConn, callbackData->request->stack_mode);

	/*
	 * reply may be NULL (they may pass NULL in to XtMakeGeometryRequest
	 * for example, so we need to do the right thing...
	 */
	if (callbackData->reply) {
	    WRITE_CARD32(iceConn, callbackData->reply->request_mode);
	    WRITE_CARD32(iceConn, callbackData->reply->x);
	    WRITE_CARD32(iceConn, callbackData->reply->y);
	    WRITE_CARD32(iceConn, callbackData->reply->width);
	    WRITE_CARD32(iceConn, callbackData->reply->height);
	    WRITE_CARD32(iceConn, callbackData->reply->border_width);
	    WRITE_CARD32(iceConn, callbackData->reply->sibling);
	    WRITE_CARD32(iceConn, callbackData->reply->stack_mode);
	} else {
	    long empty = 0L;

	    WRITE_CARD32(iceConn, empty);
	    WRITE_CARD32(iceConn, empty);
	    WRITE_CARD32(iceConn, empty);
	    WRITE_CARD32(iceConn, empty);
	    WRITE_CARD32(iceConn, empty);
	    WRITE_CARD32(iceConn, empty);
	    WRITE_CARD32(iceConn, empty);
	    WRITE_CARD32(iceConn, empty);
	}
    }
    
    for (i=0 ; i<padding ; i++) {
	char zero = '\0';
	IceWriteData(iceConn, 1, &zero);
    }
#ifdef RAP_DEBUG
    printf("FLUSHING GEOMETRY NOTIFY: widget = %lx\n", callbackData->widget);
#endif
    IceFlush(iceConn);
} /* RapGeometryHookProc */

/************************************************************************/
/*                                                                      */
/* RapDestroyHookProc                                                   */
/*                                                                      */
/*      The Xt-type callback procedure that gets called when a          */
/*      widget is destroyed.                                            */
/*                                                                      */
/*      The clientData parameter, iceConn, is the IceConn associated    */
/*      with connection with the agent that initiated a call to RapInit.*/
/*                                                                      */
/*      [[[WDW - This is still to be designed.  For now, just send a    */
/*               message that says a hook was called.                   */
/*                                                                      */
/************************************************************************/
void RapDestroyHookProc(Widget                  hookObject,
                        XtPointer               clientData,
                        XtPointer               callData)
{
    XtDestroyHookData   callbackdata = (XtDestroyHookData) callData;
    IceConn             iceConn = (IceConn) clientData;
    RapMessageHeader    *hookInfo = NULL;
    int			totalSizeInBytes = 0, padding = 0;
    register int	i;
    int			rapMajorOpcode = RapGetMajorOpcode();

 
    /* Prepare a reply that contains the hook info.
     */


    IceGetHeader(iceConn,
                 rapMajorOpcode,
                 RapDestroyNotify,
                 sizeof(RapMessageHeader),
                 RapMessageHeader,
                 hookInfo);

    totalSizeInBytes += STRING_BYTES(strlen(callbackdata->type));
    totalSizeInBytes += 4;

    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;

    hookInfo->length += WORD64COUNT(totalSizeInBytes);

    WRITE_STRING(iceConn, callbackdata->type);
    WRITE_CARD32(iceConn, callbackdata->widget);

    for (i=0 ; i<padding ; i++) {
	char zero = '\0';
	IceWriteData(iceConn, 1, &zero);
    }
#ifdef RAP_DEBUG
    printf("FLUSHING DESTROY NOTIFY: widget = %lx\n", callbackdata->widget);
#endif
    IceFlush(iceConn);
 
} /* RapDestroyHookProc */

int
RapEventProc(Display *display, XEvent *hostEvent, xEvent *wireEvent)
{
    RapMessageHeader    *event_msg = NULL;
    int                  event_size;
    IceConn             iceConn;
    int                 rapMajorOpcode = RapGetMajorOpcode();
    Status		status;
 
    iceConn = (IceConn) RapGetIceConnection();
#ifdef RAP_DEBUG
    fprintf(stderr,"RapEventProc: just got an event of type %d\n", wireEvent->u.u.type);
#endif

    /* Convert to host format */
    status = _XWireToEvent(display, hostEvent, wireEvent);
 
    IceGetHeader(iceConn,
                 rapMajorOpcode,
                 RapEventNotify,
                 sizeof(RapMessageHeader),
                 RapMessageHeader,
                 event_msg);

   if (event_msg) {
       /* fix up length first */
       event_size = PADDED_BYTES64( sizeof(xEvent) );
       event_msg->length += WORD64COUNT(event_size);
       IceWriteData(iceConn, event_size, wireEvent);
       IceFlush(iceConn);
   } else {
#ifdef RAP_DEBUG
       fprintf(stderr, "Error getting ice header \n");
#endif
   }

    return status;
}

int
RapFlushProc(Display *display, XExtCodes *ext, char *data, long len)
{
    short	*cur_requests;
    xReq 	*request;
    IceConn 	iceConn = (IceConn) RapGetIceConnection();
    int     	rapMajorOpcode = RapGetMajorOpcode();
    int 	req_size;
    long 	usedLen = 0;
    int 	padding = 0;
    register int i;
 
    cur_requests = (short *)  RapGetRequestTypes();
 
    while (usedLen < len) {
	request = (xReq*) (data + usedLen);
	if (request->length < 1) break; /* sanity check */
 
	usedLen += request->length * REQ_UNITS_IN_BYTES;
 
	if (cur_requests[request->reqType]) {
	    RapMessageHeader *req_msg;
 
	    /* generate the message header*/
	    IceGetHeader(iceConn,
			 rapMajorOpcode,
			 RapRequestNotify,
			 sizeof(RapMessageHeader),
			 RapMessageHeader,
			 req_msg);
 
	    /* Length is specified in Word32's, so we convert to bytes */
	    req_size = request->length * REQ_UNITS_IN_BYTES;
	    /* compute the padding required */
	    padding = PAD64(req_size);
	    /* Set the outgoing message length */
	    req_msg->length += WORD64COUNT(req_size);

#ifdef RAP_DEBUG
/* debug sue: 5/24 */
        if (request->reqType==56 )
	{
		xChangeGCReq *req = (xChangeGCReq *) request;
                fprintf(stderr, "GOT CH req = %d!! gc %ld\n", request->reqType, req->gc);
	}
        if (request->reqType==55 )
	{
		xCreateGCReq *req = (xCreateGCReq *) request;
                fprintf(stderr, "GOT CR req = %d!! gc %ld\n", request->reqType, req->gc);
	}
#endif

	    /* Write the request data */
	    IceWriteData(iceConn, request->length * REQ_UNITS_IN_BYTES,
			 request);

	    /* Write any required padding */
	    for (i=0 ; i<padding ; i++) {
		char zero = '\0';
		IceWriteData(iceConn, 1, &zero);
	    }
			
#ifdef  RAP_DEBUG
	    fprintf(stderr,
		    "about to send out req %d, size = %d bytes, %d WORD64s\n",
		    request->reqType, request->length * REQ_UNITS_IN_BYTES,
		    req_msg->length);
#endif

	    IceFlush(iceConn);

	}
    }
 
}

/***
*** _XGenerateGCList is a static function in CrGC.c in the Xlib source.  Since it's
*** static I have to make a copy of it here to call.
***/
static void
_MyXGenerateGCList (dpy, gc, req)
    register Display *dpy;
    xReq *req;
    GC gc;
{
    unsigned long values[32];
    register unsigned long *value = values;
    long nvalues;
    XID *pval = (XID *) &req[1];
    register XGCValues *gv = &gc->values;
    register unsigned long dirty = gc->dirty;

#ifdef RAP_DEBUG
    /* sue:  hack, I'm really reaching with this ...	  
    dirty = dirty | GCFont;  */
#endif


    /*
     * Note: The order of these tests are critical; the order must be the
     * same as the GC mask bits in the word.
     */
    if (dirty & GCFunction)          *value++ = gv->function;
    if (dirty & GCPlaneMask)         *value++ = gv->plane_mask;
    if (dirty & GCForeground)        *value++ = gv->foreground;
    if (dirty & GCBackground)        *value++ = gv->background;
    if (dirty & GCLineWidth)         *value++ = gv->line_width;
    if (dirty & GCLineStyle)         *value++ = gv->line_style;
    if (dirty & GCCapStyle)          *value++ = gv->cap_style;
    if (dirty & GCJoinStyle)         *value++ = gv->join_style;
    if (dirty & GCFillStyle)         *value++ = gv->fill_style;
    if (dirty & GCFillRule)          *value++ = gv->fill_rule;
    if (dirty & GCTile)              *value++ = gv->tile;
    if (dirty & GCStipple)           *value++ = gv->stipple;
    if (dirty & GCTileStipXOrigin)   *value++ = gv->ts_x_origin;
    if (dirty & GCTileStipYOrigin)   *value++ = gv->ts_y_origin;
    if (dirty & GCFont)              *value++ = gv->font;
    if (dirty & GCSubwindowMode)     *value++ = gv->subwindow_mode;
    if (dirty & GCGraphicsExposures) *value++ = gv->graphics_exposures;
    if (dirty & GCClipXOrigin)       *value++ = gv->clip_x_origin;
    if (dirty & GCClipYOrigin)       *value++ = gv->clip_y_origin;
    if (dirty & GCClipMask)          *value++ = gv->clip_mask;
    if (dirty & GCDashOffset)        *value++ = gv->dash_offset;
    if (dirty & GCDashList)          *value++ = gv->dashes;
    if (dirty & GCArcMode)           *value++ = gv->arc_mode;

    req->length += (nvalues = value - values);

    /* 
     * note: Data is a macro that uses its arguments multiple
     * times, so "nvalues" is changed in a separate assignment
     * statement 
     */

    nvalues <<= 2;
#ifdef RAP_DEBUG
	/* no idea if this will help or not ..  
    Data32 (dpy, (long *) values, nvalues);  

   memcpy(pval, values, (value - values) ); */
#endif

}

int
RapGCFlushProc(Display *display, GC gc, XExtCodes codes)
{
    /*
      typedef struct {
      CARD8 reqType;
      BYTE pad;
      CARD16 length B16;
      GContext gc B32;
      Drawable drawable B32;
      CARD32 mask B32;
      } xCreateGCReq;
      */

   xCreateGCReq *gcReq;
   char *gcData;
   RapMessageHeader *req_msg;
   IceConn iceConn = (IceConn) RapGetIceConnection();
   int     rapMajorOpcode = RapGetMajorOpcode();
   int req_size;
   int padding = 0;
   register int i;

   /* the size of the req is header + values (max is 23 longwords) */
   gcReq = (xCreateGCReq*) malloc(SIZEOF(xCreateGCReq) * (23 * sizeof(CARD32)));
   
   /* Coerce the GC into an xCreateGCReq request */
   gcReq->reqType = X_CreateGC;
   gcReq->length = (SIZEOF(xCreateGCReq) >> 2);
   gcReq->gc = gc->gid;
   gcReq->drawable = DefaultRootWindow(display);	/* ok to use root? */
   gcReq->mask = gc->dirty;

#ifdef RAP_DEBUG
   /* hack to get font info  
   gcReq->mask = gc->dirty | GCFont;  */
#endif
   _MyXGenerateGCList(display, gc, (xReq *) gcReq);

   /* At this point, req looks just like  "real" X request */

#ifdef RAP_DEBUG
   fprintf(stderr,"about to send out GC FLUSH req %ld\n", gcReq->reqType);

#endif
   
   /* send out the message!! */
   IceGetHeader(iceConn,
                rapMajorOpcode,
                RapRequestNotify,
                sizeof(RapMessageHeader),
                RapMessageHeader,
                req_msg);


   /* Length is specified in Word32's, so we convert to bytes */
   req_size = gcReq->length * REQ_UNITS_IN_BYTES;
   /* compute the padding required */
   padding = PAD64(req_size);
   /* Set the outgoing message length */
   req_msg->length += WORD64COUNT(req_size);

   /* Write the request data */
   IceWriteData(iceConn, gcReq->length * REQ_UNITS_IN_BYTES, gcReq);

   /* Write any required padding */
   for (i=0 ; i<padding ; i++) {
       char zero = '\0';
       IceWriteData(iceConn, 1, &zero);
   }
   IceFlush(iceConn);

#ifdef RAP_DEBUG
   printf("Just flushed gc %ld \n", gc->gid);
   /*getchar();	*/
#endif
	
}
 
#if 0

int
RapGCFlushProc(Display *display, GC gc, XExtCodes codes)
{
        /* not sure what's up with this one ... */
   /* ok, gc request struct is as follows:

typedef struct {
    CARD8 reqType;
    BYTE pad;
    CARD16 length B16;
    GContext gc B32;
    Drawable drawable B32;
    CARD32 mask B32;
} xCreateGCReq;

  just fill in what you know -- for now ... is there a way to
  get the rest?? */
 
   xCreateGCReq gcReq;
   RapMessageHeader *req_msg;
   IceConn iceConn = (IceConn) RapGetIceConnection();
   int     rapMajorOpcode = RapGetMajorOpcode();
   int req_size;
   char *scratch;


   gcReq.reqType = X_CreateGC;
   /* is this length in bytes?  probably not ... 
      asumming 8 byte cards for now .... */
   gcReq.length = WORD64COUNT(sizeof(xCreateGCReq) );
   gcReq.gc = XGContextFromGC(gc);

   /* ok, now send over the request ... */
   
 
#ifdef RAP_DEBUG
   fprintf(stderr,"about to send out req %d\n", gcReq.reqType);
#endif

   
   /* send out the message!! */
   IceGetHeader(iceConn,
                rapMajorOpcode,
                RapRequestNotify,
                sizeof(RapMessageHeader),
                RapMessageHeader,
                req_msg);
 
   /* fix up length first */
   req_size = PADDED_BYTES64(sizeof(xCreateGCReq) );
   req_msg->length += WORD64COUNT(req_size);
 
   /* now set up scratch area   - may not really need this
     scratch area, but if you want to send over data in 8 byte
     cards ... better to use scratch area */
   scratch = (char *) malloc(req_size);
   memcpy(scratch, &gcReq, req_size);
   /* finally, write out the data  */
   IceWriteData(iceConn, req_size, scratch);
   IceFlush(iceConn);
   free(scratch);
 
}
 
#endif


