/*
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/
/************************************************************************/
/*									*/
/* client.c								*/
/*									*/
/*	Test application that just creates a push button in its window.	*/
/*	It also makes a call to RapInstall using the toplevel shell.	*/
/*									*/
/************************************************************************/
#include <stdio.h>
#include <X11/Xmd.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>

Arg		argList[10];
int		argCnt=0;    
Widget		topLevel,mainWindow;
XtAppContext	appContext;


 
/************************************************************************/
/*									*/
/* Application Resources						*/
/*									*/
/************************************************************************/
typedef struct 
{
    int		mode;
} OptionsRec;

OptionsRec options;

#define Offset(field) XtOffsetOf(OptionsRec,field)
XtResource resources[] = 
{
    {"mode", "Mode", XtRInt, sizeof(int),
     Offset(mode), XtRImmediate, (XtPointer) 1},
    NULL
};
#undef Offset

String fallbackResources[] = 
{
    "*mainWindow.width: 100",
    "*mainWindow.height: 50",
    NULL
};

XrmOptionDescRec optionDesc[] = 
{
    {"-mode", "*mode", XrmoptionSepArg, (XtPointer) 1},
};

void zapProc(Widget w,
	     XtPointer clientData,
	     XtPointer *callbackData)
{
    XtDestroyWidget(w);
}

/************************************************************************/
/*									*/
/* buttonProc - called when the button is pressed.			*/
/*									*/
/************************************************************************/
void buttonProc(Widget 		w, 
		XtPointer 	clientData, 
		XtPointer	*callbackData)
{
    Widget mainWindow = (Widget) clientData;
    Widget anotherButton;
    
    printf("client: keith's buttonProc, mainWindow = %lx\n", mainWindow);
    anotherButton = XtVaCreateManagedWidget("anotherButton",
					    commandWidgetClass,
					    mainWindow,
					    XtNlabel, "ANOTHER BUTTON",
					    NULL);
    XtRealizeWidget(anotherButton);
    
    XtAddCallback(anotherButton,XtNcallback,(XtCallbackProc)zapProc,0);
} /* buttonProc */

/************************************************************************/
/*									*/
/* CreateUI								*/
/*									*/
/************************************************************************/
void CreateUI(Widget topLevel)
{
    Widget someButton;

    mainWindow = XtCreateManagedWidget("mainWindow", 
				       boxWidgetClass, 
				       topLevel,
				       NULL, 0);

    printf("main window is %lx\n", mainWindow);

    someButton = XtVaCreateManagedWidget("listButton", 
					 commandWidgetClass, 
					 mainWindow,
					 XtNlabel,"Push Me",
					 NULL);

    XtAddCallback(someButton,XtNcallback,(XtCallbackProc)buttonProc,mainWindow);
    
} /* CreateUI */

/************************************************************************/
/*									*/
/* Main routine								*/
/*									*/
/************************************************************************/
main(int argc, char **argv)
{
    /********************************************************************/
    /*									*/
    /* Initialize the toolkit.						*/
    /*									*/
    /********************************************************************/
    topLevel = XtAppInitialize(&appContext,
			       "client",
			       optionDesc,XtNumber(optionDesc),
			       &argc,argv,
			       fallbackResources,
			       NULL,0);

    XteaSetupClient(topLevel);

    CreateUI(topLevel);
    
    XtRealizeWidget(topLevel);
    XtAppMainLoop(appContext);
    
} /* main */
