/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/
/*
 * RAP library internals.  Lots of macros and stuff to help out with the rap lib
 * implementation.
 *
 *
 */

#ifndef _RAP_LIB_INT_H_
#define _RAP_LIB_INT_H_


/* byte swap a long literal */
#define LSWAPL(_val) ((((_val) & 0xff) << 24) |\
                   (((_val) & 0xff00) << 8) |\
                   (((_val) & 0xff0000) >> 8) |\
                   (((_val) >> 24) & 0xff))
 
/* byte swap a short literal */
#define LSWAPS(_val) ((((_val) & 0xff) << 8) | (((_val) >> 8) & 0xff))

/* compute the 64-bit-boundary padding for the specified number of bytes */
#define PAD64(_bytes)		((8 - ((unsigned int) (_bytes) % 8)) % 8)

/* compute the total size, including padding, for the specified number of bytes */
#define PADDED_BYTES64(_bytes)	((_bytes) + PAD64(_bytes))

/* Number of 8-byte units in _bytes */
#define WORD64COUNT(_bytes)	(((unsigned int) ((_bytes) + 7)) >> 3)

/* Number of bytes needed to store a string of length _len*/
#define STRING_BYTES(_len)	(4 + (_len) + PAD64(4 + (_len)))

/* Number of bytes needed to store a widget list */
#define CARD32_LIST_BYTES(_len)	(4 + ((_len) * 4))

/* Number of bytes needed to store a CARD16 list */
#define CARD16_LIST_BYTES(_len)	(4 + ((_len) * 2))
    
/* remove a CARD32 from a protocol stream */    
#define EXTRACT_CARD32(_buf, _swap, _val) \
		{ \
      		    (_val) = *((CARD32 *) (_buf)); \
		    (_buf) += 4; \
		    if (_swap) \
			(_val) = LSWAPL(_val); \
		}

/* remove a CARD16 from a protocol stream */
#define EXTRACT_CARD16(_buf, _swap, _val) \
		{ \
      		    (_val) = *((CARD16 *) (_buf)); \
		    (_buf) += 2; \
		    if (_swap) \
			(_val) = LSWAPS(_val); \
		}

/* remove a string from a protocol stream */
#define EXTRACT_STRING(_buf, _swap, _string) \
		{ \
		    CARD32 _len; \
		    EXTRACT_CARD32(_buf, _swap, _len); \
		    (_string) = (char *) malloc((_len) + 1); \
		    memcpy((_string), (_buf), (_len)); \
		    (_string)[(_len)] = '\0'; \
		    (_buf) += (_len); \
		    (_buf) += PAD64(4 + (_len)); \
		}

/* remove an array of bytes from a protocol stream, reading its length*/
#define EXTRACT_BYTES(_buf, _swap, _len, _array) \
		{ \
      		    EXTRACT_CARD32((_buf), (_swap), (_len)); \
		    (_array) = (char *) malloc((_len) + 1); \
		    memcpy((_array), (_buf), (_len)); \
		    (_array)[(_len)] = '\0'; \
		    (_buf) += (_len); \
		    (_buf) += PAD64(4 + (_len)); \
		}

/* remove an array of bytes from a protocol stream, GIVEN its length*/
#define EXTRACT_SIZED_BYTES(_buf, _swap, _len, _array) \
		{ \
		    (_array) = (char *) malloc((_len) + 1); \
		    memcpy((_array), (_buf), (_len)); \
		    (_array)[(_len)] = '\0'; \
		    (_buf) += (_len); \
		    (_buf) += PAD64(4 + (_len)); \
		}

/* push a CARD32 onto a protocol stream */
#define STORE_CARD32(_buf, _val) \
		{ \
	            *((CARD32 *) (_buf)) = (CARD32) (_val); \
		    (_buf) += 4; \
		}

/* push a CARD16 onto a protocol stream */
#define STORE_CARD16(_buf, _val) \
		{ \
	            *((CARD16 *) (_buf)) = (CARD16) (_val); \
		    (_buf) += 2; \
		}

/* push a string onto a protocol stream */
#define STORE_STRING(_buf, _len, _str) \
		{ \
		    STORE_CARD32((_buf), (_len)); \
		    memcpy((_buf), (_str), (_len)); \
		    (_buf) += (_len); \
		    (_buf) += PAD64(4 + (_len)); \
		}

/* push an array of bytes onto a protocol stream, including its length */
#define STORE_BYTES(_buf, _len, _array) \
		{ \
      		    STORE_CARD32((_buf), (_len)); \
		    memcpy((_buf), (_array), (_len)); \
		    (_buf) += (_len); \
		    (_buf) += PAD64(4 + (_len)); \
		}
/* push an array of bytes onto a protocol stream, EXCLUDING its length */
#define STORE_SIZED_BYTES(_buf, _len, _array) \
		{ \
		    memcpy((_buf), (_array), (_len)); \
		    (_buf) += (_len); \
		    (_buf) += PAD64(4 + (_len)); \
		}
      
/* write a widget onto an iceconn */
#define WRITE_CARD32(conn, widget) \
		IceWriteData32((conn), sizeof(Widget), &(widget));

#define WRITE_CARD32_LIST(conn, widgets, numWidgets) \
		{ \
      		    IceWriteData32((conn), sizeof(CARD32), &(numWidgets)); \
		    IceWriteData32((conn), sizeof(Widget) * (numWidgets), \
				   (widgets)); \
		}

#define WRITE_STRING(conn, str) \
		{ \
		    CARD32 _size = strlen(str); \
		    register int _j; \
		    char zero = '\0'; \
		    IceWriteData32((conn), sizeof(CARD32), &_size); \
		    IceWriteData((conn), _size, (str)); \
		    for (_j=0 ; _j<PAD64(4 + _size) ; _j++) \
		        IceWriteData((conn), 1, &zero); \
		}

#define WRITE_STRING_LIST(conn, strings, numStrings) \
		{ \
	            register int _i; \
		    IceWriteData32((conn), sizeof(CARD32), &(numStrings)); \
		    for(_i=0 ; _i<(numStrings) ; _i++) \
			WRITE_STRING((conn), (strings)[_i]); \
		}

#endif /* _RAP_LIB_INT_H_ */
