/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/

#ifndef _RAP_PROTO_H_
#define _RAP_PROTO_H_


#include <X11/Xmd.h>


/***************************************************************************

                    T H E   R A P   P R O T O C O L

  RAP begins communication via a ClientMessage event sent to the
  application.  The format of this message is:

  	message_type = Atom("ExternalAgent")
  	format = 32
  
  	data.l[0] = Atom("RAP")
  	data.l[1] = Atom("IceNetworkIds")
  	data.l[2] = agent window

  This format is compatible with the Xtea specification.  data[0] is
  the name of the protocol we wish to speak.  data[1] is the name of
  the property where the agent's ICE network ids are stored.  data[2]
  is the window on which this property exists.

  RAP is a binary protocol and makes use of the following conventions:

  	CARD8		8-bit unsigned integer
	CARD16		16-bit unsigned integer
	CARD32		32-bit unsigned integer
	INT16		16-bit signed integer
	WINDOW		32-bit value
	WIDGET		32-bit value
	STRING8		ListOfCARD8

	LIST OF FOO comprises a list of FOOs in the following format:

	number		CARD16
	FOO[0]
	   ...
	FOO[number-1]

  RAP defines three basic categories of messages:

	REQUEST         Requests travel from the agent to the client,
			and must be followed by a Reply from the
			client.

	REPLY           Replies travel from the client to the agent and
			are generated in response to a request.

	NOTIFY          Notifies are asynchronous messages which are
			sent to the agent from a client to notify it of
			some change in its state.

  All messages contain the following header information:

  	majorOpcode	Unique to RAP
	minorOpcode	Identifies the type of the message
	sequenceNumber	Identifies message instances uniquely
	length		Total length of the message.

  All replies also contain:

	replySequence   The sequence number of the message being
			replied to.



  BELOW IS THE DEFINITION OF THE FORMAT OF INDIVIDUAL MESSAGES


  RapError

	error code	CARD16
	error string	STRING

	A RapError is generated in response to any request which causes
	an error condition.  The error code uniquely identifies the
	type of the error; the error string provides additional
	information.

  RapAck

	A RapAck is a positive response to a request.  It contains no
	information.

  RapHelloRequest

	A RapHelloRequest is generated by an agent immediately after
	the initiation of a lcient connection.  It it used to pass
	handshaking information back and forth between agent and
	client.

	Currently it passes no data.

  RapHelloReply

	RapHelloReply is generated by a client in response to a
	RapHelloRequest.  Eventually it will return handshaking
	information (resource types, supported callbacks,
	unimplemented messages, etc.) to the agent.

	Currently it passes no data.

  RapQueryTreeRequest

	RapQueryTreeRequest is generated by the agent to request that
	the client transmit its entire widget tree.  It passes no
	information.

  RapQueryTreeReply

	entries				LIST OF entries
	each entry:  
	    widget 			WIDGET 
	    parent              	WIDGET 
	    name 			STRING 
	    class               	STRING 
	    window 			WINDOW 
	    toolkit             	STRING	--editres uses this??

	RapQueryTreeReply is generated in response to a
	RapQueryTreeRequest message.  It contains information about the
	entire widget tree in the application.  It is a requirement of
	the protocol that widgets be returned in a parent-to-child
	order (to simplify implementation of agents).

	The widget uniquely identifies this object in the client.
	Parent is the direct ancestor of the widget (this value can be
	zero if the object has no parent).  Class and name are
	Xrm-style strings which identify the object.  Window is the XID
	of the window currently used by the widget.  If thewidget is
	unrealized then 0 is returned; if the widget is a gadget
	(non-windowed object) then 2 is returned.

	Toolkit is a string which identifies the widget set or toolkit
	the widget belongs to (this was added in R6 editres?)

  RapGetResourcesRequest

	widgets				LIST OF WIDGETS

	RapGetResourcesRequest passes a list of widget identifiers to
	the client.  The client is expected to return the names, class,
	type, and kind (either normal or constraint) of each of the
	widgets in the list.

  RapGetResourcesReply

  	entries				LIST OF entries
	each entry:
		widget			WIDGET
		error code		CARD16
		[ error string 		STRING ]
		[ resources:		LIST OF resources
		each resource:
			name		STRING
			class		STRING
			kind		[normal | constraint]
			type		STRING ]

	RapGetResourcesReply is used to return a list of all of the
	resources in a list of widgets.  For each widget requested,
	GetResourcesReply passes back the widget identifier, an error
	code which indicates whether or not an error occurred for the
	particular widget (or RapErrorNone if no error occurred).

	Depending on whether an error occurred, either an error string
	or the number of resources for the particular widget will be
	returned.  If there was no error, a list of resources will
	follow, containing the kind, name, class, and type of the
	resource.

  RapGetGCValuesRequest
 
        GCc                         LIST OF GCS
 
 
  RapGetGCValuesReply
 
        entries                         LIST OF entries
        each entry:
                gc                      GC
                error code              CARD16
                [ error string          STRING ]
                [ values:               XGCValues ]
 


  RapGetValuesRequest

  	widget				WIDGET
	resource names			LIST OF STRINGS
	
  	RapGetValuesRequest is used by an agent to download a list of resource
	values from a specified widget.  The information is passed back via
	RapGetValuesReply.

  RapGetValuesReply

  	widget				WIDGET
	values				LIST OF values
	each value:
		name			STRING
		error code		CARD16
		error string		STRING
		native type		STRING
		return type		STRING
		length of data		CARD16
		data			LIST OF CARD8
		
	RapGetValuesReply passes back a list of resource values to the
	requesting agent.  The call includes the widget for which it is
	returning values, and the number of values returned.

	Each value contains the name of the resource, and an error code
	indicating whether the request failed for the particular
	resource.  If an error did occur, the value contains an error
	string.  Otherwise, it includes the "native" type of the
	resource value, the type it is actually being transmitted in,
	and then a variable-length array of bytes containing the actual
	data.

  RapFullQueryTreeRequest
        RapFullQueryTreeRequest sends over request forall widgets, all 
        resources of those widgets and all values of those resources

  RapFullQueryTreeReply
        entries                         LIST OF entries
        each entry:
            widget                      WIDGET
            [ resources:                LIST OF resources
                each resource:
                        name            STRING
                        class           STRING
                        kind            [normal | constraint]
                        type            STRING ]
			length of data  CARD16
			data		LIST OF CARD8		
            parent                      WIDGET
            name                        STRING
            class                       STRING
            window                      WINDOW
            toolkit                     STRING  --editres uses this??

  RapSetValuesRequest

  	name				STRING
	type				STRING	-- editres has this... why?
	value				STRING
	widgets				LIST OF WIDGETS
		
  	RapSetValuesRequest allows an aget to change the value of a particular
	resource across a set of widgets.  The message passes the name and value
	of a resource (specified as a string?), and a list of widgets to set the
	resource on.  This message generates a RapSetValuesReply.

  RapSetValuesReply

  	name				STRING
	type				STRING
	value				STRING
	entries				LIST OF entries
	each entry:
		widget			WIDGET
		error code		CARD16
		error string		STRING

	RapSetValuesReply returns an indication of success or failure
	to an agent which has issued a RapSetValuesRequest message.
	The message returns the resource name, type, and value
	information, along with a list of entries.  Each entry includes
	the widget, an error code (which may be RapErrorNone), and an
	error string.

  RapAddNotifyRequest

	callback ids			LIST OF CARD8

	This request installs a set of callbacks for asynchronous
	notifications on the client.  The tokens representing the
	individual callbacks may be implementation dependent (perhaps
	HelloReply should send back a list of supported callbacks?)

	The client responds with either RapError or RapAck.

  RapRemoveNotifyRequest

	callback ids			LIST OF CARD8

	This request removes a set of callbacks for asynchronous
	notifications from the client.  The tokens representing the
	individual callbacks may be implementation dependent.  The
	client responds with either  RapError or RapAck.

  RapObjectToWindowRequest

	widgets				LIST OF WIDGETS

  RapObjectToWindowReply

	entries				LIST OF entries
	each entry:
		widget			WIDGET
		error code		CARD16
		[ error string		STRING ]
		[ window		WINDOW ]

  RapWindowToObjectRequest

	windows				LIST OF WINDOW

  RapWindowToObjectReply

	entries				LIST OF entry
	each entry:
		window			WINDOW
		error code		CARD16
		error string		STRING
		widget			WIDGET

  RapLocateObjectRequest
 
	widgets				LIST OF WIDGET

  RapLocateObjectReply

	entries				LIST OF entry
	each entry:
		widget			WIDGET
		error code		CARD16
		[ error string		STRING ]
		[ window		WINDOW
		mapped			CARD8
		x			INT16
		y			INT16
		width			CARD16
		height			CARD16
		border width		CARD16 ]

  RapGetActionsRequest

  RapGetActionsReply

  RapDoActionRequest

  RapSelectEventRequest

	event ids			LIST OF CARD16

  RapSelectRequestRequest

	request ids			LIST OF CARD16

  RapCloseConnectionRequest

  RapCreateNotify

  RapConfigNotify

  RapDestroyNotify

  RapGeometryNotify

  Rap ChangeNotify

  RapRequestNotify

  RapEventNotify


  *************************************************************************/  



/*
** Definition of constants used by RAP.
*/

#define RapName				"RAP"
#define RapVendor			"X Consortium"
#define RapRelease			"1.0"
#define RapMajorVersion			1
#define RapMinorVersion			0

/*
** Definition of RAP minor opcodes
*/

typedef enum {
    RapError				= 0,
    RapAcknowledge			= 1,

    RapHelloRequest			= 2,
    RapQueryTreeRequest			= 3,
    RapFullQueryTreeRequest		= 4,
    RapGetResourcesRequest		= 5,
    RapGetValuesRequest			= 6,
    RapGetGCValuesRequest		= 7,
    RapSetValuesRequest			= 8,
    RapWindowToObjectRequest		= 9,
    RapObjectToWindowRequest		= 10,
    RapLocateObjectRequest		= 11,
    RapSelectEventRequest		= 12,
    RapSelectRequestRequest		= 13,
    RapAddNotifyRequest			= 14,
    RapRemoveNotifyRequest		= 15,
    RapCloseConnectionRequest		= 16,

    RapHelloReply			= 17,
    RapQueryTreeReply			= 18,
    RapFullQueryTreeReply		= 19,	
    RapGetResourcesReply		= 20,
    RapGetValuesReply			= 21,
    RapGetGCValuesReply			= 22,
    RapWindowToObjectReply		= 23,
    RapObjectToWindowReply		= 24,
    RapLocateObjectReply		= 25,

    RapCreateNotify			= 26,
    RapDestroyNotify			= 27,
    RapChangeNotify			= 28,
    RapConfigureNotify			= 29,
    RapGeometryNotify			= 30,

    RapEventNotify			= 31,
    RapRequestNotify			= 32
} RapMessageTypes;

/*
** Definition of RAP error codes -- INCOMPLETE
*/

typedef enum {
    RapNoError				= 0,
    RapNoSuchObject			= 1,
    RapNoSuchResource			= 2,
    RapNoSuchGraphicsContext		= 3,
    RapCannotConvertType		= 4,
    RapErrorAddNotify			= 5
} RapErrorTypes;


/*
** Boolean types for resource kinds (see RapGetResourcesReply)
*/

typedef enum {
    RapNormalResource 			= 0,
    RapConstraintResource 		= 1
} RapResourceKind;

/*
** Special tokens for windows
*/
typedef enum {
    RapUnrealized			= 0,
    RapGadget				= 2
} RapWindowTypes;

/*
** Enumeration for the hook callbacks
*/

typedef enum {
    RapCreateHook			= 1,
    RapDestroyHook			= 2,
    RapChangeHook			= 3,
    RapConfigureHook			= 4,
    RapGeometryHook			= 5
} HookCallbacks;


/*
** Structure defs for messages.
*/


/* This header is sent on every message */
typedef struct _RapMessageHeader {
    CARD8	majorOpcode;
    CARD8	minorOpcode;
    CARD16	sequenceNumber;
    CARD32	length;
} RapMessageHeader;

/* RapHelloMessage  - its used to store handshaking information 
   between agent and client.  It current stores no data */
typedef struct _RapHelloMessage
{
    CARD8       majorOpcode;
    CARD8       minorOpcode;
    CARD16      sequenceNumber;
    CARD32      length;
    CARD32 	window;
    CARD32	pad; 
} RapHelloMessage;


/* RapAckMessage
 *
 * When the client receives a addnotify request -- it will either send a
 * ack message or an error msg -- this is the set ack message structure
 */
typedef struct _RapAckMessage
{
    CARD8       majorOpcode;
    CARD8       minorOpcode;
    CARD16      sequenceNumber;
    CARD32      length;
    CARD16      reqSequenceNumber;   /* sequence number of requesting msg */
    CARD32      pad1;
    CARD16      pad2;
} RapAckMessage;


/* RapErrMessage
 *
 * When the client receives a addnotify request -- it will either send a
 * ack message or an error msg -- this is the set err message structure
 * note: there is no need to pad here - since there is gonna be a
 * variable length string attached to end of msg anyway ...
 */
typedef struct _RapErrMessage
{
    CARD8       majorOpcode;
    CARD8       minorOpcode;
    CARD16      sequenceNumber;
    CARD32      length B32;
    CARD16      reqSequenceNumber;   /* seq num of requesting message */
    CARD16      errCode;
} RapErrMessage;
 

#define sz_RapMessageHeader	8
#define sz_RapAckMessage	16
#define sz_RapHelloMessage	16

#endif /* _RAP_PROTO_H_ */
