/*
  this is the code to the fakepoll function.
  Joel Zimba November 13, 1996
*/
/*details
  ok, what i am going to do is call select with all the descriptors in the
  pollfd array in each field of select() i.e. read_fds, write_fds and 
  except_fds.
  that way I can use the macros to check them and set the revents right.
*/
#include "poll.h"

int poll(pollfd *fds,unsigned long nfds, 
	 int timeout)
{
  struct timeval *tv;
  int result=0;
  fd_set reads,writes,exceptions;
  unsigned long i;

  FD_ZERO(&reads);
  FD_ZERO(&writes);
  FD_ZERO(&exceptions);
  //set up the descriptors
  for(i=0;i<nfds;i++){
    //this reads events and put them in right place
    if(fds[i].events==POLLIN | POLLRDNORM | POLLRDBAND | POLLPRI){
      FD_SET(fds[i].fd,&reads);
    }
    if(fds[i].events==POLLOUT | POLLWRNORM | POLLWRBAND){
      FD_SET(fds[i].fd,&writes);
    }
    if(fds[i].events== POLLERR | POLLHUP | POLLNVAL){
      FD_SET(fds[i].fd,&exceptions);
    }
  }

  //set up the timeval structure 
  if(timeout==INFTIM){
    tv=NULL;
  }
  else{
    tv->tv_sec=timeout;
    tv->tv_usec=0;
  }

  //actual cal to select
  result=select((int)nfds,&reads,&writes,&exceptions,tv);

  //set all the fds info properly
  //since select doesn't break things down into the nice values
  //like pol has for POLLOUT and things,
  //it just lets it work or not,
  //I will just set most ofthe relavent flags.
  //if we need more granularity, we can use errno or something later
  if(result){
    //i don't think we need to do much unless there are fds that have 
    //been active!
    for(i=0;i<nfds;i++){
       
      if(FD_ISSET(fds[i].fd,&reads)){
	fds[i].revents=POLLIN | POLLRDNORM | POLLRDBAND | POLLPRI;
      }
      if(FD_ISSET(fds[i].fd,&writes)){
	fds[i].revents=POLLOUT | POLLWRNORM | POLLWRBAND;
      }
      if(FD_ISSET(fds[i].fd,&exceptions)){
	fds[i].revents=POLLERR | POLLHUP | POLLNVAL;
      }

    }
  }

  return(result);
}
