//  -*-Mode: C++-*-
#ifndef __Action_h__
#define __Action_h__

//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//  The Action class is a virtual base class intended fur use by other
//  classes in Mercator that wish to participate in the action
//  mechanism in the applications model manager.  The subclass should define 
//  an Invoke method that performs that action required.  The Invoke method is
//  passed several strings containing information that might be of interest to
//  the action class.  The contents and number of arguments varies depending
//  on the particular event that invokes the action.
//

//
// I've made some changes to this class so that it can be extended to new
// subclasses more easily.  Now, whenever an action-derived instance is
// created, it is automatically added into a list of action instances
// maintained in Action, and indexed by name.
//
// When a named action is invoked, the corresponding subclass instance is
// looked up and it's virtual Invoke method is fired.
//
// Currently I'm using a dict for this, so it's impossible to have multiple
// actions with the same name.  Eventually it'd be nice to support multiple
// actions (perhaps from multiple subclasses, eg, TCL, Python, IN3, ...)
// that all get fired when the action is invoked.
//
//			-keith 21jan95
//

//
// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Action.h,v $
// $Revision: 6.3 $
// $Date: 1995/07/28 14:00:01 $
//
 
#ident "$Id: Action.h,v 6.3 1995/07/28 14:00:01 spine Exp $"


class Action;
typedef Action *ActionPtr;

#include "mString.h"
#include "mDictValPtr.h"

class Action {
private:
    static mDictValPtr<mString,Action> * actions;
    
    static void		CreateAction(Action *, const char *name);
    static void		DeleteAction(const char *name);
    
protected:
    char 		*name;

public:
    Action(const char *name);
    virtual ~Action();
    
    virtual int 	Invoke(int argc, char *argv[],
			       const char **result = NULL) = 0;

    static int 		Invoke(const char *name, int argc, char *argv[],
			       const char **result = NULL);
    
    const char 		*GetName() const
			    { return name; }
};


#endif __Action_h__
