//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
// The is the loadable Alva module.
//
// The same Alva shared object is used for all Alva models (well, at least
// the two we have experience with:  the Alva 3/20 and Alva 3/80).  When
// an Alva instance is created, it queries the Config database and searches
// for an entry called "alvaModel" of type string.  Legal values for this
// entry are "Alva 3/20" and "Alva 3/80."  Depending on the value of this
// entry, the object will configure itself appropriately for either model.
//
// If there is an error, the class defaults to Alva 3/20.
//
// Some of the key processing is handled internally to this class, and other
// processing is dispatched externally. 
//
// The DisplayBraille() method takes a string of text to display, and a
// position to display it at (the position parameter defaults to zero if it
// is not provided).  The default behavior sets the "text" member data to
// point to this data and displays as much as possible of it on the display
// cells.  If the text is larger than will fit on the display cells, then the
// left/right keys pan over this area.
//
// If a position is provided, the text is inserted at the specified position.
//
// The other keys are dispatched externally, since they require interaction
// with the screenreader.
//
//		keith edwards	26jul95
//

#ifndef _ALVA_H_
#define _ALVA_H_

#include "Braille.h"

class Alva : public Braille {
private:
    int			fd;
    FILE                *file;
    char		**text;
    int			row, column;
    int			numRows;
    int                 jumpScrollMode;
    
    Capabilities	capabilities;

    static int		OpenAlva(const char *hostname, short port);

public:
    Alva(const char *hostname = NULL, short port = 1350);
    virtual ~Alva();

    virtual const Capabilities& GetCapabilities() const;

            int         ConnectServer(const char *hostname = "localhost",
				      short port = 1350);
            int         StartServer();

    virtual int		HandleActivity(int fd, short type);
    virtual void	Shutdown(int fd);

    virtual void	DisplayRaw(const char *text, int rawPosition = 0);
    virtual void	DisplayBraille(const char *chars, int position = 0);
    virtual void	DisplayStatus(const char *chars);

    virtual int         JumpScroll();
    virtual void        JumpScroll(int js);
    virtual void        ToggleJumpScroll();

    virtual void	SetText(const char *text[], int numRows);

    virtual void        ShutDown();
};

#endif /* _ALVA_H_ */
