// -*-Mode:C++-*-
#ifndef __APPMODELMGR_H__
#define __APPMODELMGR_H__

//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
// This is the class AppModelMgr.  It is a derived class of Storage.
// There should only be one instance of AppModelMgr at a time.
// It stores information about the AppModelMgr (i.e. Clients) currently
// active in the Mercator system.  It also stores information that the
// clients need to know to run.
// Note: In the previous versions of Mercator this file was called AppModelMgr
// The data that is stored is specifically:
//
//   Data
//   ----
//   List of clients (i.e. list of the applications)
//   XtObject pointer stroed by XtObject number (ex. XtObject1)
//   XtObject pointer stored by window number
//   List of event grabs for all applications (i.e. global key grabs)
//


//
// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/AppModelMgr.h,v $
// $Revision: 6.8 $
// $Date: 1995/08/29 22:15:38 $
//

#ident "$Id: AppModelMgr.h,v 6.8 1995/08/29 22:15:38 keith Exp $"


class AppModelMgr;
typedef AppModelMgr *AppModelMgrPtr;

class RAP;

#include <X11/Intrinsic.h>
#include "Action.h"
#include "Client.h"
#include "Storage.h"
#include "XServer.h"
#include "mString.h"
#include "mList.h"
#include "mDictPtrPtr.h"
#include "mDictValPtr.h"

class AppModelMgr : public Storage {
    //
    // This is for XServer::CreateEventBind(char*, EventPtr, Action*)
    // Specifically so that it can use xtobjectNames
    //
    friend class XServer; 

private:
    mList<Client>                   *clientList;      // all clients
    ClientPtr                        currentClient;   // the current client
    mDictValPtr<mString,Client>     *clientNames;     // Clients by name
    mDictValPtr<mString,XtObject>   *xtobjectNames;   // names to objects
    mDictValPtr<Window,XtObject>    *xtobjectWindows; // window ids to objects
    mDictPtrPtr<EventStruct,Action> *eventActions;    // events & their actions
  
public:
    AppModelMgr();
    virtual ~AppModelMgr();

    // It'd be nice if we could pass an XID when we create clients...
    Client 		*CreateClient(RAP *rap);
    int      		DeleteClient(XID);
    Client 		*GetClient(XID);
    Client 		*GetClient(mString);
    Client 		*GetCurrentClient();
    void     		SetCurrentClient(Client *c);
    Client 		*SetToNextClient(); // Set current Client to next Client
    Client 		*SetToPrevClient(); // Set current Client to prev Client

    mList<Client> 	*GetAllClients();

    //called when XtObject is created by its Client
    void 		CreateXtObject(XtObject*);
    //called when XtObject is deleted by its Client
    void 		DeleteXtObject(XtObject*);
    //called when the win Id of an XtObject changes
    void 		ChangeWindowId(XtObject *obj, Window newId);

    XtObject 		*GetXtObject(Window id);
    XtObject 		*GetXtObject(const mString& name);
    XtObject 		*GetXtObjectByLongName(const mString& name);

    // virtual functions required to be implemented by the base class Storage
    int  		DispatchEvent(EventPtr event);
    void 		CreateEventBind(EventPtr event, Action *function);
    int  		DeleteEventBind(EventPtr event);

    mDictPtrPtr<EventStruct,Action> *GetActions();
};

inline Client *AppModelMgr::GetCurrentClient()
		{ return currentClient; }
inline mList<Client> *AppModelMgr::GetAllClients()
		{ return clientList; }
inline mDictPtrPtr<EventStruct, Action> *AppModelMgr::GetActions()
		{ return eventActions; }
inline XtObject *AppModelMgr::GetXtObject(Window w)
		{ return xtobjectWindows->lookup(w); }
inline XtObject *AppModelMgr::GetXtObject(const mString& name)
		{ return xtobjectNames->lookup(name); }
inline void AppModelMgr::CreateEventBind(EventPtr event, Action *function)
		{ eventActions->insert(event, function); }

#endif /* __APPMODELMGR_H__ */
