// -*- c++ -*-
// Attribute.h
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

#ifndef _attribute_h
#define _attribute_h

#include "Util.h"
#include "Template.h"
#include "mString.h"
#include "mDictValPtr.h"

// void ProcessAttributeLine(char *name, int ftype, char *defaultname,int cltype);
extern "C" int attribparse(void);

class Attribute;
class Config;

class Attribute {
    
    friend Config;
    
public:
    // CLASS and APP must match with attributes.y
    typedef enum {CLASS=1, APP=2} COType;

private:
    static mDictValPtr<mString, Attribute> *classAttributeDictionary;
    static mDictValPtr<mString, Attribute> *appAttributeDictionary;
    
    static int 		EmptyDicts();
    static int 		validDicts;

    typedef struct {
	char 		*name;
	FieldType 	tag;
	char 		*refname;
	COType 		ctype;
    } AttribStruct;

    static AttribStruct 	implicit[];

    char		*name;    // what's our name, ref'd by TCL code as well
    FieldType 		tag; // what are we ? Boolean? Integer? String or list?
    char 		*reference_name; // name referenced by everyone else
    COType 		ctype;  // APP or CLASS

public:
    Attribute(const char *n, const char *ref_name, FieldType t, COType cotype);
    virtual ~Attribute();

    const char 		*Name() const;
    const char 		*RefName() const;
    FieldType 		Tag() const;
    COType 		CType() const;

    int 		InstallAttribute();
    
    static void 	InstallImplicitAttributes();
    static Attribute 	*GetClassAttributeByName(const char *n);
    static Attribute 	*GetAppAttributeByName(const char *n);

    static int 		ParseAttribFile(const char *n);
    static void		FreeAttrib();
};

inline const char 	*Attribute::Name() const     { return name; }
inline const char 	*Attribute::RefName() const  { return reference_name; }
inline FieldType 	 Attribute::Tag() const      { return tag; }
inline Attribute::COType Attribute::CType() const    { return ctype; }

#endif

