// -*-Mode: C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
//
// This is my implementation of the generic audio interface for the
// AudioFIle subsystem. 
//

// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/AudioFile.h,v $
// $Date: 1995/07/28 20:17:08 $

// STUPID RCS WON'T DO C++ COMMENTS!
/* 
 * $Log: AudioFile.h,v $
 * Revision 1.2  1995/07/28  20:17:08  spine
 * Added copyright notice.
 *
 * Revision 1.1  1995/07/25  23:01:46  spine
 * Initial revision
 *
 */

#ifndef AUDIOFILE_H
#define AUDIOFILE_H

#include "Audio.h"
extern "C" {
#include "os-interface.h"
}
#include "Thread.h"

// AudioFile Magic 
#include <AF/audio.h>
#include <AF/AFlib.h>
#include <AF/AFUtils.h>
extern "C" {
#include <AF/aconvert.h>
}
#include <synch.h>

//Rouge Wave template to hold the work crew 
#include <rw/tpordvec.h>
// Rogue Wave template to hold the dictionary mapping SOUNDID to threads 
#include <rw/tvhdict.h>


// This type holds all the information about a given thread in the work crew 
// This is subclassed from Thread to get our own Thread  for each WorkCrewMember

class WorkCrewMember : public Thread {
    // crucial: this says if we are in the process of playing a sound right now or not
    int active;
    // fd that the data is being read from
    int fd;
    // boolean: TRUE iff we are need to muffle the current sound
    int muffle; 
    // boolean: TRUE iff we are looped
    int looped;
    // boolean: TRUE when we need to change the volume
    int volumeChanged;
    // holds the volume
    int volume;
    // These hold the conversion contexts so we can do the 8-->16 and
    // 16-->8 bit conversions
    AConvertCtxt *sixteen2eightCtx,*eight2sixteenCtx;

    // These two are the mutex/condvar pair that allows the driver (main)
    // thread ot signal us and tell us we should start a new sound... the caller
    // uses the mutex to lock access to the active slot (above) and after
    // doing that (and unlocking) uses COND_signal to tell us to check it again.
    COND_T condvar;
    MUTEX_T mutex;
    // per connection audio file information
    AFAudioConn *aud;
    AC ac;
    // af attributes... sets the type and preemption mode
    AFSetACAttributes attributes;
    // What time is it w.r.t. AF?
    ATime currentTime;
    // buffering info (the buffer, its size in bytes, its size in samples )
    unsigned char *audioBuffer;
    unsigned bufSize;
    unsigned sampsPerBuffer;
    // private section to get the compiler to tell us when they are
    // called by others... 
    // THESE METHOD CALLS ARE NOT ALLOWED AS THEY ARE NONSENSE.
    WorkCrewMember(const WorkCrewMember &w);
    WorkCrewMember & operator=(const WorkCrewMember &w);
    int operator<(const WorkCrewMember &);
public:
    // These override the thread class' methods 
    virtual void Initialize(); // init the worker 
    virtual void Main(); // wait to go 
    // create one ready for service 
    WorkCrewMember(unsigned samplingRate,unsigned bytesPerSample, 
		   unsigned channels);
    // start it going ...
    void StartSound(int fd, int looped,int volume,int muffled);
    // stop it by telling the thread to stop spooling to AF
    void StopSound();
    // make sure Rogue Wave has some help
    int operator==(const WorkCrewMember &w) const;
    // retrieve our status... this checks the active slot (above)
    int IsActive() const;
    // change the volume of the current sound
    void SetVolume(int newVolume);
};

//
//This the subclass of the Audio object that is actually loaded
// into Mercator and used  to play sound files.
//

class AudioFile : Audio {
private:
    // Hold the work crew 
    RWTPtrOrderedVector<WorkCrewMember> WorkCrew;
    // Dictionary to find the work crew for a sound 
    RWTValHashDictionary<unsigned,unsigned> *SoundIdDictionary;
    // counter for the sound ids... we give out IDs starting with 1 (not 0)
    unsigned soundId;
public:
    // public constructor: Calling this with a hostname (really a DISPLAY environment
    // variable value) will cause us to connect to that server... You should probably
    // just use NULL as that follows the user's already existing DISPLAY var
    AudioFile(const char *hostname=NULL);
    // delete the sucker... this causes a closedown of the audio connection
    // and the destruction of the threads
    virtual ~AudioFile(void);
    // main API entry point... play a sound file
    virtual SoundID PlaySound(const char *sound_file, float *rate, 
			      int volume, int muffle, int looped);
    // stop a sound that is in progress
    virtual int StopSound(Audio::SoundID);
    
};

#endif /* AUDIOFILE_H */

