//
// Base-class for Braille loadables.
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
#include "Braille.h"
#include "Error.h"

Braille::Capabilities::Capabilities() :
    displayCells(0),
    statusCells(0),
    highlightSupported(0),
    hasCursorKeys(0),
    hasProgKey(0),
    hasHomeKey(0),
    otherKeys(0)
{
}

Braille::Capabilities::Capabilities(int displayCells_, int statusCells_,
				    int highlightSupported_,
				    int hasCursorKeys_, int hasProgKey_,
				    int hasHomeKey_, int otherKeys_) :
    displayCells(displayCells_),
    statusCells(statusCells_),
    highlightSupported(highlightSupported_),
    hasCursorKeys(hasCursorKeys_),
    hasProgKey(hasProgKey_),
    hasHomeKey(hasHomeKey_),
    otherKeys(otherKeys_)
{
}

Braille::Capabilities::Capabilities(const Capabilities& other) :
    displayCells(other.displayCells),
    statusCells(other.statusCells),
    highlightSupported(other.highlightSupported),
    hasCursorKeys(other.hasCursorKeys),
    hasProgKey(other.hasProgKey),
    hasHomeKey(other.hasHomeKey),
    otherKeys(other.otherKeys)
{
}

Braille::Capabilities&
Braille::Capabilities::operator=(const Capabilities& other)
{
    displayCells = other.displayCells;
    statusCells = other.statusCells;
    highlightSupported = other.highlightSupported;
    hasCursorKeys = other.hasCursorKeys;
    hasProgKey = other.hasProgKey;
    hasHomeKey =other.hasHomeKey;
    otherKeys = other.otherKeys;

    return *this;
}

////////////////////////////////////////////////////////////////////////

Braille::Braille(const char *name) :
    FDInterest(name),
    brailleName(NULL),
    translation(NULL)
{
    brailleName = Strdup(name);

    // Initialize a default translation
    Translation(Braille::UCPT);
}

Braille::~Braille()
{
    delete [] brailleName;
    delete [] translation;
}

const Braille::Capabilities&
Braille::GetCapabilities() const
{
    static Capabilities cap;
    return cap;
}

//
// If the baseclass HandleActivity() is getting called then someone's either
// gone and registered us as a handler for a descriptor (a bad idea), or
// a derived class has not provided its own HandleActivity.
//
// Either case indicates a serious logic error, so we abort.
//
int
Braille::HandleActivity(int fd, short type)
{
    Error(EL_ABORT, "BrailleHandleActivity",
	  "This method can never be called.\n",
	  "\tEither someone has registered the generic Braille class as a",
	  "\thandler for a descriptor, or (more likely) a derived class is",
	  "\tnot implementing HandleActivity().  This indicates a serious",
	  "\tprogrammer logic error and we cannot continue.", NULL);
    
    return 0;
}

void
Braille::Shutdown(int fd)
{
    MarkForCleanup();
}

void
Braille::DisplayRaw(const char *text, int rawPosition)
{
    Error(EL_INFORMATIONAL, "Braille::Raw Output", text, NULL);
}

void
Braille::DisplayBraille(const char *text, int position)
{
    Error(EL_INFORMATIONAL, "Braille::Output", text, NULL);
}

void
Braille::DisplayStatus(const char *text)
{
    Error(EL_INFORMATIONAL, "Braille::Status Cell Output", text, NULL);
}

void
Braille::ToggleJumpScroll()
{
    Error(EL_DEBUG, "Braille::ToggleJumpScroll", "toggled", NULL);
}

int
Braille::JumpScroll()
{
    Error(EL_DEBUG, "Braille::JumpScroll", NULL);
    return 0;
}

void
Braille::JumpScroll(int js)
{
    Error(EL_DEBUG, "Braille::JumpScroll", NULL);
}
