//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
/* File: BrailleTables.c */
/* Braille translation table initializations */

#include "Braille.h"

#define d0 0
#define d1 1
#define d2 2
#define d3 4
#define d4 8
#define d5 16
#define d6 32
#define d7 64
#define d8 128

/* Upper Case Primary Translation Table */
/*  USA Braille */
const unsigned char Braille::UCPT[] = {
    d0,                          /* space (null) */
    d1|d7|d8,                  /* face */
    d1|d2|d7|d8,              /* face inverted */
    d1|d4|d7|d8,              /* hearts */
    d1|d4|d5|d7|d8,          /* diamonds */
    d1|d5|d7|d8,              /* clubs */
    d1|d2|d4|d7|d8,          /* spades */
    d1|d2|d4|d5|d7|d8,      /* dot */
    d1|d2|d5|d7|d8,          /* inverted dot */
    d2|d4|d7|d8,              /* circle */
    d2|d4|d5|d7|d8,          /* inverted circle */
    d1|d3|d7|d8,              /* man symbol */
    d1|d2|d3|d7|d8,          /* woman symbol */
    d1|d3|d4|d7|d8,          /* note */
    d1|d3|d4|d5|d7|d8,      /* note double */
    d1|d3|d5|d7|d8,          /* sun */
    d1|d2|d3|d4|d7|d8,      /* triangle right */
    d1|d2|d3|d4|d5|d7|d8,  /* triangle left */
    d1|d2|d3|d5|d7|d8,      /* arrow up and down */
    d2|d3|d4|d7|d8,          /* exclamation sign double */
    d2|d3|d4|d5|d7|d8,      /* paragraph English */
    d1|d3|d6|d7|d8,          /* paragraph German */
    d1|d2|d3|d6|d7|d8,      /* block horizontal */
    d2|d4|d5|d6|d7|d8,      /* arrow up, down, underlined */
    d1|d3|d4|d6|d7|d8,      /* arrow up */
    d1|d3|d4|d5|d6|d7|d8,  /* arrow down */
    d1|d3|d5|d6|d7|d8,      /* arrow right */
    d2|d4|d6|d7|d8,          /* arrow left */
    d1|d2|d5|d6|d7|d8,      /* rectangle half */
    d1|d2|d4|d5|d6|d7|d8,  /* arrow left and right */
    d4|d5|d7|d8,              /* triangle up */
    d4|d5|d6|d7|d8,          /* triangle down */
    d0,                          /* space */
    d2|d3|d4|d6,              /* exclamation mark */
    d5,                          /* double quotes */
    d3|d4|d5|d6,              /* (number sign) */
    d1|d2|d4|d6,              /* dollar */
    d1|d4|d6,                  /* percent */
    d1|d2|d3|d4|d6,          /* (ampersand) */
    d3,                          /* apostroph */
    d1|d2|d3|d5|d6,          /* parenthese open */
    d2|d3|d4|d5|d6,         /* parenthese close */
    d1|d6,                      /* asterisk */
    d3|d4|d6,                  /* plus */
    d6,                          /* comma */
    d3|d6,                      /* minus */
    d4|d6,                      /* point */
    d3|d4,                      /* slash */
    d3|d5|d6,                  /* zero */
    d2,                          /* one */
    d2|d3,                      /* two */
    d2|d5,                      /* three */
    d2|d5|d6,                  /* four */
    d2|d6,                      /* five */
    d2|d3|d5,                  /* six */
    d2|d3|d5|d6,              /* seven */
    d2|d3|d6,                  /* eight */
    d3|d5,                      /* nine */
    d1|d5|d6,                  /* colon */
    d5|d6,                      /* semi-colon */
    d1|d2|d6,                  /* less */
    d1|d2|d3|d4|d5|d6,      /* equal */
    d3|d4|d5,                  /* greater */
    d1|d4|d5|d6,              /* question mark */
    d4|d7,                      /* at sign */
    d1|d7,                      /* capital a */
    d1|d2|d7,                  /* capital b */
    d1|d4|d7,                  /* capital c */
    d1|d4|d5|d7,              /* capital d */
    d1|d5|d7,                  /* capital e */
    d1|d2|d4|d7,              /* capital f */
    d1|d2|d4|d5|d7,          /* capital g */
    d1|d2|d5|d7,              /* capital h */
    d2|d4|d7,                  /* capital i */
    d2|d4|d5|d7,              /* capital j */
    d1|d3|d7,                  /* capital k */
    d1|d2|d3|d7,              /* capital l */
    d1|d3|d4|d7,              /* capital m */
    d1|d3|d4|d5|d7,          /* capital n */
    d1|d3|d5|d7,              /* capital o */
    d1|d2|d3|d4|d7,          /* capital p */
    d1|d2|d3|d4|d5|d7,      /* capital q */
    d1|d2|d3|d5|d7,          /* capital r */
    d2|d3|d4|d7,              /* capital s */
    d2|d3|d4|d5|d7,          /* capital t */
    d1|d3|d6|d7,              /* capital u */
    d1|d2|d3|d6|d7,          /* capital v */
    d2|d4|d5|d6|d7,          /* capital w */
    d1|d3|d4|d6|d7,          /* capital x */
    d1|d3|d4|d5|d6|d7,      /* capital y */
    d1|d3|d5|d6|d7,          /* capital z */
    d2|d4|d6|d7,              /* rectangular bracket open */
    d1|d2|d5|d6|d7,          /* backslash */
    d1|d2|d4|d5|d6|d7,      /* rectangular bracket close */
    d4|d5|d7,                  /* accent circonflex */
    d4|d5|d6|d7,              /* underline */
    d4,                          /* accent grave */
    d1,                          /* a */
    d1|d2,                      /* b */
    d1|d4,                      /* c */
    d1|d4|d5,                  /* d */
    d1|d5,                      /* e */
    d1|d2|d4,                  /* f */
    d1|d2|d4|d5,              /* g */
    d1|d2|d5,                  /* h */
    d2|d4,                      /* i */
    d2|d4|d5,                  /* j */
    d1|d3,                      /* k */
    d1|d2|d3,                  /* l */
    d1|d3|d4,                  /* m */
    d1|d3|d4|d5,              /* n */
    d1|d3|d5,                  /* o */
    d1|d2|d3|d4,              /* p */
    d1|d2|d3|d4|d5,          /* q */
    d1|d2|d3|d5,              /* r */
    d2|d3|d4,                  /* s */
    d2|d3|d4|d5,              /* t */
    d1|d3|d6,                  /* u */
    d1|d2|d3|d6,              /* v */
    d2|d4|d5|d6,              /* w */
    d1|d3|d4|d6,              /* x */
    d1|d3|d4|d5|d6,          /* y */
    d1|d3|d5|d6,              /* z */
    d2|d4|d6,                  /* brace open */
    d1|d2|d5|d6,              /* dashed vertical line */
    d1|d2|d4|d5|d6,          /* brace close */
    d4|d5,                      /* tilde */
    d4|d5|d6,                  /* open triangle */
    d1|d2|d3|d4|d6|d7,      /* capital c cedille */
    d1|d2|d5|d6,              /* u diaeresis */
    d1|d2|d3|d4|d5|d6,      /* e aigue */
    d1|d6,                      /* a circonflex */
    d3|d4|d5,                  /* a diaeresis */
    d1|d2|d3|d5|d6,          /* a grave */
    d5|d6|d7,                  /* a circle (scandinavian) */
    d1|d2|d3|d4|d6,          /* c cedille */
    d1|d2|d6,                  /* e circonflex */
    d1|d2|d4|d6,              /* e diaeresis */
    d2|d3|d4|d6,              /* e grave */
    d1|d2|d4|d5|d6,          /* i diaeresis */
    d1|d4|d6,                  /* i circonflex */
    d3|d4,                      /* i grave */
    d3|d4|d5|d7,              /* capital a diaeresis (umlaut) */
    d5|d6|d7,                  /* capital a circle */
    d1|d2|d3|d4|d5|d6|d7,  /* capital e aigue */
    d5|d6|d7,                  /* ae (scandinavian) */
    d5|d6|d7,                  /* capital ae (scandinavian) */
    d1|d4|d5|d6,              /* o circonflex */
    d2|d4|d6,                  /* o diaeresis */
    d3|d4|d6,                  /* o grave */
    d1|d5|d6,                  /* u dakje */
    d2|d3|d4|d5|d6,          /* u grave */
    d5|d6|d7,                  /* y diaeresis (Dutch ij) */
    d2|d4|d6|d7,              /* capital o diaeresis */
    d1|d2|d5|d6|d7,          /* capital u diaeresis */
    d5|d6|d7,                  /* dollarcent */
    d5|d6|d7,                  /* pound */
    d5|d6|d7,                  /* yen */
    d5|d6|d7,                  /* peseta */
    d5|d6|d7,                  /* gulden (Dutch) */
    d1|d2|d3|d5|d6,          /* a aigue */
    d3|d4,                      /* i aigue */
    d3|d4|d6,                  /* o aigue */
    d2|d3|d4|d5|d6,          /* u aigue */
    d5|d6|d7,                  /* n tilde (Spanish) */
    d5|d6|d7,                  /* capital n tilde */
    d5|d6|d7,                  /* a underline */
    d5|d6|d7,                  /* o underline */
    d5|d6|d7,                  /* reversed qestion mark (Spanish) */
    d5|d6|d7,                  /* angle left/up */
    d5|d6|d7,                  /* angle right/up */
    d5|d6|d7,                  /* half */
    d5|d6|d7,                  /* quarter */
    d5|d6|d7,                  /* reversed exclamation sign (Spa) */
    d5|d6|d7,                  /* double arrow left */
    d5|d6|d7,                  /* double arrow right */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* line up/down */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line left/right */
    d6|d7,                      /* line up/down/left/right */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line left/right */
    d6|d7,                      /* line up/down/left/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line left/right/up/down */
    d6|d7,                      /* line left/right/up/down */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line right/down */
    d6|d7,                      /* block */
    d6|d7,                      /* block lower half */
    d6|d7,                      /* block left half */
    d6|d7,                      /* block right half */
    d6|d7,                      /* block upper half */
    d6|d7,                      /* alpha */
    d6|d7,                      /* beta */
    d6|d7,                      /* gamma capital */
    d6|d7,                      /* pi */
    d6|d7,                      /* sigma capital */
    d6|d7,                      /* sigma */
    d6|d7,                      /* mu */
    d6|d7,                      /* tau */
    d6|d7,                      /* phi capital */
    d6|d7,                      /* teta */
    d6|d7,                      /* omega capital */
    d6|d7,                      /* delta */
    d6|d7,                      /* infinite */
    d6|d7,                      /* phi small */
    d6|d7,                      /* element of (capital epsilon) */
    d6|d7,                      /* intersection (reversed u) */
    d6|d7,                      /* identical */
    d6|d7,                      /* plusminus */
    d6|d7,                      /* greater or equal */
    d6|d7,                      /* less or equal */
    d6|d7,                      /* integral sign upper part */
    d6|d7,                      /* integral sign lower part */
    d6|d7,                      /* divide by (dash, dot above and) */
    d6|d7,                      /* apprximately equal */
    d6|d7,                      /* grade mark */
    d6|d7,                      /* filled grade mark */
    d6|d7,                      /* dot (bold) */
    d6|d7,                      /* root */
    d6|d7,                      /* nth power */
    d6|d7,                      /* power 2 */
    d6|d7,                      /* vertical small block */
    d0,                          /* space ('ff') */
};

/* Lower Case Primary Translation Table */
/* USA Braille */

const unsigned char Braille::LCPT[]= {
    d0,                          /* space (null) */
    d1|d7|d8,                  /* face */
    d1|d2|d7|d8,              /* face inverted */
    d1|d4|d7|d8,              /* hearts */
    d1|d4|d5|d7|d8,          /* diamonds */
    d1|d5|d7|d8,              /* clubs */
    d1|d2|d4|d7|d8,          /* spades */
    d1|d2|d4|d5|d7|d8,      /* dot */
    d1|d2|d5|d7|d8,          /* inverted dot */
    d2|d4|d7|d8,              /* circle */
    d2|d4|d5|d7|d8,          /* inverted circle */
    d1|d3|d7|d8,              /* man symbol */
    d1|d2|d3|d7|d8,          /* woman symbol */
    d1|d3|d4|d7|d8,          /* note */
    d1|d3|d4|d5|d7|d8,      /* note double */
    d1|d3|d5|d7|d8,          /* sun */
    d1|d2|d3|d4|d7|d8,      /* triangle right */
    d1|d2|d3|d4|d5|d7|d8,  /* triangle left */
    d1|d2|d3|d5|d7|d8,      /* arrow up and down */
    d2|d3|d4|d7|d8,          /* exclamation sign double */
    d2|d3|d4|d5|d7|d8,      /* paragraph English */
    d1|d3|d6|d7|d8,          /* paragraph German */
    d1|d2|d3|d6|d7|d8,      /* block horizontal */
    d2|d4|d5|d6|d7|d8,      /* arrow up, down, underlined */
    d1|d3|d4|d6|d7|d8,      /* arrow up */
    d1|d3|d4|d5|d6|d7|d8,  /* arrow down */
    d1|d3|d5|d6|d7|d8,      /* arrow right */
    d2|d4|d6|d7|d8,          /* arrow left */
    d1|d2|d5|d6|d7|d8,      /* rectangle half */
    d1|d2|d4|d5|d6|d7|d8,  /* arrow left and right */
    d4|d5|d7|d8,              /* triangle up */
    d4|d5|d6|d7|d8,          /* triangle down */
    d0,                          /* space */
    d2|d3|d4|d6,              /* exclamation mark */
    d5,                          /* double quotes */
    d3|d4|d5|d6,              /* (number sign) */
    d1|d2|d4|d6,              /* dollar */
    d1|d4|d6,                  /* percent */
    d1|d2|d3|d4|d6,          /* (ampersand) */
    d3,                          /* apostroph */
    d1|d2|d3|d5|d6,          /* parenthese open */
    d2|d3|d4|d5|d6,         /* parenthese close */
    d1|d6,                      /* asterisk */
    d3|d4|d6,                  /* plus */
    d6,                          /* comma */
    d3|d6,                      /* minus */
    d4|d6,                      /* point */
    d3|d4,                      /* slash */
    d3|d5|d6,                  /* zero */
    d2,                          /* one */
    d2|d3,                      /* two */
    d2|d5,                      /* three */
    d2|d5|d6,                  /* four */
    d2|d6,                      /* five */
    d2|d3|d5,                  /* six */
    d2|d3|d5|d6,              /* seven */
    d2|d3|d6,                  /* eight */
    d3|d5,                      /* nine */
    d1|d5|d6,                  /* colon */
    d5|d6,                      /* semi-colon */
    d1|d2|d6,                  /* less */
    d1|d2|d3|d4|d5|d6,      /* equal */
    d3|d4|d5,                  /* greater */
    d1|d4|d5|d6,              /* question mark */
    d4|d7,                      /* at sign */
    d1,                      /* capital a */
    d1|d2,                  /* capital b */
    d1|d4,                  /* capital c */
    d1|d4|d5,              /* capital d */
    d1|d5,                  /* capital e */
    d1|d2|d4,              /* capital f */
    d1|d2|d4|d5,          /* capital g */
    d1|d2|d5,              /* capital h */
    d2|d4,                  /* capital i */
    d2|d4|d5,              /* capital j */
    d1|d3,                  /* capital k */
    d1|d2|d3,              /* capital l */
    d1|d3|d4,              /* capital m */
    d1|d3|d4|d5,          /* capital n */
    d1|d3|d5,              /* capital o */
    d1|d2|d3|d4,          /* capital p */
    d1|d2|d3|d4|d5,      /* capital q */
    d1|d2|d3|d5,          /* capital r */
    d2|d3|d4,              /* capital s */
    d2|d3|d4|d5,          /* capital t */
    d1|d3|d6,              /* capital u */
    d1|d2|d3|d6,          /* capital v */
    d2|d4|d5|d6,          /* capital w */
    d1|d3|d4|d6,          /* capital x */
    d1|d3|d4|d5|d6,      /* capital y */
    d1|d3|d5|d6,          /* capital z */
    d2|d4|d6|d7,              /* rectangular bracket open */
    d1|d2|d5|d6|d7,          /* backslash */
    d1|d2|d4|d5|d6|d7,      /* rectangular bracket close */
    d4|d5|d7,                  /* accent circonflex */
    d4|d5|d6|d7,              /* underline */
    d4,                          /* accent grave */
    d1,                          /* a */
    d1|d2,                      /* b */
    d1|d4,                      /* c */
    d1|d4|d5,                  /* d */
    d1|d5,                      /* e */
    d1|d2|d4,                  /* f */
    d1|d2|d4|d5,              /* g */
    d1|d2|d5,                  /* h */
    d2|d4,                      /* i */
    d2|d4|d5,                  /* j */
    d1|d3,                      /* k */
    d1|d2|d3,                  /* l */
    d1|d3|d4,                  /* m */
    d1|d3|d4|d5,              /* n */
    d1|d3|d5,                  /* o */
    d1|d2|d3|d4,              /* p */
    d1|d2|d3|d4|d5,          /* q */
    d1|d2|d3|d5,              /* r */
    d2|d3|d4,                  /* s */
    d2|d3|d4|d5,              /* t */
    d1|d3|d6,                  /* u */
    d1|d2|d3|d6,              /* v */
    d2|d4|d5|d6,              /* w */
    d1|d3|d4|d6,              /* x */
    d1|d3|d4|d5|d6,          /* y */
    d1|d3|d5|d6,              /* z */
    d2|d4|d6,                  /* brace open */
    d1|d2|d5|d6,              /* dashed vertical line */
    d1|d2|d4|d5|d6,          /* brace close */
    d4|d5,                      /* tilde */
    d4|d5|d6,                  /* open triangle */
    d1|d2|d3|d4|d6|d7,      /* capital c cedille */
    d1|d2|d5|d6,              /* u diaeresis */
    d1|d2|d3|d4|d5|d6,      /* e aigue */
    d1|d6,                      /* a circonflex */
    d3|d4|d5,                  /* a diaeresis */
    d1|d2|d3|d5|d6,          /* a grave */
    d5|d6|d7,                  /* a circle (scandinavian) */
    d1|d2|d3|d4|d6,          /* c cedille */
    d1|d2|d6,                  /* e circonflex */
    d1|d2|d4|d6,              /* e diaeresis */
    d2|d3|d4|d6,              /* e grave */
    d1|d2|d4|d5|d6,          /* i diaeresis */
    d1|d4|d6,                  /* i circonflex */
    d3|d4,                      /* i grave */
    d3|d4|d5|d7,              /* capital a diaeresis (umlaut) */
    d5|d6|d7,                  /* capital a circle */
    d1|d2|d3|d4|d5|d6|d7,  /* capital e aigue */
    d5|d6|d7,                  /* ae (scandinavian) */
    d5|d6|d7,                  /* capital ae (scandinavian) */
    d1|d4|d5|d6,              /* o circonflex */
    d2|d4|d6,                  /* o diaeresis */
    d3|d4|d6,                  /* o grave */
    d1|d5|d6,                  /* u dakje */
    d2|d3|d4|d5|d6,          /* u grave */
    d5|d6|d7,                  /* y diaeresis (Dutch ij) */
    d2|d4|d6|d7,              /* capital o diaeresis */
    d1|d2|d5|d6|d7,          /* capital u diaeresis */
    d5|d6|d7,                  /* dollarcent */
    d5|d6|d7,                  /* pound */
    d5|d6|d7,                  /* yen */
    d5|d6|d7,                  /* peseta */
    d5|d6|d7,                  /* gulden (Dutch) */
    d1|d2|d3|d5|d6,          /* a aigue */
    d3|d4,                      /* i aigue */
    d3|d4|d6,                  /* o aigue */
    d2|d3|d4|d5|d6,          /* u aigue */
    d5|d6|d7,                  /* n tilde (Spanish) */
    d5|d6|d7,                  /* capital n tilde */
    d5|d6|d7,                  /* a underline */
    d5|d6|d7,                  /* o underline */
    d5|d6|d7,                  /* reversed qestion mark (Spanish) */
    d5|d6|d7,                  /* angle left/up */
    d5|d6|d7,                  /* angle right/up */
    d5|d6|d7,                  /* half */
    d5|d6|d7,                  /* quarter */
    d5|d6|d7,                  /* reversed exclamation sign (Spa) */
    d5|d6|d7,                  /* double arrow left */
    d5|d6|d7,                  /* double arrow right */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* line up/down */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line left/right */
    d6|d7,                      /* line up/down/left/right */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line left/right */
    d6|d7,                      /* line up/down/left/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line left/right/up/down */
    d6|d7,                      /* line left/right/up/down */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line right/down */
    d6|d7,                      /* block */
    d6|d7,                      /* block lower half */
    d6|d7,                      /* block left half */
    d6|d7,                      /* block right half */
    d6|d7,                      /* block upper half */
    d6|d7,                      /* alpha */
    d6|d7,                      /* beta */
    d6|d7,                      /* gamma capital */
    d6|d7,                      /* pi */
    d6|d7,                      /* sigma capital */
    d6|d7,                      /* sigma */
    d6|d7,                      /* mu */
    d6|d7,                      /* tau */
    d6|d7,                      /* phi capital */
    d6|d7,                      /* teta */
    d6|d7,                      /* omega capital */
    d6|d7,                      /* delta */
    d6|d7,                      /* infinite */
    d6|d7,                      /* phi small */
    d6|d7,                      /* element of (capital epsilon) */
    d6|d7,                      /* intersection (reversed u) */
    d6|d7,                      /* identical */
    d6|d7,                      /* plusminus */
    d6|d7,                      /* greater or equal */
    d6|d7,                      /* less or equal */
    d6|d7,                      /* integral sign upper part */
    d6|d7,                      /* integral sign lower part */
    d6|d7,                      /* divide by (dash, dot above and) */
    d6|d7,                      /* apprximately equal */
    d6|d7,                      /* grade mark */
    d6|d7,                      /* filled grade mark */
    d6|d7,                      /* dot (bold) */
    d6|d7,                      /* root */
    d6|d7,                      /* nth power */
    d6|d7,                      /* power 2 */
    d6|d7,                      /* vertical small block */
    d0,                          /* space ('ff') */
};

/* Upper Case Secondary Translation  Table */
/* German Braille */

const unsigned char Braille::UCST[] = {
    d0,                          /* space (null) */
    d1|d7|d8,                  /* face */
    d1|d2|d7|d8,              /* face inverted */
    d1|d4|d7|d8,              /* hearts */
    d1|d4|d5|d7|d8,          /* diamonds */
    d1|d5|d7|d8,              /* clubs */
    d1|d2|d4|d7|d8,          /* spades */
    d1|d2|d4|d5|d7|d8,      /* dot */
    d1|d2|d5|d7|d8,          /* inverted dot */
    d2|d4|d7|d8,              /* circle */
    d2|d4|d5|d7|d8,          /* inverted circle */
    d1|d3|d7|d8,              /* man symbol */
    d1|d2|d3|d7|d8,          /* woman symbol */
    d1|d3|d4|d7|d8,          /* note */
    d1|d3|d4|d5|d7|d8,      /* note double */
    d1|d3|d5|d7|d8,          /* sun */
    d1|d2|d3|d4|d7|d8,      /* triangle right */
    d1|d2|d3|d4|d5|d7|d8,  /* triangle left */
    d1|d2|d3|d5|d7|d8,      /* arrow up and down */
    d2|d3|d4|d7|d8,          /* exclamation sign double */
    d2|d3|d4|d5|d7|d8,      /* paragraph English */
    d1|d3|d6|d7|d8,          /* paragraph German */
    d1|d2|d3|d6|d7|d8,      /* block horizontal */
    d2|d4|d5|d6|d7|d8,      /* arrow up, down, underlined */
    d1|d3|d4|d6|d7|d8,      /* arrow up */
    d1|d3|d4|d5|d6|d7|d8,  /* arrow down */
    d1|d3|d5|d6|d7|d8,      /* arrow right */
    d1|d2|d3|d5|d6|d7|d8,  /* arrow left */
    d3|d4|d7|d8,              /* rectangle half */
    d2|d3|d4|d5|d6|d7|d8,  /* arrow left and right */
    d2|d3|d4|d6|d7|d8,      /* triangle up */
    d1|d2|d4|d5|d6|d7|d8,  /* triangle down */
    d0,                          /* space */
    d5,                          /* exclamation mark */
    d4,                          /* double quotes */
    d3|d4|d5|d6,              /* (number sign) */
    d4|d6,                      /* dollar */
    d1|d2|d3|d4|d5|d6,      /* percent */
    d1|d2|d3|d4|d6,          /* (ampersand) */
    d6,                          /* apostroph */
    d2|d3|d6,                  /* parenthese open */
    d3|d5|d6,                  /* parenthese close */
    d3|d5,                      /* asterisk */
    d2|d3|d5,                  /* plus */
    d2,                          /* comma */
    d3|d6,                      /* minus */
    d3,                          /* point */
    d2|d5|d6,                  /* slash */
    d3|d4|d6,                  /* zero */
    d1|d6,                      /* one */
    d1|d2|d6,                  /* two */
    d1|d4|d6,                  /* three */
    d1|d4|d5|d6,              /* four */
    d1|d5|d6,                  /* five */
    d1|d2|d4|d6,              /* six */
    d1|d2|d4|d5|d6,          /* seven */
    d1|d2|d5|d6,              /* eight */
    d2|d4|d6,                  /* nine */
    d2|d5,                      /* colon */
    d2|d3,                      /* semi-colon */
    d5|d6,                      /* less */
    d2|d3|d5|d6,              /* equal */
    d4|d5,                      /* greater */
    d2|d6,                      /* question mark */
    d3|d4|d5|d7,              /* at sign */
    d1|d7,                      /* capital a */
    d1|d2|d7,                  /* capital b */
    d1|d4|d7,                  /* capital c */
    d1|d4|d5|d7,              /* capital d */
    d1|d5|d7,                  /* capital e */
    d1|d2|d4|d7,              /* capital f */
    d1|d2|d4|d5|d7,          /* capital g */
    d1|d2|d5|d7,              /* capital h */
    d2|d4|d7,                  /* capital i */
    d2|d4|d5|d7,              /* capital j */
    d1|d3|d7,                  /* capital k */
    d1|d2|d3|d7,              /* capital l */
    d1|d3|d4|d7,              /* capital m */
    d1|d3|d4|d5|d7,          /* capital n */
    d1|d3|d5|d7,              /* capital o */
    d1|d2|d3|d4|d7,          /* capital p */
    d1|d2|d3|d4|d5|d7,      /* capital q */
    d1|d2|d3|d5|d7,          /* capital r */
    d2|d3|d4|d7,              /* capital s */
    d2|d3|d4|d5|d7,          /* capital t */
    d1|d3|d6|d7,              /* capital u */
    d1|d2|d3|d6|d7,          /* capital v */
    d2|d4|d5|d6|d7,          /* capital w */
    d1|d3|d4|d6|d7,          /* capital x */
    d1|d3|d4|d5|d6|d7,      /* capital y */
    d1|d3|d5|d6|d7,          /* capital z */
    d1|d2|d3|d5|d6|d7,      /* rectangular bracket open */
    d3|d4|d7,                  /* backslash */
    d2|d3|d4|d5|d6|d7,      /* rectangular bracket close */
    d2|d3|d4|d6|d7,          /* accent circonflex */
    d4|d5|d6|d7,              /* underline */
    d3|d4|d5,                  /* accent grave */
    d1,                          /* a */
    d1|d2,                      /* b */
    d1|d4,                      /* c */
    d1|d4|d5,                  /* d */
    d1|d5,                      /* e */
    d1|d2|d4,                  /* f */
    d1|d2|d4|d5,              /* g */
    d1|d2|d5,                  /* h */
    d2|d4,                      /* i */
    d2|d4|d5,                  /* j */
    d1|d3,                      /* k */
    d1|d2|d3,                  /* l */
    d1|d3|d4,                  /* m */
    d1|d3|d4|d5,              /* n */
    d1|d3|d5,                  /* o */
    d1|d2|d3|d4,              /* p */
    d1|d2|d3|d4|d5,          /* q */
    d1|d2|d3|d5,              /* r */
    d2|d3|d4,                  /* s */
    d2|d3|d4|d5,              /* t */
    d1|d3|d6,                  /* u */
    d1|d2|d3|d6,              /* v */
    d2|d4|d5|d6,              /* w */
    d1|d3|d4|d6,              /* x */
    d1|d3|d4|d5|d6,          /* y */
    d1|d3|d5|d6,              /* z */
    d1|d2|d3|d5|d6,          /* brace open */
    d3|d4,                      /* dashed vertical line */
    d2|d3|d4|d5|d6,          /* brace close */
    d2|d3|d4|d6,              /* tilde */
    d4|d5|d6,                  /* open triangle */
    d1|d2|d3|d4|d6|d7,      /* capital c cedille */
    d1|d2|d5|d6,              /* u diaeresis */
    d1|d2|d3|d4|d5|d6,      /* e aigue */
    d1|d6,                      /* a circonflex */
    d3|d4|d5,                  /* a diaeresis */
    d1|d2|d3|d5|d6,          /* a grave */
    d5|d6|d7,                  /* a circle (scandinavian) */
    d1|d2|d3|d4|d6,          /* c cedille */
    d1|d2|d6,                  /* e circonflex */
    d1|d2|d4|d6,              /* e diaeresis */
    d2|d3|d4|d6,              /* e grave */
    d1|d2|d4|d5|d6,          /* i diaeresis */
    d1|d4|d6,                  /* i circonflex */
    d3|d4,                      /* i grave */
    d3|d4|d5|d7,              /* capital a diaeresis (umlaut) */
    d5|d6|d7,                  /* capital a circle */
    d1|d2|d3|d4|d5|d6|d7,  /* capital e aigue */
    d5|d6|d7,                  /* ae (scandinavian) */
    d5|d6|d7,                  /* capital ae (scandinavian) */
    d1|d4|d5|d6,              /* o circonflex */
    d2|d4|d6,                  /* o diaeresis */
    d3|d4|d6,                  /* o grave */
    d1|d5|d6,                  /* u dakje */
    d2|d3|d4|d5|d6,          /* u grave */
    d5|d6|d7,                  /* y diaeresis (Dutch ij) */
    d2|d4|d6|d7,              /* capital o diaeresis */
    d1|d2|d5|d6|d7,          /* capital u diaeresis */
    d5|d6|d7,                  /* dollarcent */
    d5|d6|d7,                  /* pound */
    d5|d6|d7,                  /* yen */
    d5|d6|d7,                  /* peseta */
    d5|d6|d7,                  /* gulden (Dutch) */
    d1|d2|d3|d5|d6,          /* a aigue */
    d3|d4,                      /* i aigue */
    d3|d4|d6,                  /* o aigue */
    d2|d3|d4|d5|d6,          /* u aigue */
    d5|d6|d7,                  /* n tilde (Spanish) */
    d5|d6|d7,                  /* capital n tilde */
    d5|d6|d7,                  /* a underline */
    d5|d6|d7,                  /* o underline */
    d5|d6|d7,                  /* reversed qestion mark (Spanish) */
    d5|d6|d7,                  /* angle left/up */
    d5|d6|d7,                  /* angle right/up */
    d5|d6|d7,                  /* half */
    d5|d6|d7,                  /* quarter */
    d5|d6|d7,                  /* reversed exclamation sign (Spa) */
    d5|d6|d7,                  /* double arrow left */
    d5|d6|d7,                  /* double arrow right */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* line up/down */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line left/right */
    d6|d7,                      /* line up/down/left/right */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line left/right */
    d6|d7,                      /* line up/down/left/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line left/right/up/down */
    d6|d7,                      /* line left/right/up/down */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line right/down */
    d6|d7,                      /* block */
    d6|d7,                      /* block lower half */
    d6|d7,                      /* block left half */
    d6|d7,                      /* block right half */
    d6|d7,                      /* block upper half */
    d6|d7,                      /* alpha */
    d6|d7,                      /* beta */
    d6|d7,                      /* gamma capital */
    d6|d7,                      /* pi */
    d6|d7,                      /* sigma capital */
    d6|d7,                      /* sigma */
    d6|d7,                      /* mu */
    d6|d7,                      /* tau */
    d6|d7,                      /* phi capital */
    d6|d7,                      /* teta */
    d6|d7,                      /* omega capital */
    d6|d7,                      /* delta */
    d6|d7,                      /* infinite */
    d6|d7,                      /* phi small */
    d6|d7,                      /* element of (capital epsilon) */
    d6|d7,                      /* intersection (reversed u) */
    d6|d7,                      /* identical */
    d6|d7,                      /* plusminus */
    d6|d7,                      /* greater or equal */
    d6|d7,                      /* less or equal */
    d6|d7,                      /* integral sign upper part */
    d6|d7,                      /* integral sign lower part */
    d6|d7,                      /* divide by (dash, dot above and) */
    d6|d7,                      /* apprximately equal */
    d6|d7,                      /* grade mark */
    d6|d7,                      /* filled grade mark */
    d6|d7,                      /* dot (bold) */
    d6|d7,                      /* root */
    d6|d7,                      /* nth power */
    d6|d7,                      /* power 2 */
    d6|d7,                      /* vertical small block */
    d0,                          /* space ('ff') */
};

/* Lower Case Secondary Translation Table */
/* German Braille */

const unsigned char Braille::LCST[] = {
    d0,                          /* space (null) */
    d1|d7|d8,                  /* face */
    d1|d2|d7|d8,              /* face inverted */
    d1|d4|d7|d8,              /* hearts */
    d1|d4|d5|d7|d8,          /* diamonds */
    d1|d5|d7|d8,              /* clubs */
    d1|d2|d4|d7|d8,          /* spades */
    d1|d2|d4|d5|d7|d8,      /* dot */
    d1|d2|d5|d7|d8,          /* inverted dot */
    d2|d4|d7|d8,              /* circle */
    d2|d4|d5|d7|d8,          /* inverted circle */
    d1|d3|d7|d8,              /* man symbol */
    d1|d2|d3|d7|d8,          /* woman symbol */
    d1|d3|d4|d7|d8,          /* note */
    d1|d3|d4|d5|d7|d8,      /* note double */
    d1|d3|d5|d7|d8,          /* sun */
    d1|d2|d3|d4|d7|d8,      /* triangle right */
    d1|d2|d3|d4|d5|d7|d8,  /* triangle left */
    d1|d2|d3|d5|d7|d8,      /* arrow up and down */
    d2|d3|d4|d7|d8,          /* exclamation sign double */
    d2|d3|d4|d5|d7|d8,      /* paragraph English */
    d1|d3|d6|d7|d8,          /* paragraph German */
    d1|d2|d3|d6|d7|d8,      /* block horizontal */
    d2|d4|d5|d6|d7|d8,      /* arrow up, down, underlined */
    d1|d3|d4|d6|d7|d8,      /* arrow up */
    d1|d3|d4|d5|d6|d7|d8,  /* arrow down */
    d1|d3|d5|d6|d7|d8,      /* arrow right */
    d1|d2|d3|d5|d6|d7|d8,  /* arrow left */
    d3|d4|d7|d8,              /* rectangle half */
    d2|d3|d4|d5|d6|d7|d8,  /* arrow left and right */
    d2|d3|d4|d6|d7|d8,      /* triangle up */
    d1|d2|d4|d5|d6|d7|d8,  /* triangle down */
    d0,                          /* space */
    d5,                          /* exclamation mark */
    d4,                          /* double quotes */
    d3|d4|d5|d6,              /* (number sign) */
    d4|d6,                      /* dollar */
    d1|d2|d3|d4|d5|d6,      /* percent */
    d1|d2|d3|d4|d6,          /* (ampersand) */
    d6,                          /* apostroph */
    d2|d3|d6,                  /* parenthese open */
    d3|d5|d6,                  /* parenthese close */
    d3|d5,                      /* asterisk */
    d2|d3|d5,                  /* plus */
    d2,                          /* comma */
    d3|d6,                      /* minus */
    d3,                          /* point */
    d2|d5|d6,                  /* slash */
    d3|d4|d6,                  /* zero */
    d1|d6,                      /* one */
    d1|d2|d6,                  /* two */
    d1|d4|d6,                  /* three */
    d1|d4|d5|d6,              /* four */
    d1|d5|d6,                  /* five */
    d1|d2|d4|d6,              /* six */
    d1|d2|d4|d5|d6,          /* seven */
    d1|d2|d5|d6,              /* eight */
    d2|d4|d6,                  /* nine */
    d2|d5,                      /* colon */
    d2|d3,                      /* semi-colon */
    d5|d6,                      /* less */
    d2|d3|d5|d6,              /* equal */
    d4|d5,                      /* greater */
    d2|d6,                      /* question mark */
    d3|d4|d5|d7,              /* at sign */
    d1,                      /* capital a */
    d1|d2,                  /* capital b */
    d1|d4,                  /* capital c */
    d1|d4|d5,              /* capital d */
    d1|d5,                  /* capital e */
    d1|d2|d4,              /* capital f */
    d1|d2|d4|d5,          /* capital g */
    d1|d2|d5,              /* capital h */
    d2|d4,                  /* capital i */
    d2|d4|d5,              /* capital j */
    d1|d3,                  /* capital k */
    d1|d2|d3,              /* capital l */
    d1|d3|d4,              /* capital m */
    d1|d3|d4|d5,          /* capital n */
    d1|d3|d5,              /* capital o */
    d1|d2|d3|d4,          /* capital p */
    d1|d2|d3|d4|d5,      /* capital q */
    d1|d2|d3|d5,          /* capital r */
    d2|d3|d4,              /* capital s */
    d2|d3|d4|d5,          /* capital t */
    d1|d3|d6,              /* capital u */
    d1|d2|d3|d6,          /* capital v */
    d2|d4|d5|d6,          /* capital w */
    d1|d3|d4|d6,          /* capital x */
    d1|d3|d4|d5|d6,      /* capital y */
    d1|d3|d5|d6,          /* capital z */
    d1|d2|d3|d5|d6|d7,      /* rectangular bracket open */
    d3|d4|d7,                  /* backslash */
    d2|d3|d4|d5|d6|d7,      /* rectangular bracket close */
    d2|d3|d4|d6|d7,          /* accent circonflex */
    d4|d5|d6|d7,              /* underline */
    d3|d4|d5,                  /* accent grave */
    d1,                          /* a */
    d1|d2,                      /* b */
    d1|d4,                      /* c */
    d1|d4|d5,                  /* d */
    d1|d5,                      /* e */
    d1|d2|d4,                  /* f */
    d1|d2|d4|d5,              /* g */
    d1|d2|d5,                  /* h */
    d2|d4,                      /* i */
    d2|d4|d5,                  /* j */
    d1|d3,                      /* k */
    d1|d2|d3,                  /* l */
    d1|d3|d4,                  /* m */
    d1|d3|d4|d5,              /* n */
    d1|d3|d5,                  /* o */
    d1|d2|d3|d4,              /* p */
    d1|d2|d3|d4|d5,          /* q */
    d1|d2|d3|d5,              /* r */
    d2|d3|d4,                  /* s */
    d2|d3|d4|d5,              /* t */
    d1|d3|d6,                  /* u */
    d1|d2|d3|d6,              /* v */
    d2|d4|d5|d6,              /* w */
    d1|d3|d4|d6,              /* x */
    d1|d3|d4|d5|d6,          /* y */
    d1|d3|d5|d6,              /* z */
    d1|d2|d3|d5|d6,          /* brace open */
    d3|d4,                      /* dashed vertical line */
    d2|d3|d4|d5|d6,          /* brace close */
    d2|d3|d4|d6,              /* tilde */
    d4|d5|d6,                  /* open triangle */
    d1|d2|d3|d4|d6|d7,      /* capital c cedille */
    d1|d2|d5|d6,              /* u diaeresis */
    d1|d2|d3|d4|d5|d6,      /* e aigue */
    d1|d6,                      /* a circonflex */
    d3|d4|d5,                  /* a diaeresis */
    d1|d2|d3|d5|d6,          /* a grave */
    d5|d6|d7,                  /* a circle (scandinavian) */
    d1|d2|d3|d4|d6,          /* c cedille */
    d1|d2|d6,                  /* e circonflex */
    d1|d2|d4|d6,              /* e diaeresis */
    d2|d3|d4|d6,              /* e grave */
    d1|d2|d4|d5|d6,          /* i diaeresis */
    d1|d4|d6,                  /* i circonflex */
    d3|d4,                      /* i grave */
    d3|d4|d5|d7,              /* capital a diaeresis (umlaut) */
    d5|d6|d7,                  /* capital a circle */
    d1|d2|d3|d4|d5|d6|d7,  /* capital e aigue */
    d5|d6|d7,                  /* ae (scandinavian) */
    d5|d6|d7,                  /* capital ae (scandinavian) */
    d1|d4|d5|d6,              /* o circonflex */
    d2|d4|d6,                  /* o diaeresis */
    d3|d4|d6,                  /* o grave */
    d1|d5|d6,                  /* u dakje */
    d2|d3|d4|d5|d6,          /* u grave */
    d5|d6|d7,                  /* y diaeresis (Dutch ij) */
    d2|d4|d6|d7,              /* capital o diaeresis */
    d1|d2|d5|d6|d7,          /* capital u diaeresis */
    d5|d6|d7,                  /* dollarcent */
    d5|d6|d7,                  /* pound */
    d5|d6|d7,                  /* yen */
    d5|d6|d7,                  /* peseta */
    d5|d6|d7,                  /* gulden (Dutch) */
    d1|d2|d3|d5|d6,          /* a aigue */
    d3|d4,                      /* i aigue */
    d3|d4|d6,                  /* o aigue */
    d2|d3|d4|d5|d6,          /* u aigue */
    d5|d6|d7,                  /* n tilde (Spanish) */
    d5|d6|d7,                  /* capital n tilde */
    d5|d6|d7,                  /* a underline */
    d5|d6|d7,                  /* o underline */
    d5|d6|d7,                  /* reversed qestion mark (Spanish) */
    d5|d6|d7,                  /* angle left/up */
    d5|d6|d7,                  /* angle right/up */
    d5|d6|d7,                  /* half */
    d5|d6|d7,                  /* quarter */
    d5|d6|d7,                  /* reversed exclamation sign (Spa) */
    d5|d6|d7,                  /* double arrow left */
    d5|d6|d7,                  /* double arrow right */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* dotted area */
    d6|d7,                      /* line up/down */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line up/down/left */
    d6|d7,                      /* line up/down */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line left/down */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line left/right */
    d6|d7,                      /* line up/down/left/right */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/down/right */
    d6|d7,                      /* line left/right */
    d6|d7,                      /* line up/down/left/right */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/up */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line left/right/down */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line up/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line down/right */
    d6|d7,                      /* line left/right/up/down */
    d6|d7,                      /* line left/right/up/down */
    d6|d7,                      /* line left/up */
    d6|d7,                      /* line right/down */
    d6|d7,                      /* block */
    d6|d7,                      /* block lower half */
    d6|d7,                      /* block left half */
    d6|d7,                      /* block right half */
    d6|d7,                      /* block upper half */
    d6|d7,                      /* alpha */
    d6|d7,                      /* beta */
    d6|d7,                      /* gamma capital */
    d6|d7,                      /* pi */
    d6|d7,                      /* sigma capital */
    d6|d7,                      /* sigma */
    d6|d7,                      /* mu */
    d6|d7,                      /* tau */
    d6|d7,                      /* phi capital */
    d6|d7,                      /* teta */
    d6|d7,                      /* omega capital */
    d6|d7,                      /* delta */
    d6|d7,                      /* infinite */
    d6|d7,                      /* phi small */
    d6|d7,                      /* element of (capital epsilon) */
    d6|d7,                      /* intersection (reversed u) */
    d6|d7,                      /* identical */
    d6|d7,                      /* plusminus */
    d6|d7,                      /* greater or equal */
    d6|d7,                      /* less or equal */
    d6|d7,                      /* integral sign upper part */
    d6|d7,                      /* integral sign lower part */
    d6|d7,                      /* divide by (dash, dot above and) */
    d6|d7,                      /* apprximately equal */
    d6|d7,                      /* grade mark */
    d6|d7,                      /* filled grade mark */
    d6|d7,                      /* dot (bold) */
    d6|d7,                      /* root */
    d6|d7,                      /* nth power */
    d6|d7,                      /* power 2 */
    d6|d7,                      /* vertical small block */
    d0,                          /* space ('ff') */
};
