// -*-Mode:C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

//
//  $Author: keith $
//  $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Client.cc,v $
//  $Revision: 6.18 $
//  $Date: 1995/09/28 19:54:41 $
//  $Log: Client.cc,v $
// Revision 6.18  1995/09/28  19:54:41  keith
// Fixed bug with dictionaries of textreps -- basically got rid of them;
// we can use the AMM's window-to-object dict for the same thing and it's
// better maintained.
//
// Revision 6.17  1995/09/04  20:25:13  keith
// Changes to get stuff compiling and using the new TextRep API.
//
// Revision 6.16  1995/08/30  18:13:24  keith
// Fixed typo, added LocateObjectRequest on gadget creation.
//
// Revision 6.15  1995/08/30  18:03:40  keith
// Added code ot set new objects' x, y, width, height, border_width when
// we get these via XGetWindowAttributes().
//
// Revision 6.14  1995/08/18  22:37:25  keith
// Fixed client deletion bug
//
// Revision 6.13  1995/08/18  01:02:14  beth
// random changes... (keith)
//
// Revision 6.12  1995/08/11  19:54:41  keith
// Added initial hooks for FontCache (used by ProtoTextRep).
//
// Revision 6.11  1995/07/28  20:18:50  spine
// Added copyright notice.
//
// Revision 6.10  1995/07/27  00:06:56  beth
// support get long name by unique name
//
// Revision 6.9  1995/07/26  00:33:49  beth
// Fixed deletion ordering bug in Client::DeleteXtObject.
//
// Revision 6.8  1995/07/25  22:05:34  keith
// Cleaned up some usage of strings, dicts, and lists.  Ran with -pta to
// fix some previously unseen bugs in the parameterized classes.
//
// Revision 6.7  1995/07/25  20:14:29  keith
// Tweaked object deletion slightly, added some comments.
//
// Revision 6.6  1995/07/11  23:19:27  beth
// debugging top level windows
//
// Revision 6.5  1995/07/01  20:02:19  spine
// Removed all references to LEDA strings and LEDA.
//
// Revision 6.4  1995/06/10  15:26:08  iansmith
// template updates
//
// Revision 6.4  1995/06/09  13:13:10  iansmith
// Changed all the old LEDA dictonaries over to the new mercator
// dictionaries.
//
// Revision 6.3  1995/06/06  18:01:25  spine
// now includes the template header from the current dir instead of DS/
//
// Revision 6.2  1995/06/05  22:05:40  iansmith
// Changed the eventActions dictionary to be a mercator dictionary
// instead of LEDA.
//
// Modified the destructor to push intelligence about storage off to the
// mercator dictionary instead of removing everything before deleting it.
//
// Had to modify DispatchEvent to understand our new dictionaries.
//
// Revision 6.1  1995/05/29  15:12:15  iansmith
// - Removed all the LEDA list declare() macros.
// - Slight change to IsToplevelWindow() to make it use mList search().
// - Made some mods to AddToplevelWindow() to make it use mList and to get rid
// of some LEDA brain damage. Also added a closure mListPrinter which can
// print out a list of clients. This is used for debugging in
// AddToplevelWindow().
// - Similar mods in DeleteToplevelWindow().
// - Added the operator== and operator< functions so we could use Client
// objects with mList.
//
// Revision 6.0  1995/05/20  22:20:21  beth
// Mercator Version 6
//
// Revision 1.6  1995/04/01  19:40:38  keith
// End of work by Keith
//
// Revision 1.5  1995/03/06  17:52:05  keith
// *** empty log message ***
//
// Revision 1.4  1995/01/19  22:22:30  keith
// Automatic global checkin
//
// Revision 1.4  1995/01/19  22:22:30  keith
// Automatic global checkin
//
// Revision 1.3  1995/01/03  20:59:16  beth
// hacking by beth :-)
//
// Revision 1.2  1994/11/23  20:19:14  beth
// *** empty log message ***
//
// Revision 1.9  1994/08/30  18:54:17  kathryn
// pop-up return feature added
//
// Revision 1.8  1994/08/17  18:20:23  kathryn
// got rid of XlibHookInterest
//
// Revision 1.7  1994/08/17  15:28:46  kathryn
// update -- has to do with identfying new clients.
//
// Revision 1.6  1994/08/12  22:46:11  kathryn
// update
//
// Revision 1.5  1994/07/18  21:30:42  kathryn
// Motif hacks
//
// Revision 1.4  1994/07/15  21:10:43  kathryn
// *** empty log message ***
//
// Revision 1.3  1994/07/13  18:29:11  stacy
// restructuring of Client Creation
//
// Revision 1.2  1994/06/30  15:40:36  stacy
// initial 4.0 changes; modify to be compatible w/XlibHook w/o psuedoserver
//
// Revision 1.1  1994/05/24  19:49:25  stacy
// Initial revision
//

static const char rcsid[] = "$Header: /net/hm10/keith/proj/mercator/6.0/RCS/Client.cc,v 6.18 1995/09/28 19:54:41 keith Exp $";
 
#include "Client.h"
#include "RAP.h"
#include "Error.h"
#include "Util.h"
#include "AppModelMgr.h"
#include "Action.h"
#include "TclAction.h"
#include "XtObject.h"
#include "Template.h"
#include "Mercator.h"
#include "TextRep.h"
#include "FontCache.h"
#include "ScreenReader.h"
#include "ProtoTextRep.h"
#include "mList.h"

//counter used for generating unique XtObject ids.
static unsigned long unique_id_count = 0;

// ************************************************************************
// **                     Class Special Functions                    ******
// ************************************************************************
// **                         Private Functions                      ******
// ************************************************************************

// Hashing function for Objects
unsigned objectHash(const Object &o) {
    return (unsigned)o;
}
// hashing function for Window : just use value, its 32 bits
static unsigned windowHash(const Window &w) {
    return (unsigned) w;
}

Client::Client(RAP *_rap) :
    clientName(NULL),
    clientClass(NULL),
    rap(_rap),
    fontCache(NULL),
    id(unique_id_count),		// use instead of XID
    xtobjectObjectIds(new mDictValPtr<Object,XtObject>(objectHash)),
    currentLocation(NULL),
    topWindows(NULL),
    eventActions(new mDictPtrPtr<EventStruct,Action>(EventStructHash)),
    ready(0)
{ 
    char buffer[100];
    XTextProperty text; 
    Display *dpy = NULL;

    // Why not just use the XID?  Can we get it? -k
    sprintf(buffer, "Client%d", unique_id_count++);
    uniqueName = Strdup(buffer);

    dpy = mercator->GetXServer()->GetDisplay();  

#ifdef HAVE_XID
    // Set the client name
    XGetWMName(dpy, xid, &text);
    
    if (text.value != NULL) {
	clientName = Strdup((const char *) text.value);
    } else {
	Error(EL_WARNING, "Client::Client", "client has no name.", NULL);
	clientName = Strdup("NULL");
    }
#else
    clientName = Strdup("NULL");
#endif

    Error(EL_DEBUG, "Client::Client", "new client ", clientName,
	  " created (unique name = " , uniqueName, ")", NULL);
}

// ************************************************************************

Client::~Client()
{ 
    XtObject *obj;
    EventStruct *es;
    int done = 0;
    
    //
    // loop over the objects.  We do this in a weird way -- whenever we
    // delete an object in the list (via AMM::DeleteXtObject()) we've just
    // deleted the current item.  Hence we need to reset the iterator.
    // Fairly expensive?  Why is there no reset() method on the iterator?
    //
    while (!done) {
      mDictValPtrIterator<Object,XtObject> iter(*xtobjectObjectIds);

      if (iter.next()) {
	// Get the object that corresponds to this ObjectID
	XtObject *obj = xtobjectObjectIds->lookup(iter.current());
	
 	// Tell appmodelmgr to get rid of it; it actually does the delete.
	mercator->GetAppModelMgr()->DeleteXtObject(obj);
	
	// AMM calls back into us to delete the object from our dict, so
	// we're cool from here.
      } else {
	done = 1;
      }
    }

    delete xtobjectObjectIds;
    // just delete it, it won't touch the pointers
    delete eventActions;

    // Don't have to delete items from topWindows, it's a list
    delete topWindows;

    if (rap)
	rap->MarkForCleanup();
    
    delete [] clientName;
    delete [] clientClass;
    delete [] uniqueName;

    delete fontCache;
}

// ************************************************************************
//
// This only sets the clientName property.  Note that we don't retain
// the actual Window ID of the initial toplevel anywhere...
void
Client::SetWindowID(XID xid)
{
    XTextProperty text;

#if 0
    //
    // This should check for the toplevel window ID != None, not the
    // client ID.
    //
    if (id != None) {
	Error(EL_ERROR, "Client::SetWindowID",
	      "Already got window ID for this client.", NULL);
    }
#endif
    // Set the client name
    mercator->GetXServer()->GetWMName(xid, &text);
    
    if (text.value != NULL) {
	clientName = Strdup((const char *) text.value);
    } else {
	Error(EL_WARNING, "Client::Client", "client has no name.", NULL);
	clientName = Strdup("NULL");
    }
}

// ************************************************************************
// **                          Public Functions                      ******
// ************************************************************************

void
Client::ClientName(const char *c)
{
    if (clientName)
	delete [] clientName;

    clientName = Strdup(c);
}

// ************************************************************************

void 
Client::ClientClass(const char *c)
{
    if (clientClass)
	delete [] clientClass;

    clientClass = Strdup(c);
}

// ************************************************************************

//
// The first time the client is made ready, call an action.
//
void
Client::Ready(int r)
{
    int argc = 0;
    char *argv[10];
    
    if (!Ready() && r) {
	argv[argc++] = Strdup(GetUniqueName());
	Action::Invoke("ClientReady", argc, argv);
	while (argc--) delete [] argv[argc];
    }
    
    ready = r;
}

// ************************************************************************

int
Client::NumChildren() const
{
    return xtobjectObjectIds->size();
}

// ************************************************************************

XtObject * 
Client::CreateXtObject(Object objId, const char *name, const char *className, 
		       XtObject *parent, Window win)
{
    XtObject *newObj = NULL;
    
    // Check to make sure the XtObject doesn't already exist
    if (xtobjectObjectIds->lookup(objId) != nil) {
	char tmp[16];
	sprintf(tmp, "%d", objId);
	Error(EL_ERROR, "Client::CreateXtObject", "object ", tmp,
	      " already exists...skipping.", NULL);
	return NULL;
    }
    
    // create the XtObject and let AppModelMgr know about the new XtObject
    newObj = new XtObject(this, objId, name, className, parent, win);
    mercator->GetAppModelMgr()->CreateXtObject(newObj);

    //
    // This reeks of cheese.  When gadgets are created we issue a RAP
    // LocateObject request to determine x, y, width, height, border_width.
    // The reasons we do this here are:
    //	- We can't call GetWindowAttributes() on a gadget since it has
    //	  no window.
    //	- It seems that we usually can't trust resource values for gadget
    //	  size and position (not directly a gadget issue, but it seems to
    //	  be a problem consistently nonetheless...)
    //	- Blocking doesn't work so we cheat here by issueing the request
    //	  early.
    //
    // REMOVE THIS CODE WHEN BLOCKING WORKS!  It'll ensure that we
    // get positions even if the gadget is updated.
    //
    if (newObj->IsGadget()) {
	Error(EL_DEBUG, "Client::CreateXtObject", "object ",
	      newObj->GetName(), " is a gadget, issuing location request",
	      NULL);
	if (!GetRAP()->LocateObjectRequest(newObj->GetObjID()))
	    Error(EL_ERROR, "Client::CreateXtObject",
		  "LocateObjectRequest failed for ", newObj->GetName(),
		  NULL);
    }

    //
    // Does this object support text mode?  If so, create a text rep and
    // screenreader for it...
    //
    // We also zap an expose event at it to force a redraw of its text, so
    // we can capture it.
    //
    if (Template::GetBoolean(*newObj, "allowTextMode")) {
	//
	// I hate the thought of generating protocol here, but it seems like
	// the most (only?) reliable way of getting the width and height
	// this early on in the game.
	//
	// GADGET - will gadgets ever support text mode?  I guess they might.
	// we should probably check for gadgethood and send the expose to
	// the parent...
	//
	if (newObj->IsGadget()) {
	    Error(EL_ERROR, "Client::CreateXtObject",
		  "Sorry, gadgets with text mode are not supported yet.",
		  NULL);
	} else {
	    XWindowAttributes attr;

	    Error(EL_DEBUG, "Client::CreateXtObject",
		  "CREATING PROTOTEXTREP FOR ", newObj->GetName(), NULL);
	    newObj->SetTextRep(new ProtoTextRep(*newObj, *rap));
	    
	    ScreenReader *sreader =new ScreenReader(newObj->GetTextRep());
	    newObj->SetScreenReader(sreader);
	    Error(EL_DEBUG, "Client::CreateXtObject",
		  "Created TextRep for object ", name, " (", className, ")",
		  NULL);

	    if (mercator->GetXServer()->GetWindowAttributes(*newObj, &attr)) {
		char tmpw[16], tmph[16];
		sprintf(tmpw, "%d", attr.width);
		sprintf(tmph, "%d", attr.height);
		
		newObj->Width(attr.width);
		newObj->Height(attr.height); 

		// Added by keith...
		newObj->X(attr.x);
		newObj->Y(attr.y);
		newObj->BorderWidth(attr.border_width);
	    
		// OTHER STUFF???

		// GADGET - how do we determine width and height on gadgets?
		// We *have* to use resources...

		// We only send the expose if we're mapped.
		if (attr.map_state == IsViewable)
		    mercator->GetXServer()->SendExposeEvent(*newObj);
	    } else {
		Error(EL_ERROR, "Client::CreateXtObject",
		      "GetWindowAttributes failed for object ",
		      newObj->GetName(), NULL);
	    }
	}
    }

    // insert the XtObject into the client's dictionary of XtObjects
    xtobjectObjectIds->insert(objId, newObj);

    // let the interface do whatever it wants to when a XtObject is created
    char     *argv[10];
    int	     argc = 0;

    argv[argc++] = Strdup(newObj->GetUniqueName());
    
    if (newObj->GetParent()) {
	argv[argc++] = Strdup(newObj->GetParent()->GetUniqueName());
    } else {
	argv[argc++] = Strdup("NULL");
    }
    
    argv[argc++] = Strdup(newObj->GetClass());
    
    Action::Invoke("XtObjectCreation",3,argv);

    while (argc--) delete [] argv[argc];

    return newObj;
}

// ************************************************************************
//
// Deletions cause problems since objects aren't reference counted.  The
// way I'm working this is to send ALL deletions of XtObject through
// AppModelMgr.  AMM is the thing that ultimately frees the object's
// memory.  It will call Client::DeleteXtObject to remove the object from
// the client's dict though.
//
// Again, NEVER CALL THIS METHOD TO DELETE AN OBJECT.  Always call
// the AppModelMgr.
//
void 
Client::DeleteXtObject(Object objId)
{
    if (XtObjectExists(objId)) {
	//
	// If the object being deleted is the current location, reset
	// current location to NULL so that we don't have a dangling
	// pointer to it.
	//
	// NOTE that we can't call SetCurrentLocation() to set it, since
	// it does some other weird undesirable things...
	//
	if (GetXtObject(objId) == GetCurrentLocation())
	    currentLocation = NULL;

	// Remove it from the toplevel map
	topWindows->del_item(GetXtObject(objId));
	// Remove it from the objectID-to-object map.
	xtobjectObjectIds->del(objId);
    } else {
	char tmp[16];
	sprintf(tmp, "%d", id);
	Error(EL_ERROR, "Client::DeleteXtObject", 
	      "internal inconsistency detected:  object ",
	      tmp, " does not exist.", NULL);
    }
}

// ************************************************************************
//
// The client will only create a FontCache for itself if it's asked
// for one.  We'll only need one if any of our XtObjects create
// ProtoTextReps.
//
FontCache&
Client::GetFontCache() const
{
    // Note fake of non-constness.
    if (!fontCache)
	((Client *) this)->fontCache = 
		new FontCache(mercator->GetXServer()->GetDisplay());

    return *fontCache;
}

// ************************************************************************

XtObject * 
Client::GetXtObject(Object objId)
{
    if (xtobjectObjectIds->lookup(objId) == nil)
	return((XtObject *) NULL);
    
    return(xtobjectObjectIds->lookup(objId));
}

// ************************************************************************

XtObject * 
Client::GetXtObject(Window win)
{
    return mercator->GetAppModelMgr()->GetXtObject(win);
}

// ************************************************************************

int 
Client::XtObjectExists(Object objId)
{
    return (xtobjectObjectIds->lookup(objId) != nil) ? True : False;
}

// ************************************************************************

void 
Client::SetCurrentLocation(XtObject *obj)
{ 
    XtObject *oldXtObject = currentLocation;

    if (obj == NULL) {
	Error(EL_ERROR, "Client::SetCurrentLocation",
	      "called with NULL XtObject parameter.", NULL);
	// This may be dangerous...
	currentLocation = NULL;
	return;
    }
    
    currentLocation = obj;

    // ANYTHING TO DO FOR GADGETS?  If Gadgets return a proper window
    // ID and X,Y coord, will everything just work??
    
    Error(EL_DEBUG, "Client::SetCurrentLocation",
	  "CURRENT LOCATION NOW ", obj->GetName(), NULL);

    char *argv[3];
    char  buffer1[100];
    char  buffer2[100];
    
    if ((oldXtObject) && (oldXtObject->GetName())) { 
	sprintf(buffer1,"\"%s\"",oldXtObject->GetName());
	argv[0] = Strdup(buffer1);
    } else {
	argv[0] = Strdup("NULL");
    }
    
    sprintf(buffer2,"\"%s\"",currentLocation->GetName());
    argv[1] = Strdup(buffer2);

    if (obj) 
	argv[2] = Strdup(obj->GetUniqueName());
    else 
	argv[2] = Strdup("NULL");
 
    Action::Invoke("XtObjectChange", 3, argv);
    
    delete [] argv[0];
    delete [] argv[1];
    delete [] argv[2];
}

// ************************************************************************

int
Client::IsToplevelWindow(const XtObject *obj)
{
    if (obj == NULL) {
	Error(EL_ERROR, "Client::IsToplevelWindow",
	      "called with NULL XtObject parameter.", NULL);
	return 0;
    }
    
    if (topWindows == NULL) {
	Error(EL_ERROR, "Client::IsToplevelWindow",
	      "No topwindows in client!?", NULL);
	return 0;
    }

    // Look through the list
    if (topWindows->search(obj) == 0)
	return FALSE;
    else
	return TRUE;
}
// ************************************************************************
// print out the list
class mListprinter : public mListClosure<XtObject> {
public:
    int action(XtObject *obj)  { 
	Error(EL_DEBUG, "Client::mListprinter",
	      "\t", obj->GetName(), NULL); 
	return 1; // keep processing
    };
};

// ************************************************************************

void 
Client::AddToplevelWindow(const XtObject *obj)
{ 
    mListprinter *p=new mListprinter();

    if (topWindows == NULL) {
	topWindows = new mList<XtObject>;
    }

    // insert it into the list
    topWindows->append(obj);

    Error(EL_DEBUG, "Client::AddToplevelWindow",
	  "list of TopLevel windows is now:", NULL);

    XtObject *tmp;
    // walk over the list printing out the values
    topWindows->walk(p,0);
    // delete the walker
    delete p;
}

// ************************************************************************

void 
Client::DeleteToplevelWindow(const XtObject *obj)
{
    XtObject * item;
    mListprinter *p=new mListprinter();

    if (topWindows == NULL) {
	Error(EL_ERROR, "Client::DeleteToplevelWindow",
	      "no topWindows for this client?!", NULL);
	return;
    }
    
    // see if the object is there
    if ((item = topWindows->search(obj)) == 0) {
	Error(EL_ERROR, "Client::DeleteToplevelWindow",
	      "procedure called for an object which was not a toplevel.",
	      NULL);
	return;
    } else {
	topWindows->del_item(obj);
    }

    //
    // Like AddToplevelWindow, we'll try to make this a little simpler.
    // Rather than change the current location here, we'll let the
    // destruction of the object trigger it in XtHook.
    //			-k
    Error(EL_DEBUG, "Client::DeleteToplevelWindow",
	  "list of TopLevel windows is now:", NULL);

    topWindows->walk(p,0);
    delete p;
}

// ************************************************************************

void
Client::PrintHierarchy() const
{
    Error(EL_ERROR, "Client::PrintHierarchy", "NOT YET IMPLEMENTED.", NULL);
}

// ************************************************************************

TextRep *
Client::FindTextRep(Window win) const
{
    XtObject *xtobj = mercator->GetAppModelMgr()->GetXtObject(win);

    if (xtobj == NULL)
	return NULL;

    return xtobj->GetTextRep();
}

// ************************************************************************

int
Client::DispatchEvent(EventPtr event)
{
    // if an EventBind has been created for the event passed in then
    // invoke the action and return TRUE.  Else return False.
 
    ActionPtr action;
 
#ifdef EVENT_DEBUG
    printf("\n***********************************************************\n");
    printf("Client::DispatchEvent: event = ");
    switch (event->eventType) {
    case KEYPRESS:
	printf("Key Press  key=%i, modifier=%X\n",
	       event->key, event->modifier);
	break;
    case KEYRELEASE:
	printf("Key Release  key=%i, modifier=%X\n",
	       event->key, event->modifier);
	break;
    default:
	printf("UNKNOWN\n ");
	break;
    }
#endif
 
    // look to see if the client is interested in the event
    action = eventActions->lookup(event);
    
    if (action == NULL) {
	// event binding not found - no action invoked - return False
	return False; 
    }
 
    // Invoke the action
    Error(EL_DEBUG, "Client::DispatchEvent", "invoking action...", NULL);
    action->Invoke(0,NULL);
 
    // event binding found and invoked - return True
    return True; 
}

// ************************************************************************

void
Client::CreateEventBind(EventPtr event, Action *function)
{
    eventActions->insert(event, function);
}

// ************************************************************************

int
Client::DeleteEventBind(EventPtr event)
{
    if (eventActions->lookup(event) == nil) {
	Error(EL_ERROR, "Client::DeleteEventBind",
	      "attempting to remove an event bind which was never inserted",
	      NULL);
	return False;
    } else {
	eventActions->del(event);
	return True;
    }
}
// ************************************************************************
// need these to allow us to put the client in a data structure
// we do these based on address in core comparison
int Client::operator==(const Client &x)
{
    return (this==&x);
}
int Client::operator<(const Client &x) 
{
    return (this < &x);
}
