// -*- c++ -*- 
// 
//	Configuration file parsing and storage. 
// 
// 
//	You can't declare a Config instance. 
// 
//		keith edwards 30jan95 
//              john selbie Aug 95
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
#ifndef _CONFIG_H_ 
#define _CONFIG_H_

#include "Util.h"
#include "mString.h"
#include "mListVal.h"
#include "XtObject.h"
#include "Template.h"


class FieldData;


class Config {
private:
    Config();

    static int		ValidateConfigFile(const char *filename);

protected:
    static mDictValPtr<mString,FieldData> *configDictionary;
    static int          validDict;
    
    static void         EmptyDict();
    
public: 
    static int          ClearDictionaryEntry(const char *id);
    static int		ParseConfigFile(const char *filename);
    static void		FreeConfig();

    static void 	InstallBooleanDecl(int, const char *, unsigned char);
    static void 	InstallIntegerDecl(int, const char *, long);
    static void 	InstallStringDecl(int, const char *, const char *);
    static void 	InstallStringListDecl(int, const char *,
					      const mListVal<mString>& sl);
    static void 	InstallIdentifierDecl(int, const char *, const char *);
    static void         InstallProcDecl(int, const char *, const char *);
#if 0
    static void         InstallImplicitAttributes();
#endif
    //
    // Sort of hacky, but used by FindByName (which is used by the Interp).
    // Return a FieldData * based on the name of the default.  This only works
    // for template defaults, and is HARD WIRED.  This code is also non-reentrant
    // (it returns a pointer to a static).
    //
    static FieldData	*GetDefaultByName(const char *s);
    //
    // "GET" functions
    //
    // For app templates...
    static unsigned char GetBoolean(char *id, const Client& obj);
    static long 	GetInteger(char *id, const Client& obj);
    static const char 	*GetString(char *id, const Client& obj);
    static const mListVal<mString>& GetStringList(char *id, const Client& obj);
    // For object templates...
    static unsigned char GetBoolean(char *id, const XtObject& obj);
    static long 	GetInteger(char *id, const XtObject& obj);
    static const char 	*GetString(char *id, const XtObject& obj);
    static const mListVal<mString>& GetStringList(char *id, const XtObject& o);
    // For non-template defaults...
    static unsigned char GetBoolean(char *id);
    static long 	GetInteger(char *id);
    static const char 	*GetString(char *id);
    static const mListVal<mString>& GetStringList(char *id);
}; 

#endif /* _CONFIG_H_ */ 


