//  -*-Mode: C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
//
// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Dectalk.cc,v $
// $Revision: 6.17 $
// $Date: 1995/12/03 01:47:04 $
// $Log: Dectalk.cc,v $
// Revision 6.17  1995/12/03  01:47:04  keith
// Fixed some memory errors that Purify was complaining about.
//
// Revision 6.16  1995/11/29  19:22:38  spine
// Got rid of a compiler warning for escaping a back quote.
//
// Revision 6.15  1995/11/15  01:07:32  spine
// Added a hardcoded filter for unpronounceable symbols when we receive only
// one character to be spoken.
//
// Use control-k to flush the dectalk buffer.
//
// Revision 6.14  1995/10/25  21:28:23  spine
// Added a debugging message to print out what is actually sent to Dectalk.
//
// Revision 6.13  1995/10/25  20:48:11  spine
// Changed Dectalk::Speak's "line ending" algorithm to strip off trailling
// spaces and add a period at the end of the line, if the last character of
// the line is not a terminating character already. (Previously the algorithm
// only looked for a period anywhere in the line, not necessarily at the
// end. This prevented words after the period to be read by Dectalk.)
//
// Revision 6.12  1995/10/25  19:57:13  spine
// changed DectalkX to Dectalk
//
// Revision 6.11  1995/10/25  19:26:52  spine
// Added a ShutDown method to prevent the object from trying to speak
// after the server has been killed. (It may cause mercator to hang until the
// client times out if the server has been killed.)
//
// Revision 6.10  1995/10/21  20:02:10  spine
// Changed the SetVoice api to take a string intead of a char.
//
// Revision 6.9  1995/10/11  20:27:20  keith
// Changed error messages a bit...
//
// Revision 6.8  1995/09/26  21:17:58  spine
// Added code to initialize the voice when the object is loaded.
//
// Revision 6.7  1995/09/21  21:39:55  keith
// Made a few cleanups, signal handling stuff...
//
// Revision 6.6  1995/09/20  21:23:36  spine
// Modified server connection debugging messages.
//
// Revision 6.5  1995/09/19  21:39:14  spine
// Added StartServer member.
//
// Revision 6.4  1995/08/17  20:38:57  spine
// Added an api to connect to the server.
//
// Revision 6.3  1995/08/09  20:23:16  keith
// Added capabilities stuff (a start anyway...)
//
// Revision 6.2  1995/07/28  20:30:34  spine
// Added copyright notice.
//
// Revision 6.1  1995/06/02  20:24:46  beth
// added copyright
//
// Revision 6.0  1995/05/20  22:20:21  beth
// Mercator Version 6
//
// Revision 1.4  1995/04/01  19:40:38  keith
// End of work by Keith
//
// Revision 1.3  1995/03/06  17:52:05  keith
// *** empty log message ***
//
// Revision 1.2  1995/01/19  22:22:30  keith
// Automatic global checkin
//
// Revision 1.2  1995/01/19  22:22:30  keith
// Automatic global checkin
//
// Revision 1.1  1994/11/16  20:46:23  spine
// Initial revision
//
// Revision 1.1  1994/05/24  19:49:25  stacy
// Initial revision
//
// Revision 1.1  1994/04/06  19:51:36  kathryn
// Initial revision
//
//
 
static const char rcsid[]="@(#) $Header: /net/hm10/keith/proj/mercator/6.0/RCS/Dectalk.cc,v 6.17 1995/12/03 01:47:04 keith Exp $";


#include "Dectalk.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/wait.h>
 
#include <netinet/in.h>
 
#include <errno.h>
#include <netdb.h>
#include <signal.h>
#include <sgtty.h>
#include <fcntl.h>

#include <ctype.h>
#include <sysent.h>
#include <unistd.h>
#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <stream.h>

#include "Error.h"
#include "Util.h"
#include "Mercator.h"
#include "Config.h"
#include "Template.h"

LOADABLE_CLASS_DEFN(Dectalk);


#define is_member(c, s)  strchr((s), (c))
#define SPECIAL_CHARS   "!@#$%^&*()/[]{}"
#define WORD_CONTINUE   "-"
#define SENTENCE_ENDER  ".?!"
#define TEXT_PUNC       ".,:;!?"
#define PUNC_NOT_DELIM  ":;"

#ifndef FALSE
#	define FALSE 0
#endif

#ifndef TRUE
#	define TRUE 1
#endif

extern Mercator *mercator;

Speech::Capabilities 	Dectalk::capabilities;
int			Dectalk::capabilitiesInitialized = 0;


Dectalk::Dectalk(const char *dectalkhost) :
    Speech("Dectalk"),
    fd(-1),
    file(NULL)
{
    Dectalk::StartServer();
    Dectalk::ConnectServer(dectalkhost);

    if (!Dectalk::capabilitiesInitialized++) {
	Dectalk::capabilities.languages.append("English");

	Dectalk::capabilities.voices.append("Perfect Paul");
	Dectalk::capabilities.voices.append("Beautiful Betty");
	Dectalk::capabilities.voices.append("Huge Harry");
	Dectalk::capabilities.voices.append("Frail Frank");
	Dectalk::capabilities.voices.append("Kit the Kid");
	Dectalk::capabilities.voices.append("Rough Rita");
	Dectalk::capabilities.voices.append("Uppity Ursula");
	Dectalk::capabilities.voices.append("Doctor Dennis");
	Dectalk::capabilities.voices.append("Whispering Wendy");
	Dectalk::capabilities.voices.append("Variable Val");

	Dectalk::capabilities.minRate = 120;
	Dectalk::capabilities.maxRate = 350;

	Dectalk::capabilities.gainSupported = 0;
    }
}

Dectalk::~Dectalk()
{
    if (file)
	fclose(file);
    
    if (fd != -1)
	close(fd);
}

int
Dectalk::ConnectServer(const char *dectalkhost)
{
    FieldData *maxTriesData = Config::GetDefaultByName("dectalkTimeout");
    int maxTries = 5;

    if (maxTriesData && (maxTriesData->Tag() == INTEGER))
	maxTries = maxTriesData->Integer();
    
    Error(EL_DEBUG, "Dectalk::ConnectServer",
	  "Connecting to Dectalk server!", NULL);

    if (dectalkhost == NULL)
	if ((dectalkhost = getenv( "DECTALKHOST" )) == NULL)
	    dectalkhost = "localhost";
    
    fd = OpenDectalk(dectalkhost, maxTries);

    if (fd == -1) {
	Error(EL_ERROR, "Dectalk::ConnectServer", "server connection",
	      " timed out", NULL);
	return -1;
    } else {
	Error(EL_DEBUG, "Dectalk::ConnectServer",
	      "connection to server successful", NULL);
	file = fdopen(fd, "w");

	// Initialize Dectalk
	SetSpeechRate(180);
	SetVoice("Perfect Paul");

	return 0;
    }
}

//
// return -1 if server was not started, 0 otherwise. (Will assume that
// if DECTALKHOST environment variable is only set if the user wants to
// use a server on another machine. So essentially setting DECTALKHOST
// to anything will prevent a server from being started.
//
int
Dectalk::StartServer()
{
    const char *host;
    
    // first check if an env. var. is set. I'm assuming that DECTALKHOST
    // is only set if the user wants to use a remote server
    if ((host = getenv("DECTALKHOST")) == NULL) {
	if (mercator->ExecList("dectalkServer")) {
	    Error(EL_DEBUG, "Dectalk::StartServer",
		  "dectalkServer not found.", NULL);
	    return -1;
	}
	return 0;
    } else
	Error(EL_DEBUG, "Dectalk::StartServer", "Server has been started on ",
	      host, NULL);
    
    return 0; // assuming the server has been started on another host
}

const Speech::Capabilities&
Dectalk::GetCapabilities() const
{
    return Dectalk::capabilities;
}

int
Dectalk::Notify(const char *type)
{
    if (fd != -1) {
	// Do whatever this is supposed to do!?
    } else {
	Speech::Notify(type);
    }
    
    return 0;
}

//
// NOTE: Right now Speak expects strings whose length are no more than 2K.
// 
int
Dectalk::Speak(const char *string)
{
    char buffer[2050];
    char *s;
    short toDel = 1;
#ifdef EXPERIMENTAL
    //
    // This code is not yet integrated.  It is intended to provide a smarter way
    // of handing off strings to the Dectalk.  We strip off trailing whitespace
    // and tack on a period if needed.
    //
    int length = strlen(string);

    strcpy(buffer, string);
    
    for (register int i = (buffer + length - 1) ; i >= buffer ; i--) {
	// Get rid of trailing white space
	if (buffer[i] == ' ' ||
	    buffer[i] == '\t' ||
	    buffer[i] == '\n')
	    buffer[i] = '\0';
	else {
	    // As soon as we see non-white space we can bail.  If the last
	    // character isn't a period, tack one on at the end.
	    if (buffer[i] != '.')
		buffer[i+1] = '.';

	    break;
	}
    }
#endif /* EXPERIMENTAL */
    Error(EL_DEBUG, "Dectalk::Speak", "|", string, "|", NULL);

    // Special case for character mode. This is a hack for now.
    if(strlen(string) == 1) {
	switch (string[0]) {
	case ' ':
	    s = Strdup("space");
	    break;
	case '\t':
	    s = Strdup("tab");
	    break;
	case '\n':
	    s = Strdup("new line");
	    break;
	case '\"':
	    s = Strdup("quote");
	    break;
	case '."':
	    s = Strdup("period");
	    break;
	case ',"':
	    s = Strdup("comma");
	    break;
	case ';':
	    s = Strdup("semi colon");
	    break;
	case ':':
	    s = Strdup("colon");
	    break;
	case '\'':
	case '`':
	    s = Strdup("apostrophe");
	    break;
	case '?':
	    s = Strdup("question mark");
	    break;
	case '!':
	    s = Strdup("exclamation mark");
	    break;
	case '@':
	    s = Strdup("at");
	    break;
	case '#':
	    s = Strdup("pound");
	    break;
	case '^':
	    s = Strdup("carret");
	    break;
	case '*':
	    s = Strdup("asterisk");
	    break;
	case '(':
	    s = Strdup("left paren");
	    break;
	case ')':
	    s = Strdup("right paren");
	    break;
	case '[':
	case '{':
	    s = Strdup("left bracket");
	    break;
	case ']':
	case '}':
	    s = Strdup("right bracket");
	    break;
	case '>':
	    s = Strdup("greater than");
	    break;
	case '<':
	    s = Strdup("less than");
	    break;
	case '_':
	    s = Strdup("underscore");
	    break;
	case '\\':
	    s = Strdup("back slash");
	    break;
	case '|':
	    s = Strdup("pipe");
	    break;
	case '~':
	    s = Strdup("tilda");
	    break;
	default:
	    s = (char *) string;
	    toDel = 0;  // do not delete s
	}
	sprintf(buffer, "%s\013\n", s);
	
	if(toDel) delete [] s;
    }
    else
	sprintf(buffer, "%s\013\n", string);
	    
    
#ifdef junk    
    if (is_member(string[strlen(string)-1],TEXT_PUNC))  
	sprintf(buffer, "%s\n", string);
    else
	sprintf(buffer, "%s.\n", string);

    // This should fix the problem of not finishing reading a line that
    // contains a period which is not at the end of the line.
    sprintf(buffer, "%s", string);

    // strip off trailing spaces
    int i = strlen(buffer) - 1;

    while((buffer[i] == ' ') && (i>0))
	i--;

    buffer[i+1] = '\0';

    // Try to be smarter about terminating a line of text
    if(i>=0) {
	switch(buffer[i]) {
	case '.':
	case ',':
	case ':':
	case ';':
	case '?':
	case '!':
	case ')':
	    strcat(buffer, "\n");
	    break;
	default:
	    strcat(buffer, ".\n");
	    break;
	}
    }
#endif /* junk */
    
    if (fd != -1) {
	write(fd, buffer, strlen(buffer));
	if (fflush(file) != 0) {
	    Error(EL_ERROR, "Dectalk::Speak", "Error flushing speech: ",
		  Perror(), NULL);
	}
	buffer[strlen(buffer)-1] = '\0';
	Error(EL_DEBUG, "Dectalk::Speak", "Sending: | ", buffer, " |", NULL);
    } else  
	Speech::Speak (string);

    return 0;
}
 
int
Dectalk::SpeakUnix(const char *string)
{
    Error(EL_DEBUG, "Dectalk::SpeakUnix", "|", string, "|", NULL);

    if (fd != -1) { 
	int size = strlen(string);
	int i = 0, o = 0;
	char buffer[1024];
   
	while ((string[i] == ' ') && (i < size)) i++;
   
	for (o = 0; i < size; i++) { 
	    if (iscntrl(string[i]))
		continue;
	    if (isalnum(string[i])) { 
		buffer[o++] = string[i];
		continue;
            }
	    if (string[i] == ' ') { 
		buffer[o++] = string[i];
		continue;
            }
	    if (is_member(string[i], SPECIAL_CHARS)) { 
		buffer[o++] = ' ';
		buffer[o++] = string[i];
		buffer[o++] = ' ';
            } 
        }
 
	if (o == 0) 
	    return 0;
	o--;
	
	while (o && (buffer[o] == ' '))
	    o--;
	
	o++;
	
	buffer[o++] = '.'; /* awful hack to get it to talk immediately */
	buffer[o++] = '\n';
	buffer[o] = '\0';
	
	Error(EL_DEBUG, "Dectalk::SpeakUnix", "|", buffer, "|", NULL);

	write(fd, buffer, o);
	if (fflush(file) != 0)
	    Error(EL_ERROR, "Dectalk::SpeakUnix",
		  "Error flushing SpeakUnix: ", Perror(), NULL);
    } else {
	Speech::SpeakUnix(string);
    }
}

int
Dectalk::SetSpeechRate(int rate)
{ 
    char buffer[256];

    if (fd != -1) { 
	sprintf(buffer, "\033P0;0z:ra%i\033\\", rate);
	write(fd, buffer, strlen(buffer));
	if (fflush(file) != 0)
	    Error(EL_ERROR, "Dectalk::SetSpeechRate",
		  "Error flushing output: ", Perror(), NULL);
    } else {
	Speech::SetSpeechRate(rate);
    }
   
    SyncSpeech();
    return 0;
}

int
Dectalk::StopSpeaking()
{
    Error(EL_DEBUG, "Dectalk::StopSpeaking", NULL);
    
    if (fd != -1) { 
	char *buffer = "\033P0;10z\033\\";
	write(fd, buffer, strlen(buffer));
	if (fflush(file) != 0)
	    Error(EL_ERROR, "Dectalk::StopSpeaking",
		  "Error flushing output: ", Perror(), NULL);
    } else {
	Speech::StopSpeaking();
    }

    return 0;
}

int
Dectalk::SetLanguage(const char *lang)
{
    mString dummy;
    
    if (!Dectalk::capabilities.languages.search(lang, dummy)) {
	Error(EL_ERROR, "Dectalk::SetLanguage", "Unsupported language: ",
	      lang, NULL);
    }

    // The Dectalk actually only supports english, so don't do anything.
    return 0;
}

int
Dectalk::SetGain(int)
{
    Error(EL_ERROR, "Dectalk::SetGain", "Gain control not supported on Dectalk",
	  NULL);
    return 0;
}

//
// "Perfect Paul": standard male voice
// "Beautiful Betty": standard female voice
// "Huge Harry": deep male voice
// "Frail Frank": older male voice
// "Kit the Kid": child's voice
// "Rough Rita": deep female voice
// "Uppity Ursula": light female voice
// "Doctor Dennis": whispery male voice
// "Whispery Wendy": whispery female voice
//
int
Dectalk::SetVoice (const char *voice_name)
{ 
    char buffer[256], cmd;

    if (!strcmp(voice_name, "Perfect Paul"))
	cmd = 'p';
    else if (!strcmp(voice_name, "Beautiful Betty"))
	cmd = 'b';
    else if (!strcmp(voice_name, "Huge Harry"))
	cmd = 'h';
    else if (!strcmp(voice_name, "Frail Frank"))
	cmd = 'f';
    else if (!strcmp(voice_name, "Kit the Kid"))
	cmd = 'k';
    else if (!strcmp(voice_name, "Rough Rita"))
	cmd = 'r';
    else if (!strcmp(voice_name, "Uppity Ursula"))
	cmd = 'u';
    else if (!strcmp(voice_name, "Doctor Dennis"))
	cmd = 'd';
    else if (!strcmp(voice_name, "Whispering Wendy"))
	cmd = 'w';
    else {
	Error(EL_ERROR, "Dectalk::SetVoice", "Invalid voice name \"",
	      voice_name, "\"", NULL);
	return -1;
    }
    
    if (fd != -1) { 
	sprintf(buffer, "\033P0;0z:n%c\033\\", cmd);
	write(fd, buffer, strlen(buffer));
	if (fflush(file) != 0) 
	    Error(EL_ERROR, "Dectalk::SetVoice",
		  "Error flushing output: ", Perror(), NULL);
	SyncSpeech();
    }
    
    return 0;
}

int
Dectalk::SetDictionary(const char *word, const char *pronun)
{
    if (fd != -1) {
	AddToDictionary(word, pronun);
	return 0;
    }
  
    return 1;
}

int
Dectalk::UnSetDictionary(const char *word)
{
    if (fd != -1) {
	DeleteFromDictionary(word);
	return 0;
    }
  
    return 1;
}

int
Dectalk::OpenDectalk(const char *hostname, int timeout)
{
    int port = 1330;
    struct sockaddr_in sin;
    struct hostent * host;
    register int i=0;
 
    host = gethostbyname(hostname);
    
    if (host == NULL) {
	Error(EL_ERROR, "Dectalk::OpenDectalk",
	      "gethostbyname failed for ", hostname, ":  ", Perror(), NULL);
	return -1;
    }
 
    sin.sin_family = AF_INET;
    sin.sin_port = htons(port);
    sin.sin_addr.s_addr = *((u_long *) host->h_addr);

    while (i<timeout) {
	// call socket() inside the loop - if connect() fails we have to reopen.
	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
	    Error(EL_ERROR, "Dectalk::OpenDectalk", "socket: ", Perror(), NULL);
	    return -1;
	}
 
	// Try to connect.  If we're interrupted by a signal, don't count it.
	if (connect(fd, (struct sockaddr *) &sin, sizeof (sin)) < 0) {
	    if (errno == EINTR) {
		close(fd);
		continue;
	    }

	    i++;
	    
	    sleep(1);
	    close(fd);
	    continue;
	} else {
	    return fd;
	}
    }
    
    Error(EL_ERROR, "Dectalk::OpenDectalk", "connect: ", Perror(), NULL);
    return -1;
}

int
Dectalk::MarkIndex(int index)
{
    char buffer[256];
    
    if (fd == -1) {
	Error(EL_ERROR, "Dectalk::MarkIndex", "No connection to server", NULL);
	return 0;
    }
 
    sprintf(buffer, "\033P0;20;%dz\033\\", index);
    write(fd, "\033P0;20;1z\033\\", 11);
    if (fflush(file) != 0) 
	Error(EL_ERROR, "Dectalk::MarkIndex",
	      "Error flushing output: ", Perror(), NULL);
 
    return 0;
}

int
Dectalk::SyncSpeech()
{
    char *buffer = "\033P0;11z\033\\";
    
    if (fd == -1) {
	Error(EL_ERROR, "Dectalk::SyncSpeech",
	      "No connection to server", NULL);
	return 0;
    }
 
    write(fd, buffer, strlen(buffer));
    if (fflush(file) != 0)
	Error(EL_ERROR, "Dectalk::SyncSpeech", 
	      "Error flushing output: ", Perror(), NULL);

    return 0;
}
 
 
 
int
Dectalk::DialPhone(const char *phone_number)
{
    char buffer[256];
    
    if (fd == -1) {
	Error(EL_ERROR, "Dectalk::DialPhone", "No connection to server", NULL);
	return 0;
    }
 
    sprintf(buffer, "\033P0;60;20;40z%s\033\\", phone_number);
    write(fd, buffer, strlen(buffer));
    if (fflush(file) != 0)
	Error(EL_ERROR, "Dectalk::DialPhone", 
	      "Error flushing output: ", Perror(), NULL);

    return 0;
}
 
 
int
Dectalk::HangupPhone()
{
    char *buffer = "^[P0;60;11z^[\\";
    
    if (fd == -1) {
	Error(EL_ERROR, "Dectalk::HangUpPhone",
	      "No connection to server", NULL);
	return 0;
    }
 
    write(fd, buffer, strlen(buffer));
    if (fflush(file) != 0)
	Error(EL_ERROR, "Dectalk::HangupPhone", 
	      "Error flushing output: ", Perror(), NULL);

    return 0;
}
 
 
int
Dectalk::AddToDictionary(const char *word, const char *pronun)
{
    char buffer[256];
    
    if (fd == -1) {
	Error(EL_ERROR, "Dectalk::AddToDictionary",
	      "No connection to server", NULL);
	return 0;
    }
 
    sprintf(buffer, "\033P0;40z %s %s\033\\", word, pronun);
    write(fd, buffer, strlen(buffer));
    if (fflush(file) != 0)
	Error(EL_ERROR, "Dectalk::AddToDictionary", 
	      "Error flushing output: ", Perror(), NULL);

    return 0;
}
 
int 
Dectalk::DeleteFromDictionary(const char *word)
{
    char buffer[256];
 
    if (fd == -1) {
	Error(EL_ERROR, "Dectalk::DeleteFromDictionary",
	      "No connection to server", NULL);
	return 0;
    }
 
    sprintf(buffer, "\033P0;40z %s \033\\", word);
    write(fd, buffer, strlen(buffer));
    if (fflush(file) != 0)
	Error(EL_ERROR, "Dectalk::DeleteToDictionary", 
	      "Error flushing output: ", Perror(), NULL);

    return 0;
}

//
// This method should be called just before mercator kills all its
// subprocesses
//
void
Dectalk::ShutDown()
{
    Error(EL_DEBUG, "Dectalk::ShutDown", "Shutting down Dectalk object",
	  NULL);

    // just need to prevent Dectalk object from trying to contact the
    // server
    fd = -1;
}
