// -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

#ifndef _Dectalk_h
#define _Dectalk_h

//
//  class definition for generic speech synthesis resouce
//
 
//
// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Dectalk.h,v $
// $Revision: 6.6 $
// $Date: 1995/10/25 19:27:00 $
/* $Log: Dectalk.h,v $
 * Revision 6.6  1995/10/25  19:27:00  spine
 * Added a ShutDown method to prevent the object from trying to speak
 * after the server has been killed. (It may cause mercator to hang until the
 * client times out if the server has been killed.)
 *
 * Revision 6.5  1995/09/21  21:39:55  keith
 * Made a few cleanups, signal handling stuff...
 *
 * Revision 6.4  1995/09/19  21:39:22  spine
 * Added StartServer member.
 *
 * Revision 6.3  1995/08/17  20:38:40  spine
 * Added an api to connect to the server.
 *
 * Revision 6.2  1995/08/09  20:23:16  keith
 * Added capabilities stuff (a start anyway...)
 *
 * Revision 6.1  1995/06/02  20:24:46  beth
 * added copyright
 *
 * Revision 6.0  1995/05/20  22:20:21  beth
 * Mercator Version 6
 *
 * Revision 1.3  1995/04/01  19:40:38  keith
 * End of work by Keith
 *
 * Revision 1.2  1995/01/19  22:23:31  keith
 * Automatic global checkin
 *
 * Revision 1.1  1994/11/16  20:46:23  spine
 * Initial revision
 *
 * Revision 1.1  1994/05/24  19:49:25  stacy
 * Initial revision
 *
 * Revision 1.2  1994/04/06  20:56:20  kathryn
 * *** empty log message ***
 *
 * Revision 1.1  1994/04/06  19:52:06  kathryn
 * Initial revision
 *
*/

#ident "$Id: Dectalk.h,v 6.6 1995/10/25 19:27:00 spine Exp $"

#include <stdio.h>		// for FILE *
#include <stdlib.h>

#include "Speech.h"
#include "Loadable.h"


class Dectalk : public Speech {
private:
    FILE *file;		// used only for fflush()
    int fd;

    // The capabilities for this particular class of device.
    static Capabilities capabilities;
    static int		capabilitiesInitialized;

    int OpenDectalk(const char *host, int time);// establish server connection
    
    int MarkIndex(int index);			// mark index (not used)
    int SyncSpeech();				//
    int DialPhone(const char *phone_number);    // (not used)
    int HangupPhone();                    	// (not used)
    int AddToDictionary(const char *word, const char *pronun); 
    int DeleteFromDictionary(const char *word);
    
public:
    Dectalk(const char *hostname=NULL);
    virtual ~Dectalk();
    //
    // Inherited from Speech
    //
    virtual const Capabilities& GetCapabilities() const;

    virtual int ConnectServer(const char *dectalkhost);
    virtual int StartServer();
    
    virtual int Speak(const char *str);		// speak some text
    virtual int SpeakUnix(const char *str);	// translate Unix and speak
    virtual int Notify(const char *type);     	// does nothing 
    virtual int SetSpeechRate(int rate);	// set the rate of speech 
    virtual int StopSpeaking();			// interrupt the speaking
    virtual int SetVoice(const char *voice);   	// Sets the voice
    virtual int SetLanguage(const char *l);	// set language
    virtual int SetGain(int g);			// set the gain
    virtual int SetDictionary(const char *word, const char *pronun);
    virtual int UnSetDictionary(const char *word);
    virtual void ShutDown();
};


#endif /* _Dectalk_h */



