//  -*-Mode: C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

// This is slightly modified version of Dectalk. I main differences are
// how attributes of dectalk express are changed since the commands are
// different from the old dectalk's.    -wl

#include "DectalkX.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/wait.h>
 
#include <netinet/in.h>
 
#include <errno.h>
#include <netdb.h>
#include <signal.h>
#include <sgtty.h>
#include <fcntl.h>

#include <ctype.h>
#include <sysent.h>
#include <unistd.h>
#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <stream.h>

#include "Error.h"
#include "Util.h"
#include "Mercator.h"
#include "Template.h"
#include "Config.h"

LOADABLE_CLASS_DEFN(DectalkX);

#define is_member(c, s)  strchr((s), (c))
#define SPECIAL_CHARS   "!@#$%^&*()/[]{}"
#define WORD_CONTINUE   "-"
#define SENTENCE_ENDER  ".?!"
#define TEXT_PUNC       ".,:;!?"
#define PUNC_NOT_DELIM  ":;"

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

extern Mercator *mercator;

Speech::Capabilities 	DectalkX::capabilities;
int			DectalkX::capabilitiesInitialized = 0;

DectalkX::DectalkX(const char *dectalkhost) :
    Speech("DectalkX"),
    file(NULL),
    fd(-1)
{
    DectalkX::StartServer();
    DectalkX::ConnectServer(dectalkhost);
    
    if (!DectalkX::capabilitiesInitialized++) {
	DectalkX::capabilities.languages.append("English");

	DectalkX::capabilities.voices.append("Perfect Paul");
	DectalkX::capabilities.voices.append("Beautiful Betty");
	DectalkX::capabilities.voices.append("Huge Harry");
	DectalkX::capabilities.voices.append("Frail Frank");
	DectalkX::capabilities.voices.append("Kit the Kid");
	DectalkX::capabilities.voices.append("Rough Rita");
	DectalkX::capabilities.voices.append("Uppity Ursula");
	DectalkX::capabilities.voices.append("Doctor Dennis");
	DectalkX::capabilities.voices.append("Whispering Wendy");

	DectalkX::capabilities.minRate = 75;
	DectalkX::capabilities.maxRate = 650;

	DectalkX::capabilities.gainSupported = 0;
    }
}

DectalkX::~DectalkX()
{
    if (file)
        fclose(file);
    
    if (fd != -1)
	close(fd);
}

int
DectalkX::ConnectServer(const char *dectalkhost)
{
    FieldData *maxTriesData = Config::GetDefaultByName("dectalkxTimeout");
    int maxTries = 5;

    if (maxTriesData && (maxTriesData->Tag() == INTEGER))
	maxTries = maxTriesData->Integer();
    
    Error(EL_DEBUG, "DectalkX::ConnectServer",
	  "Connecting to Dectalk Express server!", NULL);

    if (dectalkhost == NULL)
	if ((dectalkhost = getenv( "DECTALKHOST" )) == NULL)
	    dectalkhost = "localhost";

    fd = OpenDectalkX(dectalkhost, maxTries);

    if (fd == -1) {
	Error(EL_ERROR, "DectalkX::ConnectServer", "server connection",
	      " timed out", NULL);
	return -1;
    } else {
	Error(EL_DEBUG, "DectalkX::ConnectServer",
	      "connection to server successful", NULL);
	file = fdopen(fd, "w");

	// Initialize DectalkX
	SetSpeechRate(180);
	SetVoice("Perfect Paul");
	return 0;
    }
}

//
// return -1 if server was not started, 0 otherwise. (Will assume that
// if DECTALKHOST environment variable is only set if the user wants to
// use a server on another machine. So essentially setting DECTALKHOST
// to anything will prevent a server from being started.
//
int
DectalkX::StartServer()
{
    const char *host;

    // first check if an env. var. is set. I'm assuming that DECTALKHOST
    // is only set if the user wants to use a remote server
    if((host = getenv("DECTALKHOST")) == NULL) {
	if(mercator->ExecList("dectalkxServer")) {
	    Error(EL_DEBUG, "DectalkX::StartServer",
		  "dectalkxServer not found.", NULL);
	    return -1;
	}
	return 0;
    } else
	Error(EL_DEBUG, "DectalkX::StartServer", "Server has been started on ",
	      host, NULL);
    
    return 0; // assuming the server has been started on another host
}

const Speech::Capabilities&
DectalkX::GetCapabilities() const
{
    return DectalkX::capabilities;
}

int
DectalkX::Notify(const char *type)
{
    if (fd != -1) {
	// Do whatever this is supposed to do!?
    } else {
	Speech::Notify(type);
    }
    
    return 0;
}

//
// NOTE: Right now Speak expects strings whose length are no more than 2K.
// 
int
DectalkX::Speak(const char *string)
{
    char buffer[2050];
    char *s;
    short toDel = 1;
    
    Error(EL_DEBUG, "DectalkX::Speak", "|", string, "|", NULL);

    if(!string) {
	Error(EL_DEBUG, "DectalkX::Speak", "Given a null string", NULL);
	return -1;
    }
	
    // if (is_member(string[strlen(string)-1],TEXT_PUNC))  
    //	sprintf(buffer, "%s\n", string);
    //else

    // Special case for character mode. This is a hack for now.
    if(strlen(string) == 1) {
	switch (string[0]) {
	case ' ':
	    s = Strdup("space");
	    break;
	case '\t':
	    s = Strdup("tab");
	    break;
	case '\n':
	    s = Strdup("new line");
	    break;
	case '\"':
	    s = Strdup("quote");
	    break;
	case '."':
	    s = Strdup("period");
	    break;
	case ',"':
	    s = Strdup("comma");
	    break;
	case ';':
	    s = Strdup("semi colon");
	    break;
	case ':':
	    s = Strdup("colon");
	    break;
	case '\'':
	case '`':
	    s = Strdup("apostrophe");
	    break;
	case '?':
	    s = Strdup("question mark");
	    break;
	case '!':
	    s = Strdup("exclamation mark");
	    break;
	case '@':
	    s = Strdup("at");
	    break;
	case '#':
	    s = Strdup("pound");
	    break;
	case '^':
	    s = Strdup("carret");
	    break;
	case '*':
	    s = Strdup("asterisk");
	    break;
	case '(':
	    s = Strdup("left paren");
	    break;
	case ')':
	    s = Strdup("right paren");
	    break;
	case '[':
	case '{':
	    s = Strdup("left bracket");
	    break;
	case ']':
	case '}':
	    s = Strdup("right bracket");
	    break;
	case '>':
	    s = Strdup("greater than");
	    break;
	case '<':
	    s = Strdup("less than");
	    break;
	case '_':
	    s = Strdup("underscore");
	    break;
	case '\\':
	    s = Strdup("back slash");
	    break;
	case '|':
	    s = Strdup("pipe");
	    break;
	case '~':
	    s = Strdup("tilda");
	    break;
	default:
	    s = (char *) string;
	    toDel = 0;  // do not delete s
	}
	sprintf(buffer, "%s\013\n", s);
	
	if(toDel) delete [] s;
    }
    else
	sprintf(buffer, "%s\013\n", string);
	    

    
#ifdef junk
    // This should fix the problem of not finishing reading a line that
    // contains a period which is not at the end of the line.
    sprintf(buffer, "%s", string);

    // strip off trailing spaces
    int i = strlen(buffer) - 1;

    while((buffer[i] == ' ') && (i>0))
	i--;

    buffer[i+1] = '\0';

    // Try to be smarter about terminating a line of text
    if(i>=0) {
	switch(buffer[i]) {
	case '.':
	case ',':
	case ':':
	case ';':
	case '?':
	case '!':
	case ')':
	    strcat(buffer, "\n");
	    break;
	default:
	    strcat(buffer, ".\n");
	    break;
	}
    }
#endif
    
    if (fd != -1) {
	write(fd, buffer, strlen(buffer));
	if (fflush(file) != 0) {
	    Error(EL_ERROR, "DectalkX::Speak", "Error flushing speech: ",
		  Perror(), NULL);
	}
	buffer[strlen(buffer)-1] = '\0';
	Error(EL_DEBUG, "DectalkX::Speak", "Sending: | ", buffer, " |", NULL);
    } else
	Speech::Speak (string);

    return 0;
}
 
int
DectalkX::SpeakUnix(const char *string)
{
    Error(EL_DEBUG, "DectalkX::SpeakUnix", "|", string, "|", NULL);

    if (fd != -1) { 
	int size = strlen(string);
	int i = 0, o = 0;
	char buffer[1024];
   
	while ((string[i] == ' ') && (i < size)) i++;
   
	for (o = 0; i < size; i++) { 
	    if (iscntrl(string[i]))
		continue;
	    if (isalnum(string[i])) { 
		buffer[o++] = string[i];
		continue;
            }
	    if (string[i] == ' ') { 
		buffer[o++] = string[i];
		continue;
            }
	    if (is_member(string[i], SPECIAL_CHARS)) { 
		buffer[o++] = ' ';
		buffer[o++] = string[i];
		buffer[o++] = ' ';
            } 
        }
 
	if (o == 0) 
	    return 0;
	o--;
	
	while (o && (buffer[o] == ' '))
	    o--;
	
	o++;
	
	buffer[o++] = '.'; /* awful hack to get it to talk immediately */
	buffer[o++] = '\n';
	buffer[o] = '\0';
	
	Error(EL_DEBUG, "DectalkX::SpeakUnix", "|", buffer, "|", NULL);

	write(fd, buffer, o);
	if (fflush(file) != 0)
	    Error(EL_ERROR, "DectalkX::SpeakUnix",
		  "Error flushing SpeakUnix: ", Perror(), NULL);
    } else {
	Speech::SpeakUnix(string);
    }
}

int
DectalkX::SetSpeechRate(int rate)
{ 
    char buffer[256];

    if (fd != -1) { 
	sprintf(buffer, "[:rate %i]", rate);
	write(fd, buffer, strlen(buffer));
	if (fflush(file) != 0)
	    Error(EL_ERROR, "DectalkX::SetSpeechRate",
		  "Error flushing output: ", Perror(), NULL);
    } else {
	Speech::SetSpeechRate(rate);
    }
   
    SyncSpeech();
    return 0;
}

int
DectalkX::StopSpeaking()
{
    Error(EL_DEBUG, "DectalkX::StopSpeaking", NULL);

    if (fd != -1) { 
	char *buffer = "\03\n";

	write(fd, buffer, strlen(buffer));
	if (fflush(file) != 0)
	    Error(EL_ERROR, "DectalkX::StopSpeaking",
		  "Error flushing output: ", Perror(), NULL);
    } else {
	Speech::StopSpeaking();
    }
    return 0;
}

int
DectalkX::SetLanguage(const char *lang)
{
    mString dummy;
    
    if (!DectalkX::capabilities.languages.search(lang, dummy)) {
	Error(EL_ERROR, "Dectalk::SetLanguage", "Unsupported language: ",
	      lang, NULL);
    }

    // The Dectalk actually only supports english, so don't do anything.
    return 0;
}

int
DectalkX::SetGain(int)
{
    Error(EL_ERROR, "DectalkX::SetGain",
	  "Gain control not supported on the external version of DectalkX",
	  NULL);
    return 0;
}

//
// "Perfect Paul": standard male voice
// "Beautiful Betty": standard female voice
// "Huge Harry": deep male voice
// "Frail Frank": older male voice
// "Kit the Kid": child's voice
// "Rough Rita": deep female voice
// "Uppity Ursula": light female voice
// "Doctor Dennis": whispery male voice
// "Whispery Wendy": whispery female voice
//
int
DectalkX::SetVoice (const char *voice_name)
{ 
    char buffer[256], cmd;

    if (!strcmp(voice_name, "Perfect Paul"))
	cmd = 'p';
    else if (!strcmp(voice_name, "Beautiful Betty"))
	cmd = 'b';
    else if (!strcmp(voice_name, "Huge Harry"))
	cmd = 'h';
    else if (!strcmp(voice_name, "Frail Frank"))
	cmd = 'f';
    else if (!strcmp(voice_name, "Kit the Kid"))
	cmd = 'k';
    else if (!strcmp(voice_name, "Rough Rita"))
	cmd = 'r';
    else if (!strcmp(voice_name, "Uppity Ursula"))
	cmd = 'u';
    else if (!strcmp(voice_name, "Doctor Dennis"))
	cmd = 'd';
    else if (!strcmp(voice_name, "Whispering Wendy"))
	cmd = 'w';
    else {
	Error(EL_ERROR, "DectalkX::SetVoice", "Invalid voice name \"",
	      voice_name, "\"", NULL);
	return -1;
    }
    
    if (fd != -1) { 
	sprintf(buffer, "[:name %c]", cmd);
	write(fd, buffer, strlen(buffer));
	if (fflush(file) != 0) 
	    Error(EL_ERROR, "DectalkX::SetVoice",
		  "Error flushing output: ", Perror(), NULL);
	SyncSpeech();
    }
    
    return 0;
}

int
DectalkX::OpenDectalkX(const char *hostname, int timeout)
{
    int port = 1330;
    struct sockaddr_in sin;
    struct hostent * host;
    register int i=0;
 
    host = gethostbyname(hostname);
    
    if (host == NULL) {
	Error(EL_ERROR, "DectalkX::OpenDectalkX",
	      "gethostbyname failed for ", hostname, ":  ", Perror(), NULL);
	return -1;
    }
 
    sin.sin_family = AF_INET;
    sin.sin_port = htons(port);
    sin.sin_addr.s_addr = *((u_long *) host->h_addr);

    while (i<timeout) {
	// call socket() inside the loop - if connect() fails we have to reopen
	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
	    Error(EL_ERROR, "DectalkX::OpenDectalkX", "socket: ",
		  Perror(), NULL);
	    return -1;
	}

	// Try to connect.  If we're interrupted by a signal, don't count it.
	if (connect(fd, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
	    if (errno == EINTR) {
		close(fd);
		continue;
	    }

	    i++;

	    sleep(1);
	    close(fd);
	    continue;
	} else {
	    return fd;
	}
    }
    
    Error(EL_ERROR, "DectalkX::OpenDectalkX" "connect: ", Perror(), NULL);
    return -1;
}

int
DectalkX::MarkIndex(int index)
{
    char buffer[256];
    
    if (fd == -1) {
	Error(EL_ERROR, "DectalkX::MarkIndex", "No connection to server", NULL);
	return 0;
    }
 
    sprintf(buffer, "[:index mark %d]", index);
    write(fd, buffer, strlen(buffer));
    if (fflush(file) != 0) 
	Error(EL_ERROR, "DectalkX::MarkIndex",
	      "Error flushing output: ", Perror(), NULL);
 
    return 0;
}

int
DectalkX::SyncSpeech()
{
    char *buffer = "[:sync]";
    
    if (fd == -1) {
	Error(EL_ERROR, "DectalkX::SyncSpeech",
	      "No connection to server", NULL);
	return 0;
    }
 
    write(fd, buffer, strlen(buffer));
    if (fflush(file) != 0)
	Error(EL_ERROR, "DectalkX::SyncSpeech", 
	      "Error flushing output: ", Perror(), NULL);

    return 0;
}
 
 
int
DectalkX::DialPhone(const char *phone_number)
{
    char buffer[256];
    
    if (fd == -1) {
	Error(EL_ERROR, "DectalkX::DialPhone", "No connection to server", NULL);
	return 0;
    }
 
    sprintf(buffer, "[:dial \"%s\"]", phone_number);
    write(fd, buffer, strlen(buffer));
    if (fflush(file) != 0)
	Error(EL_ERROR, "DectalkX::DialPhone", 
	      "Error flushing output: ", Perror(), NULL);

    return 0;
}
 
 
int
DectalkX::HangupPhone()
{
    char *buffer = "^[P0;60;11z^[\\";
    
    if (fd == -1) {
	Error(EL_ERROR, "DectalkX::HangUpPhone",
	      "No connection to server", NULL);
	return 0;
    }
 
    write(fd, buffer, strlen(buffer));
    if (fflush(file) != 0)
	Error(EL_ERROR, "DectalkX::HangupPhone", 
	      "Error flushing output: ", Perror(), NULL);

    return 0;
}

//
// This method should be called just before mercator kills all its
// subprocesses
//
void
DectalkX::ShutDown()
{
    Error(EL_DEBUG, "DectalkX::ShutDown", "Shutting down DectalkX object",
	  NULL);

    // just need to prevent DectalkX object from trying to contact the
    // server
    fd = -1;
}
