// -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

#ifndef _DectalkX_h
#define _DectalkX_h

//
//  class definition for generic speech synthesis resouce
//

#include <stdio.h>		// for FILE *
#include <stdlib.h>

#include "Speech.h"
#include "Loadable.h"


class DectalkX : public Speech {
private:
    FILE *file;		// used only for fflush()
    int fd;

    // The capabilities for this particular class of device.
    static Capabilities capabilities;
    static int		capabilitiesInitialized;

    int OpenDectalkX(const char *host, int t);// establish server connection
    
    int MarkIndex(int index);			// mark index (not used)
    int SyncSpeech();				//
    int DialPhone(const char *phone_number);    // (not used)
    int HangupPhone();                    	// (not used)
    
public:
    DectalkX(const char *hostname=NULL);
    virtual ~DectalkX();

    //
    // Inherited from Speech
    //
    virtual const Capabilities& GetCapabilities() const;

    virtual int ConnectServer(const char *dectalkhost);
    virtual int StartServer();
    
    virtual int Speak(const char *str);		// speak some text
    virtual int SpeakUnix(const char *str);	// translate Unix and then speak
    virtual int Notify(const char *type);     	// does nothing 
    virtual int SetSpeechRate(int rate);	// set the rate of speech 
    virtual int StopSpeaking();			// interrupt the speaking
    virtual int SetVoice(const char *voice);   	// Sets the voice
    virtual int SetLanguage(const char *l);	// set language
    virtual int SetGain(int g);			// set the gain
    virtual void ShutDown();
};

#endif /* _DectalkX_h */
