//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
/************************************************************************** 
 * REVISION LOG 
 * 
 * $Author: keith $ 
 * $Source: /net/hm10/keith/proj/mercator/6.0/RCS/DynaLoader.cc,v $ 
 * $Revision: 6.6 $ 
 * $Date: 1995/12/03 01:47:04 $ 
 * $Log: DynaLoader.cc,v $
// Revision 6.6  1995/12/03  01:47:04  keith
// Fixed some memory errors that Purify was complaining about.
//
// Revision 6.5  1995/09/01  21:33:25  spine
// Changed loading message from EL_ERROR to EL_DEBUG (sorry, it's been bugging
// me).
//
// Revision 6.4  1995/08/18  15:20:19  keith
// fixed constness on SearchPath() method...was causing warnings.
//
// Revision 6.3  1995/07/28  20:31:27  spine
// Added copyright notice.
//
// Revision 6.2  1995/07/01  20:02:19  spine
// Removed all references to LEDA strings and LEDA.
//
// Revision 6.1  1995/05/29  15:23:21  iansmith
// - Changed the DynaLoader's SearchPath() function to return a
// void instead of the const char * it used to and to take an
// mListVal<string> as a parameter. This is the function that gets called
// when the Config obj sets the SearchPath from the value in the config
// file.
// - Added a static variable to hold the search path in... of type
// mListVal<string>. This makes checking each of the directories
// substantially easier.
// - Modified Load() function to try everything on the search path before
// falling back to LD_LIBRARY_PATH.
//
// Revision 6.0  1995/05/20  22:20:21  beth
// Mercator Version 6
//
// Revision 1.1  1995/04/01  19:40:38  keith
// End of work by Keith
//
// Revision 1.4  1994/09/18  16:09:54  keith
// Automatic global checkin
//
// Revision 1.4  1994/09/18  16:09:54  keith
// Automatic global checkin
//
// Revision 1.3  1994/09/18  16:08:36  keith
// Automatic global checkin
//
// Revision 1.3  1994/09/18  16:08:36  keith
// Automatic global checkin
//
// Revision 1.2  1994/05/09  16:11:19  keith
// *** empty log message ***
//
// Revision 1.2  1994/05/09  16:11:19  keith
// *** empty log message ***
//
 * 
 **************************************************************************/ 

#include <string.h>
#include "DynaLoader.h"

mListVal<mString> DynaLoader::searchPath;

void DynaLoader::SearchPath(const mListVal<mString> & l)
{
    searchPath=l;
}

DynaLoader::Handle
DynaLoader::Load(const char *objectFile, int mode)
{
    //
    // N.B. Default argument for mode causes RTLD_LAZY behavior.
    //

    Handle handle = NULL;
    mString currentPath;
    // init iterator on our list of interest
    mListValIterator<mString> iter(searchPath);
    while (iter.next()) {
	char *loadfile;
	
	currentPath=iter.current();
	
	loadfile = new char[strlen((const char *) currentPath) +
			   strlen(objectFile) + 2];
	
	sprintf(loadfile, "%s/%s", (const char *) currentPath, objectFile);
	Error(EL_DEBUG, "DynaLoader::Load", "trying to load ", loadfile, NULL);
	// try it...
	if ((handle = ::dlopen(loadfile, mode)) != NULL) {
	    // got it, bail out
	    delete [] loadfile;
	    return handle;
	} else {
	    Error(EL_ERROR,"DynaLoader::Load","Error loading  ", loadfile,
		  ":" , ::dlerror(), NULL);
	}

	delete [] loadfile;
    }

    
    // if all else fails, try to dlopen w/the LD_LIBRARY_PATH
    // brain damage
    if ((handle = ::dlopen(objectFile, mode)) == NULL) {
	Error(EL_ERROR, "DynaLoader::Load", "dlopen of '", objectFile,
	      "' failed: ", ::dlerror(), NULL);
    }
    return handle;
}

int
DynaLoader::Unload(Handle handle)
{
    if (handle == NULL) {
	Error(EL_ERROR, "DynaLoader::Unload", "no object has been loaded.",
	      NULL);
	return 0;
    }

    if (::dlclose(handle) != 0) {
	Error(EL_ERROR, "DynaLoader::Unload", "dlclose failed: ",
	      dlerror(), NULL);
	return 0;
    }

    return 1;
}

DynaLoader::Handle
DynaLoader::Symbol(Handle handle, const char *name)
{
    Handle addr = NULL;

    if (handle == NULL) {
	Error(EL_ERROR, "DynaLoader::Symbol", "no object has been loaded.",
	      NULL);
	return NULL;
    }

    if ((addr = dlsym(handle, name)) == NULL) {
	Error(EL_ERROR, "DynaLoader::Symbol", "dlsym failed: ",
	      dlerror(), NULL);
	return NULL;
    } else {
	return addr;
    }
}


#if 0

DynaLoader::DynaLoader() :
    searchPath(NULL),
    handle(NULL)
{
}

DynaLoader::~DynaLoader()
{
    if (handle != NULL)
	if (dlclose(handle) != 0)
	    Error(EL_ERROR, "DynaLoader::DynaLoader", "dlclose failed: ",
		  dlerror(), NULL);

    if (searchPath != NULL)
	delete [] searchPath;
}

const char *
DynaLoader::SearchPath(const char *d)
{
    if (d) {
	delete [] searchPath;
	searchPath = strdup(d);
    }

    return searchPath;
}

int
DynaLoader::Load(const char *objectFile, int mode)
{
    //
    // Default argument for m causes RTLD_LAZY behavior.
    //
    // NOTE that we currently don't use SearchPath.
    //

    if ((handle = dlopen(objectFile, mode)) == NULL) {
	Error(EL_ERROR, "DynaLoader::Load", "dlopen of '", objectFile, "' failed: ",
	      dlerror(), NULL);
	return 0;
    } else {
	return 1;
    }
}

int
DynaLoader::Unload()
{
    if (handle == NULL) {
	Error(EL_ERROR, "DynaLoader::Unload", "no object has been loaded.",
	      NULL);
	return 0;
    }

    if (dlclose(handle) != 0) {
	Error(EL_ERROR, "DynaLoader::Unload", "dlclose failed: ",
	      dlerror(), NULL);
	return 0;
    }

    return 1;
}

void *
DynaLoader::Symbol(const char *name)
{
    void *addr = NULL;

    if (handle == NULL) {
	Error(EL_ERROR, "DynaLoader::Symbol", "no object has been loaded.",
	      NULL);
	return NULL;
    }

    if ((addr = dlsym(handle, name)) == NULL) {
	Error(EL_ERROR, "DynaLoader::Symbol", "dlsym failed: ",
	      dlerror(), NULL);
	return NULL;
    } else {
	return addr;
    }
}
#endif

