//
//	Simple error handling package.
//
//				keith edwards 31dec93
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
/************************************************************************** 
 * REVISION LOG 
 * 
 * $Author: keith $ 
 * $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Error.cc,v $ 
 * $Revision: 6.7 $ 
 * $Date: 1995/12/13 21:51:34 $ 
 * $Log: Error.cc,v $
// Revision 6.7  1995/12/13  21:51:34  keith
// Fixed some dumpstack stuff.
//
// Revision 6.6  1995/12/04  20:39:46  keith
// Added DumpStack()
//
// Revision 6.5  1995/10/23  18:26:48  spine
// Added boundary checking when setting the error level.
//
// Revision 6.4  1995/10/17  20:48:47  spine
// Added an EO_CONSOLE element to the errorOutputs enum type. This indicates
// that the user want the error messages to be sent to the console.
//
// Revision 6.3  1995/10/12  20:11:54  spine
// Overloaded the ErrorOutput method to accept a file descriptor and a fstream
// to print output to.
//
// Revision 6.2  1995/10/10  15:43:22  spine
// Overloaded ErrorOutput to direct error messages to an fstream.
//
// Revision 6.1  1995/07/28  20:32:19  spine
// Added copyright notice.
//
// Revision 6.0  1995/05/20  22:20:21  beth
// Mercator Version 6
//
// Revision 1.2  1995/04/19  14:55:38  iansmith
// Added some #ifdefs which allow Error to be called from the AudioFile
// object which is multithreaded.
//
// Revision 1.1  1995/01/19  22:23:31  keith
// Automatic global checkin
//
// Revision 1.9  1994/09/28  18:12:48  keith
// Automatic global checkin
//
// Revision 1.8  1994/09/18  16:09:54  keith
// Automatic global checkin
//
// Revision 1.7  1994/09/18  16:08:36  keith
// Automatic global checkin
//
// Revision 1.7  1994/09/18  16:08:36  keith
// Automatic global checkin
//
// Revision 1.6  1994/09/18  16:01:58  keith
// Automatic global checkin
//
// Revision 1.6  1994/09/18  16:01:58  keith
// Automatic global checkin
//
// Revision 1.5  1994/09/18  15:52:06  keith
// Automatic global checkin
//
// Revision 1.5  1994/09/18  15:52:06  keith
// Automatic global checkin
//
// Revision 1.4  1994/05/28  19:55:41  keith
// Made this file MT safe (if JB_MT_SAFE is #defined).  Also used the
// libC stream_locker class to ensure serialized access to I/O streams.
//
// Revision 1.3  1994/05/23  18:29:27  keith
// Added the calling thread id to the printout.
//
// Revision 1.2  1994/05/09  16:11:19  keith
// *** empty log message ***
//
 * 
 **************************************************************************/ 

#include <string.h>
#ifdef BEAM
#include <stdarg.h>
#else
#include <varargs.h>
#endif
#include <errno.h>
#include <stdlib.h>		// for exit()
#include <fstream.h>
#include "Error.h"


#ifdef JB_MT_HOT
#include <thread.h>
#endif /* JB_MT_HOT */

#ifdef JB_MT_SAFE
#include "Mutex.h"
#endif /* JB_MT_SAFE */

#ifdef MT_SAFE_OS_INTERFACE
extern "C" {
#include "os-interface.h"
}
#endif

static ErrorLevels 	errorLevel = EL_ALL_MESSAGES;
static ErrorOutputs 	errorOutput = EO_STDERR;

fstream *errout = NULL;


#ifdef MT_SAFE_OS_INTERFACE
static MUTEX_T errLock;
static errorInitCount=0;
__ErrorInit::__ErrorInit()
{
    MUTEX_init(&errLock);
}
__ErrorInit::~__ErrorInit()
{
    // no cleanup needed
}

#endif /* MT_SAFE_OS_INTERFACE */
#ifdef JB_MT_SAFE

static Mutex *errorLock = NULL;

static short errorInitCount = 0;

__ErrorInit::__ErrorInit()
{
    if (errorInitCount++ == 0)
	errorLock = new Mutex;
}

__ErrorInit::~__ErrorInit()
{
    if (--errorInitCount == 0)
	delete errorLock;
}

#endif /* JB_MT_SAFE */


ErrorLevels
ErrorLevel(void)
{
    return errorLevel;
}

ErrorLevels
ErrorLevel(ErrorLevels err)
{
    ErrorLevels old = errorLevel;

    // do some boundary checking
    if(err < EL_NO_MESSAGES)
	errorLevel = EL_NO_MESSAGES;
    else if(err > EL_ALL_MESSAGES)
	errorLevel = EL_ALL_MESSAGES;
    else
	errorLevel = err;

    return old;
}

ErrorOutputs
ErrorOutput(void)
{
    return errorOutput;
}

ErrorOutputs
ErrorOutput(ErrorOutputs err)
{
    ErrorOutputs old = errorOutput;
#ifdef JB_MT_SAFE
    stream_locker sLock(*errout, stream_locker::lock_now);
#endif /* JB_MT_SAFE */

    if (err == EO_SYSLOG) {
	*errout << "ErrorOutput: EO_SYSLOG currently not supported " << 
	    "without a given file descriptor or fstream.\n";
    } else {
        errorOutput = err;
    }

    return old;
}

//
// Send error output to the given fd. Assuming fd is already opened.
//
ErrorOutputs
ErrorOutput(ErrorOutputs err, int fd)
{
    if (fd > 1) {
	errout->close();
    }
    
    errout->attach(fd);
    errorOutput = err;

    return errorOutput;
}

//
// Send error output to the given file stream. Assuming the file stream is
// ready to be written to.
//
ErrorOutputs
ErrorOutput(ErrorOutputs err, fstream *ostr)
{
    if(errout)
	errout->close();

    if(ostr) {
	errout = ostr;
	errorOutput = err;
    }
    else
	errorOutput = EO_STDERR;
    
    return errorOutput;
}

#ifdef JB_MT_HOT
static void
SendThreadID()
{
    thread_t self = thr_self();
#ifdef JB_MT_SAFE
    stream_locker sLock(*errout, stream_locker::lock_now);
#endif /* JB_MT_SAFE */

    switch (errorOutput) {
	case EO_IGNORE :
	    return;
	case EO_SYSLOG :
	    // *errout << "SendErrorOutput:  EO_SYSLOG currently not supported.\n";
	    *errout << "[" << self << "] ";
	    errout->flush();
	    break;
	case EO_STDERR :
        case EO_CONSOLE :
	    *errout << "[" << self << "] ";
	    errout->flush();
	    break;
	default :
	    *errout << "SendErrorOutput: unknown output specifier.\n";
	    *errout << "[" << self << "] ";
	    errout->flush();
	    break;
    }
}
#endif /* JB_MT_HOT */

static void
SendErrorOutput(const char *s)
{
#ifdef JB_MT_SAFE
    stream_locker sLock(*errout, stream_locker::lock_now);
#endif /* JB_MT_SAFE */

    switch (errorOutput) {
      case EO_IGNORE :
	return;
      case EO_SYSLOG :
      case EO_CONSOLE :
	if (errout) {
	    *errout << s;
	    errout->flush();
	}
	else {
	    cerr << "SendErrorOutput: Log file not specified.\n";
	    cerr << s;
	    cerr.flush();
	}
	break;
      case EO_STDERR :
	cerr << s;
	cerr.flush();
	break;
      default :
	cerr << "SendErrorOutput: unknown output specifier.\n";
	cerr << s;
	cerr.flush();
	break;
    }
}

void
Error(int level, const char *name ...)
{
    va_list arglist;
    char *string;
    
#ifdef JB_MT_SAFE
    if (errorLock)
	errorLock->Lock();
#endif /* JB_MT_SAFE */
#ifdef MT_SAFE_OS_INTERFACE
    MUTEX_lock(&errLock);
#endif /* MT_SAFE_OS_INTERFACE */
    if (level > errorLevel) {
#ifdef JB_MT_SAFE
	if (errorLock)
	    errorLock->Unlock();
#endif /* JB_MT_SAFE*/
#ifdef MT_SAFE_OS_INTERFACE
	MUTEX_unlock(&errLock);
#endif /* MT_SAFE_OS_INTERFACE */
	
	// Are these necessary?
#ifdef BEAM
	va_start(arglist, name);
#else
	va_start(arglist);
#endif
	va_end(arglist);
	return;
    }

#ifdef JB_MT_HOT
    SendThreadID();
#endif /* JB_MT_HOT */

    switch (level) {
    case EL_ABORT :
	SendErrorOutput("Abort: ");
	break;
    case EL_FATAL :
	SendErrorOutput("Fatal: ");
	break;
    case EL_ERROR :
	SendErrorOutput("Error: ");
	break;
    case EL_WARNING :
	SendErrorOutput("Warning: ");
	break;
    case EL_STATUS :
	SendErrorOutput("Status: ");
	break;
    case EL_INFORMATIONAL :
	SendErrorOutput("Informational: ");
	break;
    case EL_DEBUG :
	SendErrorOutput("Debug: ");
	break;
    default :
	break;
    }

    SendErrorOutput(name);
    SendErrorOutput(": ");

#ifdef BEAM
    va_start(arglist, name);
#else
    va_start(arglist);
#endif
    while (1) {
	string = va_arg(arglist, char *);
	if (string == NULL)
	    break;
	SendErrorOutput(string);
    }
    va_end(arglist);

    SendErrorOutput("\n");

    if (level == EL_FATAL) {
	exit(1);
    }

    if (level == EL_ABORT) {
	abort();
    }

#ifdef JB_MT_SAFE
    if (errorLock)
	errorLock->Unlock();
#endif /* JB_MT_SAFE */
#ifdef MT_SAFE_OS_INTERFACE
    MUTEX_unlock(&errLock);
#endif /* MT_SAFE_OS_INTERFACE */

}


const char *
Perror(int e)
{
    return strerror(e);
}

const char *
Perror(void)
{
    return strerror(errno);
}

#ifdef DUMP_STACK
//
// DumpStack() is a particularly scary (but useful) routine to dump out the
// current stack backtrace.  This will only work on Solaris 2.4 and later,
// SPARC, x86, and PowerPC.
//
// If you enable this, you'll have to link with -ldl to get dladdr().
//

#include <dlfcn.h>
#include <setjmp.h>
#include <sys/types.h>
#include <sys/reg.h>
#include <sys/frame.h>

#if defined(sparc) || defined(__sparc)
#define FLUSHWIN() asm("ta 3");
#define FRAME_PTR_INDEX 1
#define SKIP_FRAMES 0
#endif

#if defined(i386) || defined(__i386)
#define FLUSHWIN() 
#define FRAME_PTR_INDEX 3
#define SKIP_FRAMES 1
#endif

#if defined(ppc) || defined(__ppc)
#define FLUSHWIN() 
#define FRAME_PTR_INDEX 0
#define SKIP_FRAMES 2
#endif

//
// This function walks up call stack, calling user-supplied
// function once for each stack frame, passing the pc and the user-supplied
// usrarg as the argument.  
//

static int
CallStack(int (*func)(void *, void *), void * usrarg)
{
    struct frame *sp;
    jmp_buf env;
    int i;
    int * iptr;

    FLUSHWIN();

    setjmp(env);
    iptr = (int*) env;

    sp = (struct frame *) iptr[FRAME_PTR_INDEX];

    for(i=0;i<SKIP_FRAMES && sp;i++)
	sp = sp->fr_savfp;

    i = 0;

    while(sp && sp->fr_savpc && ++i && (*func)((void*)sp->fr_savpc, usrarg)) {
	sp = sp->fr_savfp;
    }

    return i;
}

static int
PrintAddress(void *pc, void * usrarg)
{
    Dl_info info;
    char *func;
    char *lib;

    if (dladdr(pc, &info) == 0) {
	func = "??";
	lib = "??";
    } else {
	lib = (char *) info.dli_fname;
	func = (char *) info.dli_sname;
    }

    printf("(%s) %s +0x%x\n", 
	   lib, 
	   func,
	   (unsigned int)pc - (unsigned int)info.dli_saddr);

    return 1;
}

void
DumpStack()
{
    Error(EL_STATUS, "DumpStack", "+++++ STACK BACKTRACE FOLLOWS +++++", NULL);
    CallStack(PrintAddress, NULL);
}


#endif /* DUMP_STACK */
