// -*-c++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
//	Simple error handling package.
//
//	There's a nifty routine here called DumpStack() (courtesy
//	Bart Smaalders at Sun Engineering) that'll print out a stack
//	backtrace.  Enable with -DDUMP_STACK and link with -ldl.
//
//			keith edwards 31dec93
//
/************************************************************************** 
 * REVISION LOG 
 * 
 * $Author: keith $ 
 * $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Error.h,v $ 
 * $Revision: 6.5 $ 
 * $Date: 1995/12/04 20:39:46 $ 
 * $Log: Error.h,v $
 * Revision 6.5  1995/12/04  20:39:46  keith
 * Added DumpStack()
 *
 * Revision 6.4  1995/10/17  20:50:55  spine
 * Changed the prototypes of ErrorOutput to take an ErrorOutputs when using fd
 * or fstream. (The one that uses an fstream is probably not useful.)
 *
 * Revision 6.3  1995/10/12  20:11:46  spine
 * Overloaded the ErrorOutput method to accept a file descriptor and a fstream
 * to print output to.
 *
 * Revision 6.2  1995/10/10  15:42:53  spine
 * Added new overload of ErrorOutput to direct error output to a file stream.
 *
 * Revision 6.1  1995/07/28  20:32:50  spine
 * Added copyright notice.
 *
 * Revision 6.0  1995/05/20  22:20:21  beth
 * Mercator Version 6
 *
 * Revision 1.2  1995/04/19  14:55:05  iansmith
 * Added some #ifdefs to allow Error to be used by the multithreaded
 * code inside the AudioFile safely.
 *
 * Revision 1.1  1995/01/19  22:23:31  keith
 * Automatic global checkin
 *
// Revision 1.7  1994/09/28  18:12:48  keith
// Automatic global checkin
//
// Revision 1.6  1994/09/18  16:09:27  keith
// Automatic global checkin
//
// Revision 1.6  1994/09/18  16:09:27  keith
// Automatic global checkin
//
// Revision 1.5  1994/09/18  15:56:31  keith
// Automatic global checkin
//
// Revision 1.5  1994/09/18  15:56:31  keith
// Automatic global checkin
//
// Revision 1.4  1994/05/18  19:04:21  iansmith
// Added support for the FeatureSet and Naming conventions.
//
// Revision 1.3  1994/05/09  16:11:19  keith
// *** empty log message ***
//
 * 
 **************************************************************************/ 

#ifndef _ERROR_H_
#define _ERROR_H_

#include <stream.h>
#ifdef BEAM
#include <fstream.h>
#endif

typedef enum { 	EL_NO_MESSAGES   = 0,
		EL_ABORT	 = 1,
		EL_FATAL	 = 2,
		EL_ERROR	 = 3,
		EL_WARNING       = 4,
		EL_STATUS        = 5,
		EL_INFORMATIONAL = 6,
		EL_DEBUG 	 = 7,
		EL_ALL_MESSAGES  = 8
} ErrorLevels;

typedef enum {  EO_IGNORE	 = 0,
		EO_SYSLOG	 = 1,
		EO_STDERR	 = 2,
		EO_CONSOLE       = 3
} ErrorOutputs;

void Error(int level, const char *name ... );

#ifdef DUMP_STACK
void DumpStack();
#endif /* DUMP_STACK */

ErrorLevels ErrorLevel(void);
ErrorLevels ErrorLevel(ErrorLevels);

ErrorOutputs ErrorOutput(void);
ErrorOutputs ErrorOutput(ErrorOutputs);

// note that these two functions will set ErrorOutput to EO_SYSLOG.
ErrorOutputs ErrorOutput(ErrorOutputs err, int fd);
ErrorOutputs ErrorOutput(ErrorOutputs err, fstream *ostr);

const char *Perror(int);
const char *Perror(void);

#ifdef _JB_MT_SAFE_
struct __ErrorInit {
    __ErrorInit();
    ~__ErrorInit();
};

static __ErrorInit errorInitObj;
#endif /* _JB_MT_SAFE_ */

#ifdef MT_SAFE_OS_INTERFACE
struct __ErrorInit {
    __ErrorInit();
    ~__ErrorInit();
};
static __ErrorInit errorInitObj;
#endif /* MT_SAFE_OS_INTERFACE */


#endif /* _ERROR_H_ */
