//  -*-Mode: C++-*-
#ifndef __FDInterest_h__
#define __FDInterest_h__

//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
//
// FDInterest is our base class for dealing with I/O in Mercator.  Each
// FDInterest instance can track multiple descriptors and have them dispatched
// via the one almighty dispatch call, MainLoop(), which is maintained as a
// class static here.
//
// Each instance provides a HandleActivity method which will be invoked whenever
// activity appears on one of the descriptors it has expressed an interest in.
//
// A few things to note about this implementation:
//
//	- Each FDInterest instance can track multiple descriptors.
//	- No descriptor may be installed more than once.
//	- Multiple instances may not express interest in the same descriptor.
//
// We depend on the uniqueness of descriptors in this code to use the
// descriptor as an index into a couple of arrays that do our bookkeeping.
// 


//
// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/FDInterest.h,v $
// $Revision: 6.2 $
// $Date: 1995/07/28 20:33:34 $
/* $Log: FDInterest.h,v $
 * Revision 6.2  1995/07/28  20:33:34  spine
 * Added copyright notice.
 *
 * Revision 6.1  1995/07/01  20:03:30  spine
 * Removed all references to LEDA strings and LEDA.
 *
 * Revision 6.0  1995/05/20  22:20:21  beth
 * Mercator Version 6
 *
 * Revision 1.3  1995/04/01  19:40:38  keith
 * End of work by Keith
 *
 * Revision 1.2  1995/03/06  17:51:28  keith
 * *** empty log message ***
 *
 * Revision 1.1  1994/11/16  20:46:23  spine
 * Initial revision
 *
 * Revision 1.1  1994/05/24  19:49:25  stacy
 * Initial revision
 *
 * Revision 1.2  1994/04/06  20:56:20  kathryn
 * *** empty log message ***
 *
 * Revision 1.1  1994/04/06  19:52:54  kathryn
 * Initial revision
 *
*/

#ident "$Id: FDInterest.h,v 6.2 1995/07/28 20:33:34 spine Exp $"

struct pollfd;


class   FDInterest;
typedef FDInterest *FDInterestPtr;


int OpenTCPSocket(const char *name, int port);


class FDInterest {
private:
    //
    // We never create an FDInterest directly.
    //
    FDInterest();

    char			*fdiName;
    
    static int 			AddFDInterest(int fd, FDInterest *fdi);
    static int 			RemoveFDInterest(int fd, FDInterest *fdi);
    
    static int			inited;
    static struct pollfd	*fds;
    static FDInterest		**callbacks;	// interp as array of ptrs
    
    static int			maxFds;
    
    int				cleanup;
    static int			quit;
    
protected:
    FDInterest(const char *fdiName);

public:
    virtual ~FDInterest(void);

    static const short		ARead, AWrite, AExcept, AEverything;
    //
    // Mark the instance as ready for deletion the next time through
    // the loop.  THIS WAS PROTECTED BUT CLIENT NEEDS IT.
    //
    void		MarkForCleanup()
			    { cleanup = 1; }

    virtual int 	HandleActivity(int fd, short type) = 0;

    int 		AddFDInterest(int fd);
    int 		RemoveFDInterest(int fd);
    short		GetIOMask(int fd) const;
    void		SetIOMask(int fd, short iomask);

    // Called when MainLoop needs for an instance to shut itself down.
    virtual void	Shutdown(int fd);

    int			NeedsCleanupP() const
			    { return cleanup; }

    static void 	MainLoop();
    static void		Quit()		{ quit = 1; }
};

//
//
// **************************** Buffer Class ******************************
//  The Buffer class is intended to be used to read sized messages
//  across a socket.
//
class Buffer {
    char 		*data;
    //
    // Index of first empty byte.  Also number of bytes used.
    //
    int 		index;
    //
    // Number of bytes it will hold.
    //
    int			size;

public:
    Buffer();
    Buffer(int);       
    virtual ~Buffer();

    //
    // Returns non-zero if  packet is filled and there might be more to read
    int 		Read(int fd);
    void 		Write(int fd);

    void 		Clear();      	// empties the buffer
    int 		Full() const;	// returns true if the buffer is full.
    int 		Empty() const;	// is the buffer empty?
    void 		Resize(int);  	// enlarge the buffer
    void 		Join(const char *, int);

    int 		GetInt();  
    char 		*Data() const
			    { return data; }
    int 		Size() const
			    { return size; }
};

#endif __FDInterest_h__
