//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//	keith edwards	10aug95
//

#include <string.h>
#include "Error.h"
#include "FontCache.h"

///////////////////////////////////////////////////////////////////////////
//
// Define a closure which will zap the dictionary contents.
//
class FreeClosure : public mDictValPtrClosure<Font, XFontStruct> {
    int		action(Font f, XFontStruct *xfs);
};

inline int FreeClosure::action(Font f, XFontStruct *xfs)
{
    XFreeFontInfo(NULL, xfs, 1);
    return 1;
}

///////////////////////////////////////////////////////////////////////////
//
// FontCache stuff
//
FontCache::~FontCache()
{
    FreeClosure freeClosure;
    
    // The destructor frees all our fonts.
    fonts.walk(freeClosure, NULL);
}

XFontStruct *
FontCache::Get(Font fid) const
{
    XFontStruct *font = fonts.lookup(fid);

    if (font) return font;

    if ((font = XQueryFont(dpy, fid)) == NULL)
	return NULL;

    // Even though we're adding, we want this to look const.
    ((FontCache *) this)->Add(font);
    return font;
}

void
FontCache::Remove(Font fid)
{
    XFontStruct *font = fonts.lookup(fid);

    if (font) {
	fonts.del(fid);			// delete the dict node.
	XFreeFontInfo(NULL, font, 1);	// delete the font.
    }
}

int
FontCache::TextWidth(Font fid, const char *str) const
{
    XFontStruct *font = Get(fid);

    if (!font)
	return -1;

    return XTextWidth(font, str, strlen(str));
}

int
FontCache::TextWidth(Font fid, const XChar2b *str) const
{
    Error(EL_ERROR, "FontCache::TextWidth", "16-bit text unsupported.", NULL);
    return -1;
}

int
FontCache::TextExtents(Font fid, const char *str,
		       int& directionReturn,
		       int& fontAscentReturn, int& fontDescentReturn,
		       XCharStruct& overallReturn) const
{
    XFontStruct *font = Get(fid);

    if (!font)
	return -1;

    return XTextExtents(font, str, strlen(str), &directionReturn,
			&fontAscentReturn, &fontDescentReturn,
			&overallReturn);
}

int
FontCache::TextExtents(Font fid, const XChar2b *str,
		       int& directionReturn,
		       int& fontAscentReturn, int& fontDescentReturn,
		       XCharStruct& overallReturn) const
{
    Error(EL_ERROR, "FontCache::TextExtents", "16-bit text unsupported.",
	  NULL);
    return -1;
}
