//  -*-Mode: C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
//
// $Author: kchen $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/GCValue.cc,v $
// $Revision: 6.9 $
// $Date: 1995/10/30 14:39:11 $
// $Log: GCValue.cc,v $
// Revision 6.9  1995/10/30  14:39:11  kchen
// fixed return error in equality comparison operator
//
// Revision 6.8  1995/10/21  22:12:03  keith
// Fixed a nasty sizing bug in ChangeGCRequest.
//
// Revision 6.7  1995/09/14  15:23:27  kchen
// Added comparison operator for GCValues
//
// Revision 6.6  1995/09/04  20:24:21  keith
// Changed copy ctor, assignment.
//
// Revision 6.5  1995/08/13  21:01:29  keith
// got rid of default ctor in favor of version with default parameter.
//
// Revision 6.4  1995/08/13  18:54:46  keith
// Cleaned up the code a bit to make it easier to use with some of the
// TextRep stuff I'm doing.
//
// Revision 6.3  1995/07/28  20:33:59  spine
// Added copyright notice.
//
// Revision 6.2  1995/06/08  12:48:48  iansmith
// Remove a dictionary macro (LEDA) which was dead code.
//
// Revision 6.1  1995/06/06  01:02:55  suelong
// added new constructor for GCValue - to copy gcvalues struct
// from rap message into GCValue object
//
// Revision 6.0  1995/05/20  22:20:21  beth
// Mercator Version 6
//
// Revision 1.3  1995/03/06  17:52:05  keith
// *** empty log message ***
//
// Revision 1.2  1995/01/19  22:22:30  keith
// Automatic global checkin
//
// Revision 1.2  1995/01/19  22:22:30  keith
// Automatic global checkin
//
// Revision 1.1  1994/11/16  20:46:23  spine
// Initial revision
//
// Revision 1.2  1994/08/12  22:46:11  kathryn
// update
//
// Revision 1.1  1994/05/24  19:49:25  stacy
// Initial revision
//
// Revision 1.1  1994/04/06  19:54:48  kathryn
// Initial revision
//
//
 
static const char rcsid[]="@(#) $Header: /net/hm10/keith/proj/mercator/6.0/RCS/GCValue.cc,v 6.9 1995/10/30 14:39:11 kchen Exp $";

#ifndef TRUE
#define TRUE  1
#endif /* TRUE */

#ifndef FALSE
#define FALSE 0
#endif /* FALSE */

#include "GCValue.h"
#ifdef BEAM
#include <rw/regexp.h>
#endif
#include "Error.h"

//
// these are the initial GC values
// stolen from src/mit/lib/X/XCrGC.c by jack
//
XGCValues GCValue::initial_GC = {
    GXcopy, 	/* function */
    AllPlanes,	/* plane_mask */
    0L,		/* foreground */
    1L,		/* background */
    0,		/* line_width */
    LineSolid,	/* line_style */
    CapButt,	/* cap_style */
    JoinMiter,	/* join_style */
    FillSolid,	/* fill_style */
    EvenOddRule,/* fill_rule */
    ArcPieSlice,/* arc_mode */
    (Pixmap)~0L,/* tile, impossible (unknown) resource */
    (Pixmap)~0L,/* stipple, impossible (unknown) resource */
    0,		/* ts_x_origin */
    0,		/* ts_y_origin */
    (Font)~0L,	/* font, impossible (unknown) resource */
    ClipByChildren, /* subwindow_mode */
    True,	/* graphics_exposures */
    0,		/* clip_x_origin */
    0,		/* clip_y_origin */
    None,	/* clip_mask */
    0,		/* dash_offset */
    4		/* dashes (list [4,4]) */
};


unsigned long GCValue::compflags = ~0;

// ************************************************************************
// **                     Class Special Functions                    ******
// ************************************************************************
//
//
// create a new initialized GCValue.  Should we check to see if one with
// the same id exists?  hmmm....
//
GCValue::GCValue(const GCValue& other)
{
    gid = other.gid;

    values.function    = other.values.function;
    values.plane_mask  = other.values.plane_mask;
    values.foreground  = other.values.foreground;
    values.background  = other.values.background;
    values.line_width  = other.values.line_width;
    values.line_style  = other.values.line_style;
    values.cap_style   = other.values.cap_style;
    values.join_style  = other.values.join_style;
    values.fill_style  = other.values.fill_style;
    values.fill_rule   = other.values.fill_rule;
    values.arc_mode    = other.values.arc_mode;
    values.tile        = other.values.tile;
    values.stipple     = other.values.stipple;
    values.ts_x_origin = other.values.ts_x_origin;
    values.ts_y_origin = other.values.ts_y_origin;
    values.font               = other.values.font;
    values.subwindow_mode     = other.values.subwindow_mode;
    values.graphics_exposures = other.values.graphics_exposures;
    values.clip_x_origin      = other.values.clip_x_origin;
    values.clip_y_origin      = other.values.clip_y_origin;
    values.clip_mask          = other.values.clip_mask;
    values.dash_offset        = other.values.dash_offset;
}

GCValue::GCValue(const GCValues& gc_info)
{
    // start with default values
    values = initial_GC;
    // get most of the values from rap reply
    gid = gc_info.gc;
    values.function    = gc_info.data.gcv.function;
    values.plane_mask  = gc_info.data.gcv.plane_mask;
    values.foreground  = gc_info.data.gcv.foreground;
    values.background  = gc_info.data.gcv.background;
    values.line_width  = gc_info.data.gcv.line_width;
    values.line_style  = gc_info.data.gcv.line_style;
    values.cap_style   = gc_info.data.gcv.cap_style;
    values.join_style  = gc_info.data.gcv.join_style;
    values.fill_style  = gc_info.data.gcv.fill_style;
    values.fill_rule   = gc_info.data.gcv.fill_rule;
    values.arc_mode    = gc_info.data.gcv.arc_mode;
    values.tile       = gc_info.data.gcv.tile;
    values.stipple     = gc_info.data.gcv.stipple;
    values.ts_x_origin = gc_info.data.gcv.ts_x_origin;
    values.ts_y_origin = gc_info.data.gcv.ts_y_origin;
 
    // font is the important one!
    values.font               = gc_info.data.gcv.font;
    values.subwindow_mode     = gc_info.data.gcv.subwindow_mode;
    values.graphics_exposures = gc_info.data.gcv.graphics_exposures;
    values.clip_x_origin      = gc_info.data.gcv.clip_x_origin;
    values.clip_y_origin      = gc_info.data.gcv.clip_y_origin;
    values.clip_mask          = gc_info.data.gcv.clip_mask;
    values.dash_offset        = gc_info.data.gcv.dash_offset;
}

GCValue&
GCValue::operator=(const GCValue& other)
{
    gid = other.gid;

    values.function    = other.values.function;
    values.plane_mask  = other.values.plane_mask;
    values.foreground  = other.values.foreground;
    values.background  = other.values.background;
    values.line_width  = other.values.line_width;
    values.line_style  = other.values.line_style;
    values.cap_style   = other.values.cap_style;
    values.join_style  = other.values.join_style;
    values.fill_style  = other.values.fill_style;
    values.fill_rule   = other.values.fill_rule;
    values.arc_mode    = other.values.arc_mode;
    values.tile        = other.values.tile;
    values.stipple     = other.values.stipple;
    values.ts_x_origin = other.values.ts_x_origin;
    values.ts_y_origin = other.values.ts_y_origin;
    values.font               = other.values.font;
    values.subwindow_mode     = other.values.subwindow_mode;
    values.graphics_exposures = other.values.graphics_exposures;
    values.clip_x_origin      = other.values.clip_x_origin;
    values.clip_y_origin      = other.values.clip_y_origin;
    values.clip_mask          = other.values.clip_mask;
    values.dash_offset        = other.values.dash_offset;

    return *this;
}

// ************************************************************************

GCValue::GCValue(GContext new_gid) :
    values(initial_GC),
    gid(new_gid)
{
}

// ************************************************************************

void
GCValue::ChangeGC(unsigned long mask, XID *pval)
{
    int bit, error = 0;
    unsigned long index;
    char tmp1[16], tmp2[16];

    // this is basically code stolen from the X server dix layer.
    for (bit = 0; mask && !error ; bit++) {
	index = 1 << bit;
	if (!(index & mask))
	    continue;

	mask &= ~index;
	switch (index) {
        case GCFunction:
	    if (((CARD8)*pval >= GXclear) && ((CARD8)*pval <= GXset)) {
	        this->values.function = (CARD8)*pval;
	    } else {
		error = BadValue;
	    }
	    pval++;
	    break;
	case GCPlaneMask:
	    this->values.plane_mask = *pval;
	    pval++;
	    break;
	case GCForeground:
	    this->values.foreground = *pval;
	    pval++;
#ifdef notdef
	    // i'm not sure what to do with this code.  it contains
	    // state that doesn't exist anywhere but in the server.
	    /*
	     * this is for CreateGC
	     */
	    if (!this->values.tileIsPixel && !this->values.tile.pixmap) {
		this->values.tileIsPixel = TRUE;
		this->values.tile.pixel = this->values.fgPixel;
	    }
#endif /* notdef */
	    break;
	case GCBackground:
	    this->values.background = *pval;
	    pval++;
	    break;
	case GCLineWidth:		/* ??? line width is a CARD16 */
	    this->values.line_width = (CARD16)*pval;
	    pval++;
	    break;
	case GCLineStyle:
	    if (((CARD8)*pval >= LineSolid) 
		&& ((CARD8)*pval <= LineDoubleDash)) {
		this->values.line_style = (CARD8)*pval;
	    } else {
		error = BadValue;
	    }
	    pval++;
	    break;
	case GCCapStyle:
	    if (((CARD8)*pval >= CapNotLast) 
		&& ((CARD8)*pval <= CapProjecting)) {
		this->values.cap_style = (CARD8)*pval;
	    } else {
		error = BadValue;
	    }
	    pval++;
	    break;
	case GCJoinStyle:
	    if (((CARD8)*pval >= JoinMiter) && ((CARD8)*pval <= JoinBevel)) {
		this->values.join_style = (CARD8)*pval;
	    } else {
		error = BadValue;
	    }
	    pval++;
	    break;
	case GCFillStyle:
	    if (((CARD8)*pval >= FillSolid) 
		&& ((CARD8)*pval <= FillOpaqueStippled)) {
		this->values.fill_style = (CARD8)*pval;
	    } else {
		error = BadValue;
	    }
	    pval++;
	    break;
	case GCFillRule:
	    if (((CARD8)*pval >= EvenOddRule) && 
		((CARD8)*pval <= WindingRule)) {
		this->values.fill_rule = (CARD8)*pval;
	    } else {
		error = BadValue;
	    }
	    pval++;
	    break;
	case GCTile:
	    this->values.tile = (Pixmap) *pval;
	    pval++;
	    break;
	case GCStipple:
	    this->values.stipple = (Pixmap) *pval;
	    pval++;
	    break;
	case GCTileStipXOrigin:
	    this->values.ts_x_origin = (INT16)*pval;
	    pval++;
	    break;
	case GCTileStipYOrigin:
	    this->values.ts_y_origin = (INT16)*pval;
	    pval++;
	    break;
	case GCFont:
	    this->values.font = (Font) *pval;
	    pval++;
	    break;
	case GCSubwindowMode:
	    if (((CARD8)*pval == ClipByChildren) ||
		((CARD8)*pval == IncludeInferiors)) {
		this->values.subwindow_mode = (CARD8)*pval;
	    } else {
		error = BadValue;
	    }
	    pval++;
	    break;
	case GCGraphicsExposures:
	    if ((Bool)*pval == xFalse) {
		this->values.graphics_exposures = FALSE;
	    } else if ((Bool)*pval == xTrue) {
		this->values.graphics_exposures = TRUE;
	    } else {
		error = BadValue;
	    }
	    pval++;
	    break;
	case GCClipXOrigin:
	    this->values.clip_x_origin = (INT16)(*pval);
	    pval++;
	    break;
	case GCClipYOrigin:
	    this->values.clip_y_origin = (INT16)(*pval);
	    pval++;
	    break;
	case GCClipMask:
	    break;
#ifdef notdef
	    {
		Pixmap pid;
		int	clipType;
	    
		pid = (Pixmap) *pval;
		if (pid == None) {
		    clipType = CT_NONE;
		} else {
		    if(fPointer) {
			pPixmap = (PixmapPtr) *pval;
		    } else {
			pPixmap = (PixmapPtr)LookupIDByType(pid, RT_PIXMAP);
		    }
		    if (pPixmap) {
			if ((pPixmap->drawable.depth != 1) ||
			    (pPixmap->drawable.pScreen != this->values.pScreen)) {
			    error = BadMatch;
			} else {
			    clipType = CT_PIXMAP;
			    pPixmap->refcnt++;
			}
		    } else {
			clientErrorValue = pid;
			error = BadPixmap;
		    }
		}
		pval++;
		if(error == Success) {
		    (*this->values.funcs->ChangeClip)(pGC, clipType, pPixmap, 0);
		}
		break;
	    }
#endif /* notdef */
	case GCDashOffset:
	    this->values.dash_offset = (CARD16)*pval;
	    pval++;
	    break;
	case GCDashList:
	    this->values.dashes = (char) *pval++;
	    break;
	case GCArcMode:
	    if (((CARD8)*pval >= ArcChord) 
		&& ((CARD8)*pval <= ArcPieSlice)) {
		this->values.arc_mode = (CARD8)*pval;
	    } else {
		error = BadValue;
	    }
	    pval++;
	    break;
	default:
	    error = BadValue;
	    pval++;
	    break;
	}
    }

    //
    // i'm not really sure what the proper behaviour is when we see an error
    // that's on it's way to the server.  There are a lot of potential
    // problems here.  for now, we just let someone know.
    //
    if (error) {
	Error(EL_ERROR, "GCValue::ChangeGC", "Got an error in handling GC", 
	      NULL);
    }
}

int 
GCValue::operator==(const GCValue &comp) {
  if (compflags & GC_COMP_FUNCTION) {
    if (values.function != comp.values.function)
      return 0;
  }
  if (compflags & GC_COMP_PLANE_MASK) {
    if (values.plane_mask != comp.values.plane_mask)
      return 0;
  }
  if (compflags & GC_COMP_FOREGROUND) {
    if (values.foreground != comp.values.foreground)
      return 0;
  }
  if (compflags & GC_COMP_BACKGROUND) {
    if (values.background != comp.values.background)
      return 0;
  }
  if (compflags & GC_COMP_LINE_WIDTH) {
    if (values.line_width != comp.values.line_width)
      return 0;
  }
  if (compflags & GC_COMP_LINE_STYLE) {
    if (values.line_style != comp.values.line_style)
      return 0;
  }
  if (compflags & GC_COMP_CAP_STYLE) {
    if (values.cap_style != comp.values.cap_style)
      return 0;
  }
  if (compflags & GC_COMP_JOIN_STYLE) {
    if (values.join_style != comp.values.join_style)
      return 0;
  }
  if (compflags & GC_COMP_FILL_STYLE) {
    if (values.fill_style != comp.values.fill_style)
      return 0;
  }
  if (compflags & GC_COMP_FILL_RULE) {
    if (values.fill_rule != comp.values.fill_rule)
      return 0;
  }
  if (compflags & GC_COMP_ARC_MODE) {
    if (values.arc_mode != comp.values.arc_mode)
      return 0;
  }
  if (compflags & GC_COMP_TS_X_ORIGIN) {
    if (values.ts_x_origin != comp.values.ts_x_origin)
      return 0;
  }
  if (compflags & GC_COMP_TS_Y_ORIGIN) {
    if (values.ts_y_origin != comp.values.ts_y_origin)
      return 0;
  }
  if (compflags & GC_COMP_SUBWINDOW_MODE) {
    if (values.subwindow_mode != comp.values.subwindow_mode)
      return 0;
  }
  if (compflags & GC_COMP_TS_Y_ORIGIN) {
    if (values.ts_y_origin != comp.values.ts_y_origin)
      return 0;
  }
  if (compflags & GC_COMP_CLIP_X_ORIGIN) {
    if (values.clip_x_origin != comp.values.clip_x_origin)
      return 0;
  }
  if (compflags & GC_COMP_CLIP_Y_ORIGIN) {
    if (values.clip_y_origin != comp.values.clip_y_origin)
      return 0;
  }
  if (compflags & GC_COMP_TS_X_ORIGIN) {
    if (values.ts_x_origin != comp.values.ts_x_origin)
      return 0;
  }
  if (compflags & GC_COMP_DASH_OFFSET) {
    if (values.dash_offset != comp.values.dash_offset)
      return 0;
  }
  if (compflags & GC_COMP_DASHES) {
    if (values.dashes != comp.values.dashes)
      return 0;
  }
  return 1;
}

int
GCValue::operator!=(const GCValue &comp) {
  return !(*this == comp);
}
