// -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#ifndef __GCValue_h__
#define __GCValue_h__

//
//  GC = Graphical Content
//  this class implements the GC cache.  Since we can't find out values
//  from the server, we have to keep the ourselves.  You use
//  InstallGCHandlers to set up the protocol handlers and then you
//  can just use the class methods to get a hold of GCValues and get
//  their values.  The code hasn't fully been tested yet, so use
//  carefully.
//
// Tom Rodriguez
//

#ident "$Id: GCValue.h,v 6.10 1995/10/30 14:39:11 kchen Exp $"

#define NEED_EVENTS
#define NEED_REPLIES

#include <X11/Xlib.h>
#include <X11/Xproto.h>
#include "mDictValPtr.h"

extern "C"
{
#include "agent.h"
}

class GCValue;
typedef GCValue *GCValuePtr;

//
// dictionary for looking up GCs
//
typedef mDictValPtr<GContext,GCValue> GCValueDict;

class GCValue {

    friend class RAP;

public:
    enum gc_compflags { GC_COMP_FUNCTION = 1<<0, GC_COMP_PLANE_MASK = 1<<1, 
			GC_COMP_FOREGROUND = 1<<2, GC_COMP_BACKGROUND = 1<<3,
			GC_COMP_LINE_WIDTH = 1<<4, GC_COMP_LINE_STYLE = 1<<5,
			GC_COMP_CAP_STYLE = 1<<6, GC_COMP_JOIN_STYLE = 1<<7,
			GC_COMP_FILL_STYLE= 1<<8, GC_COMP_FILL_RULE = 1<<9,
			GC_COMP_ARC_MODE= 1<<10 , GC_COMP_TILE = 1<<11,
			GC_COMP_STIPPLE = 1<<12, GC_COMP_TS_X_ORIGIN = 1<<13,
			GC_COMP_TS_Y_ORIGIN = 1<<14, GC_COMP_FONT = 1<<15,
			GC_COMP_SUBWINDOW_MODE = 1<<16, 
			GC_GRAPHICS_EXPOSURES = 1<<17,
			GC_COMP_CLIP_X_ORIGIN = 1<<18, 
			GC_COMP_CLIP_Y_ORIGIN = 1<<19,
			GC_COMP_CLIP_MASK = 1<<20, GC_COMP_DASH_OFFSET = 1<<21,
			GC_COMP_DASHES = 1<<22 };
    GCValue(const GCValue& other);	// copy ctor
    GCValue(const GCValues& gcinfo);	// from struct
    GCValue(GContext new_gid = 0);

    GCValue&		operator=(const GCValue& other);
    
    Font 		GetFontID() const;	// get the font used by a GC
    void 		SetFontID(Font);	// set the Font in a GC

    GContext		GetGID() const;

    operator XGCValues();
    operator const XGCValues() const;
    int operator==(const GCValue &comp);
    int operator!=(const GCValue &comp);
    static unsigned long GetCompFlags(void) { return compflags; }
    static void SetCompFlags(unsigned long newvals) { compflags = newvals; }
    static void AddCompFlags(unsigned long newvals) { compflags |= newvals; }
    static void RemoveCompFlags(unsigned long newvals) { 
      compflags &= ~newvals; }

private:    
    // default initial values for GCs.
    static XGCValues 	initial_GC;
    
    // internal utility for changing GCs
    void 		ChangeGC(unsigned long mask, XID *pval);

    XGCValues 		values;  // actual state of state
    GContext  		gid;     // XID used to reference it.
    static unsigned long compflags;
};

inline GCValue::operator XGCValues() { return values; }
inline GCValue::operator const XGCValues() const { return values; }

inline Font GCValue::GetFontID() const { return values.font; }
inline void GCValue::SetFontID(Font f) { values.font = f; }

inline GContext GCValue::GetGID() const { return gid; } 

#endif __GCValue_h__
