//  -*-Mode: C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

//
// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Interp.cc,v $
// $Revision: 6.66 $
// $Date: 1995/12/07 19:52:20 $
//
 
static const char rcsid[]="@(#) $Header: /net/hm10/keith/proj/mercator/6.0/RCS/Interp.cc,v 6.66 1995/12/07 19:52:20 keith Exp $";


// ************************************************************************
// include stuff
// ************************************************************************

#include <strstream.h>

#include "Interp.h"
#include "Config.h"
#include "Client.h"
#include "Speech.h"
#include "Audio.h"
#include "ScreenReader.h"
#include "RAP.h"
#include "Braille.h"
#include "RAPListener.h"
#include "TclAction.h"
#include "AppModelMgr.h"
#include "Error.h"
#include "Util.h"
#include "Template.h"
#include "Attribute.h"
#include <iostream.h>
#include <stdlib.h>
#include <unistd.h>
#include <values.h>		// for MAXLONG
#include <sys/types.h>
#include <stropts.h>
#include <sys/conf.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/time.h>
#include <X11/X.h>
#include <ctype.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xaw/Text.h>
#include <errno.h>
#include "mString.h"

#ifdef BEAM
struct timespec {
	long	tv_sec;
	long	tv_nsec;
};
#undef NULL
#define NULL	0
#endif

/* Global variable */
static Tcl_Interp * default_interp = NULL;
struct timeval start;

// static
FILE *Interp::outFile = NULL;


#ifndef BEAM
declare2(dictionary,Object,XtObjectPtr);
#endif

// ************************************************************************
// TCL stuff 
// ************************************************************************

int bindEvent(ClientData, Tcl_Interp *, int, char *[]);
int callAction(ClientData, Tcl_Interp *, int, char *[]);
int playSound(ClientData, Tcl_Interp *, int, char *[]);
int addAction(ClientData, Tcl_Interp *, int, char *[]); 
int modelManager(ClientData, Tcl_Interp *, int, char *[]);
int currentObject(ClientData, Tcl_Interp *, int, char *[]);
int currentClient(ClientData, Tcl_Interp *, int, char *[]);
int advanceClient(ClientData, Tcl_Interp *, int, char *[]);
int backupClient(ClientData, Tcl_Interp *, int, char *[]);
int setFocus(ClientData, Tcl_Interp *, int, char *[]);
int button(ClientData, Tcl_Interp *, int, char *[]);
int key(ClientData, Tcl_Interp *, int, char *[]);
int relativemotion(ClientData, Tcl_Interp *, int, char *[]);
int speak(ClientData, Tcl_Interp *, int, char *[]);
int sreader(ClientData, Tcl_Interp *, int, char *[]);
int resource(ClientData, Tcl_Interp *, int, char *[]);
int xwin(ClientData, Tcl_Interp *, int, char *[]);
int long2short(ClientData, Tcl_Interp *, int, char *[]);
int short2long(ClientData, Tcl_Interp *, int, char *[]);
int connect(ClientData, Tcl_Interp *, int, char *[]);
int getTextVertCoord(ClientData, Tcl_Interp *, int, char *[]);
int getTextHorizCoord(ClientData, Tcl_Interp *, int, char *[]);
int logtofile(ClientData, Tcl_Interp *, int, char *[]);
int StartTimer(ClientData, Tcl_Interp *, int, char *[]);
int quit(ClientData, Tcl_Interp *, int, char *[]);
int wtemplate(ClientData, Tcl_Interp *, int, char *[]);
int stopAllSounds(ClientData, Tcl_Interp *, int, char *[]);
int client(ClientData, Tcl_Interp *, int, char *[]);
int braille(ClientData, Tcl_Interp *, int, char *[]);
int configVal(ClientData, Tcl_Interp *, int, char *[]);
int disconnectPipe(ClientData, Tcl_Interp *, int, char *[]);
int connectPipe(ClientData, Tcl_Interp *, int, char *[]);
int LoadTemplates(ClientData, Tcl_Interp *, int, char*[]);
int LoadAttributes(ClientData, Tcl_Interp *, int, char*[]);
int LoadConfig(ClientData, Tcl_Interp *, int, char*[]);
int LoadAll(ClientData, Tcl_Interp *, int, char*[]);
int WriteOut(ClientData, Tcl_Interp *, int, char*[]);
int ErrorShell(ClientData, Tcl_Interp *, int, char*[]);
int audio(ClientData, Tcl_Interp *, int, char*[]);

#ifdef TEXTREP_DEBUG
int textrepdebug(ClientData, Tcl_Interp *, int, char *[]);
#endif

typedef struct {
    char * name;
    int  (*func)(ClientData, Tcl_Interp *, int, char *[]);
} TclFunc;

static TclFunc funclist[] = {
    { "bindevent", bindEvent },
    { "callaction", callAction },
    { "playsound", playSound },
    { "addaction", addAction },
    { "setFocus", setFocus },
    { "currentObject", currentObject },
    { "currentClient", currentClient },
    { "advanceClient", advanceClient },
    { "backupClient", backupClient },
    { "modelManager", modelManager },
    { "button", button },
    { "key", key },
    { "relativemotion", relativemotion },
    { "speak", speak },
    { "sreader", sreader },
    { "resource", resource },
    { "xwin", xwin },
    { "long2short", long2short },
    { "short2long", short2long },
    { "connect", connect },
    { "getTextVertCoord", getTextVertCoord },
    { "getTextHorizCoord", getTextHorizCoord },
    { "StartTimer", StartTimer },
    { "logtofile", logtofile },
    { "quit", quit },
    { "wtemplate", wtemplate },
    { "stopAllSounds", stopAllSounds },
    { "client", client },
    { "braille", braille },
    { "configVal", configVal },
    { "disconnectPipe", disconnectPipe },
    { "connectPipe", connectPipe },
    { "loadconfig", LoadConfig, },
    { "loadattrib", LoadAttributes },
    { "loadtemplates", LoadTemplates},
    { "loadall", LoadAll },
    { "writeout", WriteOut},
    { "Error", ErrorShell},
    { "audio", audio}
#ifdef TEXTREP_DEBUG
    , { "textrepdebug", textrepdebug }
#endif
};

// ************************************************************************

static Tcl_Interp *
NewInterp()
{
    Tcl_Interp * interp;
    int i;

    interp = Tcl_CreateInterp();

    for (i = 0; i < XtNumber(funclist); i++) 
	Tcl_CreateCommand(interp, funclist[i].name,
			  funclist[i].func, (ClientData) mercator,
			  (void (*)(ClientData)) NULL);

    return interp;
}

// ************************************************************************
// **                     Class Special Functions                    ******
// ************************************************************************
Interp::Interp(int in, int out, Tcl_Interp * interp) :
    FDInterest("Interp"),
    infd(in),
    outfd(out)
{
    if (interp == NULL) {
	if (default_interp == NULL)
	    default_interp = NewInterp();
	interpreter = default_interp;
    } else {
	interpreter = interp;
    }

#ifdef TCL7 
    Tcl_DStringInit(&cmdbuf);
#else
    cmdbuf = Tcl_CreateCmdBuf();
#endif

    SetDescriptors(in, out);
}

// ************************************************************************

Interp::~Interp(void)
{
    RemoveFDInterest(Interp::infd);
    
#ifdef TCL7
    Tcl_DStringFree(&cmdbuf);
#else
    Tcl_DeleteCmdBuf(cmdbuf);
#endif
    
    if (interpreter)
	Tcl_DeleteInterp(interpreter);
}

// ************************************************************************
// **                          Public Functions                      ******
// ************************************************************************

void
Interp::SetDescriptors(int in, int out)
{
   // Note that Interp never opened the descriptors it gets, so it
   // never closes them either.

   RemoveFDInterest(Interp::infd);

   if((Interp::outFile) && (Interp::outFile != stdout)) {
      if(fclose(outFile) == EOF) {
	 Error(EL_ERROR, "Interp::SetDescriptors", "fclose ", Perror(), NULL);
      }
   }

   Interp::infd = in;
   Interp::outfd = out;

   if((Interp::infd != 0) && !(Interp::outFile = fdopen(Interp::outfd, "w"))) {
       Error(EL_ERROR, "Interp::SetDescriptors", "fdopen ", Perror(), NULL);
   }

   AddFDInterest(Interp::infd);
   SetIOMask(Interp::infd, ARead);

   char tmp[50];
   sprintf(tmp, "infd: %d, outfd: %d", Interp::infd, Interp::outfd); 
   Error(EL_DEBUG, "Interp::SetDescriptors", tmp, NULL);
}

// ************************************************************************

int
Interp::SourceInitialTCLfile()
{
      // prepend a path to the mercator.tcl script.
    char sourceCommand[1024];
    //sprintf(sourceCommand, "source %s/mercator.tcl", Config::MercatorTclPath());
    sprintf(sourceCommand, "source %s/mercator.tcl",
	    Config::GetString("mercatorTclPath"));
    
    Tcl_GlobalEval(interpreter, sourceCommand);

    // give em a prompt.
    // write(outfd, "\nmercator> ", 11);
    fflush(outFile);
    
    return 1;
}

// ************************************************************************

int
Interp::HandleActivity(int desc, short type)
{
    char buffer[1024];
#ifndef TCL7
    char * cmd;
#endif
 
    if (type & ARead) {
	int len;
 
	len = read(Interp::infd, buffer, 1023);
	buffer[len] = '\0';
 
#ifdef TCL7
 
	Tcl_DStringAppend(&cmdbuf, buffer, -1);
	if (!Tcl_CommandComplete(cmdbuf.string))
	    return 0;
	Tcl_GlobalEval(interpreter, cmdbuf.string);
	Tcl_DStringFree(&cmdbuf);
	Tcl_DStringInit(&cmdbuf);
 
#else
 
	cmd = Tcl_AssembleCmd(cmdbuf, buffer);
	if (cmd == NULL)
	    return 0;
	Tcl_GlobalEval(interpreter, cmd);
 
#endif
 
	if (*interpreter->result != NULL) {
	    write(Interp::outfd, interpreter->result,
		  strlen(interpreter->result));
	}

	// only print a prompt if we don't have a console application
	if(Interp::outfd == 1)
	    write(Interp::outfd, "\nmercator> ", 11);

	fflush(outFile);
	
    } else if (type & AExcept) {
	if (desc != 0) {
	    Error(EL_WARNING, "Interp::HandleActivity",
	        "Console has gone away?  Using stdin/stdout", NULL);

	    /* mercator->RemoveNamedPipes(); */

	    SetDescriptors(0, 1);
	}
    } else {
	Error(EL_ERROR, "Interp::HandleActivity",
	      "Why am I being called!?", NULL);
    }

    return 0;
}

int
Interp::DisconnectPipe()
{
    int ret = 0;
    
    if (Interp::infd > 1) {
	if (close(Interp::infd) == -1) {
	    Error(EL_ERROR, "Interp::DisconnectPipe", Perror(), NULL);
	    ret = -1;
	}
    }

    if(Interp::outfd > 1) {
	if (close(Interp::outfd) == -1) {
	    Error(EL_ERROR, "Interp::DisconnectPipe", Perror(), NULL);
	    ret = -1;
	}
    }

    SetDescriptors(0, 1);

    return ret;
}

int
Interp::ConnectPipe()
{
    int tmpin, tmpout;
    int ret = 0;

    if ((tmpout = open("/tmp/to-console", O_WRONLY)) < 0) {
	if (errno != ENXIO) {
	    Error(EL_ERROR, "Interp::ConnectPipe",
	          "Could not open to-console fifo: ", Perror(), NULL);
	    tmpout = 1;
	    ret = -1;
	}
    }

    if ((tmpin = open("/tmp/from-console", O_RDONLY)) < 0) {
	Error(EL_ERROR, "Interp::ConnectPipe",
	      "Could not open from-console fifo: ", Perror(), NULL);
	tmpin = 0;
	ret = -1;
    }
    
    Error(EL_DEBUG, "Interp::ConnectPipe", "pipes opened", NULL);

    SetDescriptors(tmpin, tmpout);

    return ret;
}

// ************************************************************************
// TCL functions
// ************************************************************************

#define CheckArgcIs(expr)\
if (!(expr)) { Tcl_AppendResult(interp, "Wrong # of args", NULL);\
return TCL_ERROR;}

// ************************************************************************

int
StartTimer(ClientData data, Tcl_Interp *interp, int argc, char *argv[])
{
    FILE *fp;
    int i = 2;


    if ((fp = fopen(argv[1], "a+")) == NULL) {
	perror(argv[1]);
	return 0;
    }

    gettimeofday(&start, NULL);

    fprintf(fp, "%ld:%ld", start.tv_sec, start.tv_usec);

    for (i=2 ; i<argc ; i++) {
	fprintf(fp, "\n        ");
	fprintf(fp, argv[i]);
    }
    fprintf(fp, "\n");
    fprintf(fp, "\n");
    fflush(fp);

    fclose(fp);
    return TCL_OK;
}

// ************************************************************************

int
logtofile(ClientData data, Tcl_Interp *interp, int argc, char *argv[])
{
    FILE *fp;
    struct timeval now;
    struct timeval *time;
    int i = 2;

    if (argv[1] == "NULL") {
	return 0;
    } else {
        if ((fp = fopen(argv[1], "a+")) == NULL) {
	    perror(argv[1]);
	    return 0;
        }
    
        gettimeofday(&now, NULL);
    
        if (now.tv_usec < start.tv_usec) {
            now.tv_usec = now.tv_usec + 1000000;
            now.tv_sec = now.tv_sec - 1;
        }
#if 0
        time->tv_sec = now.tv_sec - start.tv_sec;
        time->tv_usec = now.tv_usec - start.tv_usec;
#endif
    
    
        fprintf(fp, "%ld:%ld",now.tv_sec - start.tv_sec,now.tv_usec - start.tv_usec);
    
        for (i=2 ; i<argc ; i++) {
	    fprintf(fp, "\n        ");
	    fprintf(fp, argv[i]);
        }
        fprintf(fp, "\n");
        fprintf(fp, "\n");
        fflush(fp);
    
        fclose(fp);
        return TCL_OK;
    }
}

// ************************************************************************

int
quit(ClientData, Tcl_Interp *, int, char *[])
{
    Error(EL_STATUS, "Interp::quit", "Shutting down mercator at user request.",
	  NULL);
    mercator->Quit();
    return TCL_OK;
}

// ************************************************************************

int
wtemplate(ClientData, Tcl_Interp *interp, int argc, char *argv[])
{
    XtObject *obj = NULL;
    
    if (argc != 3) {
	Tcl_AppendResult(interp, "Usage:  ", argv[0], " object attribute",
			 NULL);
	return TCL_ERROR;
    }
    
    obj = mercator->GetAppModelMgr()->GetXtObject(mString(argv[1]));

    if (obj == NULL) {
	Tcl_AppendResult(interp, argv[0], ": no such object", NULL);
	return TCL_ERROR;
    }

    const FieldData *data = Template::FindByName(*obj, argv[2]);

    if (data == NULL) {
	Tcl_AppendResult(interp, argv[0], ": No such attribute: ",
			 argv[2], NULL);
	return TCL_ERROR;
    }

    switch (data->Tag()) {
    case BOOLEAN:
	if (data->Boolean(*obj))
	    Tcl_Return(interp, "TRUE", TCL_STATIC);
	else
	    Tcl_Return(interp, "FALSE", TCL_STATIC);
	return TCL_OK;
	
    case INTEGER:
	// Can we use a local?
	char tmp[16];
	sprintf(tmp, "%d", data->Integer(*obj));
	Tcl_Return(interp, tmp, TCL_VOLATILE);
	return TCL_OK;
	
    case STRING:
	// Cast away constness because TCL is stupid.
	Tcl_Return(interp, (char *) data->String(*obj), TCL_STATIC);
	return TCL_OK;
	
    case STRINGLIST:
	Tcl_AppendResult(interp, argv[0], ":Cannot convert stringlist yet", NULL);
	return TCL_ERROR;
	
    default:
	Error(EL_ABORT, argv[0], "Unknown field data tag, attribute = ",
	      argv[2], "\n"
	      "\tThis is an unrecoverable error as it indicates that recompilation\n"
	      "\tis necessary on this platform.", NULL);
	return TCL_ERROR;
    }
}

// ************************************************************************

// rewritten 8/17/95  Kevin
//   to allow multiple modifier masks in event binding

int
bindEvent(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    enum bindwhich {bindXtObj, bindClient, bindWidgetClass, bindNone};

    XServer      *xserver = mercator->GetXServer();
    KeySym       keysym;
    Action       *action;
    int          press;
    int          modifier = FALSE;
    unsigned int modifierMask = 0;
    XtObject     *obj = NULL;
    Client       *client = NULL;
    bindwhich    which = bindNone;
    char         *widgetClass, *actionStr;
    EventPtr     event;

    // This syntax is case *sensitive*:
    // bindevent <obj | client> [ press | release ] <shift><control><meta>
    //    keysym proc

    if ((argc < 4) || (argc > 8)) {
	Error(EL_ERROR, "bindevent", "Incorrect number of arguments.\n", NULL);
	Tcl_AppendResult(interp, "Usage: bindevent <obj | client> [press | release] <shift> <control> <meta> keysym proc", NULL);
	return TCL_ERROR;
    }

    // Handle first optional arg: <obj | client>
    if (strcmp(argv[1], "press") && strcmp(argv[1], "release")) {
	mString str(argv[1]);
	obj = mercator->GetAppModelMgr()->GetXtObject(str);
	if (obj)
	    which = bindXtObj;
	else {
	    client = mercator->GetAppModelMgr()->GetClient(str);
	    if (client)
		which = bindClient;
	    else {
		which = bindWidgetClass;
		widgetClass = argv[1];
	    }
	}
	argv++; argc--;
    }
    
    // Handle [press | release] arg
    if (!strcmp(argv[1], "press"))
	press = TRUE;
    else if (!strcmp(argv[1], "release"))
	press = FALSE;
    else {
	Error(EL_ERROR, "bindevent", "Illegal argument \"", argv[1],
	      "; should be \"press\" or \"release\" instead.", NULL);
	Tcl_AppendResult(interp, "Illegal argument \"", argv[1],
			 "; should be \"press\" or \"release\" instead.", NULL);
	return TCL_ERROR;
    }
  
    actionStr = argv[argc-1];
    keysym = xserver->StringToKeysym(argv[argc-2]);
    if (keysym == NoSymbol) {
	Error(EL_ERROR, "bindevent", "Symbol not recognized: ", argv[argc-2],
	      ".", NULL);
	Tcl_AppendResult(interp, "Symbol not recognized: ", argv[argc-2],
			 ".", NULL);
	return TCL_ERROR;
    }

    // Handle any key modifiers.  Note that these can actually occur in
    //   any order.  We're also not picky about specifying an argument
    //   twice or three times in a row.  Hey, if they want it that much,
    //   they can have it.  B)

    //
    // The modifier masks here are for Sun Type 5 keyboard.  They may
    // be different on others...
    //
    while (--argc-2 > 1) 
	if (!strcmp(argv[argc-2], "shift"))
	    modifierMask |= xserver->SHIFT_MASK;
	else if (!strcmp(argv[argc-2], "control"))
	    modifierMask |= xserver->CONTROL_MASK;
	else if (!strcmp(argv[argc-2], "meta"))
	    modifierMask |= xserver->META_MASK;
	else if (!strcmp(argv[argc-2], "alt"))
	    modifierMask |= xserver->ALT_MASK;
	else {
	    Error(EL_ERROR, "Modifier code not recognized: ", argv[argc-2],
		  ".", NULL);
	    Tcl_AppendResult(interp, "Modifier code not recognized: ",
			     argv[argc-2], ".", NULL);
	    return TCL_ERROR;
	}
  
    action = new TclAction(interp, actionStr);
    event = new EventStruct;
    event->eventType = (press) ? KEYPRESS : KEYRELEASE;
    event->key = keysym;
    event->modifier = modifierMask;
    switch (which) {
    case bindXtObj: 
	xserver->CreateEventBind(obj, event, action); 
	break;
    case bindClient:
	xserver->CreateEventBind(client, event, action);
	break;
    case bindWidgetClass:
	xserver->CreateEventBind(widgetClass, event, action);
	break;
    case bindNone:
	xserver->CreateEventBind(event, action);
	break;
    default:
	Error(EL_ERROR, "bindevent",
	      "bindevent error -- unknown bind type set.", NULL);
	Tcl_AppendResult(interp,
			 "bindevent error -- unknown bind type set.", NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}

// ************************************************************************

int
playSound(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{ 
    double rated;
    float  ratef;
    int    volume, muffle, looped;
    static Audio::SoundID lastSound=-1; // The last sound that was played;
    struct timespec delay;
    long millisecs;

    Audio::SoundID result;

    if (!(argc == 2 || argc == 7)) {
	Tcl_AppendResult(interp, "Wrong number of args.", NULL);		     
	return TCL_ERROR;
    }

    if (lastSound != -1) {
	mercator->GetAudio()->StopSound(lastSound);
    }

    if (argc == 2) { // standard parameters are used
	ratef  = 1.0;
	volume = 100;
	muffle = 0;
	looped = 0;
	result = mercator->GetAudio()->PlaySound(argv[1],
						 &ratef,
						 volume,
						 muffle,
						 looped);
	if (result < 0) {
	    Tcl_AppendResult(interp, "Couldn't play sound \"", argv[1], 
			     "\".", NULL);
	    return TCL_ERROR;
	}
    } else { // argc == 6 - user specified parameters
	// the following lines does string(argv[2]) ==> float(rate) 
	rated = atof(argv[2]);
	ratef = (float) rated;
	millisecs = atol(argv[6]);
	delay.tv_sec = 0;
	delay.tv_nsec=millisecs*1000000;

    	
	// translate the rest of the parameters and play
	if (!((Tcl_GetInt(interp, argv[3], &volume) == TCL_OK) && 
	      (Tcl_GetInt(interp, argv[4], &muffle) == TCL_OK) && 
	      (Tcl_GetInt(interp, argv[5], &looped) == TCL_OK) &&
	      ((result = mercator->GetAudio()-> PlaySound(argv[1],
							  &ratef,
							  volume,
							  muffle,
							  looped)) >= 0) )) {
	    Tcl_AppendResult(interp, "Couldn't play sound \"", argv[1], 
			     "\".", NULL);
	    return TCL_ERROR;
	}

	nanosleep(&delay,NULL);
    }

    lastSound = result;
    return TCL_OK;
}

// ************************************************************************

int
stopAllSounds(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{ 
    mercator->GetAudio()->StopAllSounds();
    return TCL_OK;
}

// ************************************************************************

int
addAction(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    Action * action;

    if (argc != 2) {
	Tcl_AppendResult(interp, "Wrong number of args", NULL);		     
	return TCL_ERROR;
    }

    action = new TclAction(interp, argv[1]);
    return TCL_OK;
}

// ************************************************************************

int
callAction(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    Action * action;
    const char *result = NULL;

    // callaction <name> [ arg arg ... ] 

    if (argc < 2) {
	Tcl_AppendResult(interp, "Wrong number of args.", NULL);		     
	return TCL_ERROR;
    }

    if ((Action::Invoke(argv[1],argc-1,argv+1, &result)) == 0) {
	Tcl_AppendResult(interp, "No such action \"", argv[1], "\".", NULL);
	return TCL_ERROR;
    }

    return TCL_OK;
}

// ************************************************************************
//
// modelManager parent <obj>
// modelManager class <obj>
// modelManager children <obj>
// modelManager longname <obj>
// modelManager name <obj>
// modelManager mapped <obj>
// modelManager location <obj>			// REQUEST loc...
// modelManager x <obj>
// modelManager y <obj>
// modelManager height <obj>
// modelManager width <obj>
// modelManager borderWidth <obj>
//
int
modelManager(ClientData data, Tcl_Interp * interp,
	     int argc, char * argv[])
{
    XtObject * obj;
    mString str;
    mDictValPtrIterator<Object,XtObject> *iter;
    CheckArgcIs(argc == 3);

    str = argv[2];
    obj = mercator->GetAppModelMgr()->GetXtObject(str);
    if (obj == NULL) {
	Tcl_AppendResult(interp, "Unknown XtObject \"", argv[2],
			 "\"", NULL);
	return TCL_ERROR;
    }

    if ((argv[1][0] == 'p') && (strcmp("parent", argv[1]) == 0)) {
	XtObject * parent  = obj->GetParent();

	if (parent == NULL) {
	    Tcl_SetResult(interp, "NULL", TCL_STATIC);
	} else {
	    // Cast away constness because TCL is stupid
	    Tcl_SetResult(interp, (char *) parent->GetUniqueName(), TCL_STATIC);
	}
    } else if (strcmp("x", argv[1]) == 0) {
	char tmp[16];
	sprintf(tmp, "%d", obj->X());
	Tcl_SetResult(interp, tmp, TCL_VOLATILE);
    } else if (strcmp("y", argv[1]) == 0) {
	char tmp[16];
	sprintf(tmp, "%d", obj->Y());
	Tcl_SetResult(interp, tmp, TCL_VOLATILE);
    } else if (strcmp("width", argv[1]) == 0) {
	char tmp[16];
	sprintf(tmp, "%d", obj->Width());
	Tcl_SetResult(interp, tmp, TCL_VOLATILE);
    } else if (strcmp("height", argv[1]) == 0) {
	char tmp[16];
	sprintf(tmp, "%d", obj->Height());
	Tcl_SetResult(interp, tmp, TCL_VOLATILE);
    } else if (strcmp("window", argv[1]) == 0) {
	char tmp[50];
	sprintf(tmp, "0x%x", obj->GetWindowId());
	Tcl_SetResult(interp, tmp, TCL_VOLATILE);
    } else if (strcmp("borderWidth", argv[1]) == 0) {
	char tmp[16];
	sprintf(tmp, "%d", obj->BorderWidth());
	Tcl_SetResult(interp, tmp, TCL_VOLATILE);
    } else if ((argv[1][0] == 'l') && (strcmp("longname", argv[1]) == 0)) {
	// Cast away constness because TCL is stupid
	Tcl_SetResult(interp, (char *) obj->GetLongName(), TCL_STATIC);
    } else if ((argv[1][0] == 'n') && (strcmp("name", argv[1]) == 0)) {
	// Cast away constness because TCL is stupid
	Tcl_SetResult(interp, (char *) obj->GetName(), TCL_STATIC);
    } else if ((argv[1][0] == 'l') && (strcmp("location", argv[1]) == 0)) {
	Client *client = mercator->GetAppModelMgr()->GetCurrentClient();
	client->GetRAP()->LocateObjectRequest(obj->GetObjID());
	Tcl_AppendResult(interp, "location request issued...", NULL);
	return TCL_OK;
    } else if ((argv[1][0] == 'c') && (strcmp("children", argv[1]) == 0)) {
	XtObject * child;
	//
	// We want to return the items in creation order.  So we need
	// to do a little sorting.
	//
	if (!obj || !obj->GetChildren()) {
	    Tcl_SetResult(interp, "NULL", TCL_STATIC);
	    return TCL_OK;
	}

	//
	// We'll store children here and then sort over it.
	//
	XtObjectPtr *childrenArray = 
		new XtObjectPtr[obj->GetChildren()->size()];
	register int i = 0;

	// need an Object
	Object current;
	// create iterator
	mDictValPtrIterator<Object,XtObject> iter(*obj->GetChildren());
	// loop over all children
	while (iter.next()) {
	    current=iter.current();
	    childrenArray[i++] = obj->GetChildren()->lookup(current);    
	}

	XtObjectPtr *sortArray =
		new XtObjectPtr[obj->GetChildren()->size()];
	register int j = 0;

	// I iterates over the destination array. 
	// J iterates over the source array.

	for (i=0 ; i<obj->GetChildren()->size() ; i++) {
	    int index;

	    // init index to a magic value so we know if we are first candidate
	    index=-1;
	    
	    for (j=0 ; j<obj->GetChildren()->size() ; j++) {
		struct timeval candidate;

		//
		// Skip this one if NULL (means we've already put it into
		// the sorted array).
		//
		if (childrenArray[j] == NULL) {
		    continue;
		}
		else {
		    // use the operator< on the XtObject class
		    if (index==-1 || childrenArray[j]->operator<(*(childrenArray[index]))) {
			index=j;
		    }
		}
	    }
	    sortArray[i] = childrenArray[index];
	    childrenArray[index] = NULL;
	}

	for (i=0 ; i<obj->GetChildren()->size() ; i++ ) {
	    child = sortArray[i];
#ifdef TCL7
	    // Cast away constness because TCL is stupid.
	    Tcl_AppendElement(interp, (char *) child->GetUniqueName());

#else
	    // Cast away constness because TCL is stupid.
	    Tcl_AppendElement(interp, (char *) child->GetUniqueName(), FALSE);
#endif /* TCL7 */

	}

	delete [] childrenArray;
	delete [] sortArray;
	
	return TCL_OK;
    } else if ((argv[1][0] == 'c') && (strcmp("class", argv[1]) == 0)) {
	// Cast away constness because TCL is stupid
	Tcl_SetResult(interp, (char *) obj->GetClass(), TCL_STATIC);
    } else if ((argv[1][0] == 'm') && (strcmp("mapped", argv[1]) == 0)) {
	int status = obj->GetMapped();
	if (status)
	    Tcl_SetResult(interp, "TRUE" , TCL_STATIC);
	else
	    Tcl_SetResult(interp, "FALSE" , TCL_STATIC);
    } else if ((argv[1][0] == 'm') && (strcmp("managed", argv[1]) == 0)) {
	int status = obj->GetManaged();
	if (status)
	    Tcl_SetResult(interp, "TRUE" , TCL_STATIC);
	else
	    Tcl_SetResult(interp, "FALSE" , TCL_STATIC);
    } else {
	Tcl_AppendResult(interp, "Unknown arg \"", argv[1],
			 "\" should be name, longname, mapped, managed, class, children, or parent", NULL);
	return TCL_ERROR;
    }

    return TCL_OK;
}

// ************************************************************************

int
currentObject(ClientData data, Tcl_Interp * interp, int, char * [])
{
    Client   *client = mercator->GetAppModelMgr()->GetCurrentClient();
    XtObject *obj;

    if (client == NULL) {
	Tcl_SetResult(interp, "There is no current client", TCL_STATIC);
	return TCL_ERROR;
    }

    obj = client->GetCurrentLocation();
    if (obj == NULL) {
	Tcl_SetResult(interp, "There is no current focus", TCL_STATIC);
	return TCL_ERROR;
    }

    // TCL will release memory via free, so use malloc-based calls.
    // Cast away constness because TCL is stupid
    Tcl_SetResult(interp, (char *) obj->GetUniqueName(), TCL_STATIC);
    return TCL_OK;
}

// ************************************************************************

int
currentClient(ClientData data, Tcl_Interp * interp, int, char * [])
{
    Client   *client   = mercator->GetAppModelMgr()->GetCurrentClient();

    if (client == NULL) {
	Tcl_SetResult(interp, "There is no current client", TCL_STATIC);
	return TCL_ERROR;
    }

    // Cast away constness because TCL is stupid
    Tcl_SetResult(interp, (char *) client->GetUniqueName(), TCL_STATIC);
    return TCL_OK;
}

// ************************************************************************

int
client(ClientData data, Tcl_Interp *interp, int argc, char *argv[])
{
    if (argc != 3) {
      Tcl_AppendResult(interp, "Usage: ", argv[0],  ": name <client>",
                       NULL);
      return TCL_ERROR;
    }

    if (strcmp(argv[1], "name") == 0) {
      Client *client = mercator->GetAppModelMgr()->GetClient(argv[2]);

      if (client == NULL) {
          Tcl_AppendResult(interp, "No such client ", argv[2], NULL);
          return TCL_ERROR;
      }

      Tcl_SetResult(interp, (char *) client->ClientName(), TCL_STATIC);
      return TCL_OK;
    } else {
      Tcl_AppendResult(interp, "Usage: ", argv[0],  ": name <client>",
                       NULL);
      return TCL_ERROR;
    }

    return TCL_OK;
}


// ************************************************************************
int
LoadAll(ClientData data, Tcl_Interp * interp, int , char * [])
{
    Attribute::ParseAttribFile(mercator->AttribFilename());
    Config::ParseConfigFile(mercator->ConfigFilename());
    mercator->LoadTemplates();
 
    return TCL_OK;
}

int
LoadConfig(ClientData data, Tcl_Interp * interp, int , char * [])
{
    Config::ParseConfigFile(mercator->ConfigFilename());
    return TCL_OK;
}

int
LoadAttributes(ClientData data, Tcl_Interp * interp, int , char * [])
{
    Attribute::ParseAttribFile(mercator->AttribFilename());
    return TCL_OK;
}

int
LoadTemplates(ClientData data, Tcl_Interp * interp, int , char * [])
{
    mercator->LoadTemplates();
    return TCL_OK;
}

// ************************************************************************

int
WriteOut(ClientData data, Tcl_Interp * interp, int argc, char *argv[])
{
    FILE *f;
    Template *tmpl;
    char which_dict;

    if (argc < 3) {
	Tcl_AppendResult(interp, "Wrong number of args.", " Usage: ", argv[0],
			 " object-name filename [dictionary]", NULL);
	return TCL_ERROR;
    }
      
    if (argc > 3) {
	which_dict = argv[3][0];
	switch (toupper(which_dict)) {
	case 'A':
	    tmpl = Template::appTemplates->lookup(argv[1]);
	    break;
	case 'C':
	    tmpl = Template::classTemplates->lookup(argv[1]);
	    break;
	case 'O':
	    tmpl = Template::objectTemplates->lookup(argv[1]);
	    break;
	default:
	    Error(EL_ERROR, "Interp-Writeout", "Unknown database", NULL);
	}
    } else {
	tmpl = Template::objectTemplates->lookup(argv[1]);
	if (tmpl == NULL)
	    tmpl = Template::classTemplates->lookup(argv[1]);
	if (tmpl == NULL)
	    tmpl = Template::appTemplates->lookup(argv[1]);
    }

    if (tmpl == NULL) {
	Tcl_AppendResult(interp, "Object not found in database", NULL);
	return TCL_ERROR;
    }

    f = fopen(argv[2], "w");
    if (f == NULL) {
	perror("fopen");
	Tcl_AppendResult(interp, "error opening file", NULL);
	return TCL_ERROR;
    }

    tmpl->WriteToFile(f);
    fclose(f);
    return TCL_OK;
}


// ************************************************************************
//  advanceClient:  switch to the next client in the app
//  model manager's list.

int
advanceClient(ClientData data, Tcl_Interp * interp, int, char * [])
{
    Client * current = mercator->GetAppModelMgr()->GetCurrentClient();
    Client * next;

    if (current == NULL) {
	Tcl_SetResult(interp, "Current client is NULL", TCL_STATIC);
	return TCL_ERROR;
    } else {
	next = mercator->GetAppModelMgr()->SetToNextClient();
    }

    if (next != NULL){
	// Cast away constness because TCL is stupid
	Tcl_SetResult(interp, (char *) next->ClientName(), TCL_STATIC);
    } else {
	Error(EL_ERROR, "advanceClient", "Nowhere to go.", NULL);
    }

    return TCL_OK;
}

// ************************************************************************
//  backupClient:  switch to the previous client in the app
//  model manager's list.

int
backupClient(ClientData data, Tcl_Interp * interp, int, char * [])
{
    Client * current = mercator->GetAppModelMgr()->GetCurrentClient();
    Client * prev;

    if (current == NULL) {
	Tcl_SetResult(interp, "Current client is NULL", TCL_STATIC);
	return TCL_ERROR;
    } else {
	prev = mercator->GetAppModelMgr()->SetToPrevClient();
    }

    Error(EL_DEBUG, "backupClient", "set to previous client", NULL);
    
    if (prev != NULL) {
	// Cast away constness because TCL is stupid
	Tcl_SetResult(interp, (char *) prev->ClientName(), TCL_STATIC);
    } else {
	Error(EL_ERROR, "backupClient", "Nowhere to go.", NULL);
    }

    return TCL_OK;
}

// ************************************************************************

int
setFocus(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    XtObject * obj;
    Client   * client;
    mString str;

    // setFocus [xtobject | client]
    CheckArgcIs(argc == 2);

    str = argv[1];
    obj = mercator->GetAppModelMgr()->GetXtObject(str);
    if (obj != NULL) {
	obj->GetClient()->SetCurrentLocation(obj);
    }
    else {
	client = mercator->GetAppModelMgr()->GetClient(str);
	if (client != NULL)
	    mercator->GetAppModelMgr()->SetCurrentClient(client);
	else {
	    Tcl_AppendResult(interp, "Unknown XtObject or Client \"",
			     argv[1], "\"", NULL);
	    return TCL_ERROR;
	}
    }

    return TCL_OK;
}

// ************************************************************************

int
button(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    XServer  * xserver = mercator->GetXServer();
    XtObject * obj;
    int        press, button;

    // button [ press | release ] [ 1 | 2 | 3 ]
    CheckArgcIs((argc == 3) || (argc == 4));

    if (argc == 4) {
	obj = mercator->GetAppModelMgr()->GetXtObject(argv[1]);
	if (obj == NULL) {
	    Tcl_AppendResult(interp, "Unknown XtObject \"", argv[1], 
			     "\"", NULL);
	    return TCL_ERROR;
	}
#if 0
	xserver->MoveMouse(obj);
#endif
	argv++;
    }

    if ((argv[1][0] == 'r') && (strcmp(argv[1], "release") == 0))
	press = 0;
    else if ((argv[1][0] == 'p') && (strcmp(argv[1], "press") == 0))
	press = 1;
    else
	goto usage;

    if (Tcl_GetInt(interp, argv[2], &button) == TCL_OK) {
	if ((button > 3) || (button < 1))
	    goto usage;
	mercator->GetXServer()->DoMouseButtonEvent(button, press);
#if 0
	if (argc == 4)
	    xserver->MoveMouse(mercator->GetAppModelMgr()->GetCurrentClient()->
			       GetCurrentLocation());
#endif
	return TCL_OK;
    }
  
    
  usage:
    Tcl_AppendResult(interp,
		     "Usage is \"button < xtobject > [ press | release ] "
		     "[ 1 | 2 | 3 ]\"",
		     NULL);
    return TCL_ERROR;
}

// ************************************************************************

int
key(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{ 
    //usage: key <obj> <key> [<modifier>] <press|release|both> <grab status>
  
    unsigned int   key;
    int            modifier = FALSE;
    unsigned int   modifierMask = 0;
    unsigned long  delay=0;
    int            press;
    int            grabbedAlready;
    XtObject      *obj;
    mString        str;
    XServer 	  *xserver = mercator->GetXServer();

    CheckArgcIs((argc == 3) || (argc == 5) || (argc == 6));

    if (argc == 3) {
      if (strcmp(argv[1], "announce")) {
	Tcl_AppendResult(interp, "Usage is \"key announce <on|off>\": ",
			 "argument ", argv[1], " is invalid.", NULL);
	return TCL_ERROR;
      }
      if (!strcmp(argv[2], "on")) 
	press = 1;
      else if (!strcmp(argv[2], "off"))
	press = 0;
      else {
	Tcl_AppendResult(interp, "Usage is \"key announce <on|off>\": ",
			 "argument ", argv[2], " is invalid.", NULL);
	return TCL_ERROR;
      }

      Display *dpy = xserver->GetDisplay();
      Window w = DefaultRootWindow(dpy);
      
      if (press)
	if (XGrabKeyboard(dpy, w, False, GrabModeAsync, GrabModeAsync, 
			CurrentTime) != GrabSuccess) {
	  Tcl_AppendResult(interp, "Could not grab keyboard.", NULL);
	  return TCL_ERROR;
	}
      xserver->SetAnnounceMode(press);
      return TCL_OK;
    }

    if (argc == 6) modifier = TRUE;

    str = argv[1];
    obj = mercator->GetAppModelMgr()->GetXtObject(str);
    if (obj == NULL) {
	Tcl_AppendResult(interp, "Unknown XtObject \"", argv[1],
			 "\"", NULL);
	return TCL_ERROR;
    }

    key = (unsigned int) XStringToKeysym(argv[2]);
    if (key == NoSymbol) {
	Tcl_AppendResult(interp,
			 "Usage is \"key <obj> <key> [<modifier>] "
			 "<press|release|both> <grabbed status>\"",
			 argv[2], " is an invaild key",NULL);
	return TCL_ERROR;
    }

    if (modifier == TRUE) {
	if (strcmp(argv[3],"shift") == 0)
	    modifierMask = modifierMask | xserver->SHIFT_MASK;
	else if (strcmp(argv[3],"control") == 0)
	    modifierMask = modifierMask | xserver->CONTROL_MASK;
	else if (strcmp(argv[3],"meta") == 0)
	    modifierMask = modifierMask | xserver->META_MASK;
	else if (strcmp(argv[3],"alt") == 0)
	    modifierMask = modifierMask | xserver->ALT_MASK;
	else {
	    Tcl_AppendResult(interp,
			     "Argument should be shift, control, "
			     "or meta not \"", argv[3], "\".", NULL);
	    return TCL_ERROR;
	}
	argv++;
    }

    if ((argv[3][0] == 'r') && (strcmp(argv[3], "release") == 0))
	press = 0; 
    else if ((argv[3][0] == 'p') && (strcmp(argv[3], "press") == 0))
	press = 1;
    else if ((argv[3][0] == 'b') && (strcmp(argv[3], "both") == 0))
	press = 2;
    else {
	Tcl_AppendResult(interp,
			 "Usage is \"key <obj> <key> [<modifier>] "
			 "<press|release|both> "
			 "<grabbed status>\" -- invalid key.", NULL);
	return TCL_ERROR;
    }

    if (Tcl_GetInt(interp, argv[4], &grabbedAlready) != TCL_OK) {
	Tcl_AppendResult(interp,
			 "Usage is \"key <obj> <key> [<modifier>] "
			 "<press|release|both> "
			 "<grabbed status>\" -- invalid key.", NULL);
	return TCL_ERROR;
    }

    Client * client = mercator->GetAppModelMgr()->GetCurrentClient();

    // try sending a release too?
    mercator->GetXServer()->
    	DoKeyEvent(obj, key, modifierMask, press, delay, grabbedAlready); 

    return TCL_OK;
}

// ************************************************************************

int
relativemotion(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    //usage: relativemotion x y
    int      x,y;

    CheckArgcIs(argc == 3);
 
    if ((Tcl_GetInt(interp, argv[1], &x) == TCL_OK) && 
	(Tcl_GetInt(interp, argv[2], &y) == TCL_OK))   {
	mercator->GetXServer()->MoveMouse( x, y);
	return TCL_OK;
    }
    Tcl_AppendResult(interp, "Usage:  relativemotion x y", NULL);
    return TCL_ERROR;
 
}

// ************************************************************************
//
// Usage:
//      speak "string"
//      speak cap <cap>
//
int
speak(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    CheckArgcIs(argc == 2 || argc == 3);

    if (argc == 2) {
	mercator->GetSpeech()->StopSpeaking();
	mercator->GetSpeech()->Speak(argv[1]);
    }
    else if (strcmp(argv[2],"nointerupt") == 0) {
	mercator->GetSpeech()->Speak(argv[1]);
    }
    else if (strcmp(argv[1], "cap") == 0) {
	Speech::Capabilities caps = mercator->GetSpeech()->GetCapabilities();
	
	if (strcmp(argv[2], "voices") == 0) {
	    if(caps.voices.entries() > 0) {
		mListValIterator<mString> iter(caps.voices);
		mString result("");
		
		while(iter.next() != NULL) {
		    result += "{";
		    result += iter.current();
		    result += "} ";
		}
		const char * resultptr = (const char *) result;
		Tcl_SetResult(interp, (char *) resultptr, TCL_VOLATILE);
	    }
	}
	else if (strcmp(argv[2], "languages") == 0) {
	    if(caps.languages.entries() > 0) {
		mListValIterator<mString> iter(caps.languages);
		mString result("");
		
		while(iter.next() != NULL) {
		    result += "{";
		    result += iter.current();
		    result += "} ";
		}
		const char * resultptr = (const char *) result;
		Tcl_SetResult(interp, (char *) resultptr, TCL_VOLATILE);
	    }
	}
	else if (strcmp(argv[2], "minRate") == 0) {
	    char tmp[128];
	    sprintf(tmp, "%d", caps.minRate);
	    Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	}
	else if (strcmp(argv[2], "maxRate") == 0) {
	    char tmp[128];
	    sprintf(tmp, "%d", caps.maxRate);
	    Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	}
	else if (strcmp(argv[2], "gainSupported") == 0) {
	    char tmp[128];
	    sprintf(tmp, "%d", caps.gainSupported);
	    Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	}
	else {
	    Tcl_AppendResult(interp, "Invalid command", NULL);
	    return TCL_ERROR;
	}
    }
    else {
	Tcl_AppendResult(interp, "Invalid command", NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}

// ************************************************************************
//
// Usage:
//      braille display "string"                ->SetText()
//      braille translate "string"              ->Translation()
//      ...
//
int
braille(ClientData data, Tcl_Interp *interp, int argc, char *argv[])
{
    if (argc < 2) {
        Tcl_AppendResult(interp, "Usage: ",
			 argv[0], " display <row1> <row2> ...\n",
			 argv[0], " status <text>\n",
			 argv[0], " cap <capability>\n",
			 argv[0], " name\n",
			 argv[0], " translate <table>", NULL);
        return TCL_ERROR;
    }

    if (strcmp(argv[1], "name") == 0) {
        Tcl_AppendResult(interp, mercator->GetBraille()->GetBrailleName(),
			 NULL);
        return TCL_OK;
    }
    else if (strcmp(argv[1], "display") == 0) {
        mercator->GetBraille()->SetText((const char **) &argv[2], argc-2);
    }
    else if (strcmp(argv[1], "status") == 0) {
	mercator->GetBraille()->DisplayStatus(argv[2]);
    }
    else if (strcmp(argv[1], "translate") == 0) {
        mercator->GetBraille()->Translation((const unsigned char *) argv[2]);
    }
    else if ((argc == 3) && (strcmp(argv[1], "jumpScroll") == 0) &&
	     (strcmp(argv[2], "on") == 0)){
        mercator->GetBraille()->JumpScroll(1);
    }
    else if ((argc == 3) && (strcmp(argv[1], "jumpScroll") == 0) &&
	     (strcmp(argv[2], "off") == 0)){
        mercator->GetBraille()->JumpScroll(0);
    }
    else if ((strcmp(argv[1], "cap") == 0) && (argc > 2)) {
	char tmp[128];
	Braille::Capabilities caps = mercator->GetBraille()->GetCapabilities();
	
	if(strcmp(argv[2], "displayCells") == 0) {
	    sprintf(tmp, "%d", caps.displayCells);
	    Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	}
	else if(strcmp(argv[2], "statusCells") == 0) {
	    sprintf(tmp, "%d", caps.statusCells);
	    Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	}
	else if(strcmp(argv[2], "highlightSupported") == 0) {
	    sprintf(tmp, "%d", caps.highlightSupported);
	    Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	}
	else if(strcmp(argv[2], "hasCursorKeys") == 0) {
	    sprintf(tmp, "%d", caps.hasCursorKeys);
	    Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	}
	else if(strcmp(argv[2], "hasProgKey") == 0) {
	    sprintf(tmp, "%d", caps.hasProgKey);
	    Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	}
	else if(strcmp(argv[2], "hasHomeKey") == 0) {
	    sprintf(tmp, "%d", caps.hasHomeKey);
	    Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	}
	else if(strcmp(argv[2], "otherKeys") == 0) {
	    sprintf(tmp, "%d", caps.otherKeys);
	    Tcl_SetResult(interp, tmp, TCL_VOLATILE);
	}
	else{
	    Tcl_AppendResult(interp, argv[2], " unknown", NULL);
	    return TCL_ERROR;
	}
    }
    else{
        Tcl_AppendResult(interp, "Usage: ", argv[0], " display <string>",
                         NULL);
        return TCL_ERROR;
    }

    return TCL_OK;
}

// ************************************************************************

int
getTextVertCoord(ClientData data, Tcl_Interp *interp, int argc, char *argv[])
{
    XtObject     *obj;
    ScreenReader *sreader;
    TextRep      *textrep;
    int          coord, dummy;
    char	 buf[32];

    if (argc != 2) {
	Tcl_AppendResult(interp, "usage: %s object", argv[0], NULL);
	return TCL_ERROR;
    }

    obj = mercator->GetAppModelMgr()->GetXtObject(argv[1]);
    if (obj == NULL) {
	Tcl_AppendResult(interp, "Unknown XtObject \"", argv[1],
			 "\"", NULL);
	return TCL_ERROR;
    }

    sreader = obj->GetScreenReader();
#if 0
    if (sreader == NULL) {
	textrep = new TextRep(obj,(XlibHook*)nil,mercator->GetXServer());
	sreader = new ScreenReader(textrep, mercator->GetSpeech(),mercator);
	made_screenReader = 1;
    }
    else
	made_screenReader = 0;
#endif 0

    if (sreader == NULL) {
	Tcl_AppendResult(interp, "object has no screenreader!?", NULL);
	return TCL_ERROR;
    }

    textrep = obj->GetTextRep();

    if (textrep == NULL) {
	Tcl_AppendResult(interp, "object has no textrep!?", NULL);
	return TCL_ERROR;
    }

    if (sreader->GetCursorRep() == NULL) {
	Tcl_AppendResult(interp, "screenreader has no cursor!?", NULL);
	return TCL_ERROR;
    }

#ifdef OLD
    coord = textrep->GetLineVerticalCoord(sreader->GetCursorRep()->GetCursorY());
#else
    textrep->GetPixelCoords(sreader->GetCursorRep()->GetCursorY(),
			    sreader->GetCursorRep()->GetCursorX(),
			    dummy, coord);
#endif
#if 0
    Error(EL_WARNING, "getTextVertCoord",
	  "NEW TEXT REP DOES NOT YET SUPPORT VERTICAL COORDS    --keith", NULL);
#endif
    sprintf(buf, "%d", coord);
    Tcl_SetResult(interp, buf, TCL_VOLATILE);
    return TCL_OK;
}

// ************************************************************************

int
getTextHorizCoord(ClientData data, Tcl_Interp *interp, int argc, char *argv[])
{
    XtObject     *obj;
    ScreenReader *sreader;
    TextRep      *textrep;
    int          coord, dummy;
    char	 buf[32];

    if (argc != 2) {
	Tcl_AppendResult(interp, "usage: %s object", argv[0], NULL);
	return TCL_ERROR;
    }

    obj = mercator->GetAppModelMgr()->GetXtObject(argv[1]);
    if (obj == NULL) {
	Tcl_AppendResult(interp, "Unknown XtObject \"", argv[1],
			 "\"", NULL);
	return TCL_ERROR;
    }

    sreader = obj->GetScreenReader();
    if (sreader == NULL) {
	Tcl_AppendResult(interp, "object has no screenreader!?", NULL);
	return TCL_ERROR;
    }

    textrep = obj->GetTextRep();

    if (textrep == NULL) {
	Tcl_AppendResult(interp, "object has no textrep!?", NULL);
	return TCL_ERROR;
    }

    if (sreader->GetCursorRep() == NULL) {
	Tcl_AppendResult(interp, "screenreader has no cursor!?", NULL);
	return TCL_ERROR;
    }

    textrep->GetPixelCoords(sreader->GetCursorRep()->GetCursorY(),
			    sreader->GetCursorRep()->GetCursorX(),
			    coord, dummy);
    sprintf(buf, "%d", coord);
    Tcl_SetResult(interp, buf, TCL_VOLATILE);
    return TCL_OK;
}

int
sreader(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    XtObject     *obj=NULL;
    ScreenReader *sreader=NULL;
    TextRep      *textrep=NULL;
    int          i=0, speed=0,made_screenReader=0;
    char *read_result=NULL;

    // sreader <obj> dumptext
    // sreader <obj> read  [this | next | prev ]
    //               [ char | word | sentence | paragraph | line | screen] <?raw>
    // sreader <obj> move  [forward | backward | front | end | top | bottom] 
    //               [ char | word | sentence | paragraph | line | screen]
    // sreader speed <integer>            => sets the speed of speech syn. 
    // sreader <obj> filter <which> <status> => query/set filters

    if (strcmp(argv[1], "speed") == 0) {
	CheckArgcIs(argc == 3);
	speed=atoi(argv[2]);
	Speech *speech = mercator->GetSpeech();
	if (!speech) {
	    Tcl_AppendResult(interp, "No speech!?", NULL);
	    return TCL_ERROR;
	}
	speech->SetSpeechRate(speed);
	//	mercator->GetSpeech()->SetVoice(argv[2]);
	//	goto endOK;
	return TCL_OK;
    }

    if (strcmp(argv[1], "voice") == 0) {
	CheckArgcIs(argc == 3);
	Speech *speech = mercator->GetSpeech();
	if (!speech) {
	    Tcl_AppendResult(interp, "No speech!?", NULL);
	    return TCL_ERROR;
	}
	speech->SetVoice(argv[2]);
	//	mercator->GetSpeech()->SetVoice(argv[2]);
	//	goto endOK;
	return TCL_OK;
    }

    obj = mercator->GetAppModelMgr()->GetXtObject(argv[1]);
    if (obj == NULL) {
	Tcl_AppendResult(interp, "Unknown XtObject \"", argv[1],
			 "\"", NULL);
	return TCL_ERROR;
    }

    sreader = obj->GetScreenReader();

    if (sreader == NULL) {
	Error(EL_ERROR, argv[0], "Object has no screenreader!", NULL);
	return TCL_OK;
    } else
	made_screenReader = 0;

    if (!strcmp(argv[2], "incoming")) {
      int current_mode;
      current_mode = sreader->GetIncomingMode();
      sreader->SetIncomingMode(++current_mode % 3);
      Tcl_AppendResult(interp, "Toggling incoming mode.", NULL); 
      return TCL_OK;
    }

    if (!strcmp(argv[2], "togglecursor")) {
	CheckArgcIs(argc == 3)
	    if (made_screenReader == 1) {
		Tcl_AppendResult(interp, 
				 "No screen reader associated with XtObject.",
				 NULL);
		delete sreader;
		delete textrep;
		return TCL_ERROR;
	    }
	int current_mode;
	switch (current_mode = sreader->GetOperationMode()) {
	case ScreenReader::CURSOR_EDIT: 
	    Tcl_AppendResult(interp, "Entering cursor REVIEW mode.", NULL); 
	    break;
	case ScreenReader::CURSOR_REVIEW:
	    Tcl_AppendResult(interp, "Entering cursor EDIT mode.", NULL);
	    break;
	default:
	    Tcl_AppendResult(interp, "Interp::sreader -- Unknown cursor mode.", 
			     NULL); 
	    return TCL_ERROR;
	}
	if ((current_mode == ScreenReader::CURSOR_REVIEW) && 
	    (sreader->GetFollowMode() == ScreenReader::FOLLOW_ON))
	    sreader->SyncReviewToEdit();
	sreader->SetOperationMode((current_mode == ScreenReader::CURSOR_EDIT) ?
				  ScreenReader::CURSOR_REVIEW :
				  ScreenReader::CURSOR_EDIT);
	return TCL_OK;
    }
    if (!strcmp(argv[2], "togglefollow")) {
	CheckArgcIs(argc == 3)
	    if (made_screenReader == 1) {
		Tcl_AppendResult(interp, 
				 "No screen reader associated with XtObject.",
				 NULL);
		delete sreader;
		delete textrep;
		return TCL_ERROR;
	    }
	int current_mode;
	switch (current_mode = sreader->GetFollowMode()) {
	case ScreenReader::FOLLOW_OFF:
	    Tcl_AppendResult(interp, "Review cursor will follow edit cursor.", 
			     NULL); break;
	case ScreenReader::FOLLOW_ON:
	    Tcl_AppendResult(interp,
			     "Review cursor will not follow edit cursor.",
			     NULL); break;
	default:
	    Tcl_AppendResult(interp, "Interp::sreader -- Unknown follow mode.", 
			     NULL); 
	    return TCL_ERROR;
	}
	sreader->SetFollowMode((current_mode == ScreenReader::FOLLOW_ON) ?
			       ScreenReader::FOLLOW_OFF :
			       ScreenReader::FOLLOW_ON);
	return TCL_OK;
    }
    if (!strcmp(argv[2], "filter")) {
	int filter, status;
	char cstatus[10];
	CheckArgcIs((argc >= 3) && (argc <= 5));
	if (made_screenReader == 1) {
	    Tcl_AppendResult(interp,
			     "No screen reader associated with XtObject.", 
			     NULL);
	    delete sreader;
	    delete textrep;
	    return TCL_ERROR;
	}
	switch (argc) {
	case 3: // filter with no args -- return max_filters
	    sreader->GetFilterCount();
	    sprintf(cstatus, "%d", sreader->GetFilterCount());
	    break;
	case 4: // filter with one arg -- return status of specified filter
	    if (Tcl_GetInt(interp, argv[3], &filter) != TCL_OK) {
		Tcl_AppendResult(interp,
				 "Interp::sreader -- bad position specified.",
				 NULL);
		return TCL_ERROR;
	    }
	    sprintf(cstatus,"%s", sreader->GetFilterActive(filter) ?
		    "ON" : "OFF");
	    break;
	case 5: // filter with two args -- set status of specified filter
	    sprintf(cstatus, "");
	    if (Tcl_GetInt(interp, argv[3], &filter) != TCL_OK) {
		Tcl_AppendResult(interp,
				 "Interp::sreader -- bad position specified.",
				 NULL);
		return TCL_ERROR;
	    }
	    if (Tcl_GetInt(interp, argv[4], &status) != TCL_OK) {
		Tcl_ResetResult(interp);
		if (!strcmp(argv[4], "name")) {
		    Tcl_AppendResult(interp, sreader->GetFilterName(filter),
				     NULL);
		    return TCL_OK;
		}
		else {
		    Tcl_AppendResult(interp,
				     "Interp::sreader -- bad filter active val.",
				     NULL);
		    return TCL_ERROR;
		}
	    }
	    sreader->SetFilterActive(filter, status);
	    break;
	}
	Tcl_AppendResult(interp, cstatus, NULL);
	return TCL_OK;
    }
    if (strcmp(argv[2], "dumptext") == 0) {
	obj->GetTextRep()->PrintContents(stdout);
	Tcl_AppendResult(interp, "done.", NULL);
	return TCL_OK;
    }
    if (strcmp(argv[2], "cursor") == 0) {
	int x, y;
	//    CheckArgcIs((argc >= 3) && (argc <= 5));
	if (made_screenReader == 1) {
	    Tcl_AppendResult(interp,
			     "No screen reader associated with XtObject.", 
			     NULL);
	    delete sreader;
	    delete textrep;
	    return TCL_ERROR;
	}
	if (argc == 3) {
	    int x, y;
	    char cx[10], cy[10];
	    sreader->GetCursorRep()->GetCursor(x, y);
	    sprintf(cx, "%d", x);
	    sprintf(cy, "%d", y);
	    switch (sreader->GetOperationMode()) {
	    case ScreenReader::CURSOR_REVIEW: 
		Tcl_AppendResult(interp, "Review mode.\n", NULL); 
		break;
	    case ScreenReader::CURSOR_EDIT:
		Tcl_AppendResult(interp, "Edit mode.\n", NULL);
		break;
	    default:
		Tcl_AppendResult(interp,
				 "Interp::sreader -- Unknown cursor mode.", 
				 NULL); 
		return TCL_ERROR;
	    }
	    switch (sreader->GetFollowMode()) {
	    case ScreenReader::FOLLOW_ON:
		Tcl_AppendResult(interp, "Review cursor follows edit cursor.\n", 
				 NULL); break;
	    case ScreenReader::FOLLOW_OFF:
		Tcl_AppendResult(interp, 
				 "Review cursor does not follow edit cursor.\n",
				 NULL); 
		break;
	    default:
		Tcl_AppendResult(interp,
				 "Interp::sreader -- Unknown follow mode.", 
				 NULL); 
		return TCL_ERROR;
	    }
	    Tcl_AppendResult(interp, "Current cursor position is ", cx,
			     ", ", cy, ".", NULL);
	    return TCL_OK;
	}
	else if (argc == 4) {
	    if (!strcmp(argv[3], "mode")) {
		Tcl_AppendResult(interp, (sreader->GetOperationMode() == 
					  ScreenReader::CURSOR_REVIEW) ? 
				 "REVIEW" : "EDIT", NULL);
		return TCL_OK;
	    }
	    else if (!strcmp(argv[3], "app")) {
	      sreader->SyncEditToApp();
	      Tcl_AppendResult(interp, "Syncing edit cursor to review cursor",
			       NULL);
	      return TCL_OK;
	    }
	    else if (!strcmp(argv[3], "coords")) {
	      int x, y;
	      char cx[10], cy[10];
	      sreader->GetCursorRep()->GetCursor(x, y);
	      sprintf(cx, "%d", x);
	      sprintf(cy, "%d", y);
	      Tcl_AppendResult(interp, cx, " ", cy, NULL);
	      return TCL_OK;
	    }
	    else {
		Tcl_AppendResult(interp, "Usage: sreader cursor mode", NULL);
		return TCL_ERROR;
	    }
	}
	else {
	    if (Tcl_GetInt(interp, argv[3], &x) != TCL_OK) {
		Tcl_AppendResult(interp, "Usage: sreader cursor <x> <y>", NULL);
		return TCL_ERROR;
	    }
	    if (Tcl_GetInt(interp, argv[4], &y) != TCL_OK) {
		Tcl_AppendResult(interp, "Usage: sreader cursor <x> <y>", NULL);
		return TCL_ERROR;
	    }
	    try {
		sreader->SetCursorPosition(x, y);
	    }
	    catch (CursorOutOfBounds err) {
		char x[10], y[10], bx[10], by[10];
		sprintf(x, "%d", err.x);
		sprintf(y, "%d", err.y);
		sprintf(bx, "%d", err.boundsx);
		sprintf(by, "%d", err.boundsy);
		Tcl_AppendResult(interp,
				 "Interp::sreader -- cursor out of bounds at ",
				 x, ", ", y, " (boundary at ", bx,
				 ", ", by, ")", NULL);
		return TCL_ERROR;
	    }
	    return TCL_OK;
	}
    }
    if (strcmp(argv[2], "read") == 0) {
	if (made_screenReader == 1) {
	    Tcl_AppendResult(interp, "Screenreader == NULL.", NULL);
	    delete sreader;
	    delete textrep;
	    return TCL_ERROR;
	}
	
	CheckArgcIs((argc == 5 || argc == 6))
	    
	if (argc == 5)
	    read_result = sreader->read(argv[3], argv[4]);
	else if (argc == 6) {
	    if (strcmp(argv[5], "raw") == 0)
		read_result = sreader->read(argv[3], argv[4], 1);
	    else {
		Tcl_AppendResult(interp, "unknown option, expected 'raw'",
				 NULL);
		return TCL_ERROR;
	    }
	}
	//	CheckArgcIs(argc == 5)
	//	    read_result = sreader->read(argv[3],argv[4]);

	Tcl_AppendResult(interp, read_result, NULL);
	if (read_result)
	    delete [] read_result;
	return TCL_OK;
    } else if (strcmp(argv[2], "move") == 0) {
	if (made_screenReader == 1) {
	    Tcl_AppendResult(interp, "Screenreader == NULL.", NULL);
	    delete sreader;
	    delete textrep;
	    return TCL_ERROR;
	}
	CheckArgcIs(argc == 5)
	    read_result = sreader->move(argv[3],argv[4]);
	Tcl_AppendResult(interp, read_result, NULL);
	if (read_result)
	    delete [] read_result;
	return TCL_OK;
    }

    Tcl_AppendResult(interp, "Unknown command \"", argv[1], "\".", NULL);
    return TCL_ERROR;

    if (made_screenReader == 1) {
	delete sreader;
	delete textrep;
    }
    return TCL_OK;
}

// ************************************************************************
//
// resource <obj> get <resourcename>
// resource <obj> set <resourcename> <resourcevalue>
// resource <obj> list
// resource <obj> request <resourcename>
//	...
//
int
resource(ClientData data, Tcl_Interp * interp,
	 int argc, char * argv[])
{
    Client *client = mercator->GetAppModelMgr()->GetCurrentClient();
    XtObject * obj;
    mString str;

    //
    // Do argc, argv sanity checking.
    //
    if (argc != 3 && argc != 4 && argc != 5) {
	Tcl_AppendResult(interp,
			 "Usage is \"resource <obj> get <name>\"\n",
			 "         \"resource <obj> set <name> <value>\"\n",
			 "         \"resource <obj> request <name>\"\n",
			 "         \"resource <obj> list\"",
			 NULL);
	return TCL_ERROR;
    }
    if ((argc == 4) && (strcmp(argv[2], "get") != 0)
	&& (strcmp(argv[2], "request") != 0)) {
	Tcl_AppendResult(interp,
			 "Usage is \"resource <obj> get <name>\"\n",
			 "         \"resource <obj> set <name> <value>\"\n",
			 "         \"resource <obj> request <name>\"\n",
			 "         \"resource <obj> list\"",
			 NULL);
	return TCL_ERROR;
    }
    if ((argc == 5) && (strcmp(argv[2], "set")) != 0) {
	Tcl_AppendResult(interp,
			 "Usage is \"resource <obj> get <name>\"\n",
			 "         \"resource <obj> set <name> <value>\"\n",
			 "         \"resource <obj> request <name>\"\n",
			 "         \"resource <obj> list\"",
			 NULL);
	return TCL_ERROR;
    }
    if ((argc == 3) && (strcmp(argv[2], "list")) != 0) {
	Tcl_AppendResult(interp,
			 "Usage is \"resource <obj> get <name>\"\n",
			 "         \"resource <obj> set <name> <value>\"\n",
			 "         \"resource <obj> request <name>\"\n",
			 "         \"resource <obj> list\"",
			 NULL);
    }

    str = argv[1];
    obj = mercator->GetAppModelMgr()->GetXtObject(str);
    if (obj == NULL) {
	Tcl_AppendResult(interp, "Unknown XtObject \"", argv[2], "\"", NULL);
	return TCL_ERROR;
    }

    if ((argv[2][0] == 'g') && (strcmp("get", argv[2]) == 0)) {
 	//
 	// GET resource value and return it.
 	//
	ostrstream valueStream;
	obj->PrintResourceValue(argv[3], valueStream);
	valueStream << ends;
	char *valueString = valueStream.str();

	if (valueString == NULL) {
	    Tcl_SetResult(interp, "NULL", TCL_STATIC);
	} else {
	    Tcl_SetResult(interp, valueString, TCL_DYNAMIC);
	}
    } else if ((argv[2][0] == 'r') && (strcmp("request", argv[2]) == 0)) {
 	//
 	// REQUEST (generate RAP) a resource.  This does not block!
 	//
	char *resourceNames[2];
	resourceNames[0] = argv[3];
	resourceNames[1] = NULL;
	
	client->GetRAP()->GetValuesRequest(obj->GetObjID(),
					   (const char **) resourceNames, 1);
	Tcl_SetResult(interp, "fetching...", TCL_STATIC);
    } else if ((argv[2][0] == 's') && (strcmp("set", argv[2]) == 0)) {
	ResourceKeyValue resource;
	//
	// SET resource value, return the new value.
	//
	resource.name = argv[3];
	resource.value = argv[4];
	client->GetRAP()->SetValuesRequest(obj->GetObjID(), &resource, 1);
	Tcl_SetResult(interp, argv[4], TCL_VOLATILE);
    } else if ((argv[2][0] == 'l') && (strcmp("list", argv[2]) == 0)) {
	ostrstream s;
	obj->PrintResources(s);
	s << ends;
	char *str = s.str();
	
	if (str == NULL) {
	    Tcl_SetResult(interp, "none", TCL_STATIC);
	} else {
	    Tcl_SetResult(interp, str, TCL_DYNAMIC);
	}
    } else {
	Tcl_AppendResult(interp, "Unknown arg \"", argv[1],
			 "\" should be get or set", NULL);
	return TCL_ERROR;
    }

    return TCL_OK;
}

// ************************************************************************

#ifdef TEXTREP_DEBUG
int
textrepdebug(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    XtObject * obj = NULL;

    if (argc != 3) {
	Tcl_AppendResult(interp, "Usage: textrepdebug <obj> [ on | off ]", 
			 NULL);
	return TCL_ERROR;
    }

    mString str(argv[1]);
  
    obj = mercator->GetAppModelMgr()->GetXtObject(str);
    if (obj == NULL) {
	Tcl_AppendResult(interp, "Unknown XtObject \"", argv[1],
			 "\"", NULL);
	return TCL_ERROR;
    }

    if (obj->GetTextRep() == NULL) {
	Tcl_AppendResult(interp, "XtObject \"", argv[1],
			 "\" doesn't have a TextRep", NULL);
	return TCL_ERROR;
    }

    if (strcmp(argv[2], "on") == 0) {
	obj->GetTextRep()->Debug(1);
	obj->GetScreenReader()->debug(1);
    } else if (strcmp(argv[2], "off") == 0) {
	obj->GetTextRep()->Debug(0);
	obj->GetScreenReader()->debug(0);
    } else {
	Tcl_AppendResult(interp,
			 "Usage: textrepdebug <obj> [ on | off ]",
			 NULL);
	return TCL_ERROR;
    }

    return TCL_OK;
}
#endif //TEXTREP_DEBUG

// ************************************************************************
///////////////////////////////////////////////////////////////////////////
////
////  All of this crap is used by the xwin proc.  It has a lot of
////  functionality, so there are plenty of static routines here
////  to help out.
////
///////////////////////////////////////////////////////////////////////////

//
// CheckFormat() is used to check for expected property formats (either
// 8- 16- or 32-bit quantities) and stick a meaningful return value in
// the interp if we get somethign we weren't looking for.
//
static int
CheckFormat(char *func, Tcl_Interp *interp, int desired, int actual, char *name)
{
    if (actual != desired) {
	Tcl_AppendResult(interp, "%s: (%s) bogus format %d (expected %d)\n",
			 func, name, actual, desired, NULL);
	return FALSE;
    } else {
	return TRUE;
    }
}

static int
xPropertySet(Tcl_Interp *interp, int argc, char *argv[])
{
    Error(EL_ERROR, "xPropertySet", "NOT IMPLEMENTED.", NULL);
    Tcl_AppendResult(interp, "xPropertySet not implemented yet.\n", NULL);
    return TCL_ERROR;
}

//	x property get <winid> <property>
//		note that <winid> may be specified as "root"
//		to get the *default* root window.
//
//	This is pretty brute force.  I select out for each return type,
//	and then try to make sure that the return format of the property
//	matches what I'm looking for for that type.  There's a lot of
//	duplicated code, but it should be pretty resilient to changes
//	in any of the data types.
//
static int
xPropertyGet(Tcl_Interp *interp, int argc, char *argv[])
{
#if 0
    Display *dpy;
    Window w;
    Atom property;
    Atom actualType;
    int actualFormat;
    unsigned long nitems;
    unsigned long bytesAfter;
    unsigned char *prop;
    int ret;
    char *v1;
    short *v2;
    long *v4;

    dpy = mercator->GetXServer()->GetDisplay();

    if (strcmp(argv[3], "root") == 0)
	w = DefaultRootWindow(dpy);
    else
        w = atol(argv[3]);

    property = XInternAtom(dpy, argv[4], TRUE);

    // the 0 and 8192 mean "get the biggest chunk we can"

    ret = XGetWindowProperty(dpy, w, property, 0, 8192, FALSE,
			     AnyPropertyType, &actualType, &actualFormat, 
			     &nitems, &bytesAfter, &prop);

    if (ret != Success) {
    }

    if (actualType == None || actualFormat == 0) {
	// the property doesn't exist.
    } else if (nitems == 0 && actualFormat != 0) {
	// type mismatch (shouldn't happen since we use AnyPropertyType)
    } else {
	// looks like we're good to go!

	char *typeName = XGetAtomName(dpy, actualType);

	// see if it's a split property
	if (bytesAfter != 0) {
	    Tcl_AppendResult(interp,
			     "xPropertyGet: (%s) don't handle split properties yet\n",
			     typeName,
			     NULL);
	    return TCL_ERROR;
	}

	// see if we can make sense of the type of the data.

	switch (actualType) {
	case XA_ARC : {
	    XArc *arc;
	    char tmp[256];
	
	    if (!CheckFormat("xPropertyGet", interp, 16, actualFormat,
			     typeName))
		return TCL_ERROR;

	    arc = (XArc *) prop;
	    sprintf(tmp, "{%d %d %d %d %d %d}", arc->x, arc->y,
		    arc->width, arc->height, arc->angle1, arc->angle2);

	    Tcl_AppendResult(interp, tmp, NULL);

	    XFree(prop);
	    return TCL_OK;
	}
	case XA_ATOM : {
	    Atom atom;
	    char *tmp;

	    if (!CheckFormat("xPropertyGet", interp, 32, actualFormat,
			     typeName))
		return TCL_ERROR;

	    atom = *((Atom *) prop);

	    tmp = XGetAtomName(dpy, atom);
	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(tmp);
	    XFree(prop);
	    return TCL_OK;
	}
	case XA_BITMAP : {
	    Pixmap bitmap;
	    char tmp[32];

	    if (!CheckFormat("xPropertyGet", interp, 32, actualFormat,
			     typeName))
		return TCL_ERROR;
	
	    bitmap = *((Pixmap *) prop);

	    sprintf(tmp, "%ld", bitmap);
	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(prop);
	    return TCL_OK;
	}
	case XA_CURSOR : {
	    Cursor cursor;
	    char tmp[32];

	    if (!CheckFormat("xPropertyGet", interp, 32, actualFormat,
			     typeName))
		return TCL_ERROR;
	
	    cursor = *((Cursor *) prop);

	    sprintf(tmp, "%ld", cursor);
	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(prop);
	    return TCL_OK;
	}	 
	case XA_DRAWABLE : {
	    Drawable drawable;
	    char tmp[32];

	    if (!CheckFormat("xPropertyGet", interp, 32, actualFormat,
			     typeName))
		return TCL_ERROR;
	
	    drawable = *((Drawable *) prop);

	    sprintf(tmp, "%ld", drawable);
	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(prop);
	    return TCL_OK;
	}	 
	case XA_FONT : {
	    Font font;
	    char tmp[32];

	    if (!CheckFormat("xPropertyGet", interp, 32, actualFormat,
			     typeName))
		return TCL_ERROR;
	
	    font = *((Font *) prop);

	    sprintf(tmp, "%ld", font);
	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(prop);
	    return TCL_OK;
	}	 
	case XA_PIXMAP : {
	    Pixmap pixmap;
	    char tmp[32];

	    if (!CheckFormat("xPropertyGet", interp, 32, actualFormat,
			     typeName))
		return TCL_ERROR;
	
	    pixmap = *((Pixmap *) prop);

	    sprintf(tmp, "%ld", pixmap);
	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(prop);
	    return TCL_OK;
	}	
	case XA_STRING : {
	    char *tmp;

	    if (!CheckFormat("xPropertyGet", interp, 8, actualFormat,
			     typeName))
		return TCL_ERROR;

	    tmp = (char *) prop;
	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(prop);
	    return TCL_OK;
	}
	case XA_VISUALID : {
	    Visual visual;
	    char tmp[32];

	    if (!CheckFormat("xPropertyGet", interp, 32, actualFormat,
			     typeName))
		return TCL_ERROR;
	
	    visual = *((Visual *) prop);

	    sprintf(tmp, "%ld", visual);
	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(prop);
	    return TCL_OK;
	}
	case XA_WINDOW : {
	    Window window;
	    char tmp[32];

	    if (!CheckFormat("xPropertyGet", interp, 32, actualFormat,
			     typeName))
		return TCL_ERROR;
	
	    window = *((Window *) prop);

	    sprintf(tmp, "%ld", window);
	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(prop);
	    return TCL_OK;
	}	    
	case XA_WM_HINTS : {
	    XWMHints *wmhints;
	    char buffer[1024];
	    char tmp[32];
	
	    // What is the format of this thing anyway??

	    wmhints = (XWMHints *) prop;

	    // clear the temporary buffer.
	    memset(buffer, 0, 1024);

	    if (wmhints->flags & InputHint) {
		sprintf(tmp, "{%d ", wmhints->input);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "{-1 ");
	    }

	    if (wmhints->flags & StateHint) {
		sprintf(tmp, "%d ", wmhints->initial_state);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 ");
	    }

	    if (wmhints->flags & IconPixmapHint) {
		sprintf(tmp, "%d ", wmhints->icon_pixmap);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 ");
	    }

	    if (wmhints->flags & IconWindowHint) {
		sprintf(tmp, "%d ", wmhints->icon_window);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 ");
	    }

	    if (wmhints->flags & IconPositionHint) {
		sprintf(tmp, "%d %d ", wmhints->icon_x, wmhints->icon_y);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 -1 ");
	    }

	    if (wmhints->flags & IconMaskHint) {
		sprintf(tmp, "%d ", wmhints->icon_mask);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 ");
	    }

	    if (wmhints->flags & WindowGroupHint) {
		sprintf(tmp, "%d}", wmhints->window_group);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1}");
	    }

	    Tcl_AppendResult(interp, buffer, NULL);
	    return TCL_OK;
	}
	case XA_WM_SIZE_HINTS : {
	    XSizeHints *sizehints;
	    char buffer[1024];
	    char tmp[32];

	    // What is the format of this thing anyway?!

	    sizehints = (XSizeHints *) prop;

	    memset(buffer, 0, 1024);

	    if (sizehints->flags & USPosition || sizehints->flags & PPosition) {
		sprintf(tmp, "{%d %d ", sizehints->x, sizehints->y);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "{-1 -1 ");
	    }

	    if (sizehints->flags & USSize || sizehints->flags & PSize) {
		sprintf(tmp, "%d %d ", sizehints->width, sizehints->height);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 -1 ");
	    }

	    if (sizehints->flags & PMinSize) {
		sprintf(tmp, "%d %d ", sizehints->min_width,
			sizehints->min_height);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 -1 ");
	    }

	    if (sizehints->flags & PMaxSize) {
		sprintf(tmp, "%d %d ", sizehints->max_width,
			sizehints->max_height);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 -1 ");
	    }

	    if (sizehints->flags & PResizeInc) {
		sprintf(tmp, "%d %d ", sizehints->width_inc,
			sizehints->height_inc);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 -1 ");
	    }

	    if (sizehints->flags & PAspect) {
		sprintf(tmp, "%d %d %d %d ",
			sizehints->min_aspect.x,
			sizehints->min_aspect.y,
			sizehints->max_aspect.x,
			sizehints->max_aspect.y);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 -1 -1 -1 ");
	    }

	    if (sizehints->flags & PBaseSize) {
		sprintf(tmp, "%d %d ", sizehints->base_width,
			sizehints->base_height);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1 -1 ");
	    }

	    if (sizehints->flags & PWinGravity) {
		sprintf(tmp, "%d}", sizehints->win_gravity);
		strcat(buffer, tmp);
	    } else {
		strcat(buffer, "-1}");
	    }

	    Tcl_AppendResult(interp, buffer, NULL);
	    return TCL_OK;
	}
	case XA_INTEGER : {
	    int *value;
	    char tmp[32], *buffer;
	    register int i;

	    if (actualFormat != 8 || actualFormat != 16 || actualFormat != 32) {
		Tcl_AppendResult(interp,
				 "xPropertyGet: unknown format for Integer: %d\n",
				 actualFormat, NULL);
		return TCL_ERROR;
	    }
		
	    // Allocate a value.
	    switch (actualFormat) {
	    case 8:
		value = (int *) new char[nitems];
		break;
	    case 16:
		value = (int *) new short[nitems];
		break;
	    case 32:
		value = (int *) new long[nitems];
		break;
	    default:
		Error(EL_ERROR, argv[0], "Unrecognized property format.", NULL);
	    }
		
	    // Try to allocate 32 bytes per item for the string rep
	    buffer = new char[nitems * 32];

	    strcat(buffer, "{");
	    for (i=0 ; i<nitems ; i++) {
		switch (actualFormat) {
		case  8 :
		    sprintf(tmp, "%d ", ((char*) value)[i]);
		    strcat(buffer, tmp);
		    break;
		case 16 :
		    sprintf(tmp, "%d ", ((short*) value)[i]);
		    strcat(buffer, tmp);
		    break;
		case 32 :
		    sprintf(tmp, "%ld ", ((long*) value)[i]);
		    strcat(buffer, tmp);
		    break;
		}
	    }

	    strcat(buffer, "}");

	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(buffer);
	    XFree(value);
	    return TCL_OK;
	}
	case XA_RECTANGLE : {
	    XRectangle *rects;
	    char tmp[32], *buffer;
	    register int i;
	    int num = (nitems * 32) / sizeof(XRectangle);

	    rects = (XRectangle *) prop;
	    buffer = new char[nitems * 32];

	    strcat(buffer, "{");

	    for (i=0 ; i<num ; i++) {
		sprintf(tmp, "{%d %d %d %d}", 
			rects[i].x, rects[i].y,
			rects[i].width, rects[i].height);
	    }

	    strcat(buffer, "}");

	    Tcl_AppendResult(interp, buffer, NULL);
	    XFree(buffer);
	    return TCL_OK;
	}
	case XA_RGB_COLOR_MAP : {
	    XStandardColormap *cm;
	    char buffer[1024];

	    cm = (XStandardColormap *) prop;

	    sprintf(buffer, "{%d %lu %lu %lu %lu %lu %lu %lu}",
		    cm->colormap, cm->red_max, cm->red_mult,
		    cm->green_max, cm->green_mult,
		    cm->blue_max, cm->blue_mult,
		    cm->base_pixel);
	    Tcl_AppendResult(interp, buffer, NULL);
	    return TCL_OK;
	}
	case XA_CARDINAL : {
	    int *value;
	    char tmp[32], *buffer;
	    register int i;

	    if (actualFormat != 8 || actualFormat != 16 || actualFormat != 32) {
		Tcl_AppendResult(interp,
				 "xPropertyGet: unknown format for Cardinal: %d\n",
				 actualFormat, NULL);
		return TCL_ERROR;
	    }
		
	    switch (actualFormat) {
	    case 8:
		value = (int *) new char[nitems];
		break;
	    case 16:
		value = (int *) new short[nitems];
		break;
	    case 32:
		value = (int *) new long[nitems];
		break;
	    default:
		Error(EL_ERROR, argv[0], "Unrecognized property format.", NULL);
		break;
	    }

	    // Try to allocate 32 bytes per item for the string rep
	    buffer = new char[nitems * 32];

	    strcat(buffer, "{");
	    for (i=0 ; i<nitems ; i++) {
		switch (actualFormat) {
		case  8 :
		    sprintf(tmp, "%u ", ((char*) value)[i]);
		    strcat(buffer, tmp);
		    break;
		case 16 :
		    sprintf(tmp, "%u ", ((short*) value)[i]);
		    strcat(buffer, tmp);
		    break;
		case 32 :
		    sprintf(tmp, "%lu ", ((long*) value)[i]);
		    strcat(buffer, tmp);
		    break;
		}
	    }

	    strcat(buffer, "}");

	    Tcl_AppendResult(interp, tmp, NULL);
	    XFree(buffer);
	    XFree(value);
	    return TCL_OK;
	    
	}
	default :
	    Tcl_AppendResult(interp,
			     "xPropertyGet: %s: unknown property type (format = %d)\n",
			     typeName, actualFormat, NULL);
	    return TCL_ERROR;
	}
    }
#endif 0
    Error(EL_ERROR, "xPropertyGet", "NOT IMPLEMENTED.", NULL);
    Tcl_AppendResult(interp, "xPropertyGet not implemented yet.\n", NULL);
    return TCL_ERROR;
}

static int
xSelectionSet(Tcl_Interp *interp, int argc, char *argv[])
{
    Error(EL_ERROR, "xSelectionSet", "NOT IMPLEMENTED.", NULL);
    Tcl_AppendResult(interp, "xSelectionSet not implemented yet.\n", NULL);
    return TCL_ERROR;
}

//
// xSelectionGet tries to get the named selection into a property on
// Mercator's unmapped window.  We always ask for the selection as text.
//
//	x selection get <selection>
//
//
// This is a WAY DANGEROUS call... it does all kinds of weird shit human
// beings were never intended to do and it may very likely break or hang
// Mercator.  Use at your own risk.
//
static int
xSelectionGet(Tcl_Interp *interp, int argc, char *argv[])
{
#if 0
    Display *dpy = mercator->GetXServer()->GetDisplay();
    Window unmapped = amm->GetUnmappedWindow();
    Atom target;
    Time time;
    Bool onlyIfExists;
    Atom dataProp;
    void (*oldhandler(int));
    int found = 0;

    // We create an atom which names a property for data to go into.
    dataProp = XInternAtom(dpy, "MERC_SELECTION_DATA",
			   onlyIfExists = FALSE);

    // For now, always ask for the data as a String.
    target = XA_STRING;

#include <sys/types.h>
#include <time.h>

    // CurrentTime is DEFINATELY a problem here... (see O'Reilly)

    XConvertSelection(dpy, XA_PRIMARY, target,
		      dataProp, unmapped, time = CurrentTime);
    XFlush(dpy);

    // Now we have to wait for a selection notify event and, if the
    // property member is the same as dataProp, the conversion went
    // fine; if the property member is None, the conversion failed.

    while (!found) {
	if (XCheckWindowEvent() == TRUE) {
	    // Process the thing!

	    fprintf(stderr, "WE GOT THE SELECTION NOTIFICATION!!\n");

	    // reinstall the old signal handler
	    signal(SIG_ALRM, ohandler);
	    found = 1;
	} else {
	    struct itimerval value;

	    ohandler = signal(SIG_ALRM, HandleSelection);

	    value.it_interval.tv_sec  = 0;
	    value.it_interval.tv_usec = 0;
	    value.it_value.tv_sec  = 0;
	    value.it_value.tv_usec = (time(0) * 100000);	// 1/10 sec

	    setitimer(ITIMER_VIRTUAL, &value, NULL);
	}
    }
#endif 0
    Error(EL_ERROR, "xSelectionGet", "NOT IMPLEMENTED.", NULL);
    Tcl_AppendResult(interp, "xSelectionGet not implemented yet.\n", NULL);
    return TCL_ERROR;
}

static int
xWindowSet(Tcl_Interp *interp, int argc, char *argv[])
{
    Error(EL_ERROR, "xWindowSet", "NOT IMPLEMENTED.", NULL);
    Tcl_AppendResult(interp, "xWindowSet not implemented yet.\n", NULL);
    return TCL_ERROR;
}

static int
xWindowGet(Tcl_Interp *interp, int argc, char *argv[])
{
    Error(EL_ERROR, "xWindowGet", "NOT IMPLEMENTED.", NULL);
    Tcl_AppendResult(interp, "xWindowGet not implemented yet.\n", NULL);
    return TCL_ERROR;
}

//	x warp <win> [<x> <y>]
//      x warp <XtObject>
//
static int
xWarp(Tcl_Interp *interp, int argc, char *argv[])
{ 
    XtObject *xtobj;

    if (argc == 5) {
	xtobj = mercator->GetAppModelMgr()->GetXtObject(argv[2]);
	mercator->GetXServer()->MoveMouse(xtobj, atoi(argv[3]), atoi(argv[4]));
	return TCL_OK;
    } else if (argc == 3) {
	xtobj = mercator->GetAppModelMgr()->GetXtObject(argv[2]);
	mercator->GetXServer()->MoveMouse(xtobj);
	return TCL_OK;
    } else {
	Tcl_AppendResult(interp,"usage: xwin warp xtobj <x y>", NULL);
	return TCL_ERROR;
    }

#ifdef HERE
    Display *dpy = mercator->GetXServer()->GetDisplay();
    Window win;
    int x = 0, y = 0;

    win = atol(argv[2]);

    if (argc == 5) {
	x = atoi(argv[3]);
	y = atoi(argv[4]);
    }

    XRaiseWindow(dpy, win);
    XWarpPointer(dpy,		// display
		 None,		// source window (None == don't care)
		 win,		// dest window
		 0, 0,		// source window x, y (don't care)
		 0, 0,		// source window width, height (don't care)
		 x, y);		// dest window x, y
    XFlush(dpy);

    Tcl_AppendResult(interp, argv[2], NULL);
    return TCL_OK;
#endif HERE
}

//	x raise <objid>
static int
xRaise(Tcl_Interp *interp, int argc, char *argv[])
{
    Display *dpy = mercator->GetXServer()->GetDisplay();
    Client *client = mercator->GetAppModelMgr()->GetCurrentClient();
    Object id;
    XtObject *obj, *top;
    Window win = None;
    static char tmp[256];

    //id = (Object) atol(argv[2]);

    obj = mercator->GetAppModelMgr()->GetXtObject(argv[2]);

#if 0
    if ((obj = client->GetXtObject(id)) == NULL) {
	Tcl_AppendResult(interp, "xRaise: no such object %lu exists\n",
			 id, NULL);
	return TCL_ERROR;
    }
#endif 0

    if (!obj) {
	Tcl_AppendResult(interp, "xRaise: no such object %s exists\n",
			 argv[2], NULL);
	return TCL_ERROR;
    }
#if 0
    //
    // Find the toplevel widget of this guy.
    //
    top = mercator->GetXServer()->GetTopWindow(obj);
    win = top->GetWindowId();
#endif

    //
    // This reeks of cheese.  The way we're determining top level status
    // is unreliable and needs to be fixed.  Right now we're using a
    // template attribute ("shell") to tell us what's a toplevel and what's
    // not.  This breaks on applications that have their class set to something
    // that's the same as a normal, non-shell widget class (such as an
    // app named "Text").
    //
    // When we say "raise" we really mean raise the *real* toplevel,
    // regardless of what our (currently somewhat screwey) model says
    // is a toplevel.
    //
    // This is very expensive, but ensures that we'll get a real toplevel.
    //
    int done = FALSE;
    Window parent = None, root = None, *children = NULL;
    unsigned nchildren = 0;

    win = obj->GetWindowId();
    
    while (!done) {
	if (XQueryTree(dpy, win, &root, &parent,
		       &children, &nchildren) == 0) {
	    Error(EL_ERROR, "xRaise", "XQueryTree failed!", NULL);
	    done = TRUE;
	}

	if (parent == root) {
	    // We're at the top, pass out "tester."
	    done = TRUE;
	} else {
	    win = parent;
	}

	if (children)
	    XFree(children);
    }

    //
    // Now get its window ID and raise it.
    //
    XRaiseWindow(dpy, win);
    XFlush(dpy);

    sprintf(tmp, "Raised window ID %x", win);
    Tcl_AppendResult(interp, tmp, NULL);
    return TCL_OK;
}

//	x lower <winid>
static int
xLower(Tcl_Interp *interp, int argc, char *argv[])
{
    Display *dpy;
    Window win;

    dpy = mercator->GetXServer()->GetDisplay();
    win = atol(argv[2]);

    Error(EL_ERROR, "xLower", "NOT IMPLEMENTED.", NULL);
    Tcl_AppendResult(interp, "xLower not implemented yet.\n", NULL);
    return TCL_ERROR;
}

//
// xwin is the general command proc which allows you to do low-level
// x-related things.  It does a whole lot of stuff based on the second
// argument, to keep the command namespace from getting cluttered.
//
//    USAGE:
//	x property set <winid> <property> <value>
//	x property get <winid> <property>
//	x selection set <selection> <value>
//	x selection get <selection>
//	x window set <winid> <attribute> <value>
//	x window get <winid> <attribute>
//	x warp <x> <y>
//	x raise <winid>
//	x lower <winid>
//

int
xwin(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    Action       *action;
    XtObject     *obj = NULL;
    int		 returnVal;

    if ((argc == 6) && (argv[1][0] == 'p') && (argv[2][0] == 's') &&
	(strcmp(argv[1], "property") == 0) && 
	(strcmp(argv[2], "set") == 0)) {
        //// x property set <winid> <property> <value>
	returnVal = xPropertySet(interp, argc, argv);
    } else if ((argc == 6) && (argv[1][0] == 'p') && (argv[2][0] == 'g') &&
	       (strcmp(argv[1], "property") == 0) &&
	       (strcmp(argv[2], "get") == 0)) {
        //// x property get <winid> <property>
	returnVal = xPropertyGet(interp, argc, argv);
    } else if ((argc == 5) && (argv[1][0] == 's') && (argv[2][0] == 's') &&
	       (strcmp(argv[1], "selection") == 0) && 
	       (strcmp(argv[2], "set") == 0)) {
        //// x selection set <selection> <value>
	returnVal = xSelectionSet(interp, argc, argv);
    } else if ((argc == 4) && (argv[1][0] == 's') && (argv[2][0] == 'g') &&
	       (strcmp(argv[1], "selection") == 0) &&
	       (strcmp(argv[2], "get") == 0)) {
        //// x property get <selection>
	returnVal = xSelectionGet(interp, argc, argv);
    } else if ((argc == 6) && (argv[1][0] == 'w') && (argv[2][0] == 's') &&
	       (strcmp(argv[1], "window") == 0) && 
	       (strcmp(argv[2], "set") == 0)) {
        //// x window set <winid> <attribute> <value>
	returnVal = xWindowSet(interp, argc, argv);
    } else if ((argc == 5) && (argv[1][0] == 'w') && (argv[2][0] == 'g') &&
	       (strcmp(argv[1], "window") == 0) &&
	       (strcmp(argv[2], "get") == 0)) {
        //// x window set <winid> <attribute> <value>
	returnVal = xWindowGet(interp, argc, argv);
    } else if (((argc == 5)||(argc == 3)) && (argv[1][0] == 'w') &&
	       (strcmp(argv[1], "warp") == 0)) {
	//// x warp xtobj <x y>
	returnVal = xWarp(interp, argc, argv);
    } else if ((argc == 3) && (argv[1][0] == 'r') &&
	       (strcmp(argv[1], "raise") == 0)) {
	// x raise <winid>
	returnVal = xRaise(interp, argc, argv);
    } else if ((argc == 3) && (argv[1][0] == 'l') &&
	       (strcmp(argv[1], "lower") == 0)) {
	// x lower <winid>
	returnVal = xLower(interp, argc, argv);
    } else {
	Tcl_AppendResult(interp, 
			 "Usage: xwin property set <property> <value>\n",
			 "       xwin property get <property>\n",
			 "       xwin selection set <selection> <value>\n",
			 "       xwin selection get <selection>\n",
			 "       xwin window set <winid> <attr> <value>\n",
			 "       xwin window get <winid> <attr>\n",
			 "       xwin warp <x> <y>\n",
			 "       xwin warp <XtObject>\n",
			 "       xwin raise <winid>\n",
			 "       xwin lower <winid>\n",
			 NULL);
	returnVal = TCL_ERROR;
    }

    return returnVal;
}

// usage:	short2long <xtobject_name>

int
short2long(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    XtObject     *obj = NULL;

    if (argc != 2) {
	Tcl_AppendResult(interp,
			 "Usage: short2long <xtobject_name>\n",
			 NULL);
	return TCL_ERROR;
    }

    obj = mercator->GetAppModelMgr()->GetXtObject(argv[1]);

    if (!obj) {
	Tcl_AppendResult(interp,
			 "short2long: no such XtObject, %s, exists.\n",
			 argv[1], NULL);
	return TCL_ERROR;
    }

    Tcl_AppendResult(interp, "{", obj->GetLongName(), "}", NULL);
    return TCL_OK;
}


// usage:  long2short <xrm_name>

int
long2short(ClientData data, Tcl_Interp *interp, int argc, char *argv[])
{
    XtObject *obj = NULL;

    if (argc != 2) {
	Tcl_AppendResult(interp,
			 "Usage:  long2short <xrm_name>\n",
			 NULL);
	return TCL_ERROR;
    }

    obj = mercator->GetAppModelMgr()->GetXtObjectByLongName(argv[1]);

    if (!obj) {
#if 0
	Tcl_AppendResult(interp,
			 "long2short:  no XtObject, %s, exists.\n",
			 argv[1], NULL);
	return TCL_ERROR;
#endif  
	Tcl_AppendResult(interp,"NULL",NULL);
        return TCL_OK;
    }

    Tcl_AppendResult(interp, obj->GetUniqueName(), NULL);
    return TCL_OK;
}

int
connect(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    AppModelMgr *amm = mercator->GetAppModelMgr();
    XID          xid;
    long         lxid;
    const char  *clientName;
    Client      *cl;
    Display     *dpy = mercator->GetXServer()->GetDisplay();


    if (argc != 2) {
	Tcl_AppendResult(interp, "Usage: connect <winid>", NULL);
	return TCL_ERROR;
    }

    // Get window number from argument (ick)
    // Assume hexadecimal notation for window id argument.
    lxid = strtol(argv[1], (char **)NULL , 16);
    if (!lxid) {
	Tcl_AppendResult(interp, "Invalid arg window id", NULL);		     
	return TCL_ERROR;
    }
    else xid = (XID)lxid;

    Error(EL_INFORMATIONAL, argv[0],
	    "connect: NOTE THAT THE FUNCTIONALITY OF THIS ROUTINE HAS CHANGED.\n"
	    "	It now only issues *requests for connection; it does not\n"
	    "	return a client name.  The ClientCreation action should be\n"
	    "	invoked when the connection has been established and the new\n"
	    "	client instance is created however.", NULL);
    
    mercator->GetRAPListener()->InitiateConnection(*(mercator->GetXServer()),
						   xid);
#if 0
    if (!amm->RequestConnectionToApp(dpy, xid)) {
	Tcl_AppendResult(interp, "AppModelMgr::RequestConnectionToApp failed.",
			 NULL);
	return TCL_ERROR;
    }
#endif
    return TCL_OK;
}

// returns specified value from the Config object
int
configVal(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    char buf[10]; // shouldn't have more than 10e9 error levels
    
    if ((argc == 2) && (!strcmp(argv[1], "ErrorLevel"))) {
	//	if (Config::ErrorLevel() < 9999) {
	if (Config::GetInteger("errorLevel") < 9999) {
	    //sprintf(buf, "%d", Config::ErrorLevel());
	    sprintf(buf, "%d", Config::GetInteger("errorLevel"));
	    Tcl_AppendResult(interp, buf, NULL);
	    return TCL_OK;
	} else {
	    Tcl_AppendResult(interp, "ErrorLevel greater than 9999!", NULL);
	}
    } else {
	Tcl_AppendResult(interp, "Unrecognized parameter(s).", NULL);
    }
    return TCL_ERROR;
}

// disconnects from the console
int
disconnectPipe(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    Error(EL_DEBUG, "Interp::disconnectPipe",
	  "Disconnecting from the console.", NULL);

    if(mercator->GetInterp()->DisconnectPipe() != -1)
	return TCL_OK;
    else
	return TCL_ERROR;
}

// connects to the console
int
connectPipe(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    Error(EL_DEBUG, "Interp::connectPipe", "Connecting to the console.",
	  NULL);

    if(mercator->GetInterp()->ConnectPipe() != -1)
	return TCL_OK;
    else
	return TCL_ERROR;
}

//
// Just a wrapper around the Error call for tcl.
//
int
ErrorShell(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    if(argc < 2) {
	Tcl_AppendResult(interp, "Usage: Error <error level> <name> [<msg1>",
			 " <msg2> ...]\n",
			 "       Error level", NULL);
	return TCL_ERROR;
    }

    // get the error level. This is kind of inefficient. Do we want a
    // dictionary here?
    int el;

    if(!strcmp(argv[1], "level")) {
	if(argc == 2) {
	    el = ErrorLevel();

	    char buf[10];
	    sprintf(buf, "%d", el);
	    Tcl_AppendResult(interp, buf, NULL);
	    return TCL_OK;
	}
	else if (argc == 3) {
	    ErrorLevel((ErrorLevels)atoi(argv[2]));
	    return TCL_OK;
	}
    }
    else if(!strcmp(argv[1], "EL_NO_MESSAGES"))
	el = EL_NO_MESSAGES;
    else if(!strcmp(argv[1], "EL_ABORT"))
	el = EL_ABORT;
    else if(!strcmp(argv[1], "EL_FATAL"))
	el = EL_FATAL;
    else if(!strcmp(argv[1], "EL_ERROR"))
	el = EL_ERROR;
    else if(!strcmp(argv[1], "EL_WARNING"))
	el = EL_WARNING;
    else if(!strcmp(argv[1], "EL_STATUS"))
        el = EL_STATUS;
    else if(!strcmp(argv[1], "EL_INFORMATIONAL"))
	el = EL_INFORMATIONAL;
    else if(!strcmp(argv[1], "EL_DEBUG"))
	el = EL_DEBUG;
    else if(!strcmp(argv[1], "EL_ALL_MESSAGES"))
	el = EL_ALL_MESSAGES;
    else {
	Tcl_AppendResult(interp, "Unknown error level: ", argv[1], NULL);
	return TCL_ERROR;
    }

    // use mString to build a big string from all the arguments
    mString errStr;
    int i;

    for(i = 3; i<argc; i++) {
	errStr += (const char *)argv[i];
    }
	
    Error(el, argv[2], (const char *) errStr, NULL);
    
    return TCL_OK;
}


//
// Get back some audio device info.
//
int
audio(ClientData data, Tcl_Interp * interp, int argc, char * argv[])
{
    if (argc < 2) {
        Tcl_AppendResult(interp, "Usage: ",
			 argv[0], " name", NULL);
        return TCL_ERROR;
    }

    if (strcmp(argv[1], "name") == 0) {
        Tcl_AppendResult(interp, mercator->GetAudio()->GetAudioName(), 
                         NULL);
    }
    else{
        Tcl_AppendResult(interp, "Usage: ",
			 argv[0], " name",
                         NULL);
        return TCL_ERROR;
    }

    return TCL_OK;
}
