// -*-Mode:C++-*-
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
// The main for the Mercator project
// This basically starts up everything and lets them wait
// for input from the desired applications. 
// 

//
// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Mercator.cc,v $
// $Revision: 6.36 $
// $Date: 1995/12/07 19:52:49 $
//

static const char rcsid[]="@(#) $Header: /net/hm10/keith/proj/mercator/6.0/RCS/Mercator.cc,v 6.36 1995/12/07 19:52:49 keith Exp $";

#include <sys/types.h>
#include <stropts.h>
#include <sys/conf.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <poll.h>
#include <pwd.h>
#include <signal.h>
#include <errno.h>
#include <fstream.h>
#include <strstream.h>

#include "Mercator.h"
#include "XServer.h"
#include "RAPListener.h"
#include "AppModelMgr.h"
#include "Resource.h"
#include "Config.h"
#include "Attribute.h"
#include "DynaLoader.h"
#include "Audio.h"
#include "Braille.h"
#include "Speech.h"
#include "ScreenReader.h"
#include "Interp.h"
#include "Error.h"
#include "Util.h"
#include "Template.h"
#include "FDInterest.h"

//
// We only allow one instance of a Mercator object to exist in our
// address space.
//
int Mercator::initialized = 0;

FILE *Mercator::outFile = NULL;


// This procedure starts up the connections and everything else that is
// needed to get mercator to run

Mercator::Mercator() :
    xServer(NULL),
    modelMgr(NULL),
    procMgr(NULL),
    audio(NULL),
    speech(NULL),
    interp(NULL),
    configFilename(NULL),
    attribFilename(NULL),
    tmpErrorLevel(-1),
    tmpErrorOutput(NULL),
    tmpErrorOutputVal(EO_STDERR),
    infd(0), outfd(1)	// initialize to stdin/out, may be connected to
                        // named pipes later.   -wl
{ 
    
    if (initialized++ > 0) {
	Error(EL_ABORT, "Mercator::Mercator",
	      "Cannot allow more than one 'Mercator' instance.\n",
	      "\tThis is an unrecoverable error ",
	      "as it indicates a programmer logic error.", NULL);
	return;
    }

    // save all error/debug output in a tmp file until we can determine if
    // they should be sent to console or not.
    Mercator::SaveMsg();
    
    char *speaker;

    // start up the applications manager - this is the list of all applications
    modelMgr = new AppModelMgr();

    // open the display and create the XServer
    xServer = new XServer();
    xServer->DoXOpenDisplay();
    xServer->AddFDInterest(xServer->GetConnectionNumber());
    xServer->SetIOMask(xServer->GetConnectionNumber(), FDInterest::ARead);
    xServer->SolicitSubstructureNotify();

    // New process manager
    procMgr = new ProcessManager();
    
    // New interpreter - note we use stdin/stdout by default...
    interp = new Interp(infd, outfd);

    // Use this interpreter for all field data
    FieldData::SetInterpreter(*interp);
 
    // Load the error handlers
    xServer->LoadErrorHandlers();

    // Initialize the ResourceTypeManager
    InitializeResourceTypeManager();
    
    // Open a RAP listener
    rapListener = new RAPListener(*xServer);

    // And have it register itself.
    if (!rapListener->Register(*xServer)) {
	Error(EL_FATAL, "Mercator::Mercator",
	      "Couldn't register RAP protocol.", NULL);
    }
}

Mercator::~Mercator()
{
    --initialized;

    // RemoveNamedPipes();    // this is done in Quit()

    delete modelMgr;
    delete speech;
    delete audio;
    delete braille;
    delete xServer;
    delete rapListener;
    delete procMgr;

    delete [] configFilename;
    delete [] attribFilename;
}

void
Mercator::RemoveNamedPipes()
{
    unlink("/tmp/to-console");
    unlink("/tmp/from-console");
}

void
Mercator::CreateNamedPipes() {

    Error(EL_DEBUG, "Mercator::CreateNamedPipes", "creating named pipes", NULL);
    
    if (mkfifo("/tmp/to-console", (S_IREAD | S_IEXEC | S_IWRITE)) != 0) {
	Error(EL_ERROR, "Mercator::CreateNamedPipes",
	      "creating /tmp/from-console", Perror(), NULL);
    }
    if (mkfifo("/tmp/from-console", (S_IREAD | S_IEXEC | S_IWRITE)) != 0) {
	Error(EL_ERROR, "Mercator::CreateNamedPipes",
	      "creating /tmp/from-console", Perror(), NULL);
    }
}


//
// Call ConnectConsole(long) with a hardcoded default wait value.
//
int
Mercator::ConnectConsole() {
    int ret = ConnectConsole(5L);
    return ret;
}

//
// This function should only be called after CreateNamedPipes has
// been called.  -wl
//
int
Mercator::ConnectConsole(long waitTime) {

    int ret = FALSE;
    int oldinfd = infd;

    Error(EL_DEBUG, "Mercator::ConnectConsole",
	  "connecting to named pipes", NULL);
    
    CreateNamedPipes();
    
    if ((infd = open("/tmp/from-console", O_RDONLY|O_NDELAY)) < 0) {
	Error(EL_ERROR, "Mercator::Mercator",
	      "Could not open /tmp/from-console fifo: ", Perror(), NULL);
	infd = 0;
    }

    Error(EL_DEBUG, "Mercator::ConnectConsole",
	  "Blocking on a ping from console", NULL);
    
    /* This is a workaround for a Solaris 2.5 bug. I have detected no
       side effects for 2.4. */
#ifdef SUNOS_5_5
    if(ioctl(infd, I_SRDOPT, RNORM) == -1){
	perror("ioctl");
	exit(1);
    }
    Error(EL_DEBUG, "Mercator::ConnectConsole", "called ioctl", NULL);
#endif /* SUNOS_5_5 */
    
    /* wait a bit to see if console is starting up */
    struct pollfd fds[2];

    fds[0].fd = infd;
    fds[0].events = POLLIN;

    int res = poll(fds, 1, waitTime*1000);

    if (res < 0) {
	Error(EL_ERROR, "Mercator::ConnectConsole", "poll interrupted, ",
	      "giving up on console", NULL);
    }
    else if((fds[0].revents & POLLIN) == POLLIN) {
	/* some one has connect to the pipe */
	Error(EL_DEBUG, "Mercator::ConnectConsole",
	      "ping received from console", NULL);

	char buf[1024];
	
	if (read(infd, buf, 1024) < 0) {
	    Error(EL_ERROR, "Mercator::ConnectConsole",
		  "reading ping message from console", Perror(), NULL);
	    RemoveNamedPipes();
	}
	else { /* try to open the writing pipe */
	    if ((outfd = open("/tmp/to-console", O_WRONLY|O_NDELAY)) < 0) {
		// if ((outfd = open("/tmp/to-console", O_WRONLY)) < 0) {
		Error(EL_ERROR, "Mercator::Mercator",
		      "Could not open to-console fifo: ", Perror(), NULL);
		outfd = 1;
		infd = 0;
		RemoveNamedPipes();
	    }
	    else {
		ret = TRUE; /* successfully connected to console */
		Error(EL_DEBUG, "Mercator::ConnectConsole",
		      "Successfully connected to console", NULL);
	    }
	}
    }

    if (ret == FALSE) { /* cannot connect to the console */
	infd = 0;
	outfd = 1;
    }

    Interp *myInterp = GetInterp();

    if (myInterp != NULL) {
	myInterp->SetDescriptors(infd, outfd);
    } else {
	Error(EL_DEBUG, "Mercator::ConnectConsole",
	      "cannot get Interp", NULL);
    }

    ioctl(outfd, I_FLUSH, FLUSHW);
    return ret;
}

//
// Start the console if all the conditions apply
//
void
Mercator::StartConsole()
{

    /* get console name and start it */
    mListVal<mString> list = Config::GetStringList("console");
    
    if (list.entries() <= 0) {
	Error(EL_DEBUG, "mercator", "console is not specified", NULL);
    }
    else {
	Error(EL_DEBUG, "mercator", "Starting console.", NULL);
	
	mercator->ExecList("console");
	
	/* connect to console */
	long cWait = Config::GetInteger("consoleWait");

	int res;
	
	if(cWait < 0)
	    res = mercator->ConnectConsole();
	else
	    res = mercator->ConnectConsole(cWait);
    }    
}

//
// This function executes a list named by listName as a command.
//
// Returns: 0 if the list listName is found in the config file,
//            there is no error.
//          1 if listName is not found in the config file and
//         -1 if there is any error
//
int
Mercator::ExecList(const char * listName) {

    mListVal<mString> list = Config::GetStringList((char *)listName);
    mListValIterator<mString> iter(list);
    int size = list.entries();
    
    // check server
    if (size > 0) {

	Error(EL_DEBUG, "execList", "Starting ", listName, NULL);

	// deal with environments if any.
	// first build the variable name (4 is from "Env\0" postfix) 
	char * envName = new char[strlen(listName) + 4];
	strcpy(envName, listName);
	strcat(envName, "Env");

	mListVal<mString> envList = Config::GetStringList((char *)envName);
	mListValIterator<mString> envIter(envList);
	int envSize = envList.entries();

	//
	// set the environment variable associated with the program to
	// be executed.  Note that the semantics of putenv require that
	// we pass in a malleable string.  Hence we use the char * converter
	// from mString.
	//
	if (envSize > 0) {
	    int j;
    
	    for(j=0; j<envSize; j++) {
		envIter.next();
		if (putenv(envIter.current()) != 0) {
		    Error(EL_ERROR, "execList",
			  "unable to set an environment variable for ",
			  listName, NULL);
		    return -1;
		}
	    }
	}	    

	/* now build an argv and execute the program */
	char **margv = new char * [(sizeof(char *) * size) + 1];
	
	int i;
    
	for(i=0; i<size; i++) {
	    iter.next();
	    // We're using char * conversion here; possible memory leak?
	    margv[i] = iter.current();
	}

	margv[i] = NULL;
	
	if (mercator->GetProcessManager()->StartProcess(size, margv, NULL,
							TRUE, NULL,
							NULL, NULL)) {
	    delete [] margv;
	    return 0;		// success
	} else {
	    delete [] margv;
	    return 1;		// success
	}
    } else {
	return 1;
    }
}

//
// DESCRIPTION OF ARGUMENTS
//	-d	Set debugging output level.  This option takes a numeric
//		argument which indicates the error level.  The argument
//		ranges from 1 to 8; a level of n shuts off all messages
//		with a priority less than n.  The priorities are:
//			0	No messages (turn off all output)
//			1	Abort messages (unrecoverable errors, coredump)
//			2	Fatal messages (unrecoverable errors, no dump)
//			3	Error messages (internal inconsistencies)
//			4	Warning messages (misuse errors)
//			5	Status messages (checkpoint messages)
//			6	Informational messages (helpful ? output)
//			7	Debug messages (trace output)
//			8	All messages (print everything)
//
//	-o	Set where debugging output goes to.  Currently supported
//		options are:
//			ignore	Throw away output.
//			stderr	Write output to stderr.
//			syslog	Use syslog to process output.
//
//	-f	Provide the location of the Mercator config file.
//	-a	Provide the location of the Mercator attrib file.
//
int
Mercator::ProcessArgs(int argc, char *argv[])
{
    extern char *optarg;
    extern int optind;
    int c;
    int errFlag = 0;
    int errorLevel;

    //
    // Note that with the exception of the name of the config file, we save off
    // the values of all of these in "temporary" variables, so that they won't
    // get overwritten when we parse the config file.
    //
    while ((c = getopt(argc, argv, "d:o:f:a:")) != EOF) {
	switch (c) {
	case 'd' :
	    tmpErrorLevel = atoi(optarg);
	    if (tmpErrorLevel < 0 || tmpErrorLevel > 8)
		errFlag++;
	    
	    break;
	    
	case 'o' :
	    delete [] tmpErrorOutput;
	    if (strcmp(optarg, "IGNORE") == 0) {
		tmpErrorOutput = Strdup(optarg);
	    } else if (strcmp(optarg, "STDERR") == 0) {
		tmpErrorOutput = Strdup(optarg);
	    } else if (strcmp(optarg, "SYSLOG") == 0) {
		tmpErrorOutput = Strdup(optarg);
	    } else if (strcmp(optarg, "CONSOLE") == 0) {
		tmpErrorOutput = Strdup(optarg);
	    } else {
		errFlag++;
	    }
	    break;

	case 'f' :
	    delete [] configFilename;
	    configFilename = Strdup(optarg);
	    break;

	case 'a' :
	    delete [] attribFilename;
	    attribFilename = Strdup(optarg);
	    break;
	    
	case '?' :
	default :
	    errFlag++;
	}
    }
    
    if (errFlag) {
	Error(EL_FATAL, argv[0], "Usage: "
	      " [-d 0-8][-o ignore|stderr|syslog][-f configfile][-a attribfile]", NULL);
	return 0;
    }
    
    return 1;
}

//
// ProcessEnvVariables.  Look for the vars that mercator uses:
//
//	MERCATOR_CONFIG			Filename of config file.
//	MERCATOR_ERROR_LEVEL	Debugging output control.
//	MERCATOR_ERROR_OUTPUT	Where to send output.
//
void
Mercator::ProcessEnvVariables()
{
    char *tmp = NULL;

    //
    // Everything except for the config/attrib file name is stored in a
    // "temporary" so
    // that we can override the values set by the config file.
    //
    if ((tmp = getenv("MERCATOR_ATTRIB")) != NULL) {
	delete [] attribFilename;
	attribFilename = Strdup(tmp);
    }

    if ((tmp = getenv("MERCATOR_CONFIG")) != NULL) {
	delete [] configFilename;
	configFilename = Strdup(tmp);
    }

    if ((tmp = getenv("MERCATOR_ERROR_LEVEL")) != NULL) {
	tmpErrorLevel = atoi(tmp);
	if (tmpErrorLevel < 0 || tmpErrorLevel > 8)
	    Error(EL_ERROR, "Mercator::ProcessEnvVariables",
		  "Value of MERCATOR_ERROR_LEVEL must be between 0 and 8.",
		  NULL);
    }

    if ((tmp = getenv("MERCATOR_ERROR_OUTPUT")) != NULL) {
	if (strcmp(tmp, "IGNORE") == 0) {
	    tmpErrorOutput = Strdup(tmp);
	} else if (strcmp(tmp, "STDERR") == 0) {
	    tmpErrorOutput = Strdup(tmp);
	} else if (strcmp(tmp, "SYSLOG") == 0) {
	    tmpErrorOutput = Strdup(tmp);
	} else if (strcmp(tmp, "CONSOLE") == 0) {
	    tmpErrorOutput = Strdup(tmp);
	} else {
	    Error(EL_ERROR, "Mercator::ProcessEnvVariables",
		  "Value of MERCATOR_ERROR_OUTPUT must be either "
		  "IGNORE, STDERR, or SYSLOG.", NULL);
	}
    }
}

//
// After all configuration information has been loaded via ProcessEnvVariables(),
// ProcessArgs(), and ProcessConfigFile(), we call ProcessDefaults() to make all
// of the configuration/default information take effect.
//
// For values which can be set via environment variable or command line flag
// in addition to config file, we store those values off so that we can give them
//priority here.
//
void
Mercator::ProcessDefaults()
{
    FieldData *fd = NULL;
    
    //
    // Set the error level and error output.  If we previously got a value
    // from an env var or flag, it takes precedence.
    //
    if (tmpErrorLevel != -1)
	Config::InstallIntegerDecl(-1, "errorLevel", tmpErrorLevel);
    //Config::ErrorLevel(tmpErrorLevel);

    if (tmpErrorOutput != NULL)
	Config::InstallStringDecl(-1, "errorOutput", tmpErrorOutput);
    //	Config::ErrorOutput(tmpErrorOutput);

    // We save the user defined errorOutput value temporarily and save
    // all error/debug messages in a tempfile until we can determine
    // if the console is up or not. Then we will set the error output,
    // after reprinting the saved error/debug messages from the tempfile.
    //
    if ((fd = Config::GetDefaultByName("errorOutput")) == NULL) {
	tmpErrorOutputVal = EO_STDERR;
    } else if (fd->Tag() != STRING) {
	tmpErrorOutputVal = EO_STDERR;
	Error(EL_ERROR, "Mercator::ProcessDefaults",
	      "errorOutput is not a String, setting to EO_STDERR.", NULL);
    } else if (strcmp(Config::GetString("errorOutput"), "IGNORE") == 0) {
	tmpErrorOutputVal = EO_IGNORE;
    } else if (strcmp(Config::GetString("errorOutput"), "STDERR") == 0) {
	tmpErrorOutputVal = EO_STDERR;
    } else if (strcmp(Config::GetString("errorOutput"), "SYSLOG") == 0) {
	tmpErrorOutputVal = EO_SYSLOG;
    } else if (strcmp(Config::GetString("errorOutput"), "CONSOLE") == 0) {
	tmpErrorOutputVal = EO_CONSOLE;
    } else {
	tmpErrorOutputVal = EO_STDERR;
	Error(EL_ERROR, "Mercator::ProcessDefaults",
	      "Unknown value for error output: ",
	      Config::GetString("errorOutput"), NULL);
    }

    if ((fd = Config::GetDefaultByName("errorLevel")) == NULL) {
	ErrorLevel((ErrorLevels) 8);
    } else if (fd->Tag() != INTEGER) {
	ErrorLevel((ErrorLevels) 8);
	Error(EL_ERROR, "Mercator::ProcessDefaults",
	      "errorLevel is not an Integer, setting to 8.", NULL);
    } else {
	ErrorLevel((ErrorLevels) Config::GetInteger("errorLevel"));
    }
    
    //
    // We don't have to do anything for the template default values.
    //

    //
    // Load any loadable modules...
    //
    // set up the dynamic loader path via the config mechanism
    if ((fd = Config::GetDefaultByName("loadableSearchPath")) == NULL) {
	mListVal<mString> searchPath;
	searchPath.append(".");
	DynaLoader::SearchPath(searchPath);
    } else if (fd->Tag() != STRINGLIST) {
	mListVal<mString> searchPath;
	searchPath.append(".");
	DynaLoader::SearchPath(searchPath);
	Error(EL_ERROR, "Mercator::ProcessDefaults",
	      "loadableSearchPath is not a StringList, setting to (.).", NULL);
    } else {
	DynaLoader::SearchPath(Config::GetStringList("loadableSearchPath"));
    }
    
    //
    // SPEECH
    //
    const char *speechLoadable = NULL;
    if (((fd = Config::GetDefaultByName("speechLoadable")) != NULL) &&
	 (fd->Tag() != STRING)) {
	Error(EL_ERROR, "Mercator::ProcessDefaults",
	      "speechLoadable is not a String, ignoring...", NULL);
    } else {
	speechLoadable = Config::GetString("speechLoadable");
    }

    int speechError = 0;

    if (speechLoadable) {
	DynaLoader::Handle dynaHandle;
	Speech *(*initProc)() = NULL;
	char *loadableName = new char[strlen(speechLoadable) +
				      strlen(".so") + 1];
	char *newFuncName = new char[strlen("new") +
				      strlen(speechLoadable) + 1];

	strcpy(loadableName, speechLoadable);
	strcat(loadableName, ".so");
	
	strcpy(newFuncName, "New");
	strcat(newFuncName, speechLoadable);
	
	if ((dynaHandle = DynaLoader::Load(loadableName)) == 0) {
	    speechError = 1;
	} else if ((initProc = (Speech *(*)())
		    DynaLoader::Symbol(dynaHandle, newFuncName)) == 0) {
	    speechError = 1;
	} else if ((speech = (*initProc)()) == NULL) {
	    speechError = 1;
	} else {
	    Error(EL_DEBUG, "Mercator::ProcessDefaults", "Successfully loaded ",
		  loadableName, " (", speech->GetSpeechName(), ")", NULL);
	}
    }

    if (speechError || !speechLoadable) {
	Error(EL_DEBUG, "Mercator::ProcessDefaults",
	      "Using generic speech.", NULL);
	speech = new Speech();
    }

    // AUDIO
    const char *audioLoadable = NULL;
    if (((fd = Config::GetDefaultByName("audioLoadable")) != NULL) &&
	 (fd->Tag() != STRING)) {
	Error(EL_ERROR, "Mercator::ProcessDefaults",
	      "audioLoadable is not a String, ignoring...", NULL);
    } else {
	audioLoadable = Config::GetString("audioLoadable");
    }

    int audioError = 0;

    if (audioLoadable) {
	DynaLoader::Handle dynaHandle;
	Audio *(*initProc)() = NULL;
	char *loadableName = new char[strlen(audioLoadable) +
				      strlen(".so") + 1];
	char *newFuncName = new char[strlen("new") +
				      strlen(audioLoadable) + 1];

	strcpy(loadableName, audioLoadable);
	strcat(loadableName, ".so");
	
	strcpy(newFuncName, "New");
	strcat(newFuncName, audioLoadable);
	
	if ((dynaHandle = DynaLoader::Load(loadableName)) == 0) {
	    audioError = 1;
	} else if ((initProc = (Audio *(*)())
		    DynaLoader::Symbol(dynaHandle, newFuncName)) == 0) {
	    audioError = 1;
	} else if ((audio = (*initProc)()) == NULL) {
	    audioError = 1;
	} else {
	    Error(EL_DEBUG, "Mercator::ProcessDefaults", "Successfully loaded ",
		  loadableName, " (", audio->GetAudioName(), ")", NULL);
	}
    }

    if (audioError || !audioLoadable) {
	Error(EL_DEBUG, "Mercator::ProcessDefaults",
	      "Using generic audio.", NULL);
	audio = new Audio();
    }

    // Braille
    const char *brailleLoadable = NULL;
    if (((fd = Config::GetDefaultByName("brailleLoadable")) != NULL) &&
	 (fd->Tag() != STRING)) {
	Error(EL_ERROR, "Mercator::ProcessDefaults",
	      "brailleLoadable is not a String, ignoring...", NULL);
    } else {
	brailleLoadable = Config::GetString("brailleLoadable");
    }

    int brailleError = 0;

    if (brailleLoadable) {
	DynaLoader::Handle dynaHandle;
	Braille *(*initProc)() = NULL;
	char *loadableName = new char[strlen(brailleLoadable) +
				      strlen(".so") + 1];
	char *newFuncName = new char[strlen("new") +
				      strlen(brailleLoadable) + 1];

	strcpy(loadableName, brailleLoadable);
	strcat(loadableName, ".so");
	
	strcpy(newFuncName, "New");
	strcat(newFuncName, brailleLoadable);
	
	if ((dynaHandle = DynaLoader::Load(loadableName)) == 0) {
	    brailleError = 1;
	} else if ((initProc = (Braille *(*)())
		    DynaLoader::Symbol(dynaHandle, newFuncName)) == 0) {
	    brailleError = 1;
	} else if ((braille = (*initProc)()) == NULL) {
	    brailleError = 1;
	} else {
	    Error(EL_DEBUG, "Mercator::ProcessDefaults",
		  "Successfully loaded ", loadableName, " (",  ")", NULL);
	}
    }

    if (brailleError || !brailleLoadable) {
	Error(EL_DEBUG, "Mercator::ProcessDefaults",
	      "Using generic braille.", NULL);
	braille = new Braille();
    }
    
    // Genovations Keypad
    const char *keypadLoadable = NULL;
    if (((fd = Config::GetDefaultByName("keypadLoadable")) != NULL) &&
	 (fd->Tag() != STRING)) {
	Error(EL_ERROR, "Mercator::ProcessDefaults",
	      "keypadLoadable is not a String, ignoring...", NULL);
    } else {
	keypadLoadable = Config::GetString("keypadLoadable");
    }

    int keypadError = 0;

    if (keypadLoadable) {
	DynaLoader::Handle dynaHandle;
	FDInterest *(*initProc)() = NULL;
	char *loadableName = new char[strlen(keypadLoadable) +
				      strlen(".so") + 1];
	char *newFuncName = new char[strlen("new") +
				      strlen(keypadLoadable) + 1];

	strcpy(loadableName, keypadLoadable);
	strcat(loadableName, ".so");
	
	strcpy(newFuncName, "New");
	strcat(newFuncName, keypadLoadable);
	
	if ((dynaHandle = DynaLoader::Load(loadableName)) == 0) {
	    keypadError = 1;
	} else if ((initProc = (FDInterest *(*)())
		    DynaLoader::Symbol(dynaHandle, newFuncName)) == 0) {
	    keypadError = 1;
	} else if (((*initProc)()) == NULL) {
	    keypadError = 1;
	} else {
	    Error(EL_DEBUG, "Mercator::ProcessDefaults",
		  "Successfully loaded ", loadableName, " (",  ")", NULL);
	}
    }

    if (keypadError)
	Error(EL_DEBUG, "Mercator::ProcessDefaults",
	      "Couldn't load keypad stuff", NULL);
    
    //
    // Anything else??
    //
}

//
// ParseConfigFile forwards to Config::ParseConfigFile.  NOTE that if we
// get to here and configFilename is NULL (that is, it hasn't been specified
// via either -f or an environment variable, we first look in
// $HOME/.mercator.config and, if that doesn't exist, in
// ~/.mercator.config and, if that doesn't exist, in
// /opt/GTsonicx/etc/mercator.config
//
int
Mercator::ParseConfigFile()
{
    if (configFilename == NULL) {
	//
	// Case #1 - Look in $HOME/.mercator.config.
	//
	char *homeDirectory = getenv("HOME");
	if (homeDirectory != NULL) {
	    // Build the filename
	    char *filename = new char[strlen(homeDirectory) +
		strlen("/.mercator.config") + 1];
	    strcpy(filename, homeDirectory);
	    strcat(filename, "/.mercator.config");

	    // See if it's accessible
	    if (access(filename, R_OK) == 0) {
		// Set configFilename and clean up
		configFilename = Strdup(filename);
		delete [] filename;
		
		return Config::ParseConfigFile(configFilename);
	    }
	}

	//
	// Case #2 - look in ~/.mercator.config
	//
	struct passwd *pw = getpwuid(getuid());
	if (pw != NULL) {
	    char *homeDirectory = pw->pw_dir;
	    char *filename = new char[strlen(homeDirectory) +
		strlen("/.mercator.config") + 1];
	    strcpy(filename, homeDirectory);
	    strcat(filename, "/.mercator.config");

	    // See if it's accessible
	    if (access(filename, R_OK) == 0) {
		// Set configFilename and clean up
		configFilename = Strdup(filename);
		delete [] filename;
		
		return Config::ParseConfigFile(configFilename);
	    }
	} else {
	    Error(EL_WARNING, "Mercator::ParseConfigFile",
		  "Can't get pwent for user?!", NULL);
	}

	//
	// Case #3 - look in /opt/GTsonicx/etc/mercator.config
	//
	if (access("/opt/GTsonicx/etc/mercator.config", R_OK) == 0) {
	    Error(EL_WARNING, "Mercator::ParseConfigFile",
		  "Using /opt/GTsonicx/etc/mercator.config.\n",
		  "\tYou may want to consider copying this file to ",
		  "$HOME/.mercator.config\n",
		  "\tfor future modifications.", NULL);
		  
	    // Set configFilename and clean up
	    configFilename = Strdup("/opt/GTsonicx/etc/mercator.config");
		
	    return Config::ParseConfigFile(configFilename);
	}

	Error(EL_FATAL, "Mercator::ProcessConfigFile",
	      "Cannot open config file, tried:\n",
	      "\t$HOME/.mercator.config\n",
	      "\t~/.mercator.config\n",
	      "\t/opt/GTsonicx/etc/mercator.config\n",
	      "\t**** Cannot continue ****", NULL);
    }

    //
    // We wind up here if the config file was set via -f or an env variable.
    //
    if (access(configFilename, R_OK) == 0) {
	return Config::ParseConfigFile(configFilename);
    } else {
	Error(EL_FATAL, "Mercator::ProcessConfigFile",
	      "Cannot open config file: ", configFilename, NULL);
    }

    return 1;
}

//
// ParseAttribFile forwards to Attribute::ParseAttribFile.  NOTE that if we
// get to here and attribFilename is NULL (that is, it hasn't been specified
// via either -a or an environment variable, we first look in
// $HOME/.mercator.attrib and, if that doesn't exist, in
// ~/.mercator.attrib and, if that doesn't exist, in
// /opt/GTsonicx/etc/mercator.attrib
//
// NOTE that an attrib file is NOT required:  unlike ParseConfigFile, we do
// not bail out if the file doesn't exist.
//
int
Mercator::ParseAttribFile()
{
    if (attribFilename == NULL) {
	//
	// Case #1 - Look in $HOME/.mercator.attrib.
	//
	char *homeDirectory = getenv("HOME");
	if (homeDirectory != NULL) {
	    // Build the filename
	    char *filename = new char[strlen(homeDirectory) +
		strlen("/.mercator.attrib") + 1];
	    strcpy(filename, homeDirectory);
	    strcat(filename, "/.mercator.attrib");

	    // See if it's accessible
	    if (access(filename, R_OK) == 0) {
		// Set attribFilename and clean up
		attribFilename = Strdup(filename);
		delete [] filename;
		
		return Attribute::ParseAttribFile(attribFilename);
	    }
	}

	//
	// Case #2 - look in ~/.mercator.attrib
	//
	struct passwd *pw = getpwuid(getuid());
	if (pw != NULL) {
	    char *homeDirectory = pw->pw_dir;
	    char *filename = new char[strlen(homeDirectory) +
		strlen("/.mercator.attrib") + 1];
	    strcpy(filename, homeDirectory);
	    strcat(filename, "/.mercator.attrib");

	    // See if it's accessible
	    if (access(filename, R_OK) == 0) {
		// Set attribFilename and clean up
		attribFilename = Strdup(filename);
		delete [] filename;
		
		return Attribute::ParseAttribFile(attribFilename);
	    }
	} else {
	    Error(EL_WARNING, "Mercator::ParseAttribFile",
		  "Can't get pwent for user?!", NULL);
	}

	//
	// Case #3 - look in /opt/GTsonicx/etc/mercator.attrib
	//
	if (access("/opt/GTsonicx/etc/mercator.attrib", R_OK) == 0) {
	    // Set attribFilename and clean up
	    attribFilename = Strdup("/opt/GTsonicx/etc/mercator.attrib");
		
	    return Attribute::ParseAttribFile(attribFilename);
	}
    } else {
	//
	// We wind up here if the attrib file was set via -a or an env variable.
	//
	if (access(attribFilename, R_OK) == 0) {
	    return Attribute::ParseAttribFile(attribFilename);
	} else {
	    Error(EL_ERROR, "Mercator::ParseAttribFile",
		  "File ", attribFilename, " is not accessible.", NULL);
	    return 0;
	}
    }

    return 1;
}

//
// LoadTemplates forwards to Template::LoadTemplates
//
int
Mercator::LoadTemplates()
{
    return Template::LoadTemplates(Config::GetStringList("templateDirectories"));
}

//
// Free up some data structures.  These simply forward.
//
void
Mercator::FreeFilters()
{
    FilterDict::FreeFilters();
}

void
Mercator::FreeTemplates()
{
    Template::FreeTemplates();
}

void
Mercator::FreeConfig()
{
    Config::FreeConfig();
}

void
Mercator::FreeAttrib()
{
    Attribute::FreeAttrib();
}

void
Mercator::LoadFilterDictionary() {
  mListVal<mString> list = Config::GetStringList(CONFIG_FILE_LIST);
  mListValIterator<mString> iter(list);
  int i, listsize = list.entries();
  char *name;

  // We need to centralize this somewhere.
  if (listsize > 0)
    for (i = 0, iter.next(); i < listsize; i++, iter.next()) {
      name = (char *)iter.current();
      Error(EL_DEBUG, "Mercator::LoadFilterDictionary", 
	    "Processing filter file ", name, ".", NULL);
      FilterDict::LoadFilters(name);
    }
  else {
    Error(EL_DEBUG, "Mercator::LoadFilterDictionary",
	  "no filter files specified -- defaulting to default.flt.",
	  NULL);
    FilterDict::LoadFilters("default.flt");
  }
}

//
// Install any signal handlers we may need.
//
void
Mercator::InstallSignalHandlers()
{
    static struct sigaction interruptAction;
    static struct sigaction pipeAction;

    memset(&interruptAction, '\0', sizeof(struct sigaction));
    memset(&pipeAction, '\0', sizeof(struct sigaction));
    
#ifdef BEAM
    interruptAction.sa_handler = (void (*)(int))Mercator::InterruptHandler;
#else
    interruptAction.sa_handler = Mercator::InterruptHandler;
#endif
    sigaction(SIGINT, &interruptAction, NULL);
    
#ifdef BEAM
    pipeAction.sa_handler = (void (*)(int))Mercator::PipeHandler;
#else
    pipeAction.sa_handler = Mercator::PipeHandler;
#endif
    sigaction(SIGPIPE, &pipeAction, NULL);
}

//
// This is our SIGPIPE signal handler
//
void
#ifdef SUNOS_5_5
Mercator::PipeHandler(int dummy)
#else
Mercator::PipeHandler()
#endif /* SUNOS_5_5 */
{
    ErrorOutput(EO_STDERR);
    Error(EL_DEBUG, "Mercator::Pipehandler",
	  "Pipe peer died, closing down...", NULL);
    
    if (mercator) {
	mercator->RemoveNamedPipes();
    }
}

void
#ifdef SUNOS_5_5
Mercator::InterruptHandler(int dummy)
#else
Mercator::InterruptHandler()
#endif /* SUNOS_5_5 */
{
    Error(EL_STATUS, "Mercator::InterruptHandler",
	  "received SIGINT, shutting down...", NULL);

    if (mercator)
	mercator->Quit();
}

//
// save debug/error messages in a temp file until console is ready
// (tempname returns a pointer to an area in static memory. It's safe
// until the next call to tempnam. We only need it within this member
// function so it is ok.)
//
void
Mercator::SaveMsg()
{
    tmpFileName = tempnam(NULL, NULL);

    if (tmpFileName) {
	tmpout.open(tmpFileName, ios::in|ios::out,
		       S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);

	if (tmpout.bad()) {
	    Error(EL_ERROR, "Mercator::SaveMsg", "cannot open temp file ",
		  tmpFileName, ", using stdout", NULL);
	}
	else {
	    Error(EL_DEBUG, "Mercator::SaveMsg",
		  "caching error/debugging messages in ", tmpFileName, NULL);
	    ErrorOutput(EO_SYSLOG, &tmpout);
	}
    }
    else {
	Error(EL_ERROR, "Mercator::SaveMsg", "tempname() error",
	      ", using stderr", NULL);
    }
}


//
// Dump out the initial error/debug messages in the temp file to either
// the console or the ErrorOutput.
//
void
Mercator::DumpMsg()
{
    if (!tmpout.bad()) {
	
	tmpout.flush();
	tmpout.seekg(0);  // seek to beginning of file

	FILE * dumptarget;

	switch(tmpErrorOutputVal) {
	  case EO_STDERR:
	    dumptarget = stderr;
	    break;
	  case EO_SYSLOG:
	  case EO_CONSOLE:
	    if((Mercator::outFile) && (Mercator::outFile != stdout) &&
	       (Mercator::outFile != stderr)) {
		if(fclose(Mercator::outFile) == EOF) {
		    Error(EL_ERROR, "Mercator::DumpMsg", "fclose ",
			  Perror(), NULL);
		}
	    }

	    if(!(Mercator::outFile = fdopen(outfd, "w"))) {
		Error(EL_ERROR, "Mercator::DumpMsg", "fdopen ", Perror(), NULL);
	    }
	
	    cerr << "*****opened Mercator::outFile\n";
	    dumptarget = Mercator::outFile;
	    break;
	  case EO_IGNORE:
	  default:
	    dumptarget = NULL;
	}

	if(dumptarget) {
	    char ch;
	
	    while(tmpout.get(ch)) {
		fputc(ch, dumptarget);
	    }
	    if(ch != '\n') {
		fputc('\n', dumptarget);
	    }
	}
	
	tmpout.close();

	if (fflush(dumptarget) != 0) {
	    Error(EL_ERROR, "Mercator::DumpMsg", "fflush ", Perror(), NULL);
	}

	// restore the error output direction
	if(tmpErrorOutputVal == EO_CONSOLE) {
	    ErrorOutput(EO_CONSOLE, Mercator::outfd);
	}
	else {
	    ErrorOutput(tmpErrorOutputVal);
	}

	// delete the temp file
	if(tmpFileName && (unlink(tmpFileName) == -1)) {
	    Error(EL_ERROR, "Mercator::DumpMsg", "Cannot unlink temp file ",
		  tmpFileName, Perror(), NULL);
	}
    }
}

void
Mercator::Quit()
{
    // clear the braille display cells
    if(braille) {
	braille->DisplayStatus(" ");
	braille->DisplayBraille(" ",0);
    }

    FDInterest::Quit();		// tell FDInterest to begin shutting down
    RemoveNamedPipes();		// clean up the filesystem

    // shut down all I/O devices which may be connected to servers
    if(speech)
	speech->ShutDown();

    if(audio)
	audio->ShutDown();

    if(braille)
	braille->ShutDown();
    
    if (GetProcessManager())
	GetProcessManager()->Shutdown(-1);	// halt subprocesses
}
