//  -*-Mode: C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

//
// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/NetAudio.cc,v $
// $Revision: 6.11 $
// $Date: 1995/12/03 21:04:06 $
// $Log: NetAudio.cc,v $
// Revision 6.11  1995/12/03  21:04:06  keith
// *really* fixed typo this time...
//
// Revision 6.10  1995/12/03  21:00:33  keith
// Fixed typo in Shutdown()...
//
// Revision 6.9  1995/12/03  20:13:49  keith
// Fixed file descriptor leaks...
//
// Revision 6.8  1995/10/25  18:48:47  spine
// Added a ShutDown method to prevent the object from trying to play sound
// after the server has been killed. (It caused mercator to hang until the
// client times out if the server has been killed.)
//
// Revision 6.7  1995/10/11  18:48:15  spine
// Changed debugging messages in StartServer() to agree with the server
// variable name the loadable module is actually looking for (netaudioServer
// instead of audioServer).
//
// Revision 6.6  1995/09/26  21:17:39  spine
// Modified server connection behavior.
//
// Revision 6.5  1995/09/21  21:39:55  keith
// Made a few cleanups, signal handling stuff...
//
// Revision 6.4  1995/09/20  21:24:21  spine
// Modified server connection debugging messages.
//
// Revision 6.3  1995/09/19  21:40:37  spine
// Added StartServer member.
//
// Revision 6.2  1995/08/17  20:34:45  spine
// Added an api to connect to the audio server.
//
// Revision 6.1  1995/07/28  20:41:50  spine
// Added copyright notice.
//
// Revision 6.0  1995/05/20  22:20:21  beth
// Mercator Version 6
//
// Revision 1.4  1995/04/01  19:40:38  keith
// End of work by Keith
//
// Revision 1.3  1995/03/06  17:52:05  keith
// *** empty log message ***
//
// Revision 1.2  1995/01/19  22:23:31  keith
// Automatic global checkin
//
// Revision 1.1  1994/11/16  20:46:23  spine
// Initial revision
//
// Revision 1.1  1994/11/16  20:46:23  spine
// Initial revision
//
// Revision 1.1  1994/05/24  19:49:25  stacy
// Initial revision
//
// Revision 1.1  1994/04/06  20:14:03  kathryn
// Initial revision
//
//
 
static const char rcsid[]="@(#) $Header: /net/hm10/keith/proj/mercator/6.0/RCS/NetAudio.cc,v 6.11 1995/12/03 21:04:06 keith Exp $";


#include <unistd.h>
#include <strings.h>
#include <stdlib.h>
#include <stream.h>
#include "NetAudio.h"
#include "Error.h"
#include "Loadable.h"
#include "Mercator.h"
#include "Template.h"
#include "Config.h"

extern Mercator *mercator;

LOADABLE_CLASS_DEFN(NetAudio);


NetAudio::NetAudio(const char *hostname) :
    Audio("NetAudio")
{
    NetAudio::StartServer();
    NetAudio::ConnectServer(hostname);
}

NetAudio::~NetAudio(void)
{
    ShutDown();
}

Audio::SoundID
NetAudio::PlaySound(const char *sound_file,
		    float *rate, int volume, int muffle, int looped)
{ 
    sound_id id;

    if (audio_server != NULL) {
	// cast away constness because na.h is stupid.
	id = play_sound(audio_server, (char *) sound_file, rate, 
			volume, muffle, looped);
	return (id);
    } else
	return (-1);
}

int
NetAudio::StopSound(Audio::SoundID id)
{
    if(audio_server != NULL)
	stop_sound(audio_server, id);

    return 0;
}

int
NetAudio::StopAllSounds()
{
    if(audio_server != NULL)
	stop_all_sounds(audio_server);
    
    return 0;
}

int
NetAudio::ConnectServer(const char *hostname)
{
    FieldData *maxTriesData = Config::GetDefaultByName("netaudioTimeout");
    int maxTries = 5;

    if (maxTriesData && (maxTriesData->Tag() == INTEGER))
	maxTries = maxTriesData->Integer();
    
    Error(EL_DEBUG, "NetAudio::ConnectServer", "Connecting to netaudio", NULL);

    if (hostname == NULL)
	if ((hostname = getenv("NETAUDIOHOST")) == NULL)
	    hostname = "localhost";
    
    for (register int i=0; i<5; i++) {
	// Cast away constness because na.h is stupid.
	audio_server = open_audio_server((char *) hostname);

	if (audio_server == NULL) {
	    sleep(1);
	    continue;
	} else {
	    break;
	}
    }

    if (audio_server == NULL) {
	clnt_spcreateerror("open_audio_server");
	Error(EL_ERROR, "NetAudio::ConnectServer", "server connection",
	      " timed out", NULL);
	return -1;
    } else {
	Error(EL_DEBUG, "NetAudio::ConnectServer",
	      "connection to server successful", NULL);
	return 0;
    }
}

//
// return -1 if server was not started, 0 otherwise. (Will assume that
// if NETAUDIOHOST environment variable is only set if the user wants to
// use a server on another machine. So essentially setting NETAUDIOHOST
// to anything will prevent a server from being started.
//
int
NetAudio::StartServer()
{
    const char *host;

    if ((host = getenv("NETAUDIOHOST")) == NULL) {
	if (mercator->ExecList("netaudioServer")) {
	    Error(EL_DEBUG, "NetAudio::StartServer",
		  "netaudioServer not found.", NULL);
	    return -1;
	}
	return 0;
    } else
	Error(EL_DEBUG, "NetAudio::StartServer", "Server has been started on ",
	      host, NULL);
    return 0;
}


//
// This method should be called just before mercator kills all its
// subprocesses
//
void
NetAudio::ShutDown()
{
    Error(EL_DEBUG, "NetAudio::ShutDown", "Shutting down NetAudio object",
	  NULL);

    // just need to prevent the object from trying to play any sound
    if (audio_server != NULL) {
	close_audio_server(audio_server);
	audio_server= NULL;
    }
}
