// -*-Mode: C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

#ifndef _RAP_LISTENER_H_
#define _RAP_LISTENER_H_

#include <X11/Xlib.h>
#include <X11/ICE/ICElib.h>

#include "FDInterest.h"

class XServer;


class RAPListener : public FDInterest {
private:
    IcePaVersionRec		*protocolVersions;
    IceListenObj		*listeners;
    int				listenerCount;
    int				majorOpcode;
    char			*networkIds;
    Window			agentWindow;
    Atom			protoNameAtom;

    void			SetupAgent(XServer& xserver);

    static Bool			AuthorizeProc(char *hostname);
    static void			IOErrorHandler(IceConn iceConn);
    static void			IOErrorProc(IceConn iceConn);
    static ProcessProtoMsgProc;
    static Status		ProtoSetupProc(IceConn iceConn,
					       int majorVersion,
					       int minorVersion,
					       char *vendorString,
					       char *releaseString,
					       IcePointer *clientDataRet,
					       char **failureReasonRet);
    static void			ProtoActivateProc(IceConn iceConn,
						  IcePointer clientData);
    void			ClassInit();

public:
    RAPListener(XServer& xserver);
    virtual ~RAPListener();

    //
    // HandleActivity can spawn new RAP objects when a connection is
    // made.
    //
    virtual int			HandleActivity(int fd, short mask);
    //
    // Shutdown() is called when zero-length reads are detected.  For
    // listener descriptors, this probably means that someone is trying
    // to connect() to us.  Don't treat it as a connection loss.
    //
    virtual void		Shutdown(int fd)
				    { return; }

    int				Register(XServer& xserver);

    //
    // InitiateConnection can be called via the XServer whenever the
    // connection initiation conditions are satisfied.  The return
    // value is TRUE if the code actually tries to initiate a connection
    // (the window was previously unseen), and FALSE otherwise.
    //
    int				InitiateConnection(XServer& xserver,
					   Window clientWindow);
};


#endif /* _RAP_LISTENER_H_ */
