// -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
//
// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Speech.cc,v $
// $Revision: 6.5 $
// $Date: 1995/09/21 21:39:23 $
// $Log: Speech.cc,v $
// Revision 6.5  1995/09/21  21:39:23  keith
// Removed StartServer, ConnectServer APIs
// y
//
// Revision 6.4  1995/09/19  21:41:04  spine
// Added StartServer member.
//
// Revision 6.3  1995/08/17  20:38:25  spine
// Added an api to connect to the server.
//
// Revision 6.2  1995/08/09  20:23:16  keith
// Added capabilities stuff (a start anyway...)
//
// Revision 6.1  1995/06/02  20:31:44  beth
// added copyright
//
// Revision 6.0  1995/05/20  22:20:21  beth
// Mercator Version 6
//
// Revision 1.4  1995/04/01  19:40:38  keith
// End of work by Keith
//
// Revision 1.3  1995/03/06  17:52:05  keith
// *** empty log message ***
//
// Revision 1.2  1995/01/19  22:22:30  keith
// Automatic global checkin
//
// Revision 1.2  1995/01/19  22:22:30  keith
// Automatic global checkin
//
// Revision 1.1  1994/11/16  20:46:23  spine
// Initial revision
//
// Revision 1.1  1994/05/24  19:49:25  stacy
// Initial revision
//
// Revision 1.1  1994/04/06  20:29:27  kathryn
// Initial revision
//
//
 
static const char rcsid[]="@(#) $Header: /net/hm10/keith/proj/mercator/6.0/RCS/Speech.cc,v 6.5 1995/09/21 21:39:23 keith Exp $";


#include "Speech.h"
#include "Error.h"
#include "Util.h"

#include <strings.h>
#include <stdlib.h>
#include <stream.h>

Speech::Capabilities::Capabilities() :
    languages(),
    voices(),
    minRate(0),
    maxRate(0),
    gainSupported(0)
{
}

Speech::Capabilities::Capabilities(const Capabilities& other) :
    languages(other.languages),
    voices(other.voices),
    minRate(other.minRate),
    maxRate(other.maxRate),
    gainSupported(other.gainSupported)
{
}

Speech::Capabilities&
Speech::Capabilities::operator=(const Capabilities& other)
{
    languages = other.languages;
    voices = other.voices;
    minRate = other.minRate;
    maxRate = other.maxRate;
    gainSupported = other.gainSupported;

    return *this;
}

///////////////////////////////////////////////////////////////////////////

Speech::Speech(const char *name)
{
    if (name)
	speechName = Strdup(name);
    else
	speechName = Strdup("Generic");
}

Speech::~Speech()
{
    delete [] speechName;
}

const Speech::Capabilities&
Speech::GetCapabilities() const
{
    static const Capabilities cap;
    return cap;
}

int
Speech::Notify(const char *type)
{ 
    Error(EL_DEBUG, "Speech::Notify", "type = ", type, NULL);
    return 0;   
}

int
Speech::Speak(const char *string)
{ 
    Error(EL_DEBUG, "Speech::Speak", string, NULL);
    return 0;   
}

int
Speech::SpeakUnix(const char * string)
{ 
    Error(EL_DEBUG, "Speech::SpeakUnix", string, NULL);
    return 0;   
}

int
Speech::SetSpeechRate(int rate)
{ 
    char tmp[16];
    sprintf(tmp, "%d", rate);
    Error(EL_DEBUG, "Speech::SetSpeechRate", "rate = ", tmp, NULL);
    return 0;   
}

int
Speech::StopSpeaking()
{ 
    Error(EL_DEBUG, "Speech::StopSpeaking", NULL);
    return 0;   
}


int
Speech::SetVoice(const char *voice)
{ 
    Error(EL_DEBUG, "Speech::SetVoice", "voice = ", voice, NULL);
    return 0;
}

int
Speech::SetLanguage(const char *lang)
{
    Error(EL_DEBUG, "Speech::SetLanguage", "language = ", lang, NULL);
    return 0;
}

int
Speech::SetGain(int gain)
{
    char tmp[16];
    sprintf(tmp, "%d", gain);
    Error(EL_DEBUG, "Speech::SetGain", "gain = ", tmp, NULL);
    return 0;
}

int
Speech::SetDictionary(const char *word, const char *pronun)
{
    Error(EL_DEBUG, "Speech::SetDictionary", "word = ", word, 
	  "pronouncation = ", pronun, NULL);
    return 0;
}

int
Speech::UnSetDictionary(const char *word)
{
    Error(EL_DEBUG, "Speech::UnSetDictionary", "word = ", word, NULL);
    return 0;
}
