// -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

#ifndef _Speech_h_
#define _Speech_h_

//
//  class definition for generic speech synthesis resouce
//


//
// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Speech.h,v $
// $Revision: 6.6 $
// $Date: 1995/10/25 18:49:25 $
/* $Log: Speech.h,v $
 * Revision 6.6  1995/10/25  18:49:25  spine
 * Added a ShutDown method to prevent the object from trying to play sound
 * after the server has been killed. (It caused mercator to hang until the
 * client times out if the server has been killed.)
 *
 * Revision 6.5  1995/09/21  21:39:23  keith
 * Removed StartServer, ConnectServer APIs
 * y
 *
 * Revision 6.4  1995/09/19  21:41:13  spine
 * Added StartServer member.
 *
 * Revision 6.3  1995/08/17  20:38:06  spine
 * Added an api to connect to the server.
 *
 * Revision 6.2  1995/08/09  20:23:16  keith
 * Added capabilities stuff (a start anyway...)
 *
 * Revision 6.1  1995/06/02  20:31:44  beth
 * added copyright
 *
 * Revision 6.0  1995/05/20  22:20:21  beth
 * Mercator Version 6
 *
 * Revision 1.3  1995/04/01  19:40:38  keith
 * End of work by Keith
 *
 * Revision 1.2  1995/01/19  22:23:31  keith
 * Automatic global checkin
 *
 * Revision 1.1  1994/11/16  20:46:23  spine
 * Initial revision
 *
 * Revision 1.1  1994/05/24  19:49:25  stacy
 * Initial revision
 *
 * Revision 1.2  1994/04/06  20:56:20  kathryn
 * *** empty log message ***
 *
 * Revision 1.1  1994/04/06  20:29:16  kathryn
 * Initial revision
 *
*/

#ident "$Id: Speech.h,v 6.6 1995/10/25 18:49:25 spine Exp $"

#include <stdlib.h>

#include "Error.h"
#include "mListVal.h"
#include "mString.h"

class Speech {
private:
    char *speechName;

public:
    Speech(const char *name = NULL);
    virtual ~Speech();
    
    class Capabilities {
    public:
	mListVal<mString>	languages;
	mListVal<mString>	voices;
	int			minRate;
	int			maxRate;
	int			gainSupported;

	Capabilities();
	Capabilities(const Capabilities& other);
	Capabilities& operator=(const Capabilities& other);
    };
    
    virtual const Capabilities& GetCapabilities() const;
    
    virtual int Speak(const char *);	// speak some text
    virtual int SpeakUnix(const char *);// translate Unix and then speak
    virtual int Notify(const char *);   // does nothing 
    virtual int SetSpeechRate(int);	// set the rate of speech 
    virtual int StopSpeaking();		// interrupt the speaking
    virtual int SetVoice(const char *); // Sets the voice
    virtual int SetLanguage(const char *);
    virtual int SetGain(int);
    virtual int SetDictionary(const char *, const char *); // Set pronounciation
    virtual int UnSetDictionary(const char *);  // unsets pronounciation
    virtual void ShutDown();            // shutdown the speech subsystem
    
    const char	*GetSpeechName();
};

inline const char *
Speech::GetSpeechName()
{
    return speechName;
}

inline void
Speech::ShutDown()
{
    Error(EL_DEBUG, "Speech::ShutDown", "Shutting down speech subsystem",
	  NULL);
}
#endif _Speech_h_
