// -*-Mode:C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#ifndef __Storage_h__
#define __Storage_h__


// This is the class Storage, a base class.  All classes that need to store 
// information derived from this class.  Currently these classes are:
//
//    Class           Information Stored
//    -----           ------------------
//    Applications    Info. about all applications currently being modeled.
//    Client          Info. about an application, also called a client.
//                    This includes a list of XtObjects in the client.
//    XtObject        Info. about a XtObject in a Client.
//
//  Information about what action goes with what key stroke are stored in
//  each class.  First, the XtObject should be asked if it has any actions,
//  then the client, and then the applications (equivalent to global).  The 
//  XtObject has two ways it can have key bindings.  First for that instance
//  of the XtObject, and second for its widget class type.


//
// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/Storage.h,v $
// $Revision: 6.3 $
// $Date: 1995/07/28 20:48:02 $
/* $Log: Storage.h,v $
 * Revision 6.3  1995/07/28  20:48:02  spine
 * Added copyright notice.
 *
 * Revision 6.2  1995/07/25  22:05:34  keith
 * Cleaned up some usage of strings, dicts, and lists.  Ran with -pta to
 * fix some previously unseen bugs in the parameterized classes.
 *
 * Revision 6.1  1995/06/05  22:14:01  iansmith
 * Modified the Event structure quite a bit. Removed the old struct EventStruct
 * and EventPtr and made EventStruct a class. RogueWave/new Mercator
 * dictionaries don't work unless you have an operator==() which I needed
 * to write. (It does the obvious comparison of fields.) Also, I added a
 * hash function which can hash event structures for insertion into
 * dictionaries.
 *
 * Revision 6.0  1995/05/20  22:20:21  beth
 * Mercator Version 6
 *
 * Revision 1.3  1995/04/01  19:40:38  keith
 * End of work by Keith
 *
 * Revision 1.2  1995/03/06  17:51:28  keith
 * *** empty log message ***
 *
 * Revision 1.1  1994/11/16  20:46:23  spine
 * Initial revision
 *
 * Revision 1.1  1994/05/24  19:49:25  stacy
 * Initial revision
 *
 * Revision 1.2  1994/04/06  20:56:20  kathryn
 * *** empty log message ***
 *
 * Revision 1.1  1994/04/06  20:29:37  kathryn
 * Initial revision
 *
*/

#ident "$Id: Storage.h,v 6.3 1995/07/28 20:48:02 spine Exp $"

#include <X11/Xlib.h>		// for KeySym

// forward declarations
class Action;

// The type of events 
typedef enum {KEYPRESS, KEYRELEASE} EventTypes;

// Had to reorg this is a little for templates.... 
//typedef struct {
//    EventTypes   eventType;
//    KeySym       key;
//    unsigned int modifier;
//} EventStruct;
class EventStruct {
public:
    EventTypes eventType;
    KeySym key;
    unsigned int modifier;
    int operator==(const EventStruct &e) const;
};

inline int EventStruct::operator==(const EventStruct& e) const
{
    return ((eventType == e.eventType) &&
	    (key == e.key) &&
	    (modifier == e.modifier));
}

// hashing function so we can use events in dictionaries
inline unsigned EventStructHash(const EventStruct &es) 
{
    unsigned result;
    result=es.eventType ^ es.key ^ es.modifier;
    return result;
}
    

typedef EventStruct *EventPtr;
 
class Storage {
protected:
    Storage()			{}
    virtual ~Storage()		{}

public:
    virtual int  DispatchEvent   (EventPtr event) = 0;
    virtual void CreateEventBind (EventPtr event, Action *function) = 0;
    virtual int  DeleteEventBind (EventPtr event) = 0;
};

#endif __Storage_h__
