//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//	Template code.		keith edwards	26jan95
//

#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include "Template.h"
#include "Mercator.h"
#include "Client.h"
#include "XtObject.h"
#include "Config.h"
#include "Error.h"
#include "Util.h"
#include "Attribute.h"

/////
///// Stuff needed to link against the template parser.
/////
extern "C" int tmplparse(void);
extern FILE *tmplin;
int templateLineno = 1;

//
// String hasher for dictionaries.
//
static unsigned StringHasher(const mString& str) { return str.hash(); }

//
// Static objects
//

mDictValPtr<mString,Template> *Template::classTemplates =
	new mDictValPtr<mString,Template>(StringHasher);
mDictValPtr<mString,Template> *Template::objectTemplates =
	new mDictValPtr<mString,Template>(StringHasher);
mDictValPtr<mString,Template> *Template::appTemplates =
	new mDictValPtr<mString,Template>(StringHasher);

int Template::validDicts = 0;

/////
///// Miscellaneous crap.
/////
#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif


///// 
///// These are routines which are called directly from yacc-generated
///// C code.
/////

extern "C" {

void
NewObjTemplate(int l, const char *name)
{
    if (Template::currentTemplate != NULL) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "NewObjTemplate", "line ", tmp, 
	      ": previous template definition has not ended.", NULL);
	return;
    }
    
    Template::currentTemplate = new Template(name, OBJECT_TEMPLATE);
}

void
NewClassTemplate(int l, const char *name)
{
    if (Template::currentTemplate != NULL) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "NewClassTemplate", "line ", tmp,
	      ": previous template definition has not ended.", NULL);
	return;
    }
    
    Template::currentTemplate = new Template(name, CLASS_TEMPLATE);
}

void
NewAppTemplate(int l, const char *name)
{
    if (Template::currentTemplate != NULL) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "NewAppTemplate", "line ", tmp,
	      ": previous template definition has not ended.", NULL);
	return;
    }
    
    Template::currentTemplate = new Template(name, APP_TEMPLATE);
}

void
EndTemplate(int l)
{
    if (Template::currentTemplate == NULL) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "EndTemplate", "line ", tmp,
	      ": No template in progress.", NULL);
	return;
    }
    
    if (Template::currentTemplate->Valid()) {
	// add it to the list maintained by its class
	switch (Template::currentTemplate->Type()) {
	case CLASS_TEMPLATE:
	case OBJECT_TEMPLATE:
	case APP_TEMPLATE:
	    Template::currentTemplate->Install();
	    break;
	default:
	    Error(EL_ABORT, "EndTemplate",
		  "Unknown template type; cannot continue.\n"
		  "\tThis is a nonrecoverable error as it indicates that ",
		  "recompilation is necessary\n\ton this platform.", NULL);
	}
    } else {
	Error(EL_ERROR, "EndTemplate", 
	      "An error was encountered while parsing the template ",
	      Template::currentTemplate->Name(), "; it will be skipped.", 
	      NULL);
	delete Template::currentTemplate;
    }

    // Reset it so that NewTemplate won't complain.
    Template::currentTemplate = NULL;

}

void
ProcessBooleanField(int l, const char *id, unsigned char b)
{
    if (Template::currentTemplate == NULL) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "ProcessBooleanField", "line ", tmp,
	      ": field found outside of template definition.", NULL);
	return;
    }
    
    if (!Template::currentTemplate->InstallBooleanField(id, b)) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "ProcessBooleanField", "line ", tmp,
	      ": Could not set field in ", Template::currentTemplate->Name(),
	      NULL);
    }
}

void
ProcessIntegerField(int l, const char *id, long i)
{
    if (Template::currentTemplate == NULL) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "ProcessIntegerField", "line ", tmp,
	      ": field found outside of template definition.", NULL);
	return;
    }
    
    if (!Template::currentTemplate->InstallIntegerField(id, i)) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "ProcessIntegerField", "line ", tmp,
	      ": Could not set field in ", Template::currentTemplate->Name(),
	      NULL);
    }
}

void
ProcessStringField(int l, const char *id, const char *s)
{
    if (Template::currentTemplate == NULL) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "ProcessStringField", "line ", tmp,
	      ": field found outside of template definition.", NULL);
	return;
    }
    
    if (!Template::currentTemplate->InstallStringField(id, s)) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "ProcessStringField", "line ", tmp,
	      ": Could not set field in ", Template::currentTemplate->Name(),
	      NULL);
    }
}

void
ProcessStringListField(int l, const char *id)
{
    if (Template::currentTemplate == NULL) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "ProcessStringListField", "line ", tmp,
	      ": field found outside of template definition.", NULL);
	return;
    }

    if (Template::currentStringList == NULL) {
	mListVal<mString> emptyStringList;
	if (!Template::currentTemplate->InstallStringListField(id,
						       emptyStringList)) {
	    char tmp[16];
	    sprintf(tmp, "%d", l);
	    Error(EL_ERROR, "ProcessStringListField", "line ", tmp,
		  ": Could not set field in ",
		  Template::currentTemplate->Name(),
		  NULL);
	}
    } else {
	if (!Template::currentTemplate->InstallStringListField(id,
					   *Template::currentStringList)) {
	    char tmp[16];
	    sprintf(tmp, "%d", l);
	    Error(EL_ERROR, "ProcessStringListField", "line ", tmp,
		  ": Could not set field in ",
		  Template::currentTemplate->Name(),
		  NULL);
	}
	delete Template::currentStringList;
    }

    Template::currentStringList = NULL;  // <- bug fix! -  selbie
}

void
ProcessProcField(int l, const char *id, const char *proc)
{
    if (Template::currentTemplate == NULL) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "ProcessProcField", "line ", tmp,
	      ": field found outside of template definition.", NULL);
	return;
    }
    
    if (!Template::currentTemplate->InstallProcField(id, proc)) {
	char tmp[16];
	sprintf(tmp, "%d", l);
	Error(EL_ERROR, "ProcessProcField", "line ", tmp,
	      ": Could not set field in ", Template::currentTemplate->Name(),
	      NULL);
    }
}

void
TmplAddToStringList(int l, const char *s)
{
    if (Template::currentStringList == NULL)
	Template::currentStringList = new mListVal<mString>;

    Template::currentStringList->append(s);
}

void
TmplLexWarning(int l, const char *s)
{
    char tmp[16];
    sprintf(tmp, "%d", l);
    Error(EL_ERROR, "TmplLexWarning", "syntax error on line ", tmp,
	  ": ", s, NULL);
}

}  /* extern "C" */

//////////////////////////////////////////////////////////////////////////


//
// Varargs procedure to load all files from a list of directories.
// Calls ParseTemplateFile() repeatedly.
//
int
Template::LoadTemplates(const mListVal<mString>& dirs)
{
    mString directory;
    int ret = 1;
    static const char SUFFIX[] = ".tmpl";
    static int SUFFIXLEN = strlen(SUFFIX);
    mListValIterator<mString> iter(dirs);

    if (Template::validDicts)
	Template::EmptyDicts();

    while (iter.next()) {
	directory=iter.current();
	// processing files in directory 'directory'
	DIR *dir = NULL;
	struct dirent *direntp = NULL;
	char *directoryName = (char *)directory;

	if ((dir = opendir(directoryName)) == NULL) {
	    Error(EL_ERROR, "Template::LoadTemplates"
		  "Couldn't open directory '", directoryName, "': ",
		  Perror(), NULL);
	    ret = 0;
	    continue;
	}
	
	while ((direntp = readdir(dir)) != NULL) {
	    unsigned int length;
	    
	    // readdir does not define whether . and .. are returned
	    if (strcmp(direntp->d_name, ".") == 0)
		continue;
	    if (strcmp(direntp->d_name, "..") == 0)
		continue;

	    // Look for files ending in .tmpl
	    length = strlen(direntp->d_name);
	    if (length <= 6)
		continue;

	    if (strcmp(&(direntp->d_name[length - SUFFIXLEN]), SUFFIX) == 0) {
		char *fullname;
		fullname = new char[strlen(directoryName) +
				    strlen(direntp->d_name) + 2];
		strcpy(fullname, directoryName);
		strcat(fullname, "/");
		strcat(fullname, direntp->d_name);

		if (!Template::ParseTemplateFile(fullname)) {
		    Error(EL_ERROR, "Template::LoadTemplates",
			  "Error loading template from file '", fullname,
			  "'", NULL);
		    ret = 0;
		}

		delete [] fullname;
	    }
	}

	if (closedir(dir) == -1) {
	    Error(EL_ERROR, "Template::LoadTemplates",
		  "Error closing directory '", directoryName, "': ",
		  Perror(), NULL);
	    ret = 0;
	}
	delete [] directoryName;
    }
    
    Template::validDicts = 1;
    return ret;
}

//
// Parse the specified template file and create or modify any templates
// as needed.
//
int
Template::ParseTemplateFile(const char *filename)
{
    FILE *templateFile = NULL;

    
    if (filename == NULL) {
	Error(EL_ERROR, "Template::ParseTemplateFile",
	      "No filename specified.", NULL);
	return 0;
    }
    
    if ((templateFile = fopen(filename, "r")) == NULL) {
	Error(EL_ERROR, "Template::ParseTemplateFile", "couldn't open file ",
	      filename, ": ", Perror(), NULL);
	return 0;
    }
    
    // tmplin is the yyin for the template parser.
    tmplin = templateFile;

    templateLineno = 1;
    
    // tmplparse() is the yyparse for the template parser.
    if (tmplparse() != 0) {
	Error(EL_ERROR, "Template::ParseTemplateFile", "Parser failed for ",
	      filename, NULL);
        fclose(tmplin);
	return 0;
    }

    // close the file
    fclose(tmplin);

    return 1;
}

void
Template::FreeTemplates()
{
    Template::EmptyDicts();
    
    delete Template::objectTemplates;
    delete Template::classTemplates;
    delete Template::appTemplates;

    Template::validDicts = 0;
}

/* to resource our template files, all we have to do is blow away
   the items in the Template Dictionaries.  Then re-parse! */
void
Template::EmptyDicts()
{
    Template::objectTemplates->kill_all(); 
    Template::classTemplates->kill_all(); 
    Template::appTemplates->kill_all(); 

    delete Template::objectTemplates;
    delete Template::classTemplates;
    delete Template::appTemplates;

    Template::objectTemplates =
	new mDictValPtr<mString, Template>(StringHasher);
    Template::classTemplates = 
	new mDictValPtr<mString, Template>(StringHasher);
    Template::appTemplates =
	new mDictValPtr<mString, Template>(StringHasher);


    Template::validDicts = 0;
}



/////////////////////////////////////////////////////////////////////////////

Tcl_Interp *FieldData::interp = NULL;

FieldData::FieldData(FieldType t) :
    tag(t),
    initialized(0),
    rep(SIMPLE)
{
    switch (tag) {
    case BOOLEAN:
	boolean = 0;
	break;
    case INTEGER:
	integer = -1;
	break;
    case STRING:
	str = NULL;
	break;
    case STRINGLIST:
	stringList = NULL;
	break;
    default:
	Error(EL_ABORT, "FieldData::FieldData", 
	      "unknown field tag, cannot continue.\n"
	      "\tThis is a nonrecoverable error because it indicates that ",
	      "recompilation is necessary\n\ton this platform.", NULL);
    }
}


FieldData::FieldData(const FieldData& fd) :
    tag(fd.tag),
    rep(fd.rep),
    initialized(fd.initialized)
{
    if (!initialized)
	return;
    
    if (rep == PROC) {
	proc = Strdup(fd.proc);
    }
    // Even if our rep is PROC, we'll copy out the data anyway.
    switch (tag) {
    case BOOLEAN:
	boolean = fd.boolean;
	break;
    case INTEGER:
	integer = fd.integer;
	break;
    case STRING:
	str = Strdup(fd.str);
	break;
    case STRINGLIST:
	stringList = new mListVal<mString>(*fd.stringList);
	break;
    default:
	Error(EL_ABORT, "FieldData::FieldData", 
	      "unknown field tag, cannot continue.\n"
	      "\tThis is a nonrecoverable error because it indicates that ",
	      "recompilation is necessary\n\ton this platform.", NULL);
    }
}

FieldData::~FieldData()
{
    FreeOldData();
}

FieldData&
FieldData::operator=(const FieldData& fd)
{
    FreeOldData();
    
    tag = fd.tag;
    rep = fd.rep;
    initialized = fd.initialized;

    if (!initialized)
	return *this;
    
    if (rep == PROC) {
	proc = Strdup(fd.proc);
    }
    switch (tag) {
    case BOOLEAN:
	boolean = fd.boolean;
	break;
    case INTEGER:
	integer = fd.integer;
	break;
    case STRING:
	str = Strdup(fd.str);
	break;
    case STRINGLIST:
	stringList = new mListVal<mString>(*fd.stringList);
	break;
    default:
	Error(EL_ABORT, "FieldData::FieldData", 
	      "unknown field tag, cannot continue.\n"
	      "\tThis is a nonrecoverable error because it indicates that ",
	      "recompilation is necessary\n\ton this platform.", NULL);
    }
    
    return *this;
}

void
FieldData::SetInterpreter(Tcl_Interp *interpreter)
{
    interp = interpreter;
}

int
FieldData::InvokeProc(const XtObject& obj,
		      FieldData::OutputStruct *output) const
{
    int result;
    
    if (interp == NULL) {
	Error(EL_ERROR, "FieldData::InvokeProc",
	      "No interpreter set for FieldData.", NULL);
	return 0;
    }

    if (output == NULL) {
	Error(EL_ERROR, "FieldData::InvokeProc",
	      "No output return space allocated.", NULL);
	return 0;
    }

    // Get rid of any crap that might be in interp->result from the last time.
    Tcl_ResetResult(interp);
    
    // Invoke the thing.
    result = Tcl_VarEval(interp, proc, " object ", obj.GetUniqueName(), NULL);

    if (result != TCL_OK) {
	Error(EL_ERROR, "FieldData::InvokeProc", "Error in procedure ",
	      proc, ": ", interp->result, NULL);
	return 0;
    }

    // Parse out the return.
    switch (tag) {
    case BOOLEAN:
	if (strcmp(interp->result, "TRUE") == 0) {
	    output->boolean = TRUE;
	} else if (strcmp(interp->result, "FALSE") == 0) {
	    output->boolean = FALSE;
	} else {
	    Error(EL_ERROR, "FieldData::InvokeProc",
		  "Unexpected return for procedure expected to return ",
		  " Boolean.  Either TRUE or FALSE expected.", NULL);
	    return 0;
	}
	break;
    case INTEGER:
	// no checking here...
	output->integer = atol(interp->result);
	break;
    case STRING:
	output->str = interp->result;
	break;
    case STRINGLIST:
	Error(EL_ERROR, "FieldData::InvokeProc",
	      "Sorry, StringList return types from procs aren't supported yet.",
	      NULL);
	break;
    default:
	Error(EL_ABORT, "FieldData::InvokeProc", "Unknown field data type.\n"
	      "\tThis is a nonrecoverable error because it indicates that ",
	      "recompilation is necessary\n\ton this platform.", NULL);
	return 0;
    }
    return 1;
}

int
FieldData::InvokeProc(const Client& client,
		      FieldData::OutputStruct *output) const
{
    int result;
    
    if (interp == NULL) {
	Error(EL_ERROR, "FieldData::InvokeProc",
	      "No interpreter set for FieldData.", NULL);
	return 0;
    }

    if (output == NULL) {
	Error(EL_ERROR, "FieldData::InvokeProc",
	      "No output return space allocated.", NULL);
	return 0;
    }

    // Free any crap that might be in interp->result from the last time.
    delete [] interp->result;

    // Invoke the thing.
    result = Tcl_VarEval(interp, proc, " client ", client.ClientName(), NULL);

    if (result != TCL_OK) {
	Error(EL_ERROR, "FieldData::InvokeProc", "Error in procedure ",
	      proc, ": ", interp->result, NULL);
	return 0;
    }

    // Parse out the return.
    switch (tag) {
    case BOOLEAN:
	if (strcmp(interp->result, "TRUE") == 0) {
	    output->boolean = TRUE;
	} else if (strcmp(interp->result, "FALSE") == 0) {
	    output->boolean = FALSE;
	} else {
	    Error(EL_ERROR, "FieldData::InvokeProc",
		  "Unexpected return for procedure expected to return ",
		  "Boolean.  Either TRUE or FALSE expected.", NULL);
	    return 0;
	}
	break;
    case INTEGER:
	// no checking here...
	output->integer = atol(interp->result);
	break;
    case STRING:
	output->str = interp->result;
	break;
    case STRINGLIST:
	Error(EL_ERROR, "FieldData::InvokeProc",
	      "Sorry, StringList return types from procs aren't supported yet.",
	      NULL);
	break;
    default:
	Error(EL_ABORT, "FieldData::InvokeProc", "Unknown field data type.\n"
	      "\tThis is a nonrecoverable error because it indicates that ",
	      "recompilation is necessary\n\ton this platform.", NULL);
	return 0;
    }
    return 1;
}

void
FieldData::SetProc(const char *p)
{
    FreeOldData();
    rep = PROC;
    proc = Strdup(p);
    initialized = TRUE; // this looks suspicious - selbie
}

FieldType
FieldData::Tag() const
{
    return tag;
}

void
FieldData::Tag(FieldType t)
{
    // If we're *changing* our type, we may have to free the old data.
    
    if (tag != t) {
        if (initialized == TRUE)
          FreeOldData();
    }
    tag = t;
    initialized = TRUE;
}

unsigned char
FieldData::Initialized() const
{
    return initialized;
}

void
FieldData::FreeOldData()
{
    if (!initialized)
	return;
    
    if (rep == PROC) {
	rep = SIMPLE;
	delete [] proc;
        proc = NULL;
    } 
    switch (tag) {
    case BOOLEAN:
    case INTEGER:
	break;
    case STRING:
	delete str;
	str = NULL;
	break;
    case STRINGLIST:
	delete stringList;
        stringList = NULL;
	break;
    default:
	Error(EL_ABORT, "FieldData::FreeOldData", 
	      "unknown field tag, cannot continue.\n"
	      "\tThis is a nonrecoverable error because it indicates that ",
	      "recompilation is necessary\n\ton this platform.", NULL);
    }
}

//
// Note that we DO NOT check to see if we are actually representing the types
// we're returning, for speed reasons.  If you call one of the "get" functions
// on a FieldData instance that is representing a different type, the results
// are undefined (and may likely result in a coredump).
//

unsigned char
FieldData::Boolean() const
{
    // we ought to do error checking and make sure (rep != proc) !
    return boolean;
}


unsigned char
FieldData::Boolean(const XtObject& arg) const
{
    OutputStruct output;
    if (rep == SIMPLE) {
	return boolean;
    } else {
	if (InvokeProc(arg, &output)) {
	    return output.boolean;
	} else {
	    Error(EL_ERROR, "FieldData::Boolean",
		  "proc invocation failed, returning FALSE", NULL);
	    return 0;
	}
    }
}

unsigned char
FieldData::Boolean(const Client& arg) const
{
    OutputStruct output;
    if (rep == SIMPLE) {
	return boolean;
    } else {
	if (InvokeProc(arg, &output)) {
	    return output.boolean;
	} else {
	    Error(EL_ERROR, "FieldData::Boolean",
		  "proc invocation failed, returning FALSE", NULL);
	    return 0;
	}
    }
}

void
FieldData::Boolean(unsigned char b)
{
    FreeOldData();
    tag = BOOLEAN;
    boolean = b;
    initialized = TRUE;
}

long
FieldData::Integer() const
{
    return integer;
}

long
FieldData::Integer(const XtObject& arg) const
{
    OutputStruct output;
    if (rep == SIMPLE) {
	return integer;
    } else {
	if (InvokeProc(arg, &output)) {
	    return output.integer;
	} else {
	    Error(EL_ERROR, "FieldData::Integer",
		  "proc invocation failed, returning 0", NULL);
	    return 0;
	}
    }
}

long
FieldData::Integer(const Client& arg) const
{
    OutputStruct output;
    if (rep == SIMPLE) {
	return integer;
    } else {
	if (InvokeProc(arg, &output)) {
	    return output.integer;
	} else {
	    Error(EL_ERROR, "FieldData::Integer",
		  "proc invocation failed, returning 0", NULL);
	    return 0;
	}
    }
}

void
FieldData::Integer(long i)
{
    FreeOldData();
    tag = INTEGER;
    integer = i;
    initialized = TRUE;
}


const char *
FieldData::String() const
{
  return str;
}

const char *
FieldData::String(const XtObject& arg) const
{
    OutputStruct output;
    if (rep == SIMPLE) {
	return str;
    } else {
	if (InvokeProc(arg, &output)) {
	    return output.str;
	} else {
	    Error(EL_ERROR, "FieldData::String",
		  "proc invocation failed, returning empty string", NULL);
	    return "";
	}
    }
}

const char *
FieldData::String(const Client& arg) const
{
    OutputStruct output;
    if (rep == SIMPLE) {
	return str;
    } else {
	if (InvokeProc(arg, &output)) {
	    return output.str;
	} else {
	    Error(EL_ERROR, "FieldData::String",
		  "proc invocation failed, returning empty string", NULL);
	    return "";
	}
    }
}

void
FieldData::String(const char *s)
{
    FreeOldData();
    tag = STRING;
    str = Strdup(s);
    initialized = TRUE;
}

const mListVal<mString>&
FieldData::StringList() const
{
    return *stringList;  
}

const mListVal<mString>&
FieldData::StringList(const XtObject& arg) const
{
    OutputStruct output;
    if (rep == SIMPLE) {
	return *stringList;
    } else {
	if (InvokeProc(arg, &output)) {
	    return *(output.stringList);
	} else {
	    static mListVal<mString> tmp;
	    Error(EL_ERROR, "FieldData::StringList",
		  "proc invocation failed, returning empty list", NULL);
	    return tmp;
	}
    }
}

const mListVal<mString>&
FieldData::StringList(const Client& arg) const
{
    OutputStruct output;
    if (rep == SIMPLE) {
	return *stringList;
    } else {
	if (InvokeProc(arg, &output)) {
	    return *(output.stringList);
	} else {
	    static mListVal<mString> tmp;
	    Error(EL_ERROR, "FieldData::StringList",
		  "proc invocation failed, returning empty list", NULL);
	    return tmp;
	}
    }
}

void
FieldData::StringList(const mListVal<mString>& sl)
{
    FreeOldData();
    tag = STRINGLIST;
    stringList = new mListVal<mString>(sl);
    initialized = TRUE;
}


//////////////////////////////////////////////////////////////////////////

Template *Template::currentTemplate = NULL;
mListVal<mString> *Template::currentStringList = NULL;

/////////////////////////////////////////////////////////////////////////////

Template::Template(const char *n, TemplateType t) :
    name(Strdup(n)),
    valid(1),
    type(t),
    attributeDict(StringHasher)
{
}

Template::~Template()
{
  delete [] name;
  attributeDict.kill_all();

}

int
Template::Valid() const
{
    return valid;
}

void
Template::Valid(int v)
{
    valid = v;
}

TemplateType
Template::Type() const
{
    return type;
}

const char *
Template::ClassName() const
{
    switch (type) {
    case CLASS_TEMPLATE:
	return "classTemplate";
    case OBJECT_TEMPLATE:
	return "objectTemplate";
    case APP_TEMPLATE:
	return "appTemplate";
    default:
	return "<unknown template type>";
    }
}

const char *
Template::Name() const
{
    return name;
}

void
Template::Name(const char *n)
{
    delete [] name;
    name = Strdup(n);
}

void
Template::Install()
{
    switch (type) {
    case CLASS_TEMPLATE:
	Template::classTemplates->insert(this->Name(), this);
	break;
    case OBJECT_TEMPLATE:
	Template::objectTemplates->insert(this->Name(), this);
	break;
    case APP_TEMPLATE:
	Template::objectTemplates->insert(this->Name(), this);
	break;
    default:
	Error(EL_ABORT, "ClassObjectTemplate::Install",
	      "Unknown ClassObjectTemplate type.\n"
	      "\tThis is a nonrecoverable error as it indicates that",
	      "recompilation is necessary\n\ton this platform.", NULL);
	return;
    }
}

int
Template::Set(const char *n, FieldData& data)
{
    // real simple - all we have to do is insert 
    // these items into the attributeDict

    FieldData *fdPtr;

    fdPtr = new FieldData(data);

    (this->attributeDict).insert(n, fdPtr);

    return 1; // success
}

int
Template::InstallBooleanField(const char *id, unsigned char b)
{
    FieldData fieldData(BOOLEAN);
    int t;
    Attribute *a;

    t = currentTemplate->Type();

    switch(t) {
    case CLASS_TEMPLATE:
    case OBJECT_TEMPLATE:
	a = Attribute::GetClassAttributeByName(id);
	break;

    case APP_TEMPLATE:
        a = Attribute::GetAppAttributeByName(id);
	break;

    default:
	Error(EL_ERROR, "Template::InstallBooleanField",
              "Unknown Template Class", NULL);
        return 0;
    }

    if (a == NULL) {
	Error(EL_ERROR, "Template::InstallBooleanField", this->Name(),
	      ": Unknown template attribute:  ", id, NULL);
	return 0;  	// lookup failure
    }

    if (a->Tag() != BOOLEAN) {
	Error(EL_ERROR, "Template::InstallBooleanField", this->Name(),
	      ": Tag/Type Mismatch with looked up attribute",
	      a->Name(), NULL);
	return 0;  	// install failure
    }
	
    fieldData.Boolean(b);
    
    this->Set(id, fieldData);
     
    return 1;    // success!
}

int
Template::InstallIntegerField(const char *id, long i)
{
    FieldData fieldData(INTEGER);
    int t;
    Attribute *a;

    t = currentTemplate->Type();

    switch(t) {
    case CLASS_TEMPLATE:
    case OBJECT_TEMPLATE:
	a = Attribute::GetClassAttributeByName(id);
	break;

    case APP_TEMPLATE:
        a = Attribute::GetAppAttributeByName(id);
	break;

    default:
	Error(EL_ERROR, "Template::InstallIntegerField",
              "Unknown Template Class", NULL);
        return 0;
    }

    if (a == NULL) {
	Error(EL_ERROR, "Template::InstallIntegerField", this->Name(),
	      ": Unknown template attribute:  ", id, NULL);
	return 0;  	// lookup failure
    }

    if (a->Tag() != INTEGER) {
	Error(EL_ERROR, "Template::InstallIntegerField", this->Name(),
	      ": Tag/Type Mismatch with looked up attribute",
	      a->Name(), NULL);
	return 0;  	// install failure
    }
	
    fieldData.Integer(i);
    
    this->Set(id, fieldData);
     
    return 1;    	// success!
}

int
Template::InstallStringField(const char *id, const char *str)
{
    FieldData fieldData(STRING);
    int t;
    Attribute *a;

    t = currentTemplate->Type();

    switch(t) {
    case CLASS_TEMPLATE:
    case OBJECT_TEMPLATE:
	a = Attribute::GetClassAttributeByName(id);
	break;

    case APP_TEMPLATE:
        a = Attribute::GetAppAttributeByName(id);
	break;

    default:
	Error(EL_ERROR, "Template::InstallStringField",
              ": Unknown Template Class", NULL);
        return 0;
    }

    if (a == NULL) {
	Error(EL_ERROR, "Template::InstallStringField", this->Name(),
	      ": Unknown template attribute:  ", id, NULL);
	return 0;  	// lookup failure
    }

    if (a->Tag() != STRING) {
	Error(EL_ERROR, "Template::InstallStringField", this->Name(),
	      ": Tag/Type Mismatch with looked up attribute",
	      a->Name(), NULL);
	return 0;  	// install failure
    }
	
    fieldData.String(str);
    this->Set(id, fieldData);
    
    return 1;    // success!
}

int
Template::InstallStringListField(const char *id, const mListVal<mString>& sl)
{
    FieldData fieldData(STRINGLIST);
    int t;
    Attribute *a;

    t = currentTemplate->Type();

    switch(t) {
    case CLASS_TEMPLATE:
    case OBJECT_TEMPLATE:
	a = Attribute::GetClassAttributeByName(id);
	break;

    case APP_TEMPLATE:
        a = Attribute::GetAppAttributeByName(id);
	break;

    default:
	Error(EL_ERROR, "Template::InstallStringListField",
              "Unknown Template Class", NULL);
        return 0;
    }

    if (a == NULL) {
	Error(EL_ERROR, "Template::InstallStringListField", this->Name(),
	      ": Unknown template attribute:  ", id, NULL);
	return 0;  	// lookup failure
    }

    if (a->Tag() != STRINGLIST) {
	Error(EL_ERROR, "Template::InstallStringListField", this->Name(),
	      ": Tag/Type Mismatch with looked up attribute",
	      a->Name(), NULL);
	return 0;  	// install failure
    }
	
    fieldData.StringList(sl);
    this->Set(id, fieldData);
    
    return 1;    	// success!
}

int
Template::InstallProcField(const char *id, const char *proc)
{
    int t;
    Attribute *a;

    t = currentTemplate->Type();

    switch(t) {
    case CLASS_TEMPLATE:
    case OBJECT_TEMPLATE:
	a = Attribute::GetClassAttributeByName(id);
	break;

    case APP_TEMPLATE:
        a = Attribute::GetAppAttributeByName(id);
	break;

    default:
	Error(EL_ERROR, "Template::InstallBooleanField",
              "Unknown Template Class", NULL);
        return 0;
    }

    if (a == NULL)
	return 0;  	// lookup failure

    FieldData fieldData(a->Tag());  // declaration

    fieldData.SetProc(proc);
    this->Set(id, fieldData);  // we can do away with template::setproc()
    return 1; 	// success
}

const FieldData *
Template::FindByName(const XtObject& obj, const char *s)
{
    mString longName(obj.GetLongName());
    mString className(obj.GetClass());
    mString attrib_name(s);
    Template *tmpl = NULL;
    FieldData *fdPtr;
    Attribute *a;

    // search the object template database first
    if ((tmpl = objectTemplates->lookup(longName)) != 0) {
	fdPtr = (tmpl->attributeDict).lookup(attrib_name);
	if (fdPtr != NULL)
	    return fdPtr;
    }

    // then search the class object template database

    if ((tmpl = classTemplates->lookup(className)) != 0) {
	fdPtr = (tmpl->attributeDict).lookup(attrib_name);
	if (fdPtr != NULL)
	    return fdPtr;
    }

    a = Attribute::GetClassAttributeByName(s);

    if (a == NULL) {
	Error(EL_WARNING, "Attribute ", s, " does not exist!", NULL);
	return NULL;
    }

    // return default
    return Config::GetDefaultByName(a->RefName());
}

const FieldData *
Template::FindByName(const Client& c, const char *s)
{
    Template *tmpl = NULL;
    mString clientName(c.ClientName());
    FieldData *fdPtr;
    mString attrib_name(s);
    Attribute *a;

    if ((tmpl = appTemplates->lookup(clientName)) != 0) {
	fdPtr = (tmpl->attributeDict).lookup(attrib_name);
	if (fdPtr != NULL)
	    return fdPtr;
    }

    a = Attribute::GetAppAttributeByName(s);
    
    if (a == NULL) {
	Error(EL_WARNING, "Attribute ", s, " does not exist!", NULL);
	return NULL;
    }

    // return default
    return Config::GetDefaultByName(a->RefName());
}
 
// This is a Friend Fuction of FieldData
int
Template::WriteToFile(FILE *f) const
{
    FieldData *fdPtr;
    mDictValPtrIterator<mString, FieldData> iter(attributeDict);
    mString ms;
    int x;

    switch (Type()) {
    case CLASS_TEMPLATE:
	fprintf(f, "classTemplate");
	break;

    case OBJECT_TEMPLATE:
	fprintf(f, "objectTemplate");
	break;

    case APP_TEMPLATE:
	fprintf(f, "appTemplate");
	break;

    default:
	Error(EL_ERROR, "Template::WriteToFile", " unknown Template type ",
	      NULL);
    }

    fprintf(f, " %s", Name());
    fprintf(f, " {\n");

    // now dump the attribute values out
    while (iter.next()) {
	ms = iter.current();
	fdPtr = attributeDict.lookup(ms);

	fprintf(f, "    ");
	fprintf(f, "%s", (char *)ms);
	for (x = 0; x < 25-strlen((char *)ms); x++)
	    fprintf(f," ");
	fprintf(f, " = ");

	if (fdPtr->rep == PROC) {
	    fprintf(f, "[%s]\n", fdPtr->proc);
	    continue;  // jump to next iteration of while
	}

	// simple
	switch (fdPtr->Tag()) {
	case BOOLEAN:
	    fprintf(f, "%s\n", fdPtr->boolean?"TRUE":"FALSE");
	    break;
	
	case INTEGER:
	    fprintf(f, "%d\n", fdPtr->integer);
	    break;
	
	case STRING:
	    fprintf(f, "\"%s\"\n", fdPtr->str);
	    break;
	
	case STRINGLIST:
	    {
		mListValIterator<mString> list_iter(*(fdPtr->stringList));
		fprintf(f, "( ");

		if (list_iter.next()) {
		    fprintf(f, "\"%s\"", (char *)list_iter.current());
		    while (list_iter.next())
			fprintf(f, ", \"%s\"",(char *)list_iter.current());
		}

		fprintf(f, " )\n");
	    }
	}
    }
  
    fprintf(f, "}\n");	  
    fprintf(f, "\n");
    return 1;
}

/////////////////////////////////////////////////////////////////////////////


