// -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

#ifndef _Truetalk_h
#define _Truetalk_h

//
//  class definition for generic speech synthesis resouce
//
 
#include <stdio.h>		// for FILE *
#include <stdlib.h>

#include "TT_funcs.h"		// in the truetalk's h subdir.
#include "Speech.h"
#include "Loadable.h"


class Truetalk : public Speech {
private:
    TTserver _server;		// the truetalk server handle
    TTclient _client;		// the client handle
    unsigned int _port[2];	// server ports
    int _encoding;		// ENCODING_MULAW or ENCODING_PCM
    int _freq;			// 8000 or 12000
    int _who;			// OUTPUT_SERVER or OUTPUT_CLIENT
    int _where;			// OUTPUT_STDIO, OUTPUT_AUDIO, OUTPUT_FILE,
				// or OUTPUT_FUNC
    FILE *_outArg;	        // This is used to set the audio output
				// to the batphone to avoid conflicting
				// with mercator's audio output.
    
    int OpenTruetalk(const char *hostname, int time);
    int SyncSpeech(unsigned long ret);

    // The capabilities for this particular class of device.
    static Capabilities capabilities;
    static int		capabilitiesInitialized;

public:
    Truetalk(const char *hostname=NULL);
    virtual ~Truetalk();

    //
    // Inherited from Speech
    //
    virtual const Capabilities& GetCapabilities() const;

            int ConnectServer(const char* hostname);
            int StartServer();
    
    virtual int Speak(const char *str);		// speak some text
    virtual int SpeakUnix(const char *str);	// translate Unix and then speak
    virtual int Notify(const char *type);     	// does nothing 
    virtual int SetSpeechRate(int rate);	// set the rate of speech 
    virtual int StopSpeaking();			// interrupt the speaking
    virtual int SetVoice(const char *voice);   	// Sets the voice
    virtual int SetLanguage(const char *l);	// set language
    virtual int SetGain(int g);			// set the gain
    virtual void ShutDown();
};

#endif /* _Truetalk_h */
