// -*-Mode: C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/XServer.cc,v $
// $Revision: 6.25 $
// $Date: 1995/12/13 23:25:36 $
//
 
static const char rcsid[]="@(#) $Header: /net/hm10/keith/proj/mercator/6.0/RCS/XServer.cc,v 6.25 1995/12/13 23:25:36 keith Exp $";


#include <sys/types.h>		// for sockaddr, etc.
#include <sys/socket.h>		// for accept
#include <stdlib.h>
#include <unistd.h>             // for sleep

#include "AppModelMgr.h"
#include "Braille.h"
#include "Client.h"
#include "XtObject.h"
#include "XServer.h"
#include "RAPListener.h"
#include "RAP.h"
#include "Mercator.h"
#include "Speech.h"
#include "Error.h"
#include "Util.h"
#include "mList.h"
#include "mDictPtrPtr.h"
#include "mDictValPtr.h"
#include <X11/Xlib.h>
#include <X11/extensions/XTest.h>
#include <X11/keysym.h>
#include <X11/Xatom.h>
#include <X11/Xmu/WinUtil.h>

//
// Global mercator instance.
//
extern Mercator *mercator;

// ************************************************************************
// **                   Misc Functions                            *********
// ************************************************************************

int 
XServer::ErrorCatcher(Display *, XErrorEvent * event)
{
    // typedef struct {
    //    int type;
    //    Display *display;       /* Display the event was read from */
    //    XID resourceid;         /* resource id */
    //    unsigned long serial;   /* serial number of failed request */
    //    unsigned char error_code;       /* error code of failed request */
    //    unsigned char request_code;     /* Major op-code of failed request */
    //    unsigned char minor_code;       /* Minor op-code of failed request */
    // } XErrorEvent;

    if (event->type == Success) {
	return 0;		/* anyone know why we get these? */
    } else if (event->type == BadWindow) {
	char tmp[16];
	sprintf(tmp, "%d", event->resourceid);
	Error(EL_ERROR, "XServer::ErrorCatcher",
	      "Bad Window ID: ", tmp, NULL);
    } else {
	char tmp[16];
	sprintf(tmp, "%d", event->type);
	Error(EL_ERROR, "XServer::ErrorCatcher",
	      "XErrorEvent received, type = ", tmp, NULL);
    }
  
    return 0;
}


// ************************************************************************
// **                     Class Special Functions                    ******
// ************************************************************************
XServer::XServer() :
    FDInterest("XServer"),
    display(NULL),
    displayName(NULL),
    interpServer(0),
    mappedToplevels(),
    announce_mode(0),
    ALT_MASK(Mod1Mask), META_MASK(Mod4Mask), CONTROL_MASK(ControlMask),
    SHIFT_MASK(ShiftMask), LOCK_MASK(LockMask), NUMLOCK_MASK(Mod3Mask)
{
    char *dispEnv;
    
    if ((dispEnv = getenv("DISPLAY"))  == NULL)
	displayName = NULL;
    else
	displayName = Strdup(dispEnv);
}

// ************************************************************************

XServer::~XServer ()
{
    delete [] displayName;

    if (display)
	XCloseDisplay(display);
}

// ************************************************************************
// **                         Private Functions                      ******
// ************************************************************************

//
// Undo a passive grab on a key.
//
// Right now mercator still puts a passive grab on all XtObjects, so
// we have to ungrab all the ancestors of the given object in order to
// undo the passive grab. -wl
//
void 
XServer::UngrabKey(const XtObject *obj, KeySym key, unsigned int modifier) 
{
    XtObject *curObj = (XtObject *)obj;
    KeyCode keycode = XKeysymToKeycode(display, key);

    while (curObj != NULL) {
	XUngrabKey(display, keycode, modifier, curObj->GetWindowId());
	curObj = curObj->GetParent();
    }
    // Flush();
    XSync(display, False);
}

//
// GrabKey:  Do a passive grab on a key.  Note that the grab is installed on
// the toplevel window for the object passed.
//
// (It is also installed on all the ancestors of the given object. This is
// probably not necessary. -wl)
//
void
XServer::GrabKey(const XtObject *obj, KeySym key, unsigned int modifier)
{
    XtObject *curObj = (XtObject *)obj;
    KeyCode keycode = XKeysymToKeycode(display, key);

    while (curObj != NULL) {
	XGrabKey(display, keycode, modifier, curObj->GetWindowId(),
		 False, GrabModeSync, GrabModeAsync);
	curObj = curObj->GetParent();
    }
    // Flush();
    XSync(display, False);
}
  
// ************************************************************************
//
// The press parameter specifies what key events are to be faked:
//    0 = release
//    1 = press
//    2 = press and release
//
//
// Note the weirdage here -- Sun type-5 keyboards have two sets of
// cursor keys, one on the numeric keypad and one on the inverted-T
// cursor pad.  If we're doing a key event for a grabbed cursor key, then
// we need to grab/ungrab its twin.  This is a bogus, special-case hack that
// only has to be done because multiple keycodes may map to the same
// keysym.  If we don't do this, we ungrab the "wrong" arrow key, and
// then the screenreader generates a different arrow key in text mode
// and things screw up...				--keith
//
int
XServer::DoKeyEvent(const XtObject *obj, unsigned int key,
		    unsigned int modifier, int press, unsigned long delay, 
		    int grabbedAlready)
{
    if (obj == NULL) {
	Error(EL_ERROR, "XServer::DoKeyEvent", "called with NULL XtObject.",
	      NULL);
	return 0;
    }

    // This is necessary in case the user hits a key while this is going on...
    XGrabServer(display); // prevent other key events from interrupting us

    if (grabbedAlready) {
	if ((key == XK_Up) || /* (key == XK_KP_Up) || */ (key == XK_KP_8)) {
	    UngrabKey(obj, XK_Up, modifier);
	    //UngrabKey(obj, XK_KP_Up, modifier);
	    // Make sure XK_KP_8 is not zero before we grab it.
	    if(XKeysymToKeycode(display, XK_KP_8) > 0)
		UngrabKey(obj, XK_KP_8, modifier);     
	} else if ((key == XK_Down) || /* (key == XK_KP_Down) || */ (key == XK_KP_2)){
	    UngrabKey(obj, XK_Down, modifier);
	    //UngrabKey(obj, XK_KP_Down, modifier);
	    if(XKeysymToKeycode(display, XK_KP_2) > 0)
		UngrabKey(obj, XK_KP_2, modifier);
	} else if ((key == XK_Right) || /* (key== XK_KP_Right) || */ (key== XK_KP_6)){
	    UngrabKey(obj, XK_Right, modifier);
	    //UngrabKey(obj, XK_KP_Right, modifier);
	    if(XKeysymToKeycode(display, XK_KP_6) > 0)
		UngrabKey(obj, XK_KP_6, modifier);
	} else if ((key == XK_Left) || /* (key == XK_KP_Left) || */ (key == XK_KP_4)){
	    UngrabKey(obj, XK_Left, modifier);
	    //UngrabKey(obj, XK_KP_Left, modifier);
	    if(XKeysymToKeycode(display, XK_KP_4) > 0)
		UngrabKey(obj, XK_KP_4, modifier);
	} else {
	    UngrabKey(obj, key, modifier);
	}
    }

    if ((press == 1) || (press == 2))
	XTestFakeKeyEvent(display, XKeysymToKeycode(display, key), 1, delay);

    if ((press == 0) || (press == 2))
	XTestFakeKeyEvent(display, XKeysymToKeycode(display, key), 0, delay);

    XSync(display, False);	// make sure the key events get out

    if (grabbedAlready) {
	if ((key == XK_Up) || /* (key == XK_KP_Up) || */ (key == XK_KP_8)) {
	    GrabKey(obj, XK_Up, modifier);
	    //GrabKey(obj, XK_KP_Up, modifier);
	    if(XKeysymToKeycode(display, XK_KP_8) > 0)
		GrabKey(obj, XK_KP_8, modifier);
	} else if ((key == XK_Down) || /* (key == XK_KP_Down) || */ (key == XK_KP_2)){
	    GrabKey(obj, XK_Down, modifier);
	    //GrabKey(obj, XK_KP_Down, modifier);
	    if(XKeysymToKeycode(display, XK_KP_2) > 0)
		GrabKey(obj, XK_KP_2, modifier);
	} else if ((key == XK_Right) || /* (key== XK_KP_Right) || */ (key== XK_KP_6)){
	    GrabKey(obj, XK_Right, modifier);
	    //GrabKey(obj, XK_KP_Right, modifier);
	    if(XKeysymToKeycode(display, XK_KP_6) > 0)
		GrabKey(obj, XK_KP_6, modifier);
	} else if ((key == XK_Left) || /* (key == XK_KP_Left) || */ (key == XK_KP_4)){
	    GrabKey(obj, XK_Left, modifier);
	    //GrabKey(obj, XK_KP_Left, modifier);
	    if(XKeysymToKeycode(display, XK_KP_4) > 0)
		GrabKey(obj, XK_KP_4, modifier);
	} else {
	    GrabKey(obj, key, modifier);
	}
    }

    XUngrabServer(display);  // allow other events to reach the server
    XSync(display, False);

    return 1;
}

int
XServer::Dispatch()
{ 
    XEvent ev;
    int    status = 1;

    // for every event
    while (XCheckMaskEvent(display,
			   KeyPressMask|KeyReleaseMask|SubstructureNotifyMask,
			   &ev)) {
	switch (ev.type) {
	case KeyRelease: if (announce_mode) break;
	case KeyPress:
	  if (announce_mode) {
	    XKeyEvent *kev = (XKeyEvent *)&ev;
	    KeySym keysym;
	    char *keysym_string;
	    keysym = XLookupKeysym(kev, 0);
	    keysym_string = KeysymToString(keysym);

	    // end mode if Shift-Ctrl-Q is pressed
	    if ((kev->state & SHIFT_MASK) && (kev->state & CONTROL_MASK) &&
		((!strcmp(keysym_string, KeysymToString(XK_q))) ||
		 (!strcmp(keysym_string, KeysymToString(XK_Q))))) {
	      XServer *xserver = mercator->GetXServer();
	      Display *dpy = xserver->GetDisplay();
	      XUngrabKeyboard(dpy, CurrentTime);
	      announce_mode = 0;
	    }
	    else {
	      mercator->GetSpeech()->StopSpeaking();
	      mercator->GetSpeech()->Speak(keysym_string);
	      mercator->GetBraille()->SetText((const char **)&keysym_string, 1);
	    }
	  } else
	    status = DispatchKeyEvent(&ev);  
	  break;
	    
        // Here are the other events that SubstructureNotifyMask gets us.
	case ReparentNotify:
	    status = HandleReparentNotify(&ev);
	    break;
	case CirculateNotify:
	case ConfigureNotify:
	case CreateNotify:
	    break;
	case DestroyNotify:
	    status = HandleDestroyNotify(&ev);
	    break;
	case GravityNotify:
	    break;
	case MapNotify:
	    Error(EL_DEBUG, "XServer::Dispatch", "GOT MAP NOTIFY!", NULL);
	    //
	    // We need to check to see if the newly-mapped window is
	    // already in our model.  If it is, then we can just flag it
	    // as mapped.  If it is not, then we should try to connect
	    // to it.
	    //
	    status = HandleMapNotify(&ev);
	    break;
	    
	case UnmapNotify:
	    status = HandleUnmapNotify(&ev);
	    break;
	    
	default:
	    char tmp[16];
	    sprintf(tmp, "%d", ev.type);
	    Error(EL_WARNING, "XServer::Dispatch", 
		  "no handler installed for event type ", tmp, NULL);
	    status = 0;
	    break;
	}
    }

    XAllowEvents(display, AsyncKeyboard, CurrentTime);
    Flush();
    
    return status;
}


// ************************************************************************

//
// This routine is the initial step toward initiating a new client
// connection.  We look for reparent events on the root window to detect
// new potential toplevels.  The technique is fairly involved.  Here's how
// this routine works:
//   - When we detect a reparented window, we walk up the window tree from
//     the reparented window to find the initial child of the root window.
//     We do this to be sure that we have a good "starting point" to run
//     XmuClientWindow on (see next).
//   - Once we have the initial child of the root window, we call 
//     XmuClientWindow to get its child which has the WM_STATE property
//     set on it.  This merely indicates that the window is a shell, however,
//     not that it's the "one" window that we should message.
//   - We look for the presence of the WM_COMMAND property on the client
//     window.  This *should* (perhaps dependent on widget set) only exist
//     on the main application shell (this is how xlsclients detects
//     apps).
//   - After we've got a good candidate, we call the routine
//     AppModelMgr::RequestConnectionFromApp() to try to connect us up.
//     A new client instance will ONLY be created if both the Xt and Xlib
//     hooks manage to connect.
//
// The walking up and down the tree may seem extreme, but it's a good
// conservative way to make sure that we're doing the right thing.  It's
// probably not needed except in extraordinary circumstances however.
//
// NOTE that if RAP supports client-initiated connection to agents at startup,
// then we'll have a cleaner way to detect new clients (just let them come
// to us!)
//
int
XServer::HandleMapNotify(const XEvent *ev)
{ 
    Window 		win;
    Window 		root, parent, *children = NULL;
    unsigned int 	nchildren = 0;
    const XMapEvent	*event = &(ev->xmap);
    XTextProperty	text;
    Window		tester = None, ret;
    int 		done = 0;
    AppModelMgr		*amm = mercator->GetAppModelMgr();
	
    if (ev->type != MapNotify) {
	Error(EL_ERROR, "XServer::HandleMapNotify",
	      "Why are we being called?", NULL);
	return 0;
    }

    // 
    // The first thing we have to do is find the initial child of the 
    // root window.  We need this because XmuClientWindow will start 
    // walking down the window tree from here, looking for a window 
    // with WM_STATE set. 
    // 
    tester = event->window; 
    
    while (!done) { 
	if (XQueryTree(display, tester, &root, &parent,  
		       &children, &nchildren) == 0) {
	    Error(EL_ERROR, "XServer::HandleMapNotify", 
		  "XQueryTree failed!", NULL); 
	    return 0;
	}
	    
	if (parent == root) {
	    // We've hit the top.  Prev is the first-level child along 
	    // the path.  Pass out 'tester.' 
	    done = True; 
	} else { 
	    tester = parent; 
	} 

	if (children)
	    XFree(children); 
    } 
	
    //
    // Now we've got the initial child of the root window in
    // tester.  Find the client window child of it via XmuClientWindow.
    //
    if ((ret = XmuClientWindow(display, tester)) != tester) { 
	//
	// Add it to the list of mapped toplevels.
	//
	//mappedToplevels.append(tester);
	mappedToplevels.append(ret);

	//
	// Request a connection from the client window.  If InitiateConnection
	// returns FALSE, then it's NOT a new connection attempt, and
	// we should just do our shenanigans to make sure that the model
	// is up-to-date, since a new toplevel has just been mapped.
	//
	if (!mercator->GetRAPListener()->InitiateConnection(*this, ret)) {
	    //
	    // Get the RAP object that corresponds to this window.  To
	    // do this, we
	    //    - Get the XtObject assocated with the window.
	    //    - Get the client associated with the XtObject.
	    //    - Get the RAP instance associated with the client.
	    // Aiiigh!
	    //
	    XtObject *obj = mercator->GetAppModelMgr()->GetXtObject(ret);

	    if (!obj)
		Error(EL_ABORT, "XServer::HandleMapNotify",
		      "Inconsistence in Window->XtObject mappings.", NULL);

	    Client *client = obj->GetClient();

	    if (!client)
		Error(EL_ABORT, "XServer::HandleMapNotify",
		      "No client for XtObject?!", NULL);

	    if (!client->GetRAP())
		Error(EL_ABORT, "XServer::HandleMapNotify",
		      "Client has no RAP instance?!", NULL);

	    client->GetRAP()->ProcessXServerMapNotify(ret);
	}
	    
	return 1;
    }
    
    return 0;
}

// ************************************************************************

//
//  Right now the only thing we use this for is to take top level
//  windows out of our list of mappedToplevels
//

int
XServer::HandleUnmapNotify(const XEvent *ev)
{ 
    Window 		win;
    Window 		root, parent, *children = NULL;
    unsigned int 	nchildren = 0;
    const XMapEvent	*event = &(ev->xmap);
    XTextProperty	text;
    Window		tester = None, ret;
    int 		done = 0;
    AppModelMgr		*amm = mercator->GetAppModelMgr();
	
    if (ev->type != UnmapNotify) {
	Error(EL_ERROR, "XServer::HandleUnmapNotify",
	      "Why are we being called?", NULL);
	return 0;
    }

    tester = event->window; 
    
    mappedToplevels.del_item(tester);
    return 0;
}

// ************************************************************************

//
// Called when a child of the root window has been destroyed.  This
// could indicate that a client we're tracking has been shutdown.
//
int
XServer::HandleDestroyNotify(const XEvent *ev)
{
    Window	window = ((XDestroyWindowEvent *) ev)->window;
    XtObject	*obj = NULL;
    AppModelMgr	*amm = mercator->GetAppModelMgr();
    
    if (ev->type != DestroyNotify) {
	Error(EL_ERROR, "XServer::HandleMapNotify",
	      "Why are we being called?", NULL);
	return 0;
    }

    //
    // Ask the AMM for the XtObject that maps to this object.
    //
    obj = amm->GetXtObject(window);
    
    if (obj) {
	Error(EL_WARNING, "XServer::HandleDestroyNotify",
	      "Get a DestroyNotify for object ", obj->GetName(),
	      ".  NOT HANDLED!", NULL);
    }
    return 1;
}

// ************************************************************************

//
// I think the only time we really get these things is with CDE tear-off
// menus...
//
int
XServer::HandleReparentNotify(const XEvent *ev)
{
    Window window = ((XReparentEvent *) ev)->window;
    Window parent = ((XReparentEvent *) ev)->parent;
    XtObject *childObj, *parentObj;

    // Find the objects for these windows...
    if ((childObj = mercator->GetAppModelMgr()->GetXtObject(window)) == NULL) {
	Error(EL_WARNING, "XServer::HandleReparentNotify",
	      "Window being reparented is not in our model", NULL);
	return 0;
    }
    
    if ((parentObj = mercator->GetAppModelMgr()->GetXtObject(parent)) == NULL) {
	Error(EL_WARNING, "XServer::HandleReparentNotify",
	      "Window getting a new child is not in our model", NULL);
	return 0;
    }

    // If this guy had a parent before, remove us from it's children list.
    if (childObj->GetParent())
	childObj->GetParent()->DeleteChild(childObj);

    // Make the new parent-child relationship.
    childObj->SetParent(parentObj);
    parentObj->AddChild(childObj);

    return 1;
}

// ************************************************************************

int
XServer::DispatchKeyEvent(const XEvent *ev)
{ 
    EventPtr      	event;
    const XKeyEvent     *keyev = (XKeyEvent *) ev;
    XtObject      	*xtObj = NULL;
    Client        	*client = NULL;
    AppModelMgr   	*appMM = mercator->GetAppModelMgr();
    int           	interested = 0;
 
    event = new EventStruct;

    // only the events of KeyPress and KeyRelease are currently implemented
    if (ev->type == KeyPress)
	event->eventType = KEYPRESS;
    else if (ev->type == KeyRelease)
	event->eventType = KEYRELEASE;
    else {
	Error(EL_WARNING, "XServer::DispatchKeyEvent",
	      "got event other than KeyPress or KeyRelease.", NULL);
	return 0;
    }

    event->key      = XLookupKeysym((XKeyEvent *) ev, 0);
    event->modifier = ev->xbutton.state;

    // see if the XtObject is interested in the event
    xtObj  = appMM->GetXtObject(keyev->window);
    
    if (xtObj == NULL) {
	char tmp[16];
	sprintf(tmp, "%d", keyev->window);
	Error(EL_ERROR, "XServer::DispatchKeyEvent",
	      "Got event for window that doesn't exist: ",
	      tmp, NULL);
	return 0;
    }
    
    // First hand the event to the object to dispatch.
    interested = xtObj->DispatchEvent(event);

    // if the XtObject wasn't interested, see if the client is
    if (!interested) {
	client = xtObj->GetClient();
	interested = client->DispatchEvent(event);
    }

    // if the XtObject and Client wern't interested, see if there
    // is a global interest
    if (!interested)
	interested = appMM->DispatchEvent(event);

    delete event;
    return interested;
}

// ************************************************************************
// **                          Public Functions                      ******
// ************************************************************************

int
XServer::DoXOpenDisplay ()
{ 
    int event_basep, error_basep, majorp, minorp;

    display = XOpenDisplay(displayName);
    
    if (display == NULL) { 
	Error(EL_FATAL, "XServer::DoXOpenDisplay",
	      "couldn't open display (name = \"", 
	      displayName, "\")...exiting.", NULL);
    }

    // check for XTest extension.
    if (XTestQueryExtension(display, &event_basep, &error_basep,
			    &majorp, &minorp) == False) {
	Error(EL_FATAL, "XServer::DoXOpenDisplay",
	      "XTEST extension does not exist on server (name=\"",
	      displayName, "\")...exiting.", NULL);
    }

    InitializeModifierMasks();
    
    return 1;
}

//
// Initialize the modifier masks for this display.  The "state" member of
// Key events contains a bit mask denoting what modifier keys are in effect.
// These are server dependent, so this code produces a server-independent
// representation for these.
//
// XGetModifierMapping() returns eight arrays of keycodes.  Each array
// represents the set of keycodes that map onto a particular modifier mask.
// The index (1..8) corresponds to the "shift factor" of the bit mask.  Each
// array contains from 0 to max_keypermod entries of keycodes that can
// map onto the modifier.
//
// Note that Shift, Control, CapsLock, and NumLock masks will always be the
// same.  What we're really after here is the mapping from alt, meta, etc.,
// to Mod?Mask masks...but we set everything anyway.
//
void
XServer::InitializeModifierMasks()
{
    XModifierKeymap *map = XGetModifierMapping(display);
    int index = 0;

    // for each modifier map...
    for (register int i=0 ; i<8 ; i++) {
	// for each possible mapping...
	for (register int j=0 ; j<map->max_keypermod ; j++) {
	    if (map->modifiermap[index]) {
		KeySym ks = XKeycodeToKeysym(display,
 					     map->modifiermap[index], 0);

		switch (ks) {
		case XK_Shift_L:
		case XK_Shift_R:
		    SHIFT_MASK = (1 << i);
		    break;
		case XK_Control_L:
		case XK_Control_R:
		    CONTROL_MASK = (1 << i);
		    break;
		case XK_Caps_Lock:
		    LOCK_MASK = (1 << i);
		    break;
		case XK_Meta_L:
		case XK_Meta_R:
		    META_MASK = (1 << i);
		    break;
		case XK_Alt_L:
		case XK_Alt_R:
		    ALT_MASK = (1 << i);
		    break;
		case XK_Num_Lock:
		    NUMLOCK_MASK = (1 << i);
		    break;
		default:
		    break;
		}
	    }

	    index++;
	}
    }

    XFree(map);
}

// ************************************************************************
//
// This method requests that Substructure changes on the root window
// be reported to Mercator over the connection number.
// We want Substructure changes to be reported so that we can automatically
// add new applications to the Mercator system.
// The SubstructureNotifyMask will let mercator know when there is a
// CreateNotify or ReparentNotify event (among others).
// Applications can also be added by hand the the TCL "connect" command.
//
void
XServer::SolicitSubstructureNotify()
{
    Window        rootWindow;
    unsigned long eventMask;

    rootWindow = RootWindow(display, DefaultScreen(display));
    eventMask  = SubstructureNotifyMask;
    
    XSelectInput(display, rootWindow, eventMask);
    
    Flush();
}

// ************************************************************************
//
// the virtual function called by the FDInterest MainLoop when there is
// activity on the application
// mostly used when the user presses a key and it has been "Grabbed"
// AppModelMgr also gets data from XProto through HandleRequest and
// HandleEvent
//

int
XServer::HandleActivity(int fd, short type)
{
    if (fd == GetConnectionNumber())
	Dispatch();
    else
	Error(EL_ERROR, "XServer::HandleActivity",
	      "Why are we being called?!", NULL);

    return 0;
}

// ************************************************************************

int
XServer::MoveMouse(const XtObject *xtObject, unsigned long delay)
{ 
    Window window = xtObject->GetValidWindowId();
    Window child;
    int    x=0, y=0;
    int    winX = 2, winY = 2;		// HACK
  
    if (xtObject == NULL) {
	Error(EL_ERROR, "XServer::MoveMouse", "called with NULL XtObject.",
	      NULL);
	return 0;
    }


    if (!xtObject->IsRealized()) {
	Error(EL_ERROR, "XServer::MoveMouse", 
	      "Can't warp to unrealized widget (race condition?)", NULL);
	return 0;
    }

    //
    // If we're a gadget, position is relative to our parent's window (which is
    // denoted by our X,Y location).
    //
    if (xtObject->IsGadget()) {
	winX = xtObject->X() + 2;
	winY = xtObject->Y() + 2;
    }
    
    if (!xtObject->GetMapped()) {
	Error(EL_WARNING, "XServer::MoveMouse",
	      "Can't move to unmapped object.", NULL);
	return 0;
    }

    // Raise window??  Expensive, probably only needed if we change clients.
    XTranslateCoordinates(display, window, DefaultRootWindow(display),
			  winX, winY, &x, &y, &child);
    XTestFakeMotionEvent(display, 0, x, y, delay);
    Flush();
    
    return 1;
}

// ************************************************************************

int
XServer::MoveMouse(Window window, unsigned long delay)
{
    Window child;
    XWindowAttributes attr;
    int    x=2, y=2;		// HACK

    switch (window) {
    case RapUnrealized:
	Error(EL_ERROR, "XServer::MoveMouse", 
	      "Can't warp to unrealized window (race condition?)", NULL);
	return 0;
 
    case RapGadget:
	Error(EL_ERROR, "XServer::MoveMouse",
	      "window is part of a gadget; we don't handle those yet.", NULL);
	return 0;
 
    default:
	//
	// We must check map state before we move the pointer into
	// never-never land.
	//
	if (!GetWindowAttributes(window, &attr)) {
	    char tmp[16];
	    sprintf(tmp, "%lx", window);
	    Error(EL_ERROR, "XServer::MoveMouse",
		  "GetWindowAttributes failed for window ", tmp, NULL);
	    return 0;
	}

	if (attr.map_state != IsViewable) {
	    char tmp[16];
	    sprintf(tmp, "%lx", window);
	    Error(EL_ERROR, "XServer::MoveMouse",
		  "Window ", tmp, " is not mapped; can't warp.", NULL);
	}

    // Raise window??  Expensive, probably only needed if we change clients.
	XTranslateCoordinates(display, window, DefaultRootWindow(display),
			      0, 0, &x, &y, &child);
	XTestFakeMotionEvent(display, 0, x, y, delay);
	Flush();
	break;
    }

    return 1;
}

// ************************************************************************

int
XServer::MoveMouse(const XtObject *obj, int x, int y, unsigned long delay)
{
    Window window;
    Window child;
    int winX = 2, winY = 2;		// HACK

    if (obj == NULL) {
	Error(EL_WARNING, "XServer::MoveMouse", "called with a NULL XtObject.",
	      NULL);
        return 0;
    }

    window = obj->GetValidWindowId();

    if (!obj->IsRealized()) {
	Error(EL_ERROR, "XServer::MoveMouse", 
	      "Can't warp to unrealized widget (race condition?)", NULL);
	return 0;
    }
    
    if (!obj->GetMapped()) {
	Error(EL_WARNING, "XServer::MoveMouse",
	      "Can't move to unmapped object.", NULL);
	return 0;
    }
	
    //
    // If we're a gadget, position is relative to our parent's window (which is
    // denoted by our X,Y location).
    //
    if (obj->IsGadget()) {
	x = obj->X() + x;
	y = obj->Y() + y;
    }

    // Raise window??  Expensive, probably only needed if we change clients.
    XWarpPointer(display, None, window, 0, 0, 0, 0, x, y);
    Flush();

    return 1;
}

// ************************************************************************

int
XServer::MoveMouse(int x, int y, unsigned long delay, int screen)
{
    if (screen == -1) {
	// Default case.
	XTestFakeRelativeMotionEvent(display, x, y, delay);
    } else {
	// Use the specified screen.
	XTestFakeMotionEvent(display, screen, x, y, delay);
    }
    Flush();
    return 1;
}

#ifdef _OLD_STUFF_KEITH_
// ************************************************************************
XServer::MoveMouse(int screen, int x, int y, unsigned long delay)
{
  XTestFakeMotionEvent(display, screen, x, y, delay);
  Flush();
  return 0;
}
#endif

// ************************************************************************
XServer::DoMouseButtonEvent (int button, int press, unsigned long delay)
{
  XTestFakeButtonEvent(display, button, press, delay);
  Flush();
  return 1;
}

// ************************************************************************

int
XServer::CreateEventBind(XtObject* obj, EventPtr event, Action *action)
{
    if (obj == NULL || event == NULL || action == NULL) {
	Error(EL_ERROR, "XServer::CreateEventBind",
	      "called with bogus params:  either obj, event, or action"
	      " is NULL.", NULL);
	return 0;
    }
    
    obj->CreateEventBind(event, action);
  
    if (obj->GetWindowId() != None) {
	XGrabKey(display, XKeysymToKeycode(display, event->key), 
		 event->modifier, obj->GetValidWindowId(),
		 False, GrabModeSync, GrabModeAsync);
    }
    
    Flush();
    return 1;
}

// ************************************************************************

int
XServer::CreateEventBind(char *widgetClass, EventPtr event, Action *action)
{
    XtObject                       *obj;
    mDictValPtr<mString,XtObject>    *dict;
    mDictValPtrIterator<mString,XtObject> *iter;

    if (widgetClass == NULL || event == NULL || action == NULL) {
	Error(EL_ERROR, "XServer::CreateEventBind",
	      "called with bogus params:  either widgetClass, event, or action"
	      " is NULL.", NULL);
	return 0;
    }
    XtObject::CreateEventBind(widgetClass,event,action);
 
    dict = mercator->GetAppModelMgr()->xtobjectNames;
    iter=new mDictValPtrIterator<mString,XtObject>(*dict);
    while (iter->next()) {
	obj=dict->lookup(iter->current());
	if (strcmp(obj->GetClass(), widgetClass) == 0) {
	    if (obj->GetWindowId() != None) {
		XGrabKey(display, XKeysymToKeycode(display, event->key), 
			 event->modifier, obj->GetValidWindowId(), 
			 False, GrabModeSync, GrabModeAsync);
	    }
	}
    }
    delete iter;
    Flush();
    return 1;
}

// ************************************************************************
// We use this closure to install our grabs on the toplevel windows
// The struct is used to initialize at startup time. N.B. This struct
// is also used by the closure below as it ends up calling  this
// closure.
typedef struct {
    Display * display;
    EventPtr event;
} closureStartupInfo;

class installGrabsClosure : public mListClosure<XtObject> {
    Display *display;
    EventPtr event;
public:
    void startup(void *cdata) {
	closureStartupInfo *s=(closureStartupInfo *)cdata;
	display=s->display;
	event=s->event;
    }
    int action(XtObject *obj) { 
	if (obj->GetWindowId() != None)
	    XGrabKey(display, XKeysymToKeycode(display, event->key), 
		     event->modifier, obj->GetValidWindowId(), 
		     False, GrabModeSync, GrabModeAsync);
	return 1; // keep processing
    }
};

// ************************************************************************

int
XServer::CreateEventBind(Client *client, EventPtr event, Action *action)
{ 
    XtObject           *obj;
    closureStartupInfo info;
    installGrabsClosure *gc=new installGrabsClosure();

    if (client == NULL || event == NULL || action == NULL) {
	Error(EL_ERROR, "XServer::CreateEventBind",
	      "called with bogus params:  either client, event, or action"
	      " is NULL.", NULL);
	return 0;
    }
    
    client->CreateEventBind(event,action);

    const mList<XtObject> *tops = client->GetTopWindows();
    
    if (tops) {
	info.display=display;
	info.event=event;
	tops->walk(gc,(void *)&info);
	delete gc;
    }
    
    Flush();
    return 1;
}

// ************************************************************************
// we use this closure to walk over the list of clients and install the
// grabs on each of their top level windows
// we reuse the startupClosureInfo from a previous closure
// because we need to pass that information through the
// the other closure.
class walkClientsClosure: public mListClosure<Client> {
    closureStartupInfo *startInfo;
public:
    void startup(void *cdata) {
	startInfo=(closureStartupInfo *)cdata;
    }
    int action(Client *c) {
	installGrabsClosure *closure=new installGrabsClosure();
	const mList<XtObject> *tops=c->GetTopWindows();
	tops->walk(closure,startInfo);
	delete closure;
	return 1; // continue processing
    }
};

// ************************************************************************

int
XServer::CreateEventBind(EventPtr event, Action *action)
{ 
    ClientPtr client;

    mList<Client> *clientList = mercator->GetAppModelMgr()->GetAllClients();
    XtObject *obj;
    closureStartupInfo info;
    walkClientsClosure *closure=new walkClientsClosure();

    if (event == NULL || action == NULL) {
	Error(EL_ERROR, "XServer::CreateEventBind",
	      "called with bogus params:  either event or action"
	      " is NULL.", NULL);
	return 0;
    }
    
    mercator->GetAppModelMgr()->CreateEventBind(event,action);
    info.display=display;
    info.event=event;
    clientList->walk(closure,&info);
    delete closure;
    Flush();
    return 1;
}

// ************************************************************************

int
XServer::SendExposeEvent(Window w, int x, int y,
			 unsigned width, unsigned height)
{
    XExposeEvent event;

    event.type = Expose;
    event.window = w;
    event.x = x;
    event.y = y;
    event.width = width;
    event.height = height;
    event.count = 0;
    
    if (!XSendEvent(display, w, True, ExposureMask, (XEvent *) &event)) {
	Error(EL_ERROR, "XServer::SendExposeEvent",
	      "XSendEvent failed.", NULL);
	return 0;
    }

    return 1;
}

// ************************************************************************

// ************************************************************************
//
// called when a top-level window appears so that it will grab the right keys
// the client and global event grabs will be set in this window
//
int 
XServer::GrabInThisWindow(XtObject *newObj, int grabNoMatterWhat)
{
    EventStruct *es;

    if (newObj == NULL) {
	Error(EL_ERROR, "XServer::GrabInThisWindow",
	      "called with a NULL XtObject.", NULL);
	return 0;
    }

    Window win = newObj->GetValidWindowId();
    
    if ((win == None) || (win == 0) || (win == 2)) {
	Error(EL_ERROR, "XServer::GrabInThisWindow",
	      "bogus window argument.", NULL);
	return 0;
    }

    mDictPtrPtr<EventStruct,Action> *actions;
    mDictPtrPtrIterator<EventStruct,Action> *iter;

    // Get all XtObject class grabs
    actions = XtObject::GetActions((char *) newObj->GetClass());
    
    // Remember that we've already put grabs on this window.
    newObj->GrabbedYet(TRUE);
    
    if (actions != NULL) {
	iter=new mDictPtrPtrIterator<EventStruct,Action>(*actions);
	while ((es=iter->next())!=0) {
	    XGrabKey(display, 
		     XKeysymToKeycode(display, es->key),
		     es->modifier, win, False, GrabModeSync, GrabModeAsync);
	}
	delete iter;
    }
	
    
    // Return if this is not a top level window and grabNoMatterWhat is not set
    
    if (!grabNoMatterWhat) {
	if (newObj->GetParent() != NULL) {
	    Flush();
	    return 0;
	}
    }

    // Get all the Client grabs
    // Just need to do the grabs--not the insertion into the Client's database
    
    actions = newObj->GetClient()->GetActions();
    iter=new mDictPtrPtrIterator<EventStruct,Action>(*actions);
    while ((es=iter->next())!=0) {
	XGrabKey(display, 
		 XKeysymToKeycode(display, es->key),
		 es->modifier,win, False, GrabModeSync, GrabModeAsync);
    }
    delete iter;

    // Get all the Global grabs 
    // Just need to do the grabs -- not the insertion into the global database
    
//    actions = newObj->GetClient()->GetAppModelMgr()->GetActions();
    actions = mercator->GetAppModelMgr()->GetActions();
    iter=new mDictPtrPtrIterator<EventStruct,Action>(*actions);
    while ((es=iter->next())!=0) {
	XGrabKey(display, 
		 XKeysymToKeycode(display, es->key),
		 es->modifier, win,False, GrabModeSync, GrabModeAsync);
    } 
    delete iter;

    Flush();
    return 1;
}

// ************************************************************************

XtObject * 
XServer::GetTopWindow(const XtObject *obj)
{
    XtObject *temp = (XtObject *) obj;
    XtObject *root = NULL;
    mList<Client> *clientList = mercator->GetAppModelMgr()->GetAllClients();
    ClientPtr client;
    XtObject *iter;
    // first iterator knows its going to 
    mListIterator<Client> clientIter(*clientList);
    mListIterator<XtObject> *XtObjectIter;
    // This returns the top window of the xtobject
    // This is necessary because if you just search for when parent == NULL,
    // then that won't cover the pop-up window case
    // [rehacked by IES to work with mListIterator May 22 '95]
    while (root == NULL) {
	clientIter.reset();
	// check all clients looking for a root
	while ((client=clientIter.next())) {
	    // init the iterator with the toplevel list
	    const mList<XtObject> *tops=client->GetTopWindows();
	    // don't bother if they don't have any toplevel windows
	    if (tops) {
		XtObjectIter=new mListIterator<XtObject>(*tops);
		while ((iter=XtObjectIter->next())) {
		    // check every toplevel
		    if (strcmp(iter->GetUniqueName(),
			       temp->GetUniqueName())== 0) {
			root = temp;
			// found it
			break;
		    }
		}
	    }
	}
	// checked all clients ... so go up
	if (temp->GetParent() == NULL) {
	    root=temp;
	} else {
	    temp = temp->GetParent();
	}
    }
    return root;
}

// ************************************************************************

void
XServer::LoadErrorHandlers()
{
  // flush the queue and sync up with the X server
  XSync(display,False);

  // Load an error handler for non-fatal error events.
  XSetErrorHandler(ErrorCatcher);
}

// ************************************************************************

int
XServer::IsMappedToplevel(Window w) const
{
    Window dummy;

    return mappedToplevels.search(w, dummy);
}
