// -*-Mode:C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

#ifndef __XSERVER_H__
#define __XSERVER_H__

//
// This is the class XServer.  
//


//
// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/XServer.h,v $
// $Revision: 6.9 $
// $Date: 1995/11/29 00:12:01 $
//

#ident "$Id: XServer.h,v 6.9 1995/11/29 00:12:01 keith Exp $"


#include <stdio.h>
#include <strings.h>
#include <X11/Xlib.h>

#include "mString.h"
#include "mListVal.h"
#include "FDInterest.h"
#include "Storage.h"	// for EventPtr
#include "XtObject.h"

class Mercator;
class Client;
class Action;


class XServer : public FDInterest {
private:
    Display  	*display;
    char    	*displayName;
    int       	interpServer;

    // Keep a list of all direct MAPPED children of the root window.
    mListVal<Window>	mappedToplevels;
  
    void 	UngrabKey(const XtObject *obj, KeySym key, 
			  unsigned int modifier);
    void 	GrabKey(const XtObject *obj, KeySym key,
			unsigned int modifier);
  
    int		Dispatch();
    int		DispatchKeyEvent(const XEvent *);
    int		HandleMapNotify(const XEvent *);
    int		HandleUnmapNotify(const XEvent *);
    int		HandleDestroyNotify(const XEvent *);
    int		HandleReparentNotify(const XEvent *);
  
    // Static function to be installed as an error handler.
    static int	ErrorCatcher(Display *dpy, XErrorEvent *event);
  
    // Default port number for the TCL interpreter.  May be overridden
    // via the TCL_PORT environment variable.
    enum 	{ TCL_PORT_DEFAULT = 10001 };

    // Flag for key echo mode
    int announce_mode;

    void		InitializeModifierMasks();
    
public:
    XServer(); 
    virtual ~XServer();

    //
    // Open the display and check for the presence of XTEST.
    //
    int      		DoXOpenDisplay();
    //
    // These are the modifier masks for various modifier keys.  Yes, these
    // are data members in the public interface...
    //
    int			ALT_MASK, META_MASK, CONTROL_MASK, SHIFT_MASK,
			LOCK_MASK, NUMLOCK_MASK;
    //
    // Is the specified window a mapped toplevel?
    //
    int			IsMappedToplevel(Window) const;
    //
    // Return the display structure. 
    //
    Display 		*GetDisplay() const;
    //
    // Return the name of the display.
    //
    const char    	*DisplayName() const;
    //
    // Get the socket fd of the X Server connection. 
    //
    int      		GetConnectionNumber() const;
    //
    // Request that we receive SubstructureNotify messages (to look for
    // new clients).
    //
    void     		SolicitSubstructureNotify();

    // The virtual function called by the FDInterest MainLoop when there is
    // activity on the application
    // mostly used when the user presses a key and it has been "Grabbed"
    // AppModelMgr also gets data from XProto through HandleRequest, 
    // HandleEvent
    int 	HandleActivity(int fd, short type);

    //
    // Warp to (0,0) of the specified XtObject.
    //
    int		MoveMouse(const XtObject *, unsigned long delay=0);
    //
    // Warp to (0,0) of the specified window.
    //
    int		MoveMouse(Window, unsigned long delay=0);
    //
    // Move to absolute (x,y) on the display.
    //
    int		MoveMouse(int x, int y, unsigned long delay = 0, 
			  int screen = -1);
#ifdef _OLD_STUFF_KEITH_
    int		MoveMouse(int screen, int x, int y, unsigned long delay=0);
#endif /* _OLD_STUFF_KEITH_ */
    //
    // Potential HACK.  I was going to roll this into the other MoveMouse
    // above which takes an XtObject.  But the implementation is very
    // different... strange... -keith
    //
    int		MoveMouse(const XtObject *obj, int x, int y, 
			  unsigned long delay = 0);

    int		DoMouseButtonEvent(int button, int press,
				   unsigned long delay=0); 
    int		DoKeyEvent(const XtObject *obj, unsigned int key, 
			   unsigned int modifier,
			   int press, unsigned long delay, 
			   int grabbedAlready);

    int		CreateEventBind(EventPtr,  Action*);
    int		CreateEventBind(Client*,   EventPtr, Action*);
    int		CreateEventBind(XtObject*, EventPtr, Action*);
    int		CreateEventBind(char *widgetClass, EventPtr, Action*);


    void 	LoadErrorHandlers();

    //
    // Miscellaneous wrappers around Xlib.
    //
    int		SendExposeEvent(const XtObject& obj);
    int		SendExposeEvent(Window w, int x, int y,
				unsigned width, unsigned height);

    Status	GetWindowAttributes(Window w, XWindowAttributes *wattr);
    Status	GetWindowAttributes(const XtObject& o, XWindowAttributes *attr);
    
    int		GrabInThisWindow(XtObject *obj, int grabNoMatterWhat = 0);

    XtObject 	*GetTopWindow(const XtObject *);

    KeySym 	StringToKeysym(const char *key);
    KeySym 	KeycodeToKeysym(KeyCode keycode);
    char	*KeysymToString(KeySym keySym);
  
    void 	Flush();
    void	SetTextProperty(Window w, XTextProperty *text, Atom property);
    Window	GetRootWindow() const;
    Pixel	GetBlackPixel() const;
    Pixel	GetWhitePixel() const;
    Atom	InternAtom(const char *propertyName, Bool onlyIfExists);
    Status	SendEvent(Window w, Bool propagate, long eventMask, XEvent *ev);
    Window	CreateWindow(Window parent, int x, int y, int width, int height,
			     int borderWidth, long border, long background);
    void	DestroyWindow(Window window);
    void	MapWindow(Window window);
    void	ResizeWindow(Window window, unsigned int width,
			     unsigned int height);
    void	ChangeWindowAttributes(Window w, long valuemask,
				       XSetWindowAttributes *attr);

    void	SelectInput(Window w, long eventMask);
    Status	GetWMName(Window w, XTextProperty *textProp);
    
    XFontStruct *LoadQueryFont(const char *fontname);
    void 	ImageText(Window window, GC gc, int x, int y, const char *str);
    
    GC 		CreateGC(Window drawable, long valueMask, XGCValues *values);
    void 	SetForeground(GC gc, unsigned long fg);
    void 	SetBackground(GC gc, unsigned long bg);
    void 	SetFunction(GC gc, int func);
    void 	SetFont(GC gc, Font font);

    void	ClearArea(Window window, int x, int y,
			  unsigned int w, unsigned int h,
			  Bool exposures = FALSE);

    void SetAnnounceMode(int val) {announce_mode = val;};
    int GetAnnounceMode(void) {return announce_mode;}
};


////////////////////////////////////////////////////////////////////////

inline Display *
XServer::GetDisplay() const
{
    return display;
}

inline const char *
XServer::DisplayName() const
{
    return displayName;
}

inline int
XServer::GetConnectionNumber() const
{
    return ConnectionNumber(display);
}

inline int
XServer::SendExposeEvent(const XtObject& obj)
{
    return SendExposeEvent(obj.GetWindowId(), 0, 0, obj.Width(), obj.Height());
}

inline Status
XServer::GetWindowAttributes(const XtObject& o, XWindowAttributes *attr)
{
    return GetWindowAttributes(o.GetWindowId(), attr);
}

inline KeySym
XServer::StringToKeysym(const char *key)
{
    return XStringToKeysym(key);
}

inline KeySym
XServer::KeycodeToKeysym(KeyCode keycode)
{
    return XKeycodeToKeysym(display, keycode, 0);
}

inline char *
XServer::KeysymToString(KeySym keySym)
{
    return XKeysymToString(keySym);
}

inline void
XServer::Flush()
{
    XFlush(display);
}

inline void
XServer::SetTextProperty(Window w, XTextProperty *text, Atom property)
{
    XSetTextProperty(display, w, text, property);
}

inline Window
XServer::GetRootWindow() const
{
    return DefaultRootWindow(display);
}

inline Pixel
XServer::GetBlackPixel() const
{
    return BlackPixel(display, DefaultScreen(display));
}

inline Pixel
XServer::GetWhitePixel() const
{
    return WhitePixel(display, DefaultScreen(display));
}

inline Atom
XServer::InternAtom(const char *propertyName, Bool onlyIfExists)
{
    return XInternAtom(display, propertyName, onlyIfExists);
}

inline Status
XServer::SendEvent(Window w, Bool propagate, long eventMask, XEvent *event)
{
    return XSendEvent(display, w, propagate, eventMask, event);
}

inline Window
XServer::CreateWindow(Window parent, int x, int y, int width, int height,
		      int borderWidth, long border, long background)
{
    return XCreateSimpleWindow(display, parent, x, y, width, height,
			       borderWidth, border, background);
}

inline void
XServer::DestroyWindow(Window window)
{
    XDestroyWindow(display, window);
}

inline void
XServer::MapWindow(Window window)
{
    XMapWindow(display, window);
}

inline void
XServer::ResizeWindow(Window window, unsigned int width, unsigned int height)
{
    XResizeWindow(display, window, width, height);
}

inline void
XServer::ChangeWindowAttributes(Window window, long valueMask,
				XSetWindowAttributes *attr)
{
    XChangeWindowAttributes(display, window, valueMask, attr);
}

inline XFontStruct *
XServer::LoadQueryFont(const char *fontname)
{
    return XLoadQueryFont(display, fontname);
}

inline void
XServer::ImageText(Window window, GC gc, int x, int y, const char *str)
{
    if (!str) return;

    XDrawString(display, window, gc, x, y, str, strlen(str));
}

inline GC
XServer::CreateGC(Window drawable, long valueMask, XGCValues *values)
{
    return XCreateGC(display, drawable, valueMask, values);
}

inline void
XServer::SetForeground(GC gc, unsigned long fg)
{
    XSetForeground(display, gc, fg);
}

inline void
XServer::SetBackground(GC gc, unsigned long bg)
{
    XSetBackground(display, gc, bg);
}

inline void
XServer::SetFunction(GC gc, int func)
{
    XSetFunction(display, gc, func);
}

inline void
XServer::SetFont(GC gc, Font font)
{
    XSetFont(display, gc, font);
}

inline void
XServer::ClearArea(Window window, int x, int y, unsigned int w, unsigned int h,
		   Bool exposures)
{
    XClearArea(display, window, x, y, w, h, exposures);
}

inline void
XServer::SelectInput(Window window, long eventMask)
{
    XSelectInput(display, window, eventMask);
}

inline Status
XServer::GetWindowAttributes(Window window, XWindowAttributes *wattr)
{
    return XGetWindowAttributes(display, window, wattr);
}

inline Status
XServer::GetWMName(Window window, XTextProperty *textProperty)
{
    return XGetWMName(display, window, textProperty);
}

#endif __XSERVER_H__
