// -*-Mode:C++-*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#ifndef _XT_OBJECT_H_
#define _XT_OBJECT_H_

//
// The XtObject maintains information about each widget or gadget in an
// interface.  For the most part it's just a piece of storage.  It has
// special behaviours.  There are some member variables that should be
// private and accessed through member functions.
// The data that is stored is specifically:
//
//   Data
//   ----
//   Resource names and their values (in string format)
//   List of event grabs for the instance of XtObject


//
// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/XtObject.h,v $
// $Revision: 6.13 $
// $Date: 1995/09/04 20:25:54 $
//

#ident "$Id: XtObject.h,v 6.13 1995/09/04 20:25:54 keith Exp $"


class XtObject;
typedef XtObject *XtObjectPtr;

#include <sys/time.h>
#include <X11/Intrinsic.h>
#include "Action.h"
#include "Storage.h"
#include "TextRep.h"
#include "mString.h"
#include "mDictPtrPtr.h"
#include "mDictValPtr.h"

typedef mDictPtrPtr<EventStruct,Action> * eventDictPtr;
typedef mDictPtrPtr<EventStruct,Action> eventDict;

class ResourceCache;
class ScreenReader;
class Client;

class XtObject : public Storage {
private:
    //
    // This is the ID of the widget as reported by Xt.  It's guaranteed
    // to be unique within a given client.
    //
    Object 	id;

    char 	*myName;     	// Name of the widget when created.
    char 	*myClass;    	// Xt Class name of the widget
    char 	**longName;   	// Xrm-style fully specified name
    char 	*uniqueName; 	// A unique, across-client identifier

    // We keep the unique id around so we can do comparisons on XtObjects
    // using it.  This is the basis of operator== and operator<.  Note that not
    // only the dictionary code needs operator== and operator< but the
    // ::modelManager (Interp.cc) function needs it so it can sort child lists.
    // This count is basically the number of times  that XtObject::XtObject 
    // has been called.
    unsigned long xtObjectIdentifier;

    XtObject 	*parent;  	// pointer to XtObject which is parent
    Window     	window;  	// XID of Window for widget

    //
    // Note that x, y, width, and height are weird.  In general, we use (and
    // trust) the resource values for these, although we do some extra work
    // to *set* the resources when we get new configuration information from
    // other sources as well (ConfigureNotify for example).
    //
    // Still, we keep these values here because at several points we need
    // to know this information before the resources are known to us.
    // In general, the routines which request these values use the resources
    // if they're available, otherwise they fall back to here.
    //
    int 	x, y;          	// window-relative x and y as reported by widget
    int 	width, height; 	// widget width and height
    int 	mapped;        	// state information
    int		managed;       	// are we managed?
    int		borderWidth;	// used by ProtoTextRep
    int		grabsInstalled;	// have we grabbed in this window yet?

    ResourceCache		*resources;
    eventDictPtr               eventActions; // events in the XtObject

    //
    // This is a dictionary of dictionaries.  The key (string) is the widget
    // class.  The associated dictionary is the dictionary of eventBinds for
    // that particular widget class.
    //
    static mDictValPtr<mString,eventDict> *classDictionary;
    
    // This stuff used to be public for some god unknown reason...
    ScreenReader	*sreader;	// ScreenReader instance for TextRep
    TextRep		*textrep;	// non-NULL if text is drawn to us
    Client		*client;	// who owns us?
    mDictValPtr<Object,XtObject> *children;	// our children
    XtObject		*popReturn;	// remember where to return to

    int			ResourceIsUnsafe(const char *name) const;
    void		DoBlockGetValues(const char *name) const;

public:
    XtObject(Client *cl, Object objID, const char *name, const char *theclass,
	     XtObject *parent, Window win);
    ~XtObject();

    // functions for getting and setting some internal values.
    Object       	GetObjID() const;
    void         	SetName(const char *newName);
    const char 		*GetName() const;
    void         	SetClass(const char *newClass);
    const char 		*GetClass() const;
    const char 		*GetLongName() const;
    const char 		*GetUniqueName() const;
    
    //
    // GrabbedYet indicates whether or not we've installed all the
    // requisite grabs in this window yet.  (Installing grabs is an
    // expensive operation, so we want to keep from doing it if possible.)
    //
    int			GrabbedYet() const;
    void		GrabbedYet(int g);
    
    ScreenReader	*GetScreenReader() const;
    void		SetScreenReader(ScreenReader *s);
    TextRep		*GetTextRep() const;
    void		SetTextRep(TextRep *t);
    Client		*GetClient() const;
    void		SetClient(Client *c);
    XtObject		*GetPopReturn() const;
    void		SetPopReturn(XtObject *p);

    mDictValPtr<Object,XtObject>	*GetChildren() const;
    
    void       		SetParent(XtObject *newParent);
    XtObject 		*GetParent() const;

    void   		SetWindowId(Window newWindowID);
    //
    // GetWindowID() returns the ID stored in this class, which may be
    // RapGadget or RapUnrealized.
    //
    // GetValidWindowID() has a special check for gadgets and returns
    // the ID of the first non-gadget parent.  This is mostly used only
    // for event binds where we have to establish key grabs.
    //
    Window 		GetWindowId() const;
    Window		GetValidWindowId() const;

    int			BorderWidth() const;
    void		BorderWidth(int b);

    short  		X() const;
    void 		X(short v);

    short 		Y() const;
    void 		Y(short v);

    short  		Width() const;
    void 		Width(short w);

    short  		Height() const;
    void 		Height(short h);

    void 		SetMapped(int m);
    int  		GetMapped() const;

    void 		SetManaged(int m);
    int  		GetManaged() const;

    int			IsRealized() const;
    int			IsGadget() const;

    // Value is interpreted as a "raw" value (as received from RAP)
    void		CreateResource(const char *name,
				       const char *className,
				       const char *type,
				       const char *value = NULL);

    void		SetResource(const char *name,
				    const char *type,
				    const char *value);

    int			ResourceExists(const char *name) const;
    // Print the entire resource
    void		PrintResource(const char *name, ostream& s) const;
    // Print just the value (primarily used by Interp to get values into stringland)
    void		PrintResourceValue(const char *name, ostream& s) const;
    // Dump all resources
    void		PrintResources(ostream& s) const;

    void		SetResourceValueRaw(const char *name, const char *val);
    
    void         	SetResourceValue(const char *name, const char *value);
    void         	SetResourceValue(const char *name, unchar value);
    void         	SetResourceValue(const char *name, ushort value);
    void         	SetResourceValue(const char *name, ulong value);

    
    const char 		*GetResourceValueString(const char *name,
						const char **rep = NULL) const;
    unchar		GetResourceValueByte(const char *name,
					     const char **rep = NULL) const;
    ushort		GetResourceValueShort(const char *name,
					      const char **rep = NULL) const;
    ulong		GetResourceValueLong(const char *name,
					     const char **rep = NULL) const;

    void 		AddChild(const XtObject *newChild);
    void 		DeleteChild(const XtObject *oldChild);

    void 		PrintInfo() const;
    
    //
    // Return whether or not this object seems to be a shell (look at
    // parentage, isShell, etc.)
    //
    int			CheckIsShell() const;
  
    // virtual functions inherited from Storage
    int  		DispatchEvent(EventPtr event);
    void 		CreateEventBind(EventPtr event, Action *function);
    static void 	CreateEventBind(const char *widgetClass,
					EventPtr event, Action *function);
    int  		DeleteEventBind(EventPtr event);
    static int 		DeleteEventBind(const char *widgetClass,
					EventPtr event);

    mDictPtrPtr<EventStruct,Action> *GetActions() 
			    { return eventActions; }
    static mDictPtrPtr<EventStruct,Action> *GetActions(const char *widgetClass);
    //
    // operator == and operator< implemented so these can be 
    // stored in data structures.  Determination is based on unique ID.
    //
    int operator==(const XtObject& obj) const;
    int operator<(const XtObject& obj) const;

};


////////////////////////////////////////////////////////////////////////

inline Object		XtObject::GetObjID() const { return id; }
inline const char * 	XtObject::GetUniqueName() const { return uniqueName; }
inline Window		XtObject::GetWindowId() const { return window; }
inline const char *	XtObject::GetName() const { return myName; }
inline const char *	XtObject::GetClass() const { return myClass; }
inline XtObject *	XtObject::GetParent() const { return parent; }
inline void		XtObject::SetWindowId(Window w) { window = w; }
inline int 		XtObject::GrabbedYet() const { return grabsInstalled;}
inline void 		XtObject::GrabbedYet(int g) { grabsInstalled = g; }
inline ScreenReader *	XtObject::GetScreenReader() const { return sreader; }
inline void 		XtObject::SetScreenReader(ScreenReader *s) {sreader= s;}
inline TextRep *	XtObject::GetTextRep() const { return textrep; }
inline void 		XtObject::SetTextRep(TextRep *t) { textrep = t; }
inline Client * 	XtObject::GetClient() const { return client; }
inline void 		XtObject::SetClient(Client *c) { client = c; }
inline XtObject * 	XtObject::GetPopReturn() const { return popReturn; }
inline void 		XtObject::SetPopReturn(XtObject *p) { popReturn = p; }

inline int		XtObject::BorderWidth() const { return borderWidth; }
inline void		XtObject::BorderWidth(int b) { borderWidth = b; }

//
// Note that these are *completely* divorced from the resource values...
//
inline void 		XtObject::X(short v) { x = v; }
inline void 		XtObject::Y(short v) { y = v; }
inline short		XtObject::X() const { return x; }
inline short		XtObject::Y() const { return y; }
inline void 		XtObject::Width(short w)
				{
				    width = w;
				    if (textrep)
					textrep->RefreshSize();
				}
inline void 		XtObject::Height(short h)
				{
				    height = h;
				    if (textrep)
					textrep->RefreshSize();
				}
inline short 		XtObject::Width() const { return width; }
inline short 		XtObject::Height() const { return height; }

inline int 		XtObject::GetManaged() const { return managed; }

#endif _XT_OBJECT_H_
