/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/
/*
 * Implementation of agent-side RAP messages.
 */

#include <stdio.h>

#include "agent.h"

#include <X11/ICE/ICEproto.h>
#include <X11/ICE/ICEmsg.h>

#include "RapLibint.h"


/*****************************************************************/

int
RAPHelloRequest(RapClient client)
{
    RapHelloMessage *msg;
    
    fprintf(stderr, "SENDING HELLO REQUEST, major = %d\n",
	    client->majorOpcode);
    
    IceSimpleMessage(client->iceConn, client->majorOpcode, RapHelloRequest); 
    IceFlush(client->iceConn);

    return 1;
}


int
RAPQueryTreeRequest(RapClient client)
{
    RapMessageHeader *msg;
    
    fprintf(stderr, "SENDING QUERY TREE REQUEST.\n");
    
    IceSimpleMessage(client->iceConn, client->majorOpcode, RapQueryTreeRequest);
    IceFlush(client->iceConn);

    return 1;
}

int
RAPFullQueryTreeRequest(RapClient client)
{
    RapMessageHeader *msg;
    
    fprintf(stderr, "SENDING FULL QUERY TREE REQUEST.\n");
 
    IceSimpleMessage(client->iceConn, client->majorOpcode, RapFullQueryTreeRequest);
    IceFlush(client->iceConn);

    return 1;
}


int
RAPGetResourcesRequest(RapClient client, WidgetList widgets, long numWidgets)
{
    RapMessageHeader *header;
    int totalSizeInBytes = 0;
    int padding = 0;

    if (client == NULL)
	return 0;
    
    if (numWidgets <= 0)
	return 0;

    printf("SENDING GET RESOURCES REQUEST\n");
    
    IceGetHeader(client->iceConn,
		 client->majorOpcode,
		 RapGetResourcesRequest,
		 SIZEOF(RapMessageHeader),
		 RapMessageHeader,
		 header);

    /* compute the size of the message */
    totalSizeInBytes += 4;				/* size of list */
    totalSizeInBytes += numWidgets * sizeof(CARD32);

    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;

    /* update the length field */
    header->length += WORD64COUNT(totalSizeInBytes);

    WRITE_CARD32_LIST(client->iceConn, widgets, numWidgets);

    /* pad to 64 boundary.  Is this necessary? */
    while (padding--) {
	char zero = '\0';
	IceWriteData(client->iceConn, 1, &zero);
    }

    printf("SENDING GET RESOURCES REQUEST: %d widgets, byte size = %ld, WORD64 size = %d\n", numWidgets, totalSizeInBytes, header->length);

    IceFlush(client->iceConn);

    return 1;
}


int
RAPGetGCValuesRequest(RapClient client, GContext *GCs, long numGCs)
{
   RapMessageHeader *header;
    int totalSizeInBytes = 0;
    int padding = 0;
 
    if (client == NULL)
        return 0;
 
    if (numGCs < 0)
        return 0;
 
    printf("SENDING GET GC VALUES REQUEST\n");
 
    IceGetHeader(client->iceConn,
                 client->majorOpcode,
                 RapGetGCValuesRequest,
                 SIZEOF(RapMessageHeader),
                 RapMessageHeader,
                 header);
 
    /* compute the size of the message */
    totalSizeInBytes += 4;                              /* size of list */
    totalSizeInBytes += numGCs * sizeof(CARD32);
 
    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;
 
    /* update the length field */
    header->length += WORD64COUNT(totalSizeInBytes);
 
    WRITE_CARD32_LIST(client->iceConn, GCs, numGCs);
 
    /* pad to 64 boundary.  Is this necessary? */
    while (padding--) {
        char zero = '\0';
        IceWriteData(client->iceConn, 1, &zero);
    }
 
    printf("SENDING GET GC VALUES REQUEST: %ld gcs, byte size = %ld, WORD64 size = %d\n", 
		 numGCs, totalSizeInBytes, header->length);
 
    IceFlush(client->iceConn);

    return 1;
}
 

int
RAPGetValuesRequest(RapClient client, Widget widget, const char **resourceNames,
		    long numResources)
{
    RapMessageHeader *header;
    register int i;
    long totalSizeInBytes = 0;
    int padding;

    if (client == NULL)
	return 0;

    if (numResources <= 0)
	return 0;
    
    printf("SENDING GET VALUES REQUEST\n");

    IceGetHeader(client->iceConn, client->majorOpcode, RapGetValuesRequest,
		 SIZEOF(RapMessageHeader), RapMessageHeader, header);


    /* compute the size of the message */
    totalSizeInBytes += 4;				/* widget */
    totalSizeInBytes += 4;				/* num strings */
    for (i=0 ; i<numResources ; i++)
	totalSizeInBytes += STRING_BYTES(strlen(resourceNames[i]));

    padding = PAD64(totalSizeInBytes);
    totalSizeInBytes += padding;
    
    printf("GetValuesRequest: %d resources, total size (bytes) = %ld\n", numResources, totalSizeInBytes);

    /* update the length field */
    header->length += WORD64COUNT(totalSizeInBytes);

    /* widget */
    WRITE_CARD32(client->iceConn, widget);

    WRITE_STRING_LIST(client->iceConn, resourceNames, numResources);   
    
    printf("SENDING GET VALUES REQUEST: widget %lx, %d resources, byte size = %ld, WORD64 size = %d\n", widget, numResources, totalSizeInBytes, header->length);

    IceFlush(client->iceConn);

    return 1;
}

int
RAPSetValuesRequest(RapClient client, const char *name, const char *type,
		    const char *value, WidgetList widgets, long numWidgets)
{
    RapMessageHeader *header;

    if (client == NULL)
	return 0;

    if (numWidgets <= 0)
	return 0;
    
    printf("SENDING SET VALUES REQUEST\n");

    IceGetHeader(client->iceConn, client->majorOpcode, RapSetValuesRequest,
		 SIZEOF(RapMessageHeader), RapMessageHeader, header);

#if 0
    NEED SIZE COMPUTATION.
	
    WRITE_STRING(client->iceConn, name);
    WRITE_STRING(client->iceConn, type);
    WRITE_STRING(client->iceConn, value);

    WRITE_CARD32_LIST(client->iceConn, widgets, numWidgets);
#endif

    IceFlush(client->iceConn);

    return 1;
}

int
RAPAddNotifyRequest(RapClient client, short *notifies, short numNotifies)
{
    RapMessageHeader *header;
    int		      msg_size, data_size, padding;	
    int 	     i;


    if (client == NULL)
	return 0;

    if (numNotifies <= 0)
	return 0;

    IceGetHeader(client->iceConn, client->majorOpcode, RapAddNotifyRequest,
		 SIZEOF(RapMessageHeader), RapMessageHeader, header);

    data_size = sizeof(short) + sizeof(short) * numNotifies;	
    padding = PAD64(data_size);
    msg_size = PADDED_BYTES64(data_size);
    header->length += WORD64COUNT(msg_size);
	
    IceWriteData16(client->iceConn, sizeof(short), &numNotifies);
    IceWriteData16(client->iceConn, sizeof(short) * numNotifies, notifies);

    /* now write out the padding 
	skip padding for now, seems to work without it ...
    for (i=0 ; i<padding ; i++) {
        char zero = '\0';
        IceWriteData(client->iceConn, 1, &zero);
    }
 
  */ 

    IceFlush(client->iceConn);

    return 1;
}

int
RAPRemoveNotifyRequest(RapClient client, short *notifies, short numNotifies)
{
    RapMessageHeader *header;

    if (client == NULL)
	return 0;

    if (numNotifies <= 0)
	return 0;

    IceGetHeader(client->iceConn, client->majorOpcode, RapRemoveNotifyRequest,
		 SIZEOF(RapMessageHeader), RapMessageHeader, header);
    
#if 0
    NEED SIZE COMPUTATION.

    IceWriteData16(client->iceConn, sizeof(short), &numNotifies);
    IceWriteData16(client->iceConn, sizeof(short) * numNotifies, notifies);
#endif
    
    IceFlush(client->iceConn);

    return 1;
}

int
RAPObjectToWindowRequest(RapClient client)
{
    return 0;
}

int
RAPWindowToObjectRequest(RapClient client)
{
    return 0;
}

int
RAPLocateObjectRequest(RapClient client)
{
    return 0;
}

int
RAPGetActionsRequest(RapClient client)
{
    return 0;
}

int
RAPDoActionsRequest(RapClient client)
{
    return 0;
}

int
RAPSelectEventRequest(RapClient client, short *events, short numEvents)
{
    RapMessageHeader *header;
    int msg_size, data_size, padding;
    int i;

    if (client == NULL)
	return 0;

    if (numEvents <= 0)
	return 0;

    IceGetHeader(client->iceConn, client->majorOpcode, RapSelectEventRequest,
		 SIZEOF(RapMessageHeader), RapMessageHeader, header);

    data_size = sizeof(short) + sizeof(short) * numEvents;
    padding = PAD64(data_size);
    msg_size = PADDED_BYTES64(data_size);
    header->length += WORD64COUNT(msg_size);


    IceWriteData16(client->iceConn, sizeof(short), &numEvents);
    IceWriteData16(client->iceConn, sizeof(short) * numEvents, events);

   /* now write out the padding 
      skip padding for now, seems to work fine without it
    for (i=0 ; i<padding ; i++) {
        char zero = '\0';
        IceWriteData(client->iceConn, 1, &zero);
    }
  */

    IceFlush(client->iceConn);

    return 1;
}

int
RAPSelectRequestRequest(RapClient client, short *requests, short numRequests)
{
    RapMessageHeader *header;
    int msg_size, data_size, padding;
    int i;

    if (client == NULL)
	return 0;

    if (numRequests <= 0)
	return 0;

    IceGetHeader(client->iceConn, client->majorOpcode, RapSelectRequestRequest,
		 SIZEOF(RapMessageHeader), RapMessageHeader, header);

    data_size = sizeof(short) + sizeof(short) * numRequests;
    padding = PAD64(data_size);
    msg_size = PADDED_BYTES64(data_size);
    header->length += WORD64COUNT(msg_size);


    IceWriteData16(client->iceConn, sizeof(short), &numRequests);
    IceWriteData16(client->iceConn, sizeof(short) * numRequests, requests);
#if 0
    /* now write out the padding */
    for (i=0 ; i<padding ; i++) {
         char zero = '\0';
         IceWriteData(client->iceConn, 1, &zero);
     }
#endif
    printf("FLUSHING SELECT REQUEST REQUEST\n");
    IceFlush(client->iceConn);

    return 1;
}

int
RAPCloseConnectionRequest(RapClient client)
{
    IceSimpleMessage(client->iceConn, client->majorOpcode,
		     RapCloseConnectionRequest);
    IceFlush(client->iceConn);

    return 1;
}
