##
##	Note:  The use of this program shall be restricted to educational,
##	non-commercial use only.  All inquiries regarding licensing this
##	software for commercial use may be addressed to:
##	
##	Georgia Tech Research Corporation
##	Technology Licensing
##	Centennial Research Building - Rm. 275
##	400 Tenth Street, N.W.
##	Atlanta, Georgia  30332-0415
##	
##	*************************************************
##	
##	       Copyright 1996
##	Georgia Tech Research Corporation
##	    All Rights Reserved
##
##
##
##
##
##
## Audio.tcl contains support for audio output.

Error EL_DEBUG audio "Sourcing audio.tcl"

set useAudio 1

set defaultSoundDir /users/b/beth/Mercator/Private/Sounds
set defaultSound $defaultSoundDir/magic.au

#########################################################################
# 
# AudioOn and AudioOff toggle sound output in Mercator.  Basically they
# just set a global variable that the rest of the routines here look at.
#
proc AudioOn {} {
    global useAudio

    set useAudio 1
}

proc AudioOff {} {
    global useAudio

    set useAudio 0
}

#########################################################################
# 
# Audio is the basic interface to playing audio in Mercator.
#
proc Audio {node delay} {
    global useAudio 
    global defaultSound
    global userLevel
    # really need a static variable for count
    global count

    if {$useAudio == 0} { return }

    # If the class we're at now has no parent, play the default sound.

    if {[modelManager parent $node] == "NULL"} {
	playsound $defaultSound
	return
    }
    
    # Set the parameters based on the templates

    set rate [expr [wtemplate $node rate] / 100.0]
    set muffle [wtemplate $node muffle]
    set volume [wtemplate $node volume]

    if { [wtemplate $node looped] == "TRUE" } {
	set looped 1
    } else {
	set looped 0
    }

    if {[resource $node get set] == "TRUE" } {
	set volume [expr $volume + 40]
	set muffle [expr $muffle - 35]
    }

    set sensitive [wtemplate $node sensitive]
    Error EL_INFORMATIONAL audio "PLAYING SOUND $rate $volume " \
	"$muffle $looped $delay"

    if {$sensitive == "TRUE"} {      
	Error EL_INFORMATIONAL audio "playsound [wtemplate $node sound] " \
	    "$rate $volume $muffle $looped $delay"
	playsound [wtemplate $node sound] $rate $volume $muffle $looped $delay

    } else {
	Error EL_DEBUG audio "insensitive object"
	Error EL_INFORMATIONAL audio "playsound " \
	    "[wtemplate $node sound] $rate 60 75 $looped $delay"
	#playsound [wtemplate $node sound] $rate 50 80 $looped $delay
	set volume [expr $volume - 25]
	playsound [wtemplate $node sound] $rate $volume 80 $looped $delay
    }
}

#
# GetPosition is a helper function that returns the position of a node
# among its siblings.  This value is used for muffling, and is invoked
# through the template code.
#
proc GetPosition node {
    set children [modelManager children [modelManager parent $node]]
    set count 1
    foreach child $children {
	if {$child == $node} {
	    puts "GetPosition count is $count"
	    return $count
	} else {
	    set count [expr $count + 1]
	}
    }
    return 0
}

#
# OutofBounds is invoked whenever the user attempts to move into
# never never land
#
proc OutofBounds args {
    global useAudio
    global defaultSoundDir

    if {$useAudio == 0} { return }

    playsound "$defaultSoundDir/bounce.au"
}

#
# Generate sound when a popup dialog appears.  Invoked from various actions
# defined in Mercator.tcl.
#
proc PopUpAudio args {
    global useAudio 
    global defaultSoundDir

    if {$useAudio == 0} { return }

    playsound "$defaultSoundDir/whistle-up.au" 1.0 100 0 0 500
}

#
# Played whenever a new client starts.
#
proc ApplicationStartUpAudio args {
    global useAudio 
    global defaultSoundDir

    if {$useAudio == 0} { return }

    playsound "$defaultSoundDir/winding.au" 1.0 100 0 0 500
    playsound "$defaultSoundDir/winding.au" 1.0 100 0 0 500
    playsound "$defaultSoundDir/winding.au" 1.0 100 0 0 500
}

#
# Played whenever a client goes away.
#
proc ApplicationShutdownAudio args {
    global useAudio 
    global defaultSoundDir

    if {$useAudio == 0} { return }

    playsound "$defaultSoundDir/ice.au" 1.0 100 0 0 500
}


#
# Played whenever a popup dialog goes away.
#
proc PopDownAudio args {
    global useAudio 
    global defaultSoundDir

    if {$useAudio == 0} { return }

    playsound "$defaultSoundDir/whistle-down.au" 1.0 100 0 0 500
}

#
# Change text mode audio.
#
proc TextEnterOrExitAudio args {
    global useAudio 
    global defaultSoundDir

    if {$useAudio == 0} { return }

    playsound "$defaultSoundDir/wood-drawer.au" 1.0 80 0 0 200
}

#
# Sound to accompany selection.
#
proc SelectAudio node {
    global useAudio 
    global defaultSoundDir

    if {$useAudio == 0} { return }

    if { $node == [long2short {xmailtool outerbox header_box}] } {
       playsound "$defaultSoundDir/camera.au" 1.00 50 0 0 0
    } else {
       playsound "$defaultSoundDir/rip.au" 1.0 50 0 0 0
    }
}

#
# Change applications.
#
proc SwitchAppAudio args {
    global useAudio 
    global defaultSoundDir

    if {$useAudio == 0} { return }

    playsound "$defaultSoundDir/flip-paper.au" 1.0 75 0 0 0
}

#
# TextAudio is called as we move through a text widget.  It changes
# the pitch of a "click" sound relative to vertical position.
#
proc TextAudio {total line} {
    global useAudio 
    global defaultSoundDir

    if {$useAudio == 0} { return }

    set line [expr abs ($line * 1.0)]
    Error EL_DEBUG audio "line number = $line"
    Error EL_DEBUG audio "TextAudio:: total number = $total"
    set muffle [expr round((($line / $total) - .5) * 150)] 
    Error EL_DEBUG audio "muffle = $muffle"
    playsound "$defaultSoundDir/ca-chick2.au" 1.0 100 $muffle 0 0
}

#
# Set volume based on whether an object is a radio button or check box.
#
proc WhatVolumeProc {type node} {
    set parent [modelManager parent $node]

    if {[resource $parent get radioBehavior] == "TRUE"} {
	# node is a radio button
        return 60
    } else {
	# node is a check box   
        return 75
    }
}

#
# Set sound based on whether object is a radio button or a check box.
#
proc WhichSoundProc {type node} {
    global defaultSoundDir

    set parent [modelManager parent $node]

    if {[resource $parent get radioBehavior] == "TRUE"} {
	# node is a radio button
        return "$defaultSoundDir/light-switch.au"
    } else {
	# node is a check box   
        return "$defaultSoundDir/pop.au"
    }
}


#addaction Audio
#addaction OutofBounds
#addaction SelectAudio
#addaction PopUpAudio
#addaction PopDownAudio
#addaction ApplicationStartUpAudio
#addaction TextEnterOrExitAudio
#addaction SwitchAppAudio
#addaction TextAudio

