
%{

/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/

void ProcessBooleanDecl(int l, const char *id, unsigned char b);
void ProcessIntegerDecl(int l, const char *id, long i);
void ProcessStringDecl(int l, const char *id, const char *str);
void ProcessIdentifierDecl(int l, const char *id, const char *str);
void ProcessStringListDecl(int i, const char *id);

void ConfigAddToStringList(int l, const char *s);
void ConfigLexWarning(int l, const char *s);

%}

integer		[0-9]+
whitespace	[ \t]+
string		\"[^\"\n]*[\"\n]
identifier	[a-zA-Z][a-zA-Z0-9_\-\.]*
newline		\n
comment		#.*
true		TRUE
false		FALSE

%%

{whitespace}	;
{newline}	{
			configLineno++;
		}
{comment}	;
{integer}	{
			sscanf(yytext, "%d", &yylval.integer);
			return INTEGER;
		}
{true}		{
			yylval.bool = 1;
			return BOOLEAN;
		}
{false}		{
			yylval.bool = 0;
			return BOOLEAN;
		}
{string}	{
			register int i;
			yylval.string = strdup(yytext);
			if (yylval.string[strlen(yylval.string) - 1] != '"') {
			    ConfigLexWarning(configLineno,
				"unterminated string");
			}
			for (i=0 ; i<(int)strlen(yylval.string) ; i++) {
			    yylval.string[i] = yylval.string[i+1];
			}
			yylval.string[strlen(yylval.string) - 1] = '\0';
			return STRING;
		}
{identifier}	{
			yylval.identifier = strdup(yytext);
			return IDENTIFIER;
		}
.		{	return yytext[0];	}
