
%{

/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/
    
extern int configLineno;

%}

%union {
    long	integer;	/* integer return */
    char	*string;	/* string return */
    char	*identifier;	/* identifier return */
    unsigned char bool;		/* boolean return */
}

%token <integer>	INTEGER
%token <string>		STRING
%token <identifier>	IDENTIFIER
%token <bool>		BOOLEAN

%type <string>		stringlist

%%

start:		lines
		;

lines:		/* empty */
		| lines line
		;

line:		  IDENTIFIER '=' BOOLEAN
		  { ProcessBooleanDecl(configLineno, $1, $3); free($1); }
		| IDENTIFIER '=' INTEGER
		  { ProcessIntegerDecl(configLineno, $1, $3); free($1); }
		| IDENTIFIER '=' STRING
		  { ProcessStringDecl(configLineno, $1, $3); free($1); free($3); }
		| IDENTIFIER '=' IDENTIFIER
		  { ProcessIdentifierDecl(configLineno, $1, $3); free($1); free($3); }
		| IDENTIFIER '=' '(' stringlist ')'
		  { ProcessStringListDecl(configLineno, $1); free($1); }
		| IDENTIFIER '=' '(' ')'
		  { ProcessStringListDecl(configLineno, $1); free($1); }
		| IDENTIFIER '=' '[' IDENTIFIER ']'
		  { ProcessProcDecl(configLineno, $1, $4); free($1); free($4); }
		;

stringlist:       stringlist "," STRING
		  { ConfigAddToStringList(configLineno, $3); free($3); }
                | STRING
		  { ConfigAddToStringList(configLineno, $1); free($1); }
		;

%%

#include "configlexer.c"

int
yywrap()
{
    return 1;
}

static int
yyerror(char *str)
{
    char tmp[16];
    sprintf(tmp, "%d", configLineno);
    fprintf(stderr, "yyerror: line %s: %s\n", tmp, str);
    return 1;
}
