//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <stropts.h>
#include <string.h>
#include <poll.h>
#include <signal.h>


void
cleanup()
{
    unlink("/tmp/to-console");
    unlink("/tmp/from-console");
}

#ifdef SVR5
void brokenPipeHandler(int dummy)
#else
void brokenPipeHandler(void)
#endif /* SVR5 */
{
    fprintf(stderr, "console is not connected to Sonic X\n");
    cleanup();
}

main(int argc, char *argv[])
{
    struct pollfd fds[2];
    int numFds = 0, infd, outfd;

    if ((outfd = open("/tmp/from-console", O_WRONLY)) < 0) {
	if (errno != ENXIO) {
	    perror("console: open");
	    exit(1);
	}
    }
    printf("console: from-console opened\n");


    if ((infd = open("/tmp/to-console", O_RDONLY|O_NDELAY)) < 0) {
	perror("console: open");
	exit(1);
    }
    printf("console: to-console opened on fd = %d\n", infd);

    
    struct sigaction action;

    action.sa_handler = brokenPipeHandler;
    sigaction(SIGPIPE, &action, NULL); /* catch SIGPIPE */

    FILE *inFile, *outFile;
    
    if ((inFile = fdopen(infd, "r")) == NULL) {
	perror("fdopen");
	exit(1);
    }

    if ((outFile = fdopen(outfd, "w")) == NULL) {
	perror("fdopen");
	exit(1);
    }

    printf("console: pinging mercator\n");
    if(fprintf(outFile, "hello") < 0) {
	perror("fprintf");
	exit(1);
    }

    sleep(2);
    
    if(fflush(outFile) != 0) {
	perror("fflush");
	exit(1);
    }
    
    printf("console: flushed outFile\n");
    

    fds[0].fd = 0;
    fds[0].events = POLLIN;
    numFds++;

    fds[1].fd = infd;
    fds[1].events = POLLIN;
    numFds++;

    while (1) {
        int result = poll(fds, numFds, INFTIM);
	register int i;

        if (result < 0) {
	    if (errno == EINTR)
		continue;
	    perror("poll");
	    exit(1);
	}

	if (fds[0].revents & POLLIN) {
		/* send to from-console */
		char buf[2048];

		if (fgets(buf, 2048, stdin) == NULL) {
		    perror("fgets stdin");
		}

		if (fwrite(buf, sizeof(char), strlen(buf) + 1, outFile) == 0) {
		    perror("fwrite outfile");
		}
		fflush(outFile);
	}
	if (fds[1].revents & POLLIN) {
		/* send to stdout */
		char buf[2048];

		if (fgets(buf, 2048, inFile) == NULL) {
		    perror("fgets inFile");
		}

		if (fwrite(buf, sizeof(char), strlen(buf) + 1, stdout) == 0) {
		    perror("fwrite stdout");
		}
		fflush(stdout);
	}
    } 
}

    
