
%{

/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/

#include <stdlib.h>
#include <string.h>

%}

letter		[a-zA-Z]
digit		[0-9]

quote           \"
esc_quote       \\\"
regexp          {quote}([^\"\n]|{esc_quote})*{quote}
expr		{letter}({letter}|{digit})*

number		{digit}+

language	[lL][aA][nN][gG][uU][aA][gG][eE]
pitch		[pP][iI][tT][cC][hH]
rate		[rR][aA][tT][eE]
voice		[vV][oO][iI][cC][eE]
volume		[vV][oO][lL][uU][mM][eE]

filter		[fF][iI][lL][tT][eE][rR]

font		[fF][oO][nN][tT]
highlight	[hH][iI][gG][hH][lL][iI][gG][hH][tT]

whitespace	[ \t]+
identifier	[a-zA-Z][a-zA-Z0-9_\-\.]*
newline		\n
comment		#.*
true		TRUE
false		FALSE

%%

{whitespace}	;       /* Ignore whitespace */
{comment}	;	/* Ignore comments   */
{newline}	{	FilterDict::filterLineCount++;	}

{filter}	{	return FILTER;			}
{font}		{	
			yylval.string = Strdup("FONT");
			return FONT;			
		}
{highlight}	{	
			yylval.string = Strdup("HIGHLIGHT");
			return HIGHLIGHT;
		}
{language}	{	
			yylval.string = Strdup("LANGUAGE");
			return LANGUAGE;
		}
{pitch}		{	
			yylval.string = Strdup("PITCH");
			return PITCH;
		}
{rate}		{	
			yylval.string = Strdup("RATE");
			return RATE;	
		}
{voice}		{	
			yylval.string = Strdup("VOICE");
			return VOICE;
		}
{volume}	{	
			yylval.string = Strdup("VOLUME");
			return VOLUME;
		}

{identifier}	{
			yylval.string = Strdup(yytext);
			return IDENTIFIER;
		}
{number}	{	
			sscanf(yytext, "%d", &yyval.integer);
			return INTEGER;
		}
{regexp}        {
			/* remove quotes */
			yytext[strlen(yytext)-1] = '\0';
                        yylval.string = Strdup(yytext+1);
			return REGEXP;
                }
.		{	return yytext[0];		}

%%
