
%{
/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/

#include <string.h>

#include "Error.h"
#include "ScreenReader.h"

%}

%union {
    long	integer;	/* integer return */
    char	*string;	/* string return */
    unsigned char bool;		/* boolean return */
}
	
%token <integer> 	INTEGER
%token <string>		IDENTIFIER REGEXP 
%token <string>		FILTER LANGUAGE PITCH RATE VOICE VOLUME FONT HIGHLIGHT
%token <bool>		BOOLEAN

%type <string> operation action

%start file

%%

file:	| file filter
		;

filter:	filterhdr filterbody
	;

filterhdr: FILTER IDENTIFIER
	{
		Error(EL_DEBUG, "FilterLoad", "Registering new filter \"",
			$2, "\".", NULL);
		FilterDict::NewFilterRegister($2);
	};

filterbody: '{' rules '}'
	{
		FilterDict::NewFilterStore();
	};

rules:	| rules rule 
		{	}
	| rules option
		{	}
	;

option:	IDENTIFIER '=' IDENTIFIER
	;

rule:	REGEXP action
	{	
		FilterDict::NewFilterAdd($1, FILTER_ACTION_REGEXP, $2);
		delete [] $1;
		delete [] $2;
	}
	| REGEXP REGEXP
	{
		FilterDict::NewFilterAdd($1, FILTER_ACTION_REGEXP, $2);
		delete [] $1;
		delete [] $2;
	}
	| FONT action
	{	
		FilterDict::NewFilterAdd($1, FILTER_ACTION_FONT, $2);
		delete [] $1;
		delete [] $2;
	}
	| FONT REGEXP
	{
		FilterDict::NewFilterAdd($1, FILTER_ACTION_FONT, $2);
		delete [] $1;
		delete [] $2;
	}
	| HIGHLIGHT action
	{	
		FilterDict::NewFilterAdd($1, FILTER_ACTION_HIGHLIGHT, $2);
		delete [] $1;
		delete [] $2;
	}
	| HIGHLIGHT REGEXP
	{
		FilterDict::NewFilterAdd($1, FILTER_ACTION_HIGHLIGHT, $2);
		delete [] $1;
		delete [] $2;
	}

	
action:	operation IDENTIFIER
	{	
		/* parse $1 for action */
		/* parse $2 for action parameters */
		/* query device for appropriate sequences */
		char buffer[512];
		buffer[0] = '\0';
		strcpy(buffer, $1);
		strcat(buffer, " ");
		strcat(buffer, $2);
		delete [] $1;
		delete [] $2;
		$$ = Strdup(buffer);
	};

operation: LANGUAGE | PITCH | RATE | VOICE | VOLUME
	{
		$$ = $1;
	};
%%

#include "filterlexer.cc"

/*
 * Replacements for yywrap and yyerror.  These keep us from having to link
 * with libraries we really don't need.
 */

int
yywrap()
{
    return 1;
}

static int
yyerror(char *str)
{
	char buffer[16];
	sprintf(buffer, "%d", FilterDict::filterLineCount);
	if (str) {
		if (FilterDict::current_filter)
			Error(EL_ERROR, "Error parsing filter ",
				FilterDict::current_filter->GetName(),
				" at line ", buffer, ".  Aborting.\n", NULL);
		else
			Error(EL_ERROR, 
				"Illegal filter syntax at line ", buffer,
				".  Aborting.\n", NULL);
		FilterDict::NewFilterAbort();
	}
	else
		printf("error: null error string in yyerror.\n");
	return 1;
}

