//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#include "mDictPtrPtr.h"
#include <rw/tphdict.h>

// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/mDictPtrPtr.cc,v $
// $Revision: 1.5 $
// $Date: 1995/07/28 21:19:35 $

//
// $Log: mDictPtrPtr.cc,v $
// Revision 1.5  1995/07/28  21:19:35  spine
// Added copyright notice.
//
// Revision 1.4  1995/07/25  22:05:34  keith
// Cleaned up some usage of strings, dicts, and lists.  Ran with -pta to
// fix some previously unseen bugs in the parameterized classes.
//
// Revision 1.3  1995/07/24  17:44:26  beth
// Ian's Version
//
// Revision 1.1  1995/06/05  22:24:31  iansmith
// Initial revision
//
//

//
// mDictPtrPtr Implementation
//

// constructor... build the implementation and make it empty
template <class K, class V>
mDictPtrPtr<K,V>::mDictPtrPtr(unsigned (*hashKey)(const K&)) :
    impl(new RWTPtrHashDictionary<K,V>(hashKey))
{
}

// copy constructor 
template <class K, class V>
mDictPtrPtr<K, V>::mDictPtrPtr(const mDictPtrPtr<K,V>& x) :
    impl(new RWTPtrHashDictionary<K, V>(*x.impl))
{
}

// destructor
template <class K, class V>
mDictPtrPtr<K,V>::~mDictPtrPtr()
{
    delete impl;
}

// operator =
template <class K, class V> mDictPtrPtr<K,V>&
mDictPtrPtr<K, V>::operator=(const mDictPtrPtr<K, V>& x)
{
    *impl = *x.impl;
    return *this;
}

// insert
template <class K, class V>
int mDictPtrPtr<K, V>::insert(K *k, V *v)
{
    V *result = impl->findValue(k);
    impl->insertKeyAndValue(k, v);
    
    if (result)
	return 1;
    else
	return 0;
}

// lookup
template <class K, class V> V *
mDictPtrPtr<K, V>::lookup(K* k) const
{
    return impl->findValue(k);
}

// del... but no destructor calls
template <class K, class V> void
mDictPtrPtr<K,V>::del(K *k)
{
    impl->remove(k);
}

// del_item ... includes destructor calls on K and V
template <class K, class V> void
mDictPtrPtr<K, V>::del_item(K* k)
{
    V *v = impl->findValue(k);
    
    if (v) {
	impl->remove(k);
	delete k;
	delete v;
    } 
    // if k isn't in the dictionary, don't bother trying to remove it
}

template <class K, class V> void 
mDictPtrPtr<K,V>::walk(mDictPtrPtrClosure<K, V>& closure,
		       void *clientData) const
{
    RWTPtrHashDictionaryIterator<K, V> iter(*impl);
    K *current;
    
    // startup info
    closure.startup(clientData);
    
    // iterate until current is nil
    while ((current = iter()) != nil) {
	// if they return 0, bail out
	if (closure.action(current, impl->findValue(current)) == 0)
	    break;
    }
    // give them the hook for ending
    closure.closedown();
}

/////////////////////////////////////////////////////////////////////////////
// iterator crap
// constructor
template <class K, class V> 
mDictPtrPtrIterator<K,V>::mDictPtrPtrIterator(const mDictPtrPtr<K, V>& x) :
    impl(new RWTPtrHashDictionaryIterator<K,V>(*(x.impl)))
{
}

// destructor
template <class K, class V>
mDictPtrPtrIterator<K,V>::~mDictPtrPtrIterator()
{
    delete impl;
}

// get the next ptr
template <class K, class V> K *
mDictPtrPtrIterator<K, V>::next()
{
    return ((*impl)());
}

// get the current size
template <class K, class V> unsigned
mDictPtrPtr<K, V>::size() const 
{
    return impl->entries();
}

