// This may look like C code, but it is really -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#ifndef MERCATOR_DICT_PTR_PTR_H
#define MERCATOR_DICT_PTR_PTR_H

// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/mDictPtrPtr.h,v $
// $Revision: 1.7 $
// $Date: 1995/07/28 21:21:05 $
//
// $Log: mDictPtrPtr.h,v $
//Revision 1.7  1995/07/28  21:21:05  spine
//Added copyright notice.
//
//Revision 1.6  1995/07/25  22:05:34  keith
//Cleaned up some usage of strings, dicts, and lists.  Ran with -pta to
//fix some previously unseen bugs in the parameterized classes.
//
//Revision 1.5  1995/07/24  17:44:26  beth
//Ian's Version
//
//Revision 1.3  1995/06/05  23:22:58  suelong
//*** empty log message ***
//
//Revision 1.2  1995/06/05  23:11:24  suelong
//commented out syntax error on line 11
//
// * Revision 1.1  1995/06/05  22:24:46  iansmith
// * Initial revision
//  *
//

// We implement this with a rogue wave dictionary right now
/* #include <rw/tphdict.h> */
#ifdef BEAM
template <class K, class V> class RWTPtrHashDictionary;
template <class K, class V> class RWTPtrHashDictionaryIterator;
#else
template <class K, class V> class RWTPtrHashDictionary<K, V>;
template <class K, class V> class RWTPtrHashDictionaryIterator<K, V>;
#endif

// 
// This is the closure object. You should subclass this to get walking
// behavior that you want. If your action function returns 0 the
// walk will terminate. If you need startup or closedown help,
// you can implement either of those two functions.
//
template <class K, class V> class mDictPtrPtrClosure {
public:
    // you can use this to setup your closure... 
    // it will get called with the client data passed to walk
    virtual void startup(void *clientData) {};
    // you will still get a closedown if you short circuit
    // out of the action procedure
    virtual void closedown();
    // remember to return 1 if you want the walk to continue
    virtual int action(K *, V*);
};

template <class K, class V> inline void mDictPtrPtrClosure<K,V>::closedown()
		{ }
template <class K, class V> inline int mDictPtrPtrClosure<K,V>::action(K *, V *)
		{ return 0; }

///////////////////////////////////////////////////////////////////////////
//
// This is a dictionary of pointers to pointers. You cannot put
// values (real objects) in this dictionary.  These use pointer
// copy semantics (shallow copy). You must implement the
// K must have following methods to use this class:
// operator==
// operator<
// There must also be a hashing function for K with the following
// sigature
// unsigned hFun(const K& );
// 
// You are not allowd to have duplicates of keys in this dict.
// You can have duplicate values.
//
template <class K, class V> class mDictPtrPtr {

    friend class mDictPtrPtrIterator<K,V>;

private:
    // The implementation
    RWTPtrHashDictionary<K, V> *impl;

    // Disallow default constructur
    mDictPtrPtr();
    
public:
    // constructor
    mDictPtrPtr(unsigned (*hashKey)(const K&));
    // copy constructor
    mDictPtrPtr(const mDictPtrPtr& other);
    // destructor
    virtual ~mDictPtrPtr();
    // operator=
    mDictPtrPtr<K, V>& 		operator=(const mDictPtrPtr<K, V>& other);
    //insert... returns 1 if the key was already present, 0 otherwise
    int 			insert(K*, V *);
    //lookup... returns 0 if the value is not present in the dictionary
    // equality of keys is with the operator== on K
    V 			       *lookup(K *) const;
    // del  ... delete's an object from the dictionary (but doesn't call
    // EITHER of the destructors... this is a NOOP if the item is 
    // not present
    void 			del(K *);
    // del_item ... same as del but calls destructors on both
    void 			del_item(K *);
    // walk ... pass a closure and client data
    void 			walk(mDictPtrPtrClosure<K, V>& closure,
				     void *clientData) const;
    // size
    unsigned 			size() const;
};

////////////////////////////////////////////////////////////////////////////

template <class K, class V> class mDictPtrPtrIterator {
private:
    // implemented with rogue wave
    RWTPtrHashDictionaryIterator<K, V> *impl;
    
public:
    // constructor
    mDictPtrPtrIterator(const mDictPtrPtr<K, V>&x);
    // destructor
    virtual ~mDictPtrPtrIterator();
    // get next ptr... returns 0 when done
    K 				*next();
};
#ifdef BEAM
#include "mDictPtrPtr.cc"
#endif

#endif /* MERCATOR_DICT_PTR_PTR_H */
