// This may look like C code, but it is really -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#ifndef MERCATOR_DICT_VAL_PTR_H
#define MERCATOR_DICT_VAL_PTR_H

// $Author: selbie $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/mDictValPtr.h,v $
// $Revision: 1.11 $
// $Date: 1995/08/04 20:02:58 $
//
// $Log: mDictValPtr.h,v $
//Revision 1.11  1995/08/04  20:02:58  selbie
//*** empty log message ***
//
//Revision 1.10  1995/08/03  22:33:14  selbie
//added declaration for "kill_all()"
//
//Revision 1.9  1995/07/28  21:21:41  spine
//Added copyright notice.
//
//Revision 1.8  1995/07/25  22:05:34  keith
//Cleaned up some usage of strings, dicts, and lists.  Ran with -pta to
//fix some previously unseen bugs in the parameterized classes.
//
//Revision 1.7  1995/07/25  20:18:53  keith
//Fixed return types on inlined expression.
//
//Revision 1.6  1995/07/25  20:15:08  keith
//Cleaned up the code a little, fixed a memory leak in iter::current().
//
//Revision 1.5  1995/07/25  17:24:11  keith
//Added def of "nil", fixed but in del_item().
//
//Revision 1.4  1995/07/24  17:44:26  beth
//Ian's Version
//
//Revision 1.2  1995/06/05  23:11:24  suelong
//commented out syntax error on line 11
//
// * Revision 1.1  1995/06/05  22:25:11  iansmith
// * Initial revision
//  *
//

// We implement this with a rogue wave dictionary right now
/* #include <rw/tphdict.h> */
#ifdef BEAM
template <class K, class V> class RWTPtrHashDictionary;
template <class K, class V> class RWTPtrHashDictionaryIterator;
#else
template <class K, class V> class RWTPtrHashDictionary<K, V>;
template <class K, class V> class RWTPtrHashDictionaryIterator<K, V>;
#endif

#ifndef nil
#define nil rwnil
#endif /* nil */

// 
// This is the closure object. You should subclass this to get walking
// behavior that you want. If your action function returns 0 the
// walk will terminate. If you need startup or closedown help,
// you can implement either of those two functions.
//
template <class K, class V> class mDictValPtrClosure {
public:
    //
    // you can use this to setup your closure... 
    // it will get called with the client data passed to walk
    //
    virtual void 	startup(void *clientData) {};
    //
    // you will still get a closedown if you short circuit
    // out of the action procedure
    //
    virtual void 	closedown();
    //
    // remember to return 1 if you want the walk to continue
    //
    virtual int 	action(K, V*);
};

template <class K, class V> inline void mDictValPtrClosure<K,V>::closedown()
		{ }
template <class K, class V> inline int mDictValPtrClosure<K,V>::action(K, V *)
		{ return 0; }

//
// This is a dictionary of pointers to pointers. You cannot put
// values (real objects) in this dictionary.  These use pointer
// copy semantics (shallow copy).
//
// K must have following methods to use this class:
//     operator==
//     operator<
//     copy constructor
//     operator=
//
// There must also be a hashing function for K with the following
// sigature:
//     unsigned hFun(const K& );
// 
// You are not allowd to have duplicates of keys in this dict.
// You can have duplicate values.
//
template <class K, class V> class mDictValPtr {

    friend class mDictValPtrIterator<K, V>;

private:
    // The implementation
    RWTPtrHashDictionary<K, V> *impl;

    // Disallow default constructor.
    mDictValPtr();
    
public:
    // constructor
    mDictValPtr(unsigned (*hashKey)(const K&));
    // copy constructor
    mDictValPtr(const mDictValPtr&);
    // destructor
    virtual ~mDictValPtr();
    // operator=
    mDictValPtr<K, V>& 	operator=(const mDictValPtr<K,V> &);
    // insert... returns 1 if the key was already present, 0 otherwise
    int 		insert(const K&, V *);
    // lookup... returns 0 if the value is not present in the dictionary.
    // equality of keys is with the operator== on K
    V 		       *lookup(const K&) const;
    // del  ... deletes an object from the dictionary 
    // since we have copies of K we delete our copy but
    // we don't touch the V *.
    void 		del(const K&);
    // del_item ... same as del but calls destructor on V *
    void 		del_item(const K&);
    // walk ... pass a closure and client data
    void 		walk(mDictValPtrClosure<K, V>& closure,
			     void *clientData) const;
    // size
    unsigned 		size() const;

    // special method to call delete on all entries
    void kill_all();
};

///////////////////////////////////////////////////////////////////////////
//
// Ok, now we need an iterator.
//
// This iterator is NOT valid (undefined) until you call the next
// method at least once.
//
template <class K, class V> class mDictValPtrIterator {
private:
    RWTPtrHashDictionaryIterator<K, V> *impl;

    // Disallow default constructor.
    mDictValPtrIterator();
    
public:
    // constructor
    mDictValPtrIterator(const mDictValPtr<K, V>& dict);
    // destructor
    virtual ~mDictValPtrIterator();
    // operator =
    mDictValPtrIterator(const mDictValPtrIterator<K, V>& other);
    // get next... returns 0 if no more, 1 if there is more
    int 		next();
    // get current - returns a copy to ensure safety!
    K 			current() const;
};

#ifdef BEAM
#include	"mDictValPtr.cc"
#include	"rw/regexp.h"
#endif

#endif /* MERCATOR_DICT_VAL_PTR_H */
