//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#include "mDictValVal.h"
#include <rw/tvhdict.h>

// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/mDictValVal.cc,v $
// $Revision: 1.5 $
// $Date: 1995/07/28 21:22:06 $

//
// $Log: mDictValVal.cc,v $
//Revision 1.5  1995/07/28  21:22:06  spine
//Added copyright notice.
//
//Revision 1.4  1995/07/25  22:05:34  keith
//Cleaned up some usage of strings, dicts, and lists.  Ran with -pta to
//fix some previously unseen bugs in the parameterized classes.
//
//Revision 1.3  1995/07/24  17:44:26  beth
//Ian's Version
//
//Revision 1.2  1995/06/10  15:46:16  iansmith
//*** empty log message ***
//
//Revision 1.2  1995/06/09  13:26:27  iansmith
//Changed the #include to include the right declaration!
//
//Removed some bogus old comments.
//
//Basically converted the ValPtr dictionary to a ValVal dictionary by
//changing each of the functions to be right.
//
//

//
// mDictValVal Implementation
//

// constructor... build the implementation and make it empty
template <class K, class V>
mDictValVal<K,V>::mDictValVal(unsigned (*hashKey)(const K&)) :
    impl(new RWTValHashDictionary<K,V>(hashKey))
{
}

// copy constructor 
template <class K, class V>
mDictValVal<K,V>::mDictValVal(const mDictValVal<K,V> &x) :
    impl(new RWTValHashDictionary<K,V>(*x.impl))
{
}

// destructor
template <class K, class V>
mDictValVal<K,V>::~mDictValVal()
{
    // blow away collection
    delete impl;
}

// operator =
template <class K, class V> mDictValVal<K,V>&
mDictValVal<K,V>::operator=(const mDictValVal<K,V> &x)
{
    *impl = *x.impl;
    return *this;
}

// insert
template <class K, class V> int
mDictValVal<K,V>::insert(const K &k, const V &v)
{
    int result;
    V bogus;

    // check to see if its already there
    result=impl->findValue(k,bogus)?1:0;
    // insert the new copy and the new copy of the  value
    impl->insertKeyAndValue(k,v);
    // return the corrent number
    return result;
}

// lookup
template <class K, class V> int
mDictValVal<K,V>::lookup(const K &k,V &v) const
{
    return (impl->findValue(k,v)?1:0);
}

// del..... get rid of 'em
template <class K, class V> void
mDictValVal<K,V>::del(const K &k)
{
    impl->remove(k);
    // if k isn't in the dictionary, this is a noop
}

template <class K, class V> void 
mDictValVal<K,V>::walk(mDictValValClosure<K, V>& closure, void *clientData) const
{
    RWTValHashDictionaryIterator<K,V> iter(*impl);
    // startup info
    closure.startup(clientData);
    // iterate until current is nil
    while (iter()) {
	 // if they return 0, bail out
	if (closure.action(iter.key(),iter.value())==0) break;
    }
    // give them the hook for ending
    closure.closedown();
}

//////////////////////////////////////////////////////////////////////////
// iterator
template <class K,class V> 
mDictValValIterator<K,V>::mDictValValIterator(const mDictValVal<K,V> &dict) :
    impl(new RWTValHashDictionaryIterator<K,V>(*(dict.impl)))
{
}

// destructor
template <class K,class V>
mDictValValIterator<K,V>::~mDictValValIterator()
{
    delete impl;
}

// move up to the next value
template <class K, class V> int
mDictValValIterator<K,V>::next()
{
    return (impl->operator()() ? 1 : 0);
}

// get the  current value
template <class K, class V> K
mDictValValIterator<K,V>::current() const
{
    
    return impl->key();
}

// get the current size
template <class K, class V>
unsigned mDictValVal<K,V>::size() const 
{
    return (impl->entries());
}

