// This may look like C code, but it is really -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#ifndef MERCATOR_DICT_VAL_VAL_H
#define MERCATOR_DICT_VAL_VAL_H

// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/mDictValVal.h,v $
// $Revision: 1.6 $
// $Date: 1995/07/28 21:22:45 $
//
// $Log: mDictValVal.h,v $
//Revision 1.6  1995/07/28  21:22:45  spine
//Added copyright notice.
//
//Revision 1.5  1995/07/25  22:05:34  keith
//Cleaned up some usage of strings, dicts, and lists.  Ran with -pta to
//fix some previously unseen bugs in the parameterized classes.
//
//Revision 1.4  1995/07/24  17:44:26  beth
//Ian's Version
//
//Revision 1.3  1995/06/10  15:46:23  iansmith
//*** empty log message ***
//
//Revision 1.3  1995/06/09  13:25:19  iansmith
//Changed the action() function of the closure to use references for
//both K and V.
//
//Changed the insert() function to use const references for V.
//
//Changed the lookup() function to have the right signature.
//
//Change the current() function of the iterator to return a K not a K&
//as this implies a return of a local.
//
// Revision 1.2  1995/06/07  20:39:26  iansmith
// This is a checkin to insure that RCS has a version for ValVal.
//

// We implement this with a rogue wave dictionary right now
/* #include <rw/tvhdict.h> */
#ifdef BEAM
template <class K, class V> class RWTValHashDictionary;
template <class K, class V> class RWTValHashDictionaryIterator;
#else
template <class K, class V> class RWTValHashDictionary<K, V>;
template <class K, class V> class RWTValHashDictionaryIterator<K, V>;
#endif

// 
// This is the closure object. You should subclass this to get walking
// behavior that you want. If your action function returns 0 the
// walk will terminate. If you need startup or closedown help,
// you can implement either of those two functions.
//
template <class K, class V> class mDictValValClosure {
public:
    // you can use this to setup your closure... 
    // it will get called with the client data passed to walk
    virtual void startup(void *clientData);
    // you will still get a closedown if you short circuit
    // out of the action procedure
    virtual void closedown();
    // remember to return 1 if you want the walk to continue
    virtual int action(K &, V &);
};

template <class K,class V> inline void mDictValValClosure<K, V>::startup(void*)
		{ }
template <class K, class V> inline void mDictValValClosure<K, V>::closedown()
		{ }
template <class K, class V> inline int mDictValValClosure<K, V>::action(K&, V&)
		{ return 0; }

//////////////////////////////////////////////////////////////////////////
//
// This is a dictionary of values to values. You must  put
// values (real objects) in this dictionary.  These use deep
// copy semantics (copy constructor).  K must implement 
// the  following methods to use this class:
// operator==
// operator<
// copy constructor
// operator=
//
// There must also be a hashing function for K with the following
// sigature
// unsigned hFun(const K& );
// 
// You are not allowd to have duplicates of keys in this dict.
// You can have duplicate values.
//
template <class K, class V> class mDictValVal {
    
    friend class mDictValValIterator<K,V>;

private:
    // The implementation
    RWTValHashDictionary<K,V> *impl;

    // Disallow default constructor
    mDictValVal();

public:
    // constructor
    mDictValVal(unsigned (*hashKey)(const K &));
    // copy constructor
    mDictValVal(const mDictValVal& );
    // destructor
    virtual ~mDictValVal();
    // operator=
    mDictValVal<K,V>& 		operator=(const mDictValVal<K,V>&);
    //insert... returns 1 if the key was already present, 0 otherwise
    int 			insert(const K&, const V&);
    //lookup... returns 0 if the value is not present in the dictionary
    // equality of keys is with the operator== on K
    // doesn't modify V if not there
    int 			lookup(const K&,V&) const;
    // del  ... delete's an object from the dictionary 
    void 			del(const K&);
    // walk ... pass a closure and client data
    void 			walk(mDictValValClosure<K, V>&closure,
				     void *clientData) const;
    // size
    unsigned 			size() const;
};

//////////////////////////////////////////////////////////////////////////
//
// Ok, now we need an iterator.
//
// This iterator is NOT valid (undefined) until you call the next
// method at least once.
//
template <class K, class V> class mDictValValIterator {
private:
    RWTValHashDictionaryIterator<K,V> *impl;

    // Disallow default constructor.
    mDictValValIterator();
    
public:
    // constructor
    mDictValValIterator(const mDictValVal<K,V>&);
    // destructor
    virtual ~mDictValValIterator();
    // operator =
    mDictValValIterator(const mDictValValIterator<K,V>&);
    // get next... returns 0 if no more, 1 if there is more
    int 			next();
    // get current
    K 				current() const;
};

#ifdef BEAM
#include "mDictValVal.cc"
#include "rw/regexp.h"
#endif
#endif /* MERCATOR_DICT_VAL_VAL_H */
