//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#include "mList.h"

// $Author: spine $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/mList.cc,v $
// $Revision: 1.5 $
// $Date: 1995/07/28 21:23:17 $

// $Log: mList.cc,v $
//Revision 1.5  1995/07/28  21:23:17  spine
//Added copyright notice.
//
//Revision 1.4  1995/07/25  22:05:34  keith
//Cleaned up some usage of strings, dicts, and lists.  Ran with -pta to
//fix some previously unseen bugs in the parameterized classes.
//
//Revision 1.3  1995/07/24  17:44:26  beth
//Ian's Version
//
//Revision 1.1  1995/05/25  14:49:40  iansmith
//Initial revision
//

//
// mList Implementation
//

// Constructor ... build the implementation and make it empty
template <class T> mList<T>::mList()
{
    impl=new RWTPtrDlist<T>();
}

// Copy constructor: shallow copy with shared pointers
template <class T> mList<T>::mList(const mList<T> &x)
{
    impl=new RWTPtrDlist<T>(*(x.impl));
}
 
// Add an item..
template <class T> void mList<T>::append(const T *v) 
{
    // cast away constness since that is not what RW is expecting (UGH)
    impl->append((T *)v);
}

// look for an item
template <class T> T* mList<T>::search(const T* v) const
{
    if (impl->contains(v)) return (impl->find(v));
    return 0;
}

// remove the item but return a pointer to it...
// comparison is with the == operator on T
template <class T> T* mList<T>::del_item(const T *v)
{
    return impl->remove(v);
}

// remove an item and call its destructor
// This is noop if its not present
template <class T> void mList<T>::destroy_item(T *v)
{
    T* tmp=del_item(v);
    if (tmp) delete tmp;
}

// apply a closure over a list
template <class T> void mList<T>::walk(mListClosure<T> *closure,
				       void *clientData) const
{
    mListIterator<T> iter(*this);
    T *val;

    closure->startup(clientData);
    // walk the values
    while (val=iter.next()) {
	if (closure->action(val)==0) break;
    }
    // give 'em a closedown
    closure->closedown();
}
// destructor just deletes the data structure 
template <class T> mList<T>::~mList()
{
    // first clear the collection but don't delete the objects
    impl->clear();
    delete impl;
}
// assignment is a shallow copy of the lists... you get two differents lists
// but the nodes in them are the same
template <class T> mList<T>& mList<T>::operator=(const mList<T>& other)
{
    *impl=*other.impl;
    return *this;
}

// get the head
template <class T> T * mList<T>::head() const 
{
    mListIterator<T> iter(*this);
    // make sure iter will work
    if (impl->isEmpty()) return 0;
    // return the first one
    return iter.next();
}

// get the tail via an iteration over the whole list
template <class T> T * mList<T>::tail() const 
{
    T * drag,*current;
    mListIterator<T> iter(*this);
    // make sure iter will work
    if (impl->isEmpty()) return 0;
    // walk and assign
    while (current=iter.next()) {
	drag=current;
    }
    // return the drag pointer... last valid value
    return drag;
}

// ok, this is a cheezy hack to get the predecessor via
// walking the list... since we are implementing using
// a Dlist we should be able to inquire with RW about
// succ/pred but we can't because they don't provide
// an API so we have to do this expensive walk.
// equality is tested via the op== on T.
// IES
// get the entry right before 
// return 0 if they don't have the entry asked for or
// succ/pred would be off the beginning/start of the list
// both of these test for equality using op == on T
template <class T> T * mList<T>::predecessor(const T *val) const
{
    mListIterator<T> iter(*this);
    T * result=0,*current;
    // walk
    while (current=iter.next()) {
	// look for the equal value
	if (current->operator==(*val)) {
	    return result;
	}
	// drag result behind
	result=current;
    }
    // didn't exist
    return 0;
}

// get the entry right after
template <class T> T * mList<T>::successor(const T *val) const
{
    mListIterator<T> iter(*this);
    // we iter once to initialize
    T * current=iter.next(),*result;
    // walk the list
    while (result=iter.next())  {
	// check on drag pointer
	if (current->operator==(*val)) {
	    // found it, return result
	    return result;
	}
	current=result;
    }
    // didn't find it, or its currently current
    // in either case definition specifies that we send back 0
    return 0;
}
// figure out how many entries there are
template <class T> unsigned mList<T>::entries() const 
{
    return (impl->entries());
}

//
// ITERATOR
// 

//copy constructor
template <class T>
mListIterator<T>::mListIterator(const mListIterator<T> &x) :
    impl(new RWTPtrDlistIterator<T>(*x.impl))
{
}

// operator =
template <class T> mListIterator<T> & 
mListIterator<T>::operator=(const mListIterator<T> &x)
{
    *impl = *x.impl;
    // return self
    return *this;
}

// public constructor
template <class T> mListIterator<T>::mListIterator(const mList<T> &mlist)
{
    // we are  a friend so we just look inside and see....
    impl=new RWTPtrDlistIterator<T>(*(mlist.impl));
}

// destructor
template <class T> mListIterator<T>::~mListIterator()
{
    delete impl;
}

// get the next value from the iterator
template <class T> T * mListIterator<T>::next()
{
    // ugh, use operator()
    return (*impl)();
}

// reset the iterator
template <class T> void mListIterator<T>::reset()
{
    impl->reset();
}
