//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/mListVal.cc,v $
// $Revision: 1.9 $
// $Date: 1995/08/12 22:00:28 $

// $Log: mListVal.cc,v $
//Revision 1.9  1995/08/12  22:00:28  keith
//fixed problem with current()
//
//Revision 1.8  1995/08/12  21:44:23  keith
//fixed typo...
//
//Revision 1.7  1995/08/12  21:39:47  keith
//Added two versions of list.at().
//
//Revision 1.6  1995/08/12  21:32:42  keith
//Added iter.prev(), iter.insertAfterPoint(), list.prepend(), and made
//iter.current() a const function (since it returns a copy).
//
//Revision 1.5  1995/07/28  21:23:50  spine
//Added copyright notice.
//
//Revision 1.4  1995/07/25  22:05:34  keith
//Cleaned up some usage of strings, dicts, and lists.  Ran with -pta to
//fix some previously unseen bugs in the parameterized classes.
//
//Revision 1.3  1995/07/24  17:44:26  beth
//Ian's Version
//
//Revision 1.1  1995/05/25  14:51:11  iansmith
//Initial revision
//

#include "mListVal.h"


////////////////////////////////////////////////////////////////////////////
//
// mListVal
//
//
// Return a copy of the item at position i.  May not be used as an l-value.
//
template <class T> T
mListVal<T>::at(int i) const
{
    return impl->at(i);
}

//
// Return the item at position i.  May be used  as an l-value.
//
template <class T> T&
mListVal<T>::at(int i)
{
    return impl->at(i);
}

//
// add this this to the list... note: RW does a copy at the
// time of the call to append, thus this reference DOES
// get copied when inserted.
//
template <class T> void
mListVal<T>::append(const T &v) 
{
    impl->append(v);
}

//
// prepend an item onto the front of the list.
//
template <class T> void
mListVal<T>::prepend(const T& t)
{
    impl->prepend(t);
}

//
// search for a value... returns 0 if not found, 1 if found
// if found, it puts the value found into the second 
// reference parameter... if not found foundValue is not touched
//
template <class T> int
mListVal<T>::search(const T & target,
					   T & foundValue) const
{
    if (impl->find(target,foundValue)) return 1;
    return 0;
}

//
// remove the first items equal to T
// comparison is with the == operator on T
// returns number of removals
//
template <class T> int
mListVal<T>::del_item(const T & v)
{
    if (impl->remove(v)) return 1;
    return 0;
}

template <class T> void
mListVal<T>::walk(mListValClosure<T> *closure, void *clientData) const
{
    mListValIterator<T> iter(*this);
    T val;

    closure->startup(clientData);
    // walk the values
    while (iter.next()) {
	val=iter.current();
	if (closure->action(val)==0) break;
    }
    // give 'em a closedown
    closure->closedown();
}

//
// constructor (empty list)
//
template <class T>
mListVal<T>::mListVal()
{
    impl=new RWTValDlist<T>;
}

//
// copy constructor.. this will result in a copy of all members of the list
//
template <class T>
mListVal<T>::mListVal(const mListVal<T> &v)
{
    impl=new RWTValDlist<T>(*(v.impl));
}

//
// destructor... this deletes the data structure and all of its copies
//
template <class T>
mListVal<T>::~mListVal()
{
    delete impl;
}

//
// assignment is a deep copy of the lists
//
template <class T> mListVal<T>&
mListVal<T>::operator=(const mListVal<T> &t)
{
    *impl=(*(t.impl));
    return *this;
}

//
// how big is the list?
//
template <class T> unsigned
mListVal<T>::entries() const
{
    return impl->entries();
}

////////////////////////////////////////////////////////////////////////////
//
// mListValIterator
//
template <class T>
mListValIterator<T>::mListValIterator(const mListVal<T> &l) :
    impl(new RWTValDlistIterator<T>(*(l.impl)))
{
}

//
// advance
//
template <class T> int
mListValIterator<T>::next() 
{
    if (impl->operator++()) return 1;
    return 0;
}

//
// reverse
//
template <class T> int
mListValIterator<T>::prev()
{
    if (impl->operator--()) return 1;
    return 0;
}

//
// get current value
//
template <class T> T
mListValIterator<T>::current() const
{
    return impl->key();
}

//
// reset to initiali undefined state
//
template <class T> void
mListValIterator<T>::reset()
{
    impl->reset();
}

//
// destroy the implementation
//
template <class T>
mListValIterator<T>::~mListValIterator()
{
    delete impl;
}

//
// copy constructor
//
template <class T>
mListValIterator<T>::mListValIterator(const mListValIterator<T> &x) :
    impl(new RWTValDlistIterator<T>(*x.impl))
{
}

//
// operator =
//
template <class T> mListValIterator<T> & 
mListValIterator<T>::operator=(const mListValIterator<T> &x)
{
    delete impl;
    impl = new RWTValDlistIterator<T>(*x.impl);
    return *this;
}

//
// Add a new item after the position of the iterator.
//
template <class T> void
mListValIterator<T>::insertAfterPoint(const T& t)
{
    impl->insertAfterPoint(t);
}
