// This may look like C code, but it is really -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#ifndef MERCATOR_LIST_VAL_H
#define MERCATOR_LIST_VAL_H

// $Author: keith $
// $Source: /net/hm10/keith/proj/mercator/6.0/RCS/mListVal.h,v $
// $Revision: 1.7 $
// $Date: 1995/08/12 22:00:28 $

// $Log: mListVal.h,v $
//Revision 1.7  1995/08/12  22:00:28  keith
//fixed problem with current()
//
//Revision 1.6  1995/08/12  21:39:47  keith
//Added two versions of list.at().
//
//Revision 1.5  1995/08/12  21:32:42  keith
//Added iter.prev(), iter.insertAfterPoint(), list.prepend(), and made
//iter.current() a const function (since it returns a copy).
//
//Revision 1.4  1995/07/28  21:24:15  spine
//Added copyright notice.
//
//Revision 1.3  1995/07/24  17:43:39  beth
//Ian's Version
//
//Revision 1.1  1995/05/25  14:50:43  iansmith
//Initial revision
//


// We implement the mList with a Rogue Wave List right now
#include <rw/tvdlist.h>

/////////////////////////////////////////////////////////////////////////
//
// mListValClosure
//
template <class T> class mListValClosure {
public:
    // use this to setup your closure
    virtual void startup(void *clientData) {};
    // you'll still get a closedown if you short circuit
    //  out of the action routine
    virtual void closedown() {};
    // if the action routine returns 0 the closure stops
    // processing
    // if you don't subclass, we end up only calling the action routine once
    virtual int action(T &) {return 0;};
};


/////////////////////////////////////////////////////////////////////////
//
// mListVal
//
// 
// Linked List for Mercator
// You can put values in this list. These lists use a copy-in
// copy-out strategy.
//
// You MUST define the following:
// operator==
// operator=
// operator<
// copy constructor
//
template <class T> class mListVal {
    
    friend class mListValIterator<T>;
    
private:
    // The implementation
    RWTValDlist<T> *impl;
    
public:
    mListVal();
    mListVal(const mListVal &);		// deep copy
    virtual ~mListVal();		// doesn't delete contents

    mListVal<T> & operator=(const mListVal<T> &);	// deep copy

    //
    // Get the item at position i, may be used as l-value
    //
    T&		at(int i);
    //
    // Get the item as position i (copy), may NOT be used as l-value.
    //
    T		at(int i) const;
    //
    // add to the end
    //
    void 	append(const T& item);
    //
    // add to the beginning
    //
    void 	prepend(const T& item);
    //
    // search for a value... returns 0 if not found, 1 if found
    // if found, it puts the value found into the second 
    // reference parameter... if not found foundValue is not touched
    //
    int 	search(const T &target, T & foundValue ) const;
    //
    // remove the first item equal to T
    // comparison is with the == operator on T
    //  if it finds an item equal it returns 1 otherwise 0
    //
    int 	del_item(const T &);
    //
    // if you want to iterate, subclass the mListClosure and
    // pass it in here... the client data is passed to the startup routine
    //
    void 	walk(mListValClosure<T> *closure,void *clientData) const;
    //
    // how big is the list?
    //
    unsigned 	entries() const;
    //
    // is it empty?
    //
    int 	empty() { return (entries==0);};
};


/////////////////////////////////////////////////////////////////////////
//
// mListValIterator
//
// At first the iterator is NOT pointing to anything valid,
// so you must perform some operation on it to make it set
// itself to the first element.  You should not  remove elements
// from the list during iteration.
//
template <class T> class mListValIterator {
private:
    // this is the implementation using rogue wave
    RWTValDlistIterator<T> *impl;

    mListValIterator();			// disallowed
    
public:
    mListValIterator(const mListVal<T> &myList);
    mListValIterator(const mListValIterator& other);
    virtual ~mListValIterator();

    mListValIterator<T> & operator=(const mListValIterator<T>& other);
    
    //
    // use next to both setup the iterator the first time and
    // to iterate during use. it returns 0 when you reach the end
    // and is in an undefined state at that point....
    // you have to query using current() to get the current value
    //
    int 	next();
    //
    // Move back one item.
    //
    int		prev();
    //
    // get the current value under the iterator.  NOTE THAT THIS IS A COPY.
    //
    T 		current() const;
    //
    // reset the iterator to the undefined state it had at creation time
    //
    void 	reset();
    //
    // Add a new T after the position of the iterator.
    //
    void	insertAfterPoint(const T& t);
};

#ifdef BEAM
#include "mListVal.cc"
#endif

#endif  /* MERCATOR_LIST_VAL_H */
