// This may look like C code, but it is really -*- C++ -*-
//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#ifndef M_STRING_H
#define M_STRING_H

#include <rw/cstring.h>
#include "mRegexp.h"
#include "Util.h"


#ifndef nil
#define nil rwnil
#endif /* nil */

class mString {
private:
  RWCString 		impl;
  
public:
  friend class mRegexp;
  // construct with null string
  mString();
  // construct from const char *
  mString(const char *c);
  mString(const char *c, int n);
  // copy constructor
  mString(const mString& m);
  // conversion to char * (returns a copy, so the caller MUST delete it)
  operator char *() const;
  // conversion to const char *.  Caller MUST NOT modify or delete!
  operator const char *() const;
  // assignment
  mString& operator=(const mString &m);
  mString& operator+=(const mString &m);
  mString& operator+=(const char *c);
  
  unsigned hash() const;
  unsigned length() const;
  
  int operator==(const mString& m) const;
  int operator<(const mString& m) const;

  int index(const char *c, int i=0) const;
  int index(const mString &m, int i=0) const;
  int index(const mRegexp &r, int i=0) const;
  int index(const mRegexp &r, unsigned int *len, unsigned int i=0) const;

  int insert(int i, const char *c);
  int insert(int i, mString& m);

  // Substitute mString or char * string at position marked by regular
  //   expression r, beginning at offset i within the mString.  The 
  //   offset of the final character substituted is returned, -1 if no
  //   substitution occurred.
  int subst(mRegexp &r, int i, mString& m);
  int subst(mRegexp &r, int i, const char *c);
  // Returns the actual string located by the regular expression matcher
  //   beginning at offset i within the mString.
  mString* substString(mRegexp &r, int i);

  // Replace the subsequence beginning at offset i with length len
  //   with mString m or char * c.
  void replace(int i, int len, mString& m);
  void replace(int i, int len, const char *c);
};

#endif /* M_STRING_H */
