//
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
#ifndef _VRS
#define _VRS

// This was all taken from IN3's vrs.h
// This version makes it compatable for c++
// This is probably not complete, so if you are missing a command, etc. go
// look at the original.

#define TPL_MAXFRAMES   509
#define VCB_COMLEN      20
#define VCB_MAXKS       64

/*      These values are required by systems processing the statistics frames */
#define VRSS_NOP        0       /* General frame statistics */
#define VRSS_AUDIO      1       /* Audio open / close notification */
#define VRSS_RECOG      19      /* Recognition Notification */
#define VRSS_REALLY_RECOG 2     /* The Real Recognition Notification */
 
/*      This value is used to specify synchronous/asynchronous behavior */
/*      Applies to timeout value for vrsEnroll and vrsUpdate only       */
#define VRS_NOWAIT      0x10000         /* issue cmd asynchronously */
 
/* In cases of system errors - errno gets or'ed into low byte of status */
/* bits 8-11 are a variable state file - clear when checking error status! */
 
#define STS_CONTINUE    0xffff  /* Command is in progress       */
 
#define STS_MASK        0xf0ff  /* Mask to elimnate state bits  */
#define STS_GOOD        0       /* No errors                    */
#define STS_ERROR       0x8000  /* Error flag for status        */
#define STS_VRSERR      0x4000  /* VRS error / !System error    */
#define STS_ERRNO       0x00ff  /* ERRNO Subfield (System err)  */
 
#define STS_ILLEGAL     0xc000  /* Illegal command              */
#define STS_INVWID      0xc001  /* Invalid Word ID              */
#define STS_INVVCB      0xc002  /* Invalid Vocab ID             */
#define STS_NOENROLL    0xc003  /* No enroll frames available   */
#define STS_ENROLL      0xc004  /* Error encountered in enroll  */
#define STS_INVPRM      0xc005  /* Invalid Parameter            */
#define STS_NOMEM       0xc006  /* Malloc failed to get mem     */
#define STS_ENDAUDIO    0xc007  /* Audio device is gone         */
#define STS_NOMATCH     0xc008  /* Update failed to match       */
#define STS_WRONGMATCH  0xc009  /* Update matched wrong templt  */
#define STS_WRONGCNT    0xc00a  /* Update match wrong number of templates */
#define STS_NOKBD       0xc00b  /* No Keyboard Device   */
#define STS_KBFAIL      0xc00c  /* Keyboard stream push failed  */
#define STS_TIMEOUT     0xc00d  /* Enroll or Update timed out   */
#define STS_CANCEL      0xc00e  /* Enroll or Update canceled    */
#define STS_TOOSOFT      0xc010  /* Candidate template was too soft */
#define STS_TOOLOUD     0xc011  /* Peak Audio level approaching clip level */
#define STS_PERM        0xc012  /* No Permission - Security check failure */
#define STS_BUSY        0xc013  /* Exclusive lock fail - already busy */
#define STS_LOCKED      0xc014  /* Subsystem is locked by another connection */
 
#define STS_VERSION     0xc0ff  /* Version error: calling process must abort! */


typedef unsigned long   Svmask;

 
/*
 * Voice template data structure.  Each frame in "framecnt" element
 * represents 11 words in "data" element array.
 */
struct vrsTemplate {
        ushort  WordId;
        ushort  FrameCnt;
        ushort  Type;
        short   Data[TPL_MAXFRAMES];
};

/*
 * Vocabulary data structure for communicating with the API library
 * and the recognition subsystem
 */
struct vrsLoadVcb {
        short   WordId;                 /* command index */
        ushort  Function;               /* Future Use   */
        Svmask  Belongs2;               /* Command Set Mask                 */
        Svmask  Enables;                /* Command Enables Mask             */
        Svmask  Disables;               /* Command Disables Mask            */
        char    Command[VCB_COMLEN];    /* Name of entry                    */
        ushort  KeyStrokes[VCB_MAXKS];  /* Keycodes for command entry       */
};



extern "C" {

  extern int  vrsCancel(long);
  extern int  vrsClear(int);
  extern void vrsClose();
  extern void vrsCloseStats();
  extern int  vrsEnroll(int,long);
  extern int  vrsGetTmplt(vrsTemplate*,int);
  extern int  vrsGetVocab(vrsLoadVcb*,int);
  extern int  vrsHalt();
  extern int  vrsLastWord();
  extern int  vrsOpen(int);
  extern int  vrsOpenStats();
  extern int  vrsPutTmplt(vrsTemplate*,int);
  extern int  vrsPutVocab(vrsLoadVcb*,int);
  extern int  vrsRecog();
  extern int  vrsRecogEnable(int);
  extern int  vrsReset();
  extern unsigned long vrsSetCS(Svmask);
  extern int  vrsUpdate(int,long);
  extern int  vrsWait(long);

};

/*      And now for a few macros        */
#define RC_OFF          0
#define RC_ON           1
#define RC_TOGGLE       2
#define vrsRecogToggle()        vrsRecogEnable( RC_TOGGLE )
#define vrsRecogOn()            vrsRecogEnable( RC_ON )
#define vrsRecogOff()           vrsRecogEnable( RC_OFF )


#endif _VRS
