##
##
##
##	Note:  The use of this program shall be restricted to educational,
##	non-commercial use only.  All inquiries regarding licensing this
##	software for commercial use may be addressed to:
##	
##	Georgia Tech Research Corporation
##	Technology Licensing
##	Centennial Research Building - Rm. 275
##	400 Tenth Street, N.W.
##	Atlanta, Georgia  30332-0415
##	
##	*************************************************
##	
##	       Copyright 1996
##	Georgia Tech Research Corporation
##	    All Rights Reserved
##
##
##
## Navigate.tcl implements the code for traversing the widget hierarchy.
## 

Error EL_DEBUG navigate "Sourcing navigate.tcl"

#
# Determine whether a given object is navigable.  This is based on
# mapped state and the 'navigable' attribute from the templates.
#
proc isNavigable {name} {
    if {[modelManager mapped $name] == "FALSE"} {
      return "FALSE" 
    }
    return [wtemplate $name navigable]
}

#
# Move to a given node.  This primary interface to navigation.
#
proc GoTo {node} {
    global userLevel

    Error EL_DEBUG navigate "Going to [modelManager longname $node]"

    xwin raise $node
    xwin warp $node 1 1
    setFocus $node

    Audio $node 0
    Error EL_DEBUG navigate "past audio"
    if {$userLevel == "Novice"} {
        if {[modelManager parent $node] == "NULL" } {
              speak [modelManager name $node] 
        } else {
            speak [wtemplate $node speakOnInfo]
            speak [wtemplate $node speakOnEnter] nointerupt
	    if {[resource $node get sensitive] == "FALSE"} {
	        speak "greyed out" nointerupt
	    }
	    if {[resource $node get set] == "TRUE"} {
	        speak "selected" nointerupt
	    }
	}
    } 
    if {$userLevel == "Intermediate"} {
        if {[modelManager parent $node] == "NULL" } {
            speak [modelManager name $node]
        } else {
            speak [wtemplate $node speakOnEnter] 
            if {[resource $node get sensitive] == "FALSE"} {
                speak "greyed out" nointerupt
                }
	    if {[resource $node get set] == "TRUE"} {
	        speak "selected" nointerupt
	    }
        }
    }
    if {$userLevel == "Expert"} {
        if {[modelManager parent $node] == "NULL" } {
            speak [modelManager name $node]
        } else {
            speak [wtemplate $node speakOnEnter]
	}
    }
}

#
# This allows the user to jump quickly to the first object in a group.
# The auditory icons for the objects being "jumped" are played, so
# that they are interrupted quickly.
# NOT HANDLED YET: if the current object is the top of the application
# we should probably go to the first application (client)
# 
# The algorithm is quite inefficient and should be improved - if the
# current object is the 5th in the list, the list will be searched
# 5 times where the length examined decreases by 1 each time.
#
# - Beth
#
proc JumpBack {args} {
    set current [currentObject]

    set parent [modelManager parent $current]

    if {$parent == "NULL"} {
	return
    } {
	set children [modelManager children $parent]

	# assignment to make loop work, doesn't make real sense
	set prev $current

	set play "FALSE"
	set target "NULL"
	while {$prev != "NULL"} {
	    set node $prev
	    set prev "NULL"
	    foreach child $children {
	        if {$child == $node} {
		    #puts "JumpBack:: match is $child"
		    break
	        } {
		    #puts "JumpBack:: setting prev to $child"
		    set prev $child
		    if {[isNavigable $child] == "TRUE"} {
			set target $child
			set play "TRUE"
		    }
	        }
	    }
	    if {$play == "TRUE"} {
		Audio $target 0
		set play "FALSE"
	    }
        }
	if {$target != "NULL"} {
	    GoTo $target
	} {
            OutofBounds
	}
    }
    return
}

#
# This allows the user to jump quickly to the last object in a group.
# The auditoryicons for the objects being "jumped" are played, so
# that they are interrupted quickly.
# NOT HANDLED YET: if the current object is the top of the application
# we should probably go to the last application (client)
# 
# - beth
#
proc JumpForward args {
    set current [currentObject]

    set parent [modelManager parent $current]

    if {$parent == "NULL"} {
	#puts "JumpForward:: at top of tree"
	#should add jump to last client, i guess
	return
    } {
	set children [modelManager children $parent]

	set target "NULL"
	set flag "FALSE"
        foreach child $children {
	    if {$flag == "TRUE"} {
		if {[isNavigable $child] == "TRUE"} {
		    set target $child
	            Audio $child 0
		}
	    }
	    if {$child == $current} {
		set flag "TRUE"
	    } 
	}
	if {$target != "NULL"} {
	    GoTo $target
	} {
            OutofBounds
	}
    }
}

############################################################################
#
# These are the basic up/down/left/right navigation routines.  In
# mercator.tcl are functions that are bound to the arrow keys.  These
# outer functions determine whether the user is in text mode or not,
# and invoke either Nav or Text functions (NavUp and TextUp for instance).
# Each of the "Nav" functions has an associated "Go" routine that's
# invoked recursively.
#
proc NavUp args  {
    set current [currentObject]
 
    # CHI hack, don't navigate out of Dialogs...
    #if {[modelManager class $current] == "XmDialogShell"} {
    #   set up "NULL"
    #}
    if {[wtemplate $current allowEscape] == "FALSE"} {
        Error EL_DEBUG navigate "Nowhere to go"
        OutofBounds
    } {
        set destination [GoUp $current]
	if {$destination == "NULL"} {
	    Error EL_DEBUG navigate "Nowhere to go"
	    OutofBounds
	} else {
	    GoTo $destination
	}
    }
}
 
proc GoUp {node} {
    set parent [modelManager parent $node]

    if {$parent == "NULL"} {
        Error EL_DEBUG navigate "No parent"
        return "NULL"
    } {
	set done FALSE
	while {$done == "FALSE"} {
	    if {[isNavigable $parent] == "FALSE"} {
		return [GoUp $parent]
	    } {
		set grandparent [modelManager parent $parent]
		if { $grandparent == "NULL" } {
		    return $parent
		} {
		    if {[llength [modelManager children $grandparent]] == 1} {
			set parent $grandparent
		    } else {
			return $parent
		    }
		}
	    }
	}
    }
}
 

proc NavDown args {
    set current [currentObject]
    set down [GoDown $current]

    if {$down == "NULL"} {
        Error EL_DEBUG navigate "Nowhere to go"
        OutofBounds
    } {
	GoTo $down
    }
}
 
proc GoDown {node} {
    set children [modelManager children $node]

    if { $children == "NULL" || $children == "" } {
        return "NULL"
    } else {
	set done FALSE
	while {$done == "FALSE"} {
	    if { [llength $children] == 1 } {
		set tmpchildren [modelManager children [lindex $children 0]]
		if { [llength $tmpchildren] > 0 } {
		    set children $tmpchildren
		} else {
		    set done TRUE
		}
	    } else {
		set done TRUE
	    }
	}
        foreach child $children {
            if { [string compare [isNavigable $child] "TRUE"] == 0 } {
                return $child
            } {
                set grandchild [GoDown $child]
                if { [string compare $grandchild "NULL"] != 0 } {
                    return $grandchild
                }
            }
        }
        return "NULL"
    }
}
 
#
#  Note that just choosing the next item in the list of children, doesn't
#  necessarily make sense.  The spatial ordering in the interface makes
#  more sense but that's too hard at the moment.
# 
proc NavRight args {
    set current [currentObject]
 
    set right [GoRight $current]
    if {$right == "NULL"} {
        Error EL_DEBUG navigate "Nowhere to go"
        OutofBounds
    } {
	GoTo $right
    }
}

proc GoRight {node} {
 
    set parent [modelManager parent $node]
 
    if {$parent == "NULL"} {
        # must be at the top of the tree already
        return "NULL"
    } {
        set children [modelManager children $parent]
        set match "FALSE"
        foreach child $children {
            if {$match == "TRUE"} {
                if {[isNavigable $child] == "TRUE"} {
                    return $child
                }
		# THE following section MIGHT mess up pop-up windows.
                set down [GoDown $child]
                if {$down != "NULL"} {
                    return $down
                }
            } {
                if {$child == $node} {
                    set match "TRUE"
                }
            }
        }
	# I try to check for a parent who is not navigable. I tested this
	# with a simple case of having one level of unnavigable parent 
	# and that works. I didn't get to test it with more than one 
	# level of unnavigable parents though. -wl
	if {[isNavigable $parent] == "FALSE"} {
	    puts "here\n"
	    set right [GoRight $parent]
	    if {$right != "NULL"} {
		return $right
	    }
	}
        return "NULL"
    }
}
 
proc NavLeft args {
    set current [currentObject]
 
    set left [GoLeft $current]
    if {$left == "NULL"} {
        Error EL_DEBUG navigate "Nowhere to go"
        OutofBounds
    } {
	GoTo $left
    }
}

proc GoLeft {node} {
    set parent [modelManager parent $node]

    # must be at the top of the tree already
    if {$parent == "NULL"} {
        return "NULL"
    } {
        set children [modelManager children $parent]
        set prev "NULL"
        foreach child $children {
            if {$child == $node} {
                break
            } {
                set prev $child
            }
        }
        # Nothing to the left; check for navigable parent. If the
	# parent is not navigable then we want to navigate to the
	# parent's left neighbor. -wl
        if {$prev == "NULL"} {
	    if {[isNavigable $parent] == "FALSE"} {
		return [GoLeft $parent]
	    }
            return "NULL"
        } {
            if {[isNavigable $prev] == "TRUE"} {
                return $prev
            } {
		# The following checks to see if $prev has any children.
		# If it does we want to navigate to the right most child.
		# If it doesn't we try its left neighbor.
                set downchildren [modelManager children $prev]
		set num [llength $downchildren]
		if {$num > "0"} {
		    set last [expr $num - 1]
		    set downprev [lindex $downchildren $last]
		    if {[isNavigable $downprev] == "TRUE"} {
			return $downprev
		    } {
			return [GoLeft $downprev]
		    }
		}
                return [GoLeft $prev]
            }
        }
    }
}

############################################################################
#
# Interclient navigation code.
#
proc NavAppRight args {
    global userLevel
    global textMode

    set old [currentObject]
    set client [advanceClient]
    if {$old != [currentObject]} {
        GoTo [currentObject]
        set textMode "FALSE"
        if {$userLevel == "Novice"} {
            speak "$client is the current application"
        } else {
            speak "$client"
	}
    } else {
      OutofBounds
    }

}

proc NavAppLeft args {
    global textMode
    global userLevel

    set old [currentObject]
    set client [backupClient]
    if {$old != [currentObject]} {
      GoTo [currentObject]
      set textMode "FALSE"
      if {$userLevel == "Novice"} {
          speak "$client is the current application"
      } else {
	  speak "$client"
      }
    } else {
      OutofBounds
    }

}
