/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/
#include "os-interface.h"
#include "udp.h"
#include "tables.h" /* 16->8 bit audio conversion stuff */
#include <stdlib.h>

/* Just in case we need it in a few places. */
#define MAXTHREADS 0x10
#define MAXBLOCKEDFDS 8

#ifdef SOLARIS
#include <thread.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/systeminfo.h>
#include <sys/filio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
static int highPriorityThreads;
/* we keep track of this to insure all runnable high-priority threads
   get scheduled */
static mutex_t numHighPriorityMutex;
/* this is where we store the termios in case we need to go back to
   blocking mode */
#if 0
struct termios originalTermios;
#endif

#endif

#ifdef IRIX4
#include <limits.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <sys/schedctl.h>
#include <audio.h>
#include <sys/stat.h>
#include <sys/termio.h>
#include <sys/wait.h>
#include <signal.h>

/* need this local variable*/
static usptr_t *sharedArena=NULL;
/* this is where we store the termios in case we need to go back to
   blocking mode */
struct termio originalTermios;
/* we need this so that when a thread gets forked onto the tricky thread
   forking area (where we fool with priorities) we know where to go */
typedef struct {
  void (*func)(void *);
  void *arg;
  int prio;
} threadInfoStruct;

/* this is the array of thread priorities for schedctl */
static int threadPriority[3];

#endif

#ifdef SUNOS
/* #define SCHEDULER_DEBUG */
#include <lwp/stackdep.h>
#include <lwp/check.h>
#include <lwp/lwpmachdep.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <audio_device.h> /* in /usr/demo/SOUND/multimedia */
#include <sys/termio.h>
#include <sys/filio.h>
#include <signal.h>

/* these are the potential status bytes */
#define STATUS_BLOCKED_IO 0 
#define STATUS_BLOCKED_MUTEX 1 
#define STATUS_BLOCKED_CONDVAR 2 
#define STATUS_SLEEPING 3
#define STATUS_READY 4
#define STATUS_JOINING 5
#define STATUS_SUSPENDED 6
#define STATUS_EXITED 7
#define STATUS_ALREADY_SCHEDULED 8
#define STATUS_NOT_IN_USE 9

/* reason for ready */
#define READY_DATA 0
#define READY_TIME 1
#define READY_NORMAL 2

/* These are the lengths of time of a time slice for each priority */
/* Bogusly, these depend on the values of the constants 
   defined in os-interface.h to pick the right slot in the array and
   this doesn't use THREAD_NUMBER_PRIOS which it should */
static int threadTimeSlice[]= { 50000, 35000, 20000};

/* This is our scheduler data structure for all threads */
typedef struct {
  thread_t tid; /* thread id */
  unsigned char status; /* what state is the thread in */
  unsigned char priority; /* HIGH, MED, OR LOW */
  unsigned long secs, millisecs; /* resumption time for sleeping or blocked */
  unsigned long numFds; /* if blocked, how many fds is he waiting on */
  int fd[MAXBLOCKEDFDS]; /* the fd vaules */
  thread_t joinee; /* if we are joining, who are joining with? */
  unsigned char whyWokenUp; /* if a thread is ready, why is it ready */
  int exitStatus; /* what he passed into thread_exit() */
} threadInfoStruct;
/* This is the scheduler data structure for threads that are
   ready to run */
typedef struct {
  thread_t tid;
  unsigned char status;
} threadReadyStruct;

/* This our per-thread data structure */
threadInfoStruct threadInfo[MAXTHREADS];

/* this finds a thread with a given thread ident */
static threadInfoStruct *findThread(thread_t t);

/* this is magic size for audio blocking (read/write) */
#define AUDIO_BLOCK_SIZE 800 

/* this is where we store the termios in case we need to go back to
   blocking mode */
struct termios originalTermios;

/* this is the schedulers thread ident */
thread_t schedulerTid;

#endif

#include <errno.h>
#include <stdio.h>
#include <stropts.h>
#include <poll.h>
#include <string.h>
#include <fcntl.h>

#ifdef SOLARIS
/* This opens the audio device and puts the resulting fd into audioFd. It
   seems like two threads should be able to use the same fd concurrently,
   as long as only one is reading and only one is writing. */
#ifndef USE_AUDIOFILE

AUDIO_T AUDIO_open()
{
#ifdef NO_AUDIO_DEBUG
  static int zero=0;
  return &zero;
#else
  int *fd=(int *)malloc(sizeof(int));;
  /* open the device */
  *fd=open("/dev/audio",O_RDWR);
  if (*fd==-1) { /* error */
    perror("opening audio device");
    PROCESS_exit(1);
  }
  return fd;
#endif
}

/* This routine closes the audio device */
void AUDIO_close( AUDIO_T *fd)
{
#ifdef NO_AUDIO_DEBUG
  *fd=NULL;
#else
  close(**fd);
  free(*fd);
  *fd=NULL;
#endif
}

int AUDIO_valid(AUDIO_T ah)
{
  if (ah) return 1;
  else return 0;
}

int AUDIO_play(AUDIO_T fd,unsigned char *buffer,unsigned int length)
{
#ifdef NO_AUDIO_DEBUG
  return(1);
#else
  unsigned int total=0,ret;
  while (total<length) {
    ret=write(*fd,buffer+total,length-total);
    if (ret==-1) return -1;
    total+=ret;
  }
  return(0);
#endif
}

int AUDIO_record(AUDIO_T fd,unsigned char *buffer,unsigned int length)
{
#ifdef NO_AUDIO_DEBUG
  return(1);
#else
  unsigned int total=0,ret;
  while (total<length) {
    ret=read(*fd,buffer+total,length-total);
    if (ret==-1) {
      return -1;
    }
    total+=ret;
  }
  return 0;
#endif
}
#endif

void MUTEX_init(MUTEX_T *mutex)
{
  if (mutex_init(mutex,USYNC_THREAD,NULL)!=0) {
    perror("mutex_init");
    PROCESS_exit(1);
  }

}

void MUTEX_lock(MUTEX_T *mutex)
{
  if (mutex_lock(mutex)!=0) {
    perror("mutex_lock");
    PROCESS_exit(1);
  }
}

void MUTEX_unlock(MUTEX_T *mutex)
{
  if (mutex_unlock(mutex)!=0) {
    perror("mutex_lock");
    PROCESS_exit(1);
  }
}

void COND_init(COND_T *cond,MUTEX_T *m)
{
  cond_init(cond,USYNC_THREAD,NULL);
  /* we should be putting this in a dictionary so we can check it
     below in COND_wait */
}

void COND_wait(COND_T *cond,MUTEX_T *mutex)
{
/* unlocks the mutex */
  if (cond_wait(cond,mutex)!=0) {
    perror("cond_wait"); /* don't exit during development */
    PROCESS_exit(1);
  }
  /* grabs the lock */
}

void COND_signal(COND_T *cond)
{
  if (cond_signal(cond)!=0) {
    perror("cond_signal");
    PROCESS_exit(1);
  }
}

int THREAD_kill(THREAD_T tid)
{
    return (thr_kill(tid,SIGINT));
}

THREAD_T THREAD_fork(void (*threadMain)(void *),void *arg,
		     int prio)
{
  thread_t newThread;
  int x;
  /* create a thread on solaris on the function playerMain */
  /* Arguments:
     NULL: use default stack location
     0: Use default stack size
     playerMain: function to start the new thread running in.
     NULL: argument to playerMain
     0: options to thr_create
     NULL: where to put the return value of the thread id of the newly
     created thread (this is of type thread_t *) if you use it. 
     In short, this is the simplest, stupidest thread creation statement. */

  x=thr_create(NULL, 0, (void *(*)(void*)) threadMain,
			arg, 0, &newThread);
  
  if (x!= 0) {
    /* didn't create it successfully... print an error an exit */
    perror("thr_create");
    PROCESS_exit(1);
  }  
  if (thr_setprio(newThread,2-prio)==-1) {
    perror("thr_setprio");
    PROCESS_exit(1);
  }
  if (prio==THREAD_HIGH_PRIORITY || prio==THREAD_MEDIUM_PRIORITY) {
    MUTEX_lock(&numHighPriorityMutex);
    ++highPriorityThreads;
    if (thr_setconcurrency(highPriorityThreads)!=0) {
      perror("thr_setconcurrency");
      PROCESS_exit(1);
    }
    MUTEX_unlock(&numHighPriorityMutex);
  }
  return newThread;
}

void THREAD_yield() { thr_yield();}

void THREAD_sleep(unsigned int s) { sleep(s);}

void getCurrentHostname(char *hostbuffer,unsigned int length) 
{
  char *ptr;

  if (sysinfo(SI_HOSTNAME,hostbuffer,64)==-1) {
    perror("sysinfo");
    PROCESS_exit(1);
  }
  if ((ptr=strchr(hostbuffer,'.'))!=NULL) {
    *ptr='\0';
  }
}

void THREAD_init()
{
  highPriorityThreads=1; /* main is high priority */
  if (thr_setprio(thr_self(),2)!=0) {
    perror("thr_setprio");
    PROCESS_exit(1);
  }
  MUTEX_init(&numHighPriorityMutex);
}

int THREAD_block(int *infd,int *outfd,unsigned int size,int delay)
{
  struct pollfd *myfd=(struct pollfd *)malloc(sizeof(struct pollfd)*size);
  int i,ret;

  if (size>MAXBLOCKEDFDS) {
    fprintf(stderr,"THREAD_block: too many file descriptors!\n");
    PROCESS_exit(1);
  }
  
  for (i=0; i<size; ++i) {
    myfd[i].fd=infd[i];
    myfd[i].events=POLLRDNORM | POLLRDBAND;
  }
  while (1) {
    /* use poll on solaris */
    if ((ret=poll(myfd,size,delay))==-1) {
      if (errno!=EINTR) {
	perror("poll");
	PROCESS_exit(1);
      } else {
	continue; /* could throw off delay amount */
      }
    }
    else {
      /* good poll */
      break;
    }
  }
  if (ret==0) {
    free(myfd);
    return -2;
  }
  if (outfd) {
    for (i=0; i<size; ++i) {
      if ((myfd[i].revents & POLLRDNORM) || (myfd[i].revents & POLLRDBAND)) 
	outfd[i]=infd[i];
      else 
	outfd[i]=-1;
    }
  }
  free(myfd);
  return 0;
}

int THREAD_join(THREAD_T t)
{
  int x;
  /* this is tricky since the correct type of x is void ** ...we are
     doing this because we know that an int cast to a void * will
     be passed at thread exit time */
  if (thr_join(t,NULL,(void **)&x)!=0) {
    perror("thr_join");
    PROCESS_exit(1);
  }
  return x;
}

void THREAD_exit(int retval)
{
  /* again we are assuming sizeof(int)==sizeof(void *)... this probably
     not a good idea */
  thr_exit((void *)retval);
}

void PROCESS_exit(int retval)
{
  exit(retval);
}

void THREAD_resume(THREAD_T t)
{
  if (thr_continue(t)!=0) {
    perror("thr_continue");
    PROCESS_exit(1);
  }
}

void THREAD_suspend(THREAD_T t)
{
  if (thr_suspend(t)!=0) {
    perror("thr_suspend");
    PROCESS_exit(1);
  }
}

int THREAD_equality(THREAD_T t1,THREAD_T t2)
{
  return (t1==t2); /* does arnold schwarzeneggar == liquid metal? */
}

THREAD_T THREAD_self()
{
  return thr_self();
}
void KEYBOARD_nonBlocking()
{
#if 0
  struct termios modified;

  ioctl(0,TCGETS,&originalTermios);
  modified=originalTermios;
  modified.c_lflag=modified.c_lflag & 0xfffffffd;
  modified.c_cc[VMIN]=1;
  modified.c_cc[VTIME]=0;
  ioctl(0,TCSETS,&modified);
#endif
}

void KEYBOARD_readChar(char *c)
{
  int zero=0;
  THREAD_block(&zero,NULL,1,-1);
  read(0,c,1);
}

void KEYBOARD_blocking()
{
#if 0
  int mask;
  
  /* restore previous settings */
  ioctl(0,TCSETS,&originalTermios);
#endif 
}

#endif

#ifdef IRIX4
/* We need a big audio buffer to get the 16->8 bit conversions to work */
static short tmpShortBuffer[0x64000];

AUDIO_T AUDIO_open() 
{
 ALconfig audio_port_config;
 long pvbuf[2];
 long buflen;
 ALport *audio_port=(ALport *)malloc(sizeof(ALport)*2);

 /*
  * set output sample rate
  */
 pvbuf[0] = AL_OUTPUT_RATE;
 pvbuf[1] = AL_RATE_8000;
 buflen   = 2;
 ALsetparams(AL_DEFAULT_DEVICE, pvbuf, buflen);

 /*
  * set input sample rate
  */
 pvbuf[0] = AL_INPUT_RATE;
 pvbuf[1] = AL_RATE_8000;
 buflen   = 2;
 ALsetparams(AL_DEFAULT_DEVICE, pvbuf, buflen);

 /*
  * configure and open the audio port 
  */
 audio_port_config=ALnewconfig();
 ALsetwidth(audio_port_config,AL_SAMPLE_16);
 ALsetchannels(audio_port_config,AL_MONO);
 ALsetqueuesize(audio_port_config, /* guess */ 2400);
 audio_port[0]= ALopenport("ms", "r", audio_port_config);

 /* we end up opening it twice, since we need both input and output */
 audio_port_config=ALnewconfig();
 ALsetwidth(audio_port_config,AL_SAMPLE_16);
 ALsetchannels(audio_port_config,AL_MONO);
 ALsetqueuesize(audio_port_config, /* guess */ 2400);
 audio_port[1]= ALopenport("ms", "w", audio_port_config);
 return audio_port;
}

void AUDIO_close(AUDIO_T *ap)
{
  AUDIO_T audio=*ap;

  ALcloseport(audio[0]);
  ALcloseport(audio[1]);
  free(audio);
  *ap=NULL;
}

int AUDIO_play(AUDIO_T ap,unsigned char *buffer,unsigned int length)
{
  int i;
  if (length>0x64000) {
    fprintf(stderr,"OS-INTERFACE: internal buffer overflow in AUDIO_play!\n\tlength must be less than %d\n",length);
    PROCESS_exit(1);
  }
  AUDIO_8to16bit(buffer,tmpShortBuffer,length);
  ALwritesamps(ap[1],tmpShortBuffer,(long)length);
  return 0;
}

int AUDIO_record(AUDIO_T ap,unsigned char *buffer,unsigned int length)
{

  if (length>0x64000) {
    fprintf(stderr,"OS-INTERFACE: internal buffer overflow in AUDIO_play!\n\tlength must be less than %d\n",length);
    PROCESS_exit(1);
  }
  ALreadsamps(ap[0],tmpShortBuffer, (long)length);
  AUDIO_16to8bit(tmpShortBuffer,buffer,length);
  return 0;
}

/* we have to do this so we can alter our scheduling priority */
static void trickyThreadForker(void *arg)
{
  threadInfoStruct me,*copy=(threadInfoStruct *)arg;
  me=*copy;

  free(copy);
/* This is what I'd like to do, but it requires that you are superuser 
  if (schedctl(NDPRI,0,threadPriority[me.prio])==-1) {
    perror("schedctl");
    PROCESS_exit(1);
  }
*/
  if (nice(me.prio*19)==-1) {
    perror("nice");
    PROCESS_exit(1);
  }
  me.func(me.arg);
}

THREAD_T THREAD_fork(void (*threadMain)(void *),void *arg,int priority)
{
  int ret;
  threadInfoStruct *info=(threadInfoStruct *)malloc(sizeof(threadInfoStruct));

  info->func=threadMain;
  info->arg=arg;
  info->prio=priority;

  if ((ret=sproc(trickyThreadForker,PR_SALL ,info))==-1) {
    fprintf(stderr,"sproc: errno=%d\n",errno);
    perror("sproc");
    PROCESS_exit(1);
  }
  return(ret);
}
/* I'm not sure what this would mean on IRIX since we have processes
 not threads*/
void THREAD_yield() { }

void THREAD_sleep(unsigned int s) { sleep(s);}

void MUTEX_init(MUTEX_T *mutex)
{
  *mutex=usnewlock(sharedArena);
  usinitlock(*mutex);
  
}

void MUTEX_lock(MUTEX_T *mutex)
{
  if (ussetlock(*mutex)==-1) {
    perror("ussetlock");
    PROCESS_exit(1);
  }
}

void MUTEX_unlock(MUTEX_T *mutex)
{
  if (usunsetlock(*mutex)==-1) {
    perror("usunsetlock");
    PROCESS_exit(1);
  }
}

void COND_init(COND_T *cond,MUTEX_T *m)
{
  *cond=usnewsema(sharedArena,0);
  /* we should be putting this in a dictionary so we can check it
     below in COND_wait */
}

void COND_wait(COND_T *cond,MUTEX_T *mutex)
{
 /* There is a question here about whether I should make the semaphore zero to
    insure that I block. The problem is that someone else might be blocked 
    already and changing the semaphore out from under them seems bad. I am 
    NOT going to zero it, and I am going to hope that they check the 
    condition in a loop and thus continue to decrement until they get a 
    block */

  /* unlocks the mutex */
  MUTEX_unlock(mutex);
  if (uspsema(*cond)==-1) {
    perror("uspsema");
    PROCESS_exit(1);
  }
  MUTEX_lock(mutex);
  /* grabs the lock */
}

void COND_signal(COND_T *cond)
{
  if (usvsema(*cond)==-1) {
    perror("usvsema");
    PROCESS_exit(1);
  }
}

void getCurrentHostname(char *buffer,unsigned length)
{
  char *ptr;
  if (gethostname(buffer,length-1)==-1) {
    perror("gethostname");
    PROCESS_exit(1);
  }
  buffer[length-1]-'\0';
  if ((ptr=strchr(buffer,'.'))!=NULL) {
    *ptr='\0';
  }
}

void THREAD_init()
{
  static char namebuf[80];
  struct stat s;
  unsigned long stackSize;

  sprintf(namebuf,"/tmp/.shared-data-%d\n",getpid());
  /*  if it exists on disk, blow it away */
  if (stat(namebuf,&s)==-1) {
    if (errno!=ENOENT) {
      perror("stat");
      PROCESS_exit(1);
    }
  }
  else {
    if ( unlink(namebuf)==-1) {
      perror("unlink");
      PROCESS_exit(1);
    }
  }
  /* set our stack size big enough to be useful (1 meg) */
  if ((stackSize=prctl(PR_SETSTACKSIZE,0x50000))==-1) {
    perror("prctl");
    PROCESS_exit(1);
  } 
  /* signal handling ...want to be able to let threads exit independantly */
  if (prctl(PR_SETEXITSIG,0)==-1) {
    perror("prctl");
    PROCESS_exit(1);
  }
  
  if (usconfig(CONF_INITUSERS,MAXTHREADS)==-1) {
    perror("usconfig");
    PROCESS_exit(1);
  }
  sharedArena=usinit(namebuf);
  if (!sharedArena) {
    perror("usinit");
    PROCESS_exit(1);
  }
  /* set up the thread priority list */
  threadPriority[THREAD_HIGH_PRIORITY]=NDPNORMMAX;
  threadPriority[THREAD_LOW_PRIORITY]=NDPNORMMIN;
  threadPriority[THREAD_MEDIUM_PRIORITY]=NDPNORMMAX+
    ((NDPNORMMIN-NDPNORMMAX)/2);
  
  /* make sure main is high priority */
  /* I'd like to do schedctl but it requires super user privilege 
  if (schedctl(NDPRI,0,threadPriority[THREAD_HIGH_PRIORITY])==-1) {
    perror("schedctl");
    PROCESS_exit(1);
  }
  */
  
}

int AUDIO_valid(AUDIO_T ah)
{
  if (ah) return 1;
  return 0;
}

int THREAD_block(int *infd,int *outfd,unsigned int size,int delay)
{
  struct pollfd *myfd=(struct pollfd *)malloc(sizeof(struct pollfd)*size);
  int i,ret;
  
  if (size>MAXBLOCKEDFDS) {
    fprintf(stderr,"THREAD_block: too many file descriptors!\n");
    PROCESS_exit(1);
  }
  
  for (i=0; i<size; ++i) {
    myfd[i].fd=infd[i];
    myfd[i].events=POLLIN;
  }
  /* use poll on SGIs...*/
  while (1) {
    if ((ret=poll(myfd,size,delay))==-1) {
      if (oserror()!=EINTR) {
	perror("poll");
	PROCESS_exit(-1);
      } else {
	continue; /* this could potentially throw off delay calculations */
      }
    } else {
      /* no error, sucessful poll */
      break;
    }
  }
  if (ret==0) {
    free(myfd);
    return -2; /* TIME EXPIRED*/
  }
  if (outfd) {
    for (i=0; i<size; ++i) {
      if (myfd[i].revents & POLLIN) outfd[i]=infd[i];
      else outfd[i]=-1;
    }
  }
  free(myfd);
  return 0;
}

void KEYBOARD_nonBlocking()
{
  struct termio modified;

  ioctl(0,TCGETA,&originalTermios);
  modified=originalTermios;
  modified.c_lflag=modified.c_lflag & 0xfffffffd;
  modified.c_cc[VMIN]=1;
  modified.c_cc[VTIME]=0;
  ioctl(0,TCSETA,&modified);
}

void KEYBOARD_readChar(char *c)
{
  int zero=0;
  THREAD_block(&zero,NULL,1,-1);
  read(0,c,1);
}

void KEYBOARD_blocking()
{
  int mask;
  
  mask=sigblock(sigmask(SIGALRM));

  /* restore previous settings */
  ioctl(0,TCSETA,&originalTermios);

  sigsetmask(mask);
}

int THREAD_join(THREAD_T thread)
{
  int status;
  pid_t ret;

  while (1) {
    ret=waitpid(thread,&status,0);
    if (ret!=thread) {
      perror("waitpid");
      PROCESS_exit(1);
    }
    if (WIFEXITED(status)) {
      return(WEXITSTATUS(status));
    }
    
  }
}

void THREAD_exit(int arg)
{
  exit(arg);
}

void PROCESS_exit(int arg)
{
  if (prctl(PR_SETEXITSIG,SIGINT)==-1) {
    perror("prctl");
  }
  exit(arg);
}

int THREAD_equality(THREAD_T t1,THREAD_T t2)
{
  return(t1==t2);
}

void THREAD_suspend(THREAD_T t1)
{
  if (blockproc(t1)==-1) {
    perror("blockproc");
    PROCESS_exit(1);
  }
}

void THREAD_resume(THREAD_T t1)
{
  if (unblockproc(t1)==-1) {
    perror("unblockproc");
    PROCESS_exit(1);
  }
}

THREAD_T THREAD_self()
{
  return(getpid());
}

#endif

#ifdef SUNOS
/* This opens the audio device and puts the resulting fd into audioFd. It
   seems like two threads should be able to use the same fd concurrently,
   as long as only one is reading and only one is writing. */
AUDIO_T AUDIO_open()
{
  int *fd=(int *)malloc(sizeof(int));;
  /* open the device */
  *fd=open("/dev/audio",O_RDWR | O_NDELAY);
  if (*fd==-1) { /* error */
    perror("opening audio device");
    PROCESS_exit(1);
  }
  return fd;
}

/* This routine closes the audio device */
void AUDIO_close( AUDIO_T *fd)
{
  close(**fd);
  free(*fd);
  *fd=NULL;
}

int AUDIO_valid(AUDIO_T ah)
{
  if (ah) return 1;
  else return 0;
}

int AUDIO_play(AUDIO_T fd,unsigned char *buffer,unsigned int length)
{
  unsigned int total=0,ret,towrite;

  while (total<length) {
    /* corresponds to 100ms */
    towrite=length-total>AUDIO_BLOCK_SIZE ? AUDIO_BLOCK_SIZE : length-total;
    ret=write(*fd,buffer+total,towrite);
    if (ret==-1) return -1;
    total+=ret;
  }
  return(0);
}

int AUDIO_record(AUDIO_T fd,unsigned char *buffer,unsigned int length)
{
  unsigned int total=0,ret,toread;
  long waiting;

  while (total<length) {
    /* corresponds to 100ms */
    toread=length-total>AUDIO_BLOCK_SIZE ? AUDIO_BLOCK_SIZE : length-total;
    if (ioctl(*fd,FIONREAD,&waiting)==-1) {
      if (errno==EBADF) return -1;
      perror("ioctl");
      PROCESS_exit(1);
    }
    if (waiting<3*AUDIO_BLOCK_SIZE/4) {
      /* don't have 100ms of buffer built up..yield to other threads */
      THREAD_MILLI_SLEEP(AUDIO_BLOCK_SIZE/8 * waiting/AUDIO_BLOCK_SIZE);
      continue;
    }
    ret=read(*fd,buffer+total,toread);
    if (ret==-1) return -1;
    total+=ret;
  }
  return 0;
}

void MUTEX_init(MUTEX_T *mutex)
{
  if (mon_create(mutex)==-1) {
    lwp_perror("mon_create");
    PROCESS_exit(1);
  }
}

void MUTEX_lock(MUTEX_T *mutex)
{
  threadInfoStruct *self;
  thread_t tid;
  int mask;

  /* turn off alarms */
  mask=sigblock(sigmask(SIGALRM));

  /* find ourselves... */
  if (lwp_self(&tid)==-1) {
    lwp_perror("lwp_self");
    PROCESS_exit(1);
  }
  self=findThread(tid);
  self->status=STATUS_BLOCKED_MUTEX;
  /* we want to make a note of our status for the scheduler, so
     he can NOT bother scheduling us */
  if (mon_enter(*mutex)==-1) {
    lwp_perror("mon_enter");
    PROCESS_exit(1);
  }
  self->status=STATUS_READY;

  /* restore alarm */
  sigsetmask(mask);
}

void MUTEX_unlock(MUTEX_T *mutex)
{
  thread_t owner,waiter;
  int i,mask;

  /* turn off signals */
  mask=sigblock(sigmask(SIGALRM));

  /* Ok, the danger here is that the scheduler might not become aware
     of threads that need to run .... so we go looking for the first
     guy waiting on this mutex and mark him as yielded */
  if (mon_waiters(*mutex,&owner,&waiter,1)!=0) {
    /* we only do this if someone is waiting... find the right thread */
    for (i=0; i<MAXTHREADS;++i) {
      if (threadInfo[i].status==STATUS_NOT_IN_USE) continue;
      if (SAMETHREAD(waiter,threadInfo[i].tid)) {
	/* found it */
	threadInfo[i].status=STATUS_READY; /* he's ready to run... */
	break;
      }
    }
  } 
  if (mon_exit(*mutex)==-1) {
    lwp_perror("mon_exit");
    PROCESS_exit(1);
  }
  /* turn on signals */
  sigsetmask(mask);
}

void COND_init(COND_T *cond, MUTEX_T *m)
{
  if (cv_create(cond,*m)==-1) {
    lwp_perror("cv_create");
    PROCESS_exit(1);
  }
}

void COND_wait(COND_T *cond,MUTEX_T *mutex)
{

  threadInfoStruct *self;
  thread_t tid;
  int mask;

  /* turn off signals */
  mask=sigblock(sigmask(SIGALRM));

  /* find ourselves... */
  if (lwp_self(&tid)==-1) {
    lwp_perror("lwp_self");
    PROCESS_exit(1);
  }
  self=findThread(tid);
  self->status=STATUS_BLOCKED_CONDVAR;
  /* we want to make a note of our status for the scheduler, so
     he can NOT bother scheduling us */
  /* unlocks the mutex... note that on sunos its already bound,
     so we can ignore the MUTEX_T *arg */
  if (cv_wait(*cond)==-1) {
    lwp_perror("cv_wait");
    PROCESS_exit(1);
  }
  self->status=STATUS_READY;
  /* grabs the lock when done*/
  
  /* turn signals back on */
  sigsetmask(mask);
}

void COND_signal(COND_T *cond)
{
  thread_t waiter;
  int i,mask;

  /* turn off signals */
  mask=sigblock(sigmask(SIGALRM));

  /* Ok, the danger here is that the scheduler might not become aware
     of threads that need to run .... so we go looking for the first
     guy waiting on this CV and mark him as yielded */
  if (cv_waiters(*cond,&waiter,1)!=0) {
    /* we only do this if someone is waiting... find the right thread */
    for (i=0; i<MAXTHREADS;++i) {
      if (threadInfo[i].status==STATUS_NOT_IN_USE) continue;
      if (SAMETHREAD(waiter,threadInfo[i].tid)) {
	/* found it */
	threadInfo[i].status=STATUS_READY; /* he's ready to run... */
	break;
      }
    }
  }
  if (cv_notify(*cond)!=0) {
    lwp_perror("cv_notify");
    PROCESS_exit(1);
  }
  /* restore signals */
  sigsetmask(mask);
}

THREAD_T THREAD_fork(void (*threadMain)(void *),void *arg,int prio)
{
  thread_t tid;
  stkalign_t*stack=lwp_newstk();
  int mask,i,ok=0;

  /* turn off signals */
  mask=sigblock(sigmask(SIGALRM));

  if (lwp_create(&tid,threadMain,MINPRIO,LWPSUSPEND,stack,1,arg)==-1) {
    /* didn't create it successfully... print an error an exit */
    lwp_perror("lwp_create");
    PROCESS_exit(1);
  }  
  for (i=0; i<MAXTHREADS; ++i) {
    if (threadInfo[i].status!=STATUS_NOT_IN_USE) continue;
    /* current thread is suspended, enter its data in the scheduler
       data structures */
    threadInfo[i].tid=tid;
    threadInfo[i].status=STATUS_READY;
    threadInfo[i].joinee=SELF;
    threadInfo[i].priority=prio;
    ok=1;
#ifdef SCHEDULER_DEBUG
    fprintf(stdout,"THREAD_fork: new thread is %x (priority %d)\n",
	    tid.thread_id,prio);
#endif
    break;
  }
  if (!ok) {
    fprintf(stderr,"THREAD_fork: unable to allocate another thread: thread maximum (%d) reached\n",MAXTHREADS);
    PROCESS_exit(1);
  }
  /* restore signals */
  sigsetmask(mask);

  return tid;
}

void getCurrentHostname(char *hostbuffer,unsigned int length) 
{
  char *ptr;
  if (gethostname(hostbuffer,length-1)==-1) {
    perror("gethostname");
    PROCESS_exit(1);
  }
  hostbuffer[length-1]-'\0';
  if ((ptr=strchr(hostbuffer,'.'))!=NULL) {
    *ptr='\0';
  }
}

void PROCESS_exit(int retval)
{
  pod_exit(retval);
}

static void timeBetween(struct timeb *now, unsigned long futureSecs, /*:-)*/
			unsigned long futureMilliSecs,
			long *secs, long *usecs)
{
  /* lets handle the past the deadline cases first */
  /* loser on seconds */
  if (now->time>futureSecs) {
   *secs=0;
   *usecs=0;
    return;
  }
  /* loser on millisecs */
  if ((now->time==futureSecs) && (now->millitm>futureMilliSecs)) {
    *secs=0;
    *usecs=0;
    return;
  }
  /* do second computation ... always positive or zero */
  *secs=futureSecs-now->time;
  /* do microsecond computatation ...might be negative*/
  *usecs=(futureMilliSecs-now->millitm)*1000;
  if (*usecs<0) {
    /* compensate by borrowing */
    *usecs=-(*usecs);
    (*secs)--; /* we know this is still positive because of the checks 
	       for deadline missed above */
  }
  return;
}

/* this function is the one called by sigalrm and rolls people off
   the processor */
static void roller(int notUsed)
{
  thread_t tid;

  if (lwp_self(&tid)==-1) {
    lwp_perror("lwp_self");
    PROCESS_exit(1);
  }
  if (SAMETHREAD(tid,schedulerTid)) {
    fprintf(stderr,"ERROR: not rolling scheduler off the processor!\n");
    fflush(stderr);
  } else {
#ifdef SCHEDULER_DEBUG
    fprintf(stdout,"SCHED: rolling someone off the processor\n");
    fflush(stdout);
#endif
    THREAD_yield();
  }


}

static void bogusHandler(int notUsed)
{
#ifdef SCHEDULER_DEBUG
  fprintf(stdout,"SCHEDULER: found a signal pending\n");
  fflush(stdout);
#endif
}

/*
 * SCHEDULER
 */

static void schedulerMain(void *notUsed)
{
  int i,j,ret,largest,*numReady,prio,tmpReady,allDead,currentMask,waiting;
  struct timeval blockTime,tmp;
  struct timeb now,future;
  struct fd_set fdset;
  threadReadyStruct **readyQueue;
  threadInfoStruct *readyThread;
  sigset_t myset;
  unsigned leftOnClock;

  readyQueue=(threadReadyStruct **)
    malloc(sizeof(threadReadyStruct *)*THREAD_NUMBER_PRIOS);
  numReady=(int *)malloc(sizeof(int)*THREAD_NUMBER_PRIOS);
  for (i=0; i<THREAD_NUMBER_PRIOS; ++i) {
    readyQueue[i]=(threadReadyStruct *)
      malloc(sizeof(threadReadyStruct)*MAXTHREADS);
    numReady[i]=0;
  }

  /* loop forever*/
  while (1) {
    /* see if all threads are dead */
    allDead=1;
    for (i=0; i<MAXTHREADS; ++i) {
      if ((threadInfo[i].status!=STATUS_EXITED) &&
	  (threadInfo[i].status!=STATUS_NOT_IN_USE)) {
	allDead=0;
	break;
      }
    }
    if (allDead) PROCESS_exit(0);

    /* zero the ready queues */
    for (i=0; i<THREAD_NUMBER_PRIOS;++i)
      numReady[i]=0;
    
    /* compute the time to block... we do this by looking at the ready times
       of all the threads and subtracting (more or less) from 'now'.  We make
       the time zero if there are yielded (runable) threads or if there are
       thread ready times that have already past. */
    /* initialize to a large value */
    blockTime.tv_sec=1000000;
    blockTime.tv_usec=0;
    /* sample the clock */
    if (ftime(&now)==-1) {
      perror("ftime");
      PROCESS_exit(1);
    }
    /* loop through looking for smallest value */
    for (i=0; i<MAXTHREADS; ++i) {
      /* avoid bogus ones */
      if (threadInfo[i].status==STATUS_NOT_IN_USE) continue;
      /* if we are yielding... just make it a poll and get out */
      if (threadInfo[i].status==STATUS_READY) {
	blockTime.tv_sec=0;
	blockTime.tv_usec=0;
	break;
      } else {
	/* might not be of interest to time related stuff */
	if ((threadInfo[i].status!=STATUS_SLEEPING) && 
	    (threadInfo[i].status!=STATUS_BLOCKED_IO)) continue;
	/* might be a thread blocked indefinately */
	if (threadInfo[i].secs==0) continue;
	/* compute time diff */
	timeBetween(&now,threadInfo[i].secs,threadInfo[i].millisecs,
		    &tmp.tv_sec,&tmp.tv_usec);
	/* is it smallest yet? */
	if (tmp.tv_sec<blockTime.tv_sec) {
	  blockTime.tv_sec=tmp.tv_sec;
	  blockTime.tv_usec=tmp.tv_usec;
	} else {
	  /* might be smaller just in micro */
	  if ((tmp.tv_sec==blockTime.tv_sec)
	      && (tmp.tv_usec<blockTime.tv_usec)) {
	    blockTime.tv_usec=tmp.tv_usec;
	  }
	}
	/* if we are doing a poll, it won't get smaller */
	if ((blockTime.tv_sec==0) && (blockTime.tv_usec==0)) break;
      }
    }
    /* initalize for this pass over fdset*/
    largest=-1;
    /* ok, we now know how long to block for build the file descriptor
       sets of interest */
    FD_ZERO(&fdset);
    /* go through looking for blocked threads and copying their descriptors */
#ifdef SCHEDULER_DEBUG
    fprintf(stdout,"SCHED: block time is (%d,%d)\n",
	    blockTime.tv_sec,blockTime.tv_usec);
#endif
    for (i=0; i<MAXTHREADS;++i) {
      if (threadInfo[i].status==STATUS_NOT_IN_USE) continue;

      if (threadInfo[i].status==STATUS_BLOCKED_IO) {
	/* its blocked on I/O */
	for (j=0; j<threadInfo[i].numFds; ++j) { /* copy them */
	  FD_SET(threadInfo[i].fd[j],&fdset);
	  /* keep track of largest one we've seen */
	  if (threadInfo[i].fd[j]>largest) largest=threadInfo[i].fd[j];
	}
      }
    }
    /* do the block */
    ret=select(largest+1,&fdset,NULL,NULL,&blockTime);
#ifdef SCHEDULER_DEBUG
    if (ret>0) {
      fprintf(stdout,"SCHED: select says data is ready (%d)\n",ret);
    } else {
      if (ret==0) {
	fprintf(stdout,"SCHED: select timed out\n");
      } else {
	fprintf(stderr,"SCHED: select indicates error: %d\n",
		errno);
      }
    }
#endif
    if (ret==-1) {
      /* error */
      if (errno!=EINTR) {
	perror("select");
	PROCESS_exit(1);
      } else {
	/* we'll assume this is a signal or some such thing and just
	 implement a timeout*/
	ret=0;
      }
    }
    if (ret>0) {
      /* data is ready */
      /* go through looking for blocked threads and marking their descriptors*/
      for (i=0; i<MAXTHREADS;++i) {
	if (threadInfo[i].status==STATUS_NOT_IN_USE) continue;

	if (threadInfo[i].status==STATUS_BLOCKED_IO) {
	  /* its blocked on I/O */
	  for (j=0; j<threadInfo[i].numFds; ++j) { /* mark em */
	    if (FD_ISSET(threadInfo[i].fd[j],&fdset)) {
#ifdef SCHEDULER_DEBUG
	      fprintf(stdout,"SCHED: thread %x has data ready on fd %d\n",
		      threadInfo[i].tid.thread_id,
		      threadInfo[i].fd[j]);
#endif
	      prio=threadInfo[i].priority;
	      readyQueue[prio][numReady[prio]].tid=threadInfo[i].tid;
	      /* This is to make sure we don't end up putting them
		 on the ready queue again if their timeout expired */
	      threadInfo[i].status=STATUS_ALREADY_SCHEDULED;
	      readyQueue[prio][numReady[prio]].status=READY_DATA;
	      numReady[prio]++;
	      break;
	    }
	  }
	}
      }
    }
    if (ret!=-1) {
      /* if there was no error (data ready or timeout) we need to look for
	 threads whose deadline has past  or are generally ready to run*/
      /* sample the clock */
      if (ftime(&now)==-1) {
	perror("ftime");
	PROCESS_exit(1);
      }
      /* go through comparing now to their finish time */
      for (i=0; i<MAXTHREADS; ++i) {
	if (threadInfo[i].status==STATUS_NOT_IN_USE) continue;
	/* yielded threads are automatically ready */
	if (threadInfo[i].status==STATUS_READY) {
#ifdef SCHEDULER_DEBUG
	  fprintf(stdout,"SCHED: thread %x is ready\n",
		  threadInfo[i].tid.thread_id);
#endif
	  prio=threadInfo[i].priority;
	  readyQueue[prio][numReady[prio]].tid=threadInfo[i].tid;
	  readyQueue[prio][numReady[prio]].status=READY_NORMAL;
	  numReady[prio]++;
	  continue;
	}
	if ((threadInfo[i].status!=STATUS_BLOCKED_IO) &&
	    (threadInfo[i].status!=STATUS_SLEEPING)) continue;

	/* if they are blocked indefinately and don't have data ready, 
	   skip them */
	if (threadInfo[i].secs==0) continue;

	/* otherwise, compute finish time and compare */
	timeBetween(&now,threadInfo[i].secs,threadInfo[i].millisecs,
		    &tmp.tv_sec,&tmp.tv_usec);
	if ((tmp.tv_sec==0) && (tmp.tv_usec==0)) {
#ifdef SCHEDULER_DEBUG
	  fprintf(stdout,"SCHED: thread %x has timed out\n",
		  threadInfo[i].tid.thread_id);
#endif
	  prio=threadInfo[i].priority;
	  readyQueue[prio][numReady[prio]].tid=threadInfo[i].tid;
	  readyQueue[prio][numReady[prio]].status=READY_TIME;
	  numReady[prio]++;
	  continue;
	}
      }
    }
    /* ok, we now know which threads are ready and why... */
    /* we need to go through the ready queues looking for threads 
       to wake up */
    for (prio=0; prio<THREAD_NUMBER_PRIOS; ++prio) {
      /* look to see if any are ready */
      if (numReady[prio]!=0) {
	/* found the right priority... start looking for threads */
	for (i=0; i<numReady[prio]; ++i) {
	  if (readyQueue[prio][i].status==READY_DATA) {
	    /* he's got data ready, so tell him which descriptors */
	    readyThread=findThread(readyQueue[prio][i].tid);
	    readyThread->status=READY_DATA;
	    /* look at all the descriptors */
	    for (j=0; j<readyThread->numFds; ++j) {
	      if (!FD_ISSET(readyThread->fd[j],&fdset)) {
		/* wipe out unready descriptors */
		readyThread->fd[j]=-1;
	      }
	    }
	  }
	  /* if its a timeout, tell him so */
	  if (readyQueue[prio][i].status==READY_TIME) {
	    readyThread=findThread(readyQueue[prio][i].tid);
	    readyThread->status=READY_TIME;
	  }
#ifdef SCHEDULER_DEBUG
	  fprintf(stdout,"SCHED: waking up thread %x (priority %d)\n",
		  readyQueue[prio][i].tid.thread_id,prio);
	  fflush(stdout);
#endif	  
	  /* resume execution */
	  if (lwp_resume(readyQueue[prio][i].tid)==-1) {
	    lwp_perror("lwp_resume");
	    PROCESS_exit(1);
	  }
	  /* check the current signal mask */
	  currentMask=sigsetmask(0);
#ifdef SCHEDULER_DEBUG
	  fprintf(stdout,"SCHED: current mask is %d\n",currentMask);
#endif
	  /* make sure sigmask doesn't include sigalarm */
	  currentMask&= ( ~ sigmask(SIGALRM) );
#ifdef SCHEDULER_DEBUG
	  fprintf(stdout,"SCHED: changed current mask to %d\n",currentMask);
#endif
	  /* reset it */
	  sigsetmask(currentMask);
	  /* ok, I'm about to give him the processor, make sure
	     I get woken up */
	  signal(SIGALRM,roller);
	  ualarm(threadTimeSlice[prio],0);
	  /* must yield to the thread so it can run... */
	  if (lwp_yield(SELF)==-1) {
	    lwp_perror("lwp_yield");
	    PROCESS_exit(1);
	  }
	  /* ok, there are two cases for getting here... with the
	     alarm going off and without ... lets try to turn off
	     the alarm*/
	  leftOnClock=ualarm(0,0);
	  if (leftOnClock==0) {
#ifdef SCHEDULER_DEBUG
	    fprintf(stdout,"SCHED: possible sigalarm pending: %d\n",
		    leftOnClock);
#endif
	    /* well, the alarm may have gone off... check that */
	    if (sigpending(&myset)==-1) {
	      perror("sigpending");
	      PROCESS_exit(1);
	    }
	    /* if there is one pending, handle it */
	    if (sigismember(&myset,SIGALRM)) {
#ifdef SCHEDULER_DEBUG
	      fprintf(stdout,"SCHED: sigalrm pending!\n");
#endif
	      signal(SIGALRM,bogusHandler);
	      sigpause(0); /* blocks until we handle it (throw it away) */
	    } else {
#ifdef SCHEDULER_DEBUG
	      fprintf(stdout,"SCHED: no sigalrm pending\n");
#endif
	    }
	  } else {
#ifdef SCHEDULER_DEBUG
	    fprintf(stdout,"SCHED: trapped the sigalarm before it went off\n");
#endif
	  }
	}
	/* there were threads ready at this priority, so stop looking
	   for prioritys and exit this loop */
	break;
      }
    }
    /* start this whole thing over again */
  }
}

void THREAD_init()
{
  int i;
  thread_t tid;
  stkalign_t*stack;

  /* get us a cache size of 10 stacks */
  lwp_setstkcache(MINSTACKSZ*sizeof(stkalign_t),10);
  stack=lwp_newstk();
  /* get us four priorities */
  if (pod_setmaxpri(MINPRIO+3)==-1) {
    lwp_perror("pod_setmaxpri");
    PROCESS_exit(1);
  }

  /* initialize the scheduler data structure for main */
  if (lwp_self(&tid)==-1) {
    lwp_perror("lwp_self");
    PROCESS_exit(1);
  }
  /* always put main at zero... it wants to run next time the scheduler
   does (at scheduler fork time) */
#ifdef SCHEDULER_DEBUG
  fprintf(stdout,"THREAD_init: main is thread %x\n",
	  tid.thread_id);
  fflush(stdout);
#endif
  threadInfo[0].tid=tid;
  threadInfo[0].status=STATUS_READY;
  threadInfo[0].joinee=SELF;

  /* initialize the other slots */
  for (i=1; i<MAXTHREADS; ++i) threadInfo[i].status=STATUS_NOT_IN_USE;

  /* create the scheduler thread ... can't use THREAD_fork because the
     scheduler doesn't want to be in its own data structure */
  if (lwp_create(&schedulerTid,schedulerMain,MINPRIO,0,stack,1,NULL)==-1) {
    /* didn't create it successfully... print an error an exit */
    lwp_perror("lwp_create");
    PROCESS_exit(1);
  }  
  /* initialize the scheduler by letting him run*/
  THREAD_yield();

}

int THREAD_block(int *infd,int *outfd,unsigned int size,int delay)
{
  int i,j,mask;
  thread_t tid;
  struct timeb now;
  threadInfoStruct *self;

  if (size>MAXBLOCKEDFDS) {
    fprintf(stderr,"THREAD_block: too many file descriptors! %d is the limit!\n",MAXBLOCKEDFDS);
    PROCESS_exit(1);
  }
  
  /* turn off signals */
  mask=sigblock(sigmask(SIGALRM));
  /* find ourselves... */
  if (lwp_self(&tid)==-1) {
    lwp_perror("lwp_self");
    PROCESS_exit(1);
  }
  self=findThread(tid);
  self->status=STATUS_BLOCKED_IO;
  if (delay==-1) self->secs=0;
  else {
    /* figure out when we are ready */
    if (ftime(&now)==-1) {
      perror("ftime");
      PROCESS_exit(1);
    }
    self->secs=(unsigned long)now.time;
    self->millisecs=(unsigned long)now.millitm;
    self->millisecs+=delay;
    if (self->millisecs>=1000) {
      /* move some time over to the seconds */
      self->secs+=self->millisecs/1000;
      self->millisecs%=1000;
    }
  }
  self->numFds=size;
  /* tell the what fds we are interested in */
  for (j=0; j<size;++j) {
    self->fd[j]=infd[j];
  }
#ifdef SCHEDULER_DEBUG
  fprintf(stdout,"SCHED: thread %x blocking on I/O: (",tid.thread_id);
  for (j=0; j<size;++j) {
    fprintf(stdout,"%d",self->fd[j]);
    if (j!=size-1) fprintf(stdout,",");
  }
  fprintf(stdout,")\n");
#endif

  /* we can now suspend ourselves... our data structure will be
     updated when we wake up */
  if (lwp_suspend(SELF)==-1) {
    lwp_perror("lwp_suspend");
    PROCESS_exit(1);
  }
  /* check to see why we returned */
  if (self->whyWokenUp==READY_TIME) {
    /* turn signals back on */
    sigsetmask(mask);
    /* scheduler says we are timed out */
    return -2;
  } else {
    /* must be data ready to read */
    /* copy the out parameter */
    if (outfd) {
      for (j=0; j<size; ++j) outfd[j]=self->fd[j];
    }
    /* turn signals back on */
    sigsetmask(mask);
    return 0;
  }
}

void THREAD_sleep(unsigned int s) 
{ 
  int i,mask;
  thread_t tid;
  struct timeb now;
  threadInfoStruct *self;

  /* turn off signals */
  mask=sigblock(sigmask(SIGALRM));
  
  if (lwp_self(&tid)==-1) {
    lwp_perror("lwp_self");
    PROCESS_exit(1);
  }

  self=findThread(tid);

  self->status=STATUS_SLEEPING;
  /* enter our wakeup time */
  if (ftime(&now)==-1) {
    perror("ftime");
    PROCESS_exit(1);
  }
  self->secs=(unsigned long)now.time;
  self->millisecs=(unsigned long)now.millitm;
  self->secs+=s;
  /* give up */
#ifdef SCHEDULER_DEBUG
  fprintf(stdout,"SCHED: thread %x going to sleep\n",
	  tid.thread_id);
#endif
  if (lwp_suspend(SELF)) {
    lwp_perror("lwp_yield");
    PROCESS_exit(1);
  }
  /* turn on signals */
  sigsetmask(mask);
  return;
}

void THREAD_yield()
{
  int i,mask;
  thread_t tid;
  threadInfoStruct *self;

  /* turn off signals */
  mask=sigblock(sigmask(SIGALRM));

  if (lwp_self(&tid)==-1) {
    lwp_perror("lwp_self");
    PROCESS_exit(1);
  }
  self=findThread(tid);
  self->status=STATUS_READY;

#ifdef SCHEDULER_DEBUG
  fprintf(stdout,"SCHED: thread %x yielding\n",
	  tid.thread_id);
#endif
  if (lwp_suspend(SELF)) {
    lwp_perror("lwp_suspend");
    PROCESS_exit(1);
  }
  /* turn on signals */
  sigsetmask(mask);

}

static threadInfoStruct *findThread(thread_t t)
{
  int i;

  for (i=0; i<MAXTHREADS; ++i) {
    if (threadInfo[i].status==STATUS_NOT_IN_USE) continue;

    if (SAMETHREAD(threadInfo[i].tid,t)) {
      return &(threadInfo[i]);
    }
  }
  fprintf(stderr,"thread error: bad thread id: %x\n",t.thread_id);
  PROCESS_exit(1);
}

void THREAD_suspend(THREAD_T t)
{
  thread_t tid;
  threadInfoStruct *target;
  int mask;

  mask=sigblock(sigmask(SIGALRM));

  target=findThread(t);
  /* tell the scheduler whats up */
  target->status=STATUS_SUSPENDED;

  /* give up */
  if (lwp_suspend(t)) {
    lwp_perror("lwp_suspend");
    PROCESS_exit(1);
  }
  /* restore signals */
  sigsetmask(mask);
}

void THREAD_resume(THREAD_T t)
{
  thread_t tid;
  threadInfoStruct *target;
  int mask;

  /* stop signals */
  mask=sigblock(sigmask(SIGALRM));
  /* just find it and mark it */
  target=findThread(t);
  target->status=STATUS_READY;

  /* resume signals */
  sigsetmask(mask);
}

int THREAD_join(THREAD_T t)
{
  threadInfoStruct *target,*self;
  thread_t tid;
  int mask;

  /* stop signals */
  mask=sigblock(sigmask(SIGALRM));
  
  target=findThread(t);
  /* is the target already done?*/
  if (target->status==STATUS_EXITED) return target->exitStatus;

  /* nope so find us */
  if (lwp_self(&tid)==-1) {
    lwp_perror("lwp_self");
    PROCESS_exit(1);
  }
  /* sanity check */
  if (SAMETHREAD(tid,t)) {
    fprintf(stderr,"thread error: thread %x tried to join itself!\n",
	    t.thread_id);
    PROCESS_exit(1);
  }
  self=findThread(tid);
  /* make sure there isn't a waiter yet */
  if (!SAMETHREAD(target->joinee,SELF)) {
    fprintf(stderr,"thread error: attempt to wait on thread %x which already has a thread joiner\n",target->tid.thread_id);
    PROCESS_exit(1);
  }
  /* tell the target we are waiting on him */
  target->joinee=tid;
  self->status=STATUS_JOINING;
#ifdef SCHEDULER_DEBUG
  fprintf(stdout,"SCHED: thread %x waiting to join thread %x\n",
	  tid.thread_id,t.thread_id);
#endif
  /* block until he exits */
  if (lwp_suspend(SELF)) {
    lwp_perror("lwp_suspend");
    PROCESS_exit(1);
  }
  /* reclaim his space */
  target->status=STATUS_NOT_IN_USE;

  /* resums signals */
  sigsetmask(mask);
  
  return target->exitStatus;
}

void THREAD_exit(int code)
{
  threadInfoStruct *self,*target;
  thread_t tid;
  int mask;

  mask=sigblock(sigmask(SIGALRM));

  if (lwp_self(&tid)==-1) {
    lwp_perror("lwp_self");
    PROCESS_exit(1);
  }
  /* find ourselves */
  self=findThread(tid);
  self->status=STATUS_EXITED;
  /* see if someone is waiting */
  if (!SAMETHREAD(self->joinee,SELF)) {
    /* tell the scheduler he is ready */
    target=findThread(self->joinee);
    target->status=STATUS_READY;
  }
  /* set up our return code in case someone wants it later... */
  self->exitStatus=code;
#ifdef SCHEDULER_DEBUG
  fprintf(stdout,"SCHED: thread %x dying\n",
	  tid.thread_id);
#endif
  /* ugh, gag */
  lwp_destroy(SELF);
  
  /* shouldn't get here */
  fprintf(stderr,"INTERNAL ERROR: EXECUTING PAST END OF THREAD_exit()\n");
  PROCESS_exit(1);
}

THREAD_T THREAD_self()
{
  thread_t tid;
  int mask;

  /* turn off signals */
  mask=sigblock(sigmask(SIGALRM));
  
  if (lwp_self(&tid)==-1) {
    lwp_perror("lwp_self");
    PROCESS_exit(1);
  }

  /* put on signals */
  sigsetmask(mask);

  return tid;
}
int THREAD_equality(THREAD_T t1,THREAD_T t2)
{
  return (SAMETHREAD(t1,t2));
}

void KEYBOARD_nonBlocking()
{
  struct termios modified;
  int mask;

  mask=sigblock(sigmask(SIGALRM));

  ioctl(0,TCGETS,&originalTermios);
  modified=originalTermios;
  modified.c_lflag=modified.c_lflag & 0xfffffffd;
  modified.c_cc[VMIN]=1;
  modified.c_cc[VTIME]=0;
  ioctl(0,TCSETS,&modified);

  sigsetmask(mask);
}

void KEYBOARD_readChar(char *c)
{
  int zero=0;
  THREAD_block(&zero,NULL,1,-1);
  read(0,c,1);
}

void KEYBOARD_blocking()
{
  int mask;
  
  mask=sigblock(sigmask(SIGALRM));

  /* restore previous settings */
  ioctl(0,TCSETS,&originalTermios);

  sigsetmask(mask);
}

#endif
#ifdef ALPHA
#endif

/* Open a UDP domain socket */
int NETWORK_open(char *mcastAddress,unsigned short port,
		 struct sockaddr_in *addr_ret,
		 int ttl,int wantLoopback)
{
  int fd;
  /* we are not going to deal with the possibility of a machine with
     multiple physical interfaces that wants to use the non-default
     interface (router) */
  
  /* set up our sending address */
  init_sockaddr(addr_ret,resolve_host(mcastAddress),port);

  /* set up our udp socket */
  fd=sock_udpcreate(*addr_ret);
  if (fd==-1) {
    perror("sock_udpcreate");
    PROCESS_exit(1);
  }
  
  if (IN_MULTICAST(resolve_host(mcastAddress))) {
    /* set the ttl */
    if (sock_setmultittl(fd,ttl)==-1) {
      perror("sock_setmultittl");
      PROCESS_exit(1);
    }
    /* if we want's loopback we are done */
    if (wantLoopback) return fd;
    
    /* no loopback*/
    if (sock_setmultiloopback(fd,0)<0) {
      perror("sock_setmultiloopback");
      PROCESS_exit(1);
    }
  }
  return fd;
}

int NETWORK_send(int fd, char *buf,unsigned int len,
		 struct sockaddr_in *sockaddr)
{
#ifdef SUNOS
  THREAD_yield();
#endif
  return (sock_sendto(fd,buf,len,sockaddr));
}

int NETWORK_receive(int fd, char *buf,unsigned int len)
{
#ifdef SUNOS
  THREAD_block(&fd,NULL,1,-1);
#endif
  return (sock_recv(fd,buf,len));
}

/*
 * This is the comparison func for searching
 */
static int searchFunc(const void *elem1,const void *elem2)
{
  linearRange *lr1=(linearRange *)elem1;
  linearRange *lr2=(linearRange *)elem2;

  if (lr1->ulaw==255) {
    /* key is lr1 ... where is it?*/
    if (lr1->lowval < lr2->lowval) return -1;
    if (lr1->lowval > lr2->highval) return 1;
    return 0;
  } else {
    /* key is lr2 */
    if (lr2->lowval < lr1->lowval) return -1;
    if (lr2->lowval > lr1->highval) return 1;
    return 0;
  }
}

void AUDIO_16to8bit(short *shortBuf,unsigned char *mulawBuf,
		   unsigned int length)
{
  int i;
  linearRange r,*res;
  r.ulaw=255; /* magic value... doesn't appear in table so we know which
		 one is the search key */

  /* convert to 8 bit ulaw */
  for (i=0; i<length; ++i) {
    r.lowval=shortBuf[i];
    res=bsearch(&r,linearValue,255,sizeof(linearRange),searchFunc);
    if (!res) {
      fprintf(stderr,"bsearch couldn't find %d\n",r.lowval);
      PROCESS_exit(1);
    }
    mulawBuf[i]=res->ulaw;
  }
}

void AUDIO_8to16bit(unsigned char *mulawBuf,short *shortBuf,
		    unsigned int length)
{
  int i;
  /* need a conversion to 16 bit samps */
  for (i=0; i<length; ++i) {
    shortBuf[i]=ulawValues[(int)mulawBuf[i]];
  }
}

void NETWORK_close(int fd)
{
  close(fd);
}

void NETWORK_initSockaddr(struct sockaddr_in *address,
			  char *target, unsigned short port)
{
  init_sockaddr(address,resolve_host(target),port);
}


#ifdef USE_AUDIOFILE
AFSetACAttributes attributes; /* like GC Set Attributes */
AFAudioConn *aud;
MUTEX_T audio_lock;

AUDIO_T AUDIO_open()
{
    AUDIO_T ac;

    MUTEX_init(&audio_lock);
    MUTEX_lock(&audio_lock);

    printf("beginning init\n");
    attributes.preempt = Mix;
    attributes.start_timeout = 0;
    attributes.end_silence = 0;
    attributes.play_gain = 0;
    attributes.rec_gain =  0;
    attributes.type = MU255;
    attributes.channels=1;

    if ( (aud=AFOpenAudioConn(""))==NULL) {
	fprintf(stderr,"Couldn't open the connection to AudioFile\n");
	PROCESS_exit(1);
    }

    /* 
     * I always connect to device 0... this is probably XXX
     */
    ac = AFCreateAC(aud, 0, (ACPlayGain  | ACEncodingType | ACChannels |
			     ACRecordGain | ACPreemption), 
		    &attributes);
    AFSync(aud, AFalse);	/* Make sure we confirm encoding type support. */
    AFFlush(aud);
    printf("end init... %x match with %x\n",ac->connection,ac);
    
    MUTEX_unlock(&audio_lock);
    return ac;
}

void AUDIO_close( AUDIO_T *ac)
{
    AFAudioConn *aud=(*ac)->connection;
    
    fprintf(stderr,"closing\n");
    MUTEX_lock(&audio_lock);
    AFFreeAC((*ac));
    AFCloseAudioConn(aud);
    *ac=NULL;
    MUTEX_unlock(&audio_lock);
    fprintf(stderr,"closing--2\n");
}

int AUDIO_valid(AUDIO_T ac)
{
    if (!ac->connection) return 0;
    MUTEX_lock(&audio_lock);
    printf("TESTING VALIDITY\n");
    AFSync(ac->connection,AFalse); /* this will crap out if the connect is bad */
    AFFlush(aud);
    MUTEX_unlock(&audio_lock);
    printf(" VALIDITY OK\n");
    fflush(stdout);
    return 1; 
}

int AUDIO_play(AUDIO_T ac,unsigned char *buffer,unsigned int length)
{
    ATime t;

    MUTEX_lock(&audio_lock);
    t=AFGetTime(ac);
    AFPlaySamples(ac,t,(int)length,buffer);
    AFFlush(aud);
    MUTEX_unlock(&audio_lock);
    return 0;
}
int AUDIO_record(AUDIO_T ac,unsigned char *buffer,unsigned int length)
{
    ATime t;

    printf("here 0 ... %x\n",ac->connection);
    if (AUDIO_valid(ac)) {
	printf("audio is valid\n");
    }
    MUTEX_lock(&audio_lock);
    AFSync(ac->connection,AFalse); 
    printf("here1\n");
    t=AFGetTime(ac);
    printf("here2\n");
    AFRecordSamples(ac,t,length,buffer,ABlock);
    AFFlush(aud);
    printf("here3\n");
    MUTEX_unlock(&audio_lock);
    return 0;
}

#endif /* USE AUDIOFILE */
