/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/
#ifndef OS_INTERFACE_H
#define OS_INTERFACE_H

/* 
 * System dependant includes 
 */

/* stuff that's the same everywhere */
#include <sys/types.h>
#include <netinet/in.h>

/*
 * These define the types for the mutex and condition vars
 */

#ifdef SOLARIS
#include <synch.h>
#include <thread.h>

#define MUTEX_T mutex_t
#define COND_T cond_t
#ifndef USE_AUDIOFILE
#define AUDIO_T int *
#endif

#define THREAD_T thread_t
#define FAKE_BCOPY
#endif

#ifdef SUNOS
#include <lwp/lwp.h>
#define MUTEX_T mon_t
#define COND_T cv_t 
#ifndef USE_AUDIOFILE
#define AUDIO_T int *
#endif
#define THREAD_T thread_t
#endif 

#ifdef IRIX4
#include <ulocks.h>
#include <audio.h>

#define MUTEX_T ulock_t
#define COND_T usema_t *
#ifndef USE_AUDIOFILE
#define AUDIO_T ALport *
#endif
#define THREAD_T pid_t
#define FAKE_BCOPY
#endif 

#ifdef ALPHA
#include <pthread.h>
#define MUTEX_T pthread_mutex_t
#define COND_T pthread_cond_t
#ifndef USE_AUDIOFILE
#define AUDIO_T int *
#endif
#endif

#ifdef USE_AUDIOFILE
#include <AF/audio.h>
#include <AF/AFlib.h>
#include <AF/AFUtils.h>
#define AUDIO_T AC
#endif /* USE_AUDIOFILE */


#define THREAD_LOW_PRIORITY 2
#define THREAD_MEDIUM_PRIORITY 1
#define THREAD_HIGH_PRIORITY 0

#define THREAD_DEFAULT_PRIORITY 0
#define THREAD_NUMBER_PRIOS 3

THREAD_T THREAD_fork(void (*threadMain)(void *),void *arg, int priority);
void THREAD_yield();
void THREAD_sleep(unsigned int);
int THREAD_block(int *infd,int *outfd,unsigned int size,int delay);
void THREAD_init();

int THREAD_join(THREAD_T);
void THREAD_resume(THREAD_T);
void THREAD_suspend(THREAD_T);
int THREAD_equality(THREAD_T, THREAD_T);
void THREAD_exit(int);
#ifdef SOLARIS
int THREAD_kill(THREAD_T);
#endif
THREAD_T THREAD_self();

#define THREAD_MILLI_SLEEP(x) THREAD_block(NULL,NULL,0,x)

void PROCESS_exit(int);

void MUTEX_init(MUTEX_T *);
void MUTEX_lock(MUTEX_T *);
void MUTEX_unlock(MUTEX_T *);

void COND_init(COND_T *,MUTEX_T *);
void COND_signal(COND_T *);
void COND_wait(COND_T *,MUTEX_T *);

int NETWORK_open(char *,unsigned short,struct sockaddr_in *,int,int);
int NETWORK_send(int, char *,unsigned int,struct sockaddr_in *);
int NETWORK_receive(int, char *,unsigned int);
void NETWORK_close(int);
void NETWORK_initSockaddr(struct sockaddr_in *,char *,unsigned short);

void getCurrentHostname(char *,unsigned int);

AUDIO_T AUDIO_open();
void AUDIO_close(AUDIO_T *);
int AUDIO_play(AUDIO_T,unsigned char *,unsigned int length);
int AUDIO_record(AUDIO_T,unsigned char *,unsigned int length);
int AUDIO_valid(AUDIO_T);
void AUDIO_8to16bit(unsigned char *,short *,unsigned int);
void AUDIO_16to8bit(short *,unsigned char *,unsigned int);

void KEYBOARD_nonBlocking();
void KEYBOARD_readChar(char *);
void KEYBOARD_blocking();

#ifdef FAKE_BCOPY
/* for solaris or other sys v machines*/
#ifndef bcopy
#define bcopy(x,y,z) memcpy(y,x,(size_t)z)
#endif
#ifndef bzero
#define bzero(x,y) memset(x,0,y)
#endif
#endif

#endif /* os-interface.h */
