/*
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
//
 * Shoot a client message at a window for debugging...
 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

int
main(int argc, char *argv[])
{
    Display *dpy = NULL;
    XClientMessageEvent cme;
    long windowid;
    Window clientWindow;
    XTextProperty prop;

    if (argc != 2) {
	fprintf(stderr, "usage: %s: window (in hex)\n", argv[0]);
	exit(1);
    }

    windowid = strtol(argv[1], (char **) NULL, 16);

    if ((dpy = XOpenDisplay(NULL)) == NULL) {
	fprintf(stderr, "%s: couldn't open display\n", argv[0]);
	exit(1);
    }

    clientWindow = XmuClientWindow(dpy, windowid);

    fprintf(stderr, "%s: client window is %lx\n", argv[0], clientWindow);

    if (XGetWMName(dpy, windowid, &prop)) {
	fprintf(stderr, "%s: WM_NAME property is %s\n", argv[0], prop.value);	
    } else {
	fprintf(stderr, "%s: No WM_NAME property on client window\n", argv[0]);
    }

    cme.window = windowid;
    cme.type = ClientMessage;
    cme.message_type = 0;
    cme.format = 0;
    cme.data.l[0] = 0L;
    cme.data.l[1] = 1L;
    cme.data.l[2] = 2L;
    cme.data.l[3] = 3L;
    cme.data.l[4] = 4L;

    fprintf(stderr, "%s: sending client message to window %lx\n", 
	    argv[0], windowid);

    if (!XSendEvent(dpy, windowid, 1, 0, (XEvent *) &cme)) {
	fprintf(stderr, "%s: XSendEvent failed.\n", argv[0]);
	exit(1);
    }

    XFlush(dpy);
}

