##
##
##
##	Note:  The use of this program shall be restricted to educational,
##	non-commercial use only.  All inquiries regarding licensing this
##	software for commercial use may be addressed to:
##	
##	Georgia Tech Research Corporation
##	Technology Licensing
##	Centennial Research Building - Rm. 275
##	400 Tenth Street, N.W.
##	Atlanta, Georgia  30332-0415
##	
##	*************************************************
##	
##	       Copyright 1996
##	Georgia Tech Research Corporation
##	    All Rights Reserved
##
##
##
##

#
#	These are procedures which are called from template code.
#

Error EL_DEBUG templateprocs "Sourcing templateprocs.tcl"

###
### Return whether or not a text area is editable, used for XmText
### (looks for the "editable" resource).
###
proc XmTextEditable {type node} {
    if {[wtemplate $node allowTextMode] == "FALSE"} {
	puts "XmTextEditable:  text not editable."
	return "FALSE"
    } else {
	if {[resource $node get editable] == "TRUE"} {
	    puts "XmTextEditable: text is editable."
	    return "TRUE"
	} else {
	    puts "XmTextEditable: editable resource doesn't exist."
	    return "FALSE"
	}
    }
}

###
### Determines whether an Athena text area is editable.
###
proc AthenaTextEditable {type node} {
    return [wtemplate $node sensitive]
}

###
### Apply a muffle function based on the number of children.  This is
### used by Forms and Boxes currently.
###
proc NumChildrenMuffle {type node} {
    set muffle [expr round (30 - 1.0/[numChildren $node] * 200)]
    if {$muffle < -80} {
	set muffle -80
    }
    #set muffle [expr (-70 + [numChildren $node] * 5)]
    Error EL_DEBUG templateprocs "muffle is  $muffle"
    return $muffle
}

###
### Apply a muffle function based on our position relative to other children
### of our parent.  Used by Commands.
###
proc PositionMuffle {type node} {
    set total [expr [numChildren [modelManager parent $node]] * 1.0]
    set muffle [expr round((([GetPosition $node] / $total) - .5) * 100 - 20)]
    return $muffle
}

###
### Apply a muffle function based on the height of the widget.  This is used
### by Text widgets.
###
proc HeightMuffle {type node} {
    set muffle [expr ([resource $node get height] - 200) / 3]
    return $muffle
}

###
### DefaultInfoSpeaker is used to present spoken information about the 
### current object. This is used to speak application names.  
###
proc DefaultInfoSpeaker {type node} {
    set class [modelManager class $node]
    return $class
}


###
### AthenaInfoSpeaker is used to present spoken information about the 
### current object. This should be installed on every widget.
###
proc AthenaInfoSpeaker {type node} {
    set class [modelManager class $node]
    if {$class == "Form" || $class == "Box"} {
      return "Container"
     } else {
       if {$class == "Paned"} {
         return "Window"
       } else {
        if {$class == "Command"} {
           return "Push Button"
        } else {
          if {$class == "Toggle"} {
             return "Toggle"
          } else {
            if {$class == "ApplicationShell"} {
             return "Pop up dialogue"
            } else {
	     return $class
            }
          }
        }
      } 
    }
}

###
### AthenaLabelSpeaker is used to retrieve the label resource for speaking.
### If this resource is not set, it returns a notification.  This should
### be installed on any widget that should have its label spoken when
### entered.
###
proc AthenaLabelSpeaker {type node} {
    set foo [resource $node get label]
    if { $foo == "NULL" } {
	return "Empty label on object"
    } else {
	return $foo
    }
}

###
### MotifLabelSpeaker does the same for the Motif widget sets... Motif
### widgets use the labelString resource instead of label.
###
proc MotifLabelSpeaker {type node} {
    set foo [resource $node get labelString]
    if { $foo == "<not found>" || $foo == "" } {
#	return "Empty label on object"
	return [modelManager name $node]
    } else {
	return $foo
    }
}

###
### MotifInfoSpeaker is used to present spoken information about the 
### current object. This should be installed on every widget.
###
proc MotifInfoSpeaker {type node} {
    set class [modelManager class $node]

    if {$class == "XmArrowButton" || $class == "XmArrowButtonGadget"} {
      return "Arrow Button"
    } else {
    if {$class == "XmCascadeButton"} {
      return "Cascade Button"
    } else {
    if {$class == "XmCommand"} {
      return "Command"
    } else {
    if {$class == "XmDialogShell"} {
      return "Pop up dialogue"
    } else {
    if {$class == "XmDrawnButton"  || $class == "XmPushButtonGadget" || $class == "XmPushButton"} {
      return "Push Button"
    } else {
    if {$class == "XmFileSelectionBox"} {
      return "File Selection Box"
    } else {
    if {$class == "XmGadget"} {
      return "Gadget"
    } else {
    if {$class == "XmLabel" || $class == "XmLabelGadget"} {
      return "Arrow Button"
    } else {
    if {$class == "XmList"} {
      return "List"
    } else {
    if {$class == "XmMainWindow"} {
      return "Main Window"
    } else {
    if {$class == "XmMenuShell"} {
      return "Menu"
    } else {
    if {$class == "Paned"} {
      return "Window"
    } else {
    if {$class == "XmScale"} {
      return "Slider"
    } else {
    if {$class == "XmScrollBar"} {
      return "Slider"
    } else {
    if {$class == "XmScrolledWindow"} {
      return "Scrolled Window"
    } else {
    if {$class == "XmSelectionBox"} {
      return "Selection Box"
    } else {
    if {$class == "XmSeparator" || $class == "XmSeparatorGadget"} {
      return "Selection Box"
    } else {
    if {$class == "XmToggleButton" || $class == "XmToggleButtonGadget"} {
      set parent [modelManager parent $node]
      if {[resource $parent get radioBehavior] == "TRUE"} {
        return "Toggle Button"
      } else {
        return "Check Box"
      }
    } else {
    if {$class == "XmText"} {
      return "Text"
    } else {
    if {$class == "XmTextField"} {
      return "Text Field"
    } else {
      return $class
    } else {
    if {$class == "XmRowColumn" || $class == "XmBulletinBoard" || $class == "XmForm" || $class == "XmDrawingArea" || $class == "XmFrame" || $class == XMManager" || $class == XmMessageBox"} {
      return "Container"
    } } } } } } } } } } } } } } } } } } } } }
}
###
### These are used by Athena text widgets to return their margin widths.
###
proc AthenaTextLeftMargin {type node} {
    set foo [resource $node get leftMargin]
    if { $foo == "NULL" }  {
	return "2"
    } else {
	return $foo
    }
}

proc AthenaTextRightMargin {type node} {
    set foo [resource $node get rightMargin]
    if { $foo == "NULL" }  {
	return "2"
    } else {
	return $foo
    }
}

proc AthenaTextTopMargin {type node} {
    set foo [resource $node get topMargin]
    if { $foo == "NULL" }  {
	return "2"
    } else {
	return $foo
    }
}

proc AthenaTextBottomMargin {type node} {
    set foo [resource $node get bottomMargin]
    if { $foo == "NULL" }  {
	return "2"
    } else {
	return $foo
    }
}

###
### These are used by Motif text widgets to return their margins.
### marginHeight is used for top and bottom, marginWidth is used
### for left and right.
###
proc MotifTextTBMargin {type node} {
    set foo [resource $node get marginHeight]
    if { $foo == "NULL" } {
	return "5"
    } else {
	return $foo
    }
}

proc MotifTextLRMargin {type node} {
    set foo [resource $node get marginWidth]
    if { $foo == "NULL" } {
	return "5"
    } else {
	return $foo
    }
}

###
### The resources "sensitive" and "mappedWhenManaged" are defined by the
### Core widget class, so we can bet that just returning their values will
### work for ALL Xt-derived widget sets.  I put these in the templates for
### a couple of reasons:  (1) a general desire to keep all resource names
### out of the TCL and C++ code, and (2) possible future support for non-Xt
### toolkits
###
proc SensitiveProc {type node} {
    if { [resource $node get sensitive] == "TRUE" } {
	return "TRUE"
    } else {
	return "FALSE"
    }
}

proc MapWhenMgdProc {type node} {
    if { [resource $node get mappedWhenManaged] == "TRUE" } {
	return "TRUE"
    } else {
	return "FALSE"
    }
}
