
%{

/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/

void NewObjTemplate(int l, const char *name);
void NewClassTemplate(int l, const char *name);
void NewAppTemplate(int l, const char *name);

void EndTemplate(int l);

void ProcessBooleanField(int l, const char *id, unsigned char b);
void ProcessIntegerField(int l, const char *id, long i);
void ProcessStringField(int l, const char *id, const char *s);
void ProcessStringListField(int l, const char *id);
void ProcessProcField(int l, const char *id, const char *proc);

void TmplAddToStringList(int l, const char *s);
void TmplLexWarning(int l, const char *s);

%}

integer		[0-9]+
whitespace	[ \t]+
string		\"[^\"\n]*[\"\n]
identifier	[a-zA-Z][a-zA-Z0-9_\-\.]*
newline		\n
comment		#.*
true		TRUE
false		FALSE

%%

{whitespace}	;
{newline}	{	
			templateLineno++;
		}
{comment}	;
{integer}	{
			sscanf(yytext, "%d", &yylval.integer);
			return INTEGER;
		}
objectTemplate	{	return OBJ_TEMPLATE;	}
classTemplate	{	return CLASS_TEMPLATE;	}
appTemplate	{	return APP_TEMPLATE;	}
{true}		{
			yylval.bool = 1;
			return BOOLEAN;
		}
{false}		{
			yylval.bool = 0;
			return BOOLEAN;
		} 
{string}	{
			register int i;
			yylval.string = strdup(yytext);
			if (yylval.string[strlen(yylval.string) - 1] != '"') {
			    TmplLexWarning(templateLineno, 
				"unterminated string");
			}
			for (i=0 ; i<(int)strlen(yylval.string) ; i++) {
			    yylval.string[i] = yylval.string[i+1];
			}
			yylval.string[strlen(yylval.string) - 1] = '\0';
			return STRING;
		}
{identifier}	{
			yylval.identifier = strdup(yytext);
			return IDENTIFIER;
		}	
.		{ 	return yytext[0];	}

