
%{
/*
//
//
//	Note:  The use of this program shall be restricted to educational,
//	non-commercial use only.  All inquiries regarding licensing this
//	software for commercial use may be addressed to:
//	
//	Georgia Tech Research Corporation
//	Technology Licensing
//	Centennial Research Building - Rm. 275
//	400 Tenth Street, N.W.
//	Atlanta, Georgia  30332-0415
//	
//	*************************************************
//	
//	       Copyright 1996
//	Georgia Tech Research Corporation
//	    All Rights Reserved
//
//
//
*/
extern int templateLineno;
%}

%union {
    char	dummy;		/* for tokens that return no value */
    long	integer;	/* integer return */
    char	*string;	/* string return */
    char	*identifier;	/* identifier return */
    unsigned char bool;		/* boolean return */
}
	
%token <integer> 	INTEGER
%token <string>		STRING
%token <identifier>	IDENTIFIER
%token <bool>		BOOLEAN

%token <dummy> 		OBJ_TEMPLATE CLASS_TEMPLATE APP_TEMPLATE NEWLINE

%type <string>		stringlist

%%

start:	 	  templates
		;

templates:	  /* empty */
		| templates template
		;

template:	  template_hdr template_body
		  { EndTemplate(templateLineno); }
		;

template_hdr:	  OBJ_TEMPLATE IDENTIFIER
		  { NewObjTemplate(templateLineno, $2); free($2); }
		| CLASS_TEMPLATE IDENTIFIER
		  { NewClassTemplate(templateLineno, $2); free($2); }
		| APP_TEMPLATE IDENTIFIER
		  { NewAppTemplate(templateLineno, $2); free($2); }
		;

template_body:	  '{' fields '}'
		;

fields:		  /* empty */
		| fields field
		;

field:		  IDENTIFIER '=' BOOLEAN
		  { ProcessBooleanField(templateLineno, $1, $3); free($1); }
		| IDENTIFIER '=' INTEGER
		  { ProcessIntegerField(templateLineno, $1, $3); free($1); }
		| IDENTIFIER '=' STRING
		  { ProcessStringField(templateLineno, $1, $3); free($1); free($3); }
		| IDENTIFIER '=' '(' stringlist ')'
		  { ProcessStringListField(templateLineno, $1); free($1); }
		| IDENTIFIER '=' '(' ')'
		  { ProcessStringListField(templateLineno, $1); free($1); }
		| IDENTIFIER '=' '[' IDENTIFIER ']'
		  { ProcessProcField(templateLineno, $1, $4); free($1); free($4); }
		;

stringlist:	  stringlist ',' STRING
		  { TmplAddToStringList(templateLineno, $3); free($3); }
		| STRING
 		  { TmplAddToStringList(templateLineno, $1); free($1); }
		;


%%

#include "tmpllexer.c"

/*
 * Replacements for yywrap and yyerror.  These keep us from having to link
 * with libraries we really don't need.
 */

int
yywrap()
{
    return 1;
}

static int
yyerror(char *str)
{
    fprintf(stderr, "yyerror: %s\n", str);
    return 1;
}
