##
##
##
##	Note:  The use of this program shall be restricted to educational,
##	non-commercial use only.  All inquiries regarding licensing this
##	software for commercial use may be addressed to:
##	
##	Georgia Tech Research Corporation
##	Technology Licensing
##	Centennial Research Building - Rm. 275
##	400 Tenth Street, N.W.
##	Atlanta, Georgia  30332-0415
##	
##	*************************************************
##	
##	       Copyright 1996
##	Georgia Tech Research Corporation
##	    All Rights Reserved
##
##
##
##
Error EL_DEBUG text Sourcing text.tcl

set textMode "FALSE"
set filterMode "FALSE"
set toggleFilter 0

proc Test args {
    #puts "Got the Enter key press"
}

proc WarpCursor currentObject {
    set xcoord [getTextHorizCoord $currentObject]
    set ycoord [getTextVertCoord $currentObject]
    Error EL_DEBUG text "x: $xcoord y: $ycoord"
    xwin warp $currentObject $xcoord $ycoord
}

proc AnnounceCursorPosition args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
	#puts "ToggleCursorMode"
	speak [sreader $current cursor]
    }
}

proc ToggleCursorMode args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
	# Also check if the text area is editable. If the text area is not
	# editable, we know we must be in the review cursor mode. In this
	# case we don't must not toggle the cursor mode to avoid strange
	# sreader navigation behavior.  -wl
	if { [wtemplate $current editable] == "FALSE" } {
	    speak "Must be in cursor review mode in a non-editable text area!"
	} else {
	    #puts "ToggleCursorMode"
	    speak [sreader $current togglecursor]
	}
    }
}

proc ToggleFollowMode args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
	#puts "ToggleFollowMode"
	speak [sreader $current togglefollow]
    }
}

proc ToggleFilterMode args {
    global filterMode toggleFilter
    set toggleFilter 0
    set filterMode [expr {($filterMode == "TRUE") ? "FALSE" : "TRUE"}]
    set status [list Filter enable mode is [expr {($filterMode == "TRUE") ? "ON" : "OFF"}]]
    speak $status
    braille display $status
}

proc FilterModeOn args {
    global filterMode toggleFilter
    if {$filterMode == "TRUE"} {
	set current [currentObject]
	sreader $current filter $toggleFilter 1
	set blah [list Filter [sreader $current filter $toggleFilter name] is now [sreader $current filter $toggleFilter]]
	speak $blah
	braille display $blah
	incr toggleFilter
	if { $toggleFilter == [sreader $current filter] } {
	    ToggleFilterMode
	}
    }
}

proc FilterModeOff args {
    global filterMode toggleFilter
    if {$filterMode == "TRUE"} {
	set current [currentObject]
	sreader $current filter $toggleFilter 0
	set blah [list Filter [sreader $current filter $toggleFilter name] is now [sreader $current filter $toggleFilter]]
	speak $blah
	braille display $blah
	incr toggleFilter
	if { $toggleFilter == [sreader $current filter] } {
	    ToggleFilterMode
	}
    }
}

proc AnnounceFilterMode args {
    global filterMode toggleFilter
    set current [currentObject]
    if {$filterMode == "TRUE"} {
	set number [list Filter position $toggleFilter of [sreader $current filter]]
	set status [list [sreader $current filter $toggleFilter name] is [sreader $current filter $toggleFilter]]
	speak $number
	braille display $number
	speak $status
	braille display $status
    } else {
	set endindex [sreader $current filter]
	if {$endindex == 0} {
	    speak "No filters installed"
	    braille display "No filters installed"
	    return
	}
	for {set loop 0} {$loop < $endindex} {incr loop} {
	    set status [list Filter $loop is [sreader $current filter $loop name] [sreader $current filter $loop]]
	    speak $status
	    braille display $status
	}
    }
}

proc ReadThisChar args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
        #puts "ReadThisChar"
	set result [sreader $current read this char]
	speak $result
	braille display $result
    }
}
proc ReadThisWord args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
        #puts "ReadThisWord"
	set result [sreader $current read this word]
	speak $result
	braille display $result
    }
}
proc ReadThisSentence args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
        #puts "ReadThisSentence"
	set result [sreader $current read this sentence]
	speak $result
	braille display $result
    }
}
proc ReadThisLine args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
	set result [sreader $current read this line]
	speak $result
	braille display $result
    }
}
proc ReadThisScreen args {
    set current [currentObject]

    if { [modelManager class $current] == "Text"} {
        speak [sreader $current read this screen]
        braille display [sreader $current read this screen]
    } else {
        if { $current == [long2short {xmailtool outerbox help_msg}] } {
            speak [sreader $current read this screen]
            braille display [sreader $current read this screen]
        } else {
            if { $current == [long2short {xmailtool outerbox folder_name}] } {
		speak [sreader $current read this screen]
		braille display [sreader $current read this screen]
            }
        }
    }
    if { [modelManager class $current] == "VT100"} {
        speak [sreader $current read this screen]
        braille display [sreader $current read this screen]
    }
}
proc ReadThisParagraph args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
	set result [sreader $current read this paragraph]
	speak $result
	braille display $result
    }
}
proc ReadNextChar args {
    set current [currentObject]

    if { [modelManager class $current] == "Text"} {
	set result [sreader $current read next char]
	speak $result
	braille display $result
    }
    if { [modelManager class $current] == "VT100"} {
	set result [sreader $current read next char]
	speak $result
	braille display $result
    }
}
proc ReadNextWord args {
    set current [currentObject]

    if { [modelManager class $current] == "Text"} {
	set result [sreader $current read next word]
	speak $result
	braille display $result
    }
    if { [modelManager class $current] == "VT100"} {
	set result [sreader $current read next word]
	speak $result
	braille display $result
    }
}
proc ReadNextSentence args {
    set current [currentObject]

    if { [modelManager class $current] == "Text"} {
	set result [sreader $current read next sentence]
	speak $result
	braille display $result
    }
    if { [modelManager class $current] == "VT100"} {
	set result [sreader $current read next sentence]
	speak $result
	braille display $result
    }
}
proc ReadNextLine args {
    set current [currentObject]

    if { [modelManager class $current] == "Text"} {
	set result [sreader $current read next line]
	speak $result
	braille display $result
    }
    if { [modelManager class $current] == "VT100"} {
	set result [sreader $current read next line]
	speak $result
	braille display $result
    }
}
proc ReadNextParagraph args {
    set current [currentObject]

    if { [modelManager class $current] == "Text"} {
	set result [sreader $current read next paragraph]
	speak $result
	braille display $result
    }
    if { [modelManager class $current] == "VT100"} {
	set result [sreader $current read next paragraph]
	speak $result
	braille display $result
    }
}
proc MoveNextChar args {
    set current [currentObject]
    global textMode
    
    if { [modelManager class $current] == "Text"} {
	key $current Right both 1
    }
    set result [sreader $current move next char]
    speak $result
    braille display $result
}

proc MoveNextWord args {
    set current [currentObject]
    global textMode
    
    set result [sreader $current move next word]
    speak $result
    braille display $result
}

proc MoveNextSentence args {
    set current [currentObject]
    global textMode
    
    set result [sreader $current move next sentence]
    speak $result
    braille display $result
}

proc MoveNextLine args {
    set current [currentObject]
    global textMode
    
    set result [sreader $current move next line]
    speak $result
    braille display $result
}

proc MoveNextParagraph args {
    set current [currentObject]
    global textMode
    
    set result [sreader $current move next paragraph]
    speak $result
    braille display $result
}

proc ReadPrevChar args {
    set current [currentObject]
    
    if { [modelManager class $current] == "Text"} {
        speak [sreader $current read prev char]
        braille display [sreader $current read prev char]
    }
}
proc ReadPrevWord args {
    set current [currentObject]
    
    if { [modelManager class $current] == "Text"} {
        speak [sreader $current read prev word]
        braille display [sreader $current read prev word]
    }
}
proc ReadPrevSentence args {
    set current [currentObject]
    
    if { [modelManager class $current] == "Text"} {
        speak [sreader $current read prev sentence]
        braille display [sreader $current read prev sentence]
    }
}
proc ReadPrevLine args {
    set current [currentObject]
    
    if { [modelManager class $current] == "Text"} {
        speak [sreader $current read prev line]
        braille display [sreader $current read prev line]
    }
}
proc ReadPrevParagraph args {
    set current [currentObject]
    
    if { [modelManager class $current] == "Text"} {
        speak [sreader $current read prev paragraph]
        braille display [sreader $current read prev paragraph]
    }
}

proc MovePrevChar args {
    set current [currentObject]
    global textMode
    
    if { [modelManager class $current] == "Text"} {
	key $current Right both 1
    }
    set result [sreader $current move prev char]
    speak $result
    braille display $result
}

proc MovePrevWord args {
    set current [currentObject]
    global textMode
    
    set result [sreader $current move prev word]
    speak $result
    braille display $result
}

proc MovePrevSentence args {
    set current [currentObject]
    global textMode
    
    set result [sreader $current move prev sentence]
    speak $result
    braille display $result
    }

proc MovePrevLine args {
    set current [currentObject]
    global textMode
    
    set result [sreader $current move prev line]
    speak $result
    braille display $result
    }

proc MovePrevParagraph args {
    set current [currentObject]
    global textMode
    
    set result [sreader $current move prev paragraph]
    speak $result
    braille display $result
    }

proc ChangeTextMode args {
    set current [currentObject]
    global textMode filterMode userLevel

    if {[wtemplate $current allowTextMode] == "TRUE" } {
        if {$textMode == "FALSE"} {
	    set textMode "TRUE"
	    Error EL_DEBUG text "ChangeTextMode to TRUE"

	    if {$userLevel == "Novice" || $userLevel == "Intermediate"} {
	        speak "entering text mode"
		braille display "entering text mode"
	    }
	    relativemotion 15 0
	    TextEnterOrExitAudio

	    button press 1
	    button release 1
	    # setFocus $current

	    # if we're going into a non-editable text widget, set the
	    # cursor mode to REVIEW mode.
	    if { [wtemplate $current editable] == "FALSE" } {
		if {[sreader $current cursor mode] == "EDIT"} {
		    sreader $current togglecursor
		}
	    } else {
		if {[sreader $current cursor mode] == "REVIEW"} {
		    sreader $current togglecursor
		}
	    }
        } else {
	    # don't switch out of text mode if we're in filter toggle mode
	    if {$filterMode != "TRUE"} {
		set textMode "FALSE"
		Error EL_DEBUG text "ChangeTextMode to FALSE"
		if {$userLevel == "Novice" || $userLevel == "Intermediate"} {
		}
		
		TextEnterOrExitAudio
		
		relativemotion -15 0
		
		GoTo $current
		
		#button press 1
		#button release 1
		#setFocus $current
	    } else {
		speak "Must exit filter enable mode before exiting text mode"
		braille display "Must exit filter enable mode before exiting text mode"
	    }
	}
	#	sreader $current cursor 0 0
	sreader $current cursor app
    } else {
	# set textMode to false, just in case
	set textMode "FALSE"
        speak "Not a text class"
	braille display "Not a text class"
    }
}

proc TextUp args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
	#key $current Up press 1
	#key $current Up release 1
    }

    set result [sreader $current move prev line]
    speak $result

    braille display $result

    set xcoord [getTextHorizCoord $current]
    set ycoord [getTextVertCoord $current]
    Error EL_DEBUG text "TextUp x: $xcoord y: $ycoord"
    #Error EL_DEBUG text "TextUp warping to " [currentObject] " 0 $ycoord"
    #    xwin warp [currentObject] 15 $ycoord
    #xwin warp $current $xcoord $ycoord
#    key [currentObject] Up both 1
    #WarpCursor [currentObject]
    #CSUN, make edit cursor follow mouse
    #button $current press 1
    #button $current release 1
    #  sreader $current read this line
}

proc TextDown args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
	#key $current Down press 1
	#key $current Down release 1
    }
    Error EL_DEBUG text "TextDown Moving down a line"

    set result [sreader $current move next line]
    speak $result

    braille display $result

    set xcoord [getTextHorizCoord $current]
    set ycoord [getTextVertCoord $current]
    Error EL_DEBUG text "TextDown x: $xcoord y: $ycoord"
    # Error EL_DEBUG text "TextDown warping to " [currentObject] " 15 $coord"
    #xwin warp [currentObject] 15 $ycoord
    #xwin warp $current $xcoord $ycoord
#    key [currentObject] Down both 1
    #WarpCursor [currentObject]
    #CSUN, make edit cursor follow mouse
    #button $current press 1
    #button $current release 1

    #  sreader $current read this line
}

proc TextRight args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
	#key $current Right press 1
	#key $current Right release 1
    }
    speak [sreader $current move next char]

    #  sreader $current read this char
    set xcoord [getTextHorizCoord $current]
    set ycoord [getTextVertCoord $current]
    Error EL_DEBUG text "TextRight x: $xcoord y: $ycoord"
    # xwin warp $current $xcoord $ycoord
#    key [currentObject] Right both 1
}

proc TextLeft args {
    set current [currentObject]

    if { [wtemplate $current allowTextMode] == "TRUE" } {
	#key $current Left press 1
	#key $current Left release 1
    }
    speak [sreader $current move prev char]

    # sreader $current read this char
    set xcoord [getTextHorizCoord $current]
    set ycoord [getTextVertCoord $current]

    Error EL_DEBUG text "TextLeft x: $xcoord y: $ycoord"
    # xwin warp $current $xcoord $ycoord

#    key [currentObject] Left both 1
}

proc BrailleProg args {
    Error EL_DEBUG BrailleProg
}

proc BrailleHome args {
    Error EL_DEBUG BrailleHome
}

proc BrailleCursor args {
    Error EL_DEBUG BrailleCursor
}

proc BrailleUp args {
    Error EL_DEBUG BrailleUp
    TextUp
}

proc BrailleUpBounds args {
    Error EL_DEBUG BrailleUpBounds
}

proc BrailleLeft args {
    Error EL_DEBUG BrailleLeft
#    TextLeft
}

proc BrailleLeftBounds args {
    Error EL_DEBUG BrailleLeftBounds
}

proc BrailleLeftJumpScroll args {
    Error EL_DEBUG BrailleLeftJumpScroll
}

proc BrailleRight args {
    Error EL_DEBUG BrailleRight
#    TextRight
}

proc BrailleRightBounds args {
    Error EL_DEBUG BrailleRightBounds
}

proc BrailleRightJumpScroll args {
    Error EL_DEBUG BrailleRightJumpScroll
}

proc BrailleDown args {
    Error EL_DEBUG BrailleDown
    TextDown
}

proc BrailleDownBounds args {
    Error EL_DEBUG BrailleBounds
}
