/*
//
//      Copyright 1992, 1993, 1994, 1995, 1996
//      Georgia Tech Research Corporation
//      All rights reserved.
//
//
// Note:  The use of this program shall be restricted to educational,
// non-commercial use only.  All inquiries regarding licensing this
// software for commercial use may be addressed to:
//
// Georgia Tech Research Corporation
// Technology Licensing
// Centennial Research Building - Rm. 275
// 400 Tenth Street, N.W.
// Atlanta, Georgia  30332-0415
//

*/
/*
 * Random socket utility subroutines
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <fcntl.h>
#include "udp.h"
#include "os-interface.h"

#define DEBUG

#ifdef DEBUG
#include <sys/time.h>
#endif

unsigned long resolve_host(char *host)
{ 
  struct hostent *hp;
  unsigned long addr;

  if ((hp = gethostbyname(host)) == NULL) {
    return inet_addr(host);
  } else {
    bcopy(hp->h_addr, (char *) &addr, sizeof(addr));
    return addr;
  }
}

void init_sockaddr(struct sockaddr_in *sa, unsigned long addr, 
		   unsigned short port)
{
  bzero((char *) sa, sizeof(struct sockaddr_in));
  sa->sin_family = AF_INET;
  sa->sin_addr.s_addr = addr;
  sa->sin_port = port;
}

int sock_reuseaddr(int sock, int reuse)
{
  return setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&reuse, 
		    sizeof(reuse));
}

int sock_addmulti(int sock, unsigned long addr,unsigned long ifaddr)
{
  struct ip_mreq mreq;

  mreq.imr_multiaddr.s_addr = addr;
  mreq.imr_interface.s_addr = ifaddr;
  return setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&mreq, 
		    sizeof(mreq));
}

int sock_delmulti(int sock, unsigned long addr)
{
  struct ip_mreq mreq;

  mreq.imr_multiaddr.s_addr = addr;
  mreq.imr_interface.s_addr = INADDR_ANY;
  return setsockopt(sock, IPPROTO_IP, IP_DROP_MEMBERSHIP, (char *)
		    &mreq, sizeof(mreq));
}

int sock_setmultittl(int sock, unsigned char ttl)
{
  return setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL, (char *)
		    &ttl, sizeof(ttl));
}

int sock_udpcreate(struct sockaddr_in lcladdr)
{
  int sock;
  unsigned long addr,ifaddr=INADDR_ANY;

  if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) == -1) return -1;

  if (sock_reuseaddr(sock, 1) == -1) return -1;


/*  if (IN_MULTICAST(addr = lcladdr.sin_addr.s_addr))
    lcladdr.sin_addr.s_addr = INADDR_ANY; */

  addr=lcladdr.sin_addr.s_addr;
  lcladdr.sin_addr.s_addr=INADDR_ANY;

  if (bind(sock, (struct sockaddr *)&lcladdr, sizeof(lcladdr)) == -1) {
    perror("bind");
    close(sock);
    lcladdr.sin_addr.s_addr = addr;
    return -1;
  }


  if (IN_MULTICAST(addr)) {
    sock_addmulti(sock, addr,ifaddr);
    lcladdr.sin_addr.s_addr = addr;
  } else {
/*    if (connect(sock,(struct sockaddr *)&lcladdr,sizeof(lcladdr))==-1) {
      perror("connect");
      close(sock);
      lcladdr.sin_addr.s_addr = addr;
      return -1; 
    } */
  }

  lcladdr.sin_addr.s_addr = addr;
  return sock;
}

int sock_sendto(int sock, char *buf, int len, struct sockaddr_in *dst)
{
  int tmp;
  tmp=sendto(sock, buf, len, 0, (struct sockaddr *)dst, sizeof(*dst));
  return tmp;
}

int sock_recv(int sock, char *buf, int len)
{
  struct sockaddr_in remaddr;
  int addrlen=sizeof(remaddr),tmp;
  tmp=recvfrom(sock, buf, len, 0, (struct sockaddr *)&remaddr, 
	       &addrlen);
  return tmp;
}

int sock_recvfrom(int sock, char *buf, int len, struct sockaddr_in *src)
{
  int addrlen=sizeof(*src);

  return recvfrom(sock, buf, len, 0, (struct sockaddr *)src, &addrlen);
}

int sock_setmultiloopback(int sock, int val)
{
  unsigned char loop=(unsigned char)val;
  return (setsockopt(sock,IPPROTO_IP,IP_MULTICAST_LOOP,(char *)&loop,
		     sizeof(loop)));

}
